@@ -51,7 +51,6 @@
 #include "imesh/particle.h"
 #include "imesh/sprite2d.h"
 #include "imesh/sprite3d.h"
-#include "imesh/stars.h"
 #include "imesh/thing.h"
 #include "iutil/cmdline.h"
 #include "iutil/comp.h"
@@ -87,16 +86,7 @@ void Demo::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (System->object_reg, iReporter));
-  if (rep)
-  {
-    rep->ReportV (severity, "crystalspace.application.demo", msg, arg);
-  }
-  else
-  {
-    csPrintfV (msg, arg);
-    csPrintf ("\n");
-  }
+  csReportV(object_reg, severity, "crystalspace.application.demo", msg, arg);
   va_end (arg);
 }
 
@@ -189,6 +179,7 @@ bool Demo::LoadDemoFile (const char* demofile)
   seqmgr->Setup ("sequences");
 
   engine->Prepare ();
+  engine->PrecacheDraw ();
 
   view = csPtr<iView> (new csView (engine, myG3D));
   view->GetCamera ()->SetSector (room);
@@ -246,44 +237,44 @@ bool Demo::Initialize (int argc, const char* const argv[],
   }
 
   // The virtual clock.
-  vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
+  vc = csQueryRegistry<iVirtualClock> (object_reg);
 
-  kbd = CS_QUERY_REGISTRY (object_reg, iKeyboardDriver);
+  kbd = csQueryRegistry<iKeyboardDriver> (object_reg);
   if (!kbd)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No keyboard driver!");
     return false;
   }
 
-  engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  engine = csQueryRegistry<iEngine> (object_reg);
   if (!engine)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No engine!");
     return false;
   }
 
-  loader = CS_QUERY_REGISTRY (object_reg, iLoader);
+  loader = csQueryRegistry<iLoader> (object_reg);
   if (!loader)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No loader!");
     return false;
   }
 
-  myG3D = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  myG3D = csQueryRegistry<iGraphics3D> (object_reg);
   if (!myG3D)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No 3D driver!");
     return false;
   }
 
-  myG2D = CS_QUERY_REGISTRY (object_reg, iGraphics2D);
+  myG2D = csQueryRegistry<iGraphics2D> (object_reg);
   if (!myG2D)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No 2D driver!");
     return false;
   }
 
-  myVFS = CS_QUERY_REGISTRY (object_reg, iVFS);
+  myVFS = csQueryRegistry<iVFS> (object_reg);
   if (!myVFS)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No VFS!");
@@ -310,8 +301,8 @@ bool Demo::Initialize (int argc, const char* const argv[],
   Report (CS_REPORTER_SEVERITY_NOTIFY, "Creating world!...");
 
   // Check the demo file and mount it if required.
-  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
-  	iCommandLineParser));
+  csRef<iCommandLineParser> cmdline (
+  	csQueryRegistry<iCommandLineParser> (object_reg));
   const char *val;
   if ((val = cmdline->GetName ()) != 0)
   {
@@ -429,7 +420,7 @@ void Demo::SetupFrame ()
     GfxWrite (tx, ty, col_black, col_bg,
     	"Download 'demodata.zip' from"); ty += 10;
     GfxWrite (tx, ty, col_fgdata, col_bg,
-    	"    http://www.crystalspace3d.org/support/release/cs098/maps/demodata.zip"); ty += 10;
+    	"    http://www.crystalspace3d.org/support/release/cs099/maps/demodata.zip"); ty += 10;
 
     ty += 10;
 
@@ -439,13 +430,13 @@ void Demo::SetupFrame ()
     	"    csdemo demodata.zip"); ty += 10;
     ty += 10;
     GfxWrite (tx, ty, col_black, col_bg,
-    	"or you can run it with OpenGL and a higher resolution:"); ty += 10;
+    	"or you can run it with a higher resolution:"); ty += 10;
     GfxWrite (tx, ty, col_fgdata, col_bg,
-    	"    csdemo demodata.zip -video=opengl -mode=800x600"); ty += 10;
+    	"    csdemo demodata.zip -mode=800x600"); ty += 10;
 
     ty += 10;
 
-    if (demos.Length () == 0)
+    if (demos.GetSize () == 0)
     {
       GfxWrite (tx, ty, col_black, col_bg,
     	"I could not find any data files in this and the data directory!");
@@ -463,7 +454,7 @@ void Demo::SetupFrame ()
       ty += 10;
       first_y = ty;
       size_t i;
-      for (i = 0 ; i < demos.Length () ; i++)
+      for (i = 0 ; i < demos.GetSize () ; i++)
       {
 	int bg = col_bg;
 	int fg = col_fgdata;
@@ -533,8 +524,8 @@ void Demo::SetupFrame ()
     csNamedPath* np = seqmgr->GetSelectedPath (map_selpath, start, total);
     if (np)
     {
-      float r = np->GetTimeValue (map_selpoint);
-      np->Calculate (r);
+      float r = np->GetTime (map_selpoint);
+      np->CalculateAtTime (r);
       debug_time = csTicks (start + total * r);
     }
     else
@@ -547,8 +538,8 @@ void Demo::SetupFrame ()
     csNamedPath* np = seqmgr->GetSelectedPath (map_selpath);
     if (np)
     {
-      float r = np->GetTimeValue (map_selpoint);
-      np->Calculate (r);
+      float r = np->GetTime (map_selpoint);
+      np->CalculateAtTime (r);
       csVector3 pos, up, forward;
       np->GetInterpolatedPosition (pos);
       np->GetInterpolatedUp (up);
@@ -649,7 +640,7 @@ void Demo::DrawEditInfo ()
     	fwd.x, fwd.y, fwd.z); hh += fh;
     GfxWrite (ww, hh, col_black, col_white, "U(%.2g,%.2g,%.2g)",
     	up.x, up.y, up.z); hh += fh;
-    float t = np->GetTimeValue (map_selpoint);
+    float t = np->GetTime (map_selpoint);
     csTicks tms = csTicks (t*total);
     GfxWrite (ww, hh, col_black, col_white, "tot time %d ms", total); hh += fh;
     GfxWrite (ww, hh, col_black, col_white, "rel time %d ms", tms); hh += fh;
@@ -659,7 +650,7 @@ void Demo::DrawEditInfo ()
       csVector3 v1;
       np->GetPositionVector (map_selpoint-1, v1);
       float d = csQsqrt (csSquaredDist::PointPoint (v, v1));
-      float t1 = np->GetTimeValue (map_selpoint-1);
+      float t1 = np->GetTime (map_selpoint-1);
       float dr = t-t1;
       float speed = (float) fabs (dr) / d;
       csTicks tms1 = csTicks (t1*total);
@@ -674,7 +665,7 @@ void Demo::DrawEditInfo ()
     {
       csVector3 v1;
       np->GetPositionVector (map_selpoint+1, v1);
-      float t1 = np->GetTimeValue (map_selpoint+1);
+      float t1 = np->GetTime (map_selpoint+1);
       float dr = t1-t;
       float d = csQsqrt (csSquaredDist::PointPoint (v, v1));
       float speed = (float) fabs (dr) / d;
@@ -707,7 +698,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
     {
       if (keyCode == CSKEY_ESC)
       {
-	csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
+	csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
 	if (q)
 	  q->GetEventOutlet()->Broadcast (csevQuit (object_reg));
         return true;
@@ -823,7 +814,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
         case '=':
 	  // Make the forward vector look along the path. i.e. let it look
 	  // to an average direction as specified by next and previous point.
-	  if (map_selpoint <= 0 || map_selpoint >= np->GetPointCount ()-1)
+          if (map_selpoint <= 0 || map_selpoint >= np->Length ()-1)
 	  {
 	    ShowMessage ("The '=' operation can't work on this point!\n");
 	  }
@@ -874,7 +865,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
         case '+':
 	  // Make the forward vector look along the path. i.e. let it look
 	  // to the next point in the path if there is one.
-	  if (map_selpoint >= np->GetPointCount ()-1)
+	  if (map_selpoint >= np->Length ()-1)
 	  {
             csVector3 v1, v2;
 	    np->GetPositionVector (map_selpoint-1, v1);
@@ -1007,9 +998,9 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	    csRef<iFile> fp (myVFS->Open (buf, VFS_FILE_WRITE));
 	    if (fp)
 	    {
-	      int i, num = np->GetPointCount ();
+	      int i, num = np->Length ();
 	      FileWrite (fp, "    NUM (%d)\n", num);
-	      const float* t = np->GetTimeValues ();
+	      const float* t = np->GetTimes ();
 	      FileWrite (fp, "    TIMES (%g", t[0]);
 	      for (i = 1 ; i < num ; i++)
 	        FileWrite (fp, ",%g", t[i]);
@@ -1049,7 +1040,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  {
 	    np->InsertPoint (map_selpoint);
 	    map_selpoint++;
-	    if (map_selpoint == np->GetPointCount ()-1)
+	    if (map_selpoint == np->Length ()-1)
 	    {
 	      csVector3 v;
 	      np->GetPositionVector (map_selpoint-1, v);
@@ -1082,18 +1073,18 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  if (np)
 	  {
 	    np->RemovePoint (map_selpoint);
-	    if (map_selpoint >= np->GetPointCount ())
+	    if (map_selpoint >= np->Length ())
 	      map_selpoint--;
 	  }
 	  break;
 	case ',':
 	  if (np)
 	  {
-	    if (map_selpoint > 0 && map_selpoint < np->GetPointCount ()-1)
+	    if (map_selpoint > 0 && map_selpoint < np->Length ()-1)
 	    {
-	      float t = np->GetTimeValue (map_selpoint);
-	      float t1 = np->GetTimeValue (map_selpoint-1);
-	      float t2 = np->GetTimeValue (map_selpoint+1);
+	      float t = np->GetTime (map_selpoint);
+	      float t1 = np->GetTime (map_selpoint-1);
+	      float t2 = np->GetTime (map_selpoint+1);
 	      float dt = (t2-t1);
 	      if (shift) dt /= 5.;
 	      else if (ctrl) dt /= 500.;
@@ -1107,33 +1098,33 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	case '.':
 	  if (np)
 	  {
-	    if (map_selpoint > 0 && map_selpoint < np->GetPointCount ()-1)
+	    if (map_selpoint > 0 && map_selpoint < np->Length ()-1)
 	    {
-	      float t = np->GetTimeValue (map_selpoint);
-	      float t1 = np->GetTimeValue (map_selpoint-1);
-	      float t2 = np->GetTimeValue (map_selpoint+1);
+	      float t = np->GetTime (map_selpoint);
+	      float t1 = np->GetTime (map_selpoint-1);
+	      float t2 = np->GetTime (map_selpoint+1);
 	      float dt = (t2-t1);
 	      if (shift) dt /= 5.;
 	      else if (ctrl) dt /= 500.;
 	      else dt /= 50.;
 	      t += dt;
 	      if (t > t2) t = t2;
-	      np->SetTimeValue (map_selpoint, t);
+	      np->SetTime (map_selpoint, t);
 	    }
 	  }
 	  break;
 	case '/':
-	  if (np && map_selpoint > 0 && map_selpoint < np->GetPointCount ()-1)
+	  if (np && map_selpoint > 0 && map_selpoint < np->Length ()-1)
 	  {
-	    float t1 = np->GetTimeValue (map_selpoint - 1);
-	    float t2 = np->GetTimeValue (map_selpoint + 1);
-	    np->SetTimeValue (map_selpoint, (t1+t2) / 2.0f);
+	    float t1 = np->GetTime (map_selpoint - 1);
+	    float t2 = np->GetTime (map_selpoint + 1);
+	    np->SetTime (map_selpoint, (t1+t2) / 2.0f);
 	  }
 	  break;
 	case '?':
 	  if (np)
 	  {
-	    int num = np->GetPointCount ();
+	    int num = np->Length ();
 	    const float* xv, * yv, * zv;
 	    xv = np->GetDimensionValues (0);
 	    yv = np->GetDimensionValues (1);
@@ -1164,14 +1155,14 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	      list[i] = tot / totlen;
 	      v0 = v1;
 	    }
-	    np->SetTimeValues (list);
+	    np->SetTimes (list);
 	  }
 	  break;
         case '>':
 	  if (np)
 	  {
             map_selpoint++;
-	    if (map_selpoint >= np->GetPointCount ())
+	    if (map_selpoint >= np->Length ())
 	      map_selpoint = 0;
 	  }
 	  break;
@@ -1180,7 +1171,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  {
             map_selpoint--;
 	    if (map_selpoint < 0)
-	      map_selpoint = np->GetPointCount ()-1;
+	      map_selpoint = np->Length ()-1;
 	  }
 	  break;
         case 'c':
@@ -1220,8 +1211,8 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  np = seqmgr->GetSelectedPath (map_selpath);
 	  if (np)
 	  {
-	    if (map_selpoint >= np->GetPointCount ())
-	      map_selpoint = np->GetPointCount ()-1;
+	    if (map_selpoint >= np->Length ())
+	      map_selpoint = np->Length ()-1;
 	  }
 	  break;
         case ']':
@@ -1229,8 +1220,8 @@ bool Demo::DemoHandleEvent (iEvent &Event)
 	  np = seqmgr->GetSelectedPath (map_selpath);
 	  if (np)
 	  {
-	    if (map_selpoint >= np->GetPointCount ())
-	      map_selpoint = np->GetPointCount ()-1;
+	    if (map_selpoint >= np->Length ())
+	      map_selpoint = np->Length ()-1;
 	  }
 	  break;
       }
@@ -1242,7 +1233,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
       //==============================
       if (keyCode == CSKEY_ESC)
       {
-	csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
+	csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
 	if (q)
 	  q->GetEventOutlet()->Broadcast (csevQuit (object_reg));
         return true;
@@ -1288,7 +1279,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
     if (do_demo == 0)
     {
       selected_demo = (csMouseEventHelper::GetY(&Event) - first_y) / 10;
-      if ((selected_demo != (size_t)-1) && selected_demo < demos.Length ())
+      if ((selected_demo != (size_t)-1) && selected_demo < demos.GetSize ())
 	do_demo = 1;
     }
     else if (do_demo < 3)
@@ -1298,7 +1289,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
     {
       csVector2 p (csMouseEventHelper::GetX(&Event), myG2D->GetHeight ()-csMouseEventHelper::GetY(&Event));
       csVector3 v;
-      view->GetCamera ()->InvPerspective (p, 1, v);
+      v = view->GetCamera ()->InvPerspective (p, 1);
       csVector3 vw = view->GetCamera ()->GetTransform ().This2Other (v);
       if (map_enabled == MAP_EDIT_FORWARD)
       {
@@ -1334,7 +1325,7 @@ bool Demo::DemoHandleEvent (iEvent &Event)
     if (do_demo == 0)
     {
       selected_demo = (csMouseEventHelper::GetY(&Event) - first_y) / 10;
-      if (!((selected_demo != (size_t)-1) && selected_demo < demos.Length ()))
+      if (!((selected_demo != (size_t)-1) && selected_demo < demos.GetSize ()))
         selected_demo = (size_t)-1;
     }
   }
