@@ -53,9 +53,10 @@ DemoSequenceManager::DemoSequenceManager (Demo* demo)
 {
   DemoSequenceManager::demo = demo;
   demoseq = this;
+  loader = 0;
   iObjectRegistry* object_reg = demo->object_reg;
   csRef<iPluginManager> plugin_mgr (
-  	CS_QUERY_REGISTRY (object_reg, iPluginManager));
+  	csQueryRegistry<iPluginManager> (object_reg));
   seqmgr = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.utilities.sequence",
   	iSequenceManager);
   if (!seqmgr)
@@ -81,11 +82,14 @@ void DemoSequenceManager::Clear ()
   seqmgr->Clear ();
   pathForMesh.DeleteAll ();
   meshRotation.DeleteAll ();
+  delete loader;
+  loader = 0;
 }
 
 void DemoSequenceManager::Setup (const char* sequenceFileName)
 {
-  DemoSequenceLoader* loader = new DemoSequenceLoader (
+  delete loader;
+  loader = new DemoSequenceLoader (
   	DemoSequenceManager::demo, this, seqmgr, sequenceFileName);
   main_sequence = loader->GetSequence ("main");
   seqmgr->RunSequence (0, main_sequence);
@@ -93,7 +97,6 @@ void DemoSequenceManager::Setup (const char* sequenceFileName)
   suspended = false;
   main_start_time = seqmgr->GetMainTime ();
   num_frames = 0;
-  delete loader;
 }
 
 void DemoSequenceManager::Suspend ()
@@ -218,7 +221,7 @@ void DemoSequenceManager::ReplacePathObject (csNamedPath* path,
 	iMeshWrapper* mesh)
 {
   size_t i;
-  for (i = 0 ; i < pathForMesh.Length () ; i++)
+  for (i = 0 ; i < pathForMesh.GetSize () ; i++)
   {
     PathForMesh* pfm = pathForMesh[i];
     if (pfm->path == path)
@@ -247,7 +250,7 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
   if (suspended) return;
   csTicks current_time = seqmgr->GetMainTime ();
   size_t i = 0;
-  size_t len = pathForMesh.Length ();
+  size_t len = pathForMesh.GetSize ();
   while (i < len)
   {
     PathForMesh* pfm = pathForMesh[i];
@@ -259,7 +262,7 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
       r = 1;
       do_path = false;
     }
-    pfm->path->Calculate (r);
+    pfm->path->CalculateAtTime (r);
     csVector3 oldpos;
     iSector* oldsector;
     csVector3 pos, up, forward;
@@ -312,7 +315,7 @@ void DemoSequenceManager::ControlPaths (iCamera* camera, csTicks elapsed_time)
   }
 
   i = 0;
-  len = meshRotation.Length ();
+  len = meshRotation.GetSize ();
   while (i < len)
   {
     MeshRotation* mrot = meshRotation[i];
@@ -335,7 +338,7 @@ void DemoSequenceManager::DebugPositionObjects (iCamera* camera,
     csTicks debug_time)
 {
   size_t i = 0;
-  size_t len = pathForMesh.Length ();
+  size_t len = pathForMesh.GetSize ();
   while (i < len)
   {
     PathForMesh* pfm = pathForMesh[i];
@@ -343,7 +346,7 @@ void DemoSequenceManager::DebugPositionObjects (iCamera* camera,
     	/ float (pfm->total_path_time);
     if (r >= 0 && r <= 1)
     {
-      pfm->path->Calculate (r);
+      pfm->path->CalculateAtTime (r);
       csVector3 pos, up, forward;
       pfm->path->GetInterpolatedPosition (pos);
       pfm->path->GetInterpolatedUp (up);
@@ -375,7 +378,7 @@ void DemoSequenceManager::DebugDrawPath (csNamedPath* np, bool hi,
   csVector3 p;
   for (r = 0 ; r <= 1 ; r += 0.001f)
   {
-    np->Calculate (r);
+    np->CalculateAtTime (r);
     np->GetInterpolatedPosition (p);
     int x = int ((p.x-tl.x)*dim / (br.x-tl.x));
     int y = int ((p.z-tl.y)*dim / (br.y-tl.y));
@@ -446,7 +449,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
 	int selpoint)
 {
   size_t i;
-  size_t len = pathForMesh.Length ();
+  size_t len = pathForMesh.GetSize ();
   csTicks current_time = seqmgr->GetMainTime ();
 
   //=====
@@ -475,7 +478,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
   if (selnp)
   {
     // Calculate where we are in time on the selected path.
-    float t = selnp->GetTimeValue (selpoint);
+    float t = selnp->GetTime (selpoint);
     seltime = csTicks (start + total*t);
   }
 
@@ -510,7 +513,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
     float r = float (ct - pfm->start_path_time)
     	/ float (pfm->total_path_time);
     if (r >= 1) r = 1;
-    pfm->path->Calculate (r);
+    pfm->path->CalculateAtTime (r);
     // We are going to show both the position as the forward vector.
     csVector3 pos, forward;
     pfm->path->GetInterpolatedPosition (pos);
@@ -526,7 +529,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
       	/ float (pfm->total_path_time);
       if (r >= 0 && r <= 1)
       {
-	pfm->path->Calculate (r);
+	pfm->path->CalculateAtTime (r);
 	pfm->path->GetInterpolatedPosition (pos);
 	pfm->path->GetInterpolatedForward (forward);
         DrawSelPoint (pos, forward, tl, br, dim, demo->col_cyan, 10);
@@ -539,7 +542,7 @@ void DemoSequenceManager::DebugDrawPaths (iCamera* camera,
 
 void DemoSequenceManager::SelectFirstPath (char* hilight)
 {
-  if (pathForMesh.Length () > 0)
+  if (pathForMesh.GetSize () > 0)
   {
     strcpy (hilight, pathForMesh[0]->path->GetName ());
   }
@@ -547,9 +550,9 @@ void DemoSequenceManager::SelectFirstPath (char* hilight)
 
 void DemoSequenceManager::SelectLastPath (char* hilight)
 {
-  if (pathForMesh.Length () > 0)
+  if (pathForMesh.GetSize () > 0)
   {
-    strcpy (hilight, pathForMesh[pathForMesh.Length ()-1]->path->GetName ());
+    strcpy (hilight, pathForMesh[pathForMesh.GetSize ()-1]->path->GetName ());
   }
 }
 
@@ -562,7 +565,7 @@ void DemoSequenceManager::SelectPreviousPath (char* hilight)
     return;
   }
   size_t i;
-  for (i = 0 ; i < pathForMesh.Length () ; i++)
+  for (i = 0 ; i < pathForMesh.GetSize () ; i++)
   {
     PathForMesh* pfm = pathForMesh[i];
     if (pfm->path == np)
@@ -591,12 +594,12 @@ void DemoSequenceManager::SelectNextPath (char* hilight)
     return;
   }
   size_t i;
-  for (i = 0 ; i < pathForMesh.Length () ; i++)
+  for (i = 0 ; i < pathForMesh.GetSize () ; i++)
   {
     PathForMesh* pfm = pathForMesh[i];
     if (pfm->path == np)
     {
-      if (i < pathForMesh.Length ()-1)
+      if (i < pathForMesh.GetSize ()-1)
       {
         pfm = pathForMesh[i+1];
 	strcpy (hilight, pfm->path->GetName ());
@@ -619,7 +622,7 @@ csNamedPath* DemoSequenceManager::GetSelectedPath (const char* hilight,
 	csTicks& start, csTicks& total)
 {
   size_t i = 0;
-  size_t len = pathForMesh.Length ();
+  size_t len = pathForMesh.GetSize ();
   while (i < len)
   {
     PathForMesh* pfm = pathForMesh[i];
@@ -640,7 +643,7 @@ void DemoSequenceManager::SetupRotatePart (iMeshWrapper* mesh,
 	float angle_speed, csTicks total_rotate_time, csTicks already_elapsed)
 {
   MeshRotation* mrot = new MeshRotation ();
-  mrot->particle = SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iParticle);
+  mrot->particle = scfQueryInterface<iParticle> (mesh->GetMeshObject ());
   if (!mrot->particle)
   {
     delete mrot;
