@@ -28,6 +28,7 @@
 #include "csutil/util.h"
 
 class Demo;
+class DemoSequenceLoader;
 class csVector3;
 class csVector2;
 struct iGraphics3D;
@@ -91,6 +92,7 @@ public:
 private:
   csRef<iSequenceManager> seqmgr;
   csRef<iSequence> main_sequence;
+  DemoSequenceLoader* loader;
   bool suspended;
   bool suspend_one_frame;
   csTicks main_start_time;
@@ -258,7 +260,7 @@ public:
   csNamedPath* FindPath (const char* name)
   {
     size_t i;
-    for (i = 0 ; i < paths.Length () ; i++)
+    for (i = 0 ; i < paths.GetSize () ; i++)
     {
       csNamedPath* p = paths[i];
       if (!strcmp (p->GetName (), name)) return p;
