@@ -37,6 +37,7 @@
 #include "imap/loader.h"
 #include "imesh/object.h"
 #include "imesh/sprite3d.h"
+#include "imesh/skeleton.h"
 #include "iutil/csinput.h"
 #include "iutil/event.h"
 #include "iutil/eventq.h"
@@ -52,7 +53,7 @@
 #include "ivideo/texture.h"
 #include "ivideo/txtmgr.h"
 #include "imesh/genmesh.h"
-#include "imesh/gmeshskel.h"
+#include "imesh/gmeshskel2.h"
 
 CS_IMPLEMENT_APPLICATION
 
@@ -144,19 +145,20 @@ void IsoTest::SetupFrame ()
     }
     // update animation state
     csRef<iGeneralMeshState> spstate (
-      SCF_QUERY_INTERFACE (actor->GetMeshObject (), iGeneralMeshState));
+      scfQueryInterface<iGeneralMeshState> (actor->GetMeshObject ()));
     csRef<iGenMeshSkeletonControlState> animcontrol (
-      SCF_QUERY_INTERFACE (spstate->GetAnimationControl (), 
-      iGenMeshSkeletonControlState));
+       
+      scfQueryInterface<iGenMeshSkeletonControlState> (spstate->GetAnimationControl ()));
+    iSkeleton* skeleton = animcontrol->GetSkeleton ();
     if(actor_is_walking && !moved)
     {
-      animcontrol->StopAll();
-      animcontrol->Execute("idle");
+      skeleton->StopAll();
+      skeleton->Execute("idle");
     }
     if(!actor_is_walking && moved)
     {
-      animcontrol->StopAll();
-      animcontrol->Execute("walk");
+      skeleton->StopAll();
+      skeleton->Execute("run");
     }
     actor_is_walking = moved;
   }
@@ -166,9 +168,8 @@ void IsoTest::SetupFrame ()
   actor_pos.y += 10.0;	// Make sure we start beam high enough.
   csVector3 end_pos, isect;
   end_pos = actor_pos; end_pos.y -= 100.0;
-  float r;
-  plane->HitBeamObject (actor_pos, end_pos, isect, &r);
-  actor_pos.y = isect.y + .8;
+  csHitBeamResult rc = plane->HitBeamObject (actor_pos, end_pos);
+  actor_pos.y = rc.isect.y;
 
   actor->GetMovable ()->SetPosition (actor_pos);
   actor->GetMovable ()->UpdateMove ();
@@ -189,9 +190,8 @@ void IsoTest::SetupFrame ()
     return;
 
   csVector2 lpos(0,0);
-  view->GetCamera()->Perspective(
-    view->GetCamera()->GetTransform ().Other2This(csVector3 (-4.7f, 1.0f, 5.5f)),
-    lpos);
+  lpos = view->GetCamera()->Perspective(
+    view->GetCamera()->GetTransform ().Other2This(csVector3 (-4.7f, 1.0f, 5.5f)));
   // display a helpful little text.
   int txtw=0, txth=0;
   font->GetMaxSize(txtw, txth);
@@ -236,7 +236,7 @@ bool IsoTest::HandleEvent (iEvent& ev)
     utf32_char c = csKeyEventHelper::GetCookedCode (&ev);
     if (c == CSKEY_ESC)
     {
-      csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
+      csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
       if (q)
 	q->GetEventOutlet()->Broadcast (csevQuit (object_reg));
       return true;
@@ -301,7 +301,7 @@ bool IsoTest::LoadMap ()
   engine->SetLightingCacheMode (0);
 
   // Set VFS current directory to the level we want to load.
-  csRef<iVFS> VFS (CS_QUERY_REGISTRY (object_reg, iVFS));
+  csRef<iVFS> VFS (csQueryRegistry<iVFS> (object_reg));
   VFS->ChDir ("/lev/isomap");
   // Load the level file which is called 'world'.
   if (!loader->LoadMapFile ("world"))
@@ -355,67 +355,35 @@ bool IsoTest::LoadMap ()
 
 bool IsoTest::CreateActor ()
 {
-  // Load a texture for our sprite.
-  iTextureManager* txtmgr = g3d->GetTextureManager ();
-  iTextureWrapper* txt = loader->LoadTexture ("vedette_fashion",
-    "/lib/std/ugly_woman.jpg", CS_TEXTURE_3D, txtmgr, false, false);
-  if (txt == 0)
+  csRef<iVFS> vfs (csQueryRegistry<iVFS> (object_reg));
+  vfs->PushDir ();
+  vfs->ChDir ("/lib/kwartz");
+  if (!loader->LoadLibraryFile ("kwartz.lib"))
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
         "crystalspace.application.isotest",
-        "Error loading texture!");
+        "Error loading kwartz!");
     return false;
   }
-  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (
-    object_reg, "crystalspace.shared.stringset", iStringSet);
-  csRef<iShaderManager> shader_mgr = CS_QUERY_REGISTRY (object_reg,
-  	iShaderManager);
-  if (shader_mgr == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.isotest",
-        "Couldn't find shader manager! This application requires new renderer!"
-	);
-    return false;
-  }
-  iShader* ambient_shader = shader_mgr->GetShader ("ambient");
-  iShader* light_shader = shader_mgr->GetShader ("light");
-  if (ambient_shader == 0 || light_shader == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.isotest",
-        "Couldn't find shaders!");
-    return false;
-  }
-  csRef<iMaterial> fash_material = engine->CreateBaseMaterial (txt);
-  fash_material->SetShader (strings->Request ("ambient"), ambient_shader);
-  fash_material->SetShader (strings->Request ("diffuse"), light_shader);
-  engine->GetMaterialList ()->NewMaterial (fash_material, "vedette_fashion");
+  vfs->PopDir ();
 
-  // Load a sprite template from disk.
-  csRef<iMeshFactoryWrapper> imeshfact (
-    loader->LoadMeshObjectFactory ("/lev/isomap/vedette.spr"));
-  if (imeshfact == 0)
-  {
-    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.application.isotest",
-        "Error loading mesh object factory!");
-    return false;
-  }
-  csMatrix3 m; m.Identity (); m *= 1.10f; // scaling factor
-  imeshfact->HardTransform (csReversibleTransform (m, csVector3 (0)));
+  iMeshFactoryWrapper* imeshfact = engine->FindMeshFactory ("kwartz_fact");
+
+  //csMatrix3 m; m.Identity ();
+  //imeshfact->HardTransform (csReversibleTransform (m, csVector3 (0, -1, 0)));
 
   // Create the sprite and add it to the engine.
   actor = engine->CreateMeshWrapper (
-    imeshfact, "MySprite", room, csVector3 (-3, 2, 3));
+    imeshfact, "MySprite", room, csVector3 (-3, 1, 3));
   actor->GetMovable ()->UpdateMove ();
   csRef<iGeneralMeshState> spstate (
-    SCF_QUERY_INTERFACE (actor->GetMeshObject (), iGeneralMeshState));
+    scfQueryInterface<iGeneralMeshState> (actor->GetMeshObject ()));
   csRef<iGenMeshSkeletonControlState> animcontrol (
-    SCF_QUERY_INTERFACE (spstate->GetAnimationControl (), 
-    iGenMeshSkeletonControlState));
-  animcontrol->StopAll();
-  animcontrol->Execute("idle");
+     
+    scfQueryInterface<iGenMeshSkeletonControlState> (spstate->GetAnimationControl ()));
+  iSkeleton* skel = animcontrol->GetSkeleton ();
+  skel->StopAll();
+  skel->Execute("idle");
 
   // The following two calls are not needed since CS_ZBUF_USE and
   // Object render priority are the default but they show how you
@@ -477,7 +445,7 @@ bool IsoTest::Initialize ()
   }
 
   // The virtual clock.
-  vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
+  vc = csQueryRegistry<iVirtualClock> (object_reg);
   if (!vc)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -487,7 +455,7 @@ bool IsoTest::Initialize ()
   }
 
   // Find the pointer to engine plugin
-  engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  engine = csQueryRegistry<iEngine> (object_reg);
   if (!engine)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -496,7 +464,7 @@ bool IsoTest::Initialize ()
     return false;
   }
 
-  loader = CS_QUERY_REGISTRY (object_reg, iLoader);
+  loader = csQueryRegistry<iLoader> (object_reg);
   if (!loader)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -505,7 +473,7 @@ bool IsoTest::Initialize ()
     return false;
   }
 
-  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  g3d = csQueryRegistry<iGraphics3D> (object_reg);
   if (!g3d)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -514,7 +482,7 @@ bool IsoTest::Initialize ()
     return false;
   }
 
-  kbd = CS_QUERY_REGISTRY (object_reg, iKeyboardDriver);
+  kbd = csQueryRegistry<iKeyboardDriver> (object_reg);
   if (!kbd)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
