@@ -62,6 +62,7 @@
 #include "ivideo/material.h"
 #include "ivideo/natwin.h"
 #include "ivideo/txtmgr.h"
+#include "imap/ldrctxt.h"
 
 #include "apps/tests/csbench/csbench.h"
 
@@ -84,15 +85,7 @@ void CsBench::Report (const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (System->object_reg, iReporter));
-  if (rep)
-    rep->ReportV (CS_REPORTER_SEVERITY_NOTIFY, "csbench", msg, arg);
-  else
-  {
-    csPrintfV (msg, arg);
-    csPrintf ("\n");
-    fflush (stdout);
-  }
+  csReportV(object_reg, CS_REPORTER_SEVERITY_NOTIFY, "csbench", msg, arg);
   va_end (arg);
 }
 
@@ -100,7 +93,7 @@ bool CsBench::ReportError (const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (System->object_reg, iReporter));
+  csRef<iReporter> rep (csQueryRegistry<iReporter> (System->object_reg));
   if (rep)
     rep->ReportV (CS_REPORTER_SEVERITY_ERROR, "csbench", msg, arg);
   else
@@ -126,8 +119,8 @@ iMeshFactoryWrapper* CsBench::CreateGenmeshLattice (int dim, float size,
   // Create our object.
   csRef<iMeshFactoryWrapper> fact = engine->CreateMeshFactory (
     "crystalspace.mesh.object.genmesh", name);
-  csRef<iGeneralFactoryState> factstate = SCF_QUERY_INTERFACE (
-    fact->GetMeshObjectFactory (), iGeneralFactoryState);
+  csRef<iGeneralFactoryState> factstate = 
+    scfQueryInterface<iGeneralFactoryState> (fact->GetMeshObjectFactory ());
   factstate->SetVertexCount (dim * dim);
   factstate->SetTriangleCount (2 * (dim-1) * (dim-1));
   int x, y;
@@ -211,11 +204,9 @@ bool CsBench::CreateTestCaseSingleBigObject ()
   // Now create an instance:
   csRef<iMeshWrapper> mesh =
     engine->CreateMeshWrapper (fact, "complex", room2, csVector3 (0, 0, 10.0));
-  csRef<iGeneralMeshState> genmesh = 
-    SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iGeneralMeshState);
-  genmesh->SetMaterialWrapper (material);
-  
-  gmSingle = genmesh;
+
+  meshObject = mesh->GetMeshObject ();
+  meshObject->SetMaterialWrapper (material);
 
   csRef<iLight> l;
   iLightList* ll = room2->GetLights ();
@@ -257,9 +248,7 @@ bool CsBench::CreateTestCaseMultipleObjects ()
   {
     csRef<iMeshWrapper> mesh =
       engine->CreateMeshWrapper (fact, "small", room2, p);
-    csRef<iGeneralMeshState> genmesh = 
-      SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iGeneralMeshState);
-    genmesh->SetMaterialWrapper (material);
+    mesh->GetMeshObject()->SetMaterialWrapper (material);
     p.x += step;
     p.y += step;
     p.z += zstep;
@@ -295,7 +284,7 @@ bool CsBench::Initialize (int argc, const char* const argv[],
 
   // Make sure the commandline has -verbose and -console for consistent
   // results.
-  cmdline = CS_QUERY_REGISTRY (object_reg, iCommandLineParser);
+  cmdline = csQueryRegistry<iCommandLineParser> (object_reg);
   cmdline->AddOption ("verbose", "-scf");
   cmdline->AddOption ("console", 0);
 
@@ -315,19 +304,19 @@ bool CsBench::Initialize (int argc, const char* const argv[],
 	CS_REQUEST_END))
     return ReportError ("Couldn't init app!");
 
-  vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
-  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  vc = csQueryRegistry<iVirtualClock> (object_reg);
+  g3d = csQueryRegistry<iGraphics3D> (object_reg);
   if (!g3d) return ReportError ("No g3d plugin!");
-  engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  engine = csQueryRegistry<iEngine> (object_reg);
   if (!engine) return ReportError ("No engine plugin!");
-  loader = CS_QUERY_REGISTRY (object_reg, iLoader);
+  loader = csQueryRegistry<iLoader> (object_reg);
   if (!loader) return ReportError ("No loader plugin!");
-  imageio = CS_QUERY_REGISTRY (object_reg, iImageIO);
+  imageio = csQueryRegistry<iImageIO> (object_reg);
   if (!imageio) return ReportError ("No image loader plugin!");
-  vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+  vfs = csQueryRegistry<iVFS> (object_reg);
   if (!vfs) return ReportError ("No iVFS plugin!");
-  strings = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg, 
-    "crystalspace.shared.stringset", iStringSet);
+  strings = csQueryRegistryTagInterface<iStringSet> 
+    (object_reg, "crystalspace.shared.stringset");
   if (!strings) return ReportError ("No string set!");
 
   iGraphics2D* g2d = g3d->GetDriver2D ();
@@ -337,8 +326,8 @@ bool CsBench::Initialize (int argc, const char* const argv[],
   unlink ("csbench_report.zip");
   vfs->Mount ("/lib/report", "csbench_report.zip");
 
-  csRef<iStandardReporterListener> stdrep = CS_QUERY_REGISTRY (object_reg,
-  	iStandardReporterListener);
+  csRef<iStandardReporterListener> stdrep = 
+  	csQueryRegistry<iStandardReporterListener> (object_reg);
   if (!stdrep) return ReportError ("No stdrep plugin!");
   stdrep->SetDebugFile ("/tmp/csbench_report.txt");
   stdrep->SetMessageDestination (CS_REPORTER_SEVERITY_BUG,
@@ -431,7 +420,7 @@ iShaderManager* CsBench::GetShaderManager ()
 {
   if (!shader_mgr)
   {
-    shader_mgr = CS_QUERY_REGISTRY (object_reg, iShaderManager);
+    shader_mgr = csQueryRegistry<iShaderManager> (object_reg);
   }
   return shader_mgr;
 }
@@ -440,7 +429,7 @@ iDocumentSystem* CsBench::GetDocumentSystem ()
 {
   if (!docsys)
   {
-    docsys = CS_QUERY_REGISTRY(object_reg, iDocumentSystem);
+    docsys = csQueryRegistry<iDocumentSystem> (object_reg);
     if (docsys == 0)
     {
       docsys.AttachNew (new csTinyDocumentSystem ());
@@ -451,7 +440,7 @@ iDocumentSystem* CsBench::GetDocumentSystem ()
 
 void CsBench::PerformShaderTest (const char* shaderPath, const char* shtype, 
 				 const char* shaderPath2, const char* shtype2,
-				 iGeneralMeshState* genmesh)
+				 iMeshObject* mesh)
 {
   csRef<iShaderCompiler> shcom = GetShaderManager ()->GetCompiler ("XMLShader");
   csRef<iDocument> shaderDoc = GetDocumentSystem ()->CreateDocument ();
@@ -470,7 +459,8 @@ void CsBench::PerformShaderTest (const char* shaderPath, const char* shtype,
     csRef<iDocumentNode> shadernode2 = shaderDoc2->GetRoot ()
     	->GetNode ("shader");
     shadertype2 = strings->Request (shtype2);
-    shader2 = shcom->CompileShader (shadernode2);
+    csRef<iLoaderContext> ldr_context = engine->CreateLoaderContext ();
+    shader2 = shcom->CompileShader (ldr_context, shadernode2);
   }
 
   csRef<iShaderPriorityList> prilist = shcom->GetPriorities (shadernode);
@@ -482,11 +472,12 @@ void CsBench::PerformShaderTest (const char* shaderPath, const char* shtype,
   csRef<iMeshWrapper> walls (engine->FindMeshObject (
     view->GetCamera()->GetSector()->QueryObject()->GetName()));
   csRef<iThingState> ws =
-    SCF_QUERY_INTERFACE (walls->GetMeshObject (), iThingState);
+    scfQueryInterface<iThingState> (walls->GetMeshObject ());
   for (i = 0 ; i < prilist->GetCount () ; i++)
   {
     int pri = prilist->GetPriority (i);
-    csRef<iShader> shader = shcom->CompileShader (shadernode, pri);
+    csRef<iLoaderContext> ldr_context = engine->CreateLoaderContext ();
+    csRef<iShader> shader = shcom->CompileShader (ldr_context, shadernode, pri);
     if (shader)
     {
       csRef<iMaterial> matinput = engine->CreateBaseMaterial (
@@ -500,7 +491,7 @@ void CsBench::PerformShaderTest (const char* shaderPath, const char* shtype,
 	matinput->SetShader (shadertype2, shader2);
       iMaterialWrapper* mat = engine->GetMaterialList ()->NewMaterial (
       	matinput, 0);
-      genmesh->SetMaterialWrapper (mat);
+      mesh->SetMaterialWrapper (mat);
       ws->ReplaceMaterial (oldmat, mat);
 
       csString name;
@@ -553,14 +544,14 @@ void CsBench::PerformTests ()
 
   view->GetCamera ()->SetSector (room_single);
   vfs->PushDir("/shader");
-  PerformShaderTest ("/shader/std_lighting.xml", "standard", 0, 0, gmSingle);
+  PerformShaderTest ("/shader/std_lighting.xml", "standard", 0, 0, meshObject);
 
   iRenderLoopManager* rlmgr = engine->GetRenderLoopManager ();
   csRef<iRenderLoop> loop = rlmgr->Load ("/shader/std_rloop_diffuse.xml");
   engine->SetCurrentDefaultRenderloop (loop);
 
   PerformShaderTest ("/shader/light_bumpmap.xml", "diffuse", 
-    "/shader/ambient.xml", "ambient", gmSingle);
+    "/shader/ambient.xml", "ambient", meshObject);
   vfs->PopDir ();
 }
 
