@@ -21,7 +21,7 @@
 #include <ctype.h>
 
 #include "cstool/initapp.h"
-#include "csutil/ansicolor.h"
+#include "csutil/ansicommand.h"
 #include "csutil/csstring.h"
 #include "csutil/databuf.h"
 #include "csutil/ref.h"
@@ -439,7 +439,7 @@ static void cmd_config (char *args)
 {
   bool real_fs;
   get_option (args, real_fs);
-  iVFS *CfgVFS = real_fs ? 0 : VFS;
+  iVFS *CfgVFS = real_fs ? (iVFS*)0 : (iVFS*)VFS;
 
   iConfigFile *config =
     Cfg->AddDomain (args, CfgVFS, iConfigManager::ConfigPriorityCmdLine);
@@ -605,14 +605,14 @@ int main (int argc, char *argv [])
 	CS_REQUEST_END))
     return -1;
 
-  VFS = CS_QUERY_REGISTRY (object_reg, iVFS);
+  VFS = csQueryRegistry<iVFS> (object_reg);
   if (!VFS)
   {
     csPrintfErr ("Cannot load iVFS plugin\n");
     return -1;
   }
 
-  Cfg = CS_QUERY_REGISTRY (object_reg, iConfigManager);
+  Cfg = csQueryRegistry<iConfigManager> (object_reg);
   if (!Cfg)
   {
     csPrintfErr ("Cannot load iConfigManager plugin\n");
@@ -628,7 +628,7 @@ int main (int argc, char *argv [])
   while (!ShutDown)
   {
     char command [999];
-    csPrintf (CS_ANSI_FI CS_ANSI_FM "%s " CS_ANSI_FI_OFF 
+    csPrintf (CS_ANSI_TEXT_BOLD_ON CS_ANSI_FM "%s " CS_ANSI_TEXT_BOLD_OFF 
       CS_ANSI_FG "#" CS_ANSI_RST " ", VFS->GetCwd ());
     fflush (stdout);
     if (!fgets (command, sizeof(command), stdin))
