@@ -30,13 +30,14 @@
 #include "imesh/object.h"
 #include "csutil/csuctransform.h"
 #include "csutil/scanstr.h"
+#include "csutil/csstring.h"
+#include "csutil/eventnames.h"
 #include "csgeom/math3d.h"
 #include "cstool/collider.h"
 #include "cstool/cspixmap.h"
+#include "cstool/enginetools.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/graph2d.h"
-#include "isound/wrapper.h"
-#include "isound/data.h"
 #include "ivaria/reporter.h"
 #include "ivaria/view.h"
 #include "iutil/event.h"
@@ -555,27 +556,20 @@ void WalkTest::MouseClick1Handler(iEvent &Event)
 // right mouse button
 void WalkTest::MouseClick2Handler(iEvent &Event)
 {
-  csVector3 v;
-  csVector2 p (csMouseEventHelper::GetX(&Event), 
-	       FRAME_HEIGHT-csMouseEventHelper::GetY(&Event));
+  int mx = csMouseEventHelper::GetX (&Event);
+  int my = csMouseEventHelper::GetY (&Event);
+  csScreenTargetResult result = csEngineTools::FindScreenTarget (
+      csVector2 (mx, my), 100.0f, view->GetCamera ());
+  iMeshWrapper* mesh = result.mesh;
+  int sel = result.polygon_idx;
 
-  view->GetCamera ()->InvPerspective (p, 1, v);
-  csVector3 vw = view->GetCamera ()->GetTransform ().This2Other (v);
-
-  iSector* sector = view->GetCamera ()->GetSector ();
-  csVector3 origin = view->GetCamera ()->GetTransform ().GetO2TTranslation ();
-  csVector3 isect;
-  int sel;
-  iMeshWrapper* mesh = sector->HitBeamPortals (origin,
-  	origin + (vw-origin) * 20, isect, &sel);
-
-  vw = isect;
-  v = view->GetCamera ()->GetTransform ().Other2This (vw);
+  csVector3 vw = result.isect;
+  csVector3 v = view->GetCamera ()->GetTransform ().Other2This (vw);
   Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
-  	"LMB down : cam:(%f,%f,%f) world:(%f,%f,%f)",
+  	"RMB down : cam:(%f,%f,%f) world:(%f,%f,%f)",
 	v.x, v.y, v.z, vw.x, vw.y, vw.z);
   Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
-  	"LMB down : cam:(%f,%f,%f) world:(%f,%f,%f)",
+  	"RMB down : cam:(%f,%f,%f) world:(%f,%f,%f)",
 	v.x, v.y, v.z, vw.x, vw.y, vw.z);
 
   if (mesh && sel != -1)
@@ -586,13 +580,24 @@ void WalkTest::MouseClick2Handler(iEvent &Event)
     Sys->selected_polygon = sel;
 
     iMeshObject* obj = mesh->GetMeshObject ();
-    csRef<iObject> psobj = SCF_QUERY_INTERFACE (obj->GetMeshWrapper (),
-    	iObject);
+    csRef<iObject> psobj = 
+    	scfQueryInterface<iObject> (obj->GetMeshWrapper ());
     Sys->Report (CS_REPORTER_SEVERITY_DEBUG, "Hit polygon '%s/%s'",
     	psobj ? psobj->GetName () : "<null>",
 	ps ? ps->GetPolygonName (sel) : "<null>");
     //Dumper::dump (sel);
   }
+  else if (mesh)
+  {
+    csRef<iObject> psobj = 
+    	scfQueryInterface<iObject> (mesh->GetMeshObject ()->GetMeshWrapper ());
+    Sys->Report (CS_REPORTER_SEVERITY_DEBUG, "Hit mesh '%s'",
+    	psobj ? psobj->GetName () : "<null>");
+  }
+  else
+  {
+    Sys->Report (CS_REPORTER_SEVERITY_DEBUG, "No hit");
+  }
 
   extern csVector2 coord_check_vector;
   coord_check_vector.x = csMouseEventHelper::GetX(&Event);
@@ -672,13 +677,16 @@ bool WalkTest::WalkHandleEvent (iEvent &Event)
 	  int last_x, last_y;
 	  last_x = csMouseEventHelper::GetX(&Event);
 	  last_y = csMouseEventHelper::GetY(&Event);
+	  float speed = 6.0f;
   	
 	  myG2D->SetMousePosition (FRAME_WIDTH / 2, FRAME_HEIGHT / 2);
 	  if (!first_time)
 	  {
 	    RotateCam (
-		      -((float)(last_y - (FRAME_HEIGHT / 2) )) / (FRAME_HEIGHT*2)*(1-2*(int)inverse_mouse),
-		      ((float)(last_x - (FRAME_WIDTH / 2) )) / (FRAME_WIDTH*2));
+		      speed * (-((float)(last_y - (FRAME_HEIGHT / 2) ))
+		      		/ (FRAME_HEIGHT*2)*(1-2*(int)inverse_mouse)),
+		      speed * (((float)(last_x - (FRAME_WIDTH / 2) ))
+		      		/ (FRAME_WIDTH*2)));
 	  }
 	  else
 	    first_time = false;
@@ -713,14 +721,13 @@ iMeshWrapper *FindNextClosestMesh (iMeshWrapper *baseMesh,
   float closestZLocation;
   iMeshWrapper *closestMesh;
   iMeshWrapper *nextMesh;
-  csBox2 screenBoundingBox;
-  csBox3 bbox3;
 
   if (baseMesh)
   {
     closestMesh = baseMesh;
-    closestZLocation = baseMesh->GetScreenBoundingBox
-    	(camera, screenBoundingBox, bbox3);
+    csScreenBoxResult box = baseMesh->GetScreenBoundingBox
+    	(camera);
+    closestZLocation = box.distance; 
     // if the baseMesh isn't in front of the camera, return
     if (closestZLocation < 0)
       return 0;
@@ -739,11 +746,11 @@ iMeshWrapper *FindNextClosestMesh (iMeshWrapper *baseMesh,
 
     if (nextMesh != baseMesh)
     {
-      thisZLocation = nextMesh->GetScreenBoundingBox(camera,
-      	screenBoundingBox, bbox3);
+      csScreenBoxResult nextBox = nextMesh->GetScreenBoundingBox(camera);
+      thisZLocation = nextBox.distance;
       if ((thisZLocation > 0) && (thisZLocation < closestZLocation))
       {
-        if (screenBoundingBox.In(screenCoord->x, screenCoord->y))
+        if (nextBox.sbox.In(screenCoord->x, screenCoord->y))
         {
           closestZLocation = thisZLocation;
           closestMesh = nextMesh;
