@@ -41,6 +41,7 @@
 #include "iengine/sector.h"
 #include "iengine/sharevar.h"
 #include "iengine/scenenode.h"
+#include "iengine/campos.h"
 #include "igeom/clip2d.h"
 #include "igraphic/imageio.h"
 #include "imap/loader.h"
@@ -48,18 +49,16 @@
 #include "imesh/object.h"
 #include "imesh/sprite3d.h"
 #include "imesh/thing.h"
-#include "isound/handle.h"
-#include "isound/listener.h"
-#include "isound/renderer.h"
-#include "isound/source.h"
-#include "isound/wrapper.h"
+#include "isndsys.h"
 #include "iutil/databuff.h"
 #include "iutil/pluginconfig.h"
 #include "iutil/vfs.h"
+#include "ivaria/bugplug.h"
 #include "ivaria/mapnode.h"
 #include "ivaria/pmeter.h" 
 #include "ivaria/reporter.h"
 #include "ivaria/view.h"
+#include "ivaria/engseq.h"
 #include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/material.h"
@@ -89,27 +88,27 @@ void SaveRecording (iVFS* vfs, const char* fName)
 {
   csRef<iFile> cf;
   cf = vfs->Open (fName, VFS_FILE_WRITE);
-  uint32 l = (int32)Sys->recording.Length();
-  l = csConvertEndian (l);
+  uint32 l = (int32)Sys->recording.GetSize ();
+  l = csLittleEndian::Convert (l);
   cf->Write ((char*)&l, sizeof (l));
   size_t i;
   csRecordedCameraFile camint;
   iSector* prev_sector = 0;
-  for (i = 0 ; i < Sys->recording.Length () ; i++)
+  for (i = 0 ; i < Sys->recording.GetSize () ; i++)
   {
     csRecordedCamera* reccam = (csRecordedCamera*)Sys->recording[i];
-    camint.m11 = csConvertEndian (csFloatToLong (reccam->mat.m11));
-    camint.m12 = csConvertEndian (csFloatToLong (reccam->mat.m12));
-    camint.m13 = csConvertEndian (csFloatToLong (reccam->mat.m13));
-    camint.m21 = csConvertEndian (csFloatToLong (reccam->mat.m21));
-    camint.m22 = csConvertEndian (csFloatToLong (reccam->mat.m22));
-    camint.m23 = csConvertEndian (csFloatToLong (reccam->mat.m23));
-    camint.m31 = csConvertEndian (csFloatToLong (reccam->mat.m31));
-    camint.m32 = csConvertEndian (csFloatToLong (reccam->mat.m32));
-    camint.m33 = csConvertEndian (csFloatToLong (reccam->mat.m33));
-    camint.x = csConvertEndian (csFloatToLong (reccam->vec.x));
-    camint.y = csConvertEndian (csFloatToLong (reccam->vec.y));
-    camint.z = csConvertEndian (csFloatToLong (reccam->vec.z));
+    camint.m11 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m11));
+    camint.m12 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m12));
+    camint.m13 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m13));
+    camint.m21 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m21));
+    camint.m22 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m22));
+    camint.m23 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m23));
+    camint.m31 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m31));
+    camint.m32 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m32));
+    camint.m33 = csLittleEndian::Convert (csFloatToLong (reccam->mat.m33));
+    camint.x = csLittleEndian::Convert (csFloatToLong (reccam->vec.x));
+    camint.y = csLittleEndian::Convert (csFloatToLong (reccam->vec.y));
+    camint.z = csLittleEndian::Convert (csFloatToLong (reccam->vec.z));
     camint.mirror = reccam->mirror;
     cf->Write ((char*)&camint, sizeof (camint));
     unsigned char len;
@@ -136,7 +135,7 @@ void SaveRecording (iVFS* vfs, const char* fName)
     }
     else
     {
-      len = 100;
+      len = 254;
       cf->Write ((char*)&len, 1);
     }
     if (reccam->arg)
@@ -148,7 +147,7 @@ void SaveRecording (iVFS* vfs, const char* fName)
     }
     else
     {
-      len = 100;
+      len = 254;
       cf->Write ((char*)&len, 1);
     }
   }
@@ -161,10 +160,10 @@ void LoadRecording (iVFS* vfs, const char* fName)
   cf = vfs->Open (fName, VFS_FILE_READ);
   if (!cf) return;
   Sys->recording.DeleteAll ();
-  Sys->recording.SetLength (0);
+  Sys->recording.SetSize (0);
   int32 l;
   cf->Read ((char*)&l, sizeof (l));
-  l = csConvertEndian (l);
+  l = csLittleEndian::Convert (l);
   csRecordedCameraFile camint;
   iSector* prev_sector = 0;
   int i;
@@ -172,18 +171,18 @@ void LoadRecording (iVFS* vfs, const char* fName)
   {
     csRecordedCamera* reccam = new csRecordedCamera ();
     cf->Read ((char*)&camint, sizeof (camint));
-    reccam->mat.m11 = csLongToFloat (csConvertEndian (camint.m11));
-    reccam->mat.m12 = csLongToFloat (csConvertEndian (camint.m12));
-    reccam->mat.m13 = csLongToFloat (csConvertEndian (camint.m13));
-    reccam->mat.m21 = csLongToFloat (csConvertEndian (camint.m21));
-    reccam->mat.m22 = csLongToFloat (csConvertEndian (camint.m22));
-    reccam->mat.m23 = csLongToFloat (csConvertEndian (camint.m23));
-    reccam->mat.m31 = csLongToFloat (csConvertEndian (camint.m31));
-    reccam->mat.m32 = csLongToFloat (csConvertEndian (camint.m32));
-    reccam->mat.m33 = csLongToFloat (csConvertEndian (camint.m33));
-    reccam->vec.x = csLongToFloat (csConvertEndian (camint.x));
-    reccam->vec.y = csLongToFloat (csConvertEndian (camint.y));
-    reccam->vec.z = csLongToFloat (csConvertEndian (camint.z));
+    reccam->mat.m11 = csLongToFloat (csLittleEndian::Convert (camint.m11));
+    reccam->mat.m12 = csLongToFloat (csLittleEndian::Convert (camint.m12));
+    reccam->mat.m13 = csLongToFloat (csLittleEndian::Convert (camint.m13));
+    reccam->mat.m21 = csLongToFloat (csLittleEndian::Convert (camint.m21));
+    reccam->mat.m22 = csLongToFloat (csLittleEndian::Convert (camint.m22));
+    reccam->mat.m23 = csLongToFloat (csLittleEndian::Convert (camint.m23));
+    reccam->mat.m31 = csLongToFloat (csLittleEndian::Convert (camint.m31));
+    reccam->mat.m32 = csLongToFloat (csLittleEndian::Convert (camint.m32));
+    reccam->mat.m33 = csLongToFloat (csLittleEndian::Convert (camint.m33));
+    reccam->vec.x = csLongToFloat (csLittleEndian::Convert (camint.x));
+    reccam->vec.y = csLongToFloat (csLittleEndian::Convert (camint.y));
+    reccam->vec.z = csLongToFloat (csLittleEndian::Convert (camint.z));
     reccam->mirror = (camint.mirror != 0);
     unsigned char len;
     cf->Read ((char*)&len, 1);
@@ -194,15 +193,16 @@ void LoadRecording (iVFS* vfs, const char* fName)
     }
     else
     {
-      char buf[100];
+      char* buf = new char[1+len];
       cf->Read (buf, 1+len);
       s = Sys->Engine->GetSectors ()->FindByName (buf);
+      delete[] buf;
     }
     reccam->sector = s;
     prev_sector = s;
 
     cf->Read ((char*)&len, 1);
-    if (len == 100)
+    if (len == 254)
     {
       reccam->cmd = 0;
     }
@@ -212,7 +212,7 @@ void LoadRecording (iVFS* vfs, const char* fName)
       cf->Read (reccam->cmd, 1+len);
     }
     cf->Read ((char*)&len, 1);
-    if (len == 100)
+    if (len == 254)
     {
       reccam->arg = 0;
     }
@@ -248,7 +248,7 @@ void WalkTest::SaveCamera (const char *fName)
     c->GetSector ()->QueryObject ()->GetName (), 
     int(c->IsMirrored ()));
 
-  myVFS->WriteFile (fName, s.GetData(), s.Length());
+  myVFS->WriteFile (fName, s.GetData(), s.Length ());
 }
 
 bool WalkTest::LoadCamera (const char *fName)
@@ -405,7 +405,7 @@ void list_meshes (void)
 
 void SetConfigOption (iBase* plugin, const char* optName, const char* optValue)
 {
-  csRef<iPluginConfig> config (SCF_QUERY_INTERFACE (plugin, iPluginConfig));
+  csRef<iPluginConfig> config (scfQueryInterface<iPluginConfig> (plugin));
   if (!config)
     Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
     	"No config interface for this plugin.");
@@ -456,7 +456,7 @@ void SetConfigOption (iBase* plugin, const char* optName, const char* optValue)
 
 void SetConfigOption (iBase* plugin, const char* optName, csVariant& optValue)
 {
-  csRef<iPluginConfig> config (SCF_QUERY_INTERFACE (plugin, iPluginConfig));
+  csRef<iPluginConfig> config (scfQueryInterface<iPluginConfig> (plugin));
   if (!config)
     Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
     	"No config interface for this plugin.");
@@ -478,7 +478,7 @@ void SetConfigOption (iBase* plugin, const char* optName, csVariant& optValue)
 
 bool GetConfigOption (iBase* plugin, const char* optName, csVariant& optValue)
 {
-  csRef<iPluginConfig> config (SCF_QUERY_INTERFACE (plugin, iPluginConfig));
+  csRef<iPluginConfig> config (scfQueryInterface<iPluginConfig> (plugin));
   if (!config)
     Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
     	"No config interface for this plugin.");
@@ -507,7 +507,7 @@ csString LookForKeyValue (iObjectIterator* it,const char* key)
   it->Reset();
   while (it->HasNext())
   {
-    csRef<iKeyValuePair> kp = SCF_QUERY_INTERFACE(it->Next(),iKeyValuePair);
+    csRef<iKeyValuePair> kp = scfQueryInterface<iKeyValuePair> (it->Next());
     if(!kp)
       continue;
     if(!strcmp(key,kp->GetKey()))
@@ -609,7 +609,7 @@ void RegisterMaterials(iObjectIterator* it,iEngine* Engine,
 
   while(it->HasNext())
   {
-    kp = SCF_QUERY_INTERFACE(it->Next(),iKeyValuePair);
+    kp = scfQueryInterface<iKeyValuePair> (it->Next());
     if(!kp)
     {
       continue;
@@ -679,14 +679,14 @@ void BuildSprite(iSector * sector, iObjectIterator* it, csVector3 position)
   iMeshWrapper* sprite = GenerateSprite((char*)(const char*)factName,
   	(char*)(const char*)sprName,sector,position);
 
-  csRef<iSprite3DState> state (SCF_QUERY_INTERFACE(sprite->GetMeshObject(),
-                          iSprite3DState));
+  csRef<iSprite3DState> state (
+                          scfQueryInterface<iSprite3DState> (sprite->GetMeshObject()));
   state->SetAction("default");
 }
 
 
 
-void BuildObject(iObjectRegistry* object_reg, iSector * sector,
+void BuildObject(iSector * sector,
 	iObjectIterator* it, iEngine* Engine,
 	csVector3 position, iGraphics3D* MyG3D, iLoader* loader,
 	iObjectRegistry* objReg)
@@ -709,19 +709,19 @@ void BuildObject(iObjectRegistry* object_reg, iSector * sector,
 void WalkTest::ParseKeyNodes(iObject* src)
 {
   csRef<iObjectIterator> it (src->GetIterator());
-  csRef<iSector> sector (SCF_QUERY_INTERFACE(src,iSector));
+  csRef<iSector> sector (scfQueryInterface<iSector> (src));
 
   while(it->HasNext())
   {
     iObject* node_obj = it->Next ();
-    csRef<iMapNode> node (SCF_QUERY_INTERFACE(node_obj, iMapNode));
+    csRef<iMapNode> node (scfQueryInterface<iMapNode> (node_obj));
     if(!node)
     {
       continue;
     }
     csRef<iObjectIterator> it2 (node_obj->GetIterator());
 
-    BuildObject(object_reg, sector,it2, Engine, node->GetPosition(), myG3D,
+    BuildObject(sector, it2, Engine, node->GetPosition(), myG3D,
 		LevelLoader, object_reg);
   }
 }
@@ -732,14 +732,14 @@ void WalkTest::ParseKeyCmds (iObject* src)
   while (it->HasNext ())
   {
     csRef<iKeyValuePair> kp (
-    	SCF_QUERY_INTERFACE (it->Next (), iKeyValuePair));
+    	scfQueryInterface<iKeyValuePair> (it->Next ()));
     if (!kp)
     {
       continue;
     }
     if (!strcmp (kp->GetKey (), "cmd_AnimateSky"))
     {
-      csRef<iSector> Sector (SCF_QUERY_INTERFACE (src, iSector));
+      csRef<iSector> Sector (scfQueryInterface<iSector> (src));
       if (Sector)
       {
         char name[100], rot[100];
@@ -752,23 +752,22 @@ void WalkTest::ParseKeyCmds (iObject* src)
     }
     else if (!strcmp (kp->GetKey (), "cmd_AnimateDirLight"))
     {
-      csRef<iMeshWrapper> wrap (SCF_QUERY_INTERFACE (src, iMeshWrapper));
+      csRef<iMeshWrapper> wrap = scfQueryInterface<iMeshWrapper> (src);
       if (wrap)
         anim_dirlight = wrap;	// @@@ anim_dirlight should be csRef
     }
     else if (!strcmp (kp->GetKey (), "entity_WavePortal"))
     {
-      csRef<iMeshWrapper> wrap = SCF_QUERY_INTERFACE (src, iMeshWrapper);
+      csRef<iMeshWrapper> wrap = scfQueryInterface<iMeshWrapper> (src);
       if (wrap)
       {
-	const csRefArray<iSceneNode>& ml = wrap->QuerySceneNode ()
-		->GetChildren ();
+        const csRef<iSceneNodeArray> ml = 
+          wrap->QuerySceneNode ()->GetChildrenArray ();
 	size_t i;
 	iMeshWrapper* pcmesh = 0;
-	for (i = 0 ; i < ml.Length () ; i++)
-	  if (ml[i]->QueryMesh ())
+        for (i = 0 ; i < ml->GetSize() ; i++)
+          if ((pcmesh = ml->Get(i)->QueryMesh ()))
 	  {
-	    pcmesh = ml[i]->QueryMesh ();
 	    break;
 	  }
 	if (pcmesh)
@@ -820,14 +819,15 @@ void WalkTest::ParseKeyCmds (iObject* src)
     }
   }
 
-  csRef<iMeshWrapper> mesh (SCF_QUERY_INTERFACE (src, iMeshWrapper));
+  csRef<iMeshWrapper> mesh (scfQueryInterface<iMeshWrapper> (src));
   if (mesh)
   {
     size_t k;
-    const csRefArray<iSceneNode>& ml = mesh->QuerySceneNode ()->GetChildren ();
-    for (k = 0 ; k < ml.Length () ; k++)
+    const csRef<iSceneNodeArray> ml = 
+      mesh->QuerySceneNode ()->GetChildrenArray ();
+    for (k = 0 ; k < ml->GetSize() ; k++)
     {
-      iMeshWrapper* spr = ml[k]->QueryMesh ();
+      iMeshWrapper* spr = ml->Get(k)->QueryMesh ();
       // @@@ Also for others?
       if (spr)
         ParseKeyCmds (spr->QueryObject ());
@@ -865,8 +865,8 @@ void WalkTest::ActivateObject (iObject* src)
   csRef<iObjectIterator> it (src->GetIterator ());
   while (it->HasNext ())
   {
-    csRef<csWalkEntity> wentity (SCF_QUERY_INTERFACE (it->Next (),
-    	csWalkEntity));
+    csRef<csWalkEntity> wentity (
+    	scfQueryInterface<csWalkEntity> (it->Next ()));
     if (wentity)
       wentity->Activate ();
   }
@@ -910,7 +910,7 @@ bool CommandHandler (const char *cmd, const char *arg)
 #   define CONPRI(m) Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, m);
     CONPRI("-*- Additional commands -*-");
     CONPRI("Visibility:");
-    CONPRI("  db_frustum farplane");
+    CONPRI("  farplane");
     CONPRI("Lights:");
     CONPRI("  addlight dellight dellights addstlight delstlight");
     CONPRI("  clrlights setlight relight");
@@ -923,23 +923,22 @@ bool CommandHandler (const char *cmd, const char *arg)
     CONPRI("  i_rotleftw i_rotrightc i_rotrightw i_rotleftx i_rotleftz");
     CONPRI("  i_rotrightx i_rotrightz do_gravity colldet freelook");
     CONPRI("Statistics:");
-    CONPRI("  stats perftest coordshow");
+    CONPRI("  perftest coordshow");
     CONPRI("Special effects:");
-    CONPRI("  addmbot delmbot addbot delbot fire explosion spiral frain");
+    CONPRI("  addmbot delmbot addbot delbot fire explosion frain decal_test");
     CONPRI("  rain snow fountain flame portal fs_inter fs_fadeout fs_fadecol");
     CONPRI("  fs_fadetxt fs_red fs_green fs_blue fs_whiteout fs_shadevert");
     CONPRI("Debugging:");
-    CONPRI("  hi zbuf debug0 debug1 debug2 palette bugplug");
+    CONPRI("  zbuf debug0 debug1 debug2 palette bugplug");
     CONPRI("  db_boxshow db_boxcam1 db_boxcam2 db_boxsize1 db_boxsize2");
-    CONPRI("  db_radstep db_radhi db_radtodo");
     CONPRI("Meshes:");
     CONPRI("  loadmesh addmesh delmesh listmeshes");
     CONPRI("  listactions setaction");
     CONPRI("Various:");
-    CONPRI("  coordsave coordload bind p_alpha s_fog");
+    CONPRI("  coordsave coordload bind s_fog");
     CONPRI("  snd_play snd_volume record play playonce clrrec saverec");
-    CONPRI("  loadrec action plugins conflist confset do_logo");
-    CONPRI("  varlist var setvar setvarv setvarc");
+    CONPRI("  loadrec plugins conflist confset do_logo");
+    CONPRI("  varlist var setvar setvarv setvarc loadmap");
     CONPRI("  saveworld");
 
 #   undef CONPRI
@@ -965,7 +964,7 @@ bool CommandHandler (const char *cmd, const char *arg)
     while (it->HasNext ())
     {
       iBase* plugin = it->Next ();
-      csRef<iFactory> fact (SCF_QUERY_INTERFACE (plugin, iFactory));
+      csRef<iFactory> fact (scfQueryInterface<iFactory> (plugin));
       Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
       	"%d: %s", i, fact->QueryDescription ());
       i++;
@@ -1135,7 +1134,7 @@ bool CommandHandler (const char *cmd, const char *arg)
 		"Bad value for plugin (see 'plugins' command)!");
       else
       {
-        csRef<iPluginConfig> config (SCF_QUERY_INTERFACE (plugin, iPluginConfig));
+        csRef<iPluginConfig> config (scfQueryInterface<iPluginConfig> (plugin));
 	if (!config)
 	  Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
 	  	"No config interface for this plugin.");
@@ -1212,24 +1211,6 @@ bool CommandHandler (const char *cmd, const char *arg)
       Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
       	"Expected index to plugin (from 'plugins' command)!");
   }
-  else if (!csStrCaseCmp (cmd, "action"))
-  {
-    csVector3 where = Sys->view->GetCamera ()->GetTransform ().This2Other (
-    	3.0f*CS_VEC_FORWARD);
-    int pidx;
-    csVector3 isect;
-    iMeshWrapper* mesh = 
-      Sys->view->GetCamera ()->GetSector ()->HitBeamPortals (
-      Sys->view->GetCamera ()->GetTransform ().GetOrigin (), where, isect, &pidx);
-
-    if (mesh && pidx != -1)
-    {
-      //Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
-      	//"Action polygon '%s'", p->GetStaticData ()->GetName ());
-      csPrintf ("ACTION\n");
-      //Sys->ActivateObject ((csObject*)(ob->QueryObject ()));
-    }
-  }
   else if (!csStrCaseCmp (cmd, "saverec"))
   {
     if (arg)
@@ -1255,7 +1236,7 @@ bool CommandHandler (const char *cmd, const char *arg)
   else if (!csStrCaseCmp (cmd, "clrrec"))
   {
     Sys->recording.DeleteAll ();
-    Sys->recording.SetLength (0);
+    Sys->recording.SetSize (0);
   }
   else if (!csStrCaseCmp (cmd, "record"))
   {
@@ -1365,51 +1346,6 @@ bool CommandHandler (const char *cmd, const char *arg)
     csCommandProcessor::change_float (arg, &size, "box2 size", 0.01f, 1000);
     Sys->debug_box2.SetSize (csVector3 (size, size, size));
   }
-  else if (!csStrCaseCmp (cmd, "db_frustum"))
-    csCommandProcessor::change_int (arg, &Sys->cfg_debug_check_frustum, "debug check frustum", 0, 2000000000);
-  else if (!csStrCaseCmp (cmd, "db_radstep"))
-  {
-#if 0
-    csRadiosity* rad;
-    csEngine* engine = (csEngine*)(iEngine*)(Sys->Engine);
-    if ((rad = engine->GetRadiosity ()) != 0)
-    {
-      int steps;
-      if (arg)
-        sscanf (arg, "%d", &steps);
-      else
-        steps = 1;
-      if (steps < 1) steps = 1;
-      rad->DoRadiosityStep (steps);
-      engine->InvalidateLightmaps ();
-    }
-#endif
-  }
-  else if (!csStrCaseCmp (cmd, "db_radtodo"))
-  {
-#if 0
-    csRadiosity* rad;
-    csEngine* engine = (csEngine*)(iEngine*)(Sys->Engine);
-    if ((rad = engine->GetRadiosity ()) != 0)
-    {
-      rad->ToggleShowDeltaMaps ();
-      engine->InvalidateLightmaps ();
-    }
-#endif
-  }
-  else if (!csStrCaseCmp (cmd, "db_radhi"))
-  {
-#if 0
-    csRadiosity* rad;
-    csEngine* engine = (csEngine*)(iEngine*)(Sys->Engine);
-    if ((rad = engine->GetRadiosity ()) != 0)
-    {
-      csRef<iPolygon3D> p (SCF_QUERY_INTERFACE (rad->GetNextPolygon (),
-      	iPolygon3D));
-      Sys->selected_polygon = p;
-    }
-#endif
-  }
   else if (!csStrCaseCmp (cmd, "palette"))
     csCommandProcessor::change_boolean (arg, &Sys->do_show_palette, "palette");
   else if (!csStrCaseCmp (cmd, "move3d"))
@@ -1434,43 +1370,14 @@ bool CommandHandler (const char *cmd, const char *arg)
   {
     csCommandProcessor::change_boolean (arg, &Sys->do_show_coord, "coordshow");
   }
-  else if (!csStrCaseCmp (cmd, "hi"))
-  {
-#if 0
-//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-    csPolygon3D* hi = arg ? Sys->view->GetCamera ()->GetSector ()->GetPolygon3D (arg) : (csPolygon3D*)0;
-    if (hi) Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
-    	"Hilighting polygon: '%s'", arg);
-    else Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Disabled hilighting.");
-    Sys->selected_polygon = hi;
-#endif
-  }
-  else if (!csStrCaseCmp (cmd, "p_alpha"))
-  {
-#if 0
-//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-    csPolygon3D* hi = Sys->selected_polygon;
-    if (hi)
-    {
-      if (hi->GetPortal ())
-      {
-        int a = hi->GetAlpha ();
-        csCommandProcessor::change_int (arg, &a, "portal alpha", 0, 100);
-	hi->SetAlpha (a);
-      }
-      else Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Only for portals!");
-    }
-    else Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "No polygon selected!");
-#endif
-  }
   else if (!csStrCaseCmp (cmd, "s_fog"))
   {
-    csFog* f = Sys->view->GetCamera ()->GetSector ()->GetFog ();
     if (!arg)
     {
+      const csFog& f = Sys->view->GetCamera ()->GetSector ()->GetFog ();
       Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
       	"Fog in current sector (%f,%f,%f) density=%f",
-      	f->red, f->green, f->blue, f->density);
+      	f.color.red, f.color.green, f.color.blue, f.density);
     }
     else
     {
@@ -1481,18 +1388,81 @@ bool CommandHandler (const char *cmd, const char *arg)
 		"Expected r,g,b,density. Got something else!");
         return false;
       }
-      f->enabled = true;
-      f->density = dens;
-      f->red = r;
-      f->green = g;
-      f->blue = b;
+      csFog f;
+      f.density = dens;
+      f.color.red = r;
+      f.color.green = g;
+      f.color.blue = b;
+      f.mode = CS_FOG_MODE_CRYSTALSPACE;
+      Sys->view->GetCamera ()->GetSector ()->SetFog (f);
+    }
+  }
+  else if (!csStrCaseCmp (cmd, "loadmap"))
+  {
+    if (arg)
+    {
+      char level[300];
+      csScanStr (arg, "%s", level);
+      if (!Sys->SetMapDir (level))
+      {
+        Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
+      	  "Couldn't open level '%s'!", level);
+	return false;
+      }
+      csRef<iEngineSequenceManager> engseq = csQueryRegistry<
+	iEngineSequenceManager> (Sys->object_reg);
+      if (engseq)
+      {
+	engseq->RemoveTriggers ();
+	engseq->RemoveSequences ();
+      }
+      Sys->Engine->DeleteAll ();
+      Sys->Engine->SetVFSCacheManager ();
+      if (!Sys->LevelLoader->LoadMapFile ("world"))
+      {
+        Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
+      	  "Couldn't load level '%s'!", level);
+	return false;
+      }
+      Sys->Engine->Prepare ();
+      // Look for the start sector in this map.
+      bool camok = false;
+      if (!camok && Sys->Engine->GetCameraPositions ()->GetCount () > 0)
+      {
+        iCameraPosition *cp = Sys->Engine->GetCameraPositions ()->Get (0);
+        if (cp->Load(Sys->views[0]->GetCamera (), Sys->Engine) &&
+	    cp->Load(Sys->views[1]->GetCamera (), Sys->Engine))
+	  camok = true;
+      }
+      if (!camok)
+      {
+        iSector* room = Sys->Engine->GetSectors ()->FindByName ("room");
+        if (room)
+        {
+	  Sys->views[0]->GetCamera ()->SetSector (room);
+	  Sys->views[1]->GetCamera ()->SetSector (room);
+	  Sys->views[0]->GetCamera ()->GetTransform ().SetOrigin (
+	      csVector3 (0, 0, 0));
+	  Sys->views[1]->GetCamera ()->GetTransform ().SetOrigin (
+	      csVector3 (0, 0, 0));
+	  camok = true;
+        }
+      }
+      if (!camok)
+      {
+        Sys->Report (CS_REPORTER_SEVERITY_ERROR,
+          "Map does not contain a valid starting point!\n"
+          "Try adding a room called 'room' or a START keyword");
+        return false;
+      }
+      Sys->InitCollDet (Sys->Engine, 0);
     }
   }
   else if (!csStrCaseCmp (cmd, "portal"))
   {
     if (arg)
     {
-      char level[100];
+      char level[300];
       csScanStr (arg, "%s", level);
       void OpenPortal (iLoader*, iView* view, char* lev);
       OpenPortal (Sys->LevelLoader, Sys->view, level);
@@ -1756,6 +1726,11 @@ bool CommandHandler (const char *cmd, const char *arg)
     extern void fire_missile ();
     fire_missile ();
   }
+  else if (!csStrCaseCmp (cmd, "decal_test"))
+  {
+    extern void test_decal();
+    test_decal ();
+  }
   else if (!csStrCaseCmp (cmd, "lightning"))
   {
     extern void show_lightning ();
@@ -1860,22 +1835,6 @@ bool CommandHandler (const char *cmd, const char *arg)
     	Sys->Engine,
 	Sys->view->GetCamera ()->GetTransform ().GetOrigin (), txtname);
   }
-  else if (!csStrCaseCmp (cmd, "spiral"))
-  {
-    char txtname[100];
-    int cnt = 0;
-    if (arg) cnt = csScanStr (arg, "%s", txtname);
-    extern void add_particles_spiral (iSector* sector, const csVector3& bottom,
-    	char* txtname);
-    if (cnt != 1)
-    {
-      Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
-      	"Expected parameter 'texture'!");
-    }
-    else
-      add_particles_spiral (Sys->view->GetCamera ()->GetSector (),
-    	Sys->view->GetCamera ()->GetTransform ().GetOrigin (), txtname);
-  }
   else if (!csStrCaseCmp (cmd, "loadmesh"))
   {
     char filename[100], tempname[100], txtname[100];
@@ -1946,9 +1905,9 @@ bool CommandHandler (const char *cmd, const char *arg)
       iMeshWrapper *wrap = Sys->Engine->GetMeshes ()->FindByName (name);
       if (wrap)
       {
-	csRef<iSprite3DFactoryState> fstate (SCF_QUERY_INTERFACE (
-		wrap->GetMeshObject ()->GetFactory (),
-		iSprite3DFactoryState));
+        csRef<iSprite3DFactoryState> fstate (
+          scfQueryInterface<iSprite3DFactoryState> (
+            wrap->GetMeshObject ()->GetFactory ()));
 	iSpriteAction* aspr_act;
 	int i;
 
@@ -1990,8 +1949,8 @@ bool CommandHandler (const char *cmd, const char *arg)
       else
       {
         csRef<iSprite3DState> state (
-		SCF_QUERY_INTERFACE (wrap->GetMeshObject (),
-		iSprite3DState));
+		
+		scfQueryInterface<iSprite3DState> (wrap->GetMeshObject ()));
         if (state)
         {
           // Test to see if the action exists for that sprite.
@@ -2150,7 +2109,7 @@ bool CommandHandler (const char *cmd, const char *arg)
       {
         ll->Remove (l);
 	size_t j;
-	for (j = 0 ; j < Sys->dynamic_lights.Length () ; j++)
+	for (j = 0 ; j < Sys->dynamic_lights.GetSize () ; j++)
 	{
 	  if (Sys->dynamic_lights[j] == l)
 	  {
@@ -2175,7 +2134,7 @@ bool CommandHandler (const char *cmd, const char *arg)
       {
         ll->Remove (l);
 	size_t j;
-	for (j = 0 ; j < Sys->dynamic_lights.Length () ; j++)
+	for (j = 0 ; j < Sys->dynamic_lights.GetSize () ; j++)
 	{
 	  if (Sys->dynamic_lights[j] == l)
 	  {
@@ -2190,7 +2149,7 @@ bool CommandHandler (const char *cmd, const char *arg)
   }
   else if (!csStrCaseCmp (cmd, "relight"))
   {
-    csRef<iConsoleOutput> console = CS_QUERY_REGISTRY(Sys->object_reg, iConsoleOutput);
+    csRef<iConsoleOutput> console = csQueryRegistry<iConsoleOutput> (Sys->object_reg);
     if(console.IsValid())
     {
       csTextProgressMeter* meter = new csTextProgressMeter(console);
@@ -2202,11 +2161,19 @@ bool CommandHandler (const char *cmd, const char *arg)
   {
     if (Sys->mySound)
     {
-      csRef<iSoundWrapper> sb (
-        CS_GET_NAMED_CHILD_OBJECT (Sys->view->GetEngine ()->
-        QueryObject (), iSoundWrapper, arg));
+      csRef<iSndSysManager> mgr = csQueryRegistry<iSndSysManager> (
+	Sys->object_reg);
+      iSndSysWrapper* sb = mgr->FindSoundByName (arg);
       if (sb)
-        sb->GetSound ()->Play();
+      {
+	csRef<iSndSysStream> sndstream = Sys->mySound->CreateStream (
+	    sb->GetData (), CS_SND3D_ABSOLUTE);
+	csRef<iSndSysSource> sndsource = Sys->mySound->CreateSource (
+	    sndstream);
+	sndsource->SetVolume (1.0f);
+        sndstream->SetLoopState (CS_SNDSYS_STREAM_DONTLOOP);
+        sndstream->Unpause ();
+      }
       else
         Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
 		"Sound '%s' not found!", arg);
@@ -2308,7 +2275,7 @@ bool CommandHandler (const char *cmd, const char *arg)
         "saveworld: Specified file `%s' already exists.", arg);
       return true;
     }
-    csRef<iSaver> saver = CS_QUERY_REGISTRY(Sys->object_reg, iSaver);
+    csRef<iSaver> saver = csQueryRegistry<iSaver> (Sys->object_reg);
     if (!saver.IsValid ())
     {
       saver = CS_LOAD_PLUGIN(Sys->plugin_mgr, "crystalspace.level.saver", iSaver);
@@ -2327,7 +2294,7 @@ bool CommandHandler (const char *cmd, const char *arg)
   }
   else if (!csStrCaseCmp (cmd, "cubemapshots"))
   {
-    csRef<iImageIO> iio = CS_QUERY_REGISTRY (Sys->object_reg, iImageIO);
+    csRef<iImageIO> iio = csQueryRegistry<iImageIO> (Sys->object_reg);
     int dim = MIN (Sys->myG3D->GetWidth (), Sys->myG3D->GetHeight ());
     dim = csFindNearestPowerOf2 (dim + 1) >> 1;
     int g2dh = Sys->myG3D->GetHeight ();
@@ -2342,8 +2309,8 @@ bool CommandHandler (const char *cmd, const char *arg)
     sideView->SetRectangle (0, 0, dim, dim);
     int cMinX, cMinY, cMaxX, xMaxY;
     Sys->myG2D->GetClipRect (cMinX, cMinY, cMaxX, xMaxY);
-    Sys->myG2D->SetClipRect (0, 0, dim, dim);
-    
+    Sys->myG2D->SetClipRect (0, 0, Sys->myG3D->GetWidth(), g2dh);
+
     static const csVector3 lookAtVecs[12] = {
       csVector3 ( 0.0f,  0.0f,  1.0f), csVector3 (0.0f, 1.0f,  0.0f), 
       csVector3 ( 0.0f,  0.0f, -1.0f), csVector3 (0.0f, 1.0f,  0.0f), 
@@ -2357,7 +2324,7 @@ bool CommandHandler (const char *cmd, const char *arg)
     for (int i = 0; i < 6; i++)
     {
       if (!Sys->myG3D->BeginDraw (Sys->Engine->GetBeginDrawFlags () 
-	| CSDRAW_3DGRAPHICS | CSDRAW_CLEARZBUFFER))
+        | CSDRAW_3DGRAPHICS | CSDRAW_CLEARZBUFFER))
       {
 	Sys->myG2D->SetClipRect (cMinX, cMinY, cMaxX, xMaxY);
 	return true;
@@ -2400,6 +2367,29 @@ bool CommandHandler (const char *cmd, const char *arg)
       stack->Free();
     }
   }
+  else if (!csStrCaseCmp (cmd, "bugplugcmd"))
+  {
+    csRef<iBugPlug> bugplug = csQueryRegistry<iBugPlug> (Sys->object_reg);
+    if (!bugplug.IsValid())
+    {
+      csRef<iPluginManager> plugmgr = 
+        csQueryRegistry<iPluginManager> (Sys->object_reg);
+      bugplug = csQueryPluginClass<iBugPlug> (plugmgr, 
+        "crystalspace.utilities.bugplug");
+    }
+    if (!bugplug.IsValid())
+    {
+      Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, 
+        "Could not get bugplug interface");
+    }
+    else
+    {
+      bool success = bugplug->ExecCommand (arg);
+      if (!success)
+        Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, 
+        "bugplug failed to execute '%s'", arg);
+    }
+  }
   else
     return false;
 
