@@ -23,6 +23,7 @@
 #include "csgeom/poly3d.h"
 #include "cstool/collider.h"
 #include "cstool/cspixmap.h"
+#include "csutil/csstring.h"
 #include "csutil/flags.h"
 #include "csutil/scanstr.h"
 #include "iengine/camera.h"
@@ -33,30 +34,23 @@
 #include "iengine/region.h"
 #include "iengine/sector.h"
 #include "iengine/sharevar.h"
-#include "igeom/objmodel.h"
+#include "imesh/objmodel.h"
 #include "igeom/polymesh.h"
 #include "imap/loader.h"
-#include "imesh/explode.h"
-#include "imesh/fire.h"
-#include "imesh/fountain.h"
+#include "imesh/particles.h"
 #include "imesh/lighting.h"
 #include "imesh/object.h"
 #include "imesh/partsys.h"
-#include "imesh/rain.h"
-#include "imesh/snow.h"
-#include "imesh/spiral.h"
 #include "imesh/sprite3d.h"
 #include "imesh/thing.h"
-#include "isound/handle.h"
-#include "isound/listener.h"
-#include "isound/renderer.h"
-#include "isound/source.h"
-#include "isound/wrapper.h"
+#include "isndsys.h"
 #include "ivaria/collider.h"
 #include "ivaria/engseq.h"
 #include "ivaria/reporter.h"
 #include "ivaria/view.h"
 #include "ivideo/graph3d.h"
+#include "ivaria/decal.h"
+#include "ivideo/material.h"
 
 #include "bot.h"
 #include "command.h"
@@ -90,9 +84,9 @@ extern void move_mesh (iMeshWrapper* sprite, iSector* where,
 void add_particles_rain (iSector* sector, char* matname, int num, float speed,
 	bool do_camera)
 {
+  iEngine* engine = Sys->view->GetEngine ();
   // First check if the material exists.
-  iMaterialWrapper* mat = Sys->view->GetEngine ()->GetMaterialList ()->
-  	FindByName (matname);
+  iMaterialWrapper* mat = engine->GetMaterialList ()->FindByName (matname);
   if (!mat)
   {
     Sys->Report (CS_REPORTER_SEVERITY_NOTIFY,
@@ -106,13 +100,13 @@ void add_particles_rain (iSector* sector, char* matname, int num, float speed,
   else
     sector->CalculateSectorBBox (bbox, true);
 
-  csRef<iMeshFactoryWrapper> mfw (Sys->view->GetEngine ()->
-    CreateMeshFactory ("crystalspace.mesh.object.rain", "rain"));
+  csRef<iMeshFactoryWrapper> mfw = engine->CreateMeshFactory (
+      "crystalspace.mesh.object.particles", "rain");
   if (!mfw) return;
 
-  csRef<iMeshWrapper> exp (
-  	Sys->view->GetEngine ()->CreateMeshWrapper (mfw, "custom rain", sector,
-					  csVector3 (0, 0, 0)));
+  csRef<iMeshWrapper> exp = engine->CreateMeshWrapper (mfw, "custom rain",
+	sector, csVector3 (0, 0, 0));
+
   if (do_camera)
   {
     iEngine* e = Sys->view->GetEngine ();
@@ -121,17 +115,43 @@ void add_particles_rain (iSector* sector, char* matname, int num, float speed,
     exp->SetRenderPriority (c);
   }
   exp->SetZBufMode(CS_ZBUF_TEST);
-
-  csRef<iRainState> rainstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iRainState));
+  exp->GetMeshObject()->SetMixMode (CS_FX_ADD);
   exp->GetMeshObject()->SetMaterialWrapper (mat);
-  rainstate->SetMixMode (CS_FX_ADD);
-  exp->GetMeshObject()->SetColor (csColor (.25,.25,.25));
-  rainstate->SetParticleCount (num);
-  rainstate->SetDropSize (0.3f/50.0f, 0.3f);
-  rainstate->SetLighting (false);
-  rainstate->SetBox (bbox.Min (), bbox.Max ());
-  rainstate->SetFallSpeed (csVector3 (0, -speed, 0));
+
+  csRef<iParticleBuiltinEmitterFactory> emit_factory = 
+      csLoadPluginCheck<iParticleBuiltinEmitterFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.emitter", false);
+  csRef<iParticleBuiltinEffectorFactory> eff_factory = 
+      csLoadPluginCheck<iParticleBuiltinEffectorFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.effector", false);
+
+  csRef<iParticleBuiltinEmitterBox> boxemit = emit_factory->CreateBox ();
+  // Time to live depends on height of sector.
+  float velocity = 2.84f * speed / 2.0f;
+  float seconds_to_live = (bbox.MaxY () - bbox.MinY ()) / velocity;
+  csBox3 emit_bbox = bbox;
+  emit_bbox.SetMin (1, emit_bbox.MaxY ());
+  boxemit->SetBox (emit_bbox);
+  boxemit->SetParticlePlacement (CS_PARTICLE_BUILTIN_VOLUME);
+  boxemit->SetEmissionRate (float (num) / seconds_to_live);
+  boxemit->SetInitialMass (5.0f, 7.5f);
+  boxemit->SetUniformVelocity (true);
+  boxemit->SetInitialTTL (seconds_to_live, seconds_to_live);
+  boxemit->SetInitialVelocity (csVector3 (0, -velocity, 0),
+      csVector3 (0));
+
+  csRef<iParticleBuiltinEffectorLinColor> lincol = eff_factory->
+    CreateLinColor ();
+  lincol->AddColor (csColor4 (.25,.25,.25,1), seconds_to_live);
+
+  csRef<iParticleSystem> partstate =
+  	scfQueryInterface<iParticleSystem> (exp->GetMeshObject ());
+  partstate->SetMinBoundingBox (bbox);
+  partstate->SetParticleSize (csVector2 (0.3f/50.0f, 0.3f));
+  partstate->SetParticleRenderOrientation (CS_PARTICLE_ORIENT_COMMON);
+  partstate->SetCommonDirection (csVector3 (0, 1, 0));
+  partstate->AddEmitter (boxemit);
+  partstate->AddEffector (lincol);
 }
 
 //===========================================================================
@@ -139,6 +159,7 @@ void add_particles_rain (iSector* sector, char* matname, int num, float speed,
 //===========================================================================
 void add_particles_snow (iSector* sector, char* matname, int num, float speed)
 {
+  iEngine* engine = Sys->view->GetEngine ();
   // First check if the material exists.
   iMaterialWrapper* mat = Sys->view->GetEngine ()->GetMaterialList ()->
   	FindByName (matname);
@@ -151,30 +172,54 @@ void add_particles_snow (iSector* sector, char* matname, int num, float speed)
   csBox3 bbox;
   sector->CalculateSectorBBox (bbox, true);
 
-  csRef<iMeshFactoryWrapper> mfw (Sys->view->GetEngine ()->
-    CreateMeshFactory ("crystalspace.mesh.object.snow", "snow"));
+  csRef<iMeshFactoryWrapper> mfw = engine->CreateMeshFactory (
+      "crystalspace.mesh.object.particles", "snow");
   if (!mfw) return;
 
-  csRef<iMeshWrapper> exp (
-  	Sys->view->GetEngine ()->CreateMeshWrapper (mfw, "custom snow", sector,
-	csVector3 (0, 0, 0)));
+  csRef<iMeshWrapper> exp = engine->CreateMeshWrapper (mfw, "custom snow",
+	sector, csVector3 (0, 0, 0));
 
   exp->SetZBufMode(CS_ZBUF_TEST);
-
-  csRef<iParticleState> partstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iParticleState));
+  exp->GetMeshObject()->SetMixMode (CS_FX_ADD);
   exp->GetMeshObject()->SetMaterialWrapper (mat);
-  partstate->SetMixMode (CS_FX_ADD);
-  exp->GetMeshObject()->SetColor (csColor (.25,.25,.25));
 
-  csRef<iSnowState> snowstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iSnowState));
-  snowstate->SetParticleCount (num);
-  snowstate->SetDropSize (0.07f, 0.07f);
-  snowstate->SetLighting (false);
-  snowstate->SetBox (bbox.Min (), bbox.Max ());
-  snowstate->SetFallSpeed (csVector3 (0, -speed, 0));
-  snowstate->SetSwirl (0.2f);
+  csRef<iParticleBuiltinEmitterFactory> emit_factory = 
+      csLoadPluginCheck<iParticleBuiltinEmitterFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.emitter", false);
+  csRef<iParticleBuiltinEffectorFactory> eff_factory = 
+      csLoadPluginCheck<iParticleBuiltinEffectorFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.effector", false);
+
+  csRef<iParticleBuiltinEmitterBox> boxemit = emit_factory->CreateBox ();
+  // Time to live depends on height of sector.
+  float velocity = 2.0f * speed / 2.0f;
+  float seconds_to_live = (bbox.MaxY () - bbox.MinY ()) / velocity;
+  csBox3 emit_bbox = bbox;
+  emit_bbox.SetMin (1, emit_bbox.MaxY ());
+  boxemit->SetBox (emit_bbox);
+  boxemit->SetParticlePlacement (CS_PARTICLE_BUILTIN_VOLUME);
+  boxemit->SetEmissionRate (float (num) / seconds_to_live);
+  boxemit->SetInitialMass (5.0f, 7.5f);
+  boxemit->SetUniformVelocity (true);
+  boxemit->SetInitialTTL (seconds_to_live, seconds_to_live);
+  boxemit->SetInitialVelocity (csVector3 (0, -velocity, 0),
+      csVector3 (0));
+
+  csRef<iParticleBuiltinEffectorLinColor> lincol = eff_factory->
+    CreateLinColor ();
+  lincol->AddColor (csColor4 (.25,.25,.25,1), seconds_to_live);
+
+  csRef<iParticleBuiltinEffectorForce> force = eff_factory->
+    CreateForce ();
+  force->SetRandomAcceleration (csVector3 (1.5f, 0.0f, 1.5f));
+
+  csRef<iParticleSystem> partstate =
+  	scfQueryInterface<iParticleSystem> (exp->GetMeshObject ());
+  partstate->SetMinBoundingBox (bbox);
+  partstate->SetParticleSize (csVector2 (0.07f, 0.07f));
+  partstate->AddEmitter (boxemit);
+  partstate->AddEffector (lincol);
+  partstate->AddEffector (force);
 }
 
 //===========================================================================
@@ -183,41 +228,70 @@ void add_particles_snow (iSector* sector, char* matname, int num, float speed)
 void add_particles_fire (iSector* sector, char* matname, int num,
 	const csVector3& origin)
 {
+  iEngine* engine = Sys->view->GetEngine ();
+
   // First check if the material exists.
-  iMaterialWrapper* mat = Sys->view->GetEngine ()->GetMaterialList ()->
-  	FindByName (matname);
+  iMaterialWrapper* mat = engine->GetMaterialList ()->FindByName (matname);
   if (!mat)
   {
     Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Can't find material '%s' in memory!", matname);
     return;
   }
 
-  csRef<iMeshFactoryWrapper> mfw (Sys->view->GetEngine ()->
-    CreateMeshFactory ("crystalspace.mesh.object.fire", "fire"));
+  csRef<iMeshFactoryWrapper> mfw = engine->CreateMeshFactory (
+      "crystalspace.mesh.object.particles", "fire");
   if (!mfw) return;
 
-  csRef<iMeshWrapper> exp (
-  	Sys->view->GetEngine ()->CreateMeshWrapper (mfw, "custom fire", sector,
-	origin));
+  csRef<iMeshWrapper> exp = engine->CreateMeshWrapper (mfw, "custom fire",
+	sector, origin);
 
   exp->SetZBufMode(CS_ZBUF_TEST);
-
-  csRef<iParticleState> partstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iParticleState));
+  exp->GetMeshObject()->SetMixMode (CS_FX_ADD);
   exp->GetMeshObject()->SetMaterialWrapper (mat);
-  partstate->SetMixMode (CS_FX_ADD);
 
-  csRef<iFireState> firestate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iFireState));
-  firestate->SetParticleCount (num);
-  //firestate->SetDropSize (.02, .04);
-  firestate->SetDropSize (0.04f, 0.08f);
-  firestate->SetLighting (false);
-  firestate->SetOrigin (csBox3(-csVector3(0.2f, 0, 0.2f),
-    csVector3(0.2f, 0.2f)));
-  firestate->SetDirection (csVector3 (0, 1.0f, 0));
-  firestate->SetSwirl (1.6f);
-  firestate->SetColorScale (0.2f);
+  csRef<iParticleBuiltinEmitterFactory> emit_factory = 
+      csLoadPluginCheck<iParticleBuiltinEmitterFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.emitter", false);
+  csRef<iParticleBuiltinEffectorFactory> eff_factory = 
+      csLoadPluginCheck<iParticleBuiltinEffectorFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.effector", false);
+
+  csRef<iParticleBuiltinEmitterSphere> sphemit = emit_factory->CreateSphere ();
+  float velocity = 0.5f;
+  float seconds_to_live = 2.0f;
+  sphemit->SetRadius (.2f);
+  sphemit->SetParticlePlacement (CS_PARTICLE_BUILTIN_VOLUME);
+  sphemit->SetEmissionRate (float (num) / seconds_to_live);
+  sphemit->SetInitialMass (5.0f, 7.5f);
+  sphemit->SetUniformVelocity (true);
+  sphemit->SetInitialTTL (seconds_to_live, seconds_to_live);
+  sphemit->SetInitialVelocity (csVector3 (0, velocity, 0),
+      csVector3 (0));
+
+  csRef<iParticleBuiltinEffectorLinColor> lincol = eff_factory->
+    CreateLinColor ();
+  lincol->AddColor (csColor4 (0.00f, 0.00f, 0.00f, 1.00f), 2.0000f);
+  lincol->AddColor (csColor4 (1.00f, 0.35f, 0.00f, 0.00f), 1.5000f);
+  lincol->AddColor (csColor4 (1.00f, 0.22f, 0.00f, 0.10f), 1.3125f);
+  lincol->AddColor (csColor4 (1.00f, 0.12f, 0.00f, 0.30f), 1.1250f);
+  lincol->AddColor (csColor4 (0.80f, 0.02f, 0.00f, 0.80f), 0.9375f);
+  lincol->AddColor (csColor4 (0.60f, 0.00f, 0.00f, 0.90f), 0.7500f);
+  lincol->AddColor (csColor4 (0.40f, 0.00f, 0.00f, 0.97f), 0.5625f);
+  lincol->AddColor (csColor4 (0.20f, 0.00f, 0.00f, 1.00f), 0.3750f);
+  lincol->AddColor (csColor4 (0.00f, 0.00f, 0.00f, 1.00f), 0.1875f);
+  lincol->AddColor (csColor4 (0.00f, 0.00f, 0.00f, 1.00f), 0.0000f);
+
+  csRef<iParticleBuiltinEffectorForce> force = eff_factory->
+    CreateForce ();
+  force->SetRandomAcceleration (csVector3 (1.5f, 1.5f, 1.5f));
+
+  csRef<iParticleSystem> partstate =
+  	scfQueryInterface<iParticleSystem> (exp->GetMeshObject ());
+  //partstate->SetMinBoundingBox (bbox);
+  partstate->SetParticleSize (csVector2 (0.04f, 0.08f));
+  partstate->AddEmitter (sphemit);
+  partstate->AddEffector (lincol);
+  partstate->AddEffector (force);
 }
 
 //===========================================================================
@@ -226,42 +300,59 @@ void add_particles_fire (iSector* sector, char* matname, int num,
 void add_particles_fountain (iSector* sector, char* matname, int num,
 	const csVector3& origin)
 {
+  iEngine* engine = Sys->view->GetEngine ();
+
   // First check if the material exists.
-  iMaterialWrapper* mat = Sys->view->GetEngine ()->GetMaterialList ()->
-  	FindByName (matname);
+  iMaterialWrapper* mat = engine->GetMaterialList ()->FindByName (matname);
   if (!mat)
   {
-    Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Can't find material '%s' in memory!", matname);
+    Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Can't find material '%s'!", matname);
     return;
   }
 
-  csRef<iMeshFactoryWrapper> mfw (Sys->view->GetEngine ()->
-    CreateMeshFactory ("crystalspace.mesh.object.fountain", "fountain"));
+  csRef<iMeshFactoryWrapper> mfw = engine->CreateMeshFactory (
+      "crystalspace.mesh.object.particles", "fountain");
   if (!mfw) return;
 
-  csRef<iMeshWrapper> exp (
-  	Sys->view->GetEngine ()->CreateMeshWrapper (mfw, "custom fountain",
-	sector, origin));
+  csRef<iMeshWrapper> exp = engine->CreateMeshWrapper (mfw, "custom fountain",
+	sector, origin);
+
   exp->SetZBufMode(CS_ZBUF_TEST);
-
-  csRef<iParticleState> partstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iParticleState));
+  exp->GetMeshObject()->SetMixMode (CS_FX_ADD);
   exp->GetMeshObject()->SetMaterialWrapper (mat);
-  partstate->SetMixMode (CS_FX_ADD);
-  exp->GetMeshObject()->SetColor (csColor (0.25f, 0.35f, 0.55f));
-  partstate->SetChangeRotation (7.5f);
 
-  csRef<iFountainState> fountstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iFountainState));
-  fountstate->SetParticleCount (num);
-  fountstate->SetDropSize (0.1f, 0.1f);
-  fountstate->SetOrigin (csVector3 (0, 0, 0));
-  fountstate->SetAcceleration (csVector3 (0, -1.0f, 0));
-  fountstate->SetFallTime (5.0f);
-  fountstate->SetSpeed (3.0f);
-  fountstate->SetElevation (3.1415926f/2.0f);
-  fountstate->SetAzimuth (0);
-  fountstate->SetOpening (0.2f);
+  csRef<iParticleBuiltinEmitterFactory> emit_factory = 
+      csLoadPluginCheck<iParticleBuiltinEmitterFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.emitter", false);
+  csRef<iParticleBuiltinEffectorFactory> eff_factory = 
+      csLoadPluginCheck<iParticleBuiltinEffectorFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.effector", false);
+
+  csRef<iParticleBuiltinEmitterCone> conemit = emit_factory->CreateCone ();
+  float velocity = 3.0f;
+  float seconds_to_live = 1.5f;
+  conemit->SetExtent (csVector3 (0, 0.5f, 0));
+  conemit->SetConeAngle (0.3f);
+  conemit->SetParticlePlacement (CS_PARTICLE_BUILTIN_VOLUME);
+  conemit->SetEmissionRate (float (num) / seconds_to_live);
+  conemit->SetInitialMass (8.0f, 10.0f);
+  conemit->SetInitialTTL (seconds_to_live, seconds_to_live);
+  conemit->SetInitialVelocity (csVector3 (0, velocity, 0), csVector3 (0));
+
+  csRef<iParticleBuiltinEffectorLinColor> lincol = eff_factory->
+    CreateLinColor ();
+  lincol->AddColor (csColor4 (0.25f, 0.35f, 0.55f, 1), seconds_to_live);
+
+  csRef<iParticleBuiltinEffectorForce> force = eff_factory->
+    CreateForce ();
+  force->SetAcceleration (csVector3 (0.0f, -3.0f, 0.0f));
+
+  csRef<iParticleSystem> partstate =
+  	scfQueryInterface<iParticleSystem> (exp->GetMeshObject ());
+  partstate->SetParticleSize (csVector2 (0.1f, 0.1f));
+  partstate->AddEmitter (conemit);
+  partstate->AddEffector (lincol);
+  partstate->AddEffector (force);
 }
 
 //===========================================================================
@@ -279,80 +370,56 @@ void add_particles_explosion (iSector* sector, iEngine* engine,
     return;
   }
 
-  csRef<iMeshFactoryWrapper> mfw (Sys->view->GetEngine ()->
-    CreateMeshFactory ("crystalspace.mesh.object.explosion", "explosion"));
+  csRef<iMeshFactoryWrapper> mfw = engine->CreateMeshFactory (
+      "crystalspace.mesh.object.particles", "explosion");
   if (!mfw) return;
 
-  csRef<iMeshWrapper> exp (
-  	Sys->view->GetEngine ()->CreateMeshWrapper (mfw, "custom explosion",
-	sector, center));
+  csRef<iMeshWrapper> exp = engine->CreateMeshWrapper (mfw, "custom explosion",
+	sector, center);
 
   exp->SetZBufMode(CS_ZBUF_TEST);
   exp->SetRenderPriority (engine->GetAlphaRenderPriority ());
-
-  csRef<iParticleState> partstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iParticleState));
   exp->GetMeshObject()->SetMaterialWrapper (mat);
-  //partstate->SetMixMode (CS_FX_SETALPHA (0.50));
+  exp->GetMeshObject()->SetMixMode (CS_FX_ALPHA);
   exp->GetMeshObject()->SetColor (csColor (1, 1, 0));
-  partstate->SetChangeRotation (5.0);
-  partstate->SetChangeSize (1.25);
-  partstate->SetSelfDestruct (3000);
 
-  csRef<iExplosionState> expstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iExplosionState));
-  expstate->SetParticleCount (100);
-  expstate->SetCenter (csVector3 (0, 0, 0));
-  expstate->SetPush (csVector3 (0, 0, 0));
-  expstate->SetNrSides (6);
-  expstate->SetPartRadius (0.15f);
-  expstate->SetLighting (true);
-  expstate->SetSpreadPos (0.6f);
-  expstate->SetSpreadSpeed (2.0f);
-  expstate->SetSpreadAcceleration (2.0f);
-  expstate->SetFadeSprites (500);
+  csRef<iParticleBuiltinEmitterFactory> emit_factory = 
+      csLoadPluginCheck<iParticleBuiltinEmitterFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.emitter", false);
+  csRef<iParticleBuiltinEffectorFactory> eff_factory = 
+      csLoadPluginCheck<iParticleBuiltinEffectorFactory> (
+        Sys->object_reg, "crystalspace.mesh.object.particles.effector", false);
+
+  csRef<iParticleBuiltinEmitterSphere> sphereemit = emit_factory->
+    CreateSphere ();
+  sphereemit->SetRadius (0.1f);
+  sphereemit->SetParticlePlacement (CS_PARTICLE_BUILTIN_CENTER);
+  sphereemit->SetPosition (csVector3 (0, 0, 0));
+  sphereemit->SetInitialVelocity (csVector3 (1, 0, 0), csVector3 (3, 3, 3));
+  sphereemit->SetUniformVelocity (false);
+  sphereemit->SetDuration (0.1f);
+  sphereemit->SetEmissionRate (1000.0f);
+  sphereemit->SetInitialTTL (1.0f, 1.0f);
+
+  csRef<iParticleBuiltinEffectorLinColor> lincol = eff_factory->
+    CreateLinColor ();
+  lincol->AddColor (csColor4 (1,1,1,1), 1.0f);
+  lincol->AddColor (csColor4 (1,1,1,0), 0.0f);
+
+  csRef<iParticleSystem> partstate =
+  	scfQueryInterface<iParticleSystem> (exp->GetMeshObject ());
+  partstate->SetParticleSize (csVector2 (0.15f, 0.15f));
+  partstate->SetRotationMode (CS_PARTICLE_ROTATE_VERTICES);
+  partstate->SetIntegrationMode (CS_PARTICLE_INTEGRATE_BOTH);
+  partstate->AddEmitter (sphereemit);
+  partstate->AddEffector (lincol);
+
+  Sys->Engine->DelayedRemoveObject (1100, exp);
+  Sys->Engine->DelayedRemoveObject (1101, mfw);
 
   exp->PlaceMesh ();
 }
 
-//===========================================================================
-// Demo particle system (spiral).
-//===========================================================================
-void add_particles_spiral (iSector* sector, const csVector3& bottom,
-	char* matname)
-{
-  // First check if the material exists.
-  iMaterialWrapper* mat = Sys->view->GetEngine ()->GetMaterialList ()->
-  	FindByName (matname);
-  if (!mat)
-  {
-    Sys->Report (CS_REPORTER_SEVERITY_NOTIFY, "Can't find material '%s' in memory!", matname);
-    return;
-  }
-
-  csRef<iMeshFactoryWrapper> mfw (Sys->view->GetEngine ()->
-    CreateMeshFactory ("crystalspace.mesh.object.spiral", "spiral"));
-  if (!mfw) return;
-
-  csRef<iMeshWrapper> exp (
-  	Sys->view->GetEngine ()->CreateMeshWrapper (mfw, "custom spiral",
-	sector, bottom));
-
-  exp->SetZBufMode(CS_ZBUF_TEST);
-
-  csRef<iParticleState> partstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iParticleState));
-  exp->GetMeshObject()->SetMaterialWrapper (mat);
-  partstate->SetMixMode (CS_FX_SETALPHA (0.50));
-  exp->GetMeshObject()->SetColor (csColor (1, 1, 0));
-  partstate->SetChangeColor (csColor(+0.01f, 0.0f, -0.012f));
-
-  csRef<iSpiralState> spirstate (
-  	SCF_QUERY_INTERFACE (exp->GetMeshObject (), iSpiralState));
-  spirstate->SetParticleCount (500);
-  spirstate->SetSource (csVector3 (0, 0, 0));
-}
-
 //===========================================================================
 // Everything for bots.
 //===========================================================================
@@ -387,7 +454,7 @@ void WalkTest::add_bot (float size, iSector* where, csVector3 const& pos,
   botWrapper->GetMovable ()->SetTransform (m);
   
   botWrapper->GetMovable ()->UpdateMove ();
-  csRef<iSprite3DState> state (SCF_QUERY_INTERFACE (botmesh, iSprite3DState));
+  csRef<iSprite3DState> state (scfQueryInterface<iSprite3DState> (botmesh));
   state->SetAction ("default");
   
   Bot* bot = new Bot (Sys->view->GetEngine(), botWrapper);
@@ -404,12 +471,12 @@ void WalkTest::del_bot (bool manual)
 {
   if (manual)
   {
-    if (manual_bots.Length () > 0)
+    if (manual_bots.GetSize () > 0)
       manual_bots.DeleteIndex (0);
   }
   else
   {
-    if (bots.Length () > 0)
+    if (bots.GetSize () > 0)
       bots.DeleteIndex (0);
   }
 }
@@ -417,7 +484,7 @@ void WalkTest::del_bot (bool manual)
 void WalkTest::move_bots (csTicks elapsed_time)
 {
   size_t i;
-  for (i = 0; i < bots.Length(); i++)
+  for (i = 0; i < bots.GetSize (); i++)
   {
     bots[i]->move (elapsed_time);
   }
@@ -441,7 +508,8 @@ struct MissileStruct
   int type;		// type == DYN_TYPE_MISSILE
   csOrthoTransform dir;
   csRef<iMeshWrapper> sprite;
-  csRef<iSoundSource> snd;
+  csRef<iSndSysSource> snd;
+  csRef<iSndSysStream> snd_stream;
 };
 
 struct ExplosionStruct
@@ -497,18 +565,28 @@ bool HandleDynLight (iLight* dyn, iEngine* engine)
         dyn->QueryObject ()->ObjRemove (ido);
         if (ms->snd)
         {
-          ms->snd->Stop();
+          ms->snd_stream->Pause();
         }
         delete ms;
         if (Sys->mySound)
         {
-          csRef<iSoundSource> sndsrc (
-	  	Sys->wMissile_boom->CreateSource (SOUND3D_ABSOLUTE));
-          if (sndsrc)
-          {
-            sndsrc->SetPosition (v);
-            sndsrc->Play();
-          }
+	  if (Sys->wMissile_boom)
+	  {
+	    csRef<iSndSysStream> st = Sys->mySound->CreateStream (
+		Sys->wMissile_boom->GetData (), CS_SND3D_ABSOLUTE);
+	    csRef<iSndSysSource> sndsource = Sys->mySound->
+	      	CreateSource (st);
+	    if (sndsource)
+	    {
+	      csRef<iSndSysSourceSoftware3D> sndsource3d
+		= scfQueryInterface<iSndSysSourceSoftware3D> (sndsource);
+
+	      sndsource3d->SetPosition (v);
+	      sndsource3d->SetVolume (1.0f);
+	      st->SetLoopState (CS_SNDSYS_STREAM_DONTLOOP);
+	      st->Unpause ();
+	    }
+	  }
         }
         ExplosionStruct* es = new ExplosionStruct;
         es->type = DYN_TYPE_EXPLOSION;
@@ -532,7 +610,13 @@ bool HandleDynLight (iLight* dyn, iEngine* engine)
       dyn->SetCenter (v);
       dyn->Setup ();
       if (ms->sprite) move_mesh (ms->sprite, s, v);
-      if (Sys->mySound && ms->snd) ms->snd->SetPosition (v);
+      if (Sys->mySound && ms->snd)
+      {
+	csRef<iSndSysSourceSoftware3D> sndsource3d
+		= scfQueryInterface<iSndSysSourceSoftware3D> (ms->snd);
+	sndsource3d->SetPosition (v);
+	sndsource3d->SetVolume (1.0f);
+      }
       break;
     }
     case DYN_TYPE_EXPLOSION:
@@ -586,8 +670,8 @@ bool HandleDynLight (iLight* dyn, iEngine* engine)
 
 void show_lightning ()
 {
-  csRef<iEngineSequenceManager> seqmgr(CS_QUERY_REGISTRY (Sys->object_reg,
-  	iEngineSequenceManager));
+  csRef<iEngineSequenceManager> seqmgr(
+  	csQueryRegistry<iEngineSequenceManager> (Sys->object_reg));
   if (seqmgr)
   {
     // This finds the light L1 (the colored light over the stairs) and
@@ -627,11 +711,17 @@ void fire_missile ()
   ms->snd = 0;
   if (Sys->mySound)
   {
-    ms->snd = Sys->wMissile_whoosh->CreateSource (SOUND3D_ABSOLUTE);
+    iSndSysData* snddata = Sys->wMissile_whoosh->GetData ();
+    ms->snd_stream = Sys->mySound->CreateStream (snddata, CS_SND3D_ABSOLUTE);
+    ms->snd = Sys->mySound->CreateSource (ms->snd_stream);
     if (ms->snd)
     {
-      ms->snd->SetPosition (pos);
-      ms->snd->Play();
+      csRef<iSndSysSourceSoftware3D> sndsource3d
+		= scfQueryInterface<iSndSysSourceSoftware3D> (ms->snd);
+
+      sndsource3d->SetPosition (pos);
+      sndsource3d->SetVolume (1.0f);
+      ms->snd_stream->Unpause ();
     }
   }
   ms->type = DYN_TYPE_MISSILE;
@@ -662,6 +752,87 @@ void fire_missile ()
   }
 }
 
+void test_decal ()
+{
+  csRef<iDecalManager> decalMgr = csLoadPluginCheck<iDecalManager> (
+  	Sys->object_reg, "crystalspace.decal.manager");
+  if (!decalMgr)
+    return;
+
+  iMaterialWrapper * material = 
+    Sys->view->GetEngine()->GetMaterialList()->FindByName("decal");
+  if (!material)
+  {
+    csRef<iLoader> loader = csQueryRegistry<iLoader>(Sys->object_reg);
+    if (!loader)
+    {
+      Sys->Report(CS_REPORTER_SEVERITY_NOTIFY, "Couldn't find iLoader");
+      return; 
+    }
+
+    if (!loader->LoadTexture ("decal", "/lib/std/cslogo2.png"))
+      Sys->Report(CS_REPORTER_SEVERITY_NOTIFY, 
+                  "Couldn't load decal texture!");
+
+    material = Sys->view->GetEngine()->GetMaterialList()->FindByName("decal");
+    if (!material)
+    {
+      Sys->Report(CS_REPORTER_SEVERITY_NOTIFY, 
+                  "Error finding decal material");
+      return;
+    }
+  }
+
+  // create a template for our new decal
+  csRef<iDecalTemplate> decalTemplate = 
+      decalMgr->CreateDecalTemplate(material);
+  decalTemplate->SetTimeToLive(5.0f);
+  
+  csVector3 start = Sys->view->GetCamera()->GetTransform().GetOrigin();
+
+  csVector3 normal = 
+    Sys->view->GetCamera()->GetTransform().This2OtherRelative(csVector3(0,0,-1));
+
+  csVector3 end = start - normal * 10000.0f;
+
+  csSectorHitBeamResult result = Sys->view->GetCamera()->GetSector()->HitBeamPortals(start, end);
+  if (!result.mesh)
+      return;
+      
+/*
+  // figure out the starting point
+  csRef<iCollideSystem> cdsys = csQueryRegistry<iCollideSystem>(Sys->object_reg);
+  if (!cdsys)
+  {
+    Sys->Report(CS_REPORTER_SEVERITY_NOTIFY, "Couldn't find iCollideSystem");
+    return;
+  }
+
+
+  // intersect with world to get a decal position
+  csVector3 iSect;
+  csIntersectingTriangle closestTri;
+  iMeshWrapper * selMesh;
+  if (csColliderHelper::TraceBeam(cdsys, Sys->view->GetCamera()->GetSector(), 
+        start, end, true, closestTri, iSect, &selMesh) <= 0.0f)
+  {
+      printf("No Decal Tracebeam\n");
+      return;
+  }
+
+  start = iSect;
+*/
+
+  // make the up direction of the decal the same as the camera
+  csVector3 up =
+    Sys->view->GetCamera()->GetTransform().This2OtherRelative(
+	csVector3(0,1,0));
+
+  // create the decal
+  decalMgr->CreateDecal(decalTemplate, result.final_sector,
+	  result.isect, up, normal, 1.0f, 0.5f);
+}
+
 void AttachRandomLight (iLight* light)
 {
   RandomLight* rl = new RandomLight;
@@ -678,13 +849,10 @@ void AttachRandomLight (iLight* light)
 
 static csPtr<iMeshWrapper> CreateMeshWrapper (const char* name)
 {
-  csRef<iPluginManager> plugin_mgr = CS_QUERY_REGISTRY (Sys->object_reg,
-  	iPluginManager);
-  csRef<iMeshObjectType> ThingType = CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-  	"crystalspace.mesh.object.thing", iMeshObjectType);
+  csRef<iMeshObjectType> ThingType = csLoadPluginCheck<iMeshObjectType> (
+  	Sys->object_reg, "crystalspace.mesh.object.thing");
   if (!ThingType)
-    ThingType = CS_LOAD_PLUGIN (plugin_mgr,
-    	"crystalspace.mesh.object.thing", iMeshObjectType);
+    return 0;
 
   csRef<iMeshObjectFactory> thing_fact = ThingType->NewFactory ();
   csRef<iMeshObject> mesh_obj = thing_fact->NewInstance ();
@@ -856,8 +1024,8 @@ static csPtr<iMeshWrapper> CreatePortalThing (const char* name, iSector* room,
   thing_fact_state->SetPolygonMaterial (CS_POLYRANGE_ALL, tm);
   thing_fact_state->SetPolygonFlags (CS_POLYRANGE_ALL, CS_POLY_COLLDET);
 
-  csRef<iLightingInfo> linfo (SCF_QUERY_INTERFACE (thing->GetMeshObject (),
-    iLightingInfo));
+  csRef<iLightingInfo> linfo (
+    scfQueryInterface<iLightingInfo> (thing->GetMeshObject ()));
   linfo->InitializeDefault (true);
   room->ShineLights (thing);
   linfo->PrepareLighting ();
