@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1998-2005 by Jorrit Tyberghein
+    Copyright (C) 1998-2006 by Jorrit Tyberghein
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -32,6 +32,7 @@
 // "crystalspace.h" rather than including only the headers we need so that we
 // test as many headers for standards conformance as possible.
 #include "crystalspace.h"
+#include "isndsys.h"
 
 #include "walktest.h"
 #include "infmaze.h"
@@ -118,6 +119,7 @@ WalkTest::WalkTest () :
   first_map = last_map = 0;
   num_maps = 0;
   cache_map = 0;
+  doSave = false;
 }
 
 WalkTest::~WalkTest ()
@@ -139,28 +141,19 @@ void WalkTest::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (object_reg, iReporter));
-  if (rep)
-  {
-    rep->ReportV (severity, "crystalspace.system", msg, arg);
-  }
-  else
-  {
-    csPrintfV (msg, arg);
-    csPrintf ("\n");
-  }
+  csReportV(object_reg, severity, "crystalspace.system", msg, arg);
   va_end (arg);
 }
 
 void WalkTest::SetDefaults ()
 {
-  csRef<iConfigManager> Config (CS_QUERY_REGISTRY (object_reg, iConfigManager));
+  csRef<iConfigManager> Config (csQueryRegistry<iConfigManager> (object_reg));
   bool do_cd = Config->GetBool ("Walktest.Settings.Colldet", true);
   collider_actor.SetCD (do_cd);
   do_logo = Config->GetBool ("Walktest.Settings.DrawLogo", true);
 
-  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
-  	iCommandLineParser));
+  csRef<iCommandLineParser> cmdline (
+  	csQueryRegistry<iCommandLineParser> (object_reg));
 
   const char *val;
   if (!(val = cmdline->GetName ()))
@@ -229,8 +222,7 @@ void WalkTest::SetDefaults ()
     Report (CS_REPORTER_SEVERITY_NOTIFY, "Logo disabled.");
   }
 
-  bool doSave = Config->GetBool ("Walktest.Settings.EnableEngineSaving", 
-    false);
+  doSave = Config->GetBool ("Walktest.Settings.EnableEngineSaving", doSave);
   doSave = cmdline->GetBoolOption ("saveable", doSave);
   Report (CS_REPORTER_SEVERITY_NOTIFY, "World saving %s.", 
     doSave ? "enabled" : "disabled");
@@ -238,7 +230,7 @@ void WalkTest::SetDefaults ()
 
 void WalkTest::Help ()
 {
-  csRef<iConfigManager> cfg (CS_QUERY_REGISTRY (object_reg, iConfigManager));
+  csRef<iConfigManager> cfg (csQueryRegistry<iConfigManager> (object_reg));
   csPrintf ("Options for WalkTest:\n");
   csPrintf ("  -exec=<script>     execute given script at startup\n");
   csPrintf ("  -[no]colldet       collision detection system (default '%scolldet')\n", collider_actor.HasCD () ? "" : "no");
@@ -266,7 +258,7 @@ void WalkTest::SetupFrame ()
   {
     // Emit recorded commands directly to the CommandHandler
     if (cfg_playrecording > 0 &&
-	recording.Length () > 0)
+	recording.GetSize () > 0)
     {
       csRecordedCamera* reccam = (csRecordedCamera*)recording[
       	cfg_playrecording];
@@ -338,12 +330,12 @@ void WalkTest::MoveSystems (csTicks elapsed_time, csTicks current_time)
   size_t i;
   busy_vector.DeleteAll ();
   csWalkEntity* wentity;
-  for (i = 0 ; i < busy_entities.Length () ; i++)
+  for (i = 0 ; i < busy_entities.GetSize () ; i++)
   {
     wentity = busy_entities[i];
     busy_vector.Push (wentity);
   }
-  for (i = 0 ; i < busy_vector.Length () ; i++)
+  for (i = 0 ; i < busy_vector.GetSize () ; i++)
   {
     wentity = busy_vector[i];
     wentity->NextFrame (elapsed_time);
@@ -373,7 +365,7 @@ void WalkTest::MoveSystems (csTicks elapsed_time, csTicks current_time)
 
     if (mySound)
     {
-      iSoundListener *sndListener = mySound->GetListener();
+      iSndSysListener *sndListener = mySound->GetListener();
       if(sndListener)
       {
         // take position/direction from view->GetCamera ()
@@ -413,13 +405,6 @@ void WalkTest::DrawFrameDebug ()
     extern void DrawPalette ();
     DrawPalette ();
   }
-  //if (selected_polygon || selected_light)
-    //view->GetEngine ()->DrawFunc (view->GetCamera (),
-      //view->GetClipper (), draw_edges, (void*)1);
-  if (cfg_debug_check_frustum)
-  {
-    // @@@
-  }
   if (do_show_debug_boxes)
   {
     extern void DrawDebugBoxes (iCamera* cam, bool do3d);
@@ -571,7 +556,7 @@ void WalkTest::DrawFrame3D (int drawflags, csTicks /*current_time*/)
 
   // Apply lighting BEFORE the very first frame
   size_t i;
-  for (i = 0 ; i < dynamic_lights.Length () ; i++)
+  for (i = 0 ; i < dynamic_lights.GetSize () ; i++)
   {
     iLight* dyn = dynamic_lights[i];
     extern bool HandleDynLight (iLight*, iEngine*);
@@ -672,12 +657,12 @@ void WalkTest::DrawFrame (csTicks elapsed_time, csTicks current_time)
       recorded_cmd = recorded_arg = 0;
       recording.Push (reccam);
     }
-    if (cfg_playrecording >= 0 && recording.Length () > 0)
+    if (cfg_playrecording >= 0 && recording.GetSize () > 0)
     {
       csRecordedCamera* reccam = (csRecordedCamera*)recording[cfg_playrecording];
       cfg_playrecording++;
       record_frame_count++;
-      if ((size_t)cfg_playrecording >= recording.Length ())
+      if ((size_t)cfg_playrecording >= recording.GetSize ())
       {
 	csTicks t1 = record_start_time;
 	csTicks t2 = csGetTicks ();
@@ -778,7 +763,7 @@ void perf_test (int num)
   int i;
   for (i = 0 ; i < num ; i++)
   {
-    if (Sys->cfg_playrecording >= 0 && Sys->recording.Length () > 0)
+    if (Sys->cfg_playrecording >= 0 && Sys->recording.GetSize () > 0)
     {
       Sys->record_frame_count++;
     }
@@ -904,45 +889,6 @@ bool WalkTest::SetMapDir (const char* map_dir)
   return true;
 }
 
-#if 0
-
-#include "csutil/thread.h"
-#include "csutil/sysfunc.h"
-#include "iutil/document.h"
-
-class MyThread : public csRunnable
-{
-private:
-  int ref;
-
-public:
-  csRef<iDocument> doc;
-  csRef<iDataBuffer> buf;
-
-public:
-  MyThread () : ref (1) { }
-  virtual ~MyThread () { }
-  virtual void Run ()
-  {
-    csSleep (2000);
-    csPrintf ("================ START PARSING!\n"); fflush (stdout);
-    const char* error = doc->Parse (buf, true);
-    if (error != 0)
-    {
-      csPrintf ("Document system error for file '%s'!", error);
-    }
-    csPrintf ("================ END PARSING!\n"); fflush (stdout);
-  }
-  virtual void IncRef () { ref++; }
-  virtual void DecRef () { ref--; if (ref <= 0) delete this; }
-};
-
-static csRef<csThread> thread1;
-static csRef<csThread> thread2;
-static csRef<csThread> thread3;
-
-#endif
-
 bool WalkTest::Initialize (int argc, const char* const argv[],
 	const char *iConfigName)
 {
@@ -955,11 +901,13 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
 
   if (!csInitializer::SetupConfigManager (object_reg, iConfigName))
   {
-    Report (CS_REPORTER_SEVERITY_ERROR, "Failed to initialize config!");
+    Report (CS_REPORTER_SEVERITY_ERROR, "Failed to initialize config!\n"
+      " (A common cause is the CRYSTAL environment variable "
+      "not being set correctly.)");
     return false;
   }
 
-  csRef<iConfigManager> cfg (CS_QUERY_REGISTRY (object_reg, iConfigManager));
+  csRef<iConfigManager> cfg (csQueryRegistry<iConfigManager> (object_reg));
 #if defined(CS_PLATFORM_WIN32)
   const bool mdumpDefault =
 #if defined(CS_COMPILER_MSVC)
@@ -995,6 +943,13 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
     return false;
   }
 
+  //Must have before help is called
+  name_reg = csEventNameRegistry::GetRegistry (object_reg);
+  Process = csevProcess (name_reg);
+  FinalProcess = csevFinalProcess (name_reg);
+  CommandLineHelp = csevCommandLineHelp (name_reg);
+  
+
   // Check for commandline help.
   if (csCommandLineHelper::CheckHelp (object_reg))
   {
@@ -1003,47 +958,43 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
     exit (0);
   }
 
-  plugin_mgr = CS_QUERY_REGISTRY (object_reg, iPluginManager);
-  vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
+  plugin_mgr = csQueryRegistry<iPluginManager> (object_reg);
+  vc = csQueryRegistry<iVirtualClock> (object_reg);
 
-  myG3D = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  myG3D = csQueryRegistry<iGraphics3D> (object_reg);
   if (!myG3D)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No iGraphics3D plugin!");
     return false;
   }
 
-  myG2D = CS_QUERY_REGISTRY (object_reg, iGraphics2D);
+  myG2D = csQueryRegistry<iGraphics2D> (object_reg);
   if (!myG2D)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No iGraphics2D plugin!");
     return false;
   }
 
-  myVFS = CS_QUERY_REGISTRY (object_reg, iVFS);
+  myVFS = csQueryRegistry<iVFS> (object_reg);
   if (!myVFS)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No iVFS plugin!");
     return false;
   }
 
-  kbd = CS_QUERY_REGISTRY (object_reg, iKeyboardDriver);
+  kbd = csQueryRegistry<iKeyboardDriver> (object_reg);
   if (!kbd)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No iKeyboardDriver!");
     return false;
   }
 
-  name_reg = csEventNameRegistry::GetRegistry (object_reg);
-  Process = csevProcess (name_reg);
-  FinalProcess = csevFinalProcess (name_reg);
-  CommandLineHelp = csevCommandLineHelp (name_reg);
   CanvasHidden = csevCanvasHidden (name_reg, myG2D);
   CanvasExposed = csevCanvasExposed (name_reg, myG2D);
   CanvasResize = csevCanvasResize (name_reg, myG2D);
 
-  myConsole = CS_QUERY_REGISTRY (object_reg, iConsoleOutput);
-  mySound = CS_QUERY_REGISTRY (object_reg, iSoundRender);
+  myConsole = csQueryRegistry<iConsoleOutput> (object_reg);
+  mySound = csQueryRegistry<iSndSysRenderer> (object_reg);
 
   // Some commercials...
   Report (CS_REPORTER_SEVERITY_NOTIFY, "Crystal Space version %s (%s).", CS_VERSION, CS_RELEASE_DATE);
@@ -1094,6 +1045,11 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   FrameHeight = Gfx2D->GetHeight ();
 
   // Find the font we'll use
+  if (!Gfx2D->GetFontServer ())
+  {
+    Report (CS_REPORTER_SEVERITY_ERROR, "No font server available");
+    return false;
+  }
   Font = Gfx2D->GetFontServer ()->LoadFont (cfg_font);
   if (!Font)
   {
@@ -1101,12 +1057,17 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
       cfg_font);
     Font = Gfx2D->GetFontServer ()->LoadFont (CSFONT_COURIER);
   }
+  if (!Font)
+  {
+    Report (CS_REPORTER_SEVERITY_ERROR, "Could not load any font");
+    return false;
+  }
 
   // Open the startup console
   start_console ();
 
   // Find the engine plugin and query the csEngine object from it...
-  Engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  Engine = csQueryRegistry<iEngine> (object_reg);
   if (!Engine)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No iEngine plugin!");
@@ -1115,7 +1076,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   Engine->SetSaveableFlag (doSave);
 
   // Find the level loader plugin
-  LevelLoader = CS_QUERY_REGISTRY (object_reg, iLoader);
+  LevelLoader = csQueryRegistry<iLoader> (object_reg);
   if (!LevelLoader)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "No level loader plugin!");
@@ -1178,43 +1139,40 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
       return false;
     }
 
-    if (do_infinite)
-    {
-      iSector* room;
-      // Create the initial (non-random) part of the maze.
-      infinite_maze = new InfiniteMaze ();
-      room = infinite_maze->create_six_room (Engine, 0, 0, 0)->sector;
-      infinite_maze->create_six_room (Engine, 0, 0, 1);
-      infinite_maze->create_six_room (Engine, 0, 0, 2);
-      infinite_maze->create_six_room (Engine, 1, 0, 2);
-      infinite_maze->create_six_room (Engine, 0, 1, 2);
-      infinite_maze->create_six_room (Engine, 1, 1, 2);
-      infinite_maze->create_six_room (Engine, 0, 0, 3);
-      infinite_maze->create_six_room (Engine, 0, 0, 4);
-      infinite_maze->create_six_room (Engine, -1, 0, 4);
-      infinite_maze->create_six_room (Engine, -2, 0, 4);
-      infinite_maze->create_six_room (Engine, 0, -1, 3);
-      infinite_maze->create_six_room (Engine, 0, -2, 3);
-      infinite_maze->create_six_room (Engine, 0, 1, 3);
-      infinite_maze->create_six_room (Engine, 0, 2, 3);
-      infinite_maze->connect_infinite (0, 0, 0, 0, 0, 1);
-      infinite_maze->connect_infinite (0, 0, 1, 0, 0, 2);
-      infinite_maze->connect_infinite (0, 0, 2, 0, 0, 3);
-      infinite_maze->connect_infinite (0, 0, 2, 1, 0, 2);
-      infinite_maze->connect_infinite (0, 0, 2, 0, 1, 2);
-      infinite_maze->connect_infinite (1, 1, 2, 0, 1, 2);
-      infinite_maze->connect_infinite (1, 1, 2, 1, 0, 2);
-      infinite_maze->connect_infinite (0, 0, 3, 0, 0, 4);
-      infinite_maze->connect_infinite (-1, 0, 4, 0, 0, 4);
-      infinite_maze->connect_infinite (-2, 0, 4, -1, 0, 4);
-      infinite_maze->connect_infinite (0, 0, 3, 0, -1, 3);
-      infinite_maze->connect_infinite (0, -1, 3, 0, -2, 3);
-      infinite_maze->connect_infinite (0, 0, 3, 0, 1, 3);
-      infinite_maze->connect_infinite (0, 1, 3, 0, 2, 3);
-      infinite_maze->create_loose_portal (-2, 0, 4, -2, 1, 4);
-      view->GetCamera ()->SetSector(room);
-      printf ("Done creation of infinite maze!\n"); fflush (stdout);
-    }
+    iSector* room;
+    // Create the initial (non-random) part of the maze.
+    infinite_maze = new InfiniteMaze ();
+    room = infinite_maze->create_six_room (Engine, 0, 0, 0)->sector;
+    infinite_maze->create_six_room (Engine, 0, 0, 1);
+    infinite_maze->create_six_room (Engine, 0, 0, 2);
+    infinite_maze->create_six_room (Engine, 1, 0, 2);
+    infinite_maze->create_six_room (Engine, 0, 1, 2);
+    infinite_maze->create_six_room (Engine, 1, 1, 2);
+    infinite_maze->create_six_room (Engine, 0, 0, 3);
+    infinite_maze->create_six_room (Engine, 0, 0, 4);
+    infinite_maze->create_six_room (Engine, -1, 0, 4);
+    infinite_maze->create_six_room (Engine, -2, 0, 4);
+    infinite_maze->create_six_room (Engine, 0, -1, 3);
+    infinite_maze->create_six_room (Engine, 0, -2, 3);
+    infinite_maze->create_six_room (Engine, 0, 1, 3);
+    infinite_maze->create_six_room (Engine, 0, 2, 3);
+    infinite_maze->connect_infinite (0, 0, 0, 0, 0, 1);
+    infinite_maze->connect_infinite (0, 0, 1, 0, 0, 2);
+    infinite_maze->connect_infinite (0, 0, 2, 0, 0, 3);
+    infinite_maze->connect_infinite (0, 0, 2, 1, 0, 2);
+    infinite_maze->connect_infinite (0, 0, 2, 0, 1, 2);
+    infinite_maze->connect_infinite (1, 1, 2, 0, 1, 2);
+    infinite_maze->connect_infinite (1, 1, 2, 1, 0, 2);
+    infinite_maze->connect_infinite (0, 0, 3, 0, 0, 4);
+    infinite_maze->connect_infinite (-1, 0, 4, 0, 0, 4);
+    infinite_maze->connect_infinite (-2, 0, 4, -1, 0, 4);
+    infinite_maze->connect_infinite (0, 0, 3, 0, -1, 3);
+    infinite_maze->connect_infinite (0, -1, 3, 0, -2, 3);
+    infinite_maze->connect_infinite (0, 0, 3, 0, 1, 3);
+    infinite_maze->connect_infinite (0, 1, 3, 0, 2, 3);
+    infinite_maze->create_loose_portal (-2, 0, 4, -2, 1, 4);
+    view->GetCamera ()->SetSector(room);
+    printf ("Done creation of infinite maze!\n"); fflush (stdout);
 
     // Prepare the engine. This will calculate all lighting and
     // prepare the lightmaps for the 3D rasterizer.
@@ -1242,8 +1200,8 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
       	first_map->map_dir);
 
     // Check if we have to load every separate map in a separate region.
-    csRef<iCommandLineParser> cmdline = CS_QUERY_REGISTRY (object_reg,
-    	iCommandLineParser);
+    csRef<iCommandLineParser> cmdline = 
+    	csQueryRegistry<iCommandLineParser> (object_reg);
     bool do_regions = false;
     if (cmdline->GetOption ("regions"))
       do_regions = true;
@@ -1339,7 +1297,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
 	  cp->Load(views[1]->GetCamera (), Engine))
 	camok = true;
     }
-    if (!camok) 
+    if (!camok)
     {
       iSector* room = Engine->GetSectors ()->FindByName ("room");
       if (room)
@@ -1366,12 +1324,10 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   // Load a few sounds.
   if (mySound)
   {
-    csRef<iSoundWrapper> w (CS_GET_NAMED_CHILD_OBJECT (
-    	Engine->QueryObject (), iSoundWrapper, "boom.wav"));
-    wMissile_boom = w ? w->GetSound () : 0;
-    w = CS_GET_NAMED_CHILD_OBJECT (Engine->QueryObject (),
-					iSoundWrapper, "whoosh.wav");
-    wMissile_whoosh = w ? w->GetSound () : 0;
+    csRef<iSndSysManager> mgr = csQueryRegistry<iSndSysManager> (
+	object_reg);
+    wMissile_boom = mgr->FindSoundByName ("boom.wav");
+    wMissile_whoosh = mgr->FindSoundByName ("whoosh.wav");
   }
 
   Report (CS_REPORTER_SEVERITY_NOTIFY,
@@ -1380,7 +1336,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
   {
     myConsole->SetVisible (false);
     myConsole->AutoUpdate (false);
-    ConsoleInput = CS_QUERY_REGISTRY (object_reg, iConsoleInput);
+    ConsoleInput = csQueryRegistry<iConsoleInput> (object_reg);
     if (ConsoleInput)
     {
       ConsoleInput->Bind (myConsole);
@@ -1392,8 +1348,8 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
     }
 
     // Set console to center of screen, if supported
-    int DeltaX = myG2D->GetWidth () / 10;
-    int DeltaY = myG2D->GetHeight () / 10;
+    int DeltaX = myG2D->GetWidth () / 18;
+    int DeltaY = myG2D->GetHeight () / 18;
     SmallConsole = myConsole->PerformExtension ("SetPos", DeltaX, DeltaY,
       myG2D->GetWidth () - DeltaX * 2, myG2D->GetHeight () - DeltaY * 2);
   }
@@ -1430,7 +1386,7 @@ bool WalkTest::Initialize (int argc, const char* const argv[],
 #if 0
 {
   csRef<iDocumentSystem> xml (
-      CS_QUERY_REGISTRY (object_reg, iDocumentSystem));
+      csQueryRegistry<iDocumentSystem> (object_reg));
 
   MyThread* t1 = new MyThread ();
   MyThread* t2 = new MyThread ();
@@ -1475,9 +1431,10 @@ int main (int argc, char* argv[])
   // (3D, 2D, network, sound, ..., engine) and initialize them.
   if (!Sys->Initialize (argc, argv, "/config/walktest.cfg"))
   {
-    Sys->Report (CS_REPORTER_SEVERITY_ERROR, "Error initializing system!");
+    Sys->Report (CS_REPORTER_SEVERITY_ERROR,
+      "Error initializing system!");
     Cleanup();
-    exit (-1);
+    return -1;
   }
 
   // Start the 'autoexec.cfg' script and fully execute it.
@@ -1497,6 +1454,6 @@ int main (int argc, char* argv[])
 
   Cleanup ();
 
-  return 1;
+  return 0;
 }
 
