@@ -42,6 +42,7 @@
 #include "igeom/clip2d.h"
 #include "igraphic/imageio.h"
 #include "imap/loader.h"
+#include "imap/ldrctxt.h"
 #include "imesh/genmesh.h"
 #include "imesh/object.h"
 #include "imesh/thing.h"
@@ -235,7 +236,7 @@ bool csWaterDemo::HandleEvent (iEvent& ev)
     {
     case CSKEY_ESC:
       {
-        csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
+        csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
         if (q) 
 	  q->GetEventOutlet()->Broadcast (csevQuit (object_reg));
         return true;
@@ -329,7 +330,7 @@ bool csWaterDemo::Initialize ()
     return false;
   }
 
-  vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
+  vc = csQueryRegistry<iVirtualClock> (object_reg);
   if (vc == 0)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -338,7 +339,7 @@ bool csWaterDemo::Initialize ()
     return false;
   }
 
-  vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+  vfs = csQueryRegistry<iVFS> (object_reg);
   if (vfs == 0)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -347,7 +348,7 @@ bool csWaterDemo::Initialize ()
     return false;
   }
 
-  r3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  r3d = csQueryRegistry<iGraphics3D> (object_reg);
   if (r3d == 0)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -358,7 +359,7 @@ bool csWaterDemo::Initialize ()
 
   font = r3d->GetDriver2D ()->GetFontServer()->LoadFont(CSFONT_LARGE);
 
-  engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  engine = csQueryRegistry<iEngine> (object_reg);
   if (engine == 0)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -367,7 +368,7 @@ bool csWaterDemo::Initialize ()
     return false;
   }
 
-  kbd = CS_QUERY_REGISTRY (object_reg, iKeyboardDriver);
+  kbd = csQueryRegistry<iKeyboardDriver> (object_reg);
   if (kbd == 0)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -376,7 +377,7 @@ bool csWaterDemo::Initialize ()
     return false;
   }
 
-  mouse = CS_QUERY_REGISTRY (object_reg, iMouseDriver);
+  mouse = csQueryRegistry<iMouseDriver> (object_reg);
   if (mouse == 0)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -385,7 +386,7 @@ bool csWaterDemo::Initialize ()
     return false;
   }
 
-  csRef<iLoader> loader = CS_QUERY_REGISTRY (object_reg, iLoader);
+  csRef<iLoader> loader = csQueryRegistry<iLoader> (object_reg);
   if (loader == 0)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
@@ -396,7 +397,7 @@ bool csWaterDemo::Initialize ()
 
   vfs->ChDir ("/tmp");
   
-  console = CS_QUERY_REGISTRY (object_reg, iConsoleOutput);
+  console = csQueryRegistry<iConsoleOutput> (object_reg);
 
   // Open the main system. This will open all the previously loaded plug-ins.
   if (!csInitializer::OpenApplication (object_reg))
@@ -434,11 +435,11 @@ bool csWaterDemo::Initialize ()
   r3d->GetDriver2D ()->SetMouseCursor( csmcNone );
 
   csRef<iPluginManager> plugin_mgr (
-    CS_QUERY_REGISTRY (object_reg, iPluginManager));
+    csQueryRegistry<iPluginManager> (object_reg));
 
   csRef<iStringSet> strings = 
-    CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg, 
-    "crystalspace.shared.stringset", iStringSet);
+    csQueryRegistryTagInterface<iStringSet> 
+    (object_reg, "crystalspace.shared.stringset");
 
   //get a custom renderloop
   csRef<iRenderLoop> rl = engine->GetRenderLoopManager ()->Create ();
@@ -455,7 +456,7 @@ bool csWaterDemo::Initialize ()
 
   step = genFact->Create ();
   rl->AddStep (step);
-  genStep = SCF_QUERY_INTERFACE (step, iGenericRenderStep);
+  genStep = scfQueryInterface<iGenericRenderStep> (step);
 
   genStep->SetShaderType ("general");
   genStep->SetZBufMode (CS_ZBUF_USE);
@@ -465,43 +466,40 @@ bool csWaterDemo::Initialize ()
   engine->SetCurrentDefaultRenderloop (rl);
 
   // Load in lighting shaders
-  csRef<iVFS> vfs (CS_QUERY_REGISTRY(object_reg, iVFS));
+  csRef<iVFS> vfs (csQueryRegistry<iVFS> (object_reg));
   csRef<iFile> shaderFile = vfs->Open ("/shader/water.xml", VFS_FILE_READ);
 
   csRef<iDocumentSystem> docsys (
-    CS_QUERY_REGISTRY(object_reg, iDocumentSystem));
+    csQueryRegistry<iDocumentSystem> (object_reg));
   csRef<iDocument> shaderDoc = docsys->CreateDocument ();
   shaderDoc->Parse (shaderFile, true);
 
   csRef<iShader> shader;
-  csRef<iShaderManager> shmgr (CS_QUERY_REGISTRY(object_reg, iShaderManager));
+  csRef<iShaderManager> shmgr (csQueryRegistry<iShaderManager> (object_reg));
   csRef<iShaderCompiler> shcom (shmgr->GetCompiler ("XMLShader"));
-  shader = shcom->CompileShader (shaderDoc->GetRoot ()->GetNode ("shader"));
+  csRef<iLoaderContext> ldr_context = engine->CreateLoaderContext ();
+  shader = shcom->CompileShader (ldr_context,
+      shaderDoc->GetRoot ()->GetNode ("shader"));
 
   // setup the mesh 
-  csRef<iMeshObjectType> gType = CS_QUERY_PLUGIN_CLASS(plugin_mgr, 
-    "crystalspace.mesh.object.genmesh", iMeshObjectType);
+  csRef<iMeshObjectType> gType = csLoadPluginCheck<iMeshObjectType> (
+  	plugin_mgr, "crystalspace.mesh.object.genmesh");
   
   if (!gType)
   {
-    gType = CS_LOAD_PLUGIN(plugin_mgr, 
-      "crystalspace.mesh.object.genmesh", iMeshObjectType);
-    if (!gType)
-    {
-      csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
+    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
         "crystalspace.application.waterdemo",
         "Error loading genmesh baseobject!");
-      return false;
-    }
+    return false;
   }
 
   
   gFact = gType->NewFactory ();
   csRef<iMeshFactoryWrapper> fw = engine->CreateMeshFactory (gFact, "waterFactory");
-  gFactState = SCF_QUERY_INTERFACE(gFact, iGeneralFactoryState);
+  gFactState = scfQueryInterface<iGeneralFactoryState> (gFact);
 
   gMesh = gFact->NewInstance ();
-  gMeshState = SCF_QUERY_INTERFACE(gMesh, iGeneralMeshState);
+  gMeshState = scfQueryInterface<iGeneralMeshState> (gMesh);
   gMeshState->SetShadowCasting (false);
   gMeshState->SetShadowReceiving (false);
 
