@@ -31,6 +31,7 @@
 #include "cstypes.h"	// for bool
 
 #include "csgeom/csrect.h"
+#include "csgeom/math.h"
 #include "csgeom/vector2.h"
 #include "csgeom/vector3.h"
 #include "csgeom/segment.h"
@@ -106,7 +107,7 @@ private:
   };
   // Index by edge number. Edge e and e+1 with e even are opposite edges.
   // (CS_BOX_EDGE_...)
-  static bEdge edges[8];
+  static const bEdge edges[8];
 
 protected:
   /// The top-left coordinate of the bounding box.
@@ -612,9 +613,19 @@ protected:
    * Index by edge number. Edge e and e+1 with e even are opposite edges.
    * (CS_BOX_EDGE_...) 
    */
-  static bEdge edges[24];
+  static const bEdge edges[24];
   /// Index by CS_BOX_SIDE_? number.
-  static bFace faces[6];
+  static const bFace faces[6];
+  /// This table also contains an array of sides visible from that region.
+  struct Outline
+  {
+    int num;
+    int vertices[8];
+    int num_sides;
+    int sides[3];
+  };
+  /// Outline lookup table.
+  static const Outline outlines[27];
 public:
   /// Get the minimum X value of the box
   inline float MinX () const { return minbox.x; }
@@ -629,10 +640,10 @@ public:
   /// Get the maximum Z value of the box
   inline float MaxZ () const { return maxbox.z; }
   /// Get Min component for 0 (x), 1 (y), or 2 (z).
-  inline float Min (int idx) const
+  inline float Min (size_t idx) const
   { return minbox[idx]; }
   /// Get Max component for 0 (x), 1 (y), or 2 (z).
-  inline float Max (int idx) const
+  inline float Max (size_t idx) const
   { return maxbox[idx]; }
   /// Get the 3d vector of minimum (x, y, z) values
   inline const csVector3& Min () const { return minbox; }
@@ -641,6 +652,14 @@ public:
   /// Compute volume of box
   inline float Volume () const
   { return (MaxX()-MinX())*(MaxY()-MinY())*(MaxZ()-MinZ()); }
+  /// Compute area of box
+  inline float Area () const
+  {
+    float x = MaxX()-MinX();
+    float y = MaxY()-MinY();
+    float z = MaxZ()-MinZ();
+    return 2.0f*(x*y + x*z + y*z);
+  }
 
   /**
    * Return every corner of this bounding box from 0
@@ -670,7 +689,7 @@ public:
    * Given a face index (#CS_BOX_SIDE_x etc.) return the four edges oriented
    * clockwise around this face (#CS_BOX_EDGE_Xyz_xyz etc.).
    */
-  inline uint8* GetFaceEdges (int face) const
+  inline const uint8* GetFaceEdges (int face) const
   {
     return faces[face];
   }
@@ -895,6 +914,20 @@ public:
     return AddBoundingVertexSmartTest (v.x, v.y, v.z);
   }
 
+  /**
+   * Add another bounding box into this one, combining the two of them
+   */
+  inline void AddBoundingBox (const csBox3& box)
+  {
+    minbox.x = csMin(minbox.x, box.minbox.x);
+    minbox.y = csMin(minbox.y, box.minbox.y);
+    minbox.z = csMin(minbox.z, box.minbox.z);
+
+    maxbox.x = csMax(maxbox.x, box.maxbox.x);
+    maxbox.y = csMax(maxbox.y, box.maxbox.y);
+    maxbox.z = csMax(maxbox.z, box.maxbox.z);
+  }
+
   /// Initialize this box to empty.
   csBox3 () :
     minbox ( CS_BOUNDINGBOX_MAXVALUE,
@@ -937,19 +970,27 @@ public:
   }
 
   /// Set Min component for 0 (x), 1 (y), or 2 (z).
-  inline void SetMin (int idx, float val)
+  inline void SetMin (size_t idx, float val)
   {
-    if (idx == 1) minbox.y = val;
-    else if (idx == 0) minbox.x = val;
-    else minbox.z = val;
+    minbox[idx] = val;
   }
 
   /// Set Max component for 0 (x), 1 (y), or 2 (z).
-  inline void SetMax (int idx, float val)
+  inline void SetMax (size_t idx, float val)
   {
-    if (idx == 1) maxbox.y = val;
-    else if (idx == 0) maxbox.x = val;
-    else maxbox.z = val;
+    maxbox[idx] = val;
+  }
+
+  /// Get Min component for 0 (x), 1 (y), or 2 (z).
+  inline float GetMin (size_t idx)
+  {
+    return minbox[idx];
+  }
+
+  /// Get Max component for 0 (x), 1 (y), or 2 (z).
+  inline float GetMax (size_t idx)
+  {
+    return maxbox[idx];
   }
 
   /**
