@@ -20,7 +20,7 @@
 #ifndef __CS_MATH3D_H__
 #define __CS_MATH3D_H__
 
-/**\file 
+/**\file
  * 3D mathematic utility functions.
  */
 /**
@@ -38,8 +38,8 @@
 #include "csutil/scf_implementation.h"
 
 #include "iutil/dbghelp.h"
-#include "iutil/string.h"
 
+struct iString;
 class csPlane2;
 class csPoly3D;
 
@@ -230,7 +230,7 @@ class CS_CRYSTALSPACE_EXPORT csSquaredDist
 public:
   /// Returns the squared distance between two points.
   static float PointPoint (const csVector3& p1, const csVector3& p2)
-  { 
+  {
     csVector3 d = (p1-p2);
     return d*d;
   }
@@ -330,6 +330,15 @@ public:
     const csVector3& normal, const csVector3& a, // plane
     csVector3& isect, float& dist);              // intersection point
 
+  /**
+   * Intersect a 3D segment with a plane.
+   * \return true if there is an
+   * intersection, segment updated to reflect the clipped part.
+   */
+  static bool SegmentPlane (
+    const csPlane3& plane,
+    csSegment3& segment);
+
   /**
    * Intersect a 3D segment with a plane.  Returns true if there is an
    * intersection, with the intersection point returned in isect.
@@ -345,12 +354,12 @@ public:
    *   then 'isect's value is (0, 0, 0).
    *
    * \remarks
-   * 'p' is the plane, expressed as: A x + B y + C z + D = 0 , where (A,B,C) is 
+   * 'p' is the plane, expressed as: A x + B y + C z + D = 0 , where (A,B,C) is
    * the normal vector of the plane.
    * 'u' and 'v' are the start (U point) and the end (V point) of the segment.
    * 'isect' is searched along the segment U + x (V - U); the unknown 'x' value
-   * is got by: x = [(A,B,C) * U + D ] / (A,B,C) * (U - V), where * is the dot 
-   * product.  
+   * is got by: x = [(A,B,C) * U + D ] / (A,B,C) * (U - V), where * is the dot
+   * product.
    */
   static bool SegmentPlane (
     const csVector3& u, const csVector3& v,
@@ -560,19 +569,36 @@ public:
   }
 
   /**
-   * Intersect a segment with a box.
+   * Intersect a segment or ray with a box.
+   * If the segment starts inside the box then isect will return
+   * the start of the segment.
    * \param box The box with which to perform the intersection.
    * \param segment The intersection candidate.
    * \param isect The returned intersection point.
-   * \param pr If not null then a number between 0 and 1 is returned which
-   * corresponds to the position on the segment. If we were in the box
+   * \param pr If not null then a number between 0 and 1 (or bigger then
+   * 1 in case we are doing ray testing) is returned which corresponds
+   * to the position on the segment or ray. If we were in the box
    * this this function will return CS_BOX_INSIDE. In this case 'isect' will
    * be set to the start of the segment and *pr to 0.
+   * \param use_ray if true then the test is done as if the segment
+   * is a ray starting at the start of the segment and going in the
+   * direction of the end of the segment. Default is false.
    * \return one of CS_BOX_SIDE_... if it intersects, CS_BOX_INSIDE if inside,
    * or -1 otherwise.
    */
   static int BoxSegment (const csBox3& box, const csSegment3& segment,
-  	csVector3& isect, float* pr = 0);
+  	csVector3& isect, float* pr = 0, bool use_ray = false);
+
+  /**
+   * Clip a segment or ray to a box.
+   * \param segment Is the segment or ray. It will be modified (if
+   * this routine returns true) to give the clipped segment.
+   * \param box is the box to clip against.
+   * \param use_ray if true then the test is done with a ray instead
+   * of a segment. Default is false.
+   */
+  static bool ClipSegmentBox (csSegment3& segment, const csBox3& box,
+	bool use_ray = false);
 
   /**
    * Intersect an AABB with a frustum. The frustum may contain up to
@@ -583,7 +609,7 @@ public:
    * frustum tests with children of the AABB (i.e. other AABB inside
    * this AABB).
    */
-  static bool BoxFrustum (const csBox3& box, csPlane3* frustum,
+  static bool BoxFrustum (const csBox3& box, const csPlane3* frustum,
   	uint32 inClipMask, uint32& outClipMask);
 
   /**
