@@ -20,20 +20,21 @@
 #ifndef __CS_MATRIX2_H__
 #define __CS_MATRIX2_H__
 
+#include "csextern.h"
+
 #include "csgeom/vector2.h"
 
 /**\file 
+ * 2x2 matrix.
  */
 /**
  * \addtogroup geom_utils
  * @{ */
 
-#include "csextern.h"
-
 /**
  * A 2x2 matrix.
  */
-class CS_CSGEOM_EXPORT csMatrix2
+class CS_CRYSTALSPACE_EXPORT csMatrix2
 {
 public:
   float m11, m12;
@@ -100,7 +101,8 @@ public:
   inline csMatrix2 GetInverse () const
   {
     float inv_det = 1 / (m11 * m22 - m12 * m21);
-    return csMatrix2 (m22 * inv_det, -m12 * inv_det, -m21 * inv_det, m11 * inv_det);
+    return csMatrix2 (m22 * inv_det, -m12 * inv_det,
+    		      -m21 * inv_det, m11 * inv_det);
   }
 
   /// Invert this matrix.
@@ -113,11 +115,14 @@ public:
   void Identity ();
 
   /// Add two matricies.
-  friend csMatrix2 operator+ (const csMatrix2& m1, const csMatrix2& m2);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix2 operator+ (const csMatrix2& m1,
+    const csMatrix2& m2);
   /// Subtract two matricies.
-  friend csMatrix2 operator- (const csMatrix2& m1, const csMatrix2& m2);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix2 operator- (const csMatrix2& m1,
+    const csMatrix2& m2);
   /// Multiply two matricies.
-  friend csMatrix2 operator* (const csMatrix2& m1, const csMatrix2& m2);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix2 operator* (const csMatrix2& m1,
+    const csMatrix2& m2);
 
   /// Multiply a vector by a matrix (transform it).
   inline friend csVector2 operator* (const csMatrix2& m, const csVector2& v)
@@ -126,11 +131,14 @@ public:
   }
 
   /// Multiply a matrix and a scalar.
-  friend csMatrix2 operator* (const csMatrix2& m, float f);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix2 operator* (const csMatrix2& m, 
+    float f);
   /// Multiply a matrix and a scalar.
-  friend csMatrix2 operator* (float f, const csMatrix2& m);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix2 operator* (float f,
+    const csMatrix2& m);
   /// Divide a matrix by a scalar.
-  friend csMatrix2 operator/ (const csMatrix2& m, float f);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix2 operator/ (const csMatrix2& m,
+    float f);
 };
 
 /** @} */
