@@ -21,17 +21,13 @@
 #define __CS_MATRIX3_H__
 
 /**\file 
+ * 3x3 matrix.
  */
 /**
  * \addtogroup geom_utils
  * @{ */
 
-#ifndef __CS_CSSYSDEFS_H__
-#error "cssysdef.h must be included in EVERY source file!"
-#endif
-
 #include "csextern.h"
-
 #include "csgeom/vector3.h"
 
 class csQuaternion;
@@ -39,7 +35,7 @@ class csQuaternion;
 /**
  * A 3x3 matrix.
  */
-class CS_CSGEOM_EXPORT csMatrix3
+class CS_CRYSTALSPACE_EXPORT csMatrix3
 {
 public:
   float m11, m12, m13;
@@ -73,34 +69,34 @@ public:
   explicit csMatrix3 (const csQuaternion &quat) { Set (quat); }
 
   /// Get the first row of this matrix as a vector.
-  csVector3 Row1() const { return csVector3 (m11,m12,m13); }
+  inline csVector3 Row1() const { return csVector3 (m11,m12,m13); }
 
   /// Get the second row of this matrix as a vector.
-  csVector3 Row2() const { return csVector3 (m21,m22,m23); }
+  inline csVector3 Row2() const { return csVector3 (m21,m22,m23); }
 
   /// Get the third row of this matrix as a vector.
-  csVector3 Row3() const { return csVector3 (m31,m32,m33); }
+  inline csVector3 Row3() const { return csVector3 (m31,m32,m33); }
 
   /// Get the first column of this matrix as a vector.
-  csVector3 Col1() const { return csVector3 (m11,m21,m31); }
+  inline csVector3 Col1() const { return csVector3 (m11,m21,m31); }
 
   /// Get the second column of this matrix as a vector.
-  csVector3 Col2() const { return csVector3 (m12,m22,m32); }
+  inline csVector3 Col2() const { return csVector3 (m12,m22,m32); }
 
   /// Get the third column of this matrix as a vector.
-  csVector3 Col3() const { return csVector3 (m13,m23,m33); }
+  inline csVector3 Col3() const { return csVector3 (m13,m23,m33); }
 
   /// Set matrix values.
-  void Set (float o11, float o12, float o13,
-            float o21, float o22, float o23,
-            float o31, float o32, float o33)
+  inline void Set (float o11, float o12, float o13,
+                   float o21, float o22, float o23,
+                   float o31, float o32, float o33)
   {
     m11 = o11; m12 = o12; m13 = o13;
     m21 = o21; m22 = o22; m23 = o23;
     m31 = o31; m32 = o32; m33 = o33;
   }
 
-  void Set (csMatrix3 const &o)
+  inline void Set (csMatrix3 const &o)
   {
     m11 = o.m11; m12 = o.m12; m13 = o.m13;
     m21 = o.m21; m22 = o.m22; m23 = o.m23;
@@ -145,7 +141,7 @@ public:
   csMatrix3 GetTranspose () const;
 
   /// Return the inverse of this matrix.
-  csMatrix3 GetInverse () const
+  inline csMatrix3 GetInverse () const
   {
     csMatrix3 C(
              (m22*m33 - m23*m32), -(m12*m33 - m13*m32),  (m12*m23 - m13*m22),
@@ -157,7 +153,7 @@ public:
   }
 
   /// Invert this matrix.
-  void Invert() { *this = GetInverse (); }
+  inline void Invert() { *this = GetInverse (); }
 
   /// Compute the determinant of this matrix.
   float Determinant () const;
@@ -169,13 +165,13 @@ public:
   bool IsIdentity () const;
 
   /// Add two matricies.
-  friend CS_CSGEOM_EXPORT csMatrix3 operator+ (const csMatrix3& m1, 
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator+ (const csMatrix3& m1, 
     const csMatrix3& m2);
   /// Subtract two matricies.
-  friend CS_CSGEOM_EXPORT csMatrix3 operator- (const csMatrix3& m1, 
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator- (const csMatrix3& m1, 
     const csMatrix3& m2);
   /// Multiply two matricies.
-  friend CS_CSGEOM_EXPORT csMatrix3 operator* (const csMatrix3& m1, 
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator* (const csMatrix3& m1, 
     const csMatrix3& m2);
 
   /// Multiply a vector by a matrix (transform it).
@@ -187,25 +183,29 @@ public:
   }
 
   /// Multiply a matrix and a scalar.
-  friend CS_CSGEOM_EXPORT csMatrix3 operator* (const csMatrix3& m, float f);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator* (const csMatrix3& m,
+  	float f);
   /// Multiply a matrix and a scalar.
-  friend CS_CSGEOM_EXPORT csMatrix3 operator* (float f, const csMatrix3& m);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator* (float f,
+  	const csMatrix3& m);
   /// Divide a matrix by a scalar.
-  friend CS_CSGEOM_EXPORT csMatrix3 operator/ (const csMatrix3& m, float f);
+  friend CS_CRYSTALSPACE_EXPORT csMatrix3 operator/ (const csMatrix3& m,
+  	float f);
   /// Check if two matricies are equal.
-  friend CS_CSGEOM_EXPORT bool operator== (const csMatrix3& m1, 
+  friend CS_CRYSTALSPACE_EXPORT bool operator== (const csMatrix3& m1, 
     const csMatrix3& m2);
   /// Check if two matricies are not equal.
-  friend CS_CSGEOM_EXPORT bool operator!= (const csMatrix3& m1, 
+  friend CS_CRYSTALSPACE_EXPORT bool operator!= (const csMatrix3& m1, 
     const csMatrix3& m2);
   /// Test if each component of a matrix is less than a small epsilon value.
-  friend CS_CSGEOM_EXPORT bool operator< (const csMatrix3& m, float f);
+  friend CS_CRYSTALSPACE_EXPORT bool operator< (const csMatrix3& m, float f);
   /// Test if each component of a matrix is greater than a small epsilon value.
-  friend CS_CSGEOM_EXPORT bool operator> (float f, const csMatrix3& m);
+  friend CS_CRYSTALSPACE_EXPORT bool operator> (float f,
+  	const csMatrix3& m);
 };
 
 /// An instance of csMatrix3 that is initialized as a rotation about X
-class CS_CSGEOM_EXPORT csXRotMatrix3 : public csMatrix3
+class CS_CRYSTALSPACE_EXPORT csXRotMatrix3 : public csMatrix3
 {
 public:
   /**
@@ -218,7 +218,7 @@ public:
 };
 
 /// An instance of csMatrix3 that is initialized as a rotation about Y.
-class CS_CSGEOM_EXPORT csYRotMatrix3 : public csMatrix3
+class CS_CRYSTALSPACE_EXPORT csYRotMatrix3 : public csMatrix3
 {
 public:
   /**
@@ -231,7 +231,7 @@ public:
 };
 
 /// An instance of csMatrix3 that is initialized as a rotation about Z.
-class CS_CSGEOM_EXPORT csZRotMatrix3 : public csMatrix3
+class CS_CRYSTALSPACE_EXPORT csZRotMatrix3 : public csMatrix3
 {
 public:
   /**
@@ -244,7 +244,7 @@ public:
 };
 
 /// An instance of csMatrix3 that is initialized to scale the X dimension.
-class CS_CSGEOM_EXPORT csXScaleMatrix3 : public csMatrix3
+class CS_CRYSTALSPACE_EXPORT csXScaleMatrix3 : public csMatrix3
 {
 public:
   /**
@@ -254,7 +254,7 @@ public:
 };
 
 /// An instance of csMatrix3 that is initialized to scale the Y dimension.
-class CS_CSGEOM_EXPORT csYScaleMatrix3 : public csMatrix3
+class CS_CRYSTALSPACE_EXPORT csYScaleMatrix3 : public csMatrix3
 {
 public:
   /**
@@ -264,7 +264,7 @@ public:
 };
 
 /// An instance of csMatrix3 that is initialized to scale the Z dimension.
-class CS_CSGEOM_EXPORT csZScaleMatrix3 : public csMatrix3
+class CS_CRYSTALSPACE_EXPORT csZScaleMatrix3 : public csMatrix3
 {
 public:
   /**
@@ -273,6 +273,7 @@ public:
   csZScaleMatrix3 (float scaler) : csMatrix3(1, 0, 0, 0, 1, 0, 0, 0, scaler) {}
 };
 
+
 /** @} */
 
 #endif // __CS_MATRIX3_H__
