@@ -39,18 +39,20 @@
 #include "csutil/csosdefs.h"
 
 // Defaults for platforms that do not define their own.
+#ifndef CS_VISIBILITY_DEFAULT
+#  define CS_VISIBILITY_DEFAULT
+#endif
+#ifndef CS_VISIBILITY_HIDDEN
+#  define CS_VISIBILITY_HIDDEN
+#endif
 #ifndef CS_EXPORT_SYM_DLL
-#  ifdef CS_VISIBILITY_DEFAULT
-#    define CS_EXPORT_SYM_DLL CS_VISIBILITY_DEFAULT
-#  else
-#    define CS_EXPORT_SYM_DLL
-#  endif
+#  define CS_EXPORT_SYM_DLL CS_VISIBILITY_DEFAULT
 #endif
 #ifndef CS_IMPORT_SYM_DLL
 #  define CS_IMPORT_SYM_DLL extern
 #endif
 #ifndef CS_EXPORT_SYM
-#  if defined(CS_VISIBILITY_DEFAULT) && defined(CS_BUILD_SHARED_LIBS)
+#  if defined(CS_BUILD_SHARED_LIBS)
 #    define CS_EXPORT_SYM CS_VISIBILITY_DEFAULT
 #  else
 #    define CS_EXPORT_SYM
@@ -68,7 +70,105 @@
  */
 
 #ifndef CS_FORCEINLINE
-#define CS_FORCEINLINE inline
+# ifdef CS_COMPILER_GCC
+#  define CS_FORCEINLINE inline __attribute__((always_inline))
+#  if (__GNUC__ == 3) && (__GNUC_MINOR__ == 4)
+    // Work around a gcc 3.4 issue where forcing inline doesn't always work
+#   define CS_FORCEINLINE_TEMPLATEMETHOD inline
+#  endif
+# else
+#  define CS_FORCEINLINE inline
+# endif
+#endif
+#ifndef CS_FORCEINLINE_TEMPLATEMETHOD
+# define CS_FORCEINLINE_TEMPLATEMETHOD CS_FORCEINLINE
+#endif
+
+/**\def CS_ATTRIBUTE_MALLOC
+ * Function attribute that can be used to mark a function as "malloc". See the
+ * gcc manual for the implications of that.
+ */
+#ifndef CS_ATTRIBUTE_MALLOC
+# define CS_ATTRIBUTE_MALLOC
+#endif
+
+/**\def CS_DEPRECATED_METHOD
+ * Use the CS_DEPRECATED_METHOD macro in front of method declarations to
+ * indicate that they are deprecated. Example:
+ * \code
+ * struct iFoo : iBase {
+ *   CS_DEPRECATED_METHOD virtual void Plankton() const = 0;
+ * }
+ * \endcode
+ * Compilers which are capable of flagging deprecation will exhibit a warning
+ * when it encounters client code invoking methods so tagged.
+ */
+#if !defined(CS_DEPRECATED_METHOD) || defined(DOXYGEN_RUN)
+#  if defined(CS_COMPILER_MSVC)
+#    define CS_DEPRECATED_METHOD	__declspec(deprecated)
+      /* Unfortunately, MSVC is overzealous with warnings; it even emits one 
+	 when a deprecated method is overridden, e.g. when implementing an 
+	 interface method. 
+	 To work around this, use msvc_deprecated_warn_off.h/
+	 msvc_deprecated_warn_on.h. */
+#  else
+#    define CS_DEPRECATED_METHOD
+#  endif
+#endif
+
+/**\def CS_DEPRECATED_METHOD_MSG
+ * A variant of #CS_DEPRECATED_METHOD that also emits the message \a msg
+ * on compilers that support it.
+ */
+#if !defined(CS_DEPRECATED_METHOD_MSG) || defined(DOXYGEN_RUN)
+#  if defined(CS_COMPILER_MSVC) && _MSC_VER >= 1400
+#    define CS_DEPRECATED_METHOD_MSG(msg) __declspec(deprecated(msg))
+#  else
+#    define CS_DEPRECATED_METHOD_MSG(msg) CS_DEPRECATED_METHOD
+#  endif
+#endif
+
+/**\def CS_DEPRECATED_TYPE
+ * Use the CS_DEPRECATED_TYPE macro after type declarations to
+ * indicate that they are deprecated. Example:
+ * \code
+ * typedef CS_DEPRECATED_TYPE csFoo csBar;
+ * class CS_DEPRECATED_TYPE csBaz { };
+ * \endcode
+ * Compilers which are capable of flagging deprecation will exhibit a warning
+ * when it encounters client code using types so tagged.
+ */
+#if !defined(CS_DEPRECATED_TYPE) || defined(DOXYGEN_RUN)
+#  if defined(CS_COMPILER_MSVC)
+#    define CS_DEPRECATED_TYPE __declspec(deprecated)
+#  else
+#    define CS_DEPRECATED_TYPE
+#  endif
+#endif
+
+/**\def CS_DEPRECATED_TYPE_MSG
+ * A variant of CS_DEPRECATED_TYPE that also emits the message \a msg
+ * on compilers that support it.
+ */
+#if !defined(CS_DEPRECATED_TYPE_MSG) || defined(DOXYGEN_RUN)
+#  if defined(CS_COMPILER_MSVC) && _MSC_VER >= 1400
+#    define CS_DEPRECATED_TYPE_MSG(msg) __declspec(deprecated(msg))
+#  else
+#    define CS_DEPRECATED_TYPE_MSG(msg) CS_DEPRECATED_TYPE
+#  endif
+#endif
+
+/**\def CS_NO_EXCEPTIONS
+ * This is defined when the project was compiled without support for 
+ * exceptions.
+ */
+#if defined(CS_COMPILER_MSVC) 
+  #include <exception>
+  #if !_HAS_EXCEPTIONS
+    #define CS_NO_EXCEPTIONS
+  #endif
+#elif defined(CS_COMPILER_GCC) && !defined(__EXCEPTIONS)
+  #define CS_NO_EXCEPTIONS
 #endif
 
 /**\def CS_MAXPATHLEN
@@ -102,45 +202,44 @@
 #  endif
 #endif
 
+
+
+
 /**\def CS_TEMP_DIR
  * Directory for temporary files
+ * \deprecated Use CS::Platform::GetTempDirectory () from syspath.h
  */
 #ifndef CS_TEMP_DIR
-#  if defined(CS_PLATFORM_UNIX)
-#    define CS_TEMP_DIR "/tmp/"
-#  else
-#    define CS_TEMP_DIR ""
-#  endif
+class csString;
+
+namespace CS
+{
+  namespace Macros
+  {
+    CS_DEPRECATED_METHOD_MSG("Use CS::Platform::GetTempDirectory () from "
+      "syspath.h") CS_CRYSTALSPACE_EXPORT csString CS_TEMP_DIR ();
+  }
+}
+#  define CS_TEMP_DIR CS::Macros::CS_TEMP_DIR.GetDataSafe()
 #endif
 
+
 /**\def CS_TEMP_FILE
  * Name for temporary files
+ * \deprecated Use CS::Platform::GetTempFilename () from syspath.h
  */
 #ifndef CS_TEMP_FILE
-#  if defined(CS_PLATFORM_UNIX)
-#    define CS_TEMP_FILE "cs%lud.tmp", (unsigned long)getpid()
-#  else
-#    define CS_TEMP_FILE "$cs$.tmp"
-#  endif
-#endif
+class csString;
 
-#ifdef CS_USE_CUSTOM_ISDIR
-static inline bool isdir (const char *path, struct dirent *de)
+namespace CS
 {
-  int pathlen = strlen (path);
-  char* fullname = new char[pathlen + 2 + strlen (de->d_name)];
-  memcpy (fullname, path, pathlen + 1);
-  if ((pathlen) && (fullname[pathlen-1] != CS_PATH_SEPARATOR))
+  namespace Macros
   {
-    fullname[pathlen++] = CS_PATH_SEPARATOR;
-    fullname[pathlen] = 0;
+    CS_DEPRECATED_METHOD_MSG("Use CS::Platform::GetTempFilename () from "
+      "syspath.h") CS_CRYSTALSPACE_EXPORT csString CS_TEMP_FILE ();
   }
-  strcat (&fullname [pathlen], de->d_name);
-  struct stat st;
-  stat (fullname, &st);
-  delete[] fullname;
-  return ((st.st_mode & S_IFMT) == S_IFDIR);
 }
+#  define CS_TEMP_FILE CS::Macros::CS_TEMP_FILE.GetDataSafe()
 #endif
 
 /**\def CS_HAVE_POSIX_MMAP
@@ -151,25 +250,6 @@ static inline bool isdir (const char *path, struct dirent *de)
  * CS/libs/csutil/generic/mmap.cpp incorporated into the csutil library.
  */
 
-// Handle platforms with no native aligned malloc
-#ifndef CS_HAVE_CSALIGNED_MALLOC
-static inline void* csAlignedMalloc (size_t size, size_t align)
-{
-  void *mallocPtr = malloc(size + align + sizeof(void*));
-  size_t ptrInt = (size_t)mallocPtr;
-
-  ptrInt = (ptrInt + align + sizeof(void*)) / align * align;
-  *(((void**)ptrInt) - 1) = mallocPtr;
-
-  return (void*)ptrInt;
-}
-
-static inline void csAlignedFree (void* ptr)
-{
-  free(*(((void**)ptr) - 1));
-}
-#endif
-
 /**
  * The CS_HEADER_GLOBAL() macro composes a pathname from two components and
  * wraps the path in `<' and `>'.  This macro is useful in cases where one does
@@ -438,7 +518,7 @@ void Name (void (*p)())                                                \
 
 #ifndef CS_IMPLEMENT_STATIC_VAR_EXT
 #define CS_IMPLEMENT_STATIC_VAR_EXT(getterFunc,Type,initParam,kill_how) \
-extern "C" {                                                            \
+namespace {                                                            \
 static Type *getterFunc ## _v=0;                                        \
 static Type* getterFunc ();                                             \
 static void getterFunc ## _kill ();					\
@@ -487,13 +567,17 @@ Type* getterFunc ()                                                     \
 #ifndef CS_DECLARE_STATIC_CLASSVAR
 #define CS_DECLARE_STATIC_CLASSVAR(var,getterFunc,Type)       \
 static Type *var;                                             \
-static Type *getterFunc ();                                   
+static Type *getterFunc ();                                   \
+static void getterFunc ## _kill ();              	      \
+static void getterFunc ## _kill_array ();
 #endif
 
 #ifndef CS_DECLARE_STATIC_CLASSVAR_REF
 #define CS_DECLARE_STATIC_CLASSVAR_REF(var,getterFunc,Type)   \
 static Type *var;                                             \
-static Type &getterFunc ();                                   
+static Type &getterFunc ();                                   \
+static void getterFunc ## _kill ();              	      \
+static void getterFunc ## _kill_array ();
 #endif
 
 /**\def CS_IMPLEMENT_STATIC_CLASSVAR(Class,var,getterFunc,Type,initParam)
@@ -510,26 +594,22 @@ static Type &getterFunc ();
 #define CS_IMPLEMENT_STATIC_CLASSVAR_EXT(Class,var,getterFunc,Type,initParam,\
   kill_how)                                                    	\
 Type *Class::var = 0;                                          	\
-extern "C" {                                                   	\
-static void Class ## _ ## getterFunc ## _kill ();              	\
-static void Class ## _ ## getterFunc ## _kill_array ();        	\
-void Class ## _ ## getterFunc ## _kill ()               	\
+void Class::getterFunc ## _kill ()               	        \
 {                                                              	\
-  delete Class::getterFunc ();                                 	\
-  (void)(&Class ## _ ## getterFunc ## _kill_array);		\
+  delete getterFunc ();                                 	\
+  var = 0;							\
 }                                                              	\
-void Class ## _ ## getterFunc ## _kill_array ()         	\
+void Class::getterFunc ## _kill_array ()         	        \
 {                                                              	\
-  delete [] Class::getterFunc ();                              	\
-  (void)(&Class ## _ ## getterFunc ## _kill);			\
-}                                                              	\
+  delete [] getterFunc ();                              	\
+  var = 0;							\
 }                                                              	\
 Type* Class::getterFunc ()                                     	\
 {                                                              	\
   if (!var)                                                    	\
   {                                                            	\
     var = new Type initParam;                                  	\
-    csStaticVarCleanup (Class ## _ ## getterFunc ## kill_how); 	\
+    csStaticVarCleanup (getterFunc ## kill_how); 	        \
   }                                                            	\
   return var;                                                  	\
 }
@@ -551,26 +631,22 @@ Type* Class::getterFunc ()                                     	\
 #define CS_IMPLEMENT_STATIC_CLASSVAR_REF_EXT(Class,var,getterFunc,Type,\
   initParam,kill_how) \
 Type *Class::var = 0;                                          \
-extern "C" {                                                   \
-static void Class ## _ ## getterFunc ## _kill ();              \
-static void Class ## _ ## getterFunc ## _kill_array ();        \
-void Class ## _ ## getterFunc ## _kill ()                      \
+void Class::getterFunc ## _kill ()                             \
 {                                                              \
-  (void) &Class ## _ ## getterFunc ## _kill_array;              \
-  delete &Class::getterFunc ();                                \
+  delete &getterFunc ();                                       \
+  var = 0;						       \
 }                                                              \
-void Class ## _ ## getterFunc ## _kill_array ()                \
+  void Class::getterFunc ## _kill_array ()                     \
 {                                                              \
-  (void) &Class ## _ ## getterFunc ## _kill;                    \
-  delete [] &Class::getterFunc ();                             \
-}                                                              \
+  delete [] &getterFunc ();                                    \
+  var = 0;						       \
 }                                                              \
 Type &Class::getterFunc ()                                     \
 {                                                              \
   if (!var)                                                    \
   {                                                            \
     var = new Type initParam;                                  \
-    csStaticVarCleanup (Class ## _ ## getterFunc ## kill_how); \
+    csStaticVarCleanup (getterFunc ## kill_how);               \
   }                                                            \
   return *var;                                                 \
 }
@@ -589,6 +665,138 @@ Type &Class::getterFunc ()                                     \
     _kill_array)
 #endif
 
+/**\def CS_FUNCTION_NAME
+ * Macro that resolves to a compiler-specific variable or string that contains 
+ * the name of the current function.
+ */
+#if defined(CS_COMPILER_GCC)
+#  define CS_FUNCTION_NAME		__PRETTY_FUNCTION__
+#elif defined(__FUNCTION__)
+#  define CS_FUNCTION_NAME		__FUNCTION__
+#else
+#  define CS_FUNCTION_NAME		"<?\?\?>"
+#endif
+
+#include <stdlib.h>
+#ifdef CS_HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+#include <new>
+
+/**\name Platform-specific memory allocation
+ * If built with ptmalloc support, these functions can be used to explicitly
+ * call the platform's default malloc/free resp. operator new/operator delete
+ * implementations. Useful when interfacing with third party libraries.
+ */
+//@{
+CS_DEPRECATED_METHOD_MSG("malloc override was removed; "
+  "platform_malloc() unnecessary, use normal malloc() instead")
+CS_FORCEINLINE CS_ATTRIBUTE_MALLOC void* platform_malloc (size_t n)
+{ return malloc (n); }
+CS_DEPRECATED_METHOD_MSG("malloc override was removed; "
+  "platform_free() unnecessary, use normal free() instead")
+CS_FORCEINLINE void platform_free (void* p)
+{ return free (p); }
+CS_DEPRECATED_METHOD_MSG("malloc override was removed; "
+  "platform_realloc() unnecessary, use normal realloc() instead")
+CS_FORCEINLINE void* platform_realloc (void* p, size_t n)
+{ return realloc (p, n); }
+CS_DEPRECATED_METHOD_MSG("malloc override was removed; "
+  "platform_calloc() unnecessary, use normal calloc() instead")
+CS_FORCEINLINE CS_ATTRIBUTE_MALLOC void* platform_calloc (size_t n, size_t s)
+{ return calloc (n, s); }
+
+namespace CS
+{
+  struct AllocPlatform {};
+  extern CS_CRYSTALSPACE_EXPORT const AllocPlatform allocPlatform;
+}
+/**
+ * Platform-dependent operator new.
+ * \remarks Won't throw an exception if allocation fails.
+ */
+CS_DEPRECATED_METHOD_MSG("operator new override was removed; "
+  "using 'platform new' should be unnecessary")
+extern CS_CRYSTALSPACE_EXPORT void* operator new (size_t s, 
+  const CS::AllocPlatform&) throw();
+/**
+ * Platform-dependent operator new.
+ * \remarks Won't throw an exception if allocation fails.
+ */
+CS_DEPRECATED_METHOD_MSG("operator new override was removed; "
+  "using 'platform new' should be unnecessary")
+extern CS_CRYSTALSPACE_EXPORT void* operator new[] (size_t s, 
+  const CS::AllocPlatform&) throw();
+CS_DEPRECATED_METHOD_MSG("operator new override was removed; "
+  "using 'platform delete' should be unnecessary")
+extern CS_CRYSTALSPACE_EXPORT void operator delete (void* p, 
+  const CS::AllocPlatform&) throw();
+CS_DEPRECATED_METHOD_MSG("operator new override was removed; "
+  "using 'platform delete' should be unnecessary")
+extern CS_CRYSTALSPACE_EXPORT void operator delete[] (void* p, 
+  const CS::AllocPlatform&) throw();
+//@}
+
+#ifndef CS_NO_PTMALLOC
+//@{
+/**\name ptmalloc memory allocation
+ * Directly use the ptmalloc allocation functions. Usually, this is not needed -
+ * use cs_malloc() etc instead.
+ */
+extern CS_CRYSTALSPACE_EXPORT CS_ATTRIBUTE_MALLOC void* ptmalloc (size_t n);
+extern CS_CRYSTALSPACE_EXPORT void ptfree (void* p);
+extern CS_CRYSTALSPACE_EXPORT void* ptrealloc (void* p, size_t n);
+extern CS_CRYSTALSPACE_EXPORT CS_ATTRIBUTE_MALLOC void* ptcalloc (size_t n,
+  size_t s);
+//@}
+
+/**\name Default Crystal Space memory allocation
+ * Always the same memory allocation functions as internally used by 
+ * Crystal Space.
+ */
+//@{
+CS_FORCEINLINE CS_ATTRIBUTE_MALLOC void* cs_malloc (size_t n)
+{ return ptmalloc (n); }
+CS_FORCEINLINE void cs_free (void* p)
+{ ptfree (p); }
+CS_FORCEINLINE void* cs_realloc (void* p, size_t n)
+{ return ptrealloc (p, n); }
+CS_FORCEINLINE CS_ATTRIBUTE_MALLOC void* cs_calloc (size_t n, size_t s)
+{ return ptcalloc (n, s); }
+//@}
+
+#else // CS_NO_PTMALLOC
+CS_FORCEINLINE CS_ATTRIBUTE_MALLOC void* cs_malloc (size_t n)
+{ return malloc (n); }
+CS_FORCEINLINE void cs_free (void* p)
+{ free (p); }
+CS_FORCEINLINE void* cs_realloc (void* p, size_t n)
+{ return realloc (p, n); }
+CS_FORCEINLINE CS_ATTRIBUTE_MALLOC void* cs_calloc (size_t n, size_t s)
+{ return calloc (n, s); }
+#endif // CS_NO_PTMALLOC
+//@}
+
+#ifdef CS_USE_CUSTOM_ISDIR
+static inline bool isdir (const char *path, struct dirent *de)
+{
+  int pathlen = strlen (path);
+  char* fullname = new char[pathlen + 2 + strlen (de->d_name)];
+  memcpy (fullname, path, pathlen + 1);
+  if ((pathlen) && (fullname[pathlen-1] != CS_PATH_SEPARATOR))
+  {
+    fullname[pathlen++] = CS_PATH_SEPARATOR;
+    fullname[pathlen] = 0;
+  }
+  strcat (&fullname [pathlen], de->d_name);
+  struct stat st;
+  stat (fullname, &st);
+  delete[] fullname;
+  return ((st.st_mode & S_IFMT) == S_IFDIR);
+}
+#endif
+
+
 // The following define should only be enabled if you have defined
 // a special version of overloaded new that accepts two additional
 // parameters: a (void*) pointing to the filename and an int with the
@@ -612,12 +820,31 @@ Type &Class::getterFunc ()                                     \
 #  endif
 #endif
 #if defined(CS_EXTENSIVE_MEMDEBUG) || defined(CS_MEMORY_TRACKER)
-extern void* operator new (size_t s, void* filename, int line);
-extern void* operator new[] (size_t s, void* filename, int line);
-#define CS_EXTENSIVE_MEMDEBUG_NEW new ((void*)__FILE__, __LINE__)
+extern void* CS_CRYSTALSPACE_EXPORT operator new (size_t s, 
+  void* filename, int line);
+inline void operator delete (void* p, void*, int) { operator delete (p); }
+extern void* CS_CRYSTALSPACE_EXPORT operator new[] (size_t s, 
+  void* filename, int line);
+inline void operator delete[] (void* p, void*, int) { operator delete[] (p); }
+
+inline void* operator new (size_t s)
+{ return operator new (s, (void*)__FILE__, 0); }
+inline void* operator new[] (size_t s)
+{ return operator new (s, (void*)__FILE__, 0); }
+
+#define CS_EXTENSIVE_MEMDEBUG_NEW new ((void*)CS_FUNCTION_NAME, __LINE__)
 #define new CS_EXTENSIVE_MEMDEBUG_NEW
 #endif
 
+namespace CS
+{
+  namespace Debug
+  {
+    extern void CS_CRYSTALSPACE_EXPORT AssertMessage (const char* expr, 
+      const char* filename, int line, const char* msg = 0);
+  } // namespace Debug
+} // namespace CS
+
 #ifdef CS_DEBUG
 #  if !defined (CS_DEBUG_BREAK)
 #    if defined (CS_PLATFORM_WIN32)
@@ -633,16 +860,8 @@ extern void* operator new[] (size_t s, void* filename, int line);
 #    endif
 #  endif
 #  if !defined (CS_ASSERT_MSG)
-    namespace CrystalSpace
-    {
-      namespace Debug
-      {
-	extern void AssertMessage (const char* expr, const char* filename, 
-	  int line, const char* msg = 0);
-      } // namespace Debug
-    } // namespace CrystalSpace
 #   define CS_ASSERT_MSG(msg,x) 					\
-      if (!(x)) CrystalSpace::Debug::AssertMessage (#x, __FILE__, __LINE__, msg);
+      if (!(x)) CS::Debug::AssertMessage (#x, __FILE__, __LINE__, msg);
 #  endif
 #  if !defined (CS_ASSERT)
 #    define CS_ASSERT(x)	CS_ASSERT_MSG(0, x)
@@ -665,50 +884,11 @@ extern void* operator new[] (size_t s, void* filename, int line);
  * as well as a call stack is printed to <tt>stderr</tt> and a debug break is
  * performed
  * \remarks Breaking execution can be avoided at runtime by setting the 
- *   environment variable "<tt>CS_ASSERT_IGNORE</tt>" to a value other than 0.
+ *   environment variable <tt>"CS_ASSERT_IGNORE"</tt> to a value other than 0.
  */
 /**\def CS_ASSERT_MSG(msg, expr)
  * Same as #CS_ASSERT(expr), but additionally prints \a msg to <tt>stderr</tt>.
  */
-    
-/**\def CS_DEPRECATED_METHOD
- * Use the CS_DEPRECATED_METHOD macro in front of method declarations to
- * indicate that they are deprecated. Example:
- * \code
- * struct iFoo : iBase {
- *   CS_DEPRECATED_METHOD virtual void Plankton() const = 0;
- * }
- * \endcode
- * Compilers which are capable of flagging deprecation will exhibit a warning
- * when it encounters client code invoking methods so tagged.
- */
-#if !defined(CS_DEPRECATED_METHOD) || defined(DOXYGEN_RUN)
-#  if defined(CS_COMPILER_MSVC)
-#    define CS_DEPRECATED_METHOD		/*__declspec(deprecated)*/
-      /* Disabled: Unfortunately, MSVC is overzealous with warnings; 
-	 it even emits one when a deprecated method is overridden, e.g. when 
-	 implementing an interface method. */
-#  else
-#    define CS_DEPRECATED_METHOD
-#  endif
-#endif
-
-/**\def CS_DEPRECATED_TYPE
- * Use the CS_DEPRECATED_TYPE macro after type declarations to
- * indicate that they are deprecated. Example:
- * \code
- * typedef csFoo csBar CS_DEPRECATED_TYPE;
- * \endcode
- * Compilers which are capable of flagging deprecation will exhibit a warning
- * when it encounters client code using types so tagged.
- */
-#if !defined(CS_DEPRECATED_TYPE) || defined(DOXYGEN_RUN)
-#  if defined(CS_COMPILER_MSVC)
-#    define CS_DEPRECATED_TYPE
-#  else
-#    define CS_DEPRECATED_TYPE
-#  endif
-#endif
 
 /**\def CS_CONST_METHOD
  * Use the CS_CONST_METHOD macro in front of method declarations to
@@ -751,10 +931,10 @@ extern void* operator new[] (size_t s, void* filename, int line);
  * strict alignment add your CPU to the !defined below to get slightly
  * smaller and faster code in some cases.
  *
- * @@@ In the future, this should be moved to csconfig.h and determined as
+ * \todo In the future, this should be moved to csconfig.h and determined as
  * part of the configuration process.
  */
-#if !defined (CS_PROCESSOR_X86)
+#if defined (CS_PROCESSOR_SPARC)
 #  define CS_STRICT_ALIGNMENT
 #endif
 
@@ -769,6 +949,9 @@ extern void* operator new[] (size_t s, void* filename, int line);
 #if !defined (CS_IEEE_DOUBLE_FORMAT)
 #  if defined (CS_PROCESSOR_X86) || \
       defined (CS_PROCESSOR_POWERPC) || \
+      defined (CS_PROCESSOR_MIPS) || \
+      defined (CS_PROCESSOR_SPARC) || \
+      defined (CS_PROCESSOR_ALPHA) || \
       defined (CS_PROCESSOR_M68K)
 #    define CS_IEEE_DOUBLE_FORMAT
 #  endif
@@ -781,9 +964,16 @@ extern void* operator new[] (size_t s, void* filename, int line);
      __attribute__((format (__printf__, format_idx, arg_idx)))
 #  define CS_GNUC_SCANF(format_idx, arg_idx) \
      __attribute__((format (__scanf__, format_idx, arg_idx)))
+// Unfortunately, gcc doesn't support format argument checking for wide strings
+#  define CS_GNUC_WPRINTF(format_idx, arg_idx) \
+     /*__attribute__((format (__wprintf__, format_idx, arg_idx)))*/
+#  define CS_GNUC_WSCANF(format_idx, arg_idx) \
+     /*__attribute__((format (__wscanf__, format_idx, arg_idx)))*/
 #else
 #  define CS_GNUC_PRINTF(format_idx, arg_idx)
 #  define CS_GNUC_SCANF(format_idx, arg_idx)
+#  define CS_GNUC_WPRINTF(format_idx, arg_idx)
+#  define CS_GNUC_WSCANF(format_idx, arg_idx)
 #endif
 
 // Remove __attribute__ on non GNUC compilers.
@@ -881,18 +1071,6 @@ extern void* operator new[] (size_t s, void* filename, int line);
 #  endif
 #endif
 
-/**\def CS_FUNCTION_NAME
- * Macro that resolves to a compiler-specific variable or string that contains 
- * the name of the current function.
- */
-#if defined(CS_COMPILER_GCC)
-#  define CS_FUNCTION_NAME		__PRETTY_FUNCTION__
-#elif defined(__FUNCTION__)
-#  define CS_FUNCTION_NAME		__FUNCTION__
-#else
-#  define CS_FUNCTION_NAME		"<?\?\?>"
-#endif
-
 #define CS_STRING_TO_WIDE_(x)   L ## x
 /**\def CS_STRING_TO_WIDE
  * Convert a string to a wide string. Also works in macros.
@@ -903,4 +1081,41 @@ extern void* operator new[] (size_t s, void* filename, int line);
  */
 #define CS_STRING_TO_WIDE(x)    CS_STRING_TO_WIDE_(x)
 
+#ifdef PACKAGE_NAME
+# define CS_NAMESPACE_PACKAGE_NAME       PACKAGE_NAME
+#else
+# define CS_NAMESPACE_PACKAGE_NAME       CS
+#endif
+
+/**\name Plugin namespace helpers
+ * To avoid symbol conflicts when employing static linking, it is a good
+ * idea to put everything into a private namespace. The 
+ * CS_PLUGIN_NAMESPACE_BEGIN and CS_PLUGIN_NAMESPACE_END macros help with 
+ * that by putting the plugin namespace into a sub-namespace for plugins.
+ *
+ * Use like:
+ * \code
+ * #include <...>
+ *
+ * CS_IMPLEMENT_PLUGIN
+ *
+ * CS_PLUGIN_NAMESPACE_BEGIN(MyPlugin)
+ * {
+ *   ...
+ *   MyClass::MyClass (...) { ... }
+ *   ...
+ * }
+ * CS_PLUGIN_NAMESPACE_END(MyPlugin)
+ * \endcode
+ * To refer to members of the namespace (e.g. for template specializations,
+ * use CS_PLUGIN_NAMESPACE_NAME.
+ * @{ */
+#define CS_PLUGIN_NAMESPACE_BEGIN(name)                                     \
+  namespace CS_NAMESPACE_PACKAGE_NAME { namespace Plugin { namespace name
+#define CS_PLUGIN_NAMESPACE_END(name)                                       \
+  } }
+#define CS_PLUGIN_NAMESPACE_NAME(name)                                      \
+  CS_NAMESPACE_PACKAGE_NAME::Plugin::name
+/** @} */
+
 #endif // __CS_CSSYSDEF_H__
