@@ -30,6 +30,7 @@
 #include "csutil/leakguard.h"
 #include "csutil/array.h"
 #include "csutil/scf_implementation.h"
+#include "csutil/set.h"
 
 #include "ivaria/collider.h"
 
@@ -41,14 +42,18 @@ struct iMeshWrapper;
 struct iMovable;
 struct iObject;
 struct iPolygonMesh;
+struct iTriangleMesh;
 struct iRegion;
 struct iSector;
+struct iTerrainSystem;
 
 struct csCollisionPair;
 class csReversibleTransform;
 
 struct csIntersectingTriangle;
 
+// for iPolygonMesh
+#include "csutil/win32/msvc_deprecated_warn_off.h"
 
 /**
  * This is a convenience object that you can use in your own
@@ -63,8 +68,8 @@ struct csIntersectingTriangle;
  * attach itself to the given object. You can use
  * csColliderWrapper::GetCollider() later to get the collider again.
  */
-class CS_CRYSTALSPACE_EXPORT csColliderWrapper : 
-  public scfImplementationExt1<csColliderWrapper, 
+class CS_CRYSTALSPACE_EXPORT csColliderWrapper :
+  public scfImplementationExt1<csColliderWrapper,
                                csObject,
                                scfFakeInterface<csColliderWrapper> >
 {
@@ -73,18 +78,50 @@ private:
   csRef<iCollider> collider;
 
 public:
-  SCF_INTERFACE(csColliderWrapper, 2,0,0);
+  SCF_INTERFACE(csColliderWrapper, 2,1,0);
 
   CS_LEAKGUARD_DECLARE (csColliderWrapper);
 
-  /// Create a collider based on a mesh.
+  /**
+   * Create a collider based on a mesh.
+   * \deprecated Use version with iTriangleMesh instead.
+   */
+#ifndef CS_DEPRECATION_SUPPRESS_HACK
+  CS_DEPRECATED_METHOD_MSG("Use version with iTriangleMesh instead.")
+#endif
   csColliderWrapper (csObject& parent, iCollideSystem* collide_system,
   	iPolygonMesh* mesh);
 
-  /// Create a collider based on a mesh.
+  /**
+   * Create a collider based on a mesh.
+   * \deprecated Use version with iTriangleMesh instead.
+   */
+#ifndef CS_DEPRECATION_SUPPRESS_HACK
+  CS_DEPRECATED_METHOD_MSG("Use version with iTriangleMesh instead.")
+#endif
   csColliderWrapper (iObject* parent, iCollideSystem* collide_system,
   	iPolygonMesh* mesh);
 
+  /**
+   * Create a collider based on a mesh.
+   */
+  csColliderWrapper (csObject& parent, iCollideSystem* collide_system,
+  	iTriangleMesh* mesh);
+
+  /**
+   * Create a collider based on a mesh.
+   */
+  csColliderWrapper (iObject* parent, iCollideSystem* collide_system,
+  	iTriangleMesh* mesh);
+
+  /// Create a collider based on a terrain.
+  csColliderWrapper (iObject* parent, iCollideSystem* collide_system,
+  	iTerraFormer* terrain);
+
+  /// Create a collider based on a terrain.
+  csColliderWrapper (iObject* parent, iCollideSystem* collide_system,
+  	iTerrainSystem* terrain);
+
   /**
    * Create a collider based on a collider. Note that it is legal to pass
    * in a 0 collider. In that case it indicates that this object has no
@@ -138,8 +175,27 @@ public:
    * Otherwise 0 is returned.
    */
   static csColliderWrapper* GetColliderWrapper (iObject* object);
+
+  /**
+   * Update collider from a polymesh.
+   * \deprecated Use version with iTriangleMesh instead.
+   */
+  CS_DEPRECATED_METHOD_MSG("Use version with iTriangleMesh instead.")
+  void UpdateCollider (iPolygonMesh* mesh);
+
+  /**
+   * Update collider from a triangle mesh.
+   */
+  void UpdateCollider (iTriangleMesh* mesh);
+
+  /// Update collider from a terraformer.
+  void UpdateCollider (iTerraFormer* terrain);
+
 };
 
+// for iPolygonMesh
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * Return structure for the csColliderHelper::TraceBeam() method.
  */
@@ -340,6 +396,9 @@ private:
   iCollideSystem* cdsys;
   iEngine* engine;
   csVector3 velWorld;
+  /// Set of meshes we hit with last call to Move.
+  csSet<csPtrKey<iMeshWrapper> > hit_meshes;
+  bool do_hit_meshes;
 
   /// For rotation - Euler angles in radians
   csVector3 rotation;
@@ -483,7 +542,27 @@ public:
   bool CheckRevertMove () const { return revertMove; }
 
   /**
-   * Move the model.
+   * Enable remembering of the meshes we hit.
+   * By default this is disabled. If this is enabled you can call
+   * GetHitMeshes() after calling Move() to get a set of all meshes
+   * that were hit.
+   */
+  void EnableHitMeshes (bool hm) { do_hit_meshes = hm; }
+
+  /// Return true if we remember the meshes we hit.
+  bool CheckHitMeshes () const { return do_hit_meshes; }
+
+  /**
+   * Return the meshes that we hit in the last call to Move().
+   * Calling Move() again will clear this set and calculate it again.
+   * This works only if EnableHitMeshes(true) is called.
+   */
+  const csSet<csPtrKey<iMeshWrapper> >& GetHitMeshes ()
+  { return hit_meshes; }
+
+  /**
+   * Move the model. If EnableHitMeshes(true) is set then you can
+   * use GetHitMeshes() after this to detect the meshes that were hit.
    * \param delta is the number of seconds (floating point) elapsed
    * time. Typically this is the elapsed time from the virtual clock
    * divided by 1000.0f.
