@@ -27,17 +27,21 @@
 
 #include "csgeom/vector2.h"
 #include "csutil/cscolor.h"
-#include "csutil/scf.h"
+#include "csutil/ref.h"
+#include "csutil/scf_interface.h"
 
 struct iImage;
 
 struct csRGBpixel;
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 /**
  * A base class which represents a value that can be computed
  * for blending purposes for each pixel.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageValue
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageValue
 {
 public:
   /// delete it
@@ -49,8 +53,9 @@ public:
 /**
  * A base class which represents a texture that can be displayed
  * on the terrain. It has a colour for each pixel
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageTexture
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageTexture
 {
 public:
   /// delete it
@@ -69,8 +74,9 @@ public:
  * at certain heights, the whole texture is computed. At each
  * pixel the base textures are blended together, based on the
  * height.
-*/
-class CS_CRYSTALSPACE_EXPORT csGenerateImage
+ * \deprecated
+ */
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImage
 {
 private:
   /// the texture to show
@@ -102,8 +108,9 @@ public:
 /**
  * This class is used to store the layers of textures per value. Used in the
  * Blend class.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageLayer
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageLayer
 {
 public:
   /// the value where this texture should show
@@ -116,8 +123,9 @@ public:
 
 /**
  * A class for a solid coloured texture.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageTextureSolid : 
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageTextureSolid : 
   public csGenerateImageTexture
 {
 public:
@@ -131,8 +139,9 @@ public:
 
 /**
  * A class for a single texture.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageTextureSingle : 
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageTextureSingle : 
   public csGenerateImageTexture
 {
 public:
@@ -158,8 +167,9 @@ public:
 /**
  * a class for a texture that is made by blending together other textures
  * based on a value. It has a set of layers to blend between.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageTextureBlend : 
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageTextureBlend : 
   public csGenerateImageTexture
 {
 public:
@@ -177,14 +187,14 @@ public:
   void AddLayer(float value, csGenerateImageTexture *tex);
 };
 
-SCF_VERSION (iGenerateImageFunction, 0, 0, 1);
-
 /**
  * This class represents a function for csGenerateImageValueFunc. Expects
  * values for dx and dy between 0 and 1 and returns a height or slope.
+ * \deprecated
  */
-struct iGenerateImageFunction : public iBase
+struct CS_DEPRECATED_TYPE iGenerateImageFunction : public virtual iBase
 {
+  SCF_INTERFACE (iGenerateImageFunction, 2, 0, 0);
   /// Get height or slope.
   virtual float GetValue (float dx, float dy) = 0;
 };
@@ -193,8 +203,9 @@ struct iGenerateImageFunction : public iBase
 /**
  * This class will generate a value using a given function. For heights
  * or slopes.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageValueFunc
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageValueFunc
 	: public csGenerateImageValue
 {
 private:
@@ -221,8 +232,9 @@ public:
 
 /**
  * This class will generate a constant value.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageValueFuncConst : 
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageValueFuncConst : 
   public csGenerateImageValue
 {
 public:
@@ -235,8 +247,9 @@ public:
 /**
  * This class will generate a value using a texture. The average of the
  * rgb values will be returned.
+ * \deprecated
  */
-class CS_CRYSTALSPACE_EXPORT csGenerateImageValueFuncTex : 
+class CS_DEPRECATED_TYPE CS_CRYSTALSPACE_EXPORT csGenerateImageValueFuncTex : 
   public csGenerateImageValue
 {
 public:
@@ -248,6 +261,7 @@ public:
   virtual float GetValue(float x, float y);
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
 
 #endif // __CS_GENTERTEX_H__
 
