@@ -26,6 +26,7 @@
 
 #include "csextern.h"
 
+#include "csutil/csstring.h"
 #include "csutil/parray.h"
 #include "csutil/stringarray.h"
 #include "csutil/zip.h"
@@ -54,10 +55,10 @@ struct csFileTime;
 class CS_CRYSTALSPACE_EXPORT csArchive
 {
 public:
-  static char hdr_central[4];
-  static char hdr_local[4];
-  static char hdr_endcentral[4];
-  static char hdr_extlocal[4];
+  static const char hdr_central[4];
+  static const char hdr_local[4];
+  static const char hdr_endcentral[4];
+  static const char hdr_extlocal[4];
 
 private:
   /// csArchive entry class
@@ -193,7 +194,7 @@ public:
 
   /// Get Nth file in archive or 0
   void *GetFile (size_t no)
-  { return (no < dir.Length ()) ? dir.Get (no) : 0; }
+  { return (no < dir.GetSize ()) ? dir.Get (no) : 0; }
 
   /// Find a file in archive; returns a handle or 0
   void *FindName (const char *name) const;
