@@ -25,13 +25,11 @@
  * Generic Array Template
  */
 
-#include "comparator.h"
+#include "csutil/allocator.h"
+#include "csutil/comparator.h"
+#include "csutil/customallocated.h"
 
-// Hack: Work around problems caused by #defining 'new'.
-#if defined(CS_EXTENSIVE_MEMDEBUG) || defined(CS_MEMORY_TRACKER)
-# undef new
-#endif
-#include <new>
+#include "csutil/custom_new_disable.h"
 
 #if defined(CS_MEMORY_TRACKER)
 #include "csutil/memdebug.h"
@@ -42,18 +40,6 @@
 /**\addtogroup util_containers
  * @{ */
 
-// Define CSARRAY_INHIBIT_TYPED_KEYS if the compiler is too old or too buggy to
-// properly support templated functions within a templated class.  When this is
-// defined, rather than using a properly typed "key" argument, search methods
-// fall back to dealing with opaque void* for the "key" argument.  Note,
-// however, that this fact is completely hidden from the client; the client
-// simply creates csArrayCmp<> functors using correct types for the keys
-// regardless of whether the compiler actually supports this feature.  (The
-// MSVC6 compiler, for example, does support templated functions within a
-// template class but crashes and burns horribly when a function pointer or
-// functor is thrown into the mix; thus this should be defined for MSVC6.)
-#if !defined(CSARRAY_INHIBIT_TYPED_KEYS)
-
 /**
  * A functor template which encapsulates a key and a comparison function for
  * use with key-related csArray<> searching methods, such as FindKey() and
@@ -110,45 +96,6 @@ private:
   CF cmp;
 };
 
-#define csArrayTemplate(K) template <class K>
-#define csArrayCmpDecl(T1,T2) csArrayCmp<T1,T2>
-#define csArrayCmpInvoke(C,R) C(R)
-
-#else // CSARRAY_INHIBIT_TYPED_KEYS
-
-class csArrayCmpAbstract
-{
-public:
-  typedef int(*CF)(void const*, void const*);
-  virtual int operator()(void const*) const = 0;
-  virtual operator CF() const = 0;
-};
-
-template <class T, class K>
-class csArrayCmp : public csArrayCmpAbstract
-{
-public:
-  typedef int(*CFTyped)(T const&, K const&);
-  csArrayCmp(K const& k, CFTyped c = DefaultCompare) : key(k), cmp(CF(c)) {}
-  csArrayCmp(csArrayCmp const& o) : key(o.key), cmp(o.cmp) {}
-  csArrayCmp& operator=(csArrayCmp const& o)
-    { key = o.key; cmp = o.cmp; return *this; }
-  virtual int operator()(void const* p) const { return cmp(p, &key); }
-  virtual operator CF() const { return cmp; }
-  operator K const&() const { return key; }
-  static int DefaultCompare(T const& r, K const& k)
-    { return csComparator<T,K>::Compare(r,k); }
-private:
-  K key;
-  CF cmp;
-};
-
-#define csArrayTemplate(K)
-#define csArrayCmpDecl(T1,T2) csArrayCmpAbstract const&
-#define csArrayCmpInvoke(C,R) C(&(R))
-
-#endif // CSARRAY_INHIBIT_TYPED_KEYS
-
 /**
  * The default element handler for csArray.
  */
@@ -156,14 +103,67 @@ template <class T>
 class csArrayElementHandler
 {
 public:
+  /// Construct an element
   static void Construct (T* address)
   {
-    new (CS_STATIC_CAST(void*,address)) T();
+    new (static_cast<void*> (address)) T();
+  }
+
+  /// Copy-construct an element
+  static void Construct (T* address, T const& src)
+  {
+    new (static_cast<void*> (address)) T(src);
+  }
+
+  /// Destroy an element
+  static void Destroy (T* address)
+  {
+    address->~T();
+  }
+
+  /// Construct a number of elements
+  static void InitRegion (T* address, size_t count)
+  {
+    for (size_t i = 0 ; i < count ; i++)
+      Construct (address + i);
+  }
+  
+  /// Reallocates a region allocated by \p Allocator.
+  template<typename Allocator>
+  static T* ResizeRegion (Allocator& alloc, T* mem, size_t relevantcount, 
+    size_t oldcount, size_t newcount)
+  {
+    (void)relevantcount; (void)oldcount;
+    return (T*)alloc.Realloc (mem, newcount * sizeof(T));
+  }
+
+  /// Move elements inside a region.
+  static void MoveElements (T* mem, size_t dest, size_t src, size_t count)
+  {
+    memmove (mem + dest, mem + src, count * sizeof(T));
+  }
+};
+
+/**
+ * Special element handler for csArray that makes sure that when
+ * the array is reallocated that the objects are properly constructed
+ * and destructed at their new position. This is needed for objects
+ * that can not be safely moved around in memory (like weak references).
+ * This is of course slower and that is the reason that this is not
+ * done by default.
+ */
+template <class T>
+class csArraySafeCopyElementHandler
+{
+public:
+  static void Construct (T* address)
+  {
+    new (static_cast<void*> (address)) T();
   }
 
   static void Construct (T* address, T const& src)
   {
-    new (CS_STATIC_CAST(void*,address)) T(src);
+    new (static_cast<void*> (address)) T(src);
   }
 
   static void Destroy (T* address)
@@ -176,94 +176,49 @@ public:
     for (size_t i = 0 ; i < count ; i++)
       Construct (address + i);
   }
-};
-
-/**
- * The default allocator for csArray. Uses malloc/free/realloc.
- */
-template <class T>
-class csArrayMemoryAllocator
-{
-public:
-  static T* Alloc (size_t count)
-  {
-    return (T*)malloc (count * sizeof(T));
-  }
-
-  static void Free (T* mem)
-  {
-    free (mem);
-  }
-
-  // The 'relevantcount' parameter should be the number of items
-  // in the old array that are initialized.
-  static T* Realloc (T* mem, size_t relevantcount, size_t oldcount,
-    size_t newcount)
-  {
-    (void)relevantcount; (void)oldcount;
-    return (T*)realloc (mem, newcount * sizeof(T));
-  }
-
-  // Move memory.
-  static void MemMove (T* mem, size_t dest, size_t src, size_t count)
-  {
-    memmove (mem + dest, mem + src, count * sizeof(T));
-  }
-};
-
-/**
- * Special allocator for csArray that makes sure that when
- * the array is reallocated that the objects are properly constructed
- * and destructed at their new position. This is needed for objects
- * that can not be safely moved around in memory (like weak references).
- * This is of course slower and that is the reason that this is not
- * done by default.
- */
-template <class T, class ElementHandler = csArrayElementHandler<T> >
-class csSafeCopyArrayMemoryAllocator
-{
-public:
-  static T* Alloc (size_t count)
-  {
-    return (T*)malloc (count * sizeof(T));
-  }
-
-  static void Free (T* mem)
-  {
-    free (mem);
-  }
-
-  static T* Realloc (T* mem, size_t relevantcount, size_t oldcount,
-    size_t newcount)
+  
+  /**
+   * Reallocates a region allocated by \p Allocator. Ensure that all elements
+   * are properly moved, ie they are copy-constructed at the new position 
+   * in memory, the old instance is then destroyed.
+   */
+  template<typename Allocator>
+  static T* ResizeRegion (Allocator& alloc, T* mem, size_t relevantcount, 
+    size_t oldcount, size_t newcount)
   {
     if (newcount <= oldcount)
     {
       // Realloc is safe.
-      T* newmem = (T*)realloc (mem, newcount * sizeof (T));
+      T* newmem = (T*)alloc.Realloc (mem, newcount * sizeof (T));
       CS_ASSERT (newmem == mem);
       return newmem;
     }
 
-    T* newmem = Alloc (newcount);
+    T* newmem = (T*)alloc.Alloc (newcount * sizeof (T));
     size_t i;
     for (i = 0 ; i < relevantcount ; i++)
     {
-      ElementHandler::Construct (newmem + i, mem[i]);
-      ElementHandler::Destroy (mem + i);
+      Construct (newmem + i, mem[i]);
+      Destroy (mem + i);
     }
-    Free (mem);
+    alloc.Free (mem);
     return newmem;
   }
 
-  static void MemMove (T* mem, size_t dest, size_t src, size_t count)
+  /**
+   * Move elements inside a region. Ensure that all elements
+   * are properly moved, ie they are copy-constructed at the new position 
+   * in memory, the old instance is then destroyed.
+   */
+  static void MoveElements (T* mem, size_t dest, size_t src, size_t count)
   {
     size_t i;
     if (dest < src)
     {
       for (i = 0 ; i < count ; i++)
       {
-        ElementHandler::Construct (mem + dest + i, mem[src + i]);
-	ElementHandler::Destroy (mem + src + i);
+        Construct (mem + dest + i, mem[src + i]);
+	Destroy (mem + src + i);
       }
     }
     else
@@ -272,13 +227,109 @@ public:
       while (i > 0)
       {
 	i--;
-        ElementHandler::Construct (mem + dest + i, mem[src + i]);
-	ElementHandler::Destroy (mem + src + i);
+        Construct (mem + dest + i, mem[src + i]);
+	Destroy (mem + src + i);
       }
     }
   }
 };
 
+
+/**
+ * csArray variable threshold for capacity handlers.
+ * The threshold is variable at run-time object construction.
+ */
+class csArrayThresholdVariable
+{
+  size_t threshold;
+public:
+  /// Construct with given threshold
+  csArrayThresholdVariable (size_t in_threshold = 0)
+  { threshold = (in_threshold > 0 ? in_threshold : 16); }
+  /// Return the threshold.
+  size_t GetThreshold() const { return threshold; }
+};
+
+/**
+ * csArray fixed threshold for capacity handlers.
+ * The threshold is specified at compile time, hence fixed at runtime.
+ */
+template<int N>
+class csArrayThresholdFixed
+{
+public:
+  /// Construct. \a x is ignored.
+  csArrayThresholdFixed (size_t x = 0)
+  { (void)x; }
+  /// Return the threshold.
+  size_t GetThreshold() const { return N; }
+  // Work around VC7 bug apparently incorrectly copying this empty class
+  csArrayThresholdFixed& operator= (const csArrayThresholdFixed&) 
+  { return *this; }
+};
+
+/**
+ * csArray capacity handler.
+ * Different capacity handlers allow to realize different csArray internal
+ * growth behaviours, if needed. This default "linear" handler will
+ * result in array capacity growth to multiples of \a Threshold and works
+ * well enough in most cases.
+ */
+template<typename Threshold = csArrayThresholdVariable>
+class csArrayCapacityLinear : public Threshold
+{
+public:
+  //@{
+  /// Construct capacity handler
+  csArrayCapacityLinear () : Threshold () {}
+  csArrayCapacityLinear (const Threshold& threshold) : Threshold (threshold)
+  {}
+  //@}
+  /**
+   * Construct capacity handler with a given size_t parameter for the 
+   * \a Threshold object. The exact meaning of \a x depends on the \a Threshold
+   * implementation.
+   * \remarks Mostly for compatibility with existing code.
+   */
+  csArrayCapacityLinear (const size_t x) : Threshold (x)
+  {}
+
+  /**
+   * Returns "true" if the given capacity is too large for the given count,
+   * that is, if GetCapacity() would return a value for \a count smaller than 
+   * \a capacity.
+   */
+  bool IsCapacityExcessive (size_t capacity, size_t count) const
+  {
+    return (capacity > this->GetThreshold() 
+      && count < capacity - this->GetThreshold());
+  }
+  /**
+   * Compute the capacity for a given number of items. The capacity will 
+   * commonly be larger than the actual \a count to reduce the number of
+   * memory allocations when pushing items onto an array.
+   */
+  size_t GetCapacity (size_t count) const
+  {
+    return ((count + this->GetThreshold() - 1) / this->GetThreshold()) * 
+      this->GetThreshold();
+  }
+};
+
+// Alias for csArrayCapacityLinear<csArrayThresholdVariable> to keep
+// SWIG generated Java classes (and thus filenames) short enough for Windows.
+// Note that a typedef wont work because SWIG would expand it.
+struct csArrayCapacityDefault :
+  public csArrayCapacityLinear<csArrayThresholdVariable>
+{
+  csArrayCapacityDefault () :
+    csArrayCapacityLinear<csArrayThresholdVariable> () {}
+  csArrayCapacityDefault (const csArrayThresholdVariable& threshold) :
+    csArrayCapacityLinear<csArrayThresholdVariable> (threshold) {}
+  csArrayCapacityDefault (const size_t x) :
+    csArrayCapacityLinear<csArrayThresholdVariable> (x) {}
+} ;
+
 /**
  * This value is returned whenever an array item could not be located or does
  * not exist.
@@ -295,37 +346,38 @@ const size_t csArrayItemNotFound = (size_t)-1;
  */
 template <class T,
 	class ElementHandler = csArrayElementHandler<T>,
-	class MemoryAllocator = csArrayMemoryAllocator<T> >
-class csArray
+        class MemoryAllocator = CS::Memory::AllocatorMalloc,
+        class CapacityHandler = csArrayCapacityDefault>
+class csArray : public CS::Memory::CustomAllocated
 {
+public:
+  typedef csArray<T, ElementHandler, MemoryAllocator, CapacityHandler> ThisType;
+  typedef T ValueType;
+  typedef ElementHandler ElementHandlerType;
+  typedef MemoryAllocator AllocatorType;
+  typedef CapacityHandler CapacityHandlerType;
+
 private:
   size_t count;
-  size_t capacity;
-  size_t threshold;
-  T* root;
-#ifdef CS_MEMORY_TRACKER
-  csMemTrackerInfo* mti;
-  void UpdateMti (int dn, int curcapacity)
+  /**
+   * Class to eliminate overhead from possibly empty CapacityHandler.
+   * See http://www.cantrip.org/emptyopt.html for details.
+   */
+  struct ArrayCapacity : public CapacityHandler
   {
-    if (!mti)
+    size_t c;
+    ArrayCapacity (size_t in_capacity)
+    { c = (in_capacity > 0 ? in_capacity : 0); }
+    ArrayCapacity (size_t in_capacity, const CapacityHandler& ch) : 
+      CapacityHandler (ch) 
+    { c = (in_capacity > 0 ? in_capacity : 0); }
+    void CopyFrom (const CapacityHandler& source)
     {
-      if (!curcapacity) return;
-      char buf[1024];
-      strcpy (buf, "csArray<");
-      size_t l = strlen (typeid (T).name ());
-      if (l > 1000) l = 1000;
-      strncat (buf, typeid (T).name (), l);
-      strcat (buf, ">");
-      mti = mtiRegisterAlloc (1 * sizeof (T), buf);
-      if (!mti) return;
-      curcapacity--;
-      if (curcapacity)
-        mtiUpdateAmount (mti, curcapacity, curcapacity * sizeof (T));
-      return;
+      CapacityHandler::operator= (source);
     }
-    mtiUpdateAmount (mti, dn, dn * sizeof (T));
-  }
-#endif
+  };
+  ArrayCapacity capacity;
+  CS::Memory::AllocatorPointerWrapper<T, MemoryAllocator> root;
 
 protected:
   /**
@@ -334,41 +386,36 @@ protected:
    */
   void InitRegion (size_t start, size_t count)
   {
-    ElementHandler::InitRegion (root+start, count);
+    ElementHandler::InitRegion (root.p+start, count);
   }
-
+  
+  /**
+   * Set the internal pointer to the data.
+   * \warning This is \em obviously dangerous.
+   */
+  void SetData (T* data) { root.p = data; }
 private:
   /// Copy from one array to this one, properly constructing the copied items.
   void CopyFrom (const csArray& source)
   {
-    if (&source != this)
-    {
-      DeleteAll ();
-      threshold = source.threshold;
-      SetSizeUnsafe (source.Length ());
-      for (size_t i=0 ; i<source.Length() ; i++)
-        ElementHandler::Construct (root + i, source[i]);
-    }
+    capacity.CopyFrom (source.capacity);
+    SetSizeUnsafe (source.GetSize ());
+    for (size_t i=0 ; i<source.GetSize () ; i++)
+      ElementHandler::Construct (root.p + i, source[i]);
   }
 
   /// Set the capacity of the array precisely to \c n elements.
   void InternalSetCapacity (size_t n)
   {
-    if (root == 0)
+    if (root.p == 0)
     {
-      root = MemoryAllocator::Alloc (n);
-#ifdef CS_MEMORY_TRACKER
-      UpdateMti (n, n);
-#endif
+      root.p = (T*)root.Alloc (n * sizeof (T));
     }
     else
     {
-      root = MemoryAllocator::Realloc (root, count, capacity, n);
-#ifdef CS_MEMORY_TRACKER
-      UpdateMti (n-capacity, n);
-#endif
+      root.p = ElementHandler::ResizeRegion (root, root.p, count, capacity.c, n);
     }
-    capacity = n;
+    capacity.c = n;
   }
 
   /**
@@ -377,9 +424,9 @@ private:
    */
   void AdjustCapacity (size_t n)
   {
-    if (n > capacity || (capacity > threshold && n < capacity - threshold))
+    if (n > capacity.c || capacity.IsCapacityExcessive (capacity.c, n))
     {
-      InternalSetCapacity (((n + threshold - 1) / threshold ) * threshold);
+      InternalSetCapacity (capacity.GetCapacity (n));
     }
   }
 
@@ -391,7 +438,7 @@ private:
    */
   void SetSizeUnsafe (size_t n)
   {
-    if (n > capacity)
+    if (n > capacity.c)
       AdjustCapacity (n);
     count = n;
   }
@@ -414,28 +461,46 @@ public:
   }
 
   /**
-   * Initialize object to have initial capacity of \c in_capacity elements, and
-   * to increase storage by \c in_threshold each time the upper bound is
-   * exceeded.
+   * Initialize object to have initial capacity of \c in_capacity elements.
+   * The storage increase depends on the specified capacity handler. The
+   * default capacity handler accepts a threshold parameter by which the 
+   * storage is increased each time the upper bound is exceeded.
    */
-  csArray (size_t in_capacity = 0, size_t in_threshold = 0)
+  csArray (size_t in_capacity = 0,
+    const CapacityHandler& ch = CapacityHandler()) : count (0), 
+    capacity (in_capacity, ch)
   {
 #ifdef CS_MEMORY_TRACKER
-    mti = 0;
+    root.SetMemTrackerInfo (typeid(*this).name());
 #endif
-    count = 0;
-    capacity = (in_capacity > 0 ? in_capacity : 0);
-    threshold = (in_threshold > 0 ? in_threshold : 16);
-    if (capacity != 0)
+    if (capacity.c != 0)
     {
-      root = MemoryAllocator::Alloc (capacity);
-#ifdef CS_MEMORY_TRACKER
-      UpdateMti (capacity, capacity);
-#endif
+      root.p = (T*)root.Alloc (capacity.c * sizeof (T));
     }
     else
     {
-      root = 0;
+      root.p = 0;
+    }
+  }
+  /**
+   * Initialize object to have initial capacity of \c in_capacity elements
+   * and with specific memory allocator and capacity handler initializations.
+   */
+  csArray (size_t in_capacity, 
+    const MemoryAllocator& alloc,
+    const CapacityHandler& ch) : count (0), 
+    capacity (in_capacity, ch), root (alloc)
+  {
+#ifdef CS_MEMORY_TRACKER
+    root.SetMemTrackerInfo (typeid(*this).name());
+#endif
+    if (capacity.c != 0)
+    {
+      root.p = (T*)root.Alloc (capacity.c * sizeof (T));
+    }
+    else
+    {
+      root.p = 0;
     }
   }
 
@@ -446,21 +511,23 @@ public:
   }
 
   /// Copy constructor.
-  csArray (const csArray& source)
+  csArray (const csArray& source) : count (0), capacity (0), root (0)
   {
 #ifdef CS_MEMORY_TRACKER
-    mti = 0;
+    root.SetMemTrackerInfo (typeid(*this).name());
 #endif
-    root = 0;
-    capacity = 0;
-    count = 0;
     CopyFrom (source);
   }
 
   /// Assignment operator.
-  csArray<T,ElementHandler>& operator= (const csArray& other)
+  csArray<T,ElementHandler,MemoryAllocator,CapacityHandler>& operator= (
+    const csArray& other)
   {
-    CopyFrom (other);
+    if (&other != this)
+    {
+      DeleteAll ();
+      CopyFrom (other);
+    }
     return *this;
   }
 
@@ -470,19 +537,10 @@ public:
     return count;
   }
 
-  /**
-   * Return the number of elements in the array.
-   * \deprecated Use GetSize() instead.
-   */
-  size_t Length () const
-  {
-    return GetSize();
-  }
-
   /// Query vector capacity.  Note that you should rarely need to do this.
   size_t Capacity () const
   {
-    return capacity;
+    return capacity.c;
   }
 
   /**
@@ -491,21 +549,17 @@ public:
    * other array will have all items that originally were in this array.
    * This operation is very efficient.
    */
+  // @@@ FIXME: What about custom allocators?
   void TransferTo (csArray& destination)
   {
     if (&destination != this)
     {
       destination.DeleteAll ();
-      destination.root = root;
+      destination.root.p = root.p;
       destination.count = count;
       destination.capacity = capacity;
-      destination.threshold = threshold;
-#ifdef CS_MEMORY_TRACKER
-      destination.mti = mti;
-      mti = 0;
-#endif
-      root = 0;
-      capacity = count = 0;
+      root.p = 0;
+      capacity.c = count = 0;
     }
   }
 
@@ -526,10 +580,10 @@ public:
     }
     else
     {
-      size_t old_len = Length ();
+      size_t old_len = GetSize ();
       SetSizeUnsafe (n);
       for (size_t i = old_len ; i < n ; i++)
-        ElementHandler::Construct (root + i, what);
+        ElementHandler::Construct (root.p + i, what);
     }
   }
 
@@ -548,33 +602,25 @@ public:
     }
     else
     {
-      size_t old_len = Length ();
+      size_t old_len = GetSize ();
       SetSizeUnsafe (n);
-      ElementHandler::InitRegion (root + old_len, n-old_len);
+      ElementHandler::InitRegion (root.p + old_len, n-old_len);
     }
   }
 
-  /** @{ */
-  /**
-   * Set the actual number of items in this array.
-   * \deprecated Use SetSize() instead.
-   */
-  void SetLength (size_t n, T const& what) { SetSize(n, what); }
-  void SetLength (size_t n) { SetSize(n); }
-  /** @} */
 
   /// Get an element (non-const).
   T& Get (size_t n)
   {
     CS_ASSERT (n < count);
-    return root[n];
+    return root.p[n];
   }
 
   /// Get an element (const).
   T const& Get (size_t n) const
   {
     CS_ASSERT (n < count);
-    return root[n];
+    return root.p[n];
   }
 
   /**
@@ -586,7 +632,19 @@ public:
   {
     if (n >= count)
       SetSize (n+1);
-    return root[n];
+    return root.p[n];
+  }
+
+  /**
+   * Get an item from the array. If the number of elements in this array is too
+   * small the array will be automatically extended, and the newly added
+   * objects will be constructed based on the given item \a what.
+   */
+  T& GetExtend (size_t n, T const& what)
+  {
+    if (n >= count)
+      SetSize (n+1, what);
+    return root.p[n];
   }
 
   /// Get an element (non-const).
@@ -606,8 +664,8 @@ public:
   {
     if (n >= count)
       SetSize (n+1);
-    ElementHandler::Destroy (root + n);
-    ElementHandler::Construct (root + n, what);
+    ElementHandler::Destroy (root.p + n);
+    ElementHandler::Construct (root.p + n, what);
   }
 
   /**
@@ -617,11 +675,11 @@ public:
    * type.
    * \return csArrayItemNotFound if not found, else item index.
    */
-  csArrayTemplate(K)
-  size_t FindKey (csArrayCmpDecl(T,K) comparekey) const
+  template <class K>
+  size_t FindKey (csArrayCmp<T,K> comparekey) const
   {
-    for (size_t i = 0 ; i < Length () ; i++)
-      if (csArrayCmpInvoke(comparekey, root[i]) == 0)
+    for (size_t i = 0 ; i < GetSize () ; i++)
+      if (comparekey (root.p[i]) == 0)
         return i;
     return csArrayItemNotFound;
   }
@@ -632,8 +690,8 @@ public:
    */
   size_t Push (T const& what)
   {
-    if (((&what >= root) && (&what < root + Length())) &&
-      (capacity < count + 1))
+    if (((&what >= root.p) && (&what < root.p + GetSize())) &&
+      (capacity.c < count + 1))
     {
       /*
         Special case: An element from this very array is pushed, and a
@@ -641,14 +699,14 @@ public:
 	element to be pushed to be read from deleted memory. Work
 	around this.
        */
-      size_t whatIndex = &what - root;
+      size_t whatIndex = &what - root.p;
       SetSizeUnsafe (count + 1);
-      ElementHandler::Construct (root + count - 1, root[whatIndex]);
+      ElementHandler::Construct (root.p + count - 1, root.p[whatIndex]);
     }
     else
     {
       SetSizeUnsafe (count + 1);
-      ElementHandler::Construct (root + count - 1, what);
+      ElementHandler::Construct (root.p + count - 1, what);
     }
     return count - 1;
   }
@@ -667,8 +725,8 @@ public:
   T Pop ()
   {
     CS_ASSERT (count > 0);
-    T ret(root [count - 1]);
-    ElementHandler::Destroy (root + count - 1);
+    T ret(root.p [count - 1]);
+    ElementHandler::Destroy (root.p + count - 1);
     SetSizeUnsafe (count - 1);
     return ret;
   }
@@ -677,14 +735,14 @@ public:
   T const& Top () const
   {
     CS_ASSERT (count > 0);
-    return root [count - 1];
+    return root.p [count - 1];
   }
 
   /// Return the top element but do not remove it. (non-const)
   T& Top ()
   {
     CS_ASSERT (count > 0);
-    return root [count - 1];
+    return root.p [count - 1];
   }
 
   /// Insert element \c item before element \c n.
@@ -695,8 +753,8 @@ public:
       SetSizeUnsafe (count + 1); // Increments 'count' as a side-effect.
       size_t const nmove = (count - n - 1);
       if (nmove > 0)
-	MemoryAllocator::MemMove (root, n+1, n, nmove);
-      ElementHandler::Construct (root + n, item);
+        ElementHandler::MoveElements (root.p, n+1, n, nmove);
+      ElementHandler::Construct (root.p + n, item);
       return true;
     }
     else
@@ -710,7 +768,7 @@ public:
   {
     CS_ASSERT (high < count && high >= low);
     csArray<T> sect (high - low + 1);
-    for (size_t i = low; i <= high; i++) sect.Push (root[i]);
+    for (size_t i = low; i <= high; i++) sect.Push (root.p[i]);
     return sect;
   }
 
@@ -719,15 +777,15 @@ public:
    * \return csArrayItemNotFound if not found, else the item index.
    * \remarks The array must be sorted.
    */
-  csArrayTemplate(K)
-  size_t FindSortedKey (csArrayCmpDecl(T,K) comparekey,
+  template <class K>
+  size_t FindSortedKey (csArrayCmp<T,K> comparekey,
                         size_t* candidate = 0) const
   {
-    size_t m = 0, l = 0, r = Length ();
+    size_t m = 0, l = 0, r = GetSize ();
     while (l < r)
     {
       m = (l + r) / 2;
-      int cmp = csArrayCmpInvoke(comparekey, root[m]);
+      int cmp = comparekey (root.p[m]);
       if (cmp == 0)
       {
         if (candidate) *candidate = csArrayItemNotFound;
@@ -738,6 +796,8 @@ public:
       else
         r = m;
     }
+    if ((m + 1) == r)
+      m++;
     if (candidate) *candidate = m;
     return csArrayItemNotFound;
   }
@@ -745,6 +805,10 @@ public:
   /**
    * Insert an element at a sorted position, using an element comparison
    * function.
+   * \param item The item to insert.
+   * \param compare [optional] Pointer to a function to compare two elements.
+   * \param equal_index [optional] Returns the index of an element equal to
+   *   the one to be inserted, if one is found. csArrayItemNotFound otherwise.
    * \return The index of the inserted item.
    * \remarks The array must be sorted.
    */
@@ -752,11 +816,11 @@ public:
     int (*compare)(T const&, T const&) = DefaultCompare,
     size_t* equal_index = 0)
   {
-    size_t m = 0, l = 0, r = Length ();
+    size_t m = 0, l = 0, r = GetSize ();
     while (l < r)
     {
       m = (l + r) / 2;
-      int cmp = compare (root [m], item);
+      int cmp = compare (root.p [m], item);
 
       if (cmp == 0)
       {
@@ -784,8 +848,8 @@ public:
    */
   size_t Find (T const& which) const
   {
-    for (size_t i = 0 ; i < Length () ; i++)
-      if (root[i] == which)
+    for (size_t i = 0 ; i < GetSize () ; i++)
+      if (root.p[i] == which)
         return i;
     return csArrayItemNotFound;
   }
@@ -802,9 +866,9 @@ public:
    */
   size_t GetIndex (const T* which) const
   {
-    CS_ASSERT (which >= root);
-    CS_ASSERT (which < (root + count));
-    return which-root;
+    CS_ASSERT (which >= root.p);
+    CS_ASSERT (which < (root.p + count));
+    return which-root.p;
   }
 
   /**
@@ -812,7 +876,7 @@ public:
    */
   void Sort (int (*compare)(T const&, T const&) = DefaultCompare)
   {
-    qsort (root, Length(), sizeof(T),
+    qsort (root.p, GetSize (), sizeof(T),
       (int (*)(void const*, void const*))compare);
   }
 
@@ -821,17 +885,14 @@ public:
    */
   void DeleteAll ()
   {
-    if (root)
+    if (root.p)
     {
       size_t i;
       for (i = 0 ; i < count ; i++)
-        ElementHandler::Destroy (root + i);
-      MemoryAllocator::Free (root);
-#     ifdef CS_MEMORY_TRACKER
-      UpdateMti (-capacity, 0);
-#     endif
-      root = 0;
-      capacity = count = 0;
+        ElementHandler::Destroy (root.p + i);
+      root.Free (root.p);
+      root.p = 0;
+      capacity.c = count = 0;
     }
   }
 
@@ -852,7 +913,7 @@ public:
     if (n < count)
     {
       for (size_t i = n; i < count; i++)
-        ElementHandler::Destroy (root + i);
+        ElementHandler::Destroy (root.p + i);
       SetSizeUnsafe(n);
     }
   }
@@ -885,7 +946,7 @@ public:
    */
   void SetCapacity (size_t n)
   {
-    if (n > Length ())
+    if (n > GetSize ())
       InternalSetCapacity (n);
   }
 
@@ -899,7 +960,7 @@ public:
   void SetMinimalCapacity (size_t n)
   {
     if (n < Capacity ()) return;
-    if (n > Length ())
+    if (n > GetSize ())
       InternalSetCapacity (n);
   }
 
@@ -914,13 +975,10 @@ public:
     {
       DeleteAll ();
     }
-    else if (count != capacity)
+    else if (count != capacity.c)
     {
-      root = MemoryAllocator::Realloc (root, count, capacity, count);
-#ifdef CS_MEMORY_TRACKER
-      UpdateMti (count-capacity, count);
-#endif
-      capacity = count;
+      root.p = ElementHandler::ResizeRegion (root, root.p, count, capacity.c, count);
+      capacity.c = count;
     }
   }
 
@@ -938,9 +996,9 @@ public:
     {
       size_t const ncount = count - 1;
       size_t const nmove = ncount - n;
-      ElementHandler::Destroy (root + n);
+      ElementHandler::Destroy (root.p + n);
       if (nmove > 0)
-	MemoryAllocator::MemMove (root, n, n+1, nmove);
+        ElementHandler::MoveElements (root.p, n, n+1, nmove);
       SetSizeUnsafe (ncount);
       return true;
     }
@@ -963,9 +1021,9 @@ public:
     {
       size_t const ncount = count - 1;
       size_t const nmove = ncount - n;
-      ElementHandler::Destroy (root + n);
+      ElementHandler::Destroy (root.p + n);
       if (nmove > 0)
-        MemoryAllocator::MemMove (root, n, ncount, 1);
+        ElementHandler::MoveElements (root.p, n, ncount, 1);
       SetSizeUnsafe (ncount);
       return true;
     }
@@ -976,25 +1034,27 @@ public:
   /**
    * Delete a given range (inclusive).
    * \remarks Will clamp \c start and \c end to the array limits.
+   * \return false in case the inputs were invalid (csArrayItemNotFound)
+   * or if the start is greater then the number of items in the array.
    */
-  void DeleteRange (size_t start, size_t end)
+  bool DeleteRange (size_t start, size_t end)
   {
-    if (start >= count) return;
+    if (start >= count) return false;
     // Treat 'csArrayItemNotFound' as invalid indices, do nothing.
-    // @@@ Assert that?
-    if (end == csArrayItemNotFound) return;
-    if (start == csArrayItemNotFound) return;//start = 0;
+    if (end == csArrayItemNotFound) return false;
+    if (start == csArrayItemNotFound) return false;//start = 0;
     if (end >= count) end = count - 1;
     size_t i;
     for (i = start ; i <= end ; i++)
-      ElementHandler::Destroy (root + i);
+      ElementHandler::Destroy (root.p + i);
 
     size_t const range_size = end - start + 1;
     size_t const ncount = count - range_size;
     size_t const nmove = count - end - 1;
     if (nmove > 0)
-      MemoryAllocator::MemMove (root, start, start + range_size, nmove);
+      ElementHandler::MoveElements (root.p, start, start + range_size, nmove);
     SetSizeUnsafe (ncount);
+    return true;
   }
 
   /**
@@ -1010,27 +1070,6 @@ public:
     return false;
   }
 
-  /**
-   * Delete the given element from the array.
-   * \remarks This is a special version of Delete() which does not
-   *   preserve the order of the remaining elements. This characteristic allows
-   *   deletions to be performed somewhat more quickly than by Delete(),
-   *   however the speed gain is largely mitigated by the fact that a linear
-   *   search is performed in order to locate \c item, thus this optimization
-   *   is mostly illusory.
-   * \deprecated The speed gain promised by this method is mostly illusory on
-   *   account of the linear search for the item. In many cases, it will be
-   *   faster to keep the array sorted, search for \c item using
-   *   FindSortedKey(), and then remove it using the plain DeleteIndex().
-   */
-  bool DeleteFast (T const& item)
-  {
-    size_t const n = Find (item);
-    if (n != csArrayItemNotFound)
-      return DeleteIndexFast (n);
-    return false;
-  }
-
   /** Iterator for the Array<> class */
   class Iterator
   {
@@ -1044,8 +1083,42 @@ public:
     { currentelem = r.currentelem; array = r.array; return *this; }
 
     /** Returns true if the next Next() call will return an element */
-    bool HasNext()
-    { return currentelem < array.Length(); }
+    bool HasNext() const
+    { return currentelem < array.GetSize (); }
+
+    /** Returns the next element in the array. */
+    T& Next()
+    { return array.Get(currentelem++); }
+
+    /** Reset the array to the first element */
+    void Reset()
+    { currentelem = 0; }
+
+  protected:
+    Iterator(csArray<T, ElementHandler, MemoryAllocator, CapacityHandler>& newarray)
+	: currentelem(0), array(newarray) {}
+    friend class csArray<T, ElementHandler, MemoryAllocator, CapacityHandler>;
+
+  private:
+    size_t currentelem;
+    csArray<T, ElementHandler, MemoryAllocator, CapacityHandler>& array;
+  };
+
+  /** Iterator for the Array<> class */
+  class ConstIterator
+  {
+  public:
+    /** Copy constructor. */
+    ConstIterator(ConstIterator const& r) :
+      currentelem(r.currentelem), array(r.array) {}
+
+    /** Assignment operator. */
+    ConstIterator& operator=(ConstIterator const& r)
+    { currentelem = r.currentelem; array = r.array; return *this; }
+
+    /** Returns true if the next Next() call will return an element */
+    bool HasNext() const
+    { return currentelem < array.GetSize (); }
 
     /** Returns the next element in the array. */
     const T& Next()
@@ -1056,18 +1129,22 @@ public:
     { currentelem = 0; }
 
   protected:
-    Iterator(const csArray<T, ElementHandler>& newarray)
-	: currentelem(0), array(newarray) {}
-    friend class csArray<T, ElementHandler>;
+    ConstIterator(const csArray<T, ElementHandler, MemoryAllocator, CapacityHandler>& newarray)
+      : currentelem(0), array(newarray) {}
+    friend class csArray<T, ElementHandler, MemoryAllocator, CapacityHandler>;
 
   private:
     size_t currentelem;
-    const csArray<T, ElementHandler>& array;
+    const csArray<T, ElementHandler, MemoryAllocator, CapacityHandler>& array;
   };
 
   /** Returns an Iterator which traverses the array. */
-  Iterator GetIterator() const
+  Iterator GetIterator()
   { return Iterator(*this); }
+
+  /** Returns an Iterator which traverses the array. */
+  ConstIterator GetIterator() const
+  { return ConstIterator(*this); }
   
   /// Check if this array has the exact same contents as \a other.
   bool operator== (const csArray& other) const
@@ -1077,6 +1154,36 @@ public:
       if (Get (i) != other[i]) return false;
     return true;
   }
+
+  bool operator!= (const csArray& other) const { return !(*this==other); }
+
+  /// Return a reference to the allocator of this array.
+  const MemoryAllocator& GetAllocator() const
+  {
+    return root;
+  }
+
+  /**
+  * Return the number of elements in the array.
+  * \deprecated Use GetSize() instead.
+  */
+  CS_DEPRECATED_METHOD_MSG("Use GetSize() instead.")
+  size_t Length () const
+  {
+    return GetSize();
+  }
+
+  /** @{ */
+  /**
+  * Set the actual number of items in this array.
+  * \deprecated Use SetSize() instead.
+  */
+  CS_DEPRECATED_METHOD_MSG("Use SetSize() instead.")
+  void SetLength (size_t n, T const& what) { SetSize(n, what); }
+  CS_DEPRECATED_METHOD_MSG("Use SetSize() instead.")
+  void SetLength (size_t n) { SetSize(n); }
+  /** @} */
+
 };
 
 /**
@@ -1087,8 +1194,7 @@ public:
 template <class T>
 class csSafeCopyArray
 	: public csArray<T,
-		csArrayElementHandler<T>,
-		csSafeCopyArrayMemoryAllocator<T> >
+		csArraySafeCopyElementHandler<T> >
 {
 public:
   /**
@@ -1096,15 +1202,12 @@ public:
    * storage by \c threshold each time the upper bound is exceeded.
    */
   csSafeCopyArray (size_t limit = 0, size_t threshold = 0)
-  	: csArray<T, csArrayElementHandler<T>,
-		     csSafeCopyArrayMemoryAllocator<T> > (limit, threshold)
+  	: csArray<T, csArraySafeCopyElementHandler<T> > (limit, threshold)
   {
   }
 };
 
-#if defined(CS_EXTENSIVE_MEMDEBUG) || defined(CS_MEMORY_TRACKER)
-# define new CS_EXTENSIVE_MEMDEBUG_NEW
-#endif
+#include "csutil/custom_new_enable.h"
 
 /** @} */
 
