@@ -24,6 +24,7 @@
 
 #include "csextern.h"
 #include "iutil/event.h"
+#include "iutil/eventhandlers.h"
 
 /**\file
  * Event system related helpers
@@ -62,7 +63,7 @@ public:
    * keyboard modifiers struct.
    * \sa CSMASK_ALT etc.
    */
-  CS_CONST_METHOD static uint32 GetModifiersBits (
+  static uint32 GetModifiersBits (
     const csKeyModifiers& modifiers);
   /**
    * Get a bitmask corresponding to the pressed modifier keys from the event.
@@ -139,7 +140,7 @@ public:
     return csKeyEventHelper::GetModifiersBits(modifiers); 
   }
   /// Retrieve event data
-  CS_PURE_METHOD static bool GetEventData (const iEvent* event, 
+  static bool GetEventData (const iEvent* event, 
     csMouseEventData& data);
 };
 
@@ -165,12 +166,6 @@ public:
 
   /// Retrieve joystick number (0, 1, 2, ...)
   CS_PURE_METHOD static uint GetNumber(const iEvent *event);
-  /// Retrieve axis 0 value (X)
-  CS_PURE_METHOD CS_DEPRECATED_METHOD static int GetX(const iEvent *event) 
-  { return csJoystickEventHelper::GetAxis(event, 0); }
-  /// Retrieve axis 1 value (Y)
-  CS_PURE_METHOD CS_DEPRECATED_METHOD static int GetY(const iEvent *event) 
-  { return csJoystickEventHelper::GetAxis(event, 1); }
   /// retrieve any axis (basis 0) value
   CS_PURE_METHOD static int GetAxis(const iEvent *event, uint);
   /// retrieve number of axes
@@ -236,7 +231,34 @@ public:
 };
 
 /** @} */
- 
+
+struct iEventQueue;
+struct iObjectRegistry;
+
+namespace CS
+{
+
+/**
+ * Helper function for registering an event handler using a weak reference.
+ * Use RemoveWeakListener() to remove an event handler registered with this
+ * function.
+ */
+csHandlerID CS_CRYSTALSPACE_EXPORT RegisterWeakListener (iEventQueue *q, 
+  iEventHandler *listener, csRef<iEventHandler> &handler);
+csHandlerID CS_CRYSTALSPACE_EXPORT RegisterWeakListener (iEventQueue *q, 
+  iEventHandler *listener, const csEventID &ename, csRef<iEventHandler> &handler);
+csHandlerID CS_CRYSTALSPACE_EXPORT RegisterWeakListener (iEventQueue *q, 
+  iEventHandler *listener, const csEventID ename[], csRef<iEventHandler> &handler);
+
+/**
+ * Helper function for removing an event handler that was registered with
+ * RegisterWeakListener().
+ */
+void CS_CRYSTALSPACE_EXPORT RemoveWeakListener (iEventQueue *q, 
+  csRef<iEventHandler> &handler);
+
+} // namespace CS
+  
 /** @} */
 
 #endif // __CS_CSUTIL_EVENT_H__
