@@ -41,16 +41,6 @@
 #define CS_FORMATTER_PROVIDE_I64
 #endif
 
-// MSVC 7.1 requires us to use a `typename' qualifier on the declaration of
-// `mantissa' in csPrintfFormatter. Although this is accepted by most other
-// compilers, it breaks on gcc 3.4.x, which complains (apparently incorrectly?)
-// that IEEEFloatMantissa is not a templated type.
-#if defined(CS_COMPILER_MSVC) && (_MSC_VER >= 1300)
-#define CS_FORMATTER_TYPENAME_QUALIFIER typename_qualifier
-#else
-#define CS_FORMATTER_TYPENAME_QUALIFIER
-#endif
-
 /**\addtogroup util
  * @{ */
 
@@ -76,7 +66,7 @@
 /**
  * Standard format string source for csPrintfFormatter.
  * \remarks Uses csUnicodeTransform::Decode(), so it supports
- *  utd8_char, utf16_char, utf32_char and wchar_t strings.
+ *  utf8_char, utf16_char, utf32_char and wchar_t strings.
  */
 template <class T>
 class csFmtDefaultReader
@@ -108,7 +98,7 @@ public:
 /**
  * Standard formatted string destination for csPrintfFormatter.
  * \remarks Uses csUnicodeTransform::Decode(), so it supports
- *  utd8_char, utf16_char, utf32_char and wchar_t strings.
+ *  utf8_char, utf16_char, utf32_char and wchar_t strings.
  */
 template <class T>
 class csFmtDefaultWriter
@@ -149,7 +139,7 @@ class csPrintfFormatter
   public:
     void WriteTo (Twriter& writer, size_t offset = 0, size_t len = (size_t)~0)
     {
-      const size_t n = MIN (len, Length());
+      const size_t n = MIN (len, GetSize ());
       for (size_t i = offset; i < n; i++) writer.Put (Get (i));
     }
   };
@@ -527,7 +517,7 @@ class csPrintfFormatter
 	  // Param delimiter
 	  else if (state.ch == '$')
 	  {
-	    // @@@ Hmm. Empty param...
+	    // \todo fix for empty param
 	    parseState = formatFlagsWidthPrecTypeConversion;
 	    break;
 	  }
@@ -630,17 +620,17 @@ class csPrintfFormatter
     size_t i;
     // Determine order of params
     csArray<FormatSpec*> paramOrder;
-    paramOrder.SetCapacity (formatSpecs.Length());
-    for (i = 0; i < formatSpecs.Length(); i++)
+    paramOrder.SetCapacity (formatSpecs.GetSize ());
+    for (i = 0; i < formatSpecs.GetSize (); i++)
     {
       FormatSpec& currentFormat = formatSpecs[i];
       if (currentFormat.conversion == convNone) continue;
-      if (paramOrder.Length() <= (size_t)currentFormat.paramIdx)
-	paramOrder.SetLength (currentFormat.paramIdx + 1, 0);
+      if (paramOrder.GetSize () <= (size_t)currentFormat.paramIdx)
+	paramOrder.SetSize (currentFormat.paramIdx + 1, 0);
       paramOrder[currentFormat.paramIdx] = &currentFormat;
     }
     // Fetch params from stack in order, store at correct place in params array
-    for (i = 0; i < paramOrder.Length(); i++)
+    for (i = 0; i < paramOrder.GetSize (); i++)
     {
       FmtParam& param = params.GetExtend (i);
       FormatSpec* fmtPtr = paramOrder[i];
@@ -695,10 +685,16 @@ class csPrintfFormatter
 		param.vSzT = va_arg (args, size_t);
 		break;
 	      case typeShort:
-		param.vInt = (short)(va_arg (args, int));
+		if (currentFormat.conversion == convInt)
+		  param.vInt = (short)(va_arg (args, int));
+		else
+		  param.vInt = (unsigned short)(va_arg (args, int));
 		break;
 	      case typeChar:
-		param.vInt = (char)(va_arg (args, int));
+		if (currentFormat.conversion == convInt)
+		  param.vInt = (char)(va_arg (args, int));
+		else
+		  param.vInt = (unsigned char)(va_arg (args, int));
 		break;
 	      default:
 		param.vInt = va_arg (args, int);
@@ -760,7 +756,6 @@ class csPrintfFormatter
       return;
     }
 
-    size_t scratchOffs = scratch.Length();
     size_t len = 0;
     {
       const T* ptr = stringPtr;
@@ -768,28 +763,55 @@ class csPrintfFormatter
     }
     if (currentFormat.precision > -1)
       len = MIN(len, (size_t)currentFormat.precision);
-    while (len > 0)
+
+    // How many utf32_chars were written
+    size_t writtenLen;
+    /* Check if we can circumvent using the scratch array:
+       we actually only need it when the string is right-justified
+       (and a width is given). */
+    bool fastTrack = currentFormat.leftJustify
+      || (currentFormat.width == 0);
+
+    if (fastTrack)
     {
-      utf32_char ch;
-      int n = csUnicodeTransform::Decode (stringPtr, len, ch);
-      scratch.Push (ch);
-      stringPtr += n;
-      len -= (size_t)n;
+      writtenLen = 0;
+      while (len > 0)
+      {
+        utf32_char ch;
+        int n = csUnicodeTransform::Decode (stringPtr, len, ch);
+        writer.Put (ch);
+        stringPtr += n;
+        len -= (size_t)n;
+        writtenLen++;
+      }
     }
-    if (!currentFormat.leftJustify 
-      && ((size_t)currentFormat.width > (scratch.Length() - scratchOffs)))
+    else
     {
-      size_t d = (size_t)currentFormat.width - scratch.Length() + scratchOffs;
-      while (d-- > 0) writer.Put (' ');
+      size_t scratchOffs = scratch.GetSize ();
+      while (len > 0)
+      {
+        utf32_char ch;
+        int n = csUnicodeTransform::Decode (stringPtr, len, ch);
+        scratch.Push (ch);
+        stringPtr += n;
+        len -= (size_t)n;
+      }
+      writtenLen = scratch.GetSize () - scratchOffs;
+      if (!currentFormat.leftJustify 
+        && ((size_t)currentFormat.width > writtenLen))
+      {
+        size_t d = (size_t)currentFormat.width - writtenLen;
+        while (d-- > 0) writer.Put (' ');
+      }
+      scratch.WriteTo (writer, scratchOffs);
+      scratch.Truncate (scratchOffs);
     }
-    scratch.WriteTo (writer, scratchOffs);
     if (currentFormat.leftJustify 
-      && ((size_t)currentFormat.width > (scratch.Length() - scratchOffs)))
+      && ((size_t)currentFormat.width > writtenLen))
     {
-      size_t d = (size_t)currentFormat.width - scratch.Length() + scratchOffs;
+      size_t d = (size_t)currentFormat.width - writtenLen;
       while (d-- > 0) writer.Put (' ');
     }
-    scratch.Truncate (scratchOffs);
   }
 
   /// Pad scratch buffer with spaces or zeros left resp. right
@@ -798,7 +820,7 @@ class csPrintfFormatter
   {
     if (currentFormat.leftJustify)
     {
-      while ((size_t)currentFormat.width > (scratch.Length() - scratchOffs))
+      while ((size_t)currentFormat.width > (scratch.GetSize () - scratchOffs))
       {
 	scratch.Push (' ');
       }
@@ -807,14 +829,14 @@ class csPrintfFormatter
     {
       if (currentFormat.padZero)
       {
-	while ((size_t)currentFormat.width > (scratch.Length() - scratchOffs))
+	while ((size_t)currentFormat.width > (scratch.GetSize () - scratchOffs))
 	{
 	  scratch.Insert (insert0offs, '0');
 	}
       }
       else
       {
-	while ((size_t)currentFormat.width > (scratch.Length() - scratchOffs))
+	while ((size_t)currentFormat.width > (scratch.GetSize () - scratchOffs))
 	{
 	  scratch.Insert (scratchOffs, ' ');
 	}
@@ -826,7 +848,7 @@ class csPrintfFormatter
   template<class T>
   void OutputInt (Twriter& writer, const FormatSpec& currentFormat, T value)
   {
-    const size_t scratchOffs = scratch.Length();
+    const size_t scratchOffs = scratch.GetSize ();
     size_t insertOffs = scratchOffs;
 
     if (value < 0)
@@ -870,7 +892,7 @@ class csPrintfFormatter
     T value, uint radix = 10, const char* prefix = 0)
   {
     const utf32_char letterFirst = currentFormat.uppercase ? 'A' : 'a';
-    const size_t scratchOffs = scratch.Length();
+    const size_t scratchOffs = scratch.GetSize ();
     size_t insertOffs = scratchOffs;
 
     if (prefix != 0)
@@ -988,8 +1010,8 @@ class csPrintfFormatter
     bool sign;
     Tbase exp;
 
-    CS_FORMATTER_TYPENAME_QUALIFIER
-    csPrintfFormatter<Twriter,Treader>::IEEEFloatMantissa<T, Tbase> mantissa;
+    typename csPrintfFormatter<Twriter,Treader>::
+      template IEEEFloatMantissa<T, Tbase> mantissa;
 
     IEEEFloatSplitter (const T& val, const int mantissaBits,
       const int expBits) 
@@ -1086,7 +1108,7 @@ class csPrintfFormatter
       return;
     }
 
-    const size_t scratchOffs = scratch.Length();
+    const size_t scratchOffs = scratch.GetSize ();
     if (vSplit.sign)
     {
       scratch.Push ('-');
@@ -1144,7 +1166,7 @@ class csPrintfFormatter
     }
     else
       scratch.Push ('+');
-    const size_t insertOffs = scratch.Length();;
+    const size_t insertOffs = scratch.GetSize ();;
     do
     {
       uint d = e % 10;
@@ -1185,7 +1207,7 @@ public:
     reader.Reset();
     size_t i = 0;
     utf32_char ch;
-    while (i < formatSpecs.Length())
+    while (i < formatSpecs.GetSize ())
     {
       const FormatSpec& currentFormat = formatSpecs[i];
       size_t n;
