@@ -26,7 +26,8 @@
 #include "csextern.h"
 #include "csutil/array.h"
 #include "csutil/comparator.h"
-
+#include "csutil/util.h"
+#include "csutil/tuple.h"
 
 /**\addtogroup util_containers
  * @{ */
@@ -81,31 +82,31 @@ public:
 /**
  * csHashComputer<> specialization for an integral type.
  */
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<void*> : public csHashComputerIntegral<void*> {};
   
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<int> : public csHashComputerIntegral<int> {}; 
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<unsigned int> : 
   public csHashComputerIntegral<unsigned int> {}; 
     
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<long> : public csHashComputerIntegral<long> {}; 
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<unsigned long> : 
   public csHashComputerIntegral<unsigned long> {}; 
 
 #if (CS_LONG_SIZE < 8)    
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<longlong> : 
   public csHashComputerIntegral<longlong> {}; 
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<ulonglong> : 
   public csHashComputerIntegral<ulonglong> {}; 
 #endif
     
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<float>
 {
 public:
@@ -121,7 +122,7 @@ public:
     return float2uint.u;
   }
 };
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<double>
 {
 public:
@@ -162,12 +163,35 @@ public:
   { ptr = other.ptr; return *this; }
 };
 
+/**
+ * A helper template to use const pointers as keys for hashes.
+ */
+template <typename T>
+class csConstPtrKey
+{
+  const T* ptr;
+public:
+  csConstPtrKey () : ptr(0) {}
+  csConstPtrKey (const T* ptr) : ptr(ptr) {}
+  csConstPtrKey (csConstPtrKey const& other) : ptr (other.ptr) {}
+
+  uint GetHash () const { return (uintptr_t)ptr; }
+  inline friend bool operator < (const csConstPtrKey& r1, const csConstPtrKey& r2)
+  { return r1.ptr < r2.ptr; }
+  operator const T* () const
+  { return ptr; }
+  const T* operator -> () const
+  { return ptr; }
+  csConstPtrKey& operator = (csConstPtrKey const& other)
+  { ptr = other.ptr; return *this; }
+};
+
 /**
  * Template that can be used as a base class for hash computers for 
  * string types (must support cast to const char*).
  * Example:
  * \code
- * CS_SPECIALIZE_TEMPLATE csHashComputer<MyString> : 
+ * template<> class csHashComputer<MyString> : 
  *   public csHashComputerString<MyString> {};
  * \endcode
  */
@@ -184,7 +208,7 @@ public:
 /**
  * csHashComputer<> specialization for strings that uses csHashCompute().
  */
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<const char*> : public csHashComputerString<const char*> {};
 
 /**
@@ -192,7 +216,7 @@ class csHashComputer<const char*> : public csHashComputerString<const char*> {};
  * structs.
  * Example:
  * \code
- * CS_SPECIALIZE_TEMPLATE csHashComputer<MyStruct> : 
+ * template<> class csHashComputer<MyStruct> : 
  *   public csHashComputerStruct<MyStruct> {};
  * \endcode
  */
@@ -206,35 +230,6 @@ public:
   }
 };
 
-/**
- * This is a simple helper class to make a copy of a const char*.
- * This can be used to have a hash that makes copies of the keys.
- * \deprecated csString can also be used for hash keys.
- */
-class csStrKey
-{
-private:
-  char* str;
-
-public:
-  csStrKey () { str = 0; }
-  csStrKey (const char* s) { str = csStrNew (s); }
-  csStrKey (const csStrKey& c) { str = csStrNew (c.str); }
-  ~csStrKey () { delete[] str; }
-  csStrKey& operator=(const csStrKey& o)
-  {
-    delete[] str; str = csStrNew (o.str);
-    return *this;
-  }
-  operator const char* () const { return str; }
-  uint GetHash() const { return csHashCompute (str); }
-};
-
-/**
- * csComparator<> specialization for csStrKey that uses strcmp().
- */
-CS_SPECIALIZE_TEMPLATE
-class csComparator<csStrKey, csStrKey> : public csComparatorString<csStrKey> {};
 
 /**
  * A generic hash table class,
@@ -242,12 +237,19 @@ class csComparator<csStrKey, csStrKey> : public csComparatorString<csStrKey> {};
  * The hash value of a key is computed using csHashComputer<>, two keys are
  * compared using csComparator<>. You need to provide appropriate 
  * specializations of those templates if you want use non-integral types 
- * (other than const char*, csStrKey and csString for which appropriate 
- * specializations are already provided) or special hash algorithms. 
+ * (other than const char* and csString for which appropriate specializations
+ * are already provided) or special hash algorithms. 
  */
-template <class T, class K = unsigned int> 
+template <class T, class K = unsigned int, 
+  class ArrayMemoryAlloc = CS::Memory::AllocatorMalloc> 
 class csHash
 {
+public:
+  typedef csHash<T, K, ArrayMemoryAlloc> ThisType;
+  typedef T ValueType;
+  typedef K KeyType;
+  typedef ArrayMemoryAlloc AllocatorType;
+
 protected:
   struct Element
   {
@@ -257,7 +259,10 @@ protected:
     Element (const K& key0, const T &value0) : key (key0), value (value0) {}
     Element (const Element &other) : key (other.key), value (other.value) {}
   };
-  csArray< csArray<Element> > Elements;
+  typedef csArray<Element, csArrayElementHandler<Element>,
+    ArrayMemoryAlloc> ElementArray;
+  csArray<ElementArray, csArrayElementHandler<ElementArray>,
+    ArrayMemoryAlloc> Elements;
 
   size_t Modulo;
 
@@ -280,21 +285,21 @@ private:
     };
 
     const size_t *p;
-    size_t elen = Elements.Length ();
+    size_t elen = Elements.GetSize ();
     for (p = Primes; *p && *p <= elen; p++) ;
     Modulo = *p;
     CS_ASSERT (Modulo);
 
-    Elements.SetLength(Modulo, csArray<Element>(0, MIN(Modulo / GrowRate, 8)));
+    Elements.SetSize (Modulo, ElementArray (0, MIN(Modulo / GrowRate, 4)));
 
     for (size_t i = 0; i < elen; i++)
     {
-      csArray<Element>& src = Elements[i];
-      size_t slen = src.Length ();
+      ElementArray& src = Elements[i];
+      size_t slen = src.GetSize ();
       for (size_t j = slen; j > 0; j--)
       {
         const Element& srcElem = src[j - 1];
-        csArray<Element>& dst = 
+        ElementArray& dst = 
 	  Elements.Get (csHashComputer<K>::ComputeHash (srcElem.key) % Modulo);
         if (&src != &dst)
         {
@@ -309,22 +314,21 @@ public:
   /**
    * Construct a hash table with an array of the given size,
    * which for optimisation reasons should be a prime number.
-   * <p>
-   * Grow_rate is the rate at which the hash table grows:
-   * Size doubles once there are size/grow_rate collisions.
-   * It will not grow after it reaches max_size.
-   * <p>
+   * 
+   * \a grow_rate is the rate at which the hash table grows:
+   * \a size doubles once there are \a size/\a grow_rate collisions.
+   * It will not grow after it reaches \a max_size.
+   * 
    * Here are a few primes: 7, 11, 19, 29, 59, 79, 101, 127, 151, 199, 251,
    * 307, 401, 503, 809, 1009, 1499, 2003, 3001, 5003, 12263, 25247, 36923,
    * 50119, 70951, 90313, 104707.
-   * <p>
+   * 
    * For a bigger list go to http://www.utm.edu/research/primes/
    */
   csHash (size_t size = 23, size_t grow_rate = 5, size_t max_size = 20000)
-    : Elements (size), Modulo (size), InitModulo (size),
+    : Modulo (size), InitModulo (size),
       GrowRate (MIN (grow_rate, size)), MaxSize (max_size), Size (0)
   {
-    Elements.SetLength (size, csArray<Element> (0, MIN (size / GrowRate, 8)));
   }
 
   /// Copy constructor.
@@ -341,21 +345,31 @@ public:
    */
   void Put (const K& key, const T &value)
   {
-    csArray<Element> &values = 
+    if (Elements.GetSize() == 0) Elements.SetSize (Modulo);
+    ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
     values.Push (Element (key, value));
     Size++;
-    if (values.Length () > Elements.Length () / GrowRate
-     && Elements.Length () < MaxSize) Grow ();
+    if (values.GetSize () > Elements.GetSize () / GrowRate
+     && Elements.GetSize () < MaxSize) Grow ();
   }
 
   /// Get all the elements with the given key, or empty if there are none.
   csArray<T> GetAll (const K& key) const
   {
-    const csArray<Element> &values = 
+    return GetAll<typename csArray<T>::ElementHandlerType, 
+      typename csArray<T>::AllocatorType> (key);
+  }
+
+  /// Get all the elements with the given key, or empty if there are none.
+  template<typename H, typename M>
+  csArray<T, H, M> GetAll (const K& key) const
+  {
+    if (Elements.GetSize() == 0) return csArray<T> ();
+    const ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    csArray<T> ret (values.Length () / 2);
-    const size_t len = values.Length ();
+    csArray<T> ret (values.GetSize () / 2);
+    const size_t len = values.GetSize ();
     for (size_t i = 0; i < len; ++i)
     {
       const Element& v = values[i];
@@ -368,9 +382,10 @@ public:
   /// Add an element to the hash table, overwriting if the key already exists.
   void PutUnique (const K& key, const T &value)
   {
-    csArray<Element> &values = 
+    if (Elements.GetSize() == 0) Elements.SetSize (Modulo);
+    ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    const size_t len = values.Length ();
+    const size_t len = values.GetSize ();
     for (size_t i = 0; i < len; ++i)
     {
       Element& v = values[i];
@@ -383,25 +398,17 @@ public:
 
     values.Push (Element (key, value));
     Size++;
-    if (values.Length () > Elements.Length () / GrowRate
-     && Elements.Length () < MaxSize) Grow ();
-  }
-
-  /**
-   * Add an element to the hash table, overwriting if the key already exists.
-   * \deprecated Use PutUnique() instead.
-   */
-  CS_DEPRECATED_METHOD void PutFirst (const K& key, const T &value)
-  {
-    PutUnique(key, value);
+    if (values.GetSize () > Elements.GetSize () / GrowRate
+     && Elements.GetSize () < MaxSize) Grow ();
   }
 
   /// Returns whether at least one element matches the given key.
   bool Contains (const K& key) const
   {
-    const csArray<Element> &values = 
+    if (Elements.GetSize() == 0) return false;
+    const ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    const size_t len = values.Length ();
+    const size_t len = values.GetSize ();
     for (size_t i = 0; i < len; ++i)
       if (csComparator<K, K>::Compare (values[i].key, key) == 0) 
 	return true;
@@ -422,9 +429,10 @@ public:
    */
   const T* GetElementPointer (const K& key) const
   {
-    const csArray<Element> &values = 
+    if (Elements.GetSize() == 0) return 0;
+    const ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    const size_t len = values.Length ();
+    const size_t len = values.GetSize ();
     for (size_t i = 0; i < len; ++i)
     {
       const Element& v = values[i];
@@ -441,9 +449,10 @@ public:
    */
   T* GetElementPointer (const K& key)
   {
-    csArray<Element> &values = 
+    if (Elements.GetSize() == 0) return 0;
+    ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    const size_t len = values.Length ();
+    const size_t len = values.GetSize ();
     for (size_t i = 0; i < len; ++i)
     {
       Element& v = values[i];
@@ -454,15 +463,24 @@ public:
     return 0;
   }
 
+  /**
+   * h["key"] shorthand notation for h.GetElementPoint ("key")
+   */
+  T* operator[] (const K& key)
+  {
+    return GetElementPointer (key);
+  }
+
   /**
    * Get the first element matching the given key, or \p fallback if there is 
    * none.
    */
   const T& Get (const K& key, const T& fallback) const
   {
-    const csArray<Element> &values = 
+    if (Elements.GetSize() == 0) return fallback;
+    const ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    const size_t len = values.Length ();
+    const size_t len = values.GetSize ();
     for (size_t i = 0; i < len; ++i)
     {
       const Element& v = values[i];
@@ -479,9 +497,10 @@ public:
    */
   T& Get (const K& key, T& fallback)
   {
-    csArray<Element> &values = 
+    if (Elements.GetSize() == 0) return fallback;
+    ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    const size_t len = values.Length ();
+    const size_t len = values.GetSize ();
     for (size_t i = 0; i < len; ++i)
     {
       Element& v = values[i];
@@ -495,10 +514,8 @@ public:
   /// Delete all the elements.
   void DeleteAll ()
   {
-    Elements.SetLength (Modulo = InitModulo);
-    size_t elen = Elements.Length ();
-    for (size_t i = 0; i < elen; i++)
-      Elements[i].Empty ();
+    Elements.DeleteAll();
+    Modulo = InitModulo;
     Size = 0;
   }
 
@@ -509,9 +526,10 @@ public:
   bool DeleteAll (const K& key)
   {
     bool ret = false;
-    csArray<Element> &values = 
+    if (Elements.GetSize() == 0) return ret;
+    ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    for (size_t i = values.Length (); i > 0; i--)
+    for (size_t i = values.GetSize (); i > 0; i--)
     {
       const size_t idx = i - 1;
       if (csComparator<K, K>::Compare (values[idx].key, key) == 0)
@@ -528,13 +546,14 @@ public:
   bool Delete (const K& key, const T &value)
   {
     bool ret = false;
-    csArray<Element> &values = 
+    if (Elements.GetSize() == 0) return ret;
+    ElementArray& values = 
       Elements[csHashComputer<K>::ComputeHash (key) % Modulo];
-    for (size_t i = values.Length (); i > 0; i--)
+    for (size_t i = values.GetSize (); i > 0; i--)
     {
       const size_t idx = i - 1;
       if ((csComparator<K, K>::Compare (values[idx].key, key) == 0) && 
-	(values[idx].value == value))
+	  (csComparator<T, T>::Compare (values[idx].value, value) == 0 ))
       {
         values.DeleteIndexFast (idx);
         ret = true;
@@ -564,7 +583,7 @@ public:
   class Iterator
   {
   private:
-    const csHash<T, K>* hash;
+    csHash<T, K>* hash;
     const K key;
     size_t bucket, size, element;
 
@@ -577,11 +596,11 @@ public:
     }
 
   protected:
-    Iterator (const csHash<T, K>* hash0, const K& key0) :
+    Iterator (csHash<T, K>* hash0, const K& key0) :
       hash(hash0),
       key(key0), 
       bucket(csHashComputer<K>::ComputeHash (key) % hash->Modulo),
-      size(hash->Elements[bucket].Length ())
+      size((hash->Elements.GetSize() > 0) ? hash->Elements[bucket].GetSize () : 0)
       { Reset (); }
 
     friend class csHash<T, K>;
@@ -612,9 +631,9 @@ public:
     }
 
     /// Get the next element's value.
-    const T& Next ()
+    T& Next ()
     {
-      const T &ret = hash->Elements[bucket][element].value;
+      T &ret = hash->Elements[bucket][element].value;
       element++;
       Seek ();
       return ret;
@@ -629,17 +648,21 @@ public:
   class GlobalIterator
   {
   private:
-    const csHash<T, K> *hash;
+    csHash<T, K> *hash;
     size_t bucket, size, element;
 
     void Zero () { bucket = element = 0; }
-    void Init () { size = hash->Elements[bucket].Length (); }
+    void Init () 
+    { 
+      size = 
+        (hash->Elements.GetSize() > 0) ? hash->Elements[bucket].GetSize () : 0;
+    }
 
     void FindItem ()
     {
       if (element >= size)
       {
-	while (++bucket < hash->Elements.Length ())
+	while (++bucket < hash->Elements.GetSize ())
 	{
           Init ();
 	  if (size != 0)
@@ -652,7 +675,7 @@ public:
     }
 
   protected:
-    GlobalIterator (const csHash<T, K> *hash0) : hash (hash0) 
+    GlobalIterator (csHash<T, K> *hash0) : hash (hash0) 
     { 
       Zero (); 
       Init (); 
@@ -662,7 +685,7 @@ public:
     friend class csHash<T, K>;
   public:
     /// Copy constructor.
-    GlobalIterator (const Iterator &o) :
+    GlobalIterator (const GlobalIterator &o) :
       hash (o.hash),
       bucket (o.bucket),
       size (o.size),
@@ -681,8 +704,186 @@ public:
     /// Returns a boolean indicating whether or not the hash has more elements.
     bool HasNext () const
     {
-      if (hash->Elements.Length () == 0) return false;
-      return element < size || bucket < hash->Elements.Length ();
+      if (hash->Elements.GetSize () == 0) return false;
+      return element < size || bucket < hash->Elements.GetSize ();
+    }
+
+    /// Advance the iterator of one step
+    void Advance ()
+    {
+      element++;
+      FindItem ();
+    }
+
+    /// Get the next element's value, don't move the iterator.
+    T& NextNoAdvance ()
+    {
+      return hash->Elements[bucket][element].value;
+    }
+
+    /// Get the next element's value.
+    T& Next ()
+    {
+      T &ret = NextNoAdvance ();
+      Advance ();
+      return ret;
+    }
+
+    /// Get the next element's value and key, don't move the iterator.
+    T& NextNoAdvance (K &key)
+    {
+      key = hash->Elements[bucket][element].key;
+      return NextNoAdvance ();
+    }
+
+    /// Get the next element's value and key.
+    T& Next (K &key)
+    {
+      key = hash->Elements[bucket][element].key;
+      return Next ();
+    }
+
+    /// Return a tuple of the value and key.
+    const csTuple2<T, K> NextTuple ()
+    {
+      csTuple2<T, K> t (NextNoAdvance (),
+          hash->Elements[bucket][element].key);
+      Advance ();
+      return t;
+    }
+
+    /// Move the iterator back to the first element.
+    void Reset () { Zero (); Init (); FindItem (); }
+  };
+  friend class GlobalIterator;
+
+  /// An const iterator class for the hash.
+  class ConstIterator
+  {
+  private:
+    const csHash<T, K>* hash;
+    const K key;
+    size_t bucket, size, element;
+
+    void Seek ()
+    {
+      while ((element < size) && 
+        (csComparator<K, K>::Compare (hash->Elements[bucket][element].key, 
+	key) != 0))
+          element++;
+    }
+
+  protected:
+    ConstIterator (const csHash<T, K>* hash0, const K& key0) :
+      hash(hash0),
+      key(key0), 
+      bucket(csHashComputer<K>::ComputeHash (key) % hash->Modulo),
+      size((hash->Elements.GetSize() > 0) ? hash->Elements[bucket].GetSize () : 0)
+      { Reset (); }
+
+    friend class csHash<T, K>;
+  public:
+    /// Copy constructor.
+    ConstIterator (const ConstIterator &o) :
+      hash (o.hash),
+      key(o.key),
+      bucket(o.bucket),
+      size(o.size),
+      element(o.element) {}
+
+    /// Assignment operator.
+    ConstIterator& operator=(const ConstIterator& o)
+    {
+      hash = o.hash;
+      key = o.key;
+      bucket = o.bucket;
+      size = o.size;
+      element = o.element;
+      return *this;
+    }
+
+    /// Returns a boolean indicating whether or not the hash has more elements.
+    bool HasNext () const
+    {
+      return element < size;
+    }
+
+    /// Get the next element's value.
+    const T& Next ()
+    {
+      const T &ret = hash->Elements[bucket][element].value;
+      element++;
+      Seek ();
+      return ret;
+    }
+
+    /// Move the iterator back to the first element.
+    void Reset () { element = 0; Seek (); }
+  };
+  friend class ConstIterator;
+
+  /// An const iterator class for the hash.
+  class ConstGlobalIterator
+  {
+  private:
+    const csHash<T, K> *hash;
+    size_t bucket, size, element;
+
+    void Zero () { bucket = element = 0; }
+    void Init () 
+    { 
+      size = 
+        (hash->Elements.GetSize() > 0) ? hash->Elements[bucket].GetSize () : 0;
+    }
+
+    void FindItem ()
+    {
+      if (element >= size)
+      {
+	while (++bucket < hash->Elements.GetSize ())
+	{
+          Init ();
+	  if (size != 0)
+	  {
+	    element = 0;
+	    break;
+	  }
+	}
+      }
+    }
+
+  protected:
+    ConstGlobalIterator (const csHash<T, K> *hash0) : hash (hash0) 
+    { 
+      Zero (); 
+      Init (); 
+      FindItem ();
+    }
+
+    friend class csHash<T, K>;
+  public:
+    /// Copy constructor.
+    ConstGlobalIterator (const ConstGlobalIterator &o) :
+      hash (o.hash),
+      bucket (o.bucket),
+      size (o.size),
+      element (o.element) {}
+
+    /// Assignment operator.
+    ConstGlobalIterator& operator=(const ConstGlobalIterator& o)
+    {
+      hash = o.hash;
+      bucket = o.bucket;
+      size = o.size;
+      element = o.element;
+      return *this;
+    }
+
+    /// Returns a boolean indicating whether or not the hash has more elements.
+    bool HasNext () const
+    {
+      if (hash->Elements.GetSize () == 0) return false;
+      return element < size || bucket < hash->Elements.GetSize ();
     }
 
     /// Advance the iterator of one step
@@ -720,10 +921,19 @@ public:
       return Next ();
     }
 
+    /// Return a tuple of the value and key.
+    const csTuple2<T, K> NextTuple ()
+    {
+      csTuple2<T, K> t (NextNoAdvance (),
+          hash->Elements[bucket][element].key);
+      Advance ();
+      return t;
+    }
+
     /// Move the iterator back to the first element.
     void Reset () { Zero (); Init (); FindItem (); }
   };
-  friend class GlobalIterator;
+  friend class ConstGlobalIterator;
 
   /// Delete the element pointed by the iterator. This is safe for this
   /// iterator, not for the others.
@@ -735,13 +945,23 @@ public:
     iterator.FindItem ();
   }
 
+  /// Delete the element pointed by the iterator. This is safe for this
+  /// iterator, not for the others.
+  void DeleteElement (ConstGlobalIterator& iterator)
+  {
+    Elements[iterator.bucket].DeleteIndex(iterator.element);
+    Size--;
+    iterator.size--;
+    iterator.FindItem ();
+  }
+
   /**
    * Return an iterator for the hash, to iterate only over the elements
    * with the given key.
    * \warning Modifying the hash (except with DeleteElement()) while you have
    *   open iterators will result in undefined behaviour.
    */
-  Iterator GetIterator (const K& key) const
+  Iterator GetIterator (const K& key)
   {
     return Iterator (this, key);
   }
@@ -751,10 +971,31 @@ public:
    * \warning Modifying the hash (except with DeleteElement()) while you have
    *   open iterators will result in undefined behaviour.
    */
-  GlobalIterator GetIterator () const
+  GlobalIterator GetIterator ()
   {
     return GlobalIterator (this);
   }
+
+  /**
+   * Return a const iterator for the hash, to iterate only over the elements
+   * with the given key.
+   * \warning Modifying the hash (except with DeleteElement()) while you have
+   *   open iterators will result in undefined behaviour.
+   */
+  ConstIterator GetIterator (const K& key) const
+  {
+    return ConstIterator (this, key);
+  }
+
+  /**
+   * Return a const iterator for the hash, to iterate over all elements.
+   * \warning Modifying the hash (except with DeleteElement()) while you have
+   *   open iterators will result in undefined behaviour.
+   */
+  ConstGlobalIterator GetIterator () const
+  {
+    return ConstGlobalIterator (this);
+  }
 };
 
 /** @} */
