@@ -19,23 +19,26 @@
 #ifndef __CS_OBJREG_H__
 #define __CS_OBJREG_H__
 
-#include <stdarg.h>
-#include <stdio.h>
+/**\file
+ * Implementation of iObjectRegistry.
+ */
+
 #include "csextern.h"
 #include "iutil/objreg.h"
-#include "scf.h"
-#include "array.h"
-#include "stringarray.h"
-#include "scopedmutexlock.h"
+#include "csutil/array.h"
+#include "csutil/scf_implementation.h"
+#include "csutil/stringarray.h"
+#include "csutil/threading/mutex.h"
 
 /**
  * This is an implementation of iObjectRegistry.
  * Thread-safe!
  */
-class CS_CSUTIL_EXPORT csObjectRegistry : public iObjectRegistry
+class CS_CRYSTALSPACE_EXPORT csObjectRegistry : 
+  public scfImplementation1<csObjectRegistry, iObjectRegistry>
 {
 private:
-  csRef<csMutex> mutex;
+  CS::Threading::RecursiveMutex mutex;
   csArray<iBase*> registry;
   csStringArray tags;
   // True when this object is being cleared; prevents external changes.
@@ -46,7 +49,6 @@ public:
   /// Client must explicitly call Clear().
   virtual ~csObjectRegistry ();
 
-  SCF_DECLARE_IBASE;
   /**
    * Clear the object registry and release all references.
    */
