@@ -19,8 +19,12 @@
 #ifndef __CS_PRFXCFG_H__
 #define __CS_PRFXCFG_H__
 
+/**\file
+ * Implementation of iConfigFile prepending a string to all key names.
+ */
+
 #include "csextern.h"
-#include "cfgfile.h"
+#include "csutil/cfgfile.h"
 
 /**
  * This is an implementation of iConfigFile that stores an ID string
@@ -29,7 +33,7 @@
  * returned instead of the real file name. The real name will still be
  * used for loading and saving.
  */
-class CS_CSUTIL_EXPORT csPrefixConfig : public csConfigFile
+class CS_CRYSTALSPACE_EXPORT csPrefixConfig : public csConfigFile
 {
 public:
   /// constructor
