@@ -98,7 +98,7 @@ class csPathsList;
  */
 #define SCF_DECLARE_IBASE						\
   int scfRefCount;		/* Reference counter */			\
-  csArray<iBase**>* scfWeakRefOwners;					\
+  csArray<void**>* scfWeakRefOwners;					\
   void scfRemoveRefOwners ();						\
   SCF_DECLARE_EMBEDDED_IBASE (iBase)
 
@@ -112,8 +112,8 @@ public:									\
   virtual void IncRef ();						\
   virtual void DecRef ();						\
   virtual int GetRefCount ();						\
-  virtual void AddRefOwner (iBase** ref_owner);				\
-  virtual void RemoveRefOwner (iBase** ref_owner);			\
+  virtual void AddRefOwner (void** ref_owner);				\
+  virtual void RemoveRefOwner (void** ref_owner);			\
   virtual void *QueryInterface (scfInterfaceID iInterfaceID, int iVersion)
 
 /**
@@ -212,9 +212,9 @@ void Class::DecRef ()							\
 void Class::scfRemoveRefOwners ()					\
 {									\
   if (!scfWeakRefOwners) return;					\
-  for (size_t i = 0 ; i < scfWeakRefOwners->Length () ; i++)		\
+  for (size_t i = 0 ; i < scfWeakRefOwners->GetSize () ; i++)		\
   {									\
-    iBase** p = (*scfWeakRefOwners)[i];					\
+    void** p = (*scfWeakRefOwners)[i];					\
     *p = 0;								\
   }									\
   delete scfWeakRefOwners;						\
@@ -226,18 +226,18 @@ void Class::scfRemoveRefOwners ()					\
  * and RemoveRefOwner() for a weak reference.
  */
 #define SCF_IMPLEMENT_IBASE_REFOWNER(Class)				\
-void Class::AddRefOwner (iBase** ref_owner)				\
+void Class::AddRefOwner (void** ref_owner)				\
 {									\
   if (!scfWeakRefOwners)						\
-    scfWeakRefOwners = new csArray<iBase**> (0, 4);			\
+    scfWeakRefOwners = new csArray<void**> (0, 4);			\
   scfWeakRefOwners->InsertSorted (ref_owner);				\
 }									\
-void Class::RemoveRefOwner (iBase** ref_owner)				\
+void Class::RemoveRefOwner (void** ref_owner)				\
 {									\
   if (!scfWeakRefOwners)						\
     return;								\
   size_t index = scfWeakRefOwners->FindSortedKey (			\
-    csArrayCmp<iBase**, iBase**> (ref_owner)); 				\
+    csArrayCmp<void**, void**> (ref_owner)); 				\
   if (index != csArrayItemNotFound) scfWeakRefOwners->DeleteIndex (	\
     index); 								\
 }
@@ -338,11 +338,11 @@ int Class::GetRefCount ()						\
  * AddRefOwner() and RemoveRefOwner() for a weak reference.
  */
 #define SCF_IMPLEMENT_EMBEDDED_IBASE_REFOWNER(Class)			\
-void Class::AddRefOwner (iBase** ref_owner)				\
+void Class::AddRefOwner (void** ref_owner)				\
 {									\
   scfParent->AddRefOwner (ref_owner);					\
 }									\
-void Class::RemoveRefOwner (iBase** ref_owner)				\
+void Class::RemoveRefOwner (void** ref_owner)				\
 {									\
   scfParent->RemoveRefOwner (ref_owner);				\
 }
@@ -396,7 +396,7 @@ void *Class::QueryInterface (scfInterfaceID iInterfaceID, int iVersion)	\
  * return a pointer to that interface if everything is correct.
  */
 #define SCF_IMPLEMENTS_INTERFACE(Interface)				\
-  csRefTrackerAccess::AddAlias (CS_STATIC_CAST(Interface*, this), this);\
+  csRefTrackerAccess::AddAlias (static_cast<Interface*> (this), this);\
   SCF_IMPLEMENTS_INTERFACE_COMMON (Interface, this)
 
 /**
@@ -414,7 +414,7 @@ void *Class::QueryInterface (scfInterfaceID iInterfaceID, int iVersion)	\
     scfCompatibleVersion (iVersion, scfInterfaceTraits<Interface>::GetVersion())) \
   {									\
     (Object)->IncRef ();						\
-    return CS_STATIC_CAST(Interface*, Object);				\
+    return static_cast<Interface*> (Object);				\
   }
 
 /**
@@ -432,8 +432,8 @@ void *Class::QueryInterface (scfInterfaceID iInterfaceID, int iVersion)	\
   virtual void IncRef ();						\
   virtual void DecRef ();						\
   virtual int GetRefCount ();						\
-  virtual void AddRefOwner (iBase** ref_owner);				\
-  virtual void RemoveRefOwner (iBase** ref_owner);			\
+  virtual void AddRefOwner (void** ref_owner);				\
+  virtual void RemoveRefOwner (void** ref_owner);			\
   virtual void *QueryInterface (scfInterfaceID iInterfaceID, int iVersion)
 
 /**
@@ -477,11 +477,11 @@ int Class::GetRefCount ()						\
  * AddRefOwner() and RemoveRefOwner() for a weak reference.
  */
 #define SCF_IMPLEMENT_IBASE_EXT_REFOWNER(Class)			\
-void Class::AddRefOwner (iBase** ref_owner)				\
+void Class::AddRefOwner (void** ref_owner)				\
 {									\
   __scf_superclass::AddRefOwner (ref_owner);				\
 }									\
-void Class::RemoveRefOwner (iBase** ref_owner)				\
+void Class::RemoveRefOwner (void** ref_owner)				\
 {									\
   __scf_superclass::RemoveRefOwner (ref_owner);				\
 }
@@ -776,9 +776,17 @@ inline csPtr<Interface> scfQueryInterface (ClassPtr object)
   return csPtr<Interface> (x);
 }
 
-//Compatibility macro for scfQueryInterface
+template<class Interface, class ClassPtr>
+inline CS_DEPRECATED_METHOD_MSG ("SCF_QUERY_INTERFACE macro is deprecated")
+csPtr<Interface> SCF_QUERY_INTERFACE_is_deprecated (ClassPtr object)
+{
+  return scfQueryInterface<Interface> (object);
+}
+/**
+ * \deprecated Compatibility macro for scfQueryInterface
+ */
 #define SCF_QUERY_INTERFACE(Object,Interface) \
-  scfQueryInterface<Interface> (Object)
+  SCF_QUERY_INTERFACE_is_deprecated<Interface> (Object)
 
 /**
  * Helper function around iBase::QueryInterface which also 
@@ -794,9 +802,18 @@ inline csPtr<Interface> scfQueryInterfaceSafe (ClassPtr object)
     scfInterfaceTraits<Interface>::GetVersion ());
   return csPtr<Interface> (x);
 }
-//Compatibility macro for scfQueryInterfaceSafe
+
+template<class Interface, class ClassPtr>
+inline CS_DEPRECATED_METHOD_MSG ("SCF_QUERY_INTERFACE_SAFE macro is deprecated")
+csPtr<Interface> SCF_QUERY_INTERFACE_SAFE_is_deprecated (ClassPtr object)
+{
+  return scfQueryInterfaceSafe<Interface> (object);
+}
+/**
+ * \deprecated Compatibility macro for scfQueryInterfaceSafe
+ */
 #define SCF_QUERY_INTERFACE_SAFE(Object,Interface) \
-  scfQueryInterfaceSafe<Interface> (Object)
+  SCF_QUERY_INTERFACE_SAFE_is_deprecated<Interface> (Object)
 
 /**
  * Handy function to create an instance of a shared class.
@@ -816,9 +833,18 @@ inline csPtr<Interface> scfCreateInstance (char const * const ClassID)
   return csPtr<Interface> (x);
 }
 
-// Compatibility macro for scfCreateInstance function
+template<class Interface>
+inline CS_DEPRECATED_METHOD_MSG ("SCF_CREATE_INSTANCE macro is deprecated")
+csPtr<Interface> SCF_CREATE_INSTANCE_is_deprecated (
+  char const * const ClassID)
+{
+  return scfCreateInstance<Interface> (ClassID);
+}
+/**
+ * \deprecated Compatibility macro for scfCreateInstance function
+ */
 #define SCF_CREATE_INSTANCE(ClassID,Interface) \
-  scfCreateInstance<Interface> (ClassID)
+  SCF_CREATE_INSTANCE_is_deprecated<Interface> (ClassID)
 
 // Give versions to above declared classes.
 SCF_VERSION (iFactory, 0, 0, 2);
