@@ -27,6 +27,7 @@
 
 #include "csextern.h"
 
+
 // -- Forward declarations
 struct iDocument;
 #if defined(CS_DEBUG) || defined(CS_MEMORY_TRACKER)
@@ -105,12 +106,12 @@ static CS_FORCEINLINE bool scfCompatibleVersion (
  */
 struct iBase
 {
-  // Jorrit: removed the code below as it causes 'pure virtual' method
-  // calls to happen upon destruction.
-  //protected:
-  //// Needed for GCC4. Otherwise emits a flood of "virtual functions but
-  //// non-virtual destructor" warnings.
-  //virtual ~iBase() {}
+protected:
+  /**
+   * Needed for GCC4. Otherwise emits a flood of "virtual functions but
+   * non-virtual destructor" warnings.
+   */
+  virtual ~iBase() {}
 public:
   SCF_INTERFACE(iBase, 1, 0, 0);
   /// Increment the number of references to this object.
@@ -127,9 +128,9 @@ public:
    */
   virtual void *QueryInterface (scfInterfaceID iInterfaceID, int iVersion) = 0;
   /// For weak references: add a reference owner.
-  virtual void AddRefOwner (iBase** ref_owner) = 0;
+  virtual void AddRefOwner (void** ref_owner) = 0;
   /// For weak references: remove a reference owner.
-  virtual void RemoveRefOwner (iBase** ref_owner) = 0;
+  virtual void RemoveRefOwner (void** ref_owner) = 0;
 };
 
 /// Type of factory function which creates an instance of an SCF class.
@@ -141,9 +142,9 @@ typedef iBase* (*scfFactoryFunc)(iBase*);
  * should not use scfXXX functions directly; instead they should obtain a
  * pointer to an iSCF object and work through that pointer.
  */
-struct iSCF : public iBase
+struct iSCF : public virtual iBase
 {
-  SCF_INTERFACE(iSCF, 2, 0,0);
+  SCF_INTERFACE(iSCF, 3,0,0);
   /**
    * This is the global instance of iSCF.  On most platforms, this variable is
    * module-global; for instance, the application has an iSCF::SCF variable,
@@ -364,17 +365,17 @@ struct iSCF : public iBase
  * For old-style interfaces it is specialized through SCF_VERSION macro,
  * with new style interfaces no specialization is needed.
  */
-template <class Interface> 
+template <typename Interface> 
 class scfInterfaceTraits
 {
 public:
-  typedef typename_qualifier Interface::InterfaceTraits::InterfaceType 
+  typedef typename Interface::InterfaceTraits::InterfaceType 
     InterfaceType;
 
   /**
    * Retrieve the interface's current version number.
    */
-  CS_FORCEINLINE static scfInterfaceVersion GetVersion ()
+  CS_FORCEINLINE_TEMPLATEMETHOD static scfInterfaceVersion GetVersion ()
   {
     return Interface::InterfaceTraits::GetVersion ();
   }
@@ -386,7 +387,7 @@ public:
    * performance reasons, by scfInterfaceID, which is typically a small
    * integer.
    */
-  CS_FORCEINLINE static scfInterfaceID GetID ()
+  CS_FORCEINLINE_TEMPLATEMETHOD static scfInterfaceID GetID ()
   {
     scfInterfaceID& ID = GetMyID ();
     if (ID == (scfInterfaceID)(-1))
@@ -400,14 +401,14 @@ public:
   /**
    * Retrieve the interface's name as a string.
    */
-  CS_FORCEINLINE static char const* GetName ()
+  CS_FORCEINLINE_TEMPLATEMETHOD static char const* GetName ()
   { 
     return Interface::InterfaceTraits::GetName ();
   }
 
 private:
   // This idiom is a Meyers singleton
-  CS_FORCEINLINE static scfInterfaceID& GetMyID ()
+  CS_FORCEINLINE_TEMPLATEMETHOD static scfInterfaceID& GetMyID ()
   {
     static scfInterfaceID ID = (scfInterfaceID)-1;
     return ID;
@@ -433,7 +434,7 @@ private:
  */
 #define SCF_VERSION(Name,Major,Minor,Micro)                \
 struct Name;                                               \
-CS_SPECIALIZE_TEMPLATE                                     \
+template<>                                                 \
 class scfInterfaceTraits<Name>                             \
 {                                                          \
 public:                                                    \
