@@ -28,8 +28,9 @@
 #include "csextern.h"
 #include "csutil/array.h"
 #include "csutil/util.h"
+#include "csutil/csstring.h"
 
-class csStringArrayElementHandler
+class csStringArrayElementHandler : public csArrayElementHandler<const char*>
 {
 public:
   static void Construct (const char** address, const char* const& src)
@@ -107,7 +108,7 @@ public:
    * \return csArrayItemNotFound if not found, else item index.
    * \remarks The array must be sorted.
    */
-  size_t FindSortedKey (csArrayCmpDecl(char const*, char const*) comparekey,
+  size_t FindSortedKey (csArrayCmp<char const*, char const*> comparekey,
     size_t* candidate = 0) const
   {
     return superclass::FindSortedKey(comparekey, candidate);
@@ -197,6 +198,72 @@ public:
   {
     return case_sensitive ? Find(str) : FindCaseInsensitive(str);
   }
+
+  /**
+   * Mode how SplitString() treats consecutive occurance of delimiters.
+   */
+  enum ConsecutiveDelimiterMode
+  {
+    /// Split at each different delimiter.
+    delimSplitEach,
+    /**
+     * Ignore consecutive delimiters (except the first). In other words,
+     * any number of consecutive delimiters is treated like there was only
+     * one delimiter.
+     */
+    delimIgnore,
+    /**
+     * Only ignore consecutive delimiters if they're different.
+     */
+    delimIgnoreDifferent
+  };
+  /**
+   * Add a number of strings to this array by splitting \a str at characters
+   * from \a delimiters.
+   */
+  size_t SplitString (const char* str, const char* delimiters, 
+    ConsecutiveDelimiterMode delimMode = delimSplitEach)
+  {
+    size_t num = 0;
+    csString currentString;
+    int lastDelim = -1;
+
+    const char* p = str;
+    while (*p != 0)
+    {
+      if (strchr (delimiters, *p))
+      {
+        bool newString = true;
+        switch (delimMode)
+        {
+          case delimSplitEach:
+            break;
+          case delimIgnore:
+            newString = lastDelim == -1;
+            break;
+          case delimIgnoreDifferent:
+            newString = (lastDelim == -1) || (lastDelim == *p);
+            break;
+        }
+        if (newString)
+        {
+          Push (currentString);
+          currentString.Empty();
+          num++;
+          lastDelim = *p;
+        }
+      }
+      else
+      {
+        currentString.Append (*p);
+        lastDelim = -1;
+      }
+      p++;
+    }
+
+    Push (currentString);
+    return num + 1;
+  }
 };
 
 #endif // __CS_STRINGARRAY_H__
