@@ -108,7 +108,8 @@ public:
    * Remove all stored strings.
    * \deprecated Use Empty() instead.
    */
-  /*CS_DEPRECATED_METHOD*/ void Clear ()
+  /*CS_DEPRECATED_METHOD("Use Empty() instead.")*/
+  void Clear ()
   { Empty(); }
 
   /// Get the number of elements in the hash.
