@@ -36,17 +36,28 @@ struct iBase;
  * all weak references pointing to that object are cleared. This kind of
  * reference is useful if you want to maintain some kind of cached objects
  * that can safely be removed as soon as the last reference to it is gone.
- * <p>
+ * 
  * Note: this class assumes that the T type implements at least the following
  * functions:
  * - AddRefOwner()
  * - RemoveRefOwner()
+ *
+ * \remarks An extended explanation on smart pointers - how they work and what
+ *   type to use in what scenario - is contained in the User's manual, 
+ *   section "Correctly Using Smart Pointers".
  */
 template <class T>
 class csWeakRef
 {
 private:
-  T* obj;
+  union
+  {
+    T* obj;
+    void* obj_void;
+  };
+#if defined(CS_DEBUG)
+  void* this_saved;
+#endif
 
   /**
    * Unlink the object pointed to by this weak reference so that
@@ -55,7 +66,7 @@ private:
    */
   void Unlink ()
   {
-    if (obj) obj->RemoveRefOwner ((iBase**)&obj);
+    if (obj) obj->RemoveRefOwner (&obj_void);
   }
 
   /**
@@ -63,20 +74,28 @@ private:
    */
   void Link ()
   {
-    if (obj) obj->AddRefOwner ((iBase**)&obj);
+    if (obj) obj->AddRefOwner (&obj_void);
   }
 
 public:
   /**
    * Construct an empty weak reference.
    */
-  csWeakRef () : obj (0) {}
+  csWeakRef () : obj (0) 
+  {
+#if defined(CS_DEBUG)
+    this_saved = this;
+#endif
+  }
 
   /**
    * Construct a weak reference from a normal pointer.
    */
   csWeakRef (T* newobj)
   {
+#if defined(CS_DEBUG)
+    this_saved = this;
+#endif
     obj = newobj;
     Link ();
   }
@@ -86,6 +105,9 @@ public:
    */
   csWeakRef (csRef<T> const& newobj)
   {
+#if defined(CS_DEBUG)
+    this_saved = this;
+#endif
     obj = newobj;
     Link ();
   }
@@ -95,6 +117,9 @@ public:
    */
   csWeakRef (csWeakRef const& other) : obj (other.obj)
   {
+#if defined(CS_DEBUG)
+    this_saved = this;
+#endif
     Link ();
   }
 
@@ -104,6 +129,9 @@ public:
    */
   csWeakRef (const csPtr<T>& newobj)
   {
+#if defined(CS_DEBUG)
+    this_saved = this;
+#endif
     csRef<T> r = newobj;
     obj = r;
     Link ();
@@ -114,6 +142,10 @@ public:
    */
   ~csWeakRef ()
   {
+#if defined(CS_DEBUG)
+    CS_ASSERT_MSG ("A csWeakRef<> was memcpy()ed, which is not allowed",
+      this_saved == this);
+#endif
     Unlink ();
   }
 
