@@ -43,8 +43,6 @@ struct iCamera;
 struct iCameraPosition;
 struct iCameraPositionList;
 struct iClipper2D;
-struct iCollection;
-struct iCollectionList;
 struct iDataBuffer;
 struct iFrustumView;
 struct iLight;
@@ -124,6 +122,9 @@ enum csRenderPrioritySorting
 /**
  * A callback that will be fired whenever the engine starts drawing
  * a frame.
+ *
+ * This callback is used by:
+ * - iEngine
  */
 struct iEngineFrameCallback : public virtual iBase
 {
@@ -137,6 +138,9 @@ struct iEngineFrameCallback : public virtual iBase
 /**
  * A callback that will be fired whenever a sector is created or
  * removed from the engine.
+ *
+ * This callback is used by:
+ * - iEngine
  */
 struct iEngineSectorCallback : public virtual iBase
 {
@@ -162,14 +166,14 @@ struct iEngineSectorCallback : public virtual iBase
  * - 3D Engine plugin (crystalspace.engine.3d)
  *
  * Main ways to get pointers to this interface:
- * - CS_QUERY_REGISTRY()
+ * - csQueryRegistry<iEngine> ()
  *
  * Main users of this interface:
  * - Application.
  */
 struct iEngine : public virtual iBase
 {
-  SCF_INTERFACE(iEngine,2,0,0);
+  SCF_INTERFACE(iEngine,2,1,0);
   
   /// Get the iObject for the engine.
   virtual iObject *QueryObject() = 0;
@@ -178,13 +182,15 @@ struct iEngine : public virtual iBase
    * @{ */
   
   /**
-   * Prepare the engine. This function must be called after
-   * you loaded/created the world. It will prepare all lightmaps
-   * for use and also free all images that were loaded for
-   * the texture manager (the texture manager should have them
-   * locally now). The optional progress meter will be used to
-   * report progress.
-   * <p>
+   * Prepare the engine. This function must be called after you loaded/created 
+   * the world and before it is rendered.
+   *
+   * Actions include:
+   * - Preparation of all lightmaps for use
+   * - Registering of textures with texture manager and freeing of loaded
+   *   images (as the texture manager will have created textures from them)
+   * The optional progress meter will be used to report progress.
+   *
    * The behaviour regarding cached lighting depends on the flag
    * you can set with the SetLightingCacheMode() function. The default
    * behaviour is to read the lightmap cache when present but don't
@@ -342,9 +348,20 @@ struct iEngine : public virtual iBase
    *   camera viewpoint).
    * - #CS_RENDPRI_SORT_BACK2FRONT: sort objects back to front.
    *
-   * \note The default render priorities are 'sky', 'portal', 'wall', 'object' 
-   * and 'alpha' (in that priority order, where sky is rendered first and 
-   * alpha is rendered last).  Should you wish to add your own render 
+   * \note The default render priorities are:
+   * - init: 1
+   * - sky: 2
+   * - sky2: 3
+   * - portal: 4
+   * - wall: 5
+   * - wall2: 6
+   * - object: 7
+   * - object2: 8
+   * - transp: 9
+   * - alpha: 10 (uses back2front sorting for meshes)
+   * - final: 11
+   * (in that priority order, where 'init' is rendered first and 
+   * 'final' is rendered last).  Should you wish to add your own render 
    * priority, you must call ClearRenderPriorities() and re-add the 
    * default render priorities along with your own new priorities.
    */
@@ -353,9 +370,8 @@ struct iEngine : public virtual iBase
 
   /**
    * Get a render priority by name.
-   * \param name is the name you want (usually one of 'sky', 'portal',
-   * 'wall', 'object', or 'alpha' but you can define your own render
-   * priorities).
+   * \param name is the name you want (one of the standard names
+   * or your own if you have defined your own render priorities).
    * \return 0 if render priority doesn't exist.
    */
   virtual long GetRenderPriority (const char* name) const = 0;
@@ -616,13 +632,27 @@ struct iEngine : public virtual iBase
   	iRegion* region = 0) = 0;
 
   /**
-   * This routine returns an iterator to iterate over
-   * all nearby sectors.
-   * Assign to a csRef.
+   * This routine returns an iterator to iterate over all nearby sectors.
+   * Assign to a csRef. In the iterator you can also fetch the
+   * position for every sector. Normally that position will be equal to
+   * the given input position here (pos) but if there are space warping
+   * portals involved then the position returned by the iterator is the
+   * warped position relative to the sector returned by the iterator.
    */
   virtual csPtr<iSectorIterator> GetNearbySectors (iSector* sector,
   	const csVector3& pos, float radius) = 0;
 
+  /**
+   * This routine returns an iterator to iterate over all nearby sectors.
+   * Assign to a csRef. In the iterator you can also fetch the
+   * position for every sector. Normally that position will be equal to
+   * the given input position here (pos) but if there are space warping
+   * portals involved then the position returned by the iterator is the
+   * warped position relative to the sector returned by the iterator.
+   */
+  virtual csPtr<iSectorIterator> GetNearbySectors (iSector* sector,
+  	const csBox3& box) = 0;
+
   /**
    * Add a frame callback. This will call IncRef() on the callback
    * So make sure you call DecRef() to release your own reference.
@@ -768,7 +798,7 @@ struct iEngine : public virtual iBase
    * portals. Otherwise it will limit the search to the sector passed in.
    */
   virtual csPtr<iMeshWrapperIterator> GetNearbyMeshes (iSector* sector,
-    const csVector3& pos, float radius, bool crossPortals = true ) = 0;
+    const csVector3& pos, float radius, bool crossPortals = true) = 0;
 
   /**
    * This routine returns an iterator to iterate over
@@ -778,7 +808,18 @@ struct iEngine : public virtual iBase
    * Portal visibility is tested with the center of the box.
    */
   virtual csPtr<iMeshWrapperIterator> GetNearbyMeshes (iSector* sector,
-    const csBox3& box, bool crossPortals = true ) = 0;
+    const csBox3& box, bool crossPortals = true) = 0;
+
+  /**
+   * This routine returns an iterator to iterate over
+   * all meshes that intersect with a beam.
+   * If crossPortals is true it will search through
+   * portals. Otherwise it will limit the search to the sector passed in.
+   * Portal visibility is tested fromt the start of the beam.
+   */
+  virtual csPtr<iMeshWrapperIterator> GetNearbyMeshes (iSector* sector,
+    const csVector3& start, const csVector3& end,
+    bool crossPortals = true) = 0;
 
   /// Get the list of meshes
   virtual iMeshList* GetMeshes () = 0;
@@ -883,6 +924,7 @@ struct iEngine : public virtual iBase
    * If the region already exists then this function will just
    * return the pointer to that region.
    * \param name the engine name for the region
+   * \remarks Can be removed with RemoveObject().
    */
   virtual iRegion* CreateRegion (const char* name) = 0;
   /// Get the list of all regions
@@ -1067,8 +1109,13 @@ struct iEngine : public virtual iBase
    * all textures must have been registered to that driver (using
    * Prepare()). Note that you need to call Prepare() again if
    * you switch to another 3D driver.
+   * <p>
+   * If a mesh is given then only that single mesh is rendered.
+   * Note that in that case the mesh will only be rendered if it
+   * is in the same sector as the camera!
    */
-  virtual void Draw (iCamera* c, iClipper2D* clipper) = 0;
+  virtual void Draw (iCamera* c, iClipper2D* clipper,
+      iMeshWrapper* mesh = 0) = 0;
 
   /**
    * Set the drawing context. This is a texture handle that is used
@@ -1224,30 +1271,12 @@ struct iEngine : public virtual iBase
     /// Get the list of all shared variables.
   virtual iSharedVariableList* GetVariableList () const = 0;
 
-  /// Get the list of collections.
-  virtual iCollectionList* GetCollections () = 0;
-  
-  /**
-   * Find the given collection. The name can be a normal
-   * name. In that case this function will look in all regions
-   * except if region is not 0 in which case it will only
-   * look in that region.
-   * If the name is specified as 'regionname/objectname' then
-   * this function will only look in the specified region and return
-   * 0 if that region doesn't contain the object or the region
-   * doesn't exist. In this case the region parameter is ignored.
-   * \param name the engine name of the desired collection
-   * \param region if specified, search only this region (also see note above)
-   */
-  virtual iCollection* FindCollection (const char* name,
-  	iRegion* region = 0) = 0;
-
   /**
    * Convenience function to 'remove' a CS object from the engine.
    * This will not clear the object but it will remove all references
    * to that object that the engine itself keeps. This function works
-   * for: iSector, iCollection, iMeshWrapper, iMeshFactoryWrapper,
-   * iCameraPosition, iLight, iMaterialWrapper, and iTextureWrapper.
+   * for: iCameraPosition, iCollection, iLight, iMaterialWrapper, 
+   * iMeshFactoryWrapper,iMeshWrapper, iRegion, iSector and iTextureWrapper.
    * Note that the object is only removed if the resulting ref count will
    * become zero. So basically this function only releases the references
    * that the engine holds.
@@ -1258,6 +1287,23 @@ struct iEngine : public virtual iBase
    * This function will also remove the object from the region it may be in.
    */
   virtual bool RemoveObject (iBase* object) = 0;
+
+  /**
+   * This function can be used to remove an object after a specific amount
+   * of time. This is mostly useful for particle systems (like explosions)
+   * that you want to live for a specific time before they are automatically
+   * cleaned up by the engine. Note that calling this function will cause
+   * the engine to keep an additional reference until it is time to delete
+   * the object.
+   */
+  virtual void DelayedRemoveObject (csTicks delay, iBase* object) = 0;
+
+  /**
+   * Clear all delayed removals.
+   * \param remove if true then the objects will also be removed from engine.
+   * Otherwise they are simply removed from this list.
+   */
+  virtual void RemoveDelayedRemoves (bool remove = false) = 0;
  
   /// Delete everything in the engine.
   virtual void DeleteAll () = 0;
