@@ -41,6 +41,7 @@ struct iMovable;
 struct iObject;
 struct iSector;
 struct iSceneNode;
+struct iShaderVariableContext;
 
 struct iBaseHalo;
 struct iCrossHalo;
@@ -148,10 +149,7 @@ enum csLightType
  * The given context will be either an instance of iRenderView, iFrustumView,
  * or else 0.
  *
- * Main creators of instances implementing this interface:
- * - Application.
- * 
- * Main users of this interface:
+ * This callback is used by:
  * - iLight
  */
 struct iLightCallback : public virtual iBase
@@ -410,6 +408,11 @@ struct iLight : public virtual iBase
    * lighting calculations.
    */
   virtual void Setup () = 0;
+
+  /**
+   * Get the shader variable context of the light.
+   */
+  virtual iShaderVariableContext* GetSVContext() = 0;
 };
 
 /**
@@ -510,8 +513,6 @@ struct iLightingProcessInfo : public iFrustumViewUserdata
   virtual void FinalizeLighting () = 0;
 };
 
-SCF_VERSION (iLightIterator, 0, 1, 0);
-
 /**
  * Iterator to iterate over all static lights in the engine.
  * This iterator assumes there are no fundamental changes
@@ -526,6 +527,8 @@ SCF_VERSION (iLightIterator, 0, 1, 0);
  */
 struct iLightIterator : public virtual iBase
 {
+  SCF_INTERFACE (iLightIterator, 0, 1, 0);
+
   /// Return true if there are more elements.
   virtual bool HasNext () = 0;
 
