@@ -20,6 +20,7 @@
 #define __CS_IENGINE_LOD_H__
 
 /**\file
+ * Level of detail interfaces
  */
 /**
  * \addtogroup engine3d_meshes
@@ -29,14 +30,13 @@
 
 struct iSharedVariable;
 
-SCF_VERSION (iLODControl, 0, 2, 0);
-
 /**
  * The iLODControl interface represents an object that has controllable
  * LOD features. The LOD manager can work with this.
  */
-struct iLODControl : public iBase
+struct iLODControl : public virtual iBase
 {
+  SCF_INTERFACE(iLODControl, 2,0,0);
   /**
    * Set a parameters for a function to compute LOD for the given
    * LOD control. The function is as follows:
