@@ -129,6 +129,11 @@ class csReversibleTransform;
  */
 #define CS_ENTITY_NOCLIP 128
 
+/**
+ * If CS_ENTITY_NODECAL is set then this entity will not accept decals.
+ */
+#define CS_ENTITY_NODECAL 256
+
 /** @} */
 
 /** \name SetLightingUpdate flags
@@ -151,8 +156,6 @@ class csReversibleTransform;
 
 /** @} */
 
-SCF_VERSION (iMeshDrawCallback, 0, 0, 1);
-
 /**
  * Set a callback which is called just before the object is drawn.
  * This is useful to do some expensive computations which only need
@@ -160,9 +163,14 @@ SCF_VERSION (iMeshDrawCallback, 0, 0, 1);
  * called even if the object is not visible. In general it is called
  * if there is a likely probability that the object is visible (i.e.
  * it is in the same sector as the camera for example).
+ *
+ * This callback is used by:
+ * - iMeshWrapper
  */
 struct iMeshDrawCallback : public iBase
 {
+  SCF_INTERFACE (iMeshDrawCallback, 0, 0, 1);
+
   /**
    * Before drawing. It is safe to delete this callback
    * in this function.
@@ -185,8 +193,8 @@ struct csHitBeamResult
   /// Only for HitBeamObject: the polygon/triangle index that was hit.
   int polygon_idx;
   /**
-   * Only for HitBeamObject: the material that was hit. Can be 0 in case
-   * the meshobject doesn't support getting the material.
+   * Only for HitBeamObject and HitBeam: the material that was hit. Can be 0
+   * in case the meshobject doesn't support getting the material.
    */
   iMaterialWrapper* material;
   /**
@@ -249,7 +257,7 @@ struct csScreenBoxResult
  */
 struct iMeshWrapper : public virtual iBase
 {
-  SCF_INTERFACE(iMeshWrapper, 2, 0, 0);
+  SCF_INTERFACE(iMeshWrapper, 2, 2, 0);
 
   /**
    * Get the iObject for this mesh object. This can be used to get the
@@ -361,45 +369,6 @@ struct iMeshWrapper : public virtual iBase
    */
   virtual void PlaceMesh () = 0;
 
-  /**
-   * Check if this mesh is hit by this object space vector.
-   * This will do a rough but fast test based on bounding box only.
-   * So this means that it might return a hit even though the object
-   * isn't really hit at all. Depends on how much the bounding box
-   * overestimates the object. This also returns the face number
-   * as defined in csBox3 on which face the hit occured. Useful for
-   * grid structures.
-   * \deprecated Use HitBeamBBox() with csHitBeamResult instead.
-   */
-  CS_DEPRECATED_METHOD virtual int HitBeamBBox (const csVector3& start,
-  	const csVector3& end, csVector3& isect, float* pr) = 0;
-
-  /**
-   * Check if this object is hit by this object space vector.
-   * Outline check.
-   * \deprecated Use HitBeamOutline() with csHitBeamResult instead.
-   */
-  CS_DEPRECATED_METHOD virtual bool HitBeamOutline (const csVector3& start,
-  	const csVector3& end, csVector3& isect, float* pr) = 0;
-
-  /**
-   * Check if this object is hit by this object space vector.
-   * Return the collision point in object space coordinates. This version
-   * is more accurate than HitBeamOutline.
-   * \deprecated Use HitBeamObject() with csHitBeamResult instead.
-   */
-  CS_DEPRECATED_METHOD virtual bool HitBeamObject (const csVector3& start,
-  	const csVector3& end, csVector3& isect, float* pr,
-	int* polygon_idx = 0) = 0;
-
-  /**
-   * Check if this object is hit by this world space vector.
-   * Return the collision point in world space coordinates.
-   * \deprecated Use HitBeamObject() with csHitBeamResult instead.
-   */
-  CS_DEPRECATED_METHOD virtual bool HitBeam (const csVector3& start,
-  	const csVector3& end, csVector3& isect, float* pr) = 0;
-
   /**
    * Check if this mesh is hit by this object space vector.
    * This will do a rough but fast test based on bounding box only.
@@ -436,10 +405,13 @@ struct iMeshWrapper : public virtual iBase
   /**
    * Check if this object is hit by this world space vector.
    * Return the collision point in world space coordinates.
+   * This version can also return the material that was hit (this will
+   * only happen if 'do_material' is true). This is not
+   * supported by all meshes so this can return 0 even if there was a hit.
    * \sa csHitBeamResult
    */
   virtual csHitBeamResult HitBeam (const csVector3& start,
-  	const csVector3& end) = 0;
+  	const csVector3& end, bool do_material = false) = 0;
 
   /**
    * Set a callback which is called just before the object is drawn.
@@ -516,13 +488,13 @@ struct iMeshWrapper : public virtual iBase
    * Enabling flags:
    * \code
    * csRef<iMeshWrapper> someWrapper = ...;
-   * someWrapper->SetFlags (CS_ENTITY_INVISIBLE | CS_ENTITY_NOCLIP);
+   * someWrapper->SetFlagsRecursive (CS_ENTITY_INVISIBLE | CS_ENTITY_NOCLIP);
    * \endcode
    * <p>
    * Disabling flags:
    * \code
    * csRef<iMeshWrapper> someWrapper = ...;
-   * someWrapper->SetFlags (CS_ENTITY_INVISIBLE | CS_ENTITY_NOCLIP, 0);
+   * someWrapper->SetFlagsRecursive (CS_ENTITY_INVISIBLE | CS_ENTITY_NOCLIP, 0);
    * \endcode
    * \remarks To set flags non-recursive, use GetFlags().Set().
    */
@@ -563,33 +535,6 @@ struct iMeshWrapper : public virtual iBase
    */
   virtual void HardTransform (const csReversibleTransform& t) = 0;
 
-  /**
-   * Get the bounding box of this object in world space.
-   * This routine will cache the bounding box and only recalculate it
-   * if the movable changes.
-   * \deprecated Use GetWorldBoundingBox that returns a csBox3 instead.
-   */
-  CS_DEPRECATED_METHOD virtual void GetWorldBoundingBox (csBox3& cbox) = 0;
-
-  /**
-   * Get the bounding box of this object after applying a transformation to it.
-   * This is really a very inaccurate function as it will take the bounding
-   * box of the object in object space and then transform this bounding box.
-   * \deprecated Use GetTransformedBoundingBox that returns a csBox3 instead.
-   */
-  CS_DEPRECATED_METHOD virtual void GetTransformedBoundingBox (
-  	const csReversibleTransform& trans, csBox3& cbox) = 0;
-
-  /**
-   * Get a very inaccurate bounding box of the object in screen space.
-   * Returns -1 if object behind the camera or else the distance between
-   * the camera and the furthest point of the 3D box.
-   * \deprecated Use GetScreenBoundingBox() that returns a csScreenBoxResult
-   * instead.
-   */
-  CS_DEPRECATED_METHOD virtual float GetScreenBoundingBox (iCamera* camera,
-  	csBox2& sbox, csBox3& cbox) = 0;
-
   /**
    * Get the bounding box of this object in world space.
    * This routine will cache the bounding box and only recalculate it
@@ -612,14 +557,8 @@ struct iMeshWrapper : public virtual iBase
    */
   virtual csScreenBoxResult GetScreenBoundingBox (iCamera* camera) = 0;
 
-  /**
-   * Get the radius of this mesh and all its children.
-   * \deprecated Use GetRadius() that returns csEllipsoid.
-   */
-  CS_DEPRECATED_METHOD virtual void GetRadius (csVector3& rad,
-  	csVector3& cent) const = 0;
   /// Get the radius of this mesh and all its children.
-  virtual csEllipsoid GetRadius () const = 0;
+  virtual csSphere GetRadius () const = 0;
 
   /**
    * Reset minimum/maximum render range to defaults (i.e. unlimited).
@@ -711,6 +650,29 @@ struct iMeshWrapper : public virtual iBase
    * Get the shader variable context of the mesh object.
    */
   virtual iShaderVariableContext* GetSVContext() = 0;
+
+  /**
+   * Adds a render mesh to the list of extra render meshes.
+   * This list is used for special cases (like decals) where additional
+   * things need to be renderered for the mesh in an abstract way.
+   */
+  virtual void AddExtraRenderMesh(csRenderMesh* renderMesh, long priority, 
+          csZBufMode zBufMode) = 0;
+
+  /** 
+   * Gets the priority of a specific extra rendermesh.
+   */
+  virtual long GetExtraRenderMeshPriority(size_t idx) const = 0;
+
+  /**
+   * Gets the z-buffer mode of a specific extra rendermesh
+   */
+  virtual csZBufMode GetExtraRenderMeshZBufMode(size_t idx) const = 0;
+
+  /**
+   * Deletes a specific extra rendermesh
+   */
+  virtual void RemoveExtraRenderMesh(csRenderMesh* renderMesh) = 0;
 };
 
 /**
@@ -761,7 +723,28 @@ struct iMeshFactoryWrapper : public virtual iBase
    * Create mesh objects from this factory. If the factory has a hierarchy
    * then a hierarchical mesh object will be created.
    */
-  virtual iMeshWrapper* CreateMeshWrapper () = 0;
+  virtual csPtr<iMeshWrapper> CreateMeshWrapper () = 0;
+
+  /**
+   * Get default flags for all meshes created from this factory.
+   * The following flags are supported:
+   * - #CS_ENTITY_DETAIL: this is a detail object. Again this is a hint
+   *   for the engine to render this object differently. Currently not used.
+   * - #CS_ENTITY_CAMERA: entity will always be centered around the camera.
+   * - #CS_ENTITY_INVISIBLEMESH: entity is not rendered. 
+   * - #CS_ENTITY_NOHITBEAM: this entity will not be considered by HitBeam() 
+   *   calls.
+   * - #CS_ENTITY_INVISIBLE: means that either CS_ENTITY_INVISIBLEMESH and 
+   *   CS_ENTITY_NOHITBEAM are set.
+   * - #CS_ENTITY_NOSHADOWS: cast no shadows.
+   * - #CS_ENTITY_NOLIGHTING: do not light this object.
+   * - #CS_ENTITY_NOCLIP: do not clip this object.
+   *
+   * \remarks Despite the name, this method does not only provide read access
+   *   to the mesh flags, as the returned reference to a csFlags object also 
+   *   provides write access.
+   */
+  virtual csFlags& GetFlags () = 0;
 
   /**
    * Get the parent of this factory. This will be 0 if this factory
