@@ -171,10 +171,16 @@ public:
 /**
  * This interface represents all information needed to render
  * some object in a current draw context.
+ *
+ * Main creators of instances implementing this interface:
+ * - iEngine
+ * 
+ * Main users of this interface:
+ * - meshes
  */
 struct iRenderView : public virtual iBase
 {
-  SCF_INTERFACE(iRenderView, 2,0,0);
+  SCF_INTERFACE(iRenderView, 2,1,0);
   /// Get the current render context.
   virtual csRenderContext* GetRenderContext () = 0;
 
@@ -201,7 +207,9 @@ struct iRenderView : public virtual iBase
 
   /**
    * Given a frustum_mask, calculate the clip settings.
+   * \deprecated Use CS::RenderViewClipper::CalculateClipSettings() instead.
    */
+  CS_DEPRECATED_METHOD_MSG("Use CS::RenderViewClipper::CalculateClipSettings() instead.")
   virtual void CalculateClipSettings (uint32 frustum_mask,
     int &clip_portal, int &clip_plane, int &clip_z_plane) = 0;
 
@@ -209,7 +217,9 @@ struct iRenderView : public virtual iBase
    * Test if the given bounding sphere (in world space coordinates)
    * is visible in this render view. The optional will
    * transform world to camera space.
+   * \deprecated Use CS::RenderViewClipper::TestBSphere() instead.
    */
+  CS_DEPRECATED_METHOD_MSG("Use CS::RenderViewClipper::TestBSphere() instead.")
   virtual bool TestBSphere (const csReversibleTransform& w2c,
   	const csSphere& sphere) = 0;
 
@@ -223,8 +233,10 @@ struct iRenderView : public virtual iBase
    * The frustum_mask will be modified according to all clip planes
    * that were relevant for the given box. That can be used to hierarchically
    * cull smaller objects.
+   * \deprecated Use CS::RenderViewClipper::CullBBox() instead.
    */
-  virtual bool ClipBBox (csPlane3* planes, uint32& frustum_mask,
+  CS_DEPRECATED_METHOD_MSG("Use CS::RenderViewClipper::CullBBox() instead.")
+  virtual bool ClipBBox (const csPlane3* planes, uint32& frustum_mask,
   	const csBox3& obox,
       	int& clip_portal, int& clip_plane, int& clip_z_plane) = 0;
 
@@ -232,7 +244,9 @@ struct iRenderView : public virtual iBase
    * Setup clipping planes in object space. The input arrays for planes
    * should each be able to hold 10 planes. Returns a mask that you can
    * use for the csIntersect3::BoxFrustum() function.
+   * \deprecated Use CS::RenderViewClipper::SetupClipPlanes() instead.
    */
+  CS_DEPRECATED_METHOD_MSG("Use CS::RenderViewClipper::SetupClipPlanes() instead.")
   virtual void SetupClipPlanes (const csReversibleTransform& tr_o2c,
   	csPlane3* planes, uint32& frustum_mask) = 0;
 
@@ -266,36 +280,14 @@ struct iRenderView : public virtual iBase
    * Check if the given bounding sphere (in camera and world space coordinates)
    * is visibile in this render view. If the sphere is visible this
    * function will also initialize the clip_plane, clip_z_plane, and
-   * clip_portal fields which can be used for DrawTriangleMesh or
-   * DrawPolygonMesh.
+   * clip_portal fields which can be used for the renderer.
+   * \deprecated Use CS::RenderViewClipper::CullBSphere() instead.
    */
+  CS_DEPRECATED_METHOD_MSG("Use CS::RenderViewClipper::CullBSphere() instead.")
   virtual bool ClipBSphere (
 	const csSphere &cam_sphere,
 	const csSphere &world_sphere,
 	int& clip_portal, int& clip_plane, int& clip_z_plane) = 0;
-  
-
-
-  // @@@ ADDED B/C OF FATLOOP PORTAL HACKING
-  // @@@ REMOVE AGAIN ASAP
-  virtual void CreateRenderContext () = 0;
-  virtual int GetRenderRecursionLevel () const = 0;
-  virtual void SetRenderRecursionLevel (int rec) = 0;
-  virtual void SetClipper (iClipper2D* clip) = 0;
-  virtual void ResetFogInfo () = 0;
-  virtual void SetPreviousSector (iSector* s) = 0;
-  virtual void SetClipPlane (const csPlane3& p) = 0;
-  virtual bool GetClipPlane (csPlane3& pl) const = 0;
-  virtual const csPlane3& GetClipPlane () const = 0;
-  virtual csPlane3& GetClipPlane () = 0;
-  virtual void UseClipPlane (bool u) = 0;
-  virtual void UseClipFrustum (bool u) = 0;
-  virtual void SetLastPortal (iPortal* por) = 0;
-  virtual bool IsClipperRequired () const = 0;
-  virtual iCamera* CreateNewCamera () = 0;
-  virtual void RestoreRenderContext () = 0;
-  virtual void SetThisSector (iSector* s) = 0;
-  virtual void SetupClipPlanes () = 0;
 };
 
 /** @} */
