@@ -20,15 +20,17 @@
 #define __CS_IGEOM_CLIP2D_H__
 
 /**\file
+ * 2D clipping interface
  */
 
 /**
  * \addtogroup geom_utils
  * @{ */
 
-#include "csutil/scf.h"
-#include "csgeom/vector2.h"
-#include "csgeom/box.h"
+#include "csutil/scf_interface.h"
+
+class csBox2;
+class csVector2;
 
 /// Maximal number of vertices in output (clipped) polygons
 #define MAX_OUTPUT_VERTICES	64
@@ -41,13 +43,15 @@
  * polygon (thus it has not changed) and partially outside, partially inside 
  * (thus it was clipped).
  * @{ */
-
-/// The input polygon is completely outside of clipper polygon
-#define CS_CLIP_OUTSIDE		0
-/// The input polygon is completely inside (thus has not changed)
-#define CS_CLIP_INSIDE		1
-/// The input polygon was partially inside, partially outside
-#define CS_CLIP_CLIPPED		2
+enum
+{
+  /// The input polygon is completely outside of clipper polygon
+  CS_CLIP_OUTSIDE = 0,
+  /// The input polygon was partially inside, partially outside
+  CS_CLIP_CLIPPED = 1,
+  /// The input polygon is completely inside (thus has not changed)
+  CS_CLIP_INSIDE = 2
+};
 /** @} */
 
 /**
@@ -56,44 +60,47 @@
 /**
  * The clipper can output additional information about each vertex in
  * output polygon. This is used for generating U/V/Z values and so on,
- * if they are needed.
- * @{ */
+ * if they are needed. 
+ */
 struct csVertexStatus
 {
   /**
-   * The type of vertex<p>
-   * One of CS_VERTEX_*.
+   * The type of vertex.
+   * One of #CS_VERTEX_ORIGINAL, #CS_VERTEX_ONEDGE, #CS_VERTEX_INSIDE.
    */
   unsigned char Type;
   /// Original vertex number (for #CS_VERTEX_ORIGINAL and #CS_VERTEX_ONEDGE)
-  unsigned char Vertex;
+  size_t Vertex;
   /// Additional information for #CS_VERTEX_ONEDGE (0..1, the 't' parameter)
   float Pos;
 };
 
-/// The output vertex is one of the input vertices
-#define CS_VERTEX_ORIGINAL	0
-/// The output vertex is located on one of the edges of the original polygon
-#define CS_VERTEX_ONEDGE	1
-/// The output vertex is located somewhere inside the original polygon
-#define CS_VERTEX_INSIDE	2
+/// Clipped vertex type
+enum
+{
+  /// The output vertex is one of the input vertices
+  CS_VERTEX_ORIGINAL = 0,
+  /// The output vertex is located on one of the edges of the original polygon
+  CS_VERTEX_ONEDGE = 1,
+  /// The output vertex is located somewhere inside the original polygon
+  CS_VERTEX_INSIDE = 2
+};
 /** @} */
 
 
-SCF_VERSION (iClipper2D, 0, 0, 3);
-
 /**
  * This interfaces represents a 2D clipper for polygons.
  */
-struct iClipper2D : public iBase
+struct iClipper2D : public virtual iBase
 {
+  SCF_INTERFACE(iClipper2D, 2, 0, 0);
   /**
    * Clip a set of 2D points and return in 'OutPolygon' which is expected
    * to contain space at least for #MAX_OUTPUT_VERTICES elements.
-   * Returns one of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
+   * \return One of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
    */
-  virtual uint8 Clip (csVector2 *InPolygon, int InCount,
-    csVector2 *OutPolygon, int &OutCount) = 0;
+  virtual uint8 Clip (csVector2 *InPolygon, size_t InCount,
+    csVector2 *OutPolygon, size_t &OutCount) = 0;
 
   /**
    * Clip a set of 2D points.
@@ -101,37 +108,36 @@ struct iClipper2D : public iBase
    * The output array is expected to contain space for at least
    * #MAX_OUTPUT_VERTICES elements. The bounding box is set to the
    * minimal rectangle that contains the output polygon.
-   * Returns one of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
+   * \return One of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
    */
-  virtual uint8 Clip (csVector2 *InPolygon, int InCount,
-    csVector2 *OutPolygon, int &OutCount, csBox2 &BoundingBox) = 0;
+  virtual uint8 Clip (csVector2 *InPolygon, size_t InCount,
+    csVector2 *OutPolygon, size_t &OutCount, csBox2 &BoundingBox) = 0;
 
   /**
-   * Clips a set of 2D points and provides additional information on each output
-   * vertex. The information type can be: vertex is one of original vertices,
-   * vertex is on the edge of the original polygon and vertex is arbitrary
-   * located inside the original polygon. Both OutPolygon and OutStatus
-   * arrays are expected to have enough storage for at least
-   * #MAX_OUTPUT_VERTICES elements.<p>
-   * Returns one of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
+   * Clips a set of 2D points and provides additional information on each
+   * output vertex. The information type can be: vertex is one of original
+   * vertices, vertex is on the edge of the original polygon and vertex is
+   * arbitrary located inside the original polygon. Both OutPolygon and
+   * OutStatus arrays are expected to have enough storage for at least
+   * #MAX_OUTPUT_VERTICES elements.
+   * \return One of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
    */
-  virtual uint8 Clip (csVector2 *InPolygon, int InCount,
-    csVector2 *OutPolygon, int &OutCount, csVertexStatus *OutStatus) = 0;
+  virtual uint8 Clip (csVector2 *InPolygon, size_t InCount,
+    csVector2 *OutPolygon, size_t &OutCount, csVertexStatus *OutStatus) = 0;
 
   /**
-   * Wrapper function: clip a polygon in-place.<p>
-   * Returns one of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
+   * Wrapper function: clip a polygon in-place.
+   * \return One of \link #CS_CLIP_OUTSIDE CS_CLIP_*\endlink.
    */
-  virtual uint8 ClipInPlace (csVector2 *InPolygon, int &InOutCount, csBox2 &BoundingBox) = 0;
+  virtual uint8 ClipInPlace (csVector2 *InPolygon, size_t &InOutCount,
+			     csBox2 &BoundingBox) = 0;
 
   /**
    * Classify some bounding box against this clipper.
    * This function returns:<p>
-   * <ul>
-   * <li> -1 if box is not visible.
-   * <li> 0 if box is partially visible.
-   * <li> 1 if box is entirely visible.
-   * </ul>
+   * - -1 if box is not visible.
+   * - 0 if box is partially visible.
+   * - 1 if box is entirely visible.
    */
   virtual int ClassifyBox (const csBox2 &box) = 0;
 
@@ -139,7 +145,7 @@ struct iClipper2D : public iBase
   virtual bool IsInside (const csVector2& v) = 0;
 
   /// Return number of vertices for this clipper polygon.
-  virtual int GetVertexCount () = 0;
+  virtual size_t GetVertexCount () = 0;
 
   /// Return a pointer to the array of csVector2's
   virtual csVector2 *GetClipPoly () = 0;
@@ -161,4 +167,3 @@ struct iClipper2D : public iBase
 /** @} */
 
 #endif // __CS_IGEOM_CLIP2D_H__
-
