@@ -25,7 +25,7 @@
  */
 /**\addtogroup loadsave
  * @{ */
-#include "csutil/scf.h"
+#include "csutil/scf_interface.h"
 
 #include "igraphic/image.h"
 #include "ivideo/txtmgr.h"
@@ -33,14 +33,13 @@
 
 struct iDocumentNode;
 struct iImage;
+struct iLight;
 struct iMaterialWrapper;
 struct iMeshWrapper;
 struct iMeshFactoryWrapper;
 struct iRegion;
 struct iSector;
-struct iSoundData;
-struct iSoundHandle;
-struct iSoundWrapper;
+struct iShader;
 struct iTextureHandle;
 struct iTextureManager;
 struct iTextureWrapper;
@@ -48,26 +47,94 @@ struct iSndSysData;
 struct iSndSysWrapper;
 struct iSndSysStream;
 
-SCF_VERSION (iLoaderStatus, 0, 1, 0);
-
 /**
  * An object to query about the status of the threaded loader.
  */
-struct iLoaderStatus : public iBase
+struct iLoaderStatus : public virtual iBase
 {
+  SCF_INTERFACE (iLoaderStatus, 1, 0, 0);
+
   /// Check if the loader is ready.
   virtual bool IsReady () = 0;
   /// Check if there was an error during loading.
   virtual bool IsError () = 0;
 };
 
-SCF_VERSION (iLoader, 0, 0, 8);
+/**
+ * This callback is called when the loader can't find some material,
+ * texture, factory, mesh, light, shader, or sector. This gives the
+ * application a chance to manually load it first.
+ */
+struct iMissingLoaderData : public virtual iBase
+{
+  SCF_INTERFACE (iMissingLoaderData, 1, 0, 0);
+
+  /**
+   * Called when a material is missing. This implementation should
+   * either attempt to find or load the material or else return 0.
+   * In the last case the loader will proceed as usual when a material
+   * is not found.
+   */
+  virtual iMaterialWrapper* MissingMaterial (const char* name,
+      	const char* filename) = 0;
+
+  /**
+   * Called when a texture is missing. This implementation should
+   * either attempt to find or load the texture or else return 0.
+   * In the last case the loader will proceed as usual when a texture
+   * is not found.
+   */
+  virtual iTextureWrapper* MissingTexture (const char* name,
+      const char* filename) = 0;
+
+  /**
+   * Called when a shader is missing. This implementation should
+   * either attempt to find or load the shader or else return 0.
+   * In the last case the loader will proceed as usual when a shader
+   * is not found.
+   */
+  virtual iShader* MissingShader (const char* name) = 0;
+
+  /**
+   * Called when a mesh factory is missing. This implementation should
+   * either attempt to find or load the mesh factory or else return 0.
+   * In the last case the loader will proceed as usual when a mesh factory
+   * is not found.
+   */
+  virtual iMeshFactoryWrapper* MissingFactory (const char* name) = 0;
+
+  /**
+   * Called when a mesh is missing. This implementation should
+   * either attempt to find or load the mesh or else return 0.
+   * In the last case the loader will proceed as usual when a mesh
+   * is not found.
+   */
+  virtual iMeshWrapper* MissingMesh (const char* name) = 0;
+
+  /**
+   * Called when a sector is missing. This implementation should
+   * either attempt to find or load the sector or else return 0.
+   * In the last case the loader will proceed as usual when a sector
+   * is not found.
+   */
+  virtual iSector* MissingSector (const char* name) = 0;
+
+  /**
+   * Called when a light is missing. This implementation should
+   * either attempt to find or load the light or else return 0.
+   * In the last case the loader will proceed as usual when a light
+   * is not found.
+   */
+  virtual iLight* MissingLight (const char* name) = 0;
+};
 
 /**
  * This interface represents the map loader.
  */
-struct iLoader : public iBase
+struct iLoader : public virtual iBase
 {
+  SCF_INTERFACE (iLoader, 2, 1, 0);
+
   /**
    * Load an image file. The image will be loaded in the format requested by
    * the engine. If no engine exists, the format is taken from the video
@@ -176,17 +243,6 @@ struct iLoader : public iBase
 	bool reg = true, bool create_material = true,
 	bool free_image = true) = 0;
 
-  /// Old Sound System: Load a sound file and return an iSoundData object
-  virtual csPtr<iSoundData> LoadSoundData (const char *fname) = 0;
-  /// Old Sound System: Load a sound file and register the sound
-  virtual csPtr<iSoundHandle> LoadSound (const char *fname) = 0;
-  /**
-   * Old Sound System: Load a sound file, register the sound and create a
-   * wrapper object for it.
-   */
-  virtual csPtr<iSoundWrapper> LoadSound (const char *name,
-  	const char *fname) = 0;
-
   /// New Sound System: Load a sound file and return an iSndSysData object
   virtual csPtr<iSndSysData> LoadSoundSysData (const char *fname) = 0;
   /**
@@ -197,16 +253,15 @@ struct iLoader : public iBase
    */
   virtual csPtr<iSndSysStream> LoadSoundStream (const char *fname,
   	int mode3d) = 0;
+
   /**
-   * New Sound System: Load a sound file, create a stream and create a
+   * New Sound System: Load a sound file, create sound data and create a
    * wrapper object for it.
    * \param name of the sound.
    * \param fname is the VFS filename.
-   * \param mode3d is one of CS_SND3D_DISABLE, CS_SND3D_RELATIVE, or
-   * CS_SND3D_ABSOLUTE.
    */
   virtual iSndSysWrapper* LoadSoundWrapper (const char *name,
-  	const char *fname, int mode3d) = 0;
+  	const char *fname) = 0;
 
   /**
    * Load a map file in a thread.
@@ -227,7 +282,7 @@ struct iLoader : public iBase
    * This function returns immediatelly. You can check the iLoaderStatus
    * instance that is returned to see if the map has finished loading or
    * if there was an error.
-   * @@@ NOT IMPLEMENTED YET @@@
+   * \todo This is not implemented! Implement me or remove me!
    */
   virtual csPtr<iLoaderStatus> ThreadedLoadMapFile (const char* filename,
 	iRegion* region = 0, bool curRegOnly = true,
@@ -258,10 +313,13 @@ struct iLoader : public iBase
    * with the same name. Only use checkDupes == true if you know that your
    * objects have unique names accross all world files.
    * \param ssource is an optional stream source for faster loading.
+   * \param missingdata is an optional callback in case data is missing.
+   * The application can then provide that missing data in some other way.
    */
   virtual bool LoadMapFile (const char* filename, bool clearEngine = true,
 	iRegion* region = 0, bool curRegOnly = true,
-	bool checkDupes = false, iStreamSource* ssource = 0) = 0;
+	bool checkDupes = false, iStreamSource* ssource = 0,
+	iMissingLoaderData* missingdata = 0) = 0;
 
   /**
    * Load a map from the given 'world' node. If 'clearEngine' is true then
@@ -288,10 +346,13 @@ struct iLoader : public iBase
    * with the same name. Only use checkDupes == true if you know that your
    * objects have unique names accross all world files.
    * \param ssource is an optional stream source for faster loading.
+   * \param missingdata is an optional callback in case data is missing.
+   * The application can then provide that missing data in some other way.
    */
   virtual bool LoadMap (iDocumentNode* world_node, bool clearEngine = true,
 	iRegion* region = 0, bool curRegOnly = true,
-	bool checkDupes = false, iStreamSource* ssource = 0) = 0;
+	bool checkDupes = false, iStreamSource* ssource = 0,
+	iMissingLoaderData* missingdata = 0) = 0;
 
   /**
    * Load library from a VFS file
@@ -308,10 +369,13 @@ struct iLoader : public iBase
    * with the same name. Only use checkDupes == true if you know that your
    * objects have unique names accross all world files.
    * \param ssource is an optional stream source for faster loading.
+   * \param missingdata is an optional callback in case data is missing.
+   * The application can then provide that missing data in some other way.
    */
   virtual bool LoadLibraryFile (const char* filename, iRegion* region = 0,
   	bool curRegOnly = true, bool checkDupes = false,
-	iStreamSource* ssource = 0) = 0;
+	iStreamSource* ssource = 0,
+	iMissingLoaderData* missingdata = 0) = 0;
 
   /**
    * Load library from a 'library' node.
@@ -328,10 +392,13 @@ struct iLoader : public iBase
    * with the same name. Only use checkDupes == true if you know that your
    * objects have unique names accross all world files.
    * \param ssource is an optional stream source for faster loading.
+   * \param missingdata is an optional callback in case data is missing.
+   * The application can then provide that missing data in some other way.
    */
   virtual bool LoadLibrary (iDocumentNode* lib_node, iRegion* region = 0,
   	bool curRegOnly = true, bool checkDupes = false,
-	iStreamSource* ssource = 0) = 0;
+	iStreamSource* ssource = 0,
+	iMissingLoaderData* missingdata = 0) = 0;
 
   /**
    * Load a Mesh Object Factory from a file.
@@ -387,10 +454,13 @@ struct iLoader : public iBase
    * \param override_name if this is given the the name of the loaded object
    * will be set to that. This only works in case of meshfact, meshobj, and
    * 3ds or md2 model.
+   * \param missingdata is an optional callback in case data is missing.
+   * The application can then provide that missing data in some other way.
    */
   virtual bool Load (const char* fname, iBase*& result, iRegion* region = 0,
   	bool curRegOnly = true, bool checkDupes = false,
-	iStreamSource* ssource = 0, const char* override_name = 0) = 0;
+	iStreamSource* ssource = 0, const char* override_name = 0,
+	iMissingLoaderData* missingdata = 0) = 0;
 
   /**
    * Load a file. This is a smart function that will try to recognize
@@ -430,10 +500,13 @@ struct iLoader : public iBase
    * \param override_name if this is given the the name of the loaded object
    * will be set to that. This only works in case of meshfact, meshobj, and
    * 3ds or md2 model.
+   * \param missingdata is an optional callback in case data is missing.
+   * The application can then provide that missing data in some other way.
    */
   virtual bool Load (iDataBuffer* buffer, iBase*& result, iRegion* region = 0,
   	bool curRegOnly = true, bool checkDupes = false,
-	iStreamSource* ssource = 0, const char* override_name = 0) = 0;
+	iStreamSource* ssource = 0, const char* override_name = 0,
+	iMissingLoaderData* missingdata = 0) = 0;
 
   /**
    * Load a node. This is a smart function that will try to recognize
@@ -472,15 +545,28 @@ struct iLoader : public iBase
    * \param override_name if this is given the the name of the loaded object
    * will be set to that. This only works in case of meshfact, meshobj, and
    * 3ds or md2 model.
+   * \param missingdata is an optional callback in case data is missing.
+   * The application can then provide that missing data in some other way.
    */
   virtual bool Load (iDocumentNode* node, iBase*& result, iRegion* region = 0,
   	bool curRegOnly = true, bool checkDupes = false,
-	iStreamSource* ssource = 0, const char* override_name = 0) = 0;
+	iStreamSource* ssource = 0, const char* override_name = 0,
+	iMissingLoaderData* missingdata = 0) = 0;
 
   /**
    * Load a shader from a file.
    */
-  virtual bool LoadShader (const char* filename) = 0;
+  virtual csRef<iShader> LoadShader (const char* filename, bool registerShader = true) = 0;
+  
+  /**
+   * Set whether to load each file into a separate region.
+   */
+  virtual void SetAutoRegions (bool autoRegions) = 0;
+  
+  /**
+   * Get whether to load each file into a separate region.
+   */
+  virtual bool GetAutoRegions () = 0;
 };
 
 /** @} */
