@@ -23,7 +23,7 @@
  * General mesh object
  */ 
 
-#include "csutil/scf.h"
+#include "csutil/scf_interface.h"
 
 struct iDocumentNode;
 struct iGenMeshAnimationControl;
@@ -31,6 +31,7 @@ struct iGenMeshAnimationControlFactory;
 struct iMaterialWrapper;
 struct iRenderBuffer;
 struct iMeshObject;
+struct iString;
 
 /**\addtogroup meshplugins
  * @{ */
@@ -43,32 +44,39 @@ class csVector2;
 class csVector3;
 class csEllipsoid;
 
-SCF_VERSION (iGeneralMeshCommonState, 0, 0, 3);
+struct iGeneralMeshSubMesh : public virtual iBase
+{
+  SCF_INTERFACE (iGeneralMeshSubMesh, 1, 0, 1);
+  
+  /// Get the index render buffer
+  virtual iRenderBuffer* GetIndices () const = 0;
+
+  /// Get the material
+  virtual iMaterialWrapper* GetMaterial () const = 0;
+  
+  /// Get the name (or 0 if none was given)
+  virtual const char* GetName () const = 0;
+  
+  /// Get the mixmode (or (uint)~0 if none was specified)
+  virtual uint GetMixmode () const = 0;
+
+  /// Set the material, or 0 to use default.
+  virtual void SetMaterial (iMaterialWrapper* material) = 0;
+};
 
 /**
  * The common interface between genmesh meshes and factories.
  * This interface is usually not used alone. Generally one
  * uses iGeneralMeshState or iGeneralFactoryState.
  */
-struct iGeneralMeshCommonState : public iBase
+struct iGeneralMeshCommonState : public virtual iBase
 {
-  /// Set material of mesh.
-  virtual void SetMaterialWrapper (iMaterialWrapper* material) = 0;
-  /// Get material of mesh.
-  virtual iMaterialWrapper* GetMaterialWrapper () const = 0;
-  /// Set mix mode.
-  virtual void SetMixMode (uint mode) = 0;
-  /// Get mix mode.
-  virtual uint GetMixMode () const = 0;
-
+  SCF_INTERFACE (iGeneralMeshCommonState, 1, 2, 0);
+  
   /// Set lighting.
   virtual void SetLighting (bool l) = 0;
   /// Is lighting enabled.
   virtual bool IsLighting () const = 0;
-  /// Set the color to use. Will be added to the lighting values.
-  virtual void SetColor (const csColor& col) = 0;
-  /// Get the color.
-  virtual const csColor& GetColor () const = 0;
   /**
    * Set manual colors. If this is set then lighting will be ignored
    * and so will the color set with SetColor(). In this case you can
@@ -96,55 +104,34 @@ struct iGeneralMeshCommonState : public iBase
   /// Is shadow receiving enabled?
   virtual bool IsShadowReceiving () const = 0;
 
-  /**
-   * Remove all submeshes added to this object
-   */
-  virtual void ClearSubMeshes () = 0;
-
-  /**
-   * Add a submesh to this object. A submesh is a subset of the mesh triangles
-   * rendered with a certain material. When a mesh has one or more submeshes,
-   * only submeshes are drawn and not original geometry. That means submeshes
-   * should cover all original triangles to avoid holes in the mesh.
-   * triangles is an array of indices into the factory triangle list
-   * tricount is the number of triangles in "triangles"
-   * material is a material to assign to the mesh
-   * Note! Submeshes added to an instance of a genmesh will override
-   * the submeshes from the factory (i.e. the submeshes of the factory will
-   * be completely ignored as soon as the instance has submeshes).
-   */
-  virtual void AddSubMesh (unsigned int *triangles,
-    int tricount,
-    iMaterialWrapper *material) = 0;
-
-  /**
-   * Add a submesh to this object. A submesh is a subset of the mesh triangles
-   * rendered with a certain material. When a mesh has one or more submeshes,
-   * only submeshes are drawn and not original geometry. That means submeshes
-   * should cover all original triangles to avoid holes in the mesh.
-   * triangles is an array of indices into the factory triangle list
-   * tricount is the number of triangles in "triangles"
-   * material is a material to assign to the mesh
-   * Note! Submeshes added to an instance of a genmesh will override
-   * the submeshes from the factory (i.e. the submeshes of the factory will
-   * be completely ignored as soon as the instance has submeshes).
-   * This version overrides the parent mixmode.
-   */
-  virtual void AddSubMesh (unsigned int *triangles,
-    int tricount,
-    iMaterialWrapper *material, uint mixmode) = 0;
-
+  /**\name Custom render buffers
+   * @{ */
   /**
    * Adds an independently named render buffer.
    */
   virtual bool AddRenderBuffer (const char *name, iRenderBuffer* buffer) = 0;
+
   /**
    * Removes an independently named render buffer.
    */
   virtual bool RemoveRenderBuffer (const char *name) = 0;
-};
 
-SCF_VERSION (iGeneralMeshState, 0, 1, 0);
+  /**
+   * Get number of independent render buffers
+   */
+  virtual int GetRenderBufferCount () const = 0;
+
+  /**
+   * Get independent render buffer by index
+   */
+  virtual csRef<iRenderBuffer> GetRenderBuffer (int index) = 0;
+
+  /**
+   * Get the name of an independent render buffer by index
+   */
+  virtual csRef<iString> GetRenderBufferName (int index) const = 0;
+  /** @} */
+};
 
 /**
  * This interface describes the API for the general mesh object.
@@ -160,8 +147,10 @@ SCF_VERSION (iGeneralMeshState, 0, 1, 0);
  * - Genmesh Loader plugin (crystalspace.mesh.loader.genmesh)
  *   
  */
-struct iGeneralMeshState : public iGeneralMeshCommonState
+struct iGeneralMeshState : public virtual iGeneralMeshCommonState
 {
+  SCF_INTERFACE (iGeneralMeshState, 1, 1, 1);
+  
   /**
    * Set the animation control to use for this mesh object.
    * See iGenMeshAnimationControl for more information.
@@ -172,9 +161,61 @@ struct iGeneralMeshState : public iGeneralMeshCommonState
    * Get the current animation control for this object.
    */
   virtual iGenMeshAnimationControl* GetAnimationControl () const = 0;
-};
+  
+  /**\name SubMesh handling
+   * @{ */
+  /**
+   * Find the index of a submesh. The index can be used with DeleteSubMesh()
+   * and the GetSubMesh...() methods. Returns 0 if the submesh was not found.
+   *
+   * The returned interface can be used for limited per-object variation of
+   * the submeshes as defined in the factory. Currently the following aspects
+   * can be overridden:
+   *  - Shader variables (by querying the iShaderVariableContext interface)
+   */
+  virtual iGeneralMeshSubMesh* FindSubMesh (const char* name) const = 0;
+  /** @} */
 
-SCF_VERSION (iGeneralFactoryState, 0, 3, 0);
+  /**\name Legacy submesh support
+   * @{ */
+  /**
+   * Add a submesh to this object. A submesh is a subset of the mesh triangles
+   * rendered with a certain material. When a mesh has one or more submeshes,
+   * only submeshes are drawn and not original geometry. That means submeshes
+   * should cover all original triangles to avoid holes in the mesh.
+   * triangles is an array of indices into the factory triangle list
+   * tricount is the number of triangles in "triangles"
+   * material is a material to assign to the mesh
+   * Note! SubMeshes added to an instance of a genmesh will override
+   * the submeshes from the factory (i.e. the submeshes of the factory will
+   * be completely ignored as soon as the instance has submeshes).
+   * \deprecated Use AddSubMesh from iGeneralFactoryState instead
+   */
+  CS_DEPRECATED_METHOD_MSG("Use AddSubMesh from iGeneralFactoryState instead")
+  virtual void AddSubMesh (unsigned int *triangles,
+    int tricount,
+    iMaterialWrapper *material) = 0;
+
+  /**
+   * Add a submesh to this object. A submesh is a subset of the mesh triangles
+   * rendered with a certain material. When a mesh has one or more submeshes,
+   * only submeshes are drawn and not original geometry. That means submeshes
+   * should cover all original triangles to avoid holes in the mesh.
+   * triangles is an array of indices into the factory triangle list
+   * tricount is the number of triangles in "triangles"
+   * material is a material to assign to the mesh
+   * Note! SubMeshes added to an instance of a genmesh will override
+   * the submeshes from the factory (i.e. the submeshes of the factory will
+   * be completely ignored as soon as the instance has submeshes).
+   * This version overrides the parent mixmode.
+   * \deprecated Use AddSubMesh from iGeneralFactoryState instead
+   */
+  CS_DEPRECATED_METHOD_MSG("Use AddSubMesh from iGeneralFactoryState instead")
+  virtual void AddSubMesh (unsigned int *triangles,
+    int tricount,
+    iMaterialWrapper *material, uint mixmode) = 0;
+  /** @} */
+};
 
 /**
  * This interface describes the API for the general mesh factory.
@@ -198,8 +239,15 @@ SCF_VERSION (iGeneralFactoryState, 0, 3, 0);
  * - Genmesh Factory Loader plugin (crystalspace.mesh.loader.factory.genmesh)
  *   
  */
-struct iGeneralFactoryState : public iGeneralMeshCommonState
+struct iGeneralFactoryState : public virtual iGeneralMeshCommonState
 {
+  SCF_INTERFACE (iGeneralFactoryState, 1, 1, 1);
+  
+  /// Set the color to use. Will be added to the lighting values.
+  virtual void SetColor (const csColor& col) = 0;
+  /// Get the color.
+  virtual const csColor& GetColor () const = 0;
+  
   /**
    * Add a vertex. This is one way to fill the vertex and other tables.
    * The other way is to use SetVertexCount() and then fill the tables
@@ -301,6 +349,14 @@ struct iGeneralFactoryState : public iGeneralMeshCommonState
    */
   virtual void GenerateBox (const csBox3& box) = 0;
 
+  /**
+   * Automatically generate a capsule of given length and radius.
+   * \param l Capsule length.
+   * \param r Capsule radius.
+   * \param sides Number of sides.
+   */
+  virtual void GenerateCapsule (float l, float r, uint sides) = 0;
+
   /**
    * Automatically generate a sphere. This will set the apropriate number 
    * of vertices and generate vertices, texels, normals, and triangles.
@@ -348,9 +404,59 @@ struct iGeneralFactoryState : public iGeneralMeshCommonState
    */
   virtual iGenMeshAnimationControlFactory* GetAnimationControlFactory ()
   	const = 0;
-};
+  
+  /**\name SubMesh handling
+   * @{ */
+  /**
+   * Remove all submeshes added to this object
+   */
+  virtual void ClearSubMeshes () = 0;
+  /**
+   * Add a submesh to this object. A submesh is a subset of the mesh triangles
+   * rendered with a certain material. When a mesh has one or more submeshes,
+   * only submeshes are drawn and not original geometry. That means submeshes
+   * should cover all original triangles to avoid holes in the mesh.
+   * \remarks SubMeshes added to an instance of a genmesh will override
+   * the submeshes from the factory (i.e. the submeshes of the factory will
+   * be completely ignored as soon as the instance has submeshes).
+   * \param indices Render buffer holding the geometry for the submesh 
+   *  in usual render buffer formatting, ie for triangles three vertices for
+   *  every triangle.
+   * \param material Material to assign to the submesh.
+   * \param name (Optional) Name to identify the submesh.
+   * \param mixmode (Optional) Mixmode to override the mesh's mixmode.
+   * \return The added submesh, if successful.
+   * \remarks This will change the indices of other submeshes.
+   */
+  virtual iGeneralMeshSubMesh* AddSubMesh (iRenderBuffer* indices, 
+    iMaterialWrapper *material, const char* name, uint mixmode = (uint)~0) = 0;
 
-SCF_VERSION (iGenMeshAnimationControl, 0, 0, 1);
+  /**
+   * Find the index of a submesh. The index can be used with DeleteSubMesh()
+   * and the GetSubMesh...() methods. Returns 0 if the submesh was not found.
+   */
+  virtual iGeneralMeshSubMesh* FindSubMesh (const char* name) const = 0;
+  
+  /**
+   * Delete a submesh.
+   */
+  virtual void DeleteSubMesh (iGeneralMeshSubMesh* mesh) = 0;
+  
+  /// Get the number of submeshes
+  virtual size_t GetSubMeshCount () const = 0;
+
+  /// Get a specific submesh
+  virtual iGeneralMeshSubMesh* GetSubMesh (size_t index) const = 0;
+  /** @} */
+
+  /**
+   * Disable auto-generated normals.
+   * This does not have an effect on the current normals, but only changes the
+   * return value of IsAutoNormals(). However, this affects saving of genmesh
+   * factories, as all vertex normals will be written out explicitly.
+   */
+  virtual void DisableAutoNormals () = 0;
+};
 
 /**
  * Implementing this class allows the creation of classes that control
@@ -370,8 +476,10 @@ SCF_VERSION (iGenMeshAnimationControl, 0, 0, 1);
  * - Genmesh plugin (crystalspace.mesh.object.genmesh)
  *   
  */
-struct iGenMeshAnimationControl : public iBase
+struct iGenMeshAnimationControl : public virtual iBase
 {
+  SCF_INTERFACE(iGenMeshAnimationControl, 2, 0, 0);
+
   /// Returns true if this control animates vertices.
   virtual bool AnimatesVertices () const = 0;
   /// Returns true if this control animates texels.
@@ -381,6 +489,11 @@ struct iGenMeshAnimationControl : public iBase
   /// Returns true if this control animates colors.
   virtual bool AnimatesColors () const = 0;
 
+  /**
+   * General update method
+   */
+  virtual void Update (csTicks current) = 0;
+
   /**
    * Given the factory vertex data, return the animated data.
    * If this control doesn't animate vertices then it will return the
@@ -425,15 +538,13 @@ struct iGenMeshAnimationControl : public iBase
    * changes. The animation control can use this to optimize the animation
    * calculation by caching the animated version of the array and returning
    * that one.
+   * \remarks \a colors may be 0. In this case all color values should be
+   *   assumed to be (0, 0, 0, 1).
    */
   virtual const csColor4* UpdateColors (csTicks current,
   	const csColor4* colors, int num_colors, uint32 version_id) = 0;
 };
 
-SCF_VERSION (iGenMeshAnimationControlFactory, 0, 0, 1);
-
-struct iDocumentNode;
-
 /**
  * This class is a factory for creating animation controls.
  * 
@@ -446,8 +557,10 @@ struct iDocumentNode;
  * Main users of this interface:
  * - Genmesh plugin (crystalspace.mesh.object.genmesh)
  */
-struct iGenMeshAnimationControlFactory : public iBase
+struct iGenMeshAnimationControlFactory : public virtual iBase
 {
+  SCF_INTERFACE(iGenMeshAnimationControlFactory, 2, 0, 0);
+
   /**
    * Create a new animation control.
    */
@@ -467,8 +580,6 @@ struct iGenMeshAnimationControlFactory : public iBase
   virtual const char* Save (iDocumentNode* parent) = 0;
 };
 
-SCF_VERSION (iGenMeshAnimationControlType, 0, 0, 1);
-
 /**
  * This class is the animation control type.
  * 
@@ -483,17 +594,17 @@ SCF_VERSION (iGenMeshAnimationControlType, 0, 0, 1);
  * - Genmesh plugin (crystalspace.mesh.object.genmesh)
  *   
  */
-struct iGenMeshAnimationControlType : public iBase
+struct iGenMeshAnimationControlType : public virtual iBase
 {
+  SCF_INTERFACE(iGenMeshAnimationControlType, 2, 0, 0);
+
   /**
    * Create a new animation control factory.
    */
   virtual csPtr<iGenMeshAnimationControlFactory> CreateAnimationControlFactory
   	() = 0;
-
 };
 
 /** @} */
 
 #endif // __CS_IMESH_GENMESH_H__
-
