@@ -29,6 +29,7 @@
  * \addtogroup meshplugins
  * @{ */
 
+struct iDecalBuilder;
 struct iLight;
 struct iMaterialWrapper;
 struct iMeshWrapper;
@@ -75,13 +76,13 @@ class csVector3;
 #define CS_FACTORY_STATICSHAPE 2
 /** @} */
 
-SCF_VERSION (iMeshObjectDrawCallback, 0, 0, 1);
-
 /**
  * Set a callback which is called just before the object is drawn.
  */
-struct iMeshObjectDrawCallback : public iBase
+struct iMeshObjectDrawCallback : virtual public iBase
 {
+  SCF_INTERFACE (iMeshObjectDrawCallback, 0, 0, 1);
+
   /// Before drawing.
   virtual bool BeforeDrawing (iMeshObject* spr, iRenderView* rview) = 0;
 };
@@ -107,7 +108,7 @@ struct iMeshObjectDrawCallback : public iBase
  */
 struct iMeshObject : public virtual iBase
 {
-  SCF_INTERFACE(iMeshObject, 2,0,0);
+  SCF_INTERFACE(iMeshObject, 2,1,0);
   /**
    * Get the reference to the factory that created this mesh object.
    */
@@ -154,7 +155,8 @@ struct iMeshObject : public virtual iBase
   /**
    * Control animation of this object.
    */
-  virtual void NextFrame (csTicks current_time,const csVector3& pos) = 0;
+  virtual void NextFrame (csTicks current_time,const csVector3& pos,
+    uint currentFrame) = 0;
 
   /**
    * Do a hard transform of this object.
@@ -269,13 +271,21 @@ struct iMeshObject : public virtual iBase
    * it's current state.
    */
   virtual void PositionChild (iMeshObject* child,csTicks current_time) = 0;
+
+  /**
+   * This mesh is being asked to build a decal for its own geometry.  The
+   * mesh is given a position and radius of the decal and must create
+   * geometry through the provided iDecalBuilder.
+   */
+  virtual void BuildDecal(const csVector3* pos, float decalRadius,
+	iDecalBuilder* decalBuilder) = 0;
 };
 
 /**
  * This object is a factory which can generate
  * mesh objects of a certain type. For example, if you want to have
  * multiple sets of sprites from the same sprite template then
- * you should have an instance of iMeshObjectFactory for evey sprite
+ * you should have an instance of iMeshObjectFactory for every sprite
  * template and an instance of iMeshObject for every sprite.
  * 
  * To use a mesh factory in the engine, you have to create a mesh factory
@@ -359,6 +369,23 @@ struct iMeshObjectFactory : public virtual iBase
    * when available as that reduces the amount of redundant memory usage.
    */
   virtual iObjectModel* GetObjectModel () = 0;
+
+  /**
+   * Set the material of the mesh. This only works for single-material
+   * factories. If not supported this function will return false.
+   */
+  virtual bool SetMaterialWrapper (iMaterialWrapper* material) = 0;
+
+  /**
+   * Get the material of the factory. If not supported this will
+   * return 0.
+   */
+  virtual iMaterialWrapper* GetMaterialWrapper () const = 0;
+
+  /// Set mix mode. Note that not all meshes may support this.
+  virtual void SetMixMode (uint mode) = 0;
+  /// Get mix mode.
+  virtual uint GetMixMode () const = 0;
 };
 
 
