@@ -20,11 +20,12 @@
 #ifndef __CS_IUTIL_EVENT_H__
 #define __CS_IUTIL_EVENT_H__
 
-#include "csutil/scf.h"
-#include "csutil/csunicode.h"
-#include "csutil/strset.h"
-#include "csutil/eventnames.h"
+#include "cssysdef.h"
 
+#include "csutil/scf_interface.h"
+#include "csutil/csunicode.h"
+#include "csutil/ref.h"
+#include "iutil/eventnames.h"
 #include "iutil/evdefs.h"
 
 
@@ -140,30 +141,6 @@ struct csKeyEventData
   csKeyCharType charType;
 };
 
-/**
- * Constants for mouse buttons.
- * Note: the possibly occuring values are not limited to those below, e.g.
- * maybe some day 6-button mice are available...
- */
-enum csMouseButton
-{
-  /// ID to designate the absence of an acting mouse button.
-  csmbNone = -1,
-  /// Left button
-  csmbLeft = 0,
-  /// Right button
-  csmbRight = 1,
-  /// Middle button
-  csmbMiddle = 2,
-  /// Wheel was scrolled up
-  csmbWheelUp = 3,
-  /// Wheel was scrolled up
-  csmbWheelDown = 4,
-  /// Thumb button 1 (e.g. on 5-button mice)
-  csmbExtra1 = 5,
-  /// Thumb button 2 (e.g. on 5-button mice)
-  csmbExtra2 = 6
-};
 
 /**
  * Structure that collects the data a mouse event carries.
@@ -498,7 +475,8 @@ struct iEventOutlet : public virtual iBase
    * Control/Shift/Alt for English characters. However, in general, it is
    * best if the entity posting the event can provide both codes.
    */
-  virtual void Key (utf32_char codeRaw, utf32_char codeCooked, bool iDown) = 0;
+  virtual void Key (utf32_char codeRaw, utf32_char codeCooked, bool iDown,
+      bool autorep = false) = 0;
 
   /**
    * Put a mouse event into event queue.<p>
