@@ -20,47 +20,34 @@
 #define __CS_IUTIL_OBJREG_H__
 
 /**\file
+ * Object registry interface
  */
  
 /**
  * \addtogroup scf
  * @{ */
 
-#include "csutil/scf.h"
 #include "csutil/ref.h"
-
-#define CS_QUERY_REGISTRY_TAG(Reg, Tag)       \
-  csPtr<iBase> ((iBase*) (CS_IMPLICIT_PTR_CAST(iObjectRegistry, Reg)->Get (Tag)))
-#define CS_QUERY_REGISTRY(Reg,Interface)      \
-  csPtr<Interface> ((Interface*)(CS_IMPLICIT_PTR_CAST(iObjectRegistry, Reg)->Get (#Interface, \
-    scfInterface<Interface>::GetID(), scfInterface<Interface>::GetVersion())))
-#define CS_QUERY_REGISTRY_TAG_INTERFACE(Reg, Tag, Interface)  \
-  csPtr<Interface> ((Interface*)(CS_IMPLICIT_PTR_CAST(iObjectRegistry, Reg)->Get (Tag,    \
-    scfInterface<Interface>::GetID(), scfInterface<Interface>::GetVersion())))
+#include "csutil/scf_interface.h"
 
 struct iObjectRegistryIterator;
 
-SCF_VERSION (iObjectRegistry, 0, 0, 4);
-
 /**
  * This interface serves as a registry of other objects.
  * The object registry is thread-safe.
- * <p>
+ *
  * Main creators of instances implementing this interface:
- *   <ul>
- *   <li>csInitializer::CreateObjectRegistry()
- *   </ul>
+ * - csInitializer::CreateObjectRegistry()
+ * 
  * Main ways to get pointers to this interface:
- *   <ul>
- *   <li>Stored by application.
- *   </ul>
+ * - Stored by application.
+ * 
  * Main users of this interface:
- *   <ul>
- *   <li>Everything.
- *   </ul>
+ * - Everything.
  */
-struct iObjectRegistry : public iBase
+struct iObjectRegistry : public virtual iBase
 {
+  SCF_INTERFACE(iObjectRegistry, 2,0,0);
   // Allow implicit casts through static function.
   CS_IMPLEMENT_IMPLICIT_PTR_CAST (iObjectRegistry);
 
@@ -135,14 +122,14 @@ struct iObjectRegistry : public iBase
   virtual csPtr<iObjectRegistryIterator> Get () = 0;
 };
 
-SCF_VERSION (iObjectRegistryIterator, 0, 1, 0);
 
 /**
  * Use an instance of this class to iterate over objects in the object
  * registry.
  */
-struct iObjectRegistryIterator : public iBase
+struct iObjectRegistryIterator : public virtual iBase
 {
+  SCF_INTERFACE(iObjectRegistryIterator, 2,0,0);
   /**
    * Restart the iterator. Returns false if there are no ellements
    * in it.
@@ -167,5 +154,95 @@ struct iObjectRegistryIterator : public iBase
 
 /** @} */
 
+/**
+ * Query an interface from the registry. The tag is the name of the interface.
+ */
+template<class Interface>
+inline csPtr<Interface> csQueryRegistry (iObjectRegistry *Reg)
+{ 
+  iBase *base = Reg->Get (scfInterfaceTraits<Interface>::GetName (),
+              scfInterfaceTraits<Interface>::GetID (),
+              scfInterfaceTraits<Interface>::GetVersion ());
+
+  if (base == 0) return csPtr<Interface> (0);
+
+  Interface *x = (Interface*)base->QueryInterface (
+    scfInterfaceTraits<Interface>::GetID (),
+    scfInterfaceTraits<Interface>::GetVersion ());
+
+  if (x) base->DecRef (); //release our base interface
+  return csPtr<Interface> (x);
+}  
+  
+/**
+ * Query an object from the registry, with a tag specified by the user.
+ */
+inline csPtr<iBase> csQueryRegistryTag (iObjectRegistry *Reg, const char* Tag)
+{  
+  return csPtr<iBase> (Reg->Get (Tag));
+}  
+   
+/**
+ * Query an interface from the registry, with a tag specified by the user.
+ */
+template<class Interface>
+inline csPtr<Interface> csQueryRegistryTagInterface (
+  iObjectRegistry *Reg, const char* Tag)
+{  
+  iBase *base = Reg->Get (Tag,
+              scfInterfaceTraits<Interface>::GetID (),
+              scfInterfaceTraits<Interface>::GetVersion ());
+
+  if (base == 0) return csPtr<Interface> (0);
+
+  Interface *x = (Interface*)base->QueryInterface (
+    scfInterfaceTraits<Interface>::GetID (),
+    scfInterfaceTraits<Interface>::GetVersion ());
+
+  if (x) base->DecRef (); //release our base interface
+  return csPtr<Interface> (x);
+}  
+
+inline CS_DEPRECATED_METHOD_MSG ("CS_QUERY_REGISTRY_TAG macro is deprecated")
+csPtr<iBase> CS_QUERY_REGISTRY_TAG_is_deprecated (iObjectRegistry *Reg, 
+  const char* Tag)
+{
+  return csQueryRegistryTag(Reg, Tag);
+}
+/**
+ * \deprecated Compatibility macro
+ * \sa csQueryRegistryTag
+ */
+#define CS_QUERY_REGISTRY_TAG(Reg, Tag) \
+  (CS_QUERY_REGISTRY_TAG_is_deprecated (Reg, Tag))
+
+template<class Interface>
+inline CS_DEPRECATED_METHOD_MSG ("CS_QUERY_REGISTRY macro is deprecated")
+csPtr<Interface> CS_QUERY_REGISTRY_is_deprecated (iObjectRegistry *Reg)
+{
+  return csQueryRegistry<Interface> (Reg);
+}
+/**
+ * \deprecated Compatibility macro
+ * \sa csQueryRegistry
+ */
+#define CS_QUERY_REGISTRY(Reg,Interface) \
+  (CS_QUERY_REGISTRY_is_deprecated<Interface> (Reg))
+
+template<class Interface>
+inline CS_DEPRECATED_METHOD_MSG (
+  "CS_QUERY_REGISTRY_TAG_INTERFACE macro is deprecated")
+csPtr<Interface> CS_QUERY_REGISTRY_TAG_INTERFACE_is_deprecated (
+  iObjectRegistry *Reg, const char* Tag)
+{
+  return csQueryRegistryTagInterface<Interface>(Reg, Tag);
+}
+/**
+ * \deprecated Compatibility macro
+ * \sa csQueryRegistryTagInterface
+ */
+#define CS_QUERY_REGISTRY_TAG_INTERFACE(Reg, Tag, Interface) \
+  (CS_QUERY_REGISTRY_TAG_INTERFACE_is_deprecated<Interface>(Reg, Tag))
+
 #endif // __CS_IUTIL_OBJREG_H__
 
