@@ -31,7 +31,7 @@
 /// This is a SCF-compatible interface for csString.
 struct iString : public virtual iBase
 {
-  SCF_INTERFACE(iString, 2,0,0);
+  SCF_INTERFACE(iString, 2,1,0);
   /**
    * Advise the string that it should allocate enough space to hold up to
    * NewSize characters.
@@ -85,17 +85,6 @@ struct iString : public virtual iBase
    */
   virtual void ShrinkBestFit () = 0;
 
-  /**
-   * Set string buffer capacity to hold exactly the current content.
-   * \remarks If the string length is greater than zero, then the buffer's
-   *   capacity will be adjusted to exactly that size.  If the string length is
-   *   zero, then the implementation may shrink the allocation so that it only
-   *   holds the implicit null terminator, or it may free the string's memory
-   *   completely.
-   * \deprecated Use ShrinkBestFit() instead.
-   */
-  CS_DEPRECATED_METHOD virtual void Reclaim () = 0;
-
   /**
    * Clear the string (so that it contains only a null terminator).
    * \remarks This is typically shorthand for Truncate(0), but more idiomatic
@@ -109,7 +98,8 @@ struct iString : public virtual iBase
    *   in terms of human language.
    * \deprecated Use Empty() instead.
    */
-  /* CS_DEPRECATED_METHOD */ virtual void Clear () = 0;
+  CS_DEPRECATED_METHOD_MSG("Use Empty() instead")
+  virtual void Clear () = 0;
 
   /// Get a copy of this string
   virtual csRef<iString> Clone () const = 0;
@@ -163,6 +153,13 @@ struct iString : public virtual iBase
   /// Get the n'th character.
   virtual char GetAt (size_t n) const = 0;
 
+  /**
+   * Delete a range of characters from the string.
+   * \param Pos Beginning of range to be deleted (zero-based).
+   * \param Count Number of characters to delete.
+   */
+  virtual void DeleteAt (size_t Pos, size_t Count = 1) = 0;
+
   /**
    * Insert another string into this one.
    * \param Pos Position at which to insert the other string (zero-based).
@@ -295,6 +292,22 @@ struct iString : public virtual iBase
    */
   virtual bool CompareNoCase (const iString* Str) const = 0;
 
+  /**
+   * Check if this string starts with another one.
+   * \param Str Other string.
+   * \param ignore_case Causes the comparison to be case insensitive if true.
+   * \return True if they are equal up to the length of Str; false if not.
+   */
+  virtual bool StartsWith (const iString* Str, bool ignore_case = false) const = 0;
+
+  /**
+   * Check if this string starts with another null-terminated C-string.
+   * \param Str Other string.
+   * \param ignore_case Causes the comparison to be case insensitive if true.
+   * \return True if they are equal up to the length of Str; false if not.
+   */
+  virtual bool StartsWith (const char* Str, bool ignore_case = false) const = 0;
+
   /// Append another string to this one.
   virtual void operator += (const iString& iStr) = 0;
 
