@@ -26,8 +26,6 @@
  * @{ */
 #include "csutil/scf_interface.h"
 
-#include "iutil/strset.h"
-
 /**
  * An identifier for a string. This identifier is equivalent to the contents
  * of a string: If two strings have the same content, they have get the same
@@ -54,8 +52,9 @@ csStringID const csInvalidStringID = (csStringID) ~0;
  *
  * \code
  * iObjectRegistry* object_reg = ...;
- * csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE(
- *   object_reg, "crystalspace.shared.stringset", iStringSet);
+ * csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+ *   object_reg, "crystalspace.shared.stringset");
+ * csStringID myid = strings->Request ("crystalspace.mygame.mystring");
  * \endcode
  *
  * \sa csStringHash
@@ -113,7 +112,8 @@ struct iStringSet : public virtual iBase
    * Remove all stored strings.
    * \deprecated Use Empty() instead.
    */
-  /*CS_DEPRECATED_METHOD*/ virtual void Clear() = 0;
+  /*CS_DEPRECATED_METHOD_MSG("Use Empty() instead.")*/
+  virtual void Clear() = 0;
 
   /// Get the number of elements in the hash.
   virtual size_t GetSize () const = 0;
