@@ -241,7 +241,7 @@ struct iVFS : public virtual iBase
   virtual csPtr<iDataBuffer> ExpandPath (
     const char *Path, bool IsDir = false) const = 0;
 
-  /// Check whenever a file exists
+  /// Check whether a file exists
   virtual bool Exists (const char *Path) const = 0;
 
   /**
@@ -269,10 +269,10 @@ struct iVFS : public virtual iBase
    * \param FileName VFS path of the file to be read.
    * \param nullterm Null-terminate the returned buffer.
    * \return An iDataBuffer containing the file contents if the file was opened
-   *  and read successfully, otherwise an invalidated iFile.  Use
+   *  and read successfully, otherwise a null reference.  Use
    *  csRef<>::IsValid() to check validity.
    * \remarks Null-termination might have a performance penalty (dependent on
-   *  where the file is stored.) Use only when needed.
+   *  where the file is stored). Use only when needed.
    */
   virtual csPtr<iDataBuffer> ReadFile (const char *FileName,
     bool nullterm = true) = 0;
@@ -298,6 +298,21 @@ struct iVFS : public virtual iBase
    */
   virtual bool Sync () = 0;
 
+  /**
+   * Create or add a symbolic link within the VFS 
+   *  (works like unix 'ln -s' command)
+   * If the link already exists, then the target will be added to the link
+   * At the moment just remounts it at `Target'.
+   * \param Target The target that the link will point to
+   * \param Link The path of the link within the VFS, if this is 0 then the 
+   *  link will be created in the current directory with the same name as 
+   *  the target
+   * \param priority Currently unused
+   * \return True if successful, else false.
+   */
+  virtual bool SymbolicLink(const char *Target, const char *Link = 0, 
+    int priority = 0) = 0;
+
   /**
    * Mount an VFS path on a "real-world-filesystem" path.
    * \param VirtualPath The location in the virtual filesystem in which to
@@ -319,7 +334,7 @@ struct iVFS : public virtual iBase
    *   in which case, the physical locations appear as a conglomerate in the
    *   virtual filesystem.  The RealPath argument allows unmounting of just a
    *   single location represented by the given VirtualPath.  If RealPath is
-   *   the null pointer, then all pysical locations represented by VirtualPath
+   *   the null pointer, then all physical locations represented by VirtualPath
    *   are umounted.
    * \return True if the unmount succeeded, else false.
    */
