@@ -44,6 +44,7 @@ struct iRenderBuffer;
 struct iRendererLightmap;
 struct iShader;
 struct iShaderVariableContext;
+struct iShaderVarStack;
 struct iTextureHandle;
 struct iTextureManager;
 
@@ -137,7 +138,7 @@ enum csZBufMode
 };
 
 // @@@ Keep in sync with values below
-// @@@ Document me better!
+// \todo Document me!
 #define CS_VATTRIB_SPECIFIC_FIRST    0
 #define CS_VATTRIB_SPECIFIC_LAST    15
 #define CS_VATTRIB_GENERIC_FIRST   100
@@ -153,6 +154,10 @@ enum csZBufMode
  */
 enum csVertexAttrib
 {
+  /**
+   * An attribute is valid, but unused by a shader program and can be 
+   * discarded */
+  CS_VATTRIB_UNUSED	      = -2,
   /// Invalid attribute
   CS_VATTRIB_INVALID	      = -1,
   /// Position vertex attribute
@@ -335,7 +340,7 @@ enum
 #define CS_FX_ADD \
     (CS_MIXMODE_BLEND(ONE, ONE) | CS_MIXMODE_ALPHATEST_DISABLE)
 /** 
- * Alpha blending. Formula: <tt>=(1-srcAlpha)*SRC + srcAlpha*DST</tt>
+ * Alpha blending. Formula: <tt>=srcAlpha*SRC + (1-srcAlpha)*DST</tt>
  * \remarks Usually used with a non-zero alpha part.
  *  \see CS_FX_MASK_ALPHA, \see CS_FX_SETALPHA
  */
@@ -500,14 +505,10 @@ enum G3D_RENDERSTATEOPTION
 /// Information about 3d renderer capabilities.
 struct csGraphics3DCaps
 {
-  bool CanClip;
+  /// Minimum texture dimensions
   int minTexHeight, minTexWidth;
+  /// Maximum texture dimensions
   int maxTexHeight, maxTexWidth;
-  bool NeedsPO2Maps;
-  int MaxAspectRatio;
-
-  // The following caps are only used by NR
-
   /**
    * Whether point sprites are supported. If \a true, geometry of the type
    * CS_MESHTYPE_POINT_SPRITES can be drawn.
@@ -581,6 +582,36 @@ enum csSimpleMeshFlags
   csSimpleMeshScreenspace = 0x01
 };
 
+/**
+ * Flags to inform the renderer about properties of a portal when
+ * calling OpenPortal(). 
+ */
+enum csOpenPortalFlags
+{
+  /**
+   * If this flag is set then renderer must do a Z-fill after rendering
+   * the portal contents. This is mainly useful for floating portals
+   * where it is possible that there is geometry in the same sector
+   * that will be rendered behind the portal (and does could accidently
+   * get written in the portal sector because the Z-buffer cannot
+   * be trusted).
+   */
+  CS_OPENPORTAL_ZFILL = 0x00000004,
+  /**
+   * If this flag is set then this portal mirrors space (changes order
+   * of the vertices of polygons).
+   */
+  CS_OPENPORTAL_MIRROR = 0x00000010,
+  /**
+   * If this flag is used then the portal must use possible available
+   * stencil buffer on the hardware to do good clipping. This flag should
+   * be used if you have a portal that is not at the boundary of the sector
+   * and that can be covered (or itself covers) other objects. It is usually
+   * used in combination with CS_OPENPORTAL_ZFILL.
+   */
+  CS_OPENPORTAL_FLOAT = 0x00000040
+};
+
 /**
  * A simple render mesh.
  */
@@ -669,7 +700,7 @@ struct csSimpleRenderMesh
  */
 struct iGraphics3D : public virtual iBase
 {
-  SCF_INTERFACE(iGraphics3D, 2, 0, 0);
+  SCF_INTERFACE(iGraphics3D, 2, 1, 1);
   
   /// Open the 3D graphics display.
   virtual bool Open () = 0;
@@ -738,13 +769,16 @@ struct iGraphics3D : public virtual iBase
    * will overwrite the screen. So you should only do this BEFORE you
    * start rendering your frame.
    * <p>
-   * If 'persistent' is true then the current contents of the texture
+   * \param persistent If this is true then the current contents of the texture
    * will be copied on screen before drawing occurs (in the first
    * call to BeginDraw). Otherwise it is assumed that you fully render
    * the texture.
+   * \param subtexture this specifies the subtexture index if the texture
+   * is a cubemap. It is in the range 0 to 5.
    */
   virtual void SetRenderTarget (iTextureHandle* handle,
-  	bool persistent = false) = 0;
+	bool persistent = false,
+	int subtexture = 0) = 0;
 
   /**
    * Get the current render target (0 for screen).
@@ -767,7 +801,7 @@ struct iGraphics3D : public virtual iBase
   /// Drawroutine. Only way to draw stuff
   virtual void DrawMesh (const csCoreRenderMesh* mymesh,
                          const csRenderMeshModes& modes,
-                         const csArray<csShaderVariable*> &stacks) = 0;
+                         const iShaderVarStack* stacks) = 0;
   /**
   * Draw a csSimpleRenderMesh on the screen.
   * Simple render meshes are intended for cases where setting up
@@ -856,6 +890,7 @@ struct iGraphics3D : public virtual iBase
 
   /**
    * Set near clip plane.
+   * The plane is in camera space.
    */
   virtual void SetNearPlane (const csPlane3& pl) = 0;
 
@@ -947,6 +982,32 @@ struct iGraphics3D : public virtual iBase
    * \remarks 'this' space is world space, 'other' space is camera space
    */
   virtual void SetWorldToCamera (const csReversibleTransform& w2c) = 0;
+
+  /**
+   * Perform a system specific exension.<p>
+   * The command is a string; any arguments may follow.
+   * There is no way to guarantee the uniquiness of
+   * commands, so please try to use descriptive command names rather
+   * than "a", "b" and so on...
+   */
+  virtual bool PerformExtension (char const* command, ...) = 0;
+
+  /**
+   * Perform a system specific exension.<p>
+   * Just like PerformExtension() except that the command arguments are passed
+   * as a `va_list'.
+   */
+  virtual bool PerformExtensionV (char const* command, va_list) = 0;
+  
+  /**
+   * Get the current world to camera transform.
+   * \remarks 'this' space is world space, 'other' space is camera space
+   */
+  virtual const csReversibleTransform& GetWorldToCamera () = 0;
+  /**
+   * Get the current drawflags
+   */
+  virtual int GetCurrentDrawFlags() const = 0;
 };
 
 /** @} */
