@@ -56,6 +56,7 @@ enum csRenderBufferType
 };
 
 /// Type of components
+// NOTE: this is stored on disk by cssynldr! *Modify with care!*
 enum csRenderBufferComponentType
 {
   CS_BUFCOMP_BYTE = 0,
@@ -75,13 +76,31 @@ enum csRenderBufferComponentType
  */
 enum csRenderBufferLockType
 {
-  CS_BUF_LOCK_NOLOCK = 0,
   /// Lock used for reading only from the buffer
-  CS_BUF_LOCK_READ,
-  /// Just get a point to the buffer, nothing special
+  CS_BUF_LOCK_READ = 1,
+  /// Get a (writeable) pointer to the buffer
   CS_BUF_LOCK_NORMAL
 };
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
+namespace CS
+{
+  namespace Deprecated
+  {
+    struct CS_DEPRECATED_TYPE_MSG("You shouldn't use CS_BUF_LOCK_NOLOCK in "
+      "the first place")
+      CS_BUF_LOCK_NOLOCK
+    {
+      static const uint value = 0;
+    };
+  }
+}
+
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
+#define CS_BUF_LOCK_NOLOCK	CS::Deprecated::CS_BUF_LOCK_NOLOCK::value
+
 /**
  * Sizes of individual buffer components in bytes.
  */
@@ -94,6 +113,21 @@ static const size_t csRenderBufferComponentSizes[CS_BUFCOMP_TYPECOUNT] =
   sizeof (double)
 };
 
+struct iRenderBuffer;
+
+/**
+ * Callback function used upon destruction of render buffer. Used by renderer
+ * to properly uncache render buffer.
+ */
+struct iRenderBufferCallback : public virtual iBase
+{
+  SCF_INTERFACE (iRenderBufferCallback, 1,0,0);
+  /**
+   * Called when render buffer is about to be destroyed and removed
+   */
+  virtual void RenderBufferDestroyed (iRenderBuffer* buffer) = 0;
+};
+
 /**
  * This is a general buffer.
  *
@@ -106,13 +140,14 @@ static const size_t csRenderBufferComponentSizes[CS_BUFCOMP_TYPECOUNT] =
  */
 struct iRenderBuffer : public virtual iBase
 {
-  SCF_INTERFACE (iRenderBuffer, 2, 0, 0);
+  SCF_INTERFACE (iRenderBuffer, 2, 1, 1);
 
   /**
-   * Lock the buffer to allow writing and return a pointer to the first 
-   * element.
-   * The pointer will be (void*)-1 if there was some error.
+   * Lock the buffer to allow reading or writing to it directly.
    * \param lockType The type of lock desired.
+   * \return A pointer to the first element or (void*)-1 if there was some 
+   *   error.
+   * \remarks After the data was used as desired Release() \b must be called.
    */
   virtual void* Lock(csRenderBufferLockType lockType) = 0;
 
@@ -124,8 +159,9 @@ struct iRenderBuffer : public virtual iBase
 
   /**
    * Copy data to the render buffer.
-   * \remarks Does not work with interleaved buffer, copy to master buffer
-   *  instead.
+   * \remarks 
+   *   Do not call on a locked buffer.
+   *   Does not work with interleaved buffer, copy to master buffer instead.
    */
   virtual void CopyInto (const void *data, size_t elementCount,
     size_t elemOffset = 0) = 0;
@@ -171,6 +207,21 @@ struct iRenderBuffer : public virtual iBase
 
   /// Number of elements in a buffer.
   virtual size_t GetElementCount() const = 0;
+
+  /// Set callback object to use
+  virtual void SetCallback (iRenderBufferCallback* cb) = 0;
+
+  /**
+   * Set the buffer data. This changes the internal pointer to the buffer data 
+   * to \a buffer instead. It will also be returned by Lock(). It is the
+   * responsibility of the caller to ensure that the memory pointed to by
+   * \a data is valid for as long as the render buffer is used.
+   * \remarks 
+   *   Do not call on a locked buffer.
+   *   Does not work with interleaved buffer, set data on master buffer
+   *   instead.
+   */
+  virtual void SetData (const void *data) = 0;
 };
 
 /**
@@ -234,9 +285,9 @@ enum csRenderBufferNameMask
   CS_BUFFER_POSITION_MASK = 1 << CS_BUFFER_POSITION,
   /// Normals
   CS_BUFFER_NORMAL_MASK = 1 << CS_BUFFER_NORMAL,
-  /// Primary color multiplied with lighting
-  CS_BUFFER_COLOR_MASK = 1 << CS_BUFFER_COLOR,
   /// Primary color 
+  CS_BUFFER_COLOR_MASK = 1 << CS_BUFFER_COLOR,
+  /// Primary color multiplied with lighting
   CS_BUFFER_COLOR_LIGHTING_MASK = 1 << CS_BUFFER_COLOR_UNLIT,
   /// Texture coordinate 0
   CS_BUFFER_TEXCOORD0_MASK = 1 << CS_BUFFER_TEXCOORD0,
@@ -302,6 +353,17 @@ public:
     return buffers[bufferName];
   }
 
+  /**
+   * Get buffer by name.
+   * This version won't call accessor
+   */
+  iRenderBuffer* GetRenderBufferNoAccessor (csRenderBufferName bufferName)
+  {
+    if (bufferName < CS_BUFFER_INDEX) return 0;
+    
+    return buffers[bufferName];
+  }
+
   /**
    * Set renderbuffer.
    */
