@@ -27,6 +27,8 @@
 #include "csgeom/math3d.h"
 #include "igeom/polymesh.h"
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 void csPolygonMeshTools::CalculateNormals (iPolygonMesh* mesh,
   csVector3* normals)
 {
@@ -422,7 +424,7 @@ struct AdjacencyCounter
   }
 };
 
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csComparator<PolyEdge, PolyEdge>
 {
 public:
@@ -435,7 +437,7 @@ public:
   }
 };
 
-CS_SPECIALIZE_TEMPLATE
+template<>
 class csHashComputer<PolyEdge>
 {
 public:
@@ -563,7 +565,7 @@ bool csPolygonMeshTools::IsMeshConvex (iPolygonMesh* polyMesh)
     	verts);
     size_t j;
     const csArray<int>& ap = adjacent_polygons[i];
-    for (j = 0 ; j < ap.Length () ; j++)
+    for (j = 0 ; j < ap.GetSize () ; j++)
     {
       float cl = plane.Classify (poly_centers[ap[j]]);
       if (cl > SMALL_EPSILON)
@@ -894,3 +896,38 @@ bool csPolygonMeshTools::BoxInClosedMesh (const csBox3& box,
   return true;
 }
 
+csArray<csArray<int> > *csPolygonMeshTools::CalculateVertexConnections (iPolygonMesh* mesh)
+{
+  int i,j,k;
+  int vert_count = mesh->GetVertexCount ();
+  csArray<csArray<int> > *link_array = new csArray<csArray<int> >(vert_count);
+
+  // Initialize array
+  csArray<int> newarray;
+  for (i = 0 ; i < vert_count ; i++)
+    link_array->Put(i,newarray);
+
+  // Fill connection information.
+  int poly_count = mesh->GetPolygonCount ();
+  csMeshedPolygon* poly = mesh->GetPolygons ();
+  for (i = 0 ; i < poly_count ; i++, poly++)
+  {
+    int* vi = poly->vertices;
+    for (j = 0 ; j < poly->num_vertices ; j++)
+    {
+      int vt1 = vi[j];
+      csArray<int> *vset = &link_array->Get(vt1);
+      for (k = 0 ; k < poly->num_vertices ; k++)
+      {
+        int vt2 = vi[k];
+        if (vt1 != vt2)
+        {
+          vset->PushSmart(vt2);
+        }
+      }
+    }
+  }
+  return link_array;
+}
+
+#include "csutil/win32/msvc_deprecated_warn_on.h"
