@@ -16,15 +16,12 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <math.h>
 #include "cssysdef.h"
+#include <math.h>
 #include "csgeom/polymesh.h"
 #include "csgeom/pmtools.h"
 
-
-SCF_IMPLEMENT_IBASE (csPolygonMesh)
-  SCF_IMPLEMENTS_INTERFACE (iPolygonMesh)
-SCF_IMPLEMENT_IBASE_END
+#include "csutil/win32/msvc_deprecated_warn_off.h"
 
 void csPolygonMesh::Triangulate ()
 {
@@ -32,10 +29,6 @@ void csPolygonMesh::Triangulate ()
   csPolygonMeshTools::Triangulate (this, triangles, triangle_count);
 }
 
-SCF_IMPLEMENT_IBASE (csPolygonMeshBox)
-  SCF_IMPLEMENTS_INTERFACE (iPolygonMesh)
-SCF_IMPLEMENT_IBASE_END
-
 csTriangle* csPolygonMeshBox::GetTriangles ()
 {
   if (triangles) return triangles;
@@ -44,3 +37,4 @@ csTriangle* csPolygonMeshBox::GetTriangles ()
   return triangles;
 }
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
