@@ -15,14 +15,16 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#include <math.h>
+
 #include "cssysdef.h"
+#include <math.h>
 #include "csgeom/math3d.h"
+#include "csgeom/matrix3.h"
 #include "csgeom/textrans.h"
-#include "qsqrt.h"
+#include "csqsqrt.h"
 
 //---------------------------------------------------------------------------
-void csTextureTrans::compute_texture_space (
+bool csTextureTrans::compute_texture_space (
   csMatrix3 &m,
   csVector3 &v,
   const csVector3 &v_orig,
@@ -32,7 +34,7 @@ void csTextureTrans::compute_texture_space (
   float B,
   float C)
 {
-  compute_texture_space (
+  return compute_texture_space (
     m,
     v,
     v_orig.x,
@@ -47,7 +49,7 @@ void csTextureTrans::compute_texture_space (
     C);
 }
 
-void csTextureTrans::compute_texture_space (
+bool csTextureTrans::compute_texture_space (
   csMatrix3 &m,
   csVector3 &v,
   float xo,
@@ -61,7 +63,7 @@ void csTextureTrans::compute_texture_space (
   float B,
   float C)
 {
-  float invl1 = qisqrt (
+  float invl1 = csQisqrt (
       (xo - x1) * (xo - x1) + (yo - y1) * (yo - y1) + (zo - z1) * (zo - z1));
   x1 = (x1 - xo) * invl1;
   y1 = (y1 - yo) * invl1;
@@ -76,7 +78,7 @@ void csTextureTrans::compute_texture_space (
   y2 = z1 * A - x1 * C;
   z2 = x1 * B - y1 * A;
 
-  float invl2 = qisqrt (x2 * x2 + y2 * y2 + z2 * z2);
+  float invl2 = csQisqrt (x2 * x2 + y2 * y2 + z2 * z2);
 
   x1 *= len1;
   y1 *= len1;
@@ -85,16 +87,16 @@ void csTextureTrans::compute_texture_space (
   y2 = y2 * len1 * invl2;
   z2 = z2 * len1 * invl2;
 
-  float invl3 = qisqrt (A * A + B * B + C * C);
+  float invl3 = csQisqrt (A * A + B * B + C * C);
   float a, b, c;
   a = A * len1 * invl3;
   b = B * len1 * invl3;
   c = C * len1 * invl3;
 
-  compute_texture_space (m, v, xo, yo, zo, x1, y1, z1, x2, y2, z2, a, b, c);
+  return compute_texture_space (m, v, xo, yo, zo, x1, y1, z1, x2, y2, z2, a, b, c);
 }
 
-void csTextureTrans::compute_texture_space (
+bool csTextureTrans::compute_texture_space (
   csMatrix3 &m,
   csVector3 &v,
   const csVector3 &v_orig,
@@ -104,17 +106,17 @@ void csTextureTrans::compute_texture_space (
   float len2)
 {
   float d = csSquaredDist::PointPoint (v_orig, v1);
-  float invl1 = qisqrt (d);
+  float invl1 = csQisqrt (d);
 
   //if (ABS (l1) < SMALL_EPSILON) l1 = SMALL_EPSILON;
   d = csSquaredDist::PointPoint  (v_orig, v2);
-  float invl2 = (d) ? qisqrt (d) : 0;
+  float invl2 = (d) ? csQisqrt (d) : 0;
 
   //if (ABS (l2) < SMALL_EPSILON) l2 = SMALL_EPSILON;
   csVector3 v_u = (v1 - v_orig) * len1 * invl1;
   csVector3 v_v = (v2 - v_orig) * len2 * invl2;
   csVector3 v_w = v_u % v_v;
-  compute_texture_space (
+  return compute_texture_space (
     m,
     v,
     v_orig.x,
@@ -131,7 +133,7 @@ void csTextureTrans::compute_texture_space (
     v_w.z);
 }
 
-void csTextureTrans::compute_texture_space (
+bool csTextureTrans::compute_texture_space (
   csMatrix3 &m,
   csVector3 &v,
   const csVector3 &v_orig,
@@ -139,7 +141,7 @@ void csTextureTrans::compute_texture_space (
   const csVector3 &v_v)
 {
   csVector3 v_w = v_u % v_v;
-  compute_texture_space (
+  return compute_texture_space (
     m,
     v,
     v_orig.x,
@@ -156,7 +158,7 @@ void csTextureTrans::compute_texture_space (
     v_w.z);
 }
 
-void csTextureTrans::compute_texture_space (
+bool csTextureTrans::compute_texture_space (
   csMatrix3 &m,
   csVector3 &v,
   float xo,
@@ -182,14 +184,19 @@ void csTextureTrans::compute_texture_space (
   m.m32 = zv;
   m.m33 = zw;
 
+  v.x = xo;
+  v.y = yo;
+  v.z = zo;
+
   float det = m.Determinant ();
   if (ABS (det) < SMALL_EPSILON)
   {
     // @@@ Warning?
+    m.Identity ();
+    return false;
   }
   else
     m.Invert ();
-  v.x = xo;
-  v.y = yo;
-  v.z = zo;
+  
+  return true;
 }
