@@ -31,6 +31,7 @@
 #include "csgeom/math3d.h"
 #include "csgeom/vector3.h"
 #include "csgeom/segment.h"
+#include "csgeom/math.h"
 
 #define FABS(x) (fabsf(x))        /* implement as is fastest on your machine */
 
@@ -67,7 +68,7 @@
     e=Ax*Cy-Ay*Cx;                                    \
     if(f>0)                                           \
     {                                                 \
-      if(e>=0 && e<=f) return true;                   \
+    if(e>=0 && e<=f) return true;                   \
     }                                                 \
     else                                              \
     {                                                 \
@@ -421,6 +422,7 @@ static inline bool compute_intervals_isectline(const csVector3 tri[3],
   }
 #endif
 
+#if 0
 /**
  * Tests if two triangles intersect and compute the line of intersection
  * (if they are not coplanar).
@@ -572,4 +574,355 @@ bool csIntersect3::TriangleTriangle(const csVector3 tri1[3],
   return true;
 }
 
+#endif
 
+//--- VERSION WHICH COMPUTES INTERSECTION LINE
+
+enum SIGN_CLASS
+{
+  D0_DIFFERENT = 0,
+  D1_DIFFERENT = 1,
+  D2_DIFFERENT = 2,
+  
+  D_ALL_SAME = 3,
+  D_COPLANAR = 4
+};
+
+/*
+ Get which of the three distances that have a different sign than the others.
+ Returns -1 if all have same sign, and are not 0
+ */
+static inline SIGN_CLASS GetSignClassification (float d0, float d1, float d2)
+{
+  const float d0d1 = d0*d1;
+  const float d0d2 = d0*d2;
+  
+  // All distances are on same side of plane
+  if (d0d1 > 0.0f && d0d2 > 0.0f)
+    return D_ALL_SAME;
+  
+  if (d0d1 > 0.0f)
+  {
+    // d0 and d1 same sign, d2 must be different
+    return D2_DIFFERENT;
+  }
+  else if (d0d2 > 0.0f)
+  {
+    // d0 and d2 same sign, d1 must be different
+    return D1_DIFFERENT;
+  }
+  else if (d1*d2 > 0.0f || d0 != 0.0f)
+  {
+    // d1 and d2 same sign, d0 different
+    // d1 or d2 0, d0 is not, d0 different
+    return D0_DIFFERENT;
+  }
+  else if (d1 != 0.0f)
+  {
+    // d0 or d2 is 0, d1 is not, d1 different
+    return D1_DIFFERENT;
+  }
+  else if (d2 != 0.0f)
+  {
+    // d0 or d1 is 0, d2 is not, d2 different
+    return D2_DIFFERENT;
+  }
+
+  return D_COPLANAR;
+}
+
+// Compute intersection between two edges [0]-[1] [0]-[2] and the plane.
+// This assumes that [0] is the vertex with different d-sign
+static inline void ComputerInterval2DProj (const csVector3 tri[3],
+                                           const float projV[3],
+                                           const float d[3],
+                                           csSegment3& isectSeg,
+                                           float& t0, float &t1)
+{
+  csVector3 tmp;
+
+  // t_edge = dv1_0 / (dv1_0 - dv1_1), see (4) in Mller
+  float t_edge = d[0] / (d[0] - d[1]);
+
+  // Project onto axis
+  t0 = projV[0] + (projV[1] - projV[0]) * t_edge;
+
+  // Compute real point of intersection
+  isectSeg.SetStart (tri[0] + (tri[1] - tri[0]) * t_edge);
+
+
+  // REDO FOR OTHER EDGE
+
+  t_edge = d[0] / (d[0] - d[2]);
+
+  // Project onto axis
+  t1 = projV[0] + (projV[2] - projV[0]) * t_edge;
+
+  // Compute real point of intersection
+  isectSeg.SetEnd (tri[0] + (tri[2] - tri[0]) * t_edge);
+
+}
+
+static inline void ComputeIntervals (const csVector3 tri[3], float d[3],
+                                     SIGN_CLASS signs, size_t projDim,
+                                     csSegment3& isectSeg,
+                                     float& t0, float &t1)
+{
+  // Convert sign-class into an index
+  size_t signIdx = (size_t)signs;
+  size_t other1 = (signIdx + 1) % 3;
+  size_t other2 = (signIdx + 2) % 3;
+
+  // Setup new triangle
+  const csVector3 newTri[3] = {tri[signIdx], 
+                               tri[other1], 
+                               tri[other2]};
+
+  const float newD[3] = {d[signIdx], d[other1], d[other2]};
+
+  const float projV[3] = {tri[signIdx][projDim], 
+                          tri[other1][projDim],
+                          tri[other2][projDim]};
+
+  // Compute the intersection
+  ComputerInterval2DProj (newTri, projV, newD, isectSeg, t0, t1);
+
+  // Make sure t0 < t1, and keep track of endpoints to t values
+  csSort (t0, t1, isectSeg.Start (), isectSeg.End ());
+}
+
+static inline size_t HandlePointsInTriangle (const csVector3 tri1[3], 
+                                             const csVector3 tri2[2],
+                                             size_t k, size_t l,
+                                             csVector3& point0,
+                                             csVector3& point1)
+{
+  size_t numP = 0;
+
+  const float a0 =  tri2[1][l] - tri2[0][l];
+  const float b0 =-(tri2[1][k] - tri2[0][k]);
+  const float c0 =-a0*tri2[0][k] - b0*tri2[0][l];
+
+  const float a1 =  tri2[2][l] - tri2[1][l];
+  const float b1 =-(tri2[2][k] - tri2[1][k]);
+  const float c1 =-a1*tri2[1][k] - b1*tri2[1][l];
+
+  const float a2 =  tri2[2][l] - tri2[1][l];
+  const float b2 =-(tri2[2][k] - tri2[1][k]);
+  const float c2 =-a2*tri2[1][k] - b2*tri2[1][l];
+
+  for (size_t idx = 0; idx < 3; ++idx)
+  {
+    const float d0 = a0*tri1[idx][k]+b0*tri1[idx][l]+c0;
+    const float d1 = a1*tri1[idx][k]+b1*tri1[idx][l]+c1;
+    const float d2 = a2*tri1[idx][k]+b2*tri1[idx][l]+c2;
+
+    if(d0*d1 > 0.0f && d0*d2 > 0.0f)
+    {
+      //have a point
+      (numP == 0 ? point0 : point1) = tri1[idx];
+      numP++;
+      if (numP == 2)
+        break;
+    }
+  }  
+
+  return numP;
+}
+
+
+
+static inline bool HandleCoplanar (const csPlane3& plane, 
+                                   const csVector3 tri1[3], 
+                                   const csVector3 tri2[2], 
+                                   csSegment3& isectline)
+{
+  // All tests here run until we have two points
+  csVector3 points[3];
+  size_t numPoints = 0;
+
+  // All computations are done in 2d projected system, get the two projection
+  // axes to use
+  size_t maxNormal = plane.norm.DominantAxis ();
+  size_t k = (maxNormal + 1) % 3;
+  size_t l = (maxNormal + 2) % 3;
+
+  // Check verts against tris 
+  for (size_t tidx = 0; tidx < 1; ++tidx)
+  {
+    numPoints += HandlePointsInTriangle ( (tidx ? tri1 : tri2),
+      (tidx ? tri2 : tri1), k, l, points[0+numPoints], points[1+numPoints]);
+
+    if (numPoints == 2)
+    {
+      isectline.Start () = points[0];
+      isectline.End () = points[1];
+      return true;
+    }
+  }
+  
+
+  // Check edges pairwise until we get our two points
+  for (size_t oe = 0; oe < 3; ++oe)
+  {
+    size_t oe2 = (oe+1) % 3;
+
+    csVector2 A;
+
+    A.x = tri1[oe2][k] - tri1[oe][k];
+    A.y = tri1[oe2][l] - tri1[oe][l];
+    for (size_t ie = 0; ie < 3; ++ie)
+    {
+      // Test edge [oe - oe+1] against [ie - ie+1]
+      size_t ie2 = (ie+1) % 3;
+
+      csVector2 B, C;
+      B.x = tri2[ie][k] - tri2[ie2][k];
+      B.y = tri2[ie][l] - tri2[ie2][l];
+
+      C.x = tri1[oe][k] - tri2[oe2][k];
+      C.y = tri1[oe][l] - tri2[oe2][l];
+     
+      float f = A.y*B.x - A.x*B.y;
+      float d = B.y*C.x - B.x*C.y;
+      if ((f>0 && d>=0 && d<=f) ||
+          (f<0 && d<=0 && d>=f))
+      {
+        float e = A.x*C.y - A.y*C.x;
+        if ((f>0 && e>=0 && e<=f) ||
+            (e<=0 && e>=f))
+        {
+          // Have intersection, compute point
+          float t = d/f;
+          csVector3 p = tri2[ie2] + t*(tri2[ie]-tri2[ie2]);
+          points[numPoints] = p;
+          numPoints++;
+
+          if (numPoints == 2)
+          {
+            isectline.Start () = points[0];
+            isectline.End () = points[1];
+            return true;
+          }
+        }
+        
+      }
+    }
+  }
+
+  if (numPoints == 1)
+  {
+    isectline.End () = isectline.Start () = points[0];
+    return true;
+  }
+
+  return false;
+}
+
+
+/**
+* Tests if two triangles intersect and compute the line of intersection
+* (if they are not coplanar).
+*
+* @param tri1 Vertices of triangle 1
+* @param tri2 Vertices of triangle 2
+* @param[out] isectline The line segment where they intersect
+* @param[out] coplanar Returns whether the triangles are coplanar
+* @return true if the triangles intersect, otherwise false
+*
+*/
+
+bool csIntersect3::TriangleTriangle(const csVector3 tri1[3],
+                                    const csVector3 tri2[3],
+                                    csSegment3& isectline, bool& coplanar)
+{
+  // Indexing.. _1, _u refers to triangle 1, _2, _v to triangle 2
+
+  // compute plane equation of triangle(tri1)
+  csPlane3 plane_1 (tri1[0], tri1[1], tri1[2]);
+
+  // Compute from tri2 vertices to plane
+  float d_2[3] = {plane_1.Classify (tri2[0]),
+                  plane_1.Classify (tri2[1]), 
+                  plane_1.Classify (tri2[2])}; 
+
+  // Robustness check
+  for (unsigned int i = 0; i < 3; ++i)
+  {
+    if (fabsf(d_2[i]) < SMALL_EPSILON) d_2[i] = 0.0f;
+  }
+  
+  // Classify the signs
+  SIGN_CLASS signClass_2 = GetSignClassification (d_2[0], d_2[1], d_2[2]);
+  
+  // All on same side, not equal to 0, no intersection
+  if (signClass_2 == D_ALL_SAME)
+    return false;
+
+  
+  // Redo computations for tri2
+
+  // compute plane equation of triangle(tri2)
+  csPlane3 plane_2 (tri2[0], tri2[1], tri2[2]);
+
+  // Compute from tri2 vertices to plane
+  float d_1[3] = {plane_2.Classify (tri1[0]),
+                  plane_2.Classify (tri1[1]),
+                  plane_2.Classify (tri1[2])};
+
+  // Robustness check
+  for (unsigned int i = 0; i < 3; ++i)
+  {
+    if (fabsf(d_1[i]) < SMALL_EPSILON) d_1[i] = 0.0f;
+  }
+
+  // Classify the signs
+  SIGN_CLASS signClass_1 = GetSignClassification (d_1[0], d_1[1], d_1[2]);
+
+  // All on same side, not equal to 0, no intersection
+  if (signClass_1 == D_ALL_SAME)
+    return false;
+
+  //Check if they are coplanar (within epsilon above)
+  if (signClass_2 == D_COPLANAR)
+  {
+    // Handle coplanar case
+    coplanar = true;
+    return HandleCoplanar (plane_1, tri1, tri2, isectline);
+  }
+
+  // Not coplanar, handle normal case
+
+  // Not both on one side, compute direction of the intersection line
+  csVector3 D = plane_1.norm % plane_2.norm;
+
+  // Get largest component of D, which will be the axis we project onto
+  // for computations
+  size_t largestD = D.DominantAxis ();
+
+  // Compute interval for triangle 1
+  csSegment3 segment_1;
+  float t_1[2];
+
+  ComputeIntervals (tri1, d_1, signClass_1, largestD, segment_1, t_1[0], t_1[1]);
+
+  // Compute interval for triangle 2
+  csSegment3 segment_2;
+  float t_2[2];
+
+  ComputeIntervals (tri2, d_2, signClass_2, largestD, segment_2, t_2[0], t_2[1]);
+
+  // Now, check if the intervals overlap
+  if (t_1[1] < t_2[0] ||
+      t_2[1] < t_1[0]) //if either end-point have higher value than the other start
+    return false; //no intersection
+
+  // Intersection line goes from the highest of t_1[0] and t_2[0] to lowest of
+  // t_1[1] and t_2[1];  reuse some data-slots
+  csSort (t_1[0], t_2[0], segment_1.Start (), segment_2.Start ());
+  csSort (t_1[1], t_2[1], segment_1.End (), segment_2.End ());
+
+  isectline.Set (segment_2.Start (), segment_1.End ());
+
+  return true;
+}
