@@ -25,6 +25,7 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <math.h>
 #include <ctype.h>
 
+#include "csutil/scfarray.h"
 #include "csutil/sysfunc.h"
 
 #include "iutil/strset.h"
@@ -70,6 +71,7 @@ enum
   OP_FUNC_CROSS,
   OP_FUNC_VEC_LEN, 
   OP_FUNC_NORMAL,
+  OP_FUNC_FLOOR,
 
   OP_FUNC_ARCSIN,
   OP_FUNC_ARCCOS,
@@ -117,6 +119,7 @@ static const char* const opNames[OP_LAST] = {
   "CROSS",
   "VECLEN",
   "NORMAL",
+  "FLOOR",
   "ARCSIN",
   "ARCCOS",
   "ARCTAN",
@@ -194,7 +197,7 @@ Currently supported patterns:
 x, y  - This will not do anything to atomic arguments. x <= y. Most of these will
 require special handling during the compile stage.
 */
-static op_args_info optimize_arg_table[] = 
+static const op_args_info optimize_arg_table[] = 
 {
   { 0, 0, false }, //  OP_INVALID
 
@@ -216,6 +219,7 @@ static op_args_info optimize_arg_table[] =
   { 1, 1, false }, // OP_FUNC_CROSS
   { 1, 1, false }, // OP_FUNC_VEC_LEN
   { 1, 1, false }, // OP_FUNC_NORMAL
+  { 1, 1, false }, // OP_FUNC_FLOOR
 
   { 1, 1, false }, //  OP_FUNC_ARCSIN
   { 1, 1, false }, //  OP_FUNC_ARCCOS
@@ -272,7 +276,7 @@ void csShaderExpression::EvalError (const char* message, ...) const
 csShaderVariable* csShaderExpression::ResolveVar (csStringID name)
 {
   if (!stacks) return 0;
-  return csGetShaderVariableFromStack (*stacks, name);
+  return csGetShaderVariableFromStack (stacks, name);
 }
 
 bool csShaderExpression::Parse(iDocumentNode * node)
@@ -280,8 +284,8 @@ bool csShaderExpression::Parse(iDocumentNode * node)
   errorMsg.Empty();
   cons * head = new cons;
 
-  strset = CS_QUERY_REGISTRY_TAG_INTERFACE (
-    obj_reg, "crystalspace.shared.stringset", iStringSet);
+  strset = csQueryRegistryTagInterface<iStringSet> (
+    obj_reg, "crystalspace.shared.stringset");
   if (!strset) 
   {
     ParseError ("Can't find string registry.");
@@ -339,8 +343,8 @@ bool csShaderExpression::Parse(iDocumentNode * node)
 
   oper_arg tmp;
   tmp.type = TYPE_INVALID;
-  tmp.vec4.Set(0.0f);
-  accstack.SetLength(MAX(acc_top, accstack_max) + 1, tmp);
+  tmp.vec4.Set (0.0f);
+  accstack.SetSize (MAX(acc_top, accstack_max) + 1, tmp);
 
   destruct_cons(head);
 
@@ -349,20 +353,29 @@ bool csShaderExpression::Parse(iDocumentNode * node)
 
 bool csShaderExpression::Evaluate(csShaderVariable* var, 
                                   csShaderVarStack& stacks)
+{
+  csRef<iShaderVarStack> wrapStacks;
+  wrapStacks.AttachNew (new scfArrayWrap <iShaderVarStack, 
+    csShaderVarStack> (stacks));
+  return Evaluate (var, wrapStacks);
+}
+
+bool csShaderExpression::Evaluate(csShaderVariable* var, 
+                                  iShaderVarStack* stacks)
 {
 #ifdef SHADEREXP_DEBUG
   int debug_counter = 0;
 #endif
 
   errorMsg.Empty();
-  if (!opcodes.Length())
+  if (!opcodes.GetSize ())
   {
     EvalError ("Empty expression");
     return false;
   }
 
   bool eval = true;
-  this->stacks = &stacks;
+  this->stacks = stacks;
 
   oper_array::Iterator iter = opcodes.GetIterator();
 
@@ -890,6 +903,7 @@ bool csShaderExpression::eval_oper(int oper, oper_arg arg1, oper_arg & output)
   case OP_FUNC_ARCTAN:  return eval_arctan(arg1, output);
   case OP_FUNC_VEC_LEN: return eval_vec_len(arg1, output);
   case OP_FUNC_NORMAL: return eval_normal(arg1, output);
+  case OP_FUNC_FLOOR: return eval_floor(arg1, output);
 
   case OP_INT_LOAD: return eval_load(arg1, output);
 
@@ -1192,6 +1206,29 @@ bool csShaderExpression::eval_arctan(const oper_arg & arg1, oper_arg & output) c
   return true;  
 }
 
+bool csShaderExpression::eval_floor(const oper_arg & arg1, oper_arg & output) const 
+{
+  switch (arg1.type)
+  {
+    case TYPE_VECTOR4:
+      output.vec4.w = floorf(arg1.vec4.w);
+    case TYPE_VECTOR3:
+      output.vec4.z = floorf(arg1.vec4.z);
+    case TYPE_VECTOR2:
+      output.vec4.y = floorf(arg1.vec4.y);
+      output.vec4.x = floorf(arg1.vec4.x);
+      break;
+    case TYPE_NUMBER:
+      output.num = floorf(arg1.num);
+      break;
+    default:
+      EvalError ("Invalid type for argument to floor, %s.", GetTypeName (arg1.type));
+      return false;
+  }
+  output.type = arg1.type;
+  return true;  
+}
+
 bool csShaderExpression::eval_dot(const oper_arg & arg1, const oper_arg & arg2, oper_arg & output) const 
 {
   if (arg1.type != TYPE_VECTOR2 ||
@@ -1356,7 +1393,8 @@ bool csShaderExpression::eval_selt12(const oper_arg & arg1, const oper_arg & arg
 {
   if (arg1.type != TYPE_NUMBER || arg2.type != TYPE_NUMBER)
   {
-    EvalError ("Arguments to selt12 aren't numbers.");
+    EvalError ("Arguments to selt12 aren't numbers (%s, %s).",
+      GetTypeName (arg1.type), GetTypeName (arg2.type));
 
     return false;
   }
@@ -2074,7 +2112,7 @@ void csShaderExpression::print_cons(const cons * head) const
 
 void csShaderExpression::print_ops(const oper_array & ops) const
 {
-  oper_array::Iterator iter = ops.GetIterator();
+  oper_array::ConstIterator iter = ops.GetIterator();
 
   while (iter.HasNext())
   {
@@ -2251,7 +2289,8 @@ static const TokenTabEntry commonTokens[] = {
   {"elt2", 4, OP_VEC_ELT2},
   {"elt3", 4, OP_VEC_ELT3},
   {"elt4", 4, OP_VEC_ELT4},
-  {"frame", 4, OP_FUNC_FRAME},
+  {"floor", 5, OP_FUNC_FLOOR},
+  {"frame", 5, OP_FUNC_FRAME},
   {"make-vector", 11, OP_PS_MAKE_VECTOR},
   {"max", 3, OP_FUNC_MAX},
   {"min", 3, OP_FUNC_MIN},
