@@ -45,7 +45,6 @@ csGraphics2D::csGraphics2D (iBase* parent) :
 {
   static uint g2d_count = 0;
 
-  scfiEventHandler = 0;
   Memory = 0;
   LineAddress = 0;
   Palette = 0;
@@ -60,6 +59,7 @@ csGraphics2D::csGraphics2D (iBase* parent) :
   AllowResizing = false;
   refreshRate = 0;
   vsync = false;
+  weakEventHandler = 0;
 
   name.Format ("graph2d.%x", g2d_count++);
 
@@ -68,12 +68,11 @@ csGraphics2D::csGraphics2D (iBase* parent) :
 
 csGraphics2D::~csGraphics2D ()
 {
-  if (scfiEventHandler)
+  if (weakEventHandler != 0)
   {
-    csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+    csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
     if (q != 0)
-      q->RemoveListener (scfiEventHandler);
-    scfiEventHandler->DecRef ();
+      CS::RemoveWeakListener (q, weakEventHandler);
   }
   Close ();
   delete [] Palette;
@@ -83,7 +82,7 @@ bool csGraphics2D::Initialize (iObjectRegistry* r)
 {
   CS_ASSERT (r != 0);
   object_reg = r;
-  plugin_mgr = CS_QUERY_REGISTRY (object_reg, iPluginManager);
+  plugin_mgr = csQueryRegistry<iPluginManager> (object_reg);
   // Get the system parameters
   config.AddConfig (object_reg, "/config/video.cfg");
   Width = config->GetInt ("Video.ScreenWidth", Width);
@@ -97,7 +96,7 @@ bool csGraphics2D::Initialize (iObjectRegistry* r)
   // Get the font server: A missing font server is NOT an error
   if (!FontServer)
   {
-    FontServer = CS_QUERY_REGISTRY (object_reg, iFontServer);
+    FontServer = csQueryRegistry<iFontServer> (object_reg);
   }
 #ifdef CS_DEBUG
   if (!FontServer)
@@ -126,15 +125,13 @@ bool csGraphics2D::Initialize (iObjectRegistry* r)
     Palette [i].blue = 0;
   }
 
-  if (!scfiEventHandler)
-    scfiEventHandler = new EventHandler (this);
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
   if (q != 0)
   {
     csEventID events[3] = { csevSystemOpen (object_reg), 
 			    csevSystemClose (object_reg), 
 			    CS_EVENTLIST_END };
-    q->RegisterListener (scfiEventHandler, events);
+    CS::RegisterWeakListener (q, this, events, weakEventHandler);
   }
   return true;
 }
@@ -144,7 +141,7 @@ bool csGraphics2D::Initialize (iObjectRegistry* r, int width, int height,
 {
   CS_ASSERT (r != 0);
   object_reg = r;
-  plugin_mgr = CS_QUERY_REGISTRY (object_reg, iPluginManager);
+  plugin_mgr = csQueryRegistry<iPluginManager> (object_reg);
   // Get the system parameters
   config.AddConfig (object_reg, "/config/video.cfg");
   Width = width;
@@ -156,7 +153,7 @@ bool csGraphics2D::Initialize (iObjectRegistry* r, int width, int height,
   // Get the font server: A missing font server is NOT an error
   if (!FontServer)
   {
-    FontServer = CS_QUERY_REGISTRY (object_reg, iFontServer);
+    FontServer = csQueryRegistry<iFontServer> (object_reg);
   }
 
   // Initialize pointers to default drawing methods
@@ -210,7 +207,7 @@ bool csGraphics2D::Initialize (iObjectRegistry* r, int width, int height,
     Palette [i].blue = 0;
   }
 
-  scfiEventHandler = 0;
+  weakEventHandler = 0;
 
   csGraphics2D::ofscb = ofscb;
 
@@ -826,13 +823,15 @@ void csGraphics2D::GetRGB (int color, int& r, int& g, int& b, int& a)
 void csGraphics2D::Write (iFont *font, int x, int y, int fg, int bg, 
 			  const char *text, uint flags) 
 { 
-  fontCache->WriteString (font, x, y, fg, bg, (utf8_char*)text, flags);
+  if (!text || !*text) return;
+  fontCache->WriteString (font, x, y, fg, bg, text, false, flags);
 }
 
-void csGraphics2D::WriteBaseline (iFont *font, int x, int y, int fg, int bg, 
-				  const char *text) 
+void csGraphics2D::Write (iFont *font, int x, int y, int fg, int bg, 
+			  const wchar_t*text, uint flags) 
 { 
-  Write (font, x, y, fg, bg, text, CS_WRITE_BASELINE);
+  if (!text || !*text) return;
+  fontCache->WriteString (font, x, y, fg, bg, text, true, flags);
 }
 
 unsigned char *csGraphics2D::GetPixelAt8 (csGraphics2D *This, int x, int y)
@@ -973,9 +972,24 @@ void csGraphics2D::Alert (int type, const char* title, const char* okMsg,
   va_end (arg);
 }
 
+void csGraphics2D::Alert (int type, const wchar_t* title, const wchar_t* okMsg, 
+			  const wchar_t* msg, ...)
+{
+  va_list arg;
+  va_start (arg, msg);
+  AlertV (type, csString (title), csString (okMsg), csString (msg), arg);
+  va_end (arg);
+}
+
+void csGraphics2D::AlertV (int type, const wchar_t* title, const wchar_t* okMsg,
+    const wchar_t* msg, va_list arg)
+{
+  AlertV (type, csString (title), csString (okMsg), csString (msg), arg);
+}
+
 iNativeWindow* csGraphics2D::GetNativeWindow ()
 {
-  return CS_STATIC_CAST(iNativeWindow*, this);
+  return static_cast<iNativeWindow*> (this);
 }
 
 void csGraphics2D::SetTitle (const char* title)
