@@ -22,8 +22,13 @@
 #define INITGUID
 #include "csplugincommon/directx/guids.h"
 
+#include <mmsystem.h>
 #include <ddraw.h>
 #include <d3d.h>
 #include <dsound.h>
+#ifndef DIRECTINPUT_VERSION
+#define DIRECTINPUT_VERSION 0x0500
+#endif
+#include <dinput.h>
 
 DEFINE_GUID(GUID_NULL,0,0,0,0,0,0,0,0,0,0,0);
