@@ -27,6 +27,7 @@
 #include "ivaria/reporter.h"
 
 #include "csgeom/csrect.h"
+#include "csgeom/math.h"
 #include "csplugincommon/canvas/scrshot.h"
 #include "csplugincommon/opengl/glcommon2d.h"
 #include "csplugincommon/opengl/glstates.h"
@@ -184,10 +185,11 @@ bool csGraphics2DGLCommon::Open ()
     glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &texUnits);
     for (GLint u = texUnits - 1; u >= 0; u--)
     {
-      statecache->SetActiveTU (u);
-      statecache->ActivateTU ();
+      statecache->SetCurrentTU (u);
+      statecache->ActivateTU (csGLStateCache::activateTexEnv);
       glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_ARB);
     }
+    statecache->ActivateTU (csGLStateCache::activateTexCoord);
   }
   ext.InitGL_ARB_multisample();
 
@@ -391,7 +393,7 @@ void csGraphics2DGLCommon::OpenDriverDB (const char* phase)
   int driverDBprio = config->GetInt ("Video.OpenGL.DriverDB.Priority",
     iConfigManager::ConfigPriorityPlugin + 10);
 
-  csRef<iVFS> vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+  csRef<iVFS> vfs = csQueryRegistry<iVFS> (object_reg);
   csRef<iFile> dbfile = vfs->Open (driverDB, VFS_FILE_READ);
   if (!dbfile)
   {
@@ -400,8 +402,8 @@ void csGraphics2DGLCommon::OpenDriverDB (const char* phase)
     return;
   }
 
-  csRef<iDocumentSystem> docsys = CS_QUERY_REGISTRY (object_reg,
-    iDocumentSystem);
+  csRef<iDocumentSystem> docsys = 
+    csQueryRegistry<iDocumentSystem> (object_reg);
   if (!docsys.IsValid())
     docsys.AttachNew (new csTinyDocumentSystem ());
   csRef<iDocument> doc (docsys->CreateDocument ());
@@ -453,8 +455,7 @@ const char* csGraphics2DGLCommon::GetRendererString (const char* str)
   {
     return CS_PLATFORM_NAME;
   }
-  else
-    return 0;
+  return 0;
 }
 
 const char* csGraphics2DGLCommon::GetVersionString (const char* ver)
@@ -463,8 +464,15 @@ const char* csGraphics2DGLCommon::GetVersionString (const char* ver)
   {
     return (char*)glGetString (GL_VERSION);
   }
-  else
-    return 0;
+  else if (strcmp (ver, "glvendor") == 0)
+  {
+    /* The "vendor" part of the GL version
+     * (separated from the normal version by a space) */
+    const char* glv = (char*)glGetString (GL_VERSION);
+    const char* space = strchr (glv, ' ');
+    if (space) return space+1;
+  }
+  return 0;
 }
 
 void csGraphics2DGLCommon::Clear (int color)
@@ -824,7 +832,7 @@ bool csGraphics2DGLCommon::DebugCommand (const char* cmdstr)
 
   if (strcasecmp (cmd, "dump_fontcache") == 0)
   {
-    csRef<iImageIO> imgsaver = CS_QUERY_REGISTRY (object_reg, iImageIO);
+    csRef<iImageIO> imgsaver = csQueryRegistry<iImageIO> (object_reg);
     if (!imgsaver)
     {
       Report (CS_REPORTER_SEVERITY_WARNING,
@@ -832,7 +840,7 @@ bool csGraphics2DGLCommon::DebugCommand (const char* cmdstr)
       return false;
     }
 
-    csRef<iVFS> vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+    csRef<iVFS> vfs = csQueryRegistry<iVFS> (object_reg);
     if (!vfs)
     {
       Report (CS_REPORTER_SEVERITY_WARNING,
@@ -846,7 +854,7 @@ bool csGraphics2DGLCommon::DebugCommand (const char* cmdstr)
     ((csGLFontCache*)fontCache)->DumpFontCache (images);
 
     csString outfn;
-    for (size_t i = 0; i < images.Length(); i++)
+    for (size_t i = 0; i < images.GetSize (); i++)
     {
       csRef<iDataBuffer> buf = imgsaver->Save (images[i], "image/png");
       if (!buf)
@@ -904,16 +912,13 @@ bool csGraphics2DGLCommon::Resize (int width, int height)
 
 
 csGraphics2DGLCommon::csGLPixelFormatPicker::csGLPixelFormatPicker(
-  csGraphics2DGLCommon* parent) : order(0)
+  csGraphics2DGLCommon* parent) : parent (parent)
 {
-  csGLPixelFormatPicker::parent = parent;
-
   Reset();
 }
 
 csGraphics2DGLCommon::csGLPixelFormatPicker::~csGLPixelFormatPicker()
 {
-  delete[] order;
 }
 
 void csGraphics2DGLCommon::csGLPixelFormatPicker::ReadStartValues ()
@@ -936,25 +941,43 @@ void csGraphics2DGLCommon::csGLPixelFormatPicker::ReadStartValues ()
 
 void csGraphics2DGLCommon::csGLPixelFormatPicker::ReadPickerValues ()
 {
-  order = csStrNew (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.ReductionOrder", "ACmasdc"));
-  orderNum = strlen (order);
-  orderPos = 0;
+  const char* order = parent->config->GetStr (
+    "Video.OpenGL.FormatPicker.ReductionOrder", "ACmasdc");
+  SetupIndexTable (order);
 
   ReadPickerValue (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.ColorBits"), values[glpfvColorBits]);
+    "Video.OpenGL.FormatPicker.ColorBits"), 
+    pixelFormats[pixelFormatIndexTable[glpfvColorBits]].possibleValues);
+
   ReadPickerValue (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.AlphaBits"), values[glpfvAlphaBits]);
+    "Video.OpenGL.FormatPicker.AlphaBits"), 
+    pixelFormats[pixelFormatIndexTable[glpfvAlphaBits]].possibleValues);
+
   ReadPickerValue (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.DepthBits"), values[glpfvDepthBits]);
+    "Video.OpenGL.FormatPicker.DepthBits"), 
+    pixelFormats[pixelFormatIndexTable[glpfvDepthBits]].possibleValues);
+
   ReadPickerValue (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.StencilBits"), values[glpfvStencilBits]);
+    "Video.OpenGL.FormatPicker.StencilBits"), 
+    pixelFormats[pixelFormatIndexTable[glpfvStencilBits]].possibleValues);
+
   ReadPickerValue (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.AccumColorBits"), values[glpfvAccumColorBits]);
+    "Video.OpenGL.FormatPicker.AccumColorBits"), 
+    pixelFormats[pixelFormatIndexTable[glpfvAccumColorBits]].possibleValues);
+
   ReadPickerValue (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.AccumAlphaBits"), values[glpfvAccumAlphaBits]);
+    "Video.OpenGL.FormatPicker.AccumAlphaBits"), 
+    pixelFormats[pixelFormatIndexTable[glpfvAccumAlphaBits]].possibleValues);
+
   ReadPickerValue (parent->config->GetStr (
-    "Video.OpenGL.FormatPicker.MultiSamples"), values[glpfvMultiSamples]);
+    "Video.OpenGL.FormatPicker.MultiSamples"), 
+    pixelFormats[pixelFormatIndexTable[glpfvMultiSamples]].possibleValues);
+}
+
+template<class T>
+static int ReverseCompare(T const& r1, T const& r2)
+{
+  return csComparator<T,T>::Compare(r2,r1);
 }
 
 void csGraphics2DGLCommon::csGLPixelFormatPicker::ReadPickerValue (
@@ -981,92 +1004,127 @@ void csGraphics2DGLCommon::csGLPixelFormatPicker::ReadPickerValue (
     }
   }
 
-  if (values.Length() == 0)
+  if (values.GetSize () == 0)
     values.Push (0);
+
+  values.Sort (ReverseCompare<int>);
 }
 
 void csGraphics2DGLCommon::csGLPixelFormatPicker::SetInitialIndices ()
 {
-  for (size_t v = 0; v < glpfvValueCount; v++)
+  for (size_t format = 0; format < glpfvValueCount; ++format)
   {
-    size_t closestIndex = 0;
-    int closestDiff = 0x7fffffff;
+    size_t formatIdx = pixelFormatIndexTable[format];
 
-    for (size_t i = 0; i < values[v].Length(); i++)
+    const csArray<int>& values = pixelFormats[formatIdx].possibleValues;
+
+    size_t closestIndex = values.GetSize () - 1;   
+
+    for (size_t i = 0; i < values.GetSize (); ++i)
     {
-      int currentDiff = values[v][i] - currentValues[v];
-      if (abs (currentDiff) < closestDiff)
+      // find first which is less than value
+      if (values[i] <= currentValues[format])
       {
-	closestDiff = abs (currentDiff);
-	closestIndex = i;
-	if (currentDiff >= 0) closestIndex++;
+        closestIndex = i;
+        break;
       }
-      if (currentDiff == 0) break;
     }
-    nextValueIndices[v] = closestIndex;
+
+    //pixelFormats[formatIdx].firstIndex = csMin (firstIndex, values.GetSize () - 1);
+    pixelFormats[formatIdx].firstIndex = closestIndex;
+    pixelFormats[formatIdx].nextIndex = pixelFormats[formatIdx].firstIndex;
+  }
+}
+
+void csGraphics2DGLCommon::csGLPixelFormatPicker::SetupIndexTable (
+  const char* orderStr)
+{
+  size_t orderIdx = 0;
+
+  while (*orderStr != 0 && orderIdx < glpfvValueCount)
+  {
+    char orderVal = *orderStr;
+
+    // Map character to value type
+    GLPixelFormatValue val = glpfvColorBits;
+    switch (orderVal)
+    {
+    case 'c':
+      val = glpfvColorBits;
+      break;
+    case 'a':
+      val = glpfvAlphaBits;
+      break;
+    case 'd':
+      val = glpfvDepthBits;
+      break;
+    case 's':
+      val = glpfvStencilBits;
+      break;
+    case 'C':
+      val = glpfvAccumColorBits;
+      break;
+    case 'A':
+      val = glpfvAccumAlphaBits;
+      break;
+    case 'm':
+      val = glpfvMultiSamples;
+      break;
+    }
+
+    //Now map orderIdx to val
+    pixelFormatIndexTable[val] = orderIdx;
+
+    //Set it to be this value too
+    pixelFormats[orderIdx].valueType = val;
+
+    orderStr++;
+    orderIdx++;
   }
 }
 
 bool csGraphics2DGLCommon::csGLPixelFormatPicker::PickNextFormat ()
 {
-  size_t startOrderPos = orderPos;
-
-  bool formatPicked = false;
-  do
+  for (size_t i = 0; i < glpfvValueCount; ++i)
   {
-    int nextValue = 0;
-    switch (order[orderPos++])
-    {
-      case 'c':
-	nextValue = glpfvColorBits;
-	break;
-      case 'a':
-	nextValue = glpfvAlphaBits;
-	break;
-      case 'd':
-	nextValue = glpfvDepthBits;
-	break;
-      case 's':
-	nextValue = glpfvStencilBits;
-	break;
-      case 'C':
-	nextValue = glpfvAccumColorBits;
-	break;
-      case 'A':
-	nextValue = glpfvAccumAlphaBits;
-	break;
-      case 'm':
-	nextValue = glpfvMultiSamples;
-	break;
-      default:
-	continue;
-    }
-    const size_t nextIndex = nextValueIndices[nextValue];
-    if (nextIndex < values[nextValue].Length())
-    {
-      currentValues[nextValue] = values[nextValue][nextIndex];
-      nextValueIndices[nextValue]++;
-      formatPicked = true;
-    }
+    currentValues[pixelFormats[i].valueType] = 
+      pixelFormats[i].possibleValues[pixelFormats[i].nextIndex];
   }
-  while ((startOrderPos != (orderPos = orderPos % orderNum)) &&
-    !formatPicked);
 
-  return formatPicked;
+  // Increment
+  bool incComplete = true;
+  size_t indexToInc = 0;
+
+  do 
+  {
+    pixelFormats[indexToInc].nextIndex++;
+    incComplete = true;
+
+    if (pixelFormats[indexToInc].nextIndex >= 
+      pixelFormats[indexToInc].possibleValues.GetSize ())
+    {
+      //roll around
+      pixelFormats[indexToInc].nextIndex = pixelFormats[indexToInc].firstIndex;
+      incComplete = false;
+      indexToInc++;
+    }
+
+  } while(!incComplete && indexToInc < glpfvValueCount);
+
+  return incComplete;
 }
 
 void csGraphics2DGLCommon::csGLPixelFormatPicker::Reset()
 {
-  delete[] order;
-
   for (size_t v = 0; v < glpfvValueCount; v++)
   {
-    values[v].DeleteAll();
+    pixelFormats[v].possibleValues.DeleteAll();
   }
 
   ReadStartValues();
   ReadPickerValues();
   SetInitialIndices();
+  PickNextFormat ();
 }
 
 bool csGraphics2DGLCommon::csGLPixelFormatPicker::GetNextFormat (
