@@ -28,7 +28,7 @@
 #endif
 
 #include "csutil/csuctransform.h"
-#include "csgfx/memimage.h"
+#include "csgfx/imagememory.h"
 #include "iutil/databuff.h"
 #include "ivideo/fontserv.h"
 
@@ -73,10 +73,10 @@ csGLFontCache::~csGLFontCache ()
 
   statecache->SetTexture (GL_TEXTURE_2D, 0);
   size_t tex;
-  for (tex = 0; tex < textures.Length (); tex++)
+  for (tex = 0; tex < textures.GetSize (); tex++)
   {
     glDeleteTextures (1, &textures[tex].handle);
-    if (!(multiTexText || intensityBlendText))
+    if (!(afpText || multiTexText || intensityBlendText))
       glDeleteTextures (1, &textures[tex].mirrorHandle);
   }
   glDeleteTextures (1, &texWhite);
@@ -110,7 +110,7 @@ void csGLFontCache::Setup()
     "Video.OpenGL.FontCache.UseIntensityBlend", true);
 
   csRef<iVerbosityManager> verbosemgr (
-    CS_QUERY_REGISTRY (G2D->object_reg, iVerbosityManager));
+    csQueryRegistry<iVerbosityManager> (G2D->object_reg));
   bool do_verbose = false;
   if (verbosemgr) 
     do_verbose = verbosemgr->Enabled ("renderer.fontcache");
@@ -216,7 +216,7 @@ csGLFontCache::GlyphCacheData* csGLFontCache::InternalCacheGlyph (
     return cacheData;
   }
   csRect texRect;
-  csSubRect* sr = 0;
+  CS::SubRectangles::SubRect* sr = 0;
 
   csBitmapMetrics bmetrics;
   csRef<iDataBuffer> alphaData;
@@ -245,7 +245,7 @@ csGLFontCache::GlyphCacheData* csGLFontCache::InternalCacheGlyph (
       ((bmetrics.height + glyphAlign - 1) / glyphAlign) * glyphAlign;
   }*/
   size_t tex = 0;
-  while (tex < textures.Length ())
+  while (tex < textures.GetSize ())
   {
     sr = textures[tex].glyphRects->Alloc (allocWidth, allocHeight, 
       texRect);
@@ -255,10 +255,10 @@ csGLFontCache::GlyphCacheData* csGLFontCache::InternalCacheGlyph (
     }
     tex++;
   }
-  if ((sr == 0) && (textures.Length () < maxTxts))
+  if ((sr == 0) && (textures.GetSize () < maxTxts))
   {
-    tex = textures.Length ();
-    textures.SetLength (textures.Length () + 1);
+    tex = textures.GetSize ();
+    textures.SetSize (textures.GetSize () + 1);
 
     textures[tex].InitRects (texSize);
 
@@ -286,12 +286,12 @@ csGLFontCache::GlyphCacheData* csGLFontCache::InternalCacheGlyph (
     textures[tex].glyphRects->Alloc (1, 1, texRect);
 
     glTexImage2D (GL_TEXTURE_2D, 0, 
-      (multiTexText || intensityBlendText) ? GL_INTENSITY : GL_ALPHA, 
+      (afpText || multiTexText || intensityBlendText) ? GL_INTENSITY : GL_ALPHA, 
       texSize, texSize, 0, 
-      (multiTexText || intensityBlendText) ? GL_LUMINANCE : GL_ALPHA, 
+      (afpText || multiTexText || intensityBlendText) ? GL_LUMINANCE : GL_ALPHA, 
       GL_UNSIGNED_BYTE, texImage);
     
-    if (!(multiTexText || intensityBlendText))
+    if (!(afpText || multiTexText || intensityBlendText))
     {
       glGenTextures (1, &textures[tex].mirrorHandle);
       statecache->SetTexture (GL_TEXTURE_2D, textures[tex].mirrorHandle);
@@ -435,7 +435,9 @@ void csGLFontCache::CopyGlyphData (iFont* /*font*/, utf32_char /*glyph*/, size_t
 	  {
 	    const uint8 val = (byte & 0x80) ? 0xff : 0;
 	    *dest++ = valXor ^ val;
-	    if ((x & 7) == 7)
+	    if (((x & 7) == 7) &&
+                ((y < bmetrics.height-1) ||
+                 (x < bmetrics.width-1))) // Skip src read on last iteration
 	    {
 	      byte = *src++;
 	    }
@@ -444,7 +446,11 @@ void csGLFontCache::CopyGlyphData (iFont* /*font*/, utf32_char /*glyph*/, size_t
 	      byte <<= 1;
 	    }
 	  }
-	  if ((bmetrics.width & 7) != 0) byte = *src++;
+
+          
+	  if (((bmetrics.width & 7) != 0)  && 
+              (y < bmetrics.height-1)) // Don't do last iteration
+            byte = *src++;
 	  dest += padX;
 	}
       }
@@ -459,9 +465,9 @@ void csGLFontCache::CopyGlyphData (iFont* /*font*/, utf32_char /*glyph*/, size_t
 
     glTexSubImage2D (GL_TEXTURE_2D, 0, texRect.xmin, texRect.ymin, 
       texRect.Width (), texRect.Height (), 
-      (multiTexText || intensityBlendText) ? GL_LUMINANCE : GL_ALPHA, 
+      (afpText || multiTexText || intensityBlendText) ? GL_LUMINANCE : GL_ALPHA, 
       GL_UNSIGNED_BYTE, intData);
-    if (!(multiTexText || intensityBlendText))
+    if (!(afpText || multiTexText || intensityBlendText))
     {
       int n = texRect.Width () * texRect.Height ();
       uint8* p = intData;
@@ -541,7 +547,7 @@ void csGLFontCache::FlushArrays ()
     const bool doBG = (job.bgVertCount != 0);
     if (doFG || doBG)
     {
-      if (multiTexText || intensityBlendText)
+      if (afpText || multiTexText || intensityBlendText)
       {
         statecache->SetTexture (GL_TEXTURE_2D, job.texture);
         if (envColor != job.bg)
@@ -613,11 +619,9 @@ csGLFontCache::TextJob& csGLFontCache::GetJob (int fg, int bg,
 }
 
 void csGLFontCache::WriteString (iFont *font, int pen_x, int pen_y, 
-				 int fg, int bg, const utf8_char* text,
-				 uint flags)
+				 int fg, int bg, const void* text,
+				 bool isWide, uint flags)
 {
-  if (!text || !*text) return;
-
   bool backgroundTransparent = false;
   int bgTrans;
   {
@@ -644,7 +648,7 @@ void csGLFontCache::WriteString (iFont *font, int pen_x, int pen_y,
   if (pen_y <= ClipY1) return;
   pen_y = G2D->vpHeight - pen_y/* - maxheight*/;
 
-  size_t textLen = strlen ((char*)text);
+  size_t textLen = isWide ? wcslen ((wchar_t*)text) : strlen ((char*)text);
 
   if (!backgroundTransparent)
   {
@@ -673,11 +677,22 @@ void csGLFontCache::WriteString (iFont *font, int pen_x, int pen_y,
   while (textLen > 0)
   {
     utf32_char glyph;
-    int skip = csUnicodeTransform::UTF8Decode (text, textLen, glyph, 0);
-    if (skip == 0) break;
+    if (isWide)
+    {
+      int skip = csUnicodeTransform::Decode ((wchar_t*)text, textLen, glyph, 0);
+      if (skip == 0) break;
 
-    text += skip;
-    textLen -= skip;
+      text = ((wchar_t*)text + skip);
+      textLen -= skip;
+    }
+    else
+    {
+      int skip = csUnicodeTransform::UTF8Decode ((utf8_char*)text, textLen, glyph, 0);
+      if (skip == 0) break;
+
+      text = ((utf8_char*)text + skip);
+      textLen -= skip;
+    }
 
     const GLGlyphCacheData* cacheData = 
       (GLGlyphCacheData*)GetCacheData (knownFont, glyph, flags);
@@ -883,7 +898,8 @@ void csGLFontCache::BeginText ()
   tcaEnabled = statecache->IsEnabled_GL_TEXTURE_COORD_ARRAY ();
   caEnabled = statecache->IsEnabled_GL_COLOR_ARRAY ();
 
-  statecache->SetActiveTU (0);
+  statecache->SetCurrentTU (0);
+  statecache->ActivateTU (csGLStateCache::activateTexCoord);
   statecache->Enable_GL_VERTEX_ARRAY();
   statecache->Enable_GL_TEXTURE_COORD_ARRAY();
   statecache->Disable_GL_COLOR_ARRAY();
@@ -934,7 +950,7 @@ void csGLFontCache::FlushText ()
 
 void csGLFontCache::DumpFontCache (csRefArray<iImage>& pages)
 {
-  for (size_t t = 0; t < textures.Length(); t++)
+  for (size_t t = 0; t < textures.GetSize (); t++)
   {
     csRef<csImageMemory> page;
     page.AttachNew (new csImageMemory (texSize, texSize, 
