@@ -24,7 +24,12 @@
 #include "csgeom/polymesh.h"
 #include "csgeom/quaternion.h"
 #include "csgeom/transfrm.h"
+
+// For backward-compatibility, collider.cpp must invoke its own deprecated
+// constructors. Suppress the deprecation warning for this special case.
+#define CS_DEPRECATION_SUPPRESS_HACK
 #include "cstool/collider.h"
+#undef CS_DEPRECATION_SUPPRESS_HACK
 
 #include "iengine/camera.h"
 #include "iengine/engine.h"
@@ -37,7 +42,7 @@
 #include "iengine/viscull.h"
 #include "iengine/scenenode.h"
 
-#include "igeom/objmodel.h"
+#include "imesh/objmodel.h"
 #include "igeom/polymesh.h"
 #include "imesh/object.h"
 #include "ivaria/collider.h"
@@ -48,6 +53,26 @@ CS_LEAKGUARD_IMPLEMENT (csColliderWrapper);
 
 
 
+csColliderWrapper::csColliderWrapper (csObject& parent,
+	iCollideSystem* collide_system,
+	iTriangleMesh* mesh)
+  : scfImplementationType (this)
+{
+  parent.ObjAdd (this);
+  csColliderWrapper::collide_system = collide_system;
+  collider = collide_system->CreateCollider (mesh);
+}
+
+csColliderWrapper::csColliderWrapper (iObject* parent,
+        iCollideSystem* collide_system,
+        iTriangleMesh* mesh)
+  : scfImplementationType (this)
+{
+  parent->ObjAdd (this);
+  csColliderWrapper::collide_system = collide_system;
+  collider = collide_system->CreateCollider (mesh);
+}
+
 csColliderWrapper::csColliderWrapper (csObject& parent,
 	iCollideSystem* collide_system,
 	iPolygonMesh* mesh)
@@ -59,8 +84,8 @@ csColliderWrapper::csColliderWrapper (csObject& parent,
 }
 
 csColliderWrapper::csColliderWrapper (iObject* parent,
-	iCollideSystem* collide_system,
-	iPolygonMesh* mesh)
+        iCollideSystem* collide_system,
+        iPolygonMesh* mesh)
   : scfImplementationType (this)
 {
   parent->ObjAdd (this);
@@ -68,6 +93,27 @@ csColliderWrapper::csColliderWrapper (iObject* parent,
   collider = collide_system->CreateCollider (mesh);
 }
 
+
+csColliderWrapper::csColliderWrapper (iObject* parent,
+	iCollideSystem* collide_system,
+	iTerraFormer* terrain)
+  : scfImplementationType (this)
+{
+  parent->ObjAdd (this);
+  csColliderWrapper::collide_system = collide_system;
+  collider = collide_system->CreateCollider (terrain);
+}
+
+csColliderWrapper::csColliderWrapper (iObject* parent,
+    iCollideSystem* collide_system,
+  	iTerrainSystem* terrain)
+  : scfImplementationType (this)
+{
+  parent->ObjAdd (this);
+  csColliderWrapper::collide_system = collide_system;
+  collider = collide_system->CreateCollider (terrain);
+}
+
 csColliderWrapper::csColliderWrapper (iObject* parent,
 	iCollideSystem* collide_system,
 	iCollider* collider)
@@ -128,70 +174,217 @@ csColliderWrapper* csColliderWrapper::GetColliderWrapper (iObject* object)
   return w;	// This will DecRef() but that's ok in this case.
 }
 
-//----------------------------------------------------------------------
+void csColliderWrapper::UpdateCollider (iPolygonMesh* mesh)
+{
+  collider = collide_system->CreateCollider (mesh);
+}
 
+void csColliderWrapper::UpdateCollider (iTriangleMesh* mesh)
+{
+  collider = collide_system->CreateCollider (mesh);
+}
+
+void csColliderWrapper::UpdateCollider (iTerraFormer* terrain)
+{
+  collider = collide_system->CreateCollider (terrain);
+}
+//----------------------------------------------------------------------
 
 csColliderWrapper* csColliderHelper::InitializeCollisionWrapper (
 	iCollideSystem* colsys, iMeshWrapper* mesh)
 {
-  iObjectModel* obj_objmodel = mesh->GetMeshObject ()->GetObjectModel ();
-  iPolygonMesh* obj_polymesh = obj_objmodel->GetPolygonMeshColldet ();
-
   iMeshFactoryWrapper* factory = mesh->GetFactory ();
-  csColliderWrapper* cw = 0;
+
+  iObjectModel* obj_objmodel = mesh->GetMeshObject ()->GetObjectModel ();
+  csStringID basemesh_id = colsys->GetBaseDataID ();
+  csStringID trianglemesh_id = colsys->GetTriangleDataID ();
+
+  // do_trimesh will be set to true if we're using the new trianglemesh
+  // system (instead of polygonmesh). We detect that we want to use the
+  // new system when the base mesh of the object model is given (even
+  // if the collision detection mesh might not be set).
+  bool do_trimesh = obj_objmodel->IsTriangleDataSet (basemesh_id);
+  // Also check factory.
+  if (!do_trimesh && factory)
+  {
+    iObjectModel* fact_objmodel = factory->GetMeshObjectFactory ()
+      ->GetObjectModel ();
+    if (fact_objmodel)
+      do_trimesh = fact_objmodel->IsTriangleDataSet (basemesh_id);
+  }
+
+  iTriangleMesh* obj_trimesh;
+  iPolygonMesh* obj_polymesh;
+  bool obj_trimesh_set = false;
+  if (do_trimesh)
+  {
+    // If the collision detection triangle mesh is not set then we
+    // use the base mesh instead.
+    obj_trimesh_set = obj_objmodel->IsTriangleDataSet (trianglemesh_id);
+    if (obj_trimesh_set)
+      obj_trimesh = obj_objmodel->GetTriangleData (trianglemesh_id);
+    else
+      obj_trimesh = obj_objmodel->GetTriangleData (basemesh_id);
+    obj_polymesh = 0;
+  }
+  else
+  {
+    obj_polymesh = obj_objmodel->GetPolygonMeshColldet ();
+    obj_trimesh = 0;
+  }
+
+  iTerraFormer* obj_terraformer = obj_objmodel->GetTerraFormerColldet ();
+  iTerrainSystem* obj_terrain = obj_objmodel->GetTerrainColldet ();
+
+  csRef<csColliderWrapper> cw;
   if (factory)
   {
     iObjectModel* fact_objmodel = factory->GetMeshObjectFactory ()
     	->GetObjectModel ();
     if (fact_objmodel)
     {
-      iPolygonMesh* fact_polymesh = fact_objmodel->GetPolygonMeshColldet ();
-      if (fact_polymesh && (fact_polymesh == obj_polymesh || !obj_polymesh))
+      if (fact_objmodel->GetTerraFormerColldet ())
       {
-        // First check if the parent factory has a collider wrapper.
-	iCollider* collider;
-        csColliderWrapper* cw_fact = csColliderWrapper::GetColliderWrapper (
-		factory->QueryObject ());
-        if (cw_fact)
+        iTerraFormer* fact_terraformer = fact_objmodel
+	  ->GetTerraFormerColldet ();
+
+        if (fact_terraformer && (fact_terraformer == obj_terraformer
+	      || !obj_terraformer))
+        {
+          // First check if the parent factory has a collider wrapper.
+          iCollider* collider;
+          csColliderWrapper* cw_fact = csColliderWrapper::GetColliderWrapper (
+            factory->QueryObject ());
+          if (cw_fact)
+          {
+            collider = cw_fact->GetCollider ();
+          }
+          else
+          {
+            csColliderWrapper *cw_fact = new csColliderWrapper (
+              factory->QueryObject (), colsys, fact_terraformer);
+            cw_fact->SetName (factory->QueryObject ()->GetName());
+            collider = cw_fact->GetCollider ();
+            cw_fact->DecRef ();
+          }
+
+          // Now add the collider wrapper to the mesh. We need a new
+          // csColliderWrapper because the csObject system is strictly
+          // a tree and one csColliderWrapper cannot have multiple parents.
+          cw.AttachNew (new csColliderWrapper (mesh->QueryObject (),
+            colsys, collider));
+          cw->SetName (mesh->QueryObject ()->GetName());
+          cw = 0;
+          // Clear object polygon mesh so we will no longer try to create
+          // a collider from that (we already have a collider).
+          obj_terraformer = 0;
+        }
+      }
+      else 
+      {
+	if (do_trimesh)
 	{
-	  collider = cw_fact->GetCollider ();
+	  if (!obj_trimesh_set)
+	  {
+	    bool fact_trimesh_set = fact_objmodel->IsTriangleDataSet (
+	        trianglemesh_id);
+            iTriangleMesh* fact_trimesh;
+	    // If the collision detection triangle mesh is not set then we
+	    // use the base mesh instead.
+	    if (fact_trimesh_set)
+	      fact_trimesh  = fact_objmodel->GetTriangleData (trianglemesh_id);
+	    else
+	      fact_trimesh  = fact_objmodel->GetTriangleData (basemesh_id);
+
+	    if (fact_trimesh)
+	    {
+              // First check if the parent factory has a collider wrapper.
+              iCollider* collider;
+              csColliderWrapper* cw_fact =
+		csColliderWrapper::GetColliderWrapper (factory->QueryObject ());
+              if (cw_fact)
+              {
+                collider = cw_fact->GetCollider ();
+              }
+              else
+              {
+                csColliderWrapper *cw_fact = new csColliderWrapper (
+                  factory->QueryObject (), colsys, fact_trimesh);
+                cw_fact->SetName (factory->QueryObject ()->GetName());
+                collider = cw_fact->GetCollider ();
+                cw_fact->DecRef ();
+              }
+
+              // Now add the collider wrapper to the mesh. We need a new
+              // csColliderWrapper because the csObject system is strictly
+              // a tree and one csColliderWrapper cannot have multiple parents.
+              cw.AttachNew (new csColliderWrapper (mesh->QueryObject (),
+                colsys, collider));
+              cw->SetName (mesh->QueryObject ()->GetName());
+	    }
+            // Clear object polygon mesh so we will no longer try to create
+            // a collider from that (we already have a collider).
+            obj_trimesh = 0;
+	  }
 	}
 	else
 	{
-	  csColliderWrapper *cw_fact = new csColliderWrapper (
-	  	factory->QueryObject (), colsys, fact_polymesh);
-	  cw_fact->SetName (factory->QueryObject ()->GetName());
-	  collider = cw_fact->GetCollider ();
-	  cw_fact->DecRef ();
-	}
+          iPolygonMesh* fact_polymesh = fact_objmodel->GetPolygonMeshColldet ();
+          if (fact_polymesh && (fact_polymesh == obj_polymesh || !obj_polymesh))
+          {
+            // First check if the parent factory has a collider wrapper.
+            iCollider* collider;
+            csColliderWrapper* cw_fact = csColliderWrapper::GetColliderWrapper (
+              factory->QueryObject ());
+            if (cw_fact)
+            {
+              collider = cw_fact->GetCollider ();
+            }
+            else
+            {
+              csColliderWrapper *cw_fact = new csColliderWrapper (
+                factory->QueryObject (), colsys, fact_polymesh);
+              cw_fact->SetName (factory->QueryObject ()->GetName());
+              collider = cw_fact->GetCollider ();
+              cw_fact->DecRef ();
+            }
 
-	// Now add the collider wrapper to the mesh. We need a new
-	// csColliderWrapper because the csObject system is strictly
-	// a tree and one csColliderWrapper cannot have multiple parents.
-	cw = new csColliderWrapper (mesh->QueryObject (),
-	  colsys, collider);
-	cw->SetName (mesh->QueryObject ()->GetName());
-	cw->DecRef ();
-	// Clear object polygon mesh so we will no longer try to create
-	// a collider from that (we already have a collider).
-	obj_polymesh = 0;
+            // Now add the collider wrapper to the mesh. We need a new
+            // csColliderWrapper because the csObject system is strictly
+            // a tree and one csColliderWrapper cannot have multiple parents.
+            cw.AttachNew (new csColliderWrapper (mesh->QueryObject (),
+              colsys, collider));
+            cw->SetName (mesh->QueryObject ()->GetName());
+            // Clear object polygon mesh so we will no longer try to create
+            // a collider from that (we already have a collider).
+            obj_polymesh = 0;
+          }
+	}
       }
     }
   }
 
-  if (obj_polymesh)
-  {
-    cw = new csColliderWrapper (mesh->QueryObject (),
-	colsys, obj_polymesh);
+  if (obj_terraformer)
+    cw.AttachNew (new csColliderWrapper (mesh->QueryObject (),
+      colsys, obj_terraformer));
+  else if (obj_terrain)
+    cw.AttachNew (new csColliderWrapper (mesh->QueryObject (),
+      colsys, obj_terrain));
+  else if (do_trimesh && obj_trimesh)
+    cw.AttachNew (new csColliderWrapper (mesh->QueryObject (),
+      colsys, obj_trimesh));
+  else if (obj_polymesh)
+    cw.AttachNew (new csColliderWrapper (mesh->QueryObject (),
+      colsys, obj_polymesh));
+  if (cw)
     cw->SetName (mesh->QueryObject ()->GetName());
-    cw->DecRef ();
-  }
 
-  const csRefArray<iSceneNode>& ml = mesh->QuerySceneNode ()->GetChildren ();
+  const csRef<iSceneNodeArray> ml = 
+    mesh->QuerySceneNode ()->GetChildrenArray ();
   size_t i;
-  for (i = 0 ; i < ml.Length () ; i++)
+  for (i = 0 ; i < ml->GetSize () ; i++)
   {
-    iMeshWrapper* child = ml[i]->QueryMesh ();
+    iMeshWrapper* child = ml->Get (i)->QueryMesh ();
     // @@@ What if we have a light containing another mesh?
     if (child)
       InitializeCollisionWrapper (colsys, child);
@@ -374,7 +567,7 @@ float csColliderHelper::TraceBeam (iCollideSystem* cdsys, iSector* sector,
 	const csArray<csIntersectingTriangle>& tris = cdsys->
 		GetIntersectingTriangles ();
 	size_t i;
-	for (i = 0 ; i < tris.Length () ; i++)
+	for (i = 0 ; i < tris.GetSize () ; i++)
 	{
 	  csVector3 isect;
 	  csIntersectingTriangle tri;
@@ -510,6 +703,7 @@ csColliderActor::csColliderActor ()
   camera = 0;
   movable = 0;
   revertCount = 0;
+  do_hit_meshes = false;
 
   // Only used in case a camera is used.
   rotation.Set (0, 0, 0);
@@ -580,8 +774,8 @@ void csColliderActor::SetCamera (iCamera* camera, bool adjustRotation)
   this->camera = camera; 
   if (adjustRotation)
   {
-    csQuaternion quat (camera->GetTransform().GetT2O());
-    quat.GetEulerAngles (rotation, true);
+    csQuaternion quat; quat.SetMatrix (camera->GetTransform().GetT2O());
+    rotation = quat.GetEulerAngles ();
     // Angle fixups.
     /* @@@ FIXME: Are those right in the math sense or do they indicate
      * csQuaternion bugs? */
@@ -765,6 +959,8 @@ int csColliderActor::CollisionDetect (
       if (reallycollided)
       {
         hits++;
+	if (do_hit_meshes)
+	  hit_meshes.Add (meshWrapper);
         if (cdsys->GetOneHitOnly ()) return 1;
       }
     }
@@ -888,8 +1084,9 @@ bool csColliderActor::AdjustForCollisions (
 
   // localvel is smaller because we can partly move the object in that direction
   localvel -= maxmove - oldpos;
+  csVector3 correctedVel(localvel);
 
-  for (i = 0; i < our_cd_contact.Length () ; i++ )
+  for (i = 0; i < our_cd_contact.GetSize () ; i++ )
   {
     csCollisionPair& cd = our_cd_contact[i];
     csPlane3 obstacle (cd.a2, cd.b2, cd.c2);
@@ -902,9 +1099,11 @@ bool csColliderActor::AdjustForCollisions (
     if (unit * localvel > 0) continue;
 
     // Bounce back
-    localvel -= unit * (unit * localvel)*1.1; //(-(localvel % unit) % unit);
+
+    correctedVel -= unit * (unit * correctedVel)*1.1; //(-(localvel % unit) % unit);
   }
  
+  localvel = correctedVel;
   newpos = maxmove + localvel;
 
   transform_newpos = csOrthoTransform (csMatrix3(), newpos);
@@ -957,24 +1156,18 @@ bool csColliderActor::AdjustForCollisions (
   while (hits > 0 && newpos.y < maxJump)
   {
     bool adjust = false;
-    for (i = 0; i < our_cd_contact.Length (); i++ )
+    for (i = 0; i < our_cd_contact.GetSize (); i++ )
     {
       csCollisionPair cd = our_cd_contact[i];
       csPlane3 obstacle (cd.a2, cd.b2, cd.c2);
       csVector3 normal = obstacle.Normal();
       float norm = normal.Norm ();
-      if (fabs (norm) < SMALL_EPSILON) continue;
+
+      // Ensure this is a big-enough triangle to count as a collision.
+      if (fabs (norm) < 1e-4f ) continue;
 
       csVector3 n = normal / norm;
 
-      //csVector3 n = -((cd.c2-cd.b2)%(cd.b2-cd.a2)).Unit ();
-
-      // Is it a collision with a ground polygon?
-      //  (this tests for the angle between ground and colldet
-      //  triangle)
-      if (n.y < 0.7)
-        continue;
-
       csVector3 line[2];
 
       // This needs to be done for numerical inaccuracies in this test
@@ -982,8 +1175,12 @@ bool csColliderActor::AdjustForCollisions (
       if(!FindIntersection (cd,line))
         continue;
 
-      // Hit a ground polygon so we are not falling
-      onground = adjust = true;
+      // Is it a collision with a ground polygon?
+      //  (this tests for the angle between ground and colldet
+      //  triangle)
+      if(!(n.y < 0.7))
+          onground = true;
+      adjust = true;
       max_y = MAX(MAX(line[0].y, line[1].y)+shift.y,max_y);
       if (max_y > maxJump)
       {
@@ -993,6 +1190,12 @@ bool csColliderActor::AdjustForCollisions (
     }
     hits = 0;
 
+    // This prevents us from going up if there is no surface to rest on.
+    if(!onground && max_y > oldpos.y)
+    {
+        newpos.y = oldpos.y;
+        break;
+    }
     if (adjust)
     {
       // Temporarily lift the model up so that it passes the final check
@@ -1039,7 +1242,7 @@ bool csColliderActor::AdjustForCollisions (
     cd = 0;
 
   hits = 0;
-  for (i = 0; i < our_cd_contact.Length () ; i++ )
+  for (i = 0; i < our_cd_contact.GetSize () ; i++ )
   {
     csCollisionPair& cd = our_cd_contact[i];
     csPlane3 obstacle (cd.a2, cd.b2, cd.c2);
@@ -1125,6 +1328,8 @@ bool csColliderActor::RotateV (float delta,
 bool csColliderActor::MoveV (float delta,
 	const csVector3& velBody)
 {
+  hit_meshes.Empty ();
+  
   if (velBody < SMALL_EPSILON && velWorld < SMALL_EPSILON
   	&& onground)
     return false;  // didn't move anywhere
@@ -1228,7 +1433,6 @@ bool csColliderActor::MoveV (float delta,
 }
 
 #define MAX_CD_INTERVAL 1
-#define MIN_CD_INTERVAL 0.01
 #define MAX_CD_ITERATIONS 20
 
 static float ComputeLocalMaxInterval (
@@ -1236,14 +1440,14 @@ static float ComputeLocalMaxInterval (
 	const csVector3& intervalSize)
 {
   float local_max_interval =
-    MAX (MIN (MIN ((fabs (bodyVel.y) < SMALL_EPSILON)
+    MIN (MIN ((fabs (bodyVel.y) < SMALL_EPSILON)
   	? MAX_CD_INTERVAL
 	: fabs (intervalSize.y/bodyVel.y), (fabs (bodyVel.x) < SMALL_EPSILON)
 		? MAX_CD_INTERVAL
 		: fabs (intervalSize.x/bodyVel.x)),
 			(fabs (bodyVel.z) < SMALL_EPSILON)
 			? MAX_CD_INTERVAL
-			: fabs (intervalSize.z/bodyVel.z)), MIN_CD_INTERVAL);
+			: fabs (intervalSize.z/bodyVel.z));
   return local_max_interval;
 }
 
@@ -1276,12 +1480,10 @@ bool csColliderActor::Move (float delta, float speed, const csVector3& velBody,
   // Calculate the total velocity (body and world) in OBJECT space.
   csVector3 bodyVel (fulltransf.Other2ThisRelative (velWorld) + velBody);
 
-  float local_max_interval = ComputeLocalMaxInterval (bodyVel, intervalSize);
+  float local_max_interval = ComputeLocalMaxInterval (bodyVel, intervalSize - csVector3(0.005f));
 
   // Compensate for speed
   local_max_interval /= speed;
-  // Err on the side of safety
-  local_max_interval -= 0.005f;
 
   int maxiter = MAX_CD_ITERATIONS;
 
@@ -1315,5 +1517,3 @@ bool csColliderActor::Move (float delta, float speed, const csVector3& velBody,
 
   return rc;
 }
-
-
