@@ -22,10 +22,10 @@
 #include "csutil/sysfunc.h"
 #include "igraphic/image.h"
 #include "csgfx/rgbpixel.h"
-#include "csgfx/memimage.h"
+#include "csgfx/imagememory.h"
 #include "csqint.h"
 
-
+#include "csutil/win32/msvc_deprecated_warn_off.h"
 
 //-- csGenerateImageTextureBlend ----------------------------------
 csGenerateImageTextureBlend::csGenerateImageTextureBlend()
