@@ -18,6 +18,7 @@
 
 #include "cssysdef.h"
 #include "cstool/initapp.h"
+#include "csutil/ansicommand.h"
 #include "csutil/cfgacc.h"
 #include "csutil/cfgfile.h"
 #include "csutil/cfgmgr.h"
@@ -203,8 +204,8 @@ iVerbosityManager* csInitializer::CreateVerbosityManager (
   iObjectRegistry* r)
 {
   csVerbosityManager* v = new csVerbosityManager;
-  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (
-    r, iCommandLineParser));
+  csRef<iCommandLineParser> cmdline (
+    csQueryRegistry<iCommandLineParser> (r));
   if (cmdline.IsValid())
   {
     for (size_t i = 0; ; i++)
@@ -234,7 +235,7 @@ iConfigManager* csInitializer::CreateConfigManager (iObjectRegistry* r)
 static void SetupPluginLoadErrVerbosity(iObjectRegistry* r)
 {
   csRef<iVerbosityManager> verbosemgr (
-    CS_QUERY_REGISTRY(r, iVerbosityManager));
+    csQueryRegistry<iVerbosityManager> (r));
   bool verbose = CS_LOAD_LIB_VERBOSE;
   if (verbosemgr.IsValid())
     verbose = verbosemgr->Enabled("loadlib");
@@ -243,37 +244,45 @@ static void SetupPluginLoadErrVerbosity(iObjectRegistry* r)
 
 iVFS* csInitializer::SetupVFS(iObjectRegistry* r, const char* pluginID)
 {
-  csRef<iVFS> VFS (CS_QUERY_REGISTRY (r, iVFS));
+  csRef<iVFS> VFS (csQueryRegistry<iVFS> (r));
   if (!VFS)
   {
-    csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (r, iPluginManager));
+    csRef<iPluginManager> plugin_mgr (csQueryRegistry<iPluginManager> (r));
     csRef<iBase> b = csPtr<iBase> (plugin_mgr->QueryPlugin (
       "iVFS", scfInterfaceTraits<iVFS>::GetVersion()));
     VFS = scfQueryInterfaceSafe<iVFS> (b);
   }
   if (!VFS)
   {
-    csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (r, iPluginManager));
+    csRef<iPluginManager> plugin_mgr (csQueryRegistry<iPluginManager> (r));
     VFS = CS_LOAD_PLUGIN (plugin_mgr, pluginID, iVFS);
     if (!VFS)
     {
       /* NB: loading the plugin should have already resulted in a message 
        * having been printed. */
       static const char highlight[] = 
-	" " CS_ANSI_FI "%s" CS_ANSI_RST " ";
+	" " CS_ANSI_TEXT_BOLD_ON "%s" CS_ANSI_RST " ";
       csFPrintf (stderr, highlight, 
 	"* This likely means that the plugins could not be found.");
       csFPrintf (stderr, "\n");
       csFPrintf (stderr, highlight, "If you're a user:");
       csFPrintf (stderr, "Check the working directory the application starts "
 	"from -\n");
+#if defined(CS_PLATFORM_MACOSX)
+      csFPrintf (stderr, "  on MaxOS/X, it should be the directory containing the\n"
+	"  '<application>.app' subdirectory, *not* '<application>.app/Contents/MacOS'.\n");
+#else
       csFPrintf (stderr, "  usually, it is the same as the directory with the "
 	"executable.\n");
+#endif
       csFPrintf (stderr, "  If in doubt, contact the vendor.\n");
       csFPrintf (stderr, highlight, "If you're a developer:");
       csFPrintf (stderr, "Check if the CRYSTAL environment var points to the\n");
       csFPrintf (stderr, "  correct location - usually the directory CS was "
 	"built in.\n");
+      csFPrintf (stderr, "  You can also use the '--verbose' command line switch "
+	"to troubleshoot\n");
+      csFPrintf (stderr, "  where CS looks for plugins.\n");
       return 0;
     }
     r->Register (VFS, "iVFS");
@@ -299,7 +308,7 @@ bool csInitializer::SetupConfigManager (
 
   csRef<iVFS> VFS = SetupVFS(r);
 
-  csRef<iConfigManager> Config (CS_QUERY_REGISTRY (r, iConfigManager));
+  csRef<iConfigManager> Config (csQueryRegistry<iConfigManager> (r));
   csRef<iConfigFile> cfg = Config->GetDynamicDomain ();
   Config->SetDomainPriority (cfg, iConfigManager::ConfigPriorityApplication);
 
@@ -325,6 +334,19 @@ bool csInitializer::SetupConfigManager (
     }
   }
 
+  // Handle command line config settings
+  {
+    csRef<iCommandLineParser> cmdline = 
+      csQueryRegistry<iCommandLineParser> (r);
+    if (cmdline.IsValid())
+    {
+      csRef<csConfigFile> cmdlineConfig;
+      cmdlineConfig.AttachNew (new csConfigFile);
+      cmdlineConfig->ParseCommandLine (cmdline, VFS);
+      Config->AddDomain (cmdlineConfig, iConfigManager::ConfigPriorityCmdLine);
+    }
+  }
+
   config_done = true;
   return true;
 }
@@ -362,7 +384,7 @@ bool csInitializer::RequestPlugins (
 
   csPluginLoader* plugldr = new csPluginLoader (r);
 
-  csArray<csPluginRequest>::Iterator i(a.GetIterator());
+  csArray<csPluginRequest>::ConstIterator i(a.GetIterator());
   while (i.HasNext())
   {
     csPluginRequest const req(i.Next());
@@ -389,7 +411,7 @@ bool csInitializer::SetupEventHandler (
   const csEventID events[])
 {
   CS_ASSERT(installed_event_handler == 0);
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY (r, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (r));
   if (q) {
     return (q->RegisterListener (evhdlr, events) != CS_HANDLER_INVALID);
   }
@@ -434,7 +456,7 @@ bool csInitializer::OpenApplication (iObjectRegistry* r)
   SetupConfigManager (r, 0);
 
   // Pass the open event to all interested listeners.
-  csRef<iEventQueue> EventQueue (CS_QUERY_REGISTRY (r, iEventQueue));
+  csRef<iEventQueue> EventQueue (csQueryRegistry<iEventQueue> (r));
   CS_ASSERT (EventQueue != 0);
   csEventID ename = csevSystemOpen(r);
   csRef<iEvent> e = EventQueue->CreateBroadcastEvent(ename);
@@ -446,7 +468,7 @@ bool csInitializer::OpenApplication (iObjectRegistry* r)
 void csInitializer::CloseApplication (iObjectRegistry* r)
 {
   // Notify all interested listeners that the system is going down
-  csRef<iEventQueue> EventQueue (CS_QUERY_REGISTRY (r, iEventQueue));
+  csRef<iEventQueue> EventQueue (csQueryRegistry<iEventQueue> (r));
   if (EventQueue)
   {
     csRef<iEvent> e = EventQueue->CreateBroadcastEvent(csevSystemClose(r));
@@ -463,7 +485,7 @@ void csInitializer::DestroyApplication (iObjectRegistry* r)
   // all other event handlers which are registered through plug-ins or
   // other sources.
   {
-    csRef<iEventQueue> q (CS_QUERY_REGISTRY (r, iEventQueue));
+    csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (r));
     if (q)
       q->RemoveAllListeners ();
   }
@@ -472,7 +494,7 @@ void csInitializer::DestroyApplication (iObjectRegistry* r)
   // some plugins hold references to the plugin manager so the plugin
   // manager will never get destructed if there are still plugins in memory.
   {
-    csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (r, iPluginManager));
+    csRef<iPluginManager> plugin_mgr (csQueryRegistry<iPluginManager> (r));
     if (plugin_mgr)
       plugin_mgr->Clear ();
     // Force cleanup here.
