@@ -24,11 +24,11 @@
 
 //---------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE_EXT (csMapNode)
-  SCF_IMPLEMENTS_INTERFACE (iMapNode)
-SCF_IMPLEMENT_IBASE_EXT_END
 
-csMapNode::csMapNode (const char* Name) : m_pSector(0), m_Position(0, 0, 0)
+csMapNode::csMapNode (const char* Name) :
+  scfImplementationType (this),
+  sector (0), position (0, 0, 0), xvector (1, 0, 0), yvector (0, 1, 0),
+  zvector (0, 0, 1)
 {
   SetName (Name);
 }
@@ -37,26 +37,11 @@ csMapNode::~csMapNode ()
 {
 }
 
-void csMapNode::SetPosition (const csVector3& pos)
-{
-  m_Position = pos;
-}
-
-const csVector3& csMapNode::GetPosition () const
-{
-  return m_Position;
-}
-
-iSector *csMapNode::GetSector () const
-{
-  return m_pSector;
-}
-
 void csMapNode::SetSector (iSector *pSector)
 {
-  if (m_pSector) m_pSector->QueryObject ()->ObjRemove (this);
-  m_pSector = pSector;
-  if (m_pSector) m_pSector->QueryObject ()->ObjAdd (this);
+  if (sector) sector->QueryObject ()->ObjRemove (this);
+  sector = pSector;
+  if (sector) sector->QueryObject ()->ObjAdd (this);
 }
 
 iMapNode* csMapNode::GetNode (iSector *pSector, const char* name,
@@ -91,7 +76,7 @@ void csNodeIterator::Reset (iSector *pSector, const char *classname)
   Classname = classname;
   iObject* n = Iterator->Next ();
   if (n)
-    CurrentNode = SCF_QUERY_INTERFACE (n, iMapNode);
+    CurrentNode = scfQueryInterface<iMapNode> (n);
   else
     CurrentNode = 0;
 
@@ -131,7 +116,7 @@ void csNodeIterator::NextNode ()
 {
   iObject* obj = Iterator->Next ();
   if (obj)
-    CurrentNode = SCF_QUERY_INTERFACE (obj, iMapNode);
+    CurrentNode = scfQueryInterface<iMapNode> (obj);
   else
     CurrentNode = 0;
 }
