@@ -20,10 +20,11 @@
 #include "cssysdef.h"
 #include <math.h>
 
-#include "csgfx/memimage.h"
+#include "csgfx/imagememory.h"
 #include "cstool/proctex.h"
 #include "csutil/hash.h"
 #include "csutil/event.h"
+#include "csutil/eventnames.h"
 #include "csutil/scf_implementation.h"
 #include "iengine/engine.h"
 #include "iengine/material.h"
@@ -66,15 +67,15 @@ public:
 
   virtual bool HandleEvent (iEvent& event);
 
-  CS_EVENTHANDLER_NAMES("crystalspace.proxtex")
+  CS_EVENTHANDLER_NAMES("crystalspace.proctex")
   CS_EVENTHANDLER_NIL_CONSTRAINTS
 
 public:
-  void PushTexture (csProcTexture* txt)
+  virtual void PushTexture (csProcTexture* txt)
   {
     textures.Add (txt);
   }
-  void PopTexture (csProcTexture* txt)
+  virtual void PopTexture (csProcTexture* txt)
   {
     textures.Delete (txt);
   }
@@ -83,7 +84,7 @@ public:
 
 bool csProcTexEventHandler::HandleEvent (iEvent& event)
 {
-  csRef<iVirtualClock> vc (CS_QUERY_REGISTRY (object_reg, iVirtualClock));
+  csRef<iVirtualClock> vc (csQueryRegistry<iVirtualClock> (object_reg));
   csTicks elapsed_time, current_time;
   elapsed_time = vc->GetElapsedTicks ();
   current_time = vc->GetCurrentTicks ();
@@ -147,11 +148,11 @@ csProcTexture::~csProcTexture ()
 iEventHandler* csProcTexture::SetupProcEventHandler (
 	iObjectRegistry* object_reg)
 {
-  csRef<iEventHandler> proceh = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg,
-  	"crystalspace.proctex.eventhandler", iEventHandler);
+  csRef<iEventHandler> proceh = csQueryRegistryTagInterface<iEventHandler>
+  	(object_reg, "crystalspace.proctex.eventhandler");
   if (proceh) return proceh;
   proceh = csPtr<iEventHandler> (new csProcTexEventHandler (object_reg));
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
   if (q != 0)
   {
     q->RegisterListener (proceh, csevPreProcess(object_reg));
@@ -173,9 +174,7 @@ struct csProcTexCallback :
 
 void csProcTexCallback::UseTexture (iTextureWrapper*)
 {
-  if (!pt->PrepareAnim ()) return;
-  pt->visible = true;
-  ((csProcTexEventHandler*)(iEventHandler*)(pt->proceh))->PushTexture (pt);
+  pt->UseTexture ();
 }
 iProcTexture* csProcTexCallback::GetProcTexture() const
 {
@@ -187,29 +186,36 @@ bool csProcTexture::Initialize (iObjectRegistry* object_reg)
   csProcTexture::object_reg = object_reg;
   proceh = SetupProcEventHandler (object_reg);
 
-  if (!proc_image.IsValid())
-    proc_image.AttachNew (new csImageMemory (mat_w, mat_h));
+  g3d = csQueryRegistry<iGraphics3D> (object_reg);
+  g2d = csQueryRegistry<iGraphics2D> (object_reg);
 
-  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
-  g2d = CS_QUERY_REGISTRY (object_reg, iGraphics2D);
-
-  csRef<iEngine> engine (CS_QUERY_REGISTRY (object_reg, iEngine));
-  tex = engine->GetTextureList ()->NewTexture (proc_image);
-  proc_image = 0;
+  csRef<iEngine> engine (csQueryRegistry<iEngine> (object_reg));
+  if (proc_image)
+  {
+    tex = engine->GetTextureList()->NewTexture (proc_image);
+    tex->SetFlags (CS_TEXTURE_3D | texFlags);
+    proc_image = 0;
+  }
+  else
+  {
+    csRef<iTextureHandle> texHandle = 
+      g3d->GetTextureManager()->CreateTexture (mat_w, mat_h, csimg2D, "rgb8",
+      CS_TEXTURE_3D | texFlags);
+    tex = engine->GetTextureList()->NewTexture (texHandle);
+  }
   if (!tex)
     return false;
 
   if (key_color)
     tex->SetKeyColor (key_red, key_green, key_blue);
 
-  tex->SetFlags (tex->GetFlags() | texFlags);
   tex->QueryObject ()->SetName (GetName ());
   if (use_cb)
   {
-    csProcTexCallback* cb = new csProcTexCallback ();
+    csRef<csProcTexCallback> cb;
+    cb.AttachNew (new csProcTexCallback ());
     cb->pt = this;
     tex->SetUseCallback (cb);
-    cb->DecRef ();
   }
   ptReady = true;
   return true;
@@ -254,6 +260,13 @@ void csProcTexture::SetAlwaysAnimate (bool enable)
   }
 }
 
+void csProcTexture::UseTexture ()
+{
+  if (!PrepareAnim ()) return;
+  visible = true;
+  ((csProcTexEventHandler*)(iEventHandler*)(proceh))->PushTexture (this);
+}
+
 //-----------------------------------------------------------------------------
 
 iObject* csProcTexture::QueryObject()
