@@ -21,6 +21,7 @@
 #include "csutil/cfgmgr.h"
 #include "csutil/csstring.h"
 #include "csutil/scf_implementation.h"
+#include "csutil/scfstringarray.h"
 #include "csutil/strhash.h"
 #include "csutil/sysfunc.h"
 #include "csutil/util.h"
@@ -83,8 +84,52 @@ private:
   csRef<csConfigManager> Config;
   csConfigDomain *CurrentDomain;
   csRef<iConfigIterator> CurrentIterator;
-  char *Subsection;
+  csRef<iConfigIterator> nextIterator;
+  csString Subsection;
   csStringHash Iterated;
+
+  const char* currentKey;
+  const char* currentValue;
+  const char* currentComment;
+
+  void FetchNextIterator ()
+  {
+    // move to next domain (which actually means previous domain!)
+    CurrentDomain = CurrentDomain->Prev;
+    if (CurrentDomain == 0)
+    {
+      nextIterator = 0;
+      return;
+    }
+    if (CurrentDomain->Cfg == 0)
+    {
+      nextIterator = 0;
+      return;
+    }
+    nextIterator = CurrentDomain->Cfg->Enumerate(Subsection);
+  }
+  void FetchNext()
+  {
+    do
+    {
+      if (CurrentIterator)
+      {
+        while (CurrentIterator->HasNext())
+        {
+          CurrentIterator->Next();
+          const char* key = CurrentIterator->GetKey();
+          if (!FindIterated(key))
+          {
+            AddIterated(key);
+            return;
+          }
+        }
+      }
+      FetchNextIterator();
+      CurrentIterator = nextIterator;
+    }
+    while (CurrentIterator);
+  }
 public:
 
   void ClearIterated()
@@ -105,15 +150,14 @@ public:
   }
 
   csConfigManagerIterator(csConfigManager *cfg, const char *sub)
-    : scfImplementationType (this), Config (cfg)
+    : scfImplementationType (this), Config (cfg), Subsection (sub)
   {
     CurrentDomain = Config->LastDomain;
-    Subsection = csStrNew(sub);
+    FetchNext();
   }
   virtual ~csConfigManagerIterator()
   {
     Config->RemoveIterator(this);
-    delete[] Subsection;
     ClearIterated();
   }
   virtual iConfigFile *GetConfigFile() const
@@ -129,55 +173,82 @@ public:
     CurrentIterator = 0;
     CurrentDomain = Config->LastDomain;
     ClearIterated();
+    FetchNextIterator();
   }
+
   virtual bool Next()
   {
-    if (CurrentIterator)
-    {
-      if (CurrentIterator->Next())
-      {
-        if (FindIterated(CurrentIterator->GetKey()))
-          return Next();
-        AddIterated(CurrentIterator->GetKey());
-        return true;
-      }
-      else
-      {
-        CurrentIterator = 0;
-      }
-    }
-    // move to next domain (which actually means previous domain!)
-    if (CurrentDomain->Prev == 0)
-      return false;
-    CurrentDomain = CurrentDomain->Prev;
-    if (CurrentDomain->Cfg == 0)
-      return false;
-    CurrentIterator = CurrentDomain->Cfg->Enumerate(Subsection);
-    return Next();
+    if (!CurrentIterator) return false;
+    currentKey = CurrentIterator->GetKey ();
+    currentValue = CurrentIterator->GetStr ();
+    currentComment = CurrentIterator->GetComment ();
+    FetchNext();
+    return true;
   }
+
+  bool HasNext()
+  {
+    return CurrentIterator.IsValid();
+  }
+
   virtual const char *GetKey(bool Local) const
   {
-    return (CurrentIterator ? CurrentIterator->GetKey(Local) : 0);
+    return currentKey ? currentKey + (Local ? Subsection.Length () : 0) : 0;
   }
   virtual int GetInt() const
   {
-    return (CurrentIterator ? CurrentIterator->GetInt() : 0);
+    return currentValue ? atoi (currentValue) : 0;
   }
   virtual float GetFloat() const
   {
-    return (CurrentIterator ? CurrentIterator->GetFloat() : 0.0f);
+    return currentValue ? atof (currentValue) : 0.0f;
   }
   virtual const char *GetStr() const
   {
-    return (CurrentIterator ? CurrentIterator->GetStr() : "");
+    return currentValue ? currentValue : "";
   }
   virtual bool GetBool() const
   {
-    return (CurrentIterator ? CurrentIterator->GetBool() : false);
+  return (currentValue &&
+    (strcasecmp(currentValue, "true") == 0 ||
+     strcasecmp(currentValue, "yes" ) == 0 ||
+     strcasecmp(currentValue, "on"  ) == 0 ||
+     strcasecmp(currentValue, "1"   ) == 0));
+  }
+  virtual csPtr<iStringArray> GetTuple() const
+  {
+    if (!currentValue)
+      return 0;
+
+    scfStringArray *items = new scfStringArray;		// the output list
+    csString item;
+
+    const char *sinp = currentValue;
+    const char *comp;
+    size_t len;
+    bool finished = false;
+
+    while (!finished)
+    {
+      comp = strchr (sinp, ',');
+      if (!comp)
+      {
+        finished = true;
+        comp = &sinp [strlen (sinp)];
+      }
+      len = strlen (sinp) - strlen (comp);
+      item = csString (sinp, len);
+      item.Trim ();
+      sinp = comp + 1;
+      items->Push (item);
+    }
+
+    csPtr<iStringArray> v(items);
+    return v;
   }
   virtual const char *GetComment() const
   {
-    return (CurrentIterator ? CurrentIterator->GetComment() : 0);
+    return currentComment ? currentComment : 0;
   }
 };
 
@@ -221,7 +292,7 @@ void csConfigManager::CleanUp ()
   }
   // every iterator holds a reference to this object, so when this is
   // deleted there shouldn't be any iterators left.
-  CS_ASSERT(Iterators.Length() == 0);
+  CS_ASSERT(Iterators.GetSize () == 0);
 }
 
 void csConfigManager::AddDomain(iConfigFile *Config, int Priority)
@@ -436,6 +507,14 @@ bool csConfigManager::GetBool(const char *Key, bool Def) const
   return Def;
 }
 
+csPtr<iStringArray> csConfigManager::GetTuple(const char *Key) const
+{
+  for (csConfigDomain *d=LastDomain; d!=0; d=d->Prev)
+    if (d->Cfg && d->Cfg->KeyExists(Key))
+      return d->Cfg->GetTuple(Key);
+  return 0;
+}
+
 const char *csConfigManager::GetComment(const char *Key) const
 {
   for (csConfigDomain *d=LastDomain; d!=0; d=d->Prev) {
@@ -470,6 +549,12 @@ void csConfigManager::SetBool (const char *Key, bool Value)
   ClearKeyAboveDynamic(Key);
 }
 
+void csConfigManager::SetTuple (const char *Key, iStringArray* Value)
+{
+  DynamicDomain->Cfg->SetTuple(Key, Value);
+  ClearKeyAboveDynamic(Key);
+}
+
 bool csConfigManager::SetComment (const char *Key, const char *Text)
 {
   if (!DynamicDomain->Cfg->SetComment(Key, Text)) return false;
@@ -549,7 +634,7 @@ void csConfigManager::FlushRemoved(size_t n)
 
 size_t csConfigManager::FindRemoved(const char *Name) const
 {
-  for (size_t i=0; i<Removed.Length(); i++)
+  for (size_t i=0; i<Removed.GetSize (); i++)
   {
     iConfigFile *cfg = Removed[i];
     if (cfg->GetFileName())
