@@ -19,6 +19,7 @@
 
 #include "cssysdef.h"
 #include "csutil/csobject.h"
+#include "csutil/util.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -44,7 +45,7 @@ public:
 
   virtual bool HasNext() const
   {
-    if (object->Children == 0 || position >= object->Children->Length())
+    if (object->Children == 0 || position >= object->Children->GetSize ())
       return false;
 
     return true;
@@ -52,7 +53,7 @@ public:
 
   virtual iObject* Next()
   {
-    if (object->Children == 0 || position >= object->Children->Length())
+    if (object->Children == 0 || position >= object->Children->GetSize ())
       return 0;
 
     iObject* result = object->Children->Get(position);
@@ -211,7 +212,7 @@ void csObject::ObjRemoveAll ()
     return;
 
   size_t i;
-  for (i=Children->Length (); i>0; i--)
+  for (i=Children->GetSize (); i>0; i--)
   {
     const size_t idx = i - 1;
     iObject* child = Children->Get (idx);
@@ -241,7 +242,7 @@ iObject* csObject::GetChild (int InterfaceID, int Version,
   }
 
   size_t i;
-  for (i = 0; i < Children->Length (); i++)
+  for (i = 0; i < Children->GetSize (); i++)
   {
     if (Name)
     {
@@ -266,7 +267,7 @@ iObject* csObject::GetChild (const char *Name) const
   if (!Children || !Name)
     return 0;
 
-  for (size_t i = 0; i < Children->Length (); i++)
+  for (size_t i = 0; i < Children->GetSize (); i++)
   {
     const char *ThisName = Children->Get (i)->GetName ();
     if (ThisName != 0 && !strcmp (ThisName, Name))
@@ -284,7 +285,7 @@ void csObject::FireNameChangeListeners (const char* oldname,
 	const char* newname)
 {
   size_t i;
-  for (i = 0 ; i < listeners.Length () ; i++)
+  for (i = 0 ; i < listeners.GetSize () ; i++)
     listeners[i]->NameChanged (this, oldname, newname);
 }
 
