@@ -28,6 +28,7 @@
 
 #if defined (WIN32) && !defined (__CYGWIN32__)
 /* It's not Unix, really.  See?  Capital letters.  */
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #define getpid() GetCurrentProcessId()
 #endif
@@ -217,7 +218,7 @@ exchange (char **argv)
     {
       /* We must extend the array.  The user plays games with us and
 	 presents new arguments.  */
-      char *new_str = malloc (top + 1);
+      char *new_str = cs_malloc (top + 1);
       if (new_str == 0)
 	nonoption_flags_len = nonoption_flags_max_len = 0;
       else
@@ -323,7 +324,7 @@ static const char *_getopt_initialize (int /*argc*/, char* const* /*argv*/,
 	      if (nonoption_flags_max_len < argc)
 		nonoption_flags_max_len = argc;
 	      __getopt_nonoption_flags =
-		(char *) malloc (nonoption_flags_max_len);
+		(char *) cs_malloc (nonoption_flags_max_len);
 	      if (__getopt_nonoption_flags == 0)
 		nonoption_flags_max_len = -1;
 	      else
