@@ -19,6 +19,7 @@
 #include "cssysdef.h"
 #include "csutil/sysfunc.h"
 #include "csutil/event.h"
+#include "csutil/eventnames.h"
 #include "csutil/scf_implementation.h"
 #include "iutil/event.h"
 #include "iutil/eventh.h"
@@ -74,10 +75,10 @@ public:
 
 bool csDefaultRunLoop (iObjectRegistry* r)
 {
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY(r, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (r));
   if (!q)
     return false;
-  csRef<iVirtualClock> vc (CS_QUERY_REGISTRY(r, iVirtualClock));
+  csRef<iVirtualClock> vc (csQueryRegistry<iVirtualClock> (r));
 
   csRef<csDefaultQuitEventHandler> eh;
   eh.AttachNew (new csDefaultQuitEventHandler (r));
