@@ -20,8 +20,8 @@
 //-----------------------------------------------------------------------------
 #include "OSXDelegate.h"
 #include "OSXMenu.h"
-#include "iutil/evdefs.h"
 #include "csconfig.h"		// CS_PLATFORM_MACOSXS
+#include "iutil/evdefs.h"
 #include <stdio.h>
 #import <AppKit/NSApplication.h>
 #import <AppKit/NSCursor.h>
@@ -35,6 +35,7 @@
 #import <Foundation/NSString.h>
 #import <Foundation/NSUserDefaults.h>
 
+
 typedef void* OSXDelegateHandle;
 typedef void* OSXEventHandle;
 typedef void* OSXViewHandle;
@@ -331,7 +332,7 @@ ND_PROTO(void,flush_graphics_context)(OSXDelegateHandle handle)
   switch (*raw)
   {
     case K_ESCAPE:    *cooked = CSKEY_ESC;       break;
-    case K_RETURN:    *cooked = CSKEY_ENTER;     break;	// *1*
+    case K_RETURN:    *raw = CSKEY_ENTER;        *cooked = CSKEY_ENTER;     break;	// *1*
     case K_TAB:       *cooked = CSKEY_TAB;       break;
     case K_BACKSPACE: *cooked = CSKEY_BACKSPACE; break;
     case K_DELETE:    *cooked = CSKEY_BACKSPACE; break; // *2*
@@ -505,23 +506,23 @@ ND_PROTO(void,flush_graphics_context)(OSXDelegateHandle handle)
   { [self mouseMoved:p forView:v]; }
 
 - (void)mouseUp:(NSEvent*)p forView:(NSView*)v
-  { [self mouseUp:p forView:v button:1]; }
+  { [self mouseUp:p forView:v button:csmbLeft]; }
 
 - (void)mouseDown:(NSEvent*)p forView:(NSView*)v
-  { [self mouseDown:p forView:v button:1]; }
+  { [self mouseDown:p forView:v button:csmbLeft]; }
 
 - (void)rightMouseDragged:(NSEvent*)p forView:(NSView*)v
   { [self mouseMoved:p forView:v]; }
 
 - (void)rightMouseUp:(NSEvent*)p forView:(NSView*)v
-  { [self mouseUp:p forView:v button:2]; }
+  { [self mouseUp:p forView:v button:csmbRight]; }
 
 - (void)rightMouseDown:(NSEvent*)p forView:(NSView*)v
 {
   if (paused)
     [[v nextResponder] rightMouseDown:p]; // Allow main menu to pop up.
   else
-    [self mouseDown:p forView:v button:2];
+    [self mouseDown:p forView:v button:csmbRight];
 }
 
 - (void)dispatchEvent:(NSEvent*)e forView:(NSView*)v
@@ -609,8 +610,8 @@ ND_PROTO(void,dispatch_event)
 {
   NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
   NSMenu* const menu = OSXMenuGenerate(self, style, config);
-  [menu setTitle:[[NSProcessInfo processInfo] processName]];
-  [NSApp setMainMenu:menu];
+  [[OSXApplication sharedApplication] setMainMenu:menu];
+  [[OSXApplication sharedApplication] setAppleMenu:[[menu itemAtIndex:0] submenu]];
   [pool release];
 }
 
@@ -722,6 +723,9 @@ ND_PROTO(void,stop_event_loop)(OSXDelegateHandle handle)
     selector:@selector(applicationDefined:)
     name:OSXAppDefinedNotification
     object:0];
+
+  [NSApp activateIgnoringOtherApps:YES];
+
   [pool release];
   return controller;
 }
