@@ -200,7 +200,7 @@ static void CompactMemEntries (unsigned long older_age)
     {
       if (mem_table[i].freed && mem_table[i].age < older_age)
       {
-        free (mem_table[i].start-DETECT_WALL-4);
+        cs_free (mem_table[i].start-DETECT_WALL-4);
       }
       else
       {
@@ -303,7 +303,7 @@ static void MemoryCheck ()
         ShowBlockInfo (me);
         csPrintf("CHK: Size in table does not correspond to size in block!\n");
         fflush (stdout);
-        DEBUG_BREAK;
+        CS_DEBUG_BREAK;
       }
       if (me.freed)
       {
@@ -312,14 +312,14 @@ static void MemoryCheck ()
           ShowBlockInfo (me);
           csPrintf ("CHK: Bad start of block for freed block!\n");
           fflush (stdout);
-          DEBUG_BREAK;
+          CS_DEBUG_BREAK;
         }
         if (strncmp (rc+4+DETECT_WALL+s, DETECTFREE, DETECT_WALL) != 0)
         {
           ShowBlockInfo (me);
           csPrintf ("CHK: Bad end of block for freed block!\n");
           fflush (stdout);
-          DEBUG_BREAK;
+          CS_DEBUG_BREAK;
         }
 #if DETECT_KEEP_FREE_MEMORY
         unsigned int j;
@@ -330,7 +330,7 @@ static void MemoryCheck ()
             ShowBlockInfo (me);
             csPrintf ("CHK: Freed memory is used at offset (%u)!\n", j);
             fflush (stdout);
-            DEBUG_BREAK;
+            CS_DEBUG_BREAK;
           }
         }
 #endif
@@ -342,14 +342,14 @@ static void MemoryCheck ()
           ShowBlockInfo (me);
           csPrintf ("CHK: Bad start of block!\n");
           fflush (stdout);
-          DEBUG_BREAK;
+          CS_DEBUG_BREAK;
         }
         if (strncmp (rc+4+DETECT_WALL+s, DETECT, DETECT_WALL_SAME) != 0)
         {
           ShowBlockInfo (me);
           csPrintf ("CHK: Bad end of block!\n");
           fflush (stdout);
-          DEBUG_BREAK;
+          CS_DEBUG_BREAK;
         }
       }
     }
@@ -384,7 +384,7 @@ static void DumpError (const char* msg, int info, char* rc)
   fflush (stdout);
   if (do_crash)
   {
-    DEBUG_BREAK;
+    CS_DEBUG_BREAK;
   }
 }
 
@@ -400,7 +400,7 @@ void* operator new (size_t s)
   if (global_age % DETECT_CHECK_MEMORY == 0) MemoryCheck ();
 #endif
   if (s <= 0) DumpError ("BAD SIZE in new %zu\n", s, 0);
-  char* rc = (char*)malloc (s+4+DETECT_WALL+DETECT_WALL);
+  char* rc = (char*)cs_malloc (s+4+DETECT_WALL+DETECT_WALL);
   memcpy (rc, DETECT, DETECT_WALL);
   memcpy (rc+DETECT_WALL, &s, 4);
   memcpy (rc+DETECT_WALL+4+s, DETECT, DETECT_WALL);
@@ -431,7 +431,7 @@ void* operator new[] (size_t s)
   if (global_age % DETECT_CHECK_MEMORY == 0) MemoryCheck ();
 #endif
   if (s <= 0) DumpError ("BAD SIZE in new[] %zu\n", s, 0);
-  char* rc = (char*)malloc (s+4+DETECT_WALL+DETECT_WALL);
+  char* rc = (char*)cs_malloc (s+4+DETECT_WALL+DETECT_WALL);
   memcpy (rc, DETECTAR, DETECT_WALL);
   memcpy (rc+DETECT_WALL, &s, 4);
   memcpy (rc+DETECT_WALL+4+s, DETECTAR, DETECT_WALL);
@@ -489,10 +489,10 @@ void operator delete (void* p)
   me->age = global_age;
 # else
   me->start = 0;
-  free (rc);
+  cs_free (rc);
 # endif
 #else
-  free (rc);
+  cs_free (rc);
 #endif
 }
 
@@ -533,10 +533,10 @@ void operator delete[] (void* p)
   me->age = global_age;
 # else
   me->start = 0;
-  free (rc);
+  cs_free (rc);
 # endif
 #else
-  free (rc);
+  cs_free (rc);
 #endif
 }
 
@@ -553,7 +553,7 @@ void* operator new (size_t s, void* filename, int line)
 {
   alloc_total += s;
   alloc_cnt++;
-  uint32* rc = (uint32*)malloc (s+8);
+  uint32* rc = (uint32*)cs_malloc (s+8);
   *rc++ = 0xdeadbeef;
   *rc++ = s;
   csPrintf ("+ %p %zu %zu %s\n",
@@ -565,7 +565,7 @@ void* operator new[] (size_t s, void* filename, int line)
 {
   alloc_total += s;
   alloc_cnt++;
-  uint32* rc = (uint32*)malloc (s+8);
+  uint32* rc = (uint32*)cs_malloc (s+8);
   *rc++ = 0xdeadbeef;
   *rc++ = s;
   csPrintf ("+ %p %zu %zu %s\n",
@@ -581,7 +581,7 @@ void operator delete (void* p)
     CS_ASSERT (*rc == 0xdeadbeef);
     alloc_total -= rc[1];
     alloc_cnt--;
-    free ((void*)rc);
+    cs_free ((void*)rc);
     csPrintf ("- %p %zu %zu\n", &alloc_total, alloc_total, alloc_cnt);
   }
 }
@@ -593,7 +593,7 @@ void operator delete[] (void* p)
     CS_ASSERT (*rc == 0xdeadbeef);
     alloc_total -= rc[1];
     alloc_cnt--;
-    free ((void*)rc);
+    cs_free ((void*)rc);
     csPrintf ("- %p %zu %zu\n", &alloc_total, alloc_total, alloc_cnt);
   }
 }
@@ -614,7 +614,7 @@ void* operator new (size_t s, void* filename, int line)
   alloc_cnt++;
   if (s > 1000) { csPrintf ("new s=%zu tot=%zu/%zu file=%s line=%d\n",
         s, alloc_total, alloc_cnt, filename, line); fflush (stdout); }
-  return (void*)malloc (s);
+  return (void*)cs_malloc (s);
 }
 void* operator new[] (size_t s, void* filename, int line)
 {
@@ -623,15 +623,15 @@ void* operator new[] (size_t s, void* filename, int line)
   if (s > 1000)
     csPrintf ("new[] s=%zu tot=%zu/%zu file=%s line=%d\n",
               s, alloc_total, alloc_cnt, filename, line); fflush (stdout);
-  return (void*)malloc (s);
+  return (void*)cs_malloc (s);
 }
 void operator delete (void* p)
 {
-  if (p) free (p);
+  if (p) cs_free (p);
 }
 void operator delete[] (void* p)
 {
-  if (p) free (p);
+  if (p) cs_free (p);
 }
 #endif  // CS_EXTENSIVE_MEMDEBUG_IMPLEMENT
 
@@ -645,19 +645,19 @@ void operator delete[] (void* p)
 #undef new
 void* operator new (size_t s, void*, int)
 {
-  return (void*)malloc (s);
+  return (void*)cs_malloc (s);
 }
 void* operator new[] (size_t s, void*, int)
 {
-  return (void*)malloc (s);
+  return (void*)cs_malloc (s);
 }
 void operator delete (void* p)
 {
-  if (p) free (p);
+  if (p) cs_free (p);
 }
 void operator delete[] (void* p)
 {
-  if (p) free (p);
+  if (p) cs_free (p);
 }
 #endif  // CS_EXTENSIVE_MEMDEBUG_IMPLEMENT
 
@@ -686,7 +686,8 @@ class csMemTrackerModule
 {
 public:
   char* Class;          // Name of class or 0 for application level.
-  csMemTrackerInfo* mti_table[10000];
+  static const int mti_table_max = 10000;
+  csMemTrackerInfo* mti_table[mti_table_max];
   int mti_table_count;
 
   csMemTrackerModule ()
@@ -694,25 +695,26 @@ public:
     mti_table_count = 0;
   }
 
-  void InsertBefore (int idx, char* filename)
+  void InsertBefore (int idx, const char* filename)
   {
     int tomove = mti_table_count - idx;
     if (tomove > 0)
       memmove (mti_table+idx+1, mti_table+idx,
           sizeof (csMemTrackerInfo*) * tomove);
     mti_table_count++;
-    mti_table[idx] = (csMemTrackerInfo*)malloc (sizeof (csMemTrackerInfo));
+    CS_ASSERT(mti_table_count <= mti_table_max);
+    mti_table[idx] = (csMemTrackerInfo*)cs_malloc (sizeof (csMemTrackerInfo));
     mti_table[idx]->Init (filename);
   }
 
   csMemTrackerInfo* FindInsertMtiTableEntry (
-        char* filename, int start, int end)
+        const char* filename, int start, int end)
   {
     // Binary search.
     if (mti_table_count <= 0)
     {
       mti_table_count++;
-      mti_table[0] = (csMemTrackerInfo*)malloc (sizeof (csMemTrackerInfo));
+      mti_table[0] = (csMemTrackerInfo*)cs_malloc (sizeof (csMemTrackerInfo));
       mti_table[0]->Init (filename);
       return mti_table[0];
     }
@@ -762,7 +764,7 @@ public:
     }
   }
 
-  csMemTrackerInfo* FindInsertMtiTableEntry (char* filename)
+  csMemTrackerInfo* FindInsertMtiTableEntry (const char* filename)
   {
     return FindInsertMtiTableEntry (filename, 0, mti_table_count-1);
   }
@@ -772,7 +774,7 @@ public:
     int i;
     csPrintf ("-----------------------------------------------------\n");
     csPrintf ("Module: %s\n", Class);
-    csPrintf ("     current      max current#     max# file\n");
+    csPrintf ("       bytes   ...max   blocks   ...max #alloc #free  file\n");
     size_t total_current_alloc = 0;
     int total_current_count = 0;
     for (i = 0 ; i < mti_table_count ; i++)
@@ -780,15 +782,17 @@ public:
       csMemTrackerInfo* mti = mti_table[i];
       if (!summary_only)
       {
-        csPrintf ("    %8zu %8zu %8d %8d %s\n", mti->current_alloc,
+        csPrintf ("    %8zu %8zu %8d %8d %6zu %6zu %s\n", mti->current_alloc,
             mti->max_alloc, mti->current_count, mti->max_count,
-            mti->file);
+            mti->totalAllocCount, mti->totalDeallocCount, mti->file);
       }
       total_current_alloc += mti->current_alloc;
       total_current_count += mti->current_count;
     }
-    csPrintf ("total_alloc=%zu total_count=%d Module=%s\n",
+  #define Sigma "\xE2\x88\x91"
+    csPrintf (Sigma "bytes=%zu " Sigma "blocks=%d Module=%s\n",
         total_current_alloc, total_current_count, Class);
+  #undef Sigma
     fflush (stdout);
   }
 };
@@ -800,7 +804,8 @@ class csMemTrackerRegistry : public scfImplementation1<csMemTrackerRegistry,
                                                        iMemoryTracker>
 {
 public:
-  csMemTrackerModule* modules[500];     // @@@ Hardcoded!
+  static const int max_modules = 500;
+  csMemTrackerModule* modules[max_modules];     // @@@ Hardcoded!
   int num_modules;
 
   csMemTrackerRegistry ()
@@ -817,11 +822,66 @@ public:
     csMemTrackerModule* mod = new csMemTrackerModule ();
     mod->Class = Class;
     modules[num_modules++] = mod;
+    CS_ASSERT(num_modules <= max_modules);
     return mod;
   }
 
   virtual void Dump (bool summary_only)
   {
+#if defined(CS_PLATFORM_WIN32)
+    /* Yes, that's horribly platform-dependent, and this may not be the right
+     * place. However, the heap statistics are somewhat useful, since they
+     * can e.g. indicate fragmentation. */
+    DWORD heapNum = GetProcessHeaps (0, 0);
+    CS_ALLOC_STACK_ARRAY(HANDLE, heaps, heapNum);
+    GetProcessHeaps (heapNum, heaps);
+
+    for (DWORD i = 0; i < heapNum; i++)
+    {
+      const HANDLE heap = heaps[i];
+
+      struct RegionInfo
+      {
+        size_t committedSize;
+        size_t uncommittedSize;
+        size_t busySize;
+        size_t blocks;
+
+        RegionInfo() : committedSize (0), uncommittedSize (0),
+          busySize (0), blocks (0) {}
+      };
+      RegionInfo regions[256];
+
+      PROCESS_HEAP_ENTRY entry;
+      entry.lpData = 0;
+      bool hasEntry = HeapWalk (heap, &entry) != FALSE;
+      while (hasEntry)
+      {
+        RegionInfo& region = regions[entry.iRegionIndex];
+        if (entry.wFlags & PROCESS_HEAP_REGION)
+        {
+          region.committedSize = entry.Region.dwCommittedSize;
+          region.uncommittedSize = entry.Region.dwUnCommittedSize;
+        }
+        else if (entry.wFlags & PROCESS_HEAP_ENTRY_BUSY)
+        {
+          region.busySize += entry.cbData;
+        }
+        if (!(entry.wFlags & (PROCESS_HEAP_REGION | PROCESS_HEAP_UNCOMMITTED_RANGE)))
+          region.blocks++;
+        hasEntry = HeapWalk (heap, &entry) != FALSE;
+      }
+      for (int i = 0; i < 256; i++)
+      {
+        const RegionInfo& region = regions[i];
+        if ((region.committedSize + region.uncommittedSize) == 0) continue;
+        csPrintf ("heap %p region %d: %9zu/%9zu/%9zu bytes in %6zu blocks\n", (void*)heap, i,
+          region.committedSize, region.uncommittedSize, region.busySize, region.blocks);
+      }
+    }
+    fflush (stdout);
+#endif
+
     int i;
     for (i = 0 ; i < num_modules ; i++)
     {
@@ -843,17 +903,16 @@ void mtiRegisterModule (char* Class)
 
   if (iSCF::SCF->object_reg)
   {
-    csRef<iMemoryTracker> mtiTR = CS_QUERY_REGISTRY_TAG_INTERFACE (
-        iSCF::SCF->object_reg, "crystalspace.utilities.memorytracker",
-        iMemoryTracker);
+    csRef<iMemoryTracker> mtiTR = csQueryRegistryTagInterface<iMemoryTracker> (
+        iSCF::SCF->object_reg, "crystalspace.utilities.memorytracker");
     if (!mtiTR)
     {
-      mtiTR = csPtr<iMemoryTracker>((iMemoryTracker*)new csMemTrackerRegistry);
+      mtiTR.AttachNew (new csMemTrackerRegistry);
       iSCF::SCF->object_reg->Register (mtiTR,
         "crystalspace.utilities.memorytracker");
     }
-    mti_this_module = ((csMemTrackerRegistry*)(iMemoryTracker*)mtiTR)
-        ->NewMemTrackerModule (Class);
+    mti_this_module = (static_cast<csMemTrackerRegistry*> (
+      (iMemoryTracker*)mtiTR))->NewMemTrackerModule (Class);
   }
   else
   {
@@ -862,19 +921,28 @@ void mtiRegisterModule (char* Class)
   }
 }
 
-csMemTrackerInfo* mtiRegisterAlloc (size_t s, void* filename)
+csMemTrackerInfo* mtiRegisterAlloc (size_t s, const char* filename)
 {
   if (mti_this_module == 0)
     return 0;   // Don't track this alloc yet.
 
   csMemTrackerInfo* mti = mti_this_module->FindInsertMtiTableEntry (
-        (char*)filename);
+        filename);
   mti->current_count++;
   mti->current_alloc += s;
   if (mti->current_count > mti->max_count)
     mti->max_count = mti->current_count;
   if (mti->current_alloc > mti->max_alloc)
     mti->max_alloc = mti->current_alloc;
+  mti->totalAllocCount++;
+  return mti;
+}
+
+csMemTrackerInfo* mtiRegister (const char* info)
+{
+  if (!mti_this_module) return 0;
+  csMemTrackerInfo* mti = mti_this_module->FindInsertMtiTableEntry (
+        info);
   return mti;
 }
 
@@ -884,6 +952,7 @@ void mtiRegisterFree (csMemTrackerInfo* mti, size_t s)
   {
     mti->current_count--;
     mti->current_alloc -= s;
+    mti->totalDeallocCount++;
   }
 }
 
@@ -897,28 +966,32 @@ void mtiUpdateAmount (csMemTrackerInfo* mti, int dcount, int dsize)
       mti->max_count = mti->current_count;
     if (mti->current_alloc > mti->max_alloc)
       mti->max_alloc = mti->current_alloc;
+    if (dcount > 0)
+      mti->totalAllocCount++;
+    else if (dcount < 0)
+      mti->totalDeallocCount++;
   }
 }
 
 void* operator new (size_t s, void* filename, int /*line*/)
 {
   //CS_ASSERT (s > 0);
-  uintptr_t* rc = (uintptr_t*)malloc (s+4*sizeof(uintptr_t));
+  uintptr_t* rc = (uintptr_t*)cs_malloc (s+4*sizeof(uintptr_t));
   memset (rc, 0xfe, s+4*sizeof(uintptr_t));
   *rc++ = s;
   *rc++ = 0xbeebbeeb;
-  *rc++ = (uintptr_t)mtiRegisterAlloc (s, filename);
+  *rc++ = (uintptr_t)mtiRegisterAlloc (s, (const char*)filename);
   *rc++ = 0xdeadbeef;
   return (void*)rc;
 }
 void* operator new[] (size_t s, void* filename, int /*line*/)
 {
   //CS_ASSERT (s > 0);
-  uintptr_t* rc = (uintptr_t*)malloc (s+4*sizeof(uintptr_t));
+  uintptr_t* rc = (uintptr_t*)cs_malloc (s+4*sizeof(uintptr_t));
   memset (rc, 0xfe, s+4*sizeof(uintptr_t));
   *rc++ = s;
   *rc++ = 0xfeedbeef;
-  *rc++ = (uintptr_t)mtiRegisterAlloc (s, filename);
+  *rc++ = (uintptr_t)mtiRegisterAlloc (s, (const char*)filename);
   *rc++ = 0xdeadbeef;
   return (void*)rc;
 }
@@ -927,10 +1000,14 @@ void operator delete (void* p)
   if (p)
   {
     uintptr_t* rc = ((uintptr_t*)p)-4;
-    if (rc[3] != 0xdeadbeef) { free (p); return; }
+    if (rc[3] != 0xdeadbeef) 
+    { 
+      platform_free (p); // Not nice, but at least try.
+      return; 
+    }
     size_t s = rc[0];
     csMemTrackerInfo* mti = (csMemTrackerInfo*)rc[2];
-    free ((void*)rc);
+    cs_free ((void*)rc);
     mtiRegisterFree (mti, s);
   }
 }
@@ -939,10 +1016,14 @@ void operator delete[] (void* p)
   if (p)
   {
     uintptr_t* rc = ((uintptr_t*)p)-4;
-    if (rc[3] != 0xdeadbeef) { free (p); return; }
+    if (rc[3] != 0xdeadbeef) 
+    { 
+      platform_free (p); // Not nice, but at least try.
+      return; 
+    }
     size_t s = rc[0];
     csMemTrackerInfo* mti = (csMemTrackerInfo*)rc[2];
-    free ((void*)rc);
+    cs_free ((void*)rc);
     mtiRegisterFree (mti, s);
   }
 }
