@@ -16,32 +16,22 @@
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "cssysdef.h"
 #include "csutil/util.h"
 #include "csutil/vfscache.h"
+#include "iutil/databuff.h"
 #include "iutil/objreg.h"
 #include "iutil/vfs.h"
 #include "ivaria/reporter.h"
 
 //------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csVfsCacheManager)
-  SCF_IMPLEMENTS_INTERFACE (iCacheManager)
-SCF_IMPLEMENT_IBASE_END
-
 csVfsCacheManager::csVfsCacheManager (iObjectRegistry* object_reg,
 	const char* vfsdir)
+  : scfImplementationType (this), object_reg (object_reg),
+  vfsdir (csStrNew (vfsdir)),
+  vfs (0), current_type (0), current_scope (0), readonly (false)
 {
-  SCF_CONSTRUCT_IBASE (0);
-  csVfsCacheManager::object_reg = object_reg;
-  csVfsCacheManager::vfsdir = csStrNew (vfsdir);
-  vfs = 0;
-  current_type = 0;
-  current_scope = 0;
 }
 
 csVfsCacheManager::~csVfsCacheManager ()
@@ -49,28 +39,27 @@ csVfsCacheManager::~csVfsCacheManager ()
   delete[] vfsdir;
   delete[] current_type;
   delete[] current_scope;
-  SCF_DESTRUCT_IBASE ();
 }
 
 iVFS* csVfsCacheManager::GetVFS ()
 {
   if (!vfs)
-    vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+    vfs = csQueryRegistry<iVFS> (object_reg);
   return vfs;
 }
 
-void csVfsCacheManager::CacheName (char* buf, const char* type,
+void csVfsCacheManager::CacheName (csStringFast<512>& buf, const char* type,
 	const char* scope, uint32 id)
 {
   if (id == (uint32)~0)
   {
     if (scope == 0)
-      sprintf (buf, "%s", type);
+      buf.Format ("%s", type);
     else
-      sprintf (buf, "%s/%s", type, scope);
+      buf.Format ("%s/%s", type, scope);
   }
   else
-    sprintf (buf, "%s/%s/%d", type, scope, id);
+    buf.Format ("%s/%s/%" PRIu32 , type, scope, id);
 }
 
 void csVfsCacheManager::SetCurrentType (const char* type)
@@ -91,22 +80,23 @@ void csVfsCacheManager::SetCurrentScope (const char* scope)
     current_scope = 0;
 }
 
-bool csVfsCacheManager::CacheData (void* data, uint32 size,
+bool csVfsCacheManager::CacheData (const void* data, size_t size,
   	const char* type, const char* scope, uint32 id)
 {
-  char buf[512];
+  if (readonly) return true;
+  csStringFast<512> buf;
   GetVFS ()->PushDir ();
   GetVFS ()->ChDir (vfsdir);
   CacheName (buf, type ? type : current_type,
   	scope ? scope : current_scope, id);
-  csRef<iFile> cf (GetVFS ()->Open (buf, VFS_FILE_WRITE));
+  csRef<iFile> cf = GetVFS ()->Open (buf, VFS_FILE_WRITE);
   GetVFS ()->PopDir ();
 
   if (!cf)
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
     	"crystalspace.vfscachemgr.createfile",
-	"Could not create file '%s' in VFS dir '%s'\n", buf, vfsdir);
+	"Could not create file '%s' in VFS dir '%s'\n", buf.GetData(), vfsdir);
     return false;
   }
 
@@ -115,7 +105,7 @@ bool csVfsCacheManager::CacheData (void* data, uint32 size,
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
     	"crystalspace.vfscachemgr.writefile",
-	"Could not write file '%s' in VFS dir '%s'\n", buf, vfsdir);
+	"Could not write file '%s' in VFS dir '%s'\n", buf.GetData(), vfsdir);
     return false;
   }
 
@@ -125,7 +115,7 @@ bool csVfsCacheManager::CacheData (void* data, uint32 size,
 csPtr<iDataBuffer> csVfsCacheManager::ReadCache (
   	const char* type, const char* scope, uint32 id)
 {
-  char buf[512];
+  csStringFast<512> buf;
   GetVFS ()->PushDir ();
   GetVFS ()->ChDir (vfsdir);
   CacheName (buf, type ? type : current_type,
@@ -153,3 +143,9 @@ bool csVfsCacheManager::ClearCache (const char* type, const char* scope,
   return false;
 }
 
+void csVfsCacheManager::Flush ()
+{
+  if (!readonly)
+    GetVFS ()->Sync ();
+}
+
