@@ -21,13 +21,14 @@
 #include "csutil/csstring.h"
 #include "csutil/csuctransform.h"
 #include "csutil/dirtyaccessarray.h"
+#include "csutil/measuretime.h"
 #include "csutil/sysfunc.h"
-#include "csutil/timemeasure.h"
 #include "csutil/util.h"
 #include "callstack-dbghelp.h"
 #include "csutil/win32/callstack.h"
 #include "csutil/win32/wintools.h"
 
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 
 #include "csutil/win32/DbgHelpAPI.h"
@@ -35,13 +36,13 @@
 
 //#define CALLSTACK_PROFILE
 
-namespace CrystalSpace
+namespace CS
 {
 namespace Debug
 {
 
 #ifdef CALLSTACK_PROFILE
-#define MEASURE_FUNCTION csMeasureTime measureLocal ("%s", CS_FUNCTION_NAME)
+#define MEASURE_FUNCTION CS::MeasureTime measureLocal ("%s", CS_FUNCTION_NAME)
 #define MEASURE_INTERMEDIATE measureLocal.PrintIntermediate (" %d", __LINE__)
 #else
 #define MEASURE_FUNCTION
@@ -189,7 +190,7 @@ static bool CreateCallStack (HANDLE hProc, HANDLE hThread, CONTEXT& context,
     {
       CallStackEntry entry;
       entry.address = (void*)(uintptr_t)frame.AddrPC.Offset;
-      entry.paramOffs = params.Length();
+      entry.paramOffs = params.GetSize ();
       entry.paramNum = fast ? csParamUnknown : GetParams (frame, params) ;
       entries.Push (entry);
     }
@@ -601,7 +602,7 @@ void* CallStackNameResolverDbgHelp::OpenParamSymbols (void* addr)
 bool CallStackNameResolverDbgHelp::GetParamName (void* h, size_t n, csString& str)
 {
   csArray<csString>* names = (csArray<csString>*)h;
-  if (n >=  names->Length()) return false;
+  if (n >=  names->GetSize ()) return false;
   str = names->Get (n);
   return true;
 }
@@ -644,9 +645,9 @@ bool CallStackNameResolverDbgHelp::GetLineNumber (void* addr, csString& str)
 }
 
 } // namespace Debug
-} // namespace CrystalSpace
+} // namespace CS
 
-using namespace CrystalSpace::Debug;
+using namespace CS::Debug;
 
 csCallStack* cswinCallStackHelper::CreateCallStack (HANDLE hProc, 
                                                     HANDLE hThread, 
@@ -655,7 +656,7 @@ csCallStack* cswinCallStackHelper::CreateCallStack (HANDLE hProc,
 {
   skip += 1; /* Adjust for this function */
   CallStackImpl* stack = new CallStackImpl();
-  if (CrystalSpace::Debug::CreateCallStack (hProc, hThread, context, skip, 
+  if (CS::Debug::CreateCallStack (hProc, hThread, context, skip, 
     fast, stack->entries, stack->params))
     return stack;
   delete stack;
