@@ -38,6 +38,7 @@
 
 #include "cachedll.h"
 
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <tlhelp32.h>
 
@@ -409,9 +410,9 @@ const char* cswinMinidumpWriter::WriteWrappedMinidump (
 
     if (object_reg)  
     {
-      csRef<iVFS> vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
-      csRef<iStandardReporterListener> stdrep = CS_QUERY_REGISTRY (object_reg,
-	iStandardReporterListener);
+      csRef<iVFS> vfs = csQueryRegistry<iVFS> (object_reg);
+      csRef<iStandardReporterListener> stdrep = 
+	csQueryRegistry<iStandardReporterListener> (object_reg);
       if (vfs && stdrep)
       {
 	csRef<iDataBuffer> realConPath = 
@@ -455,8 +456,6 @@ const char* cswinMinidumpWriter::WriteWrappedMinidump (
 
 //#define TEST_EXCEPTION_HANDLER
 
-extern "C" BOOL WINAPI IsDebuggerPresent();
-
 LONG WINAPI cswinMinidumpWriter::ExceptionFilter (
   struct _EXCEPTION_POINTERS* ExceptionInfo)
 {
