@@ -37,13 +37,14 @@
 #include "ivideo/natwin.h"
 #include "ivideo/cursor.h"
 
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <imm.h>
 
 #include "csutil/win32/wintools.h"
 #include "win32kbd.h"
 #include "cachedll.h"
 
-#include <windows.h>
-
 #include <stdio.h>
 #include <time.h>
 
@@ -222,7 +223,7 @@ bool csPlatformStartup(iObjectRegistry* r)
    */
   SetThreadAffinityMask (GetCurrentThread(), 1);
   
-  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (r, iCommandLineParser));
+  csRef<iCommandLineParser> cmdline (csQueryRegistry<iCommandLineParser> (r));
 
   csPathsList* pluginpaths = csGetPluginPaths (cmdline->GetAppPath());
 
@@ -309,7 +310,7 @@ bool csPlatformShutdown(iObjectRegistry* r)
     r->Unregister(assi, "iWin32Assistant");
     Win32Assistant* a = (Win32Assistant*)((iWin32Assistant*)assi);
     a->Shutdown();
-    assistants.DeleteFast (a);
+    assistants.Delete (a);
     return true;
   }
   return false;
@@ -324,7 +325,7 @@ BOOL WINAPI Win32Assistant::ConsoleHandlerRoutine (DWORD dwCtrlType)
     case CTRL_LOGOFF_EVENT:
     case CTRL_SHUTDOWN_EVENT:
       {
-	for (size_t i = 0; i < assistants.Length(); i++)
+	for (size_t i = 0; i < assistants.GetSize (); i++)
 	{
 	  assistants[i]->GetEventOutlet()->ImmediateBroadcast (
 	    csevQuit (assistants[i]->registry), 0);
@@ -335,6 +336,14 @@ BOOL WINAPI Win32Assistant::ConsoleHandlerRoutine (DWORD dwCtrlType)
   return result;
 }
 
+/// Determine whether a standard handle was redirected to a file
+static bool IsStdHandleRedirected (DWORD nHandle)
+{
+  HANDLE file = GetStdHandle (nHandle);
+  return (file != INVALID_HANDLE_VALUE) && (file != NULL) 
+    && (GetFileType (file) != FILE_TYPE_CHAR);
+}
+
 Win32Assistant::Win32Assistant (iObjectRegistry* r) 
   : scfImplementationType (this),
   ApplicationActive (true),
@@ -362,7 +371,7 @@ Win32Assistant::Win32Assistant (iObjectRegistry* r)
 
   use_own_message_loop = true;
 
-  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (r, iCommandLineParser));
+  csRef<iCommandLineParser> cmdline (csQueryRegistry<iCommandLineParser> (r));
   console_window = cmdline->GetBoolOption ("console", console_window);
 
   cmdline_help_wanted = (cmdline->GetOption ("help") != 0);
@@ -388,9 +397,14 @@ Win32Assistant::Win32Assistant (iObjectRegistry* r)
     if (console_window || cmdline_help_wanted)
     {
       AllocConsole ();
-      freopen("CONOUT$", "a", stderr);
-      freopen("CONOUT$", "a", stdout);
-      freopen("CONIN$", "a", stdin);
+      /* "Redirect" C runtime standard files to console, if not redirected 
+       * by the user. */
+      if (!IsStdHandleRedirected (STD_ERROR_HANDLE)) 
+        freopen("CONOUT$", "a", stderr);
+      if (!IsStdHandleRedirected (STD_OUTPUT_HANDLE)) 
+        freopen("CONOUT$", "a", stdout);
+      if (!IsStdHandleRedirected (STD_INPUT_HANDLE)) 
+        freopen("CONIN$", "r", stdin);
     }
   }
 
@@ -438,7 +452,7 @@ Win32Assistant::Win32Assistant (iObjectRegistry* r)
     {
       strcat (apppath, ".ico");
     }
-    appIcon = (HICON)LoadImage (ModuleHandle, apppath, IMAGE_ICON,
+    appIcon = (HICON)LoadImageA (ModuleHandle, apppath, IMAGE_ICON,
       0, 0, LR_DEFAULTSIZE | LR_LOADFROMFILE);
   }
   // finally the default one
@@ -500,7 +514,7 @@ Win32Assistant::Win32Assistant (iObjectRegistry* r)
 
   m_hCursor = LoadCursor (0, IDC_ARROW);
 
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY (registry, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (registry));
   CS_ASSERT (q != 0);
   csEventID events[] = {
     csevPreProcess (registry),
@@ -527,12 +541,12 @@ Win32Assistant::Win32Assistant (iObjectRegistry* r)
   }
 
   csRef<iBase> currentKbd = 
-    CS_QUERY_REGISTRY_TAG (r, "iKeyboardDriver");
+    csQueryRegistryTag (r, "iKeyboardDriver");
   if (currentKbd != 0)
   {
     // Bit hacky: remove old keyboard driver
-    csRef<iEventHandler> eh = SCF_QUERY_INTERFACE (currentKbd, 
-      iEventHandler);
+    csRef<iEventHandler> eh =  
+      scfQueryInterface<iEventHandler> (currentKbd);
     q->RemoveListener (eh);
     r->Unregister (currentKbd, "iKeyboardDriver");
   }
@@ -553,7 +567,7 @@ Win32Assistant::~Win32Assistant ()
 
 void Win32Assistant::Shutdown()
 {
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY (registry, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (registry));
   if (q != 0)
     q->RemoveListener(this);
   if (!is_console_app && (cmdline_help_wanted || console_window))
@@ -599,7 +613,7 @@ iEventOutlet* Win32Assistant::GetEventOutlet()
 {
   if (!EventOutlet.IsValid())
   {
-    csRef<iEventQueue> q (CS_QUERY_REGISTRY(registry, iEventQueue));
+    csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (registry));
     if (q != 0)
       EventOutlet = q->CreateEventOutlet(this);
   }
@@ -868,12 +882,20 @@ LRESULT CALLBACK Win32Assistant::WindowProc (HWND hWnd, UINT message,
       if (assistant != 0)
       {
         iEventOutlet* outlet = assistant->GetEventOutlet();
-        ::SetCursor (assistant->m_hCursor);
         outlet->Mouse (csmbNone, false, short(LOWORD(lParam)), 
 	  short(HIWORD(lParam)));
       }
       return TRUE;
     }
+    case WM_SETCURSOR:
+    {
+      if ((assistant != 0) && (LOWORD (lParam) == HTCLIENT))
+      {
+        ::SetCursor (assistant->m_hCursor);
+        return TRUE;
+      }
+      break;
+    }
     case WM_SIZE:
     {
       if (assistant != 0)
@@ -969,27 +991,56 @@ bool Win32Assistant::SetCursor (int cursor)
 
 void Win32Assistant::DisableConsole ()
 {
-  if (console_window)
+  if (!console_window) return;
+  console_window = false;
+
+  DWORD lasterr;
+  csString outName;
   {
-    console_window = false;
+    char apppath[MAX_PATH];
+    GetModuleFileName (0, apppath, sizeof(apppath));
+    lasterr = GetLastError ();
+    if (lasterr != ERROR_INSUFFICIENT_BUFFER)
+      outName = apppath;
+  }
+  if (lasterr == ERROR_INSUFFICIENT_BUFFER)
+  {
+    DWORD bufSize = 2*MAX_PATH;
+    char* buf = 0;
+    while (lasterr == ERROR_INSUFFICIENT_BUFFER)
+    {
+      buf = (char*)cs_realloc (buf, bufSize);
+      GetModuleFileName (0, buf, bufSize);
+      bufSize += MAX_PATH;
+      lasterr = GetLastError ();
+    }
+    outName = buf;
+    cs_free (buf);
   }
 
-  char apppath[MAX_PATH];
-
-  GetModuleFileName (0, apppath, sizeof(apppath));
-
-  char *dot = strrchr (apppath, '.');
-  if (dot) 
   {
-    strcpy (dot, ".txt");
+    size_t basePos = outName.FindLast ('\\');
+    if (basePos != (size_t)-1) outName.DeleteAt (0, basePos+1);
   }
-  else
   {
-    strcat (apppath, ".txt");
+    size_t dot = outName.FindLast ('.');
+    if (dot != (size_t)-1)
+      outName.Overwrite (dot, ".txt");
+    else
+      outName.Append (".txt");
+  }
+  {
+    char tmp[MAX_PATH];
+    if (GetTempPath (sizeof (tmp), tmp) != 0)
+      outName.Insert (0, tmp);
   }
 
-  freopen(apppath, "w", stderr);
-  freopen(apppath, "w", stdout);
+  /* Redirect only those handles that were not initially redirected by
+   * the user */
+  if (!IsStdHandleRedirected (STD_ERROR_HANDLE)) 
+    freopen (outName, "w", stderr);
+  if (!IsStdHandleRedirected (STD_OUTPUT_HANDLE)) 
+    freopen (outName, "w", stdout);
   FreeConsole();
 
   struct tm *now;
