@@ -20,7 +20,10 @@
 #include "cssysdef.h"
 #include "csutil/csuctransform.h"
 #include "iutil/event.h"
+
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
+#include <imm.h>
 
 #include "win32kbd.h"
 #include "csutil/win32/wintools.h"
@@ -339,7 +342,7 @@ bool csWin32KeyboardDriver::HandleKeyMessage (HWND hWnd, UINT message,
 	    DWORD dwSize = Imm32::ImmGetCompositionStringW (hIMC, 
 	      GCS_RESULTSTR, 0, 0);
 
-	    IMEComposeBuf.SetLength (dwSize / sizeof (WCHAR));
+	    IMEComposeBuf.SetSize (dwSize / sizeof (WCHAR));
 
 	    // Get the result strings that is generated by the IME.
 	    Imm32::ImmGetCompositionStringW (hIMC, GCS_RESULTSTR, 
@@ -359,14 +362,14 @@ bool csWin32KeyboardDriver::HandleKeyMessage (HWND hWnd, UINT message,
 
 	    int bufsize = MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, 
 	      composeResult, dwSize, 0, 0);
-	    IMEComposeBuf.SetLength (bufsize);
+	    IMEComposeBuf.SetSize (bufsize);
 	    MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, 
 	      composeResult, -1, IMEComposeBuf.GetArray(), bufsize);
 	  }
 
 	  Imm32::ImmReleaseContext (hWnd, hIMC);
 
-	  size_t remaining = IMEComposeBuf.Length();
+	  size_t remaining = IMEComposeBuf.GetSize ();
 	  utf16_char* str = (utf16_char*)IMEComposeBuf.GetArray();
           while (remaining > 0)
 	  {
@@ -459,19 +462,21 @@ bool csWin32KeyboardDriver::HandleKeyMessage (HWND hWnd, UINT message,
 	    #ifdef CS_KEY_DEBUG_ENABLE
 	      if (IsKeyboardDebugging()) csPrintf ("Shift quirk: ");
 	    #endif
-	      DoKey (CSKEY_SHIFT_LEFT, CSKEY_SHIFT, lshiftDown, autoRep, type);
+	      DoKey (CSKEY_SHIFT_LEFT, CSKEY_SHIFT, lshiftDown, 
+                autoRep & lshiftDown, type);
 	    }
 	    if ((rshiftState != rshiftDown) || rshiftDown)
 	    {
 	    #ifdef CS_KEY_DEBUG_ENABLE
 	      if (IsKeyboardDebugging()) csPrintf ("Shift quirk: ");
 	    #endif
-	      DoKey (CSKEY_SHIFT_RIGHT, CSKEY_SHIFT, rshiftDown, autoRep, type);
+	      DoKey (CSKEY_SHIFT_RIGHT, CSKEY_SHIFT, rshiftDown, 
+                autoRep & rshiftDown, type);
 	    }
 
 	    doDoKey = false;
 	  }
-  	  if (doDoKey) DoKey (raw, cooked, down, autoRep, type);
+  	  if (doDoKey) DoKey (raw, cooked, down, autoRep & down, type);
 	}
       }
       return !(wParam == VK_F4 && 
