@@ -20,11 +20,15 @@
 #define __CSUTIL_XMLTINYPRIV_H__
 
 #include "csextern.h"
+#include "csutil/pooledscfclass.h"
 #include "csutil/scf_implementation.h"
 #include "iutil/document.h"
 #include "tinyxml.h"
 
 class csTinyXmlDocument;
+struct csTinyXmlNode;
+
+using namespace CS::Implementation::TinyXml;
 
 /**
  * This is an SCF compatible wrapper for an attribute iterator.
@@ -36,7 +40,7 @@ struct CS_CRYSTALSPACE_EXPORT csTinyXmlAttributeIterator :
 private:
   size_t current;
   size_t count;
-  TiXmlElement* parent;
+  csRef<TiXmlElement> parent;
 
 public:
   csTinyXmlAttributeIterator (TiDocumentNode* parent);
@@ -138,33 +142,37 @@ struct CS_CRYSTALSPACE_EXPORT csTinyXmlNodeIterator :
 {
 private:
   csTinyXmlDocument* doc;
-  TiDocumentNode* current;
-  TiDocumentNodeChildren* parent;
+  csRef<TiDocumentNode> current;
+  csRef<csTinyXmlNode> parent;
   char* value;
 
+  size_t currentPos, endPos;
 public:
   csTinyXmlNodeIterator (csTinyXmlDocument* doc,
-	TiDocumentNodeChildren* parent, const char* value);
+	csTinyXmlNode* parent, const char* value);
   virtual ~csTinyXmlNodeIterator ();
 
   virtual bool HasNext ();
   virtual csRef<iDocumentNode> Next ();
+
+  virtual size_t GetNextPosition () { return currentPos; }
+  virtual size_t GetEndPosition ();
 };
 
 /**
  * This is an SCF compatible wrapper for a node in TinyXml.
  */
 struct CS_CRYSTALSPACE_EXPORT csTinyXmlNode : 
-  public scfImplementation1<csTinyXmlNode, iDocumentNode>
+  public scfImplementationPooled<scfImplementation1<csTinyXmlNode, 
+                                                    iDocumentNode> >
 {
 private:
   friend class csTinyXmlDocument;
-  TiDocumentNode* node;
-  TiDocumentNodeChildren* node_children;
+  csRef<TiDocumentNode> node;
+  csRef<TiDocumentNode> lastChild;
   // We keep a reference to 'doc' to avoid it being cleaned up too early.
   // We need 'doc' for the pool.
   csRef<csTinyXmlDocument> doc;
-  csTinyXmlNode* next_pool;	// Next element in pool.
 
   csTinyXmlNode (csTinyXmlDocument* doc);
 
@@ -173,13 +181,24 @@ private:
 public:
   virtual ~csTinyXmlNode ();
 
-  virtual void DecRef ();
+  void DecRef()
+  {
+    /* When the document gets destructed due the last ref to it being released
+       in the destructor of a node it's node pool asserts as the node being 
+       destructed is not yet freed. 
+       Work around that by keeping an extra ref until after the node is deleted.
+     */
+    csRef<csTinyXmlDocument> doc (this->doc);
+    scfPooledImplementationType::DecRef();
+  }
 
   TiDocumentNode* GetTiNode () { return node; }
+  TiDocumentNodeChildren* GetTiNodeChildren () 
+  { return static_cast<TiDocumentNodeChildren*> (GetTiNode ()); }
   void SetTiNode (TiDocumentNode* node)
   {
     csTinyXmlNode::node = node;
-    node_children = node->ToDocumentNodeChildren ();
+    lastChild = 0;
   }
 
   
@@ -227,23 +246,22 @@ class CS_CRYSTALSPACE_EXPORT csTinyXmlDocument :
   public scfImplementation1<csTinyXmlDocument, iDocument>
 {
 private:
-  TiDocument* root;
+  friend struct csTinyXmlNode;
+  friend struct csTinyXmlNodeIterator;
+  csRef<TiDocument> root;
   // We keep a reference to 'sys' to avoid it being cleaned up too early.
   csRef<csTinyDocumentSystem> sys;
 
-  csTinyXmlNode* pool;
+  csTinyXmlNode::Pool pool;
 
+  /// Allocate a node instance
+  csTinyXmlNode* Alloc ();
+  /// Allocate a node instance
+  csTinyXmlNode* Alloc (TiDocumentNode*);
 public:
   csTinyXmlDocument (csTinyDocumentSystem* sys);
   virtual ~csTinyXmlDocument ();
 
-  /// Internal function: don't use!
-  csTinyXmlNode* Alloc ();
-  /// Internal function: don't use!
-  csTinyXmlNode* Alloc (TiDocumentNode*);
-  /// Internal function: don't use!
-  void Free (csTinyXmlNode* n);
-
   virtual void Clear ();
   virtual csRef<iDocumentNode> CreateRoot ();
 
