@@ -33,34 +33,52 @@
 #include "iutil/comp.h"
 #include "csgeom/vector3.h"
 #include "csutil/dirtyaccessarray.h"
+#include "csutil/scf_implementation.h"
 #include "ivaria/collider.h"
 #include "csgeom/transfrm.h"
+#include "imesh/terrain2.h"
 #include "CSopcodecollider.h"
+#include "csTerraFormerCollider.h"
 #include "Opcode.h"
 
 struct iObjectRegistry;
 
+CS_PLUGIN_NAMESPACE_BEGIN(csOpcode)
+{
+
 /**
  * Opcode implementation of the collision detection system.
  */
-class csOPCODECollideSystem : public iCollideSystem
+class csOPCODECollideSystem :
+  public scfImplementation2<csOPCODECollideSystem, iCollideSystem, iComponent>
 {
+
+  bool Collide (csOPCODECollider* collider1,
+    const csReversibleTransform* trans1, csTerraFormerCollider* terraformer,
+    const csReversibleTransform* trans2);
+  
+  bool Collide (csOPCODECollider* collider1, const csReversibleTransform*
+    trans1, iTerrainSystem* terrain, const csReversibleTransform* terrainTrans);
+
+  bool TestTriangleTerraFormer (csVector3 triangle[3],
+    csTerraFormerCollider* c, csCollisionPair* pair);
+
 public:
-  cspluginOpcode::Opcode::AABBTreeCollider TreeCollider;
-  cspluginOpcode::Opcode::RayCollider RayCol;
-  cspluginOpcode::Opcode::BVTCache ColCache;
+  Opcode::AABBTreeCollider TreeCollider;
+  Opcode::RayCollider RayCol;
+  Opcode::BVTCache ColCache;
 
   csDirtyAccessArray<csCollisionPair> pairs;
   csArray<int> collision_faces;
   csArray<csIntersectingTriangle> intersecting_triangles;
   iObjectRegistry *object_reg;
+  csStringID trianglemesh_id;
+  csStringID basemesh_id;
  
   static iObjectRegistry* rep_object_reg;
   static void OpcodeReportV (int severity, const char* message, 
     va_list args);
 
-  SCF_DECLARE_IBASE;
-
   csOPCODECollideSystem (iBase* parent);
   virtual ~csOPCODECollideSystem ();
   bool Initialize (iObjectRegistry* iobject_reg);
@@ -69,7 +87,16 @@ public:
   // to 'pairs'.
   void CopyCollisionPairs (csOPCODECollider* col1, csOPCODECollider* col2);
 
+  void CopyCollisionPairs (csOPCODECollider* col2,
+      csTerraFormerCollider* terraformer);
+
+  virtual csStringID GetTriangleDataID () { return trianglemesh_id; }
+  virtual csStringID GetBaseDataID () { return basemesh_id; }
+
   virtual csPtr<iCollider> CreateCollider (iPolygonMesh* mesh);
+  virtual csPtr<iCollider> CreateCollider (iTriangleMesh* mesh);
+  virtual csPtr<iCollider> CreateCollider (iTerraFormer* mesh);
+  virtual csPtr<iCollider> CreateCollider (iTerrainSystem* mesh);
 
   /**
    * Test collision between two colliders.
@@ -81,6 +108,15 @@ public:
   bool CollideRaySegment (
   	iCollider* collider, const csReversibleTransform* trans,
 	const csVector3& start, const csVector3& end, bool use_ray);
+  bool CollideRaySegment (
+  	csOPCODECollider* collider, const csReversibleTransform* trans,
+	const csVector3& start, const csVector3& end, bool use_ray);
+  bool CollideRaySegment (
+  	iTerrainSystem* terrain, const csReversibleTransform* trans,
+	const csVector3& start, const csVector3& end, bool use_ray);
+  bool CollideRaySegment (
+  	csTerraFormerCollider* terraformer, const csReversibleTransform* trans,
+	const csVector3& start, const csVector3& end, bool use_ray);
   virtual bool CollideRay (
   	iCollider* collider, const csReversibleTransform* trans,
 	const csVector3& start, const csVector3& end)
@@ -98,6 +134,11 @@ public:
   {
     return intersecting_triangles;
   }
+  bool CalculateIntersections (csTerraFormerCollider* terraformer);
+  bool TestSegmentTerraFormer (
+    csTerraFormerCollider* terraformer,
+    const IceMaths::Matrix4x4& transform,
+    const csVector3& start, const csVector3& end);
 
   /**
    * Get pointer to current array of collision pairs.
@@ -128,15 +169,9 @@ public:
    * For CD systems that support one hit only this will always return true.
    */
   virtual bool GetOneHitOnly ();
-
-  struct Component : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csOPCODECollideSystem);
-    virtual bool Initialize (iObjectRegistry* object_reg)
-    {
-      return scfParent->Initialize (object_reg);
-    }
-  } scfiComponent;
 };
 
+}
+CS_PLUGIN_NAMESPACE_END(csOpcode)
+
 #endif // __CS_OPCODE_PLUGIN_H__
