@@ -35,27 +35,42 @@
 #include "csgeom/tri.h"
 #include "CSopcodecollider.h"
 #include "igeom/polymesh.h"
+#include "igeom/trimesh.h"
 #include "ivaria/collider.h"
 
 #include "OPC_TreeBuilders.h"
 
-using namespace cspluginOpcode;
-using namespace cspluginOpcode::Opcode;
+CS_PLUGIN_NAMESPACE_BEGIN(csOpcode)
+{
 
-SCF_IMPLEMENT_IBASE (csOPCODECollider)
-  SCF_IMPLEMENTS_INTERFACE (iCollider)
-SCF_IMPLEMENT_IBASE_END
+using namespace Opcode;
 
-csOPCODECollider::csOPCODECollider (iPolygonMesh* mesh)
+csOPCODECollider::csOPCODECollider (iTriangleMesh* mesh) :
+  scfImplementationType(this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   m_pCollisionModel = 0;
   indexholder = 0;
   vertholder = 0;
-  transform.m[0][3] = 0;
-  transform.m[1][3] = 0;
-  transform.m[2][3] = 0;
-  transform.m[3][3] = 1;  
+  //transform.m[0][3] = 0;
+  //transform.m[1][3] = 0;
+  //transform.m[2][3] = 0;
+  //transform.m[3][3] = 1;
+
+  opcMeshInt.SetCallback (&MeshCallback, this);
+
+  GeometryInitialize (mesh);
+}
+
+csOPCODECollider::csOPCODECollider (iPolygonMesh* mesh) :
+  scfImplementationType(this)
+{
+  m_pCollisionModel = 0;
+  indexholder = 0;
+  vertholder = 0;
+  //transform.m[0][3] = 0;
+  //transform.m[1][3] = 0;
+  //transform.m[2][3] = 0;
+  //transform.m[3][3] = 1;
 
   opcMeshInt.SetCallback (&MeshCallback, this);
 
@@ -67,30 +82,32 @@ inline float min3 (float a, float b, float c)
 inline float max3(float a, float b, float c)
 { return (a > b ? (a > c ? a : (c > b ? c : b)) : (b > c ? b : c)); }
 
-void csOPCODECollider::GeometryInitialize (iPolygonMesh* mesh)
+void csOPCODECollider::GeometryInitialize (csVector3* vertices,
+    size_t vertcount, csTriangle* triangles, size_t tri_count)
 {
   OPCODECREATE OPCC;
-  int i;
-  // first, count the number of triangles polyset contains
-  csVector3* vertices = mesh->GetVertices ();
-  int vertcount = mesh->GetVertexCount ();
-  csTriangle* triangles = mesh->GetTriangles ();
-  int tri_count = mesh->GetTriangleCount ();
-  
+  size_t i;
+
   if (tri_count>=1)
   {
-    m_pCollisionModel = new cspluginOpcode::Opcode::Model;
+    m_pCollisionModel = new Opcode::Model;
     if (!m_pCollisionModel)
       return;
 
     vertholder = new Point [vertcount];
     indexholder = new unsigned int[3*tri_count];
 
+    csBox3 tmp;
+    tmp.StartBoundingBox ();
     for (i = 0; i < vertcount; i++)
     {
+      tmp.AddBoundingVertex (vertices[i]);
       vertholder[i].Set (vertices[i].x , vertices[i].y , vertices[i].z);
     }
-    
+
+    radius = max3 (tmp.MaxX ()- tmp.MinX (), tmp.MaxY ()- tmp.MinY (),
+	tmp.MaxZ ()- tmp.MinZ ());
+
     int index = 0;
     for (i = 0 ; i < tri_count ; i++)
     {
@@ -98,7 +115,7 @@ void csOPCODECollider::GeometryInitialize (iPolygonMesh* mesh)
       indexholder[index++] = triangles[i].b;
       indexholder[index++] = triangles[i].c;
     }
-   
+
     opcMeshInt.SetNbTriangles (tri_count);
     opcMeshInt.SetNbVertices (vertcount);
 
@@ -113,10 +130,31 @@ void csOPCODECollider::GeometryInitialize (iPolygonMesh* mesh)
   else
     return;
 
-  bool status = m_pCollisionModel->Build (OPCC);  // this should create the OPCODE model
+  // this should create the OPCODE model
+  bool status = m_pCollisionModel->Build (OPCC);
   if (!status) { return; };
 }
 
+void csOPCODECollider::GeometryInitialize (iTriangleMesh* mesh)
+{
+  // first, count the number of triangles polyset contains
+  csVector3* vertices = mesh->GetVertices ();
+  size_t vertcount = mesh->GetVertexCount ();
+  csTriangle* triangles = mesh->GetTriangles ();
+  size_t tri_count = mesh->GetTriangleCount ();
+  GeometryInitialize (vertices, vertcount, triangles, tri_count);
+}
+
+void csOPCODECollider::GeometryInitialize (iPolygonMesh* mesh)
+{
+  // first, count the number of triangles polyset contains
+  csVector3* vertices = mesh->GetVertices ();
+  size_t vertcount = mesh->GetVertexCount ();
+  csTriangle* triangles = mesh->GetTriangles ();
+  size_t tri_count = mesh->GetTriangleCount ();
+  GeometryInitialize (vertices, vertcount, triangles, tri_count);
+}
+
 csOPCODECollider::~csOPCODECollider ()
 {
   if (m_pCollisionModel)
@@ -127,11 +165,10 @@ csOPCODECollider::~csOPCODECollider ()
 
   delete[] indexholder;
   delete[] vertholder;
-  SCF_DESTRUCT_IBASE ();
 }
 
-void csOPCODECollider::MeshCallback (udword triangle_index, 
-				     VertexPointers& triangle, 
+void csOPCODECollider::MeshCallback (udword triangle_index,
+				     VertexPointers& triangle,
 				     void* user_data)
 {
   csOPCODECollider* collider = (csOPCODECollider*)user_data;
@@ -143,3 +180,6 @@ void csOPCODECollider::MeshCallback (udword triangle_index,
   triangle.Vertex[1] = &vertholder [tri_array[index + 1]];
   triangle.Vertex[2] = &vertholder [tri_array[index + 2]];
 }
+
+}
+CS_PLUGIN_NAMESPACE_END(csOpcode)
