@@ -51,8 +51,10 @@ To do:
 // Precompiled Header
 #include "Stdafx.h"
 
-using namespace cspluginOpcode;
-using namespace cspluginOpcode::IceCore;
+CS_PLUGIN_NAMESPACE_BEGIN(csOpcode)
+{
+
+using namespace IceCore;
 
 #define INVALIDATE_RANKS	mCurrentSize|=0x80000000
 #define VALIDATE_RANKS		mCurrentSize&=0x7fffffff
@@ -521,3 +523,6 @@ udword RadixSort::GetUsedRam() const
 	UsedRam += 2*CURRENT_SIZE*sizeof(udword);	// 2 lists of indices
 	return UsedRam;
 }
+
+}
+CS_PLUGIN_NAMESPACE_END(csOpcode)
