@@ -21,22 +21,25 @@
 #define __CS_CSLOADER_H__
 
 #include <stdarg.h>
-#include "ivideo/graph3d.h"
-#include "imap/loader.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "csutil/util.h"
-#include "csutil/strhash.h"
-#include "csutil/hash.h"
-#include "csutil/array.h"
-#include "csutil/refarr.h"
-#include "csutil/scopedmutexlock.h"
+
 #include "csgeom/quaternion.h"
-#include "iutil/plugin.h"
-#include "imap/services.h"
+#include "csutil/array.h"
+#include "csutil/csstring.h"
+#include "csutil/hash.h"
+#include "csutil/refarr.h"
+#include "csutil/scf_implementation.h"
+#include "csutil/threading/thread.h"
+#include "csutil/strhash.h"
+#include "csutil/util.h"
 #include "imap/ldrctxt.h"
-#include "ivaria/engseq.h"
+#include "imap/loader.h"
+#include "imap/services.h"
 #include "isndsys/ss_renderer.h"
+#include "iutil/comp.h"
+#include "iutil/eventh.h"
+#include "iutil/plugin.h"
+#include "ivaria/engseq.h"
+#include "ivideo/graph3d.h"
 
 class csGenerateImageTexture;
 class csGenerateImageValue;
@@ -47,13 +50,11 @@ struct csRGBcolor;
 struct iImageIO;
 struct iRegion;
 struct iObjectModel;
-struct iSoundLoader;
 struct iSndSysLoader;
 struct iSndSysManager;
 struct iEngine;
 struct iVFS;
 struct iGraphics3D;
-struct iSoundRender;
 struct iLoaderPlugin;
 struct iBinaryLoaderPlugin;
 struct iObjectRegistry;
@@ -63,13 +64,15 @@ struct iDocumentNode;
 struct iDocument;
 struct iFile;
 struct iPolygonMesh;
+struct iTriangleMesh;
 struct iShaderManager;
 struct iMeshGenerator;
 struct iSceneNode;
+struct iRenderLoop;
+struct iImposter;
 
 struct iObject;
 struct iThingState;
-struct iCollection;
 struct iMaterialWrapper;
 struct iMeshFactoryWrapper;
 struct iMeshWrapper;
@@ -83,6 +86,7 @@ struct iSequenceTrigger;
 struct iSequenceWrapper;
 struct iEngineSequenceParameters;
 struct iSharedVariable;
+struct iSceneNodeArray;
 
 class csLoader;
 struct csLoaderPluginRec;
@@ -90,28 +94,31 @@ struct csLoaderPluginRec;
 /*
  * Context class for the standard loader.
  */
-class StdLoaderContext : public iLoaderContext
+class StdLoaderContext : public scfImplementation1<StdLoaderContext,
+			 iLoaderContext>
 {
 private:
   iEngine* Engine;
   iRegion* region;
   csLoader* loader;
+  csRef<iMissingLoaderData> missingdata;
   bool checkDupes;
   bool curRegOnly;
+
 public:
   StdLoaderContext (iEngine* Engine, iRegion* region, bool curRegOnly,
-    csLoader* loader, bool checkDupes);
+    csLoader* loader, bool checkDupes, iMissingLoaderData* missingdata);
   virtual ~StdLoaderContext ();
 
-  SCF_DECLARE_IBASE;
-
   virtual iSector* FindSector (const char* name);
   virtual iMaterialWrapper* FindMaterial (const char* name);
-  virtual iMaterialWrapper* FindNamedMaterial (const char* name, const char *filename);
+  virtual iMaterialWrapper* FindNamedMaterial (const char* name,
+      const char *filename);
   virtual iMeshFactoryWrapper* FindMeshFactory (const char* name);
   virtual iMeshWrapper* FindMeshObject (const char* name);
   virtual iTextureWrapper* FindTexture (const char* name);
-  virtual iTextureWrapper* FindNamedTexture (const char* name, const char *filename);
+  virtual iTextureWrapper* FindNamedTexture (const char* name,
+      const char *filename);
   virtual iLight* FindLight (const char *name);
   virtual iShader* FindShader (const char *name);
   virtual bool CheckDupes () const { return checkDupes; }
@@ -122,7 +129,8 @@ public:
 /*
  * Context class for the threaded loader.
  */
-class ThreadedLoaderContext : public iLoaderContext
+class ThreadedLoaderContext :
+  public scfImplementation1<ThreadedLoaderContext, iLoaderContext>
 {
 private:
   iEngine* Engine;
@@ -142,15 +150,15 @@ public:
     csLoader* loader, bool checkDupes);
   virtual ~ThreadedLoaderContext ();
 
-  SCF_DECLARE_IBASE;
-
   virtual iSector* FindSector (const char* name);
   virtual iMaterialWrapper* FindMaterial (const char* name);
-  virtual iMaterialWrapper* FindNamedMaterial (const char* name, const char *filename);
+  virtual iMaterialWrapper* FindNamedMaterial (const char* name,
+  	const char *filename);
   virtual iMeshFactoryWrapper* FindMeshFactory (const char* name);
   virtual iMeshWrapper* FindMeshObject (const char* name);
   virtual iTextureWrapper* FindTexture (const char* name);
-  virtual iTextureWrapper* FindNamedTexture (const char* name, const char *filename);
+  virtual iTextureWrapper* FindNamedTexture (const char* name,
+  	const char *filename);
   virtual iLight* FindLight (const char *name);
   virtual iShader* FindShader (const char *name);
   virtual bool CheckDupes () const { return checkDupes; }
@@ -161,7 +169,8 @@ public:
 /**
  * Status class for the threaded loader.
  */
-class csLoaderStatus : public iLoaderStatus
+class csLoaderStatus :
+  public scfImplementation1<csLoaderStatus, iLoaderStatus>
 {
 private:
   //csRef<csMutex> mutex;
@@ -172,15 +181,18 @@ public:
   csLoaderStatus ();
   virtual ~csLoaderStatus ();
 
-  SCF_DECLARE_IBASE;
   virtual bool IsReady () { return ready; }
   virtual bool IsError () { return error; }
 };
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 /**
  * The loader for Crystal Space maps.
  */
-class csLoader : public iLoader
+class csLoader : public scfImplementation2<csLoader,
+                                           iLoader,
+                                           iComponent>
 {
   friend class StdLoaderContext;
   friend class ThreadedLoaderContext;
@@ -200,14 +212,18 @@ private:
   /// Pointer to built-in image texture loader.
   csRef<iLoaderPlugin> BuiltinImageTexLoader;
   /// Pointer to built-in checkerboard texture loader.
-  csRef<iLoaderPlugin> BuiltinCheckerTexLoader;
+  //csRef<iLoaderPlugin> BuiltinCheckerTexLoader;
+  csRef<iLoaderPlugin> BuiltinErrorTexLoader;
 
+  /// Auto regions flag
+  bool autoRegions;
+  
   //Returns in the 'meshesArray' array all the meshes encountered walking thru
   //the hierarchy of meshes starting from 'meshWrapper'.
   void CollectAllChildren (iMeshWrapper* meshWrapper, csRefArray<iMeshWrapper>&
     meshesArray);
-  //Two useful private functions to set the CS_POLYMESH_CLOSED and
-  //CS_POLYMESH_CONVEX flags on a single mesh wrapper.
+  //Two useful private functions to set the CS_TRIMESH_CLOSED and
+  //CS_TRIMESH_CONVEX flags on a single mesh wrapper.
   void ConvexFlags (iMeshWrapper* mesh);
   void ClosedFlags (iMeshWrapper* mesh);
 
@@ -215,7 +231,7 @@ private:
   {
   private:
     /// Mutex to make the plugin vector thread-safe.
-    csRef<csMutex> mutex;
+    CS::Threading::RecursiveMutex mutex;
     iObjectRegistry* object_reg;
 
     csArray<csLoaderPluginRec*> vector;
@@ -257,6 +273,24 @@ private:
 
   //------------------------------------------------------------------------
 
+  /**
+   * Parse a key/value pair.
+   * Takes "editoronly" attribute into account: KVPs should only be parsed 
+   * if they're not editor-only or when the engine is in "saveable" mode.
+   */
+  bool ParseKey (iDocumentNode* node, iObject* obj);
+/*
+          iKeyValuePair* kvp = 0;
+          SyntaxService->ParseKey (child, kvp);
+          if (kvp)
+          {
+            Engine->QueryObject()->ObjAdd (kvp->QueryObject ());
+	    kvp->DecRef ();
+          }
+	  else
+	    return false;
+*/
+
   /// Parse a quaternion definition
   bool ParseQuaternion (iDocumentNode* node, csQuaternion &q);
   /// Parse a color definition
@@ -300,7 +334,7 @@ private:
   /// Process the attributes of one shared variable
   bool ParseSharedVariable (iLoaderContext* ldr_context, iDocumentNode* node);
   /// Process the attributes of an <imposter> tag in a mesh specification.
-  bool ParseImposterSettings(iMeshWrapper* mesh,iDocumentNode *node);
+  bool ParseImposterSettings(iImposter* mesh, iDocumentNode *node);
 
   /// Parse a texture definition and add the texture to the engine
   iTextureWrapper* ParseTexture (iLoaderContext* ldr_context,
@@ -319,9 +353,8 @@ private:
   /// Parse a material definition and add the material to the engine
   iMaterialWrapper* ParseMaterial (iLoaderContext* ldr_context,
   	iDocumentNode* node, const char* prefix = 0);
-  /// Parse a collection definition and add the collection to the engine
-  iCollection* ParseCollection (iLoaderContext* ldr_context,
-  	iDocumentNode* node);
+  /// Parse a renderloop.
+  iRenderLoop* ParseRenderLoop (iDocumentNode* node, bool& set);
   /// Parse a camera position.
   bool ParseStart (iDocumentNode* node, iCameraPosition* campos);
   /// Parse a static light definition and add the light to the engine
@@ -346,19 +379,21 @@ private:
   /// Find the named shared variable and verify its type if specified
   iSharedVariable *FindSharedVariable(const char *colvar,
 				      int verify_type );
-  /// Parse a 'polymesh' block.
-  bool ParsePolyMesh (iDocumentNode* node, iObjectModel* objmodel);
-  bool ParsePolyMeshChildBox (iDocumentNode* child,
-	csRef<iPolygonMesh>& polymesh);
-  bool ParsePolyMeshChildMesh (iDocumentNode* child,
-	csRef<iPolygonMesh>& polymesh);
+  /// Parse a 'trimesh' block.
+  bool ParseTriMesh (iDocumentNode* node, iObjectModel* objmodel);
+  bool ParseTriMeshChildBox (iDocumentNode* child,
+	csRef<iPolygonMesh>& polymesh, csRef<iTriangleMesh>& trimesh);
+  bool ParseTriMeshChildMesh (iDocumentNode* child,
+	csRef<iPolygonMesh>& polymesh, csRef<iTriangleMesh>& trimesh);
 
   /// -----------------------------------------------------------------------
   /// Parse a shaderlist
+  bool LoadShaderExpressions (iLoaderContext* ldr_context,
+  	iDocumentNode* node);
   bool ParseShaderList (iLoaderContext* ldr_context, iDocumentNode* node);
   bool ParseShader (iLoaderContext* ldr_context, iDocumentNode* node,
     iShaderManager* shaderMgr);
-  virtual bool LoadShader (const char* filename);
+  virtual csRef<iShader> LoadShader (const char* filename, bool registerShader = true);
 
   /// For heightgen.
   csGenerateImageTexture* ParseHeightgenTexture (iDocumentNode* node);
@@ -403,9 +438,9 @@ private:
   	iMeshWrapper* mesh, iMeshWrapper* parent, iDocumentNode* node,
 	iStreamSource* ssource);
   /**
-   * Load the polymesh object from the map file.
+   * Load the trimesh object from the map file.
    */
-  bool LoadPolyMeshInSector (iLoaderContext* ldr_context,
+  bool LoadTriMeshInSector (iLoaderContext* ldr_context,
   	iMeshWrapper* mesh, iDocumentNode* node, iStreamSource* ssource);
 
   /**
@@ -413,7 +448,7 @@ private:
    * This version will parse FACTORY statement to directly create
    * a mesh from a factory.
    */
-  iMeshWrapper* LoadMeshObjectFromFactory (iLoaderContext* ldr_context,
+  csRef<iMeshWrapper> LoadMeshObjectFromFactory (iLoaderContext* ldr_context,
   	iDocumentNode* node, iStreamSource* ssource);
 
   /**
@@ -458,11 +493,11 @@ private:
    * thing templates, sounds and textures.
    */
   bool LoadLibrary (iLoaderContext* ldr_context, iDocumentNode* node,
-  	iStreamSource* ssource);
+  	iStreamSource* ssource, iMissingLoaderData* missingdata);
 
   /// Load map from a memory buffer
   bool LoadMap (iLoaderContext* ldr_context, iDocumentNode* world_node,
-  	iStreamSource* ssource);
+  	iStreamSource* ssource, iMissingLoaderData* missingdata);
 
   /// Get the engine sequence manager (load it if not already present).
   iEngineSequenceManager* GetEngineSequenceManager ();
@@ -508,7 +543,7 @@ private:
    * Add children to the region.
    */
   void AddChildrenToRegion (iLoaderContext* ldr_context,
-	const csRefArray<iSceneNode>& children);
+    const iSceneNodeArray* children);
 
   /// Report any error.
   void ReportError (const char* id, const char* description, ...)
@@ -535,8 +570,6 @@ private:
 
 public:
   /********** iLoader implementation **********/
-  SCF_DECLARE_IBASE;
-
   static bool do_verbose;
 
   // system driver
@@ -548,19 +581,15 @@ public:
   // image loader
   csRef<iImageIO> ImageLoader;
   // sound loader
-  csRef<iSoundLoader> SoundLoader;
-  // sound loader
   csRef<iSndSysLoader> SndSysLoader;
   // sound manager
   csRef<iSndSysManager> SndSysManager;
+  // sound renderer
+  csRef<iSndSysRenderer> SndSysRender;
   // engine
   csRef<iEngine> Engine;
   // graphics renderer
   csRef<iGraphics3D> G3D;
-  // sound renderer
-  csRef<iSoundRender> SoundRender;
-  // sound renderer
-  csRef<iSndSysRenderer> SndSysRender;
 
   // constructor
   csLoader(iBase *p);
@@ -589,58 +618,59 @@ public:
 	bool reg = true, bool create_material = true,
 	bool free_image = true, iRegion* region = 0);
 
-  virtual csPtr<iSoundData> LoadSoundData (const char *fname);
-  virtual csPtr<iSoundHandle> LoadSound (const char *fname);
-  virtual csPtr<iSoundWrapper> LoadSound (const char *name, const char *fname);
   virtual csPtr<iSndSysData> LoadSoundSysData (const char *fname);
   virtual csPtr<iSndSysStream> LoadSoundStream (const char *fname,
   	int mode3d);
-  virtual iSndSysWrapper* LoadSoundWrapper (const char *name, const char *fname,
-  	int mode3d);
+  virtual iSndSysWrapper* LoadSoundWrapper (const char *name, const char *fname);
 
   virtual csPtr<iLoaderStatus> ThreadedLoadMapFile (const char* filename,
 	iRegion* region, bool curRegOnly, bool checkDupes);
   virtual bool LoadMapFile (const char* filename, bool clearEngine,
 	iRegion* region, bool curRegOnly, bool checkDupes,
-	iStreamSource* ssource);
+	iStreamSource* ssource, iMissingLoaderData* missingdata);
   virtual bool LoadMap (iDocumentNode* world_node, bool clearEngine,
 	iRegion* region, bool curRegOnly, bool checkDupes,
-	iStreamSource* ssource);
+	iStreamSource* ssource, iMissingLoaderData* missingdata);
   virtual bool LoadLibraryFile (const char* filename, iRegion* region,
   	bool curRegOnly, bool checkDupes,
-	iStreamSource* ssource);
+	iStreamSource* ssource, iMissingLoaderData* missingdata);
   virtual bool LoadLibrary (iDocumentNode* lib_node, iRegion* region,
   	bool curRegOnly, bool checkDupes,
-	iStreamSource* ssource);
+	iStreamSource* ssource, iMissingLoaderData* missingdata);
   bool LoadLibraryFromNode (iLoaderContext* ldr_context,
 	iDocumentNode* child,
-	iStreamSource* ssource);
+	iStreamSource* ssource, iMissingLoaderData* missingdata);
 
   bool Load (iDataBuffer* buffer, const char* fname, iBase*& result,
   	iRegion* region, bool curRegOnly, bool checkDupes,
-	iStreamSource* ssource, const char* override_name);
+	iStreamSource* ssource, const char* override_name,
+	iMissingLoaderData* missingdata);
   virtual bool Load (const char* fname, iBase*& result, iRegion* region,
   	bool curRegOnly, bool checkDupes, iStreamSource* ssource,
-	const char* override_name);
+	const char* override_name, iMissingLoaderData* missingdata);
   virtual bool Load (iDataBuffer* buffer, iBase*& result, iRegion* region,
   	bool curRegOnly, bool checkDupes, iStreamSource* ssource,
-	const char* override_name);
+	const char* override_name, iMissingLoaderData* missingdata);
   virtual bool Load (iDocumentNode* node, iBase*& result, iRegion* region,
   	bool curRegOnly, bool checkDupes, iStreamSource* ssource,
-	const char* override_name);
+	const char* override_name, iMissingLoaderData* missingdata);
 
+  virtual void SetAutoRegions (bool autoRegions)
+  {
+    csLoader::autoRegions = autoRegions;
+  }
+  
+  virtual bool GetAutoRegions ()
+  {
+    return autoRegions;
+  }
+  
   virtual csPtr<iMeshFactoryWrapper> LoadMeshObjectFactory (const char* fname,
   	iStreamSource* ssource);
   virtual csPtr<iMeshWrapper> LoadMeshObject (const char* fname,
   	iStreamSource* ssource);
-
-  struct eiComponent : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE(csLoader);
-    virtual bool Initialize (iObjectRegistry* p)
-    { return scfParent->Initialize(p); }
-    eiComponent();
-  } scfiComponent;
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 #endif // __CS_CSLOADER_H__
