@@ -54,7 +54,6 @@ struct csLoaderPluginRec
 csLoader::csLoadedPluginVector::csLoadedPluginVector ()
 {
   plugin_mgr = 0;
-  mutex = csMutex::Create (true);
 }
 
 csLoader::csLoadedPluginVector::~csLoadedPluginVector ()
@@ -64,14 +63,14 @@ csLoader::csLoadedPluginVector::~csLoadedPluginVector ()
 
 void csLoader::csLoadedPluginVector::DeleteAll ()
 {
-  csScopedMutexLock lock (mutex);
+  CS::Threading::RecursiveMutexScopedLock lock (mutex);
   size_t i;
-  for (i = 0 ; i < vector.Length () ; i++)
+  for (i = 0 ; i < vector.GetSize () ; i++)
   {
     csLoaderPluginRec* rec = vector[i];
     if (rec->Component && plugin_mgr)
     {
-      csRef<iComponent> comp = SCF_QUERY_INTERFACE (rec->Component, iComponent);
+      csRef<iComponent> comp = scfQueryInterface<iComponent> (rec->Component);
       if (comp)
         plugin_mgr->UnloadPlugin (comp);
     }
@@ -83,9 +82,9 @@ void csLoader::csLoadedPluginVector::DeleteAll ()
 csLoaderPluginRec* csLoader::csLoadedPluginVector::FindPluginRec (
 	const char* name)
 {
-  csScopedMutexLock lock (mutex);
+  CS::Threading::RecursiveMutexScopedLock lock (mutex);
   size_t i;
-  for (i=0 ; i<vector.Length () ; i++)
+  for (i=0 ; i<vector.GetSize () ; i++)
   {
     csLoaderPluginRec* pl = vector.Get (i);
     if ((!pl->ShortName.IsEmpty ()) && !strcmp (name, pl->ShortName))
@@ -102,7 +101,7 @@ bool csLoader::csLoadedPluginVector::GetPluginFromRec (
 {
   if (!rec->Component)
   {
-    rec->Component = CS_QUERY_REGISTRY_TAG (object_reg, rec->ClassID);
+    rec->Component = csQueryRegistryTag (object_reg, rec->ClassID);
     if (!rec->Component)
     {
       csRef<iComponent> comp = CS_LOAD_PLUGIN (plugin_mgr,
@@ -111,9 +110,9 @@ bool csLoader::csLoadedPluginVector::GetPluginFromRec (
     }
     if (rec->Component)
     {
-      rec->Plugin = SCF_QUERY_INTERFACE (rec->Component, iLoaderPlugin);
-      rec->BinPlugin = SCF_QUERY_INTERFACE (rec->Component,
-      	iBinaryLoaderPlugin);
+      rec->Plugin = scfQueryInterface<iLoaderPlugin> (rec->Component);
+      rec->BinPlugin = 
+      	scfQueryInterface<iBinaryLoaderPlugin> (rec->Component);
     }
   }
   plug = rec->Plugin;
@@ -125,7 +124,7 @@ bool csLoader::csLoadedPluginVector::FindPlugin (
 	const char* Name, iLoaderPlugin*& plug,
 	iBinaryLoaderPlugin*& binplug, iDocumentNode*& defaults)
 {
-  csScopedMutexLock lock (mutex);
+  CS::Threading::RecursiveMutexScopedLock lock (mutex);
   // look if there is already a loading record for this plugin
   csLoaderPluginRec* pl = FindPluginRec (Name);
   if (pl)
@@ -137,14 +136,14 @@ bool csLoader::csLoadedPluginVector::FindPlugin (
   // create a new loading record
   vector.Push (new csLoaderPluginRec (0, Name, 0, 0, 0));
   defaults = 0;
-  return GetPluginFromRec (vector.Get(vector.Length()-1),
+  return GetPluginFromRec (vector.Get(vector.GetSize ()-1),
   	plug, binplug);
 }
 
 void csLoader::csLoadedPluginVector::NewPlugin
 	(const char *ShortName, iDocumentNode* child)
 {
-  csScopedMutexLock lock (mutex);
+  CS::Threading::RecursiveMutexScopedLock lock (mutex);
   csRef<iDocumentNode> id = child->GetNode ("id");
   csRef<iDocumentNode> defaults = child->GetNode ("defaults");
 
