@@ -25,14 +25,18 @@
 
 #include "ivideo/graph3d.h"
 
+#include "csgeom/math.h"
 #include "csgfx/imagecubemapmaker.h"
+#include "csgfx/imagememory.h"
 #include "csgfx/imagevolumemaker.h"
+#include "csgfx/rgbpixel.h"
 #include "csgfx/xorpat.h"
 #include "csutil/cscolor.h"
+#include "csutil/scfstr.h"
 #include "iengine/engine.h"
 #include "iengine/material.h"
-#include "iengine/texture.h"
 #include "iengine/region.h"
+#include "iengine/texture.h"
 #include "igraphic/image.h"
 #include "igraphic/imageio.h"
 #include "imap/reader.h"
@@ -47,6 +51,46 @@
 #include "csloader.h"
 #include "loadtex.h"
 
+static void ReportError (iObjectRegistry* object_reg,
+    const char* id, const char* description, ...)
+{
+  va_list arg;
+  va_start (arg, description);
+  csReportV (object_reg, CS_REPORTER_SEVERITY_ERROR, id, description, arg);
+  va_end (arg);
+}
+
+static csPtr<iImage> GenerateErrorTexture (int width, int height)
+{
+  static const csRGBpixel colorTable[] = 
+    {csRGBpixel (0,0,0,255), csRGBpixel (255,0,0,255),
+     csRGBpixel (0,255,0,255), csRGBpixel (0,0,255,255)};
+
+  size_t colorIndex = 0;
+
+  csRef<csImageMemory> image; 
+  image.AttachNew (new csImageMemory (width, height));
+  csRGBpixel *pixel = (csRGBpixel*)image->GetImagePtr();
+  for (int y = 0; y < height; y+=4)
+  {
+    for (int y2 = 0; y2 < 4; ++y2)
+    {
+      for (int x = 0; x < width; x+=4)
+      {
+        for (int x2 = 0; x2 < 4; ++x2)
+        {
+          *pixel++ = colorTable[colorIndex];
+        }
+
+        colorIndex ^= 0x1;// Flip lowest bit 
+      }
+    }
+    colorIndex ^= 0x2; // Flip higher bit
+  }
+
+  return csPtr<iImage> (image);
+}
+
 csPtr<iImage> csLoader::LoadImage (iDataBuffer* buf, const char* fname,
 	int Format)
 {
@@ -111,8 +155,9 @@ csPtr<iTextureHandle> csLoader::LoadTexture (iDataBuffer* buf, int Flags,
   {
     ReportWarning (
 	"crystalspace.maploader.parse.texture",
-	"Couldn't load image. Using checkerboard instead!");
-    Image = csCreateXORPatternImage (32, 32, 5);
+	"Couldn't load image. Using error texture instead!");
+    //Image = csCreateXORPatternImage (32, 32, 5);
+    Image = GenerateErrorTexture (32, 32);
     if (!Image)
       return 0;
   }
@@ -122,12 +167,16 @@ csPtr<iTextureHandle> csLoader::LoadTexture (iDataBuffer* buf, int Flags,
   if (!tm)
     return 0;
   
-  csRef<iTextureHandle> TexHandle = tm->RegisterTexture (Image, Flags);
+  csRef<scfString> fail_reason;
+  fail_reason.AttachNew (new scfString ());
+  csRef<iTextureHandle> TexHandle = tm->RegisterTexture (Image, Flags,
+      fail_reason);
   if (!TexHandle)
   {
     ReportError (
 	"crystalspace.maploader.parse.texture",
-	"Cannot create texture!");
+	"Cannot create texture: %s",
+	fail_reason->GetData ());
     return 0;
   }
 
@@ -192,9 +241,10 @@ csPtr<iTextureHandle> csLoader::LoadTexture (const char *fname, int Flags,
   {
     ReportWarning (
 	"crystalspace.maploader.parse.texture",
-	"Couldn't load image '%s', using checkerboard instead!",
+	"Couldn't load image '%s', using error texture instead!",
 	fname);
-    Image = csCreateXORPatternImage (32, 32, 5);
+    //Image = csCreateXORPatternImage (32, 32, 5);
+    Image = GenerateErrorTexture (32, 32);
     if (!Image)
       return 0;
   }
@@ -204,12 +254,16 @@ csPtr<iTextureHandle> csLoader::LoadTexture (const char *fname, int Flags,
   if (!tm)
     return 0;
   
-  csRef<iTextureHandle> TexHandle (tm->RegisterTexture (Image, Flags));
+  csRef<scfString> fail_reason;
+  fail_reason.AttachNew (new scfString ());
+  csRef<iTextureHandle> TexHandle (tm->RegisterTexture (Image, Flags,
+	fail_reason));
   if (!TexHandle)
   {
     ReportError (
 	"crystalspace.maploader.parse.texture",
-	"Cannot create texture from '%s'!", fname);
+	"Cannot create texture from '%s': '%s'", fname,
+	fail_reason->GetData ());
     return 0;
   }
 
@@ -257,14 +311,9 @@ iTextureWrapper* csLoader::LoadTexture (const char *name,
 
 //----------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE(TextureLoaderContext);
-  SCF_IMPLEMENTS_INTERFACE(iTextureLoaderContext);
-SCF_IMPLEMENT_IBASE_END
-
-TextureLoaderContext::TextureLoaderContext (const char* texname)
+TextureLoaderContext::TextureLoaderContext (const char* texname) :
+  scfImplementationType(this)
 {
-  SCF_CONSTRUCT_IBASE (0);
-
   has_flags = false;
   flags = CS_TEXTURE_3D;
 
@@ -279,7 +328,6 @@ TextureLoaderContext::TextureLoaderContext (const char* texname)
 
 TextureLoaderContext::~TextureLoaderContext ()
 {
-  SCF_DESTRUCT_IBASE();
 }
 
 void TextureLoaderContext::SetFlags (int Flags)
@@ -347,19 +395,13 @@ const char* TextureLoaderContext::GetClass ()
 
 //----------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE(csBaseTextureLoader);
-  SCF_IMPLEMENTS_INTERFACE(iLoaderPlugin);
-  SCF_IMPLEMENTS_INTERFACE(iComponent);
-SCF_IMPLEMENT_IBASE_END
-
-csBaseTextureLoader::csBaseTextureLoader (iBase *p)
+csBaseTextureLoader::csBaseTextureLoader (iBase *p) :
+  scfImplementationType(this, p)
 {
-  SCF_CONSTRUCT_IBASE (p);
 }
 
 csBaseTextureLoader::~csBaseTextureLoader ()
 {
-  SCF_DESTRUCT_IBASE();
 }
 
 bool csBaseTextureLoader::Initialize(iObjectRegistry *object_reg)
@@ -373,7 +415,7 @@ bool csBaseTextureLoader::Initialize(iObjectRegistry *object_reg)
 SCF_IMPLEMENT_FACTORY(csImageTextureLoader)
 
 csImageTextureLoader::csImageTextureLoader (iBase *p) : 
-  csBaseTextureLoader(p)
+  scfImplementationType(this, p)
 {
 }
 
@@ -384,22 +426,29 @@ csPtr<iBase> csImageTextureLoader::Parse (iDocumentNode* /*node*/,
 {
   if (!context) return 0;
   csRef<iTextureLoaderContext> ctx = csPtr<iTextureLoaderContext>
-    (SCF_QUERY_INTERFACE (context, iTextureLoaderContext));
+    (scfQueryInterface<iTextureLoaderContext> (context));
   if (!ctx) return 0;
   if (!ctx->HasImage() || !ctx->GetImage())
     return 0;
 
-  csRef<iGraphics3D> G3D = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  csRef<iGraphics3D> G3D = csQueryRegistry<iGraphics3D> (object_reg);
   if (!G3D) return 0;
   csRef<iTextureManager> tm = G3D->GetTextureManager();
   if (!tm) return 0;
-  csRef<iEngine> Engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  csRef<iEngine> Engine = csQueryRegistry<iEngine> (object_reg);
   if (!Engine)
     return 0;
 
+  csRef<scfString> fail_reason;
+  fail_reason.AttachNew (new scfString ());
   csRef<iTextureHandle> TexHandle (tm->RegisterTexture (ctx->GetImage(), 
-    ctx->HasFlags() ? ctx->GetFlags() : CS_TEXTURE_3D));
-  if (!TexHandle) return 0;
+    ctx->HasFlags() ? ctx->GetFlags() : CS_TEXTURE_3D, fail_reason));
+  if (!TexHandle)
+  {
+    ReportError (object_reg, "crystalspace.imagetextureloader",
+	"Error creating texture: %s", fail_reason->GetData ());
+    return 0;
+  }
 
   csRef<iTextureWrapper> TexWrapper =
 	Engine->GetTextureList ()->NewTexture(TexHandle);
@@ -413,7 +462,7 @@ csPtr<iBase> csImageTextureLoader::Parse (iDocumentNode* /*node*/,
 SCF_IMPLEMENT_FACTORY(csCheckerTextureLoader)
 
 csCheckerTextureLoader::csCheckerTextureLoader (iBase *p) : 
-  csBaseTextureLoader(p)
+  scfImplementationType(this, p)
 {
 }
 
@@ -427,7 +476,7 @@ csPtr<iBase> csCheckerTextureLoader::Parse (iDocumentNode* node,
   csRef<iTextureLoaderContext> ctx;
   if (context)
   {
-    ctx = SCF_QUERY_INTERFACE (context, iTextureLoaderContext);
+    ctx = scfQueryInterface<iTextureLoaderContext> (context);
     if (ctx)
     {
       if (ctx->HasSize())
@@ -450,7 +499,7 @@ csPtr<iBase> csCheckerTextureLoader::Parse (iDocumentNode* node,
     if (colorNode)
     {
       csRef<iSyntaxService> synserv = 
-	CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+	csQueryRegistry<iSyntaxService> (object_reg);
       if (synserv)
       {
 	synserv->ParseColor (colorNode, color);
@@ -461,17 +510,24 @@ csPtr<iBase> csCheckerTextureLoader::Parse (iDocumentNode* node,
   csRef<iImage> Image = csCreateXORPatternImage (w, h, depth, color.red,
   	color.green, color.blue);
 
-  csRef<iGraphics3D> G3D = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  csRef<iGraphics3D> G3D = csQueryRegistry<iGraphics3D> (object_reg);
   if (!G3D) return 0;
   csRef<iTextureManager> tm = G3D->GetTextureManager();
   if (!tm) return 0;
-  csRef<iEngine> Engine = CS_QUERY_REGISTRY (object_reg, iEngine);
+  csRef<iEngine> Engine = csQueryRegistry<iEngine> (object_reg);
   if (!Engine)
     return 0;
 
+  csRef<scfString> fail_reason;
+  fail_reason.AttachNew (new scfString ());
   csRef<iTextureHandle> TexHandle (tm->RegisterTexture (Image, 
-    (ctx && ctx->HasFlags()) ? ctx->GetFlags() : CS_TEXTURE_3D));
-  if (!TexHandle) return 0;
+    (ctx && ctx->HasFlags()) ? ctx->GetFlags() : CS_TEXTURE_3D, fail_reason));
+  if (!TexHandle)
+  {
+    ReportError (object_reg, "crystalspace.checkerloader",
+	"Error creating texture: %s", fail_reason->GetData ());
+    return 0;
+  }
 
   csRef<iTextureWrapper> TexWrapper =
 	Engine->GetTextureList ()->NewTexture(TexHandle);
@@ -481,12 +537,14 @@ csPtr<iBase> csCheckerTextureLoader::Parse (iDocumentNode* node,
   return csPtr<iBase> ((iBase*)TexWrapper);
 }
 
+
+
 //----------------------------------------------------------------------------
 
 SCF_IMPLEMENT_FACTORY(csCubemapTextureLoader)
 
 csCubemapTextureLoader::csCubemapTextureLoader (iBase *p) : 
-  csBaseTextureLoader(p)
+  scfImplementationType(this, p)
 {
   InitTokenTable (xmltokens);
 }
@@ -498,15 +556,15 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
 {
   if (!context) return 0;
   csRef<iTextureLoaderContext> ctx = csPtr<iTextureLoaderContext>
-    (SCF_QUERY_INTERFACE (context, iTextureLoaderContext));
+    (scfQueryInterface<iTextureLoaderContext> (context));
   if (!ctx) return 0;
   
-  csRef<iEngine> Engine = CS_QUERY_REGISTRY (object_reg, iEngine);
-  csRef<iGraphics3D> G3D = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  csRef<iEngine> Engine = csQueryRegistry<iEngine> (object_reg);
+  csRef<iGraphics3D> G3D = csQueryRegistry<iGraphics3D> (object_reg);
   csRef<iTextureManager> tm = G3D->GetTextureManager();
-  csRef<iLoader> loader = CS_QUERY_REGISTRY (object_reg, iLoader);
+  csRef<iLoader> loader = csQueryRegistry<iLoader> (object_reg);
   csRef<iSyntaxService> SyntaxService = 
-    CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+    csQueryRegistry<iSyntaxService> (object_reg);
 
   csRef<csImageCubeMapMaker> cube;
   cube.AttachNew (new csImageCubeMapMaker (ctx->GetImage()));
@@ -524,6 +582,7 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
     csStringID id = xmltokens.Request (value);
     switch (id)
     {
+      case XMLTOKEN_POSZ:
       case XMLTOKEN_NORTH:
         fname = child->GetContentsValue ();
 	if (!fname)
@@ -537,6 +596,7 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
 	cube->SetSubImage (4, csRef<iImage>(loader->LoadImage (fname, Format)));
         break;
     
+      case XMLTOKEN_NEGZ:
       case XMLTOKEN_SOUTH:
         fname = child->GetContentsValue ();
 	if (!fname)
@@ -550,6 +610,7 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
 	cube->SetSubImage (5, csRef<iImage>(loader->LoadImage (fname, Format)));
         break;
     
+      case XMLTOKEN_POSX:
       case XMLTOKEN_EAST:
         fname = child->GetContentsValue ();
 	if (!fname)
@@ -563,6 +624,7 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
 	cube->SetSubImage (0, csRef<iImage>(loader->LoadImage (fname, Format)));
         break;
     
+      case XMLTOKEN_NEGX:
       case XMLTOKEN_WEST:
         fname = child->GetContentsValue ();
 	if (!fname)
@@ -576,6 +638,7 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
 	cube->SetSubImage (1, csRef<iImage>(loader->LoadImage (fname, Format)));
         break;
     
+      case XMLTOKEN_POSY:
       case XMLTOKEN_TOP:
         fname = child->GetContentsValue ();
 	if (!fname)
@@ -589,6 +652,7 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
 	cube->SetSubImage (2, csRef<iImage>(loader->LoadImage (fname, Format)));
         break;
     
+      case XMLTOKEN_NEGY:
       case XMLTOKEN_BOTTOM:
         fname = child->GetContentsValue ();
 	if (!fname)
@@ -605,9 +669,16 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
   }
 
 
+  csRef<scfString> fail_reason;
+  fail_reason.AttachNew (new scfString ());
   csRef<iTextureHandle> TexHandle (tm->RegisterTexture (cube, 
-    ctx->HasFlags() ? ctx->GetFlags() : CS_TEXTURE_3D));
-  if (!TexHandle) return 0;
+    ctx->HasFlags() ? ctx->GetFlags() : CS_TEXTURE_3D, fail_reason));
+  if (!TexHandle)
+  {
+    ReportError (object_reg, "crystalspace.checkertextureloader",
+	"Error creating texture: %s", fail_reason->GetData ());
+    return 0;
+  }
 
   csRef<iTextureWrapper> TexWrapper =
 	Engine->GetTextureList ()->NewTexture(TexHandle);
@@ -621,7 +692,8 @@ csPtr<iBase> csCubemapTextureLoader::Parse (iDocumentNode* node,
 
 SCF_IMPLEMENT_FACTORY(csTexture3DLoader)
 
-csTexture3DLoader::csTexture3DLoader (iBase *p) : csBaseTextureLoader(p)
+csTexture3DLoader::csTexture3DLoader (iBase *p) :
+  scfImplementationType(this, p)
 {
   InitTokenTable (xmltokens);
 }
@@ -633,15 +705,15 @@ csPtr<iBase> csTexture3DLoader::Parse (iDocumentNode* node,
 {
   if (!context) return 0;
   csRef<iTextureLoaderContext> ctx = csPtr<iTextureLoaderContext>
-    (SCF_QUERY_INTERFACE (context, iTextureLoaderContext));
+    (scfQueryInterface<iTextureLoaderContext> (context));
   if (!ctx) return 0;
   
-  csRef<iEngine> Engine = CS_QUERY_REGISTRY (object_reg, iEngine);
-  csRef<iGraphics3D> G3D = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  csRef<iEngine> Engine = csQueryRegistry<iEngine> (object_reg);
+  csRef<iGraphics3D> G3D = csQueryRegistry<iGraphics3D> (object_reg);
   csRef<iTextureManager> tm = G3D->GetTextureManager();
-  csRef<iLoader> loader = CS_QUERY_REGISTRY (object_reg, iLoader);
+  csRef<iLoader> loader = csQueryRegistry<iLoader> (object_reg);
   csRef<iSyntaxService> SyntaxService = 
-    CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+    csQueryRegistry<iSyntaxService> (object_reg);
 
   int Format = tm->GetTextureFormat ();
   csRef<csImageVolumeMaker> vol;
@@ -687,9 +759,16 @@ csPtr<iBase> csTexture3DLoader::Parse (iDocumentNode* node,
   }
 
 
+  csRef<scfString> fail_reason;
+  fail_reason.AttachNew (new scfString ());
   csRef<iTextureHandle> TexHandle (tm->RegisterTexture (vol, 
-    ctx->HasFlags() ? ctx->GetFlags() : CS_TEXTURE_3D));
-  if (!TexHandle) return 0;
+    ctx->HasFlags() ? ctx->GetFlags() : CS_TEXTURE_3D, fail_reason));
+  if (!TexHandle)
+  {
+    ReportError (object_reg, "crystalspace.3dtextureloader",
+	"Error creating texture: %s", fail_reason->GetData ());
+    return 0;
+  }
 
   csRef<iTextureWrapper> TexWrapper =
 	Engine->GetTextureList ()->NewTexture(TexHandle);
@@ -698,3 +777,65 @@ csPtr<iBase> csTexture3DLoader::Parse (iDocumentNode* node,
   TexWrapper->IncRef ();
   return csPtr<iBase> (TexWrapper);
 }
+
+
+//----------------------------------------------------------------------------
+
+csMissingTextureLoader::csMissingTextureLoader (iBase *p) : 
+  scfImplementationType(this, p)
+{
+}
+
+csPtr<iBase> csMissingTextureLoader::Parse (iDocumentNode* node, 
+                                            iStreamSource*,
+                                            iLoaderContext* /*ldr_context*/,
+                                            iBase* context)
+{
+  int width = 64, height = 64;
+  csRef<iTextureLoaderContext> ctx;
+  if (context)
+  {
+    ctx = scfQueryInterface<iTextureLoaderContext> (context);
+    if (ctx)
+    {
+      if (ctx->HasSize())
+      {
+        ctx->GetSize (width, height);
+
+        // Make square and next lower pow2, and at least 2^4 = 16
+        int a = csLog2 (width), b = csLog2 (height);
+        int newP = csMax (csMin (a, b), 4);
+        width = height = 1 << newP;        
+      }
+    }
+  }
+ 
+  csRef<iImage> image = GenerateErrorTexture (width, height);
+
+
+  csRef<iGraphics3D> G3D = csQueryRegistry<iGraphics3D> (object_reg);
+  if (!G3D) return 0;
+  csRef<iTextureManager> tm = G3D->GetTextureManager();
+  if (!tm) return 0;
+  csRef<iEngine> Engine = csQueryRegistry<iEngine> (object_reg);
+  if (!Engine)
+    return 0;
+
+  csRef<scfString> fail_reason;
+  fail_reason.AttachNew (new scfString ());
+  csRef<iTextureHandle> TexHandle (tm->RegisterTexture (image, 
+    (ctx && ctx->HasFlags()) ? ctx->GetFlags() : CS_TEXTURE_3D, fail_reason));
+  if (!TexHandle)
+  {
+    ReportError (object_reg, "crystalspace.missingtextureloader",
+      "Error creating texture: %s", fail_reason->GetData ());
+    return 0;
+  }
+
+  csRef<iTextureWrapper> TexWrapper =
+    Engine->GetTextureList ()->NewTexture(TexHandle);
+  TexWrapper->SetImageFile (image);
+
+  TexWrapper->IncRef ();
+  return csPtr<iBase> ((iBase*)TexWrapper);
+}
