@@ -20,6 +20,7 @@
 #include "cssysdef.h"
 #include "csloader.h"
 #include "csutil/scanstr.h"
+#include "csutil/scfstr.h"
 #include "iutil/document.h"
 #include "csgfx/rgbpixel.h"
 #include "cstool/gentrtex.h"
@@ -30,12 +31,16 @@
 #include "iutil/object.h"
 #include "csutil/csstring.h"
 #include "csutil/databuf.h"
+#include "csutil/scf_implementation.h"
 #include "igraphic/imageio.h"
 #include "iutil/objreg.h"
 #include "iutil/vfs.h"
 #include "iutil/cache.h"
 
-struct PrsHeightMapData : public iGenerateImageFunction
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
+struct PrsHeightMapData :
+  public scfImplementation1<PrsHeightMapData, iGenerateImageFunction>
 {
   csRef<iImage> im;
   int iw, ih;	// Image width and height.
@@ -44,26 +49,15 @@ struct PrsHeightMapData : public iGenerateImageFunction
   float hscale, hshift;
   bool slope;
   bool flipx, flipy;
-  SCF_DECLARE_IBASE;
 
-  PrsHeightMapData (bool s) : slope (s)
-  {
-    flipx=false; flipy=false;
-    SCF_CONSTRUCT_IBASE (0);
-  }
-  virtual ~PrsHeightMapData ()
-  {
-    SCF_DESTRUCT_IBASE();
-  }
+  PrsHeightMapData (bool s) :
+    scfImplementationType(this), slope (s), flipx(false), flipy(false) { }
+  virtual ~PrsHeightMapData () { }
   float GetHeight (float dx, float dy);
   float GetSlope (float dx, float dy);
   virtual float GetValue (float dx, float dy);
 };
 
-SCF_IMPLEMENT_IBASE (PrsHeightMapData)
-  SCF_IMPLEMENTS_INTERFACE (iGenerateImageFunction)
-SCF_IMPLEMENT_IBASE_END
-
 float PrsHeightMapData::GetSlope (float x, float y)
 {
   float div = 0.02f;
@@ -444,7 +438,7 @@ bool csLoader::ParseHeightgen (iLoaderContext* ldr_context, iDocumentNode* node)
           
           csString cache_enable = child->GetAttributeValue ("cache");
           csRef<iImage> img;
-          csRef<iImageIO> imageio (CS_QUERY_REGISTRY (object_reg, iImageIO));
+          csRef<iImageIO> imageio (csQueryRegistry<iImageIO> (object_reg));
 
           // get from cache
           if(Engine && cache_enable == "yes")
@@ -492,13 +486,15 @@ bool csLoader::ParseHeightgen (iLoaderContext* ldr_context, iDocumentNode* node)
           }
 
           // add texture
+	  csRef<scfString> fail_reason;
+	  fail_reason.AttachNew (new scfString ());
 	  csRef<iTextureHandle> TexHandle (G3D->GetTextureManager ()
-	  	->RegisterTexture (img, CS_TEXTURE_3D));
+	  	->RegisterTexture (img, CS_TEXTURE_3D, fail_reason));
 	  if (!TexHandle)
 	  {
 	    ReportError (
 	      "crystalspace.maploader.parse.heightgen",
-	      "Cannot create texture!");
+	      "Cannot create texture : %s", fail_reason->GetData ());
 	    return false;
 	  }
 	  iTextureWrapper *TexWrapper = Engine->GetTextureList ()
@@ -519,3 +515,4 @@ bool csLoader::ParseHeightgen (iLoaderContext* ldr_context, iDocumentNode* node)
   return true;
 }
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
