@@ -115,8 +115,14 @@ bool csLoader::ParseTextureList (iLoaderContext* ldr_context,
 	        return false;
         break;
       case XMLTOKEN_HEIGHTGEN:
+	SyntaxService->Report (
+	  "crystalspace.maploader.parse.texture",
+	  CS_REPORTER_SEVERITY_WARNING,
+	  child,
+	  "<heightmap> is obsolete: it was intended for use together with the "
+	  "'terrfunc' mesh which is long gone.");
         if (!ParseHeightgen (ldr_context, child))
-	        return false;
+	  return false;
         break;
       case XMLTOKEN_CUBEMAP:
         if (!ParseCubemap (ldr_context, child))
@@ -339,7 +345,7 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
     if (image && type.IsEmpty ())
     {
       // special treatment for animated textures
-      csRef<iAnimatedImage> anim = SCF_QUERY_INTERFACE (image, iAnimatedImage);
+      csRef<iAnimatedImage> anim = scfQueryInterface<iAnimatedImage> (image);
       if (anim && anim->IsAnimated())
       {
 	type = PLUGIN_TEXTURELOADER_ANIMIMG;
@@ -418,8 +424,8 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
   if (plugin)
   {
     csRef<iBase> b = plugin->Parse (ParamsNode,
-      0/*ssource*/, ldr_context, CS_STATIC_CAST(iBase*, &context));
-    if (b) tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
+      0/*ssource*/, ldr_context, static_cast<iBase*> (&context));
+    if (b) tex = scfQueryInterface<iTextureWrapper> (b);
   }
 
   if (!tex)
@@ -429,17 +435,36 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
       CS_REPORTER_SEVERITY_WARNING,
       node, "Could not load texture '%s', using checkerboard instead", txtname);
 
-    if (!BuiltinCheckerTexLoader)
+    /*if (!BuiltinCheckerTexLoader)
     {
       csCheckerTextureLoader* ctl = new csCheckerTextureLoader (0);
       ctl->Initialize (object_reg);
       BuiltinCheckerTexLoader.AttachNew (ctl);
     }
     csRef<iBase> b = BuiltinCheckerTexLoader->Parse (ParamsNode,
-      0/*ssource*/, ldr_context, CS_STATIC_CAST(iBase*, &context));
-    CS_ASSERT(b);
-    tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
-    CS_ASSERT(tex);
+      0, ldr_context, static_cast<iBase*> (&context));*/
+    if (!BuiltinErrorTexLoader)
+    {
+      csMissingTextureLoader* mtl = new csMissingTextureLoader (0);
+      mtl->Initialize (object_reg);
+      BuiltinErrorTexLoader.AttachNew (mtl);
+    }
+    csRef<iBase> b = BuiltinErrorTexLoader->Parse (ParamsNode,
+      0, ldr_context, static_cast<iBase*> (&context));
+    if (!b.IsValid())
+    {
+      static bool noMissingWarned = false;
+      if (!noMissingWarned)
+      {
+	SyntaxService->Report (
+	  "crystalspace.maploader.parse.texture",
+	  CS_REPORTER_SEVERITY_ERROR,
+	  node, "Could not create default texture!");
+	noMissingWarned = true;
+	return 0;
+      }
+    }
+    tex = scfQueryInterface<iTextureWrapper> (b);
   }
 
   if (tex)
@@ -455,22 +480,15 @@ iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
     if (overrideAlphaType)
       tex->GetTextureHandle()->SetAlphaType (alphaType);
 
-    csRef<iProcTexture> ipt = SCF_QUERY_INTERFACE (tex, iProcTexture);
+    csRef<iProcTexture> ipt = scfQueryInterface<iProcTexture> (tex);
     if (ipt)
       ipt->SetAlwaysAnimate (always_animate);
     AddToRegion (ldr_context, tex->QueryObject ());
 
     size_t i;
-    for (i = 0 ; i < key_nodes.Length () ; i++)
+    for (i = 0 ; i < key_nodes.GetSize () ; i++)
     {
-      iKeyValuePair* kvp = 0;
-      SyntaxService->ParseKey (key_nodes[i], kvp);
-      if (kvp)
-      {
-        tex->QueryObject()->ObjAdd (kvp->QueryObject ());
-	kvp->DecRef ();
-      }
-      else
+      if (!ParseKey (key_nodes[i], tex->QueryObject()))
 	return 0;
     }
   }
@@ -498,11 +516,7 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
 
   iTextureWrapper* texh = 0;
   bool col_set = false;
-  csRGBcolor col;
-  float diffuse = CS_DEFMAT_DIFFUSE;
-  float ambient = CS_DEFMAT_AMBIENT;
-  float reflection = CS_DEFMAT_REFLECTION;
-
+  csColor col;
   
   bool shaders_mentioned = false;	// If true there were shaders.
   csArray<csStringID> shadertypes;
@@ -539,28 +553,21 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
       case XMLTOKEN_COLOR:
 	{
           col_set = true;
-	  csColor color;
-          if (!SyntaxService->ParseColor (child, color))
+          if (!SyntaxService->ParseColor (child, col))
 	    return 0;
-	  col.red = csQint (color.red * 255.99f);
-	  col.green = csQint (color.green * 255.99f);
-	  col.blue = csQint (color.blue * 255.99f);
 	}
         break;
       case XMLTOKEN_DIFFUSE:
-	diffuse = child->GetContentsValueAsFloat ();
-        break;
       case XMLTOKEN_AMBIENT:
-	ambient = child->GetContentsValueAsFloat ();
-        break;
       case XMLTOKEN_REFLECTION:
-	reflection = child->GetContentsValueAsFloat ();
+        ReportWarning ("crystalspace.maploader.parse.material",
+          child, "Syntax not supported any more. Use shader variables instead");
         break;
       case XMLTOKEN_SHADER:
         {
 	  shaders_mentioned = true;
-          csRef<iShaderManager> shaderMgr = CS_QUERY_REGISTRY (object_reg,
-	  	iShaderManager);
+          csRef<iShaderManager> shaderMgr = 
+	  	csQueryRegistry<iShaderManager> (object_reg);
           if (!shaderMgr)
           {
             ReportNotify ("iShaderManager not found, ignoring shader!");
@@ -591,24 +598,12 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
         break;
       case XMLTOKEN_SHADERVAR:
         {
-          csRef<iShaderManager> shaderMgr = CS_QUERY_REGISTRY (object_reg,
-            iShaderManager);
-          if (!shaderMgr)
-          {
-            ReportNotify ("iShaderManager not found, ignoring shadervar!");
-            break;
-          }
           //create a new variable
-          const char* varname = child->GetAttributeValue ("name");
-          /*csRef<csShaderVariable> var = 
-            shaderMgr->CreateVariable (strings->Request (varname));*/
 	  csRef<csShaderVariable> var;
-	  var.AttachNew (new csShaderVariable (stringSet->Request (varname)));
+	  var.AttachNew (new csShaderVariable);
 
-          if (!SyntaxService->ParseShaderVar (child, *var))
+          if (!SyntaxService->ParseShaderVar (ldr_context, child, *var))
           {
-            ReportNotify ("Error loading shader variable '%s' in material '%s'.", 
-              varname, matname);
             break;
           }
           shadervars.Push (var);
@@ -623,8 +618,11 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
   csRef<iMaterial> material = Engine->CreateBaseMaterial (texh);
 
   if (col_set)
-    material->SetFlatColor (col);
-  material->SetReflection (diffuse, ambient, reflection);
+  {
+    csShaderVariable* flatSV = material->GetVariableAdd (
+      stringSet->Request (CS_MATERIAL_VARNAME_FLATCOLOR));
+    flatSV->SetValue (col);
+  }
 
   iMaterialWrapper *mat;
  
@@ -643,23 +641,17 @@ iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
   }
   
   size_t i;
-  for (i=0; i<shaders.Length (); i++)
+  for (i=0; i<shaders.GetSize (); i++)
     //if (shaders[i]->Prepare ())
       material->SetShader (shadertypes[i], shaders[i]);
-  for (i=0; i<shadervars.Length (); i++)
+  for (i=0; i<shadervars.GetSize (); i++)
     material->AddVariable (shadervars[i]);
 
   // dereference material since mat already incremented it
 
-  for (i = 0 ; i < key_nodes.Length () ; i++)
+  for (i = 0 ; i < key_nodes.GetSize () ; i++)
   {
-    iKeyValuePair* kvp = 0;
-    SyntaxService->ParseKey (key_nodes[i], kvp);
-    if (kvp)
-    {
-      mat->QueryObject ()->ObjAdd (kvp->QueryObject ());
-      kvp->DecRef ();
-    } else
+    if (!ParseKey (key_nodes[i], mat->QueryObject()))
       return 0;
   }
   AddToRegion (ldr_context, mat->QueryObject ());
@@ -691,7 +683,7 @@ iTextureWrapper* csLoader::ParseCubemap (iLoaderContext* ldr_context,
 
   csRef<iBase> b = plugin->Parse (node, 0/*ssource*/, ldr_context, context);
   csRef<iTextureWrapper> tex;
-  if (b) tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
+  if (b) tex = scfQueryInterface<iTextureWrapper> (b);
 
   if (tex)
   {
@@ -727,7 +719,7 @@ iTextureWrapper* csLoader::ParseTexture3D (iLoaderContext* ldr_context,
 
   csRef<iBase> b = plugin->Parse (node, 0/*ssource*/, ldr_context, context);
   csRef<iTextureWrapper> tex;
-  if (b) tex = SCF_QUERY_INTERFACE (b, iTextureWrapper);
+  if (b) tex = scfQueryInterface<iTextureWrapper> (b);
 
   if (tex)
   {
