@@ -24,6 +24,7 @@
 #include "csutil/util.h"
 #include "csutil/scfstr.h"
 #include "csutil/event.h"
+#include "csutil/eventnames.h"
 #include "iutil/event.h"
 #include "iutil/eventq.h"
 #include "csgeom/frustum.h"
@@ -34,7 +35,7 @@
 #include "csgeom/sphere.h"
 #include "csgeom/kdtree.h"
 #include "igeom/polymesh.h"
-#include "igeom/objmodel.h"
+#include "imesh/objmodel.h"
 #include "csutil/flags.h"
 #include "iutil/objreg.h"
 #include "ivideo/graph2d.h"
@@ -58,22 +59,10 @@ CS_IMPLEMENT_PLUGIN
 SCF_IMPLEMENT_FACTORY (csFrustumVis)
 
 
-SCF_IMPLEMENT_IBASE (csFrustumVis)
-  SCF_IMPLEMENTS_INTERFACE (iVisibilityCuller)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csFrustumVis::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csFrustumVis::eiEventHandler)
-  SCF_IMPLEMENTS_INTERFACE (iEventHandler)
-SCF_IMPLEMENT_IBASE_END
-
 //----------------------------------------------------------------------
 
-class csFrustVisObjIt : public iVisibilityObjectIterator
+class csFrustVisObjIt :
+  public scfImplementation1<csFrustVisObjIt, iVisibilityObjectIterator>
 {
 private:
   csArray<iVisibilityObject*>* vector;
@@ -81,12 +70,10 @@ private:
   bool* vistest_objects_inuse;
 
 public:
-  SCF_DECLARE_IBASE;
-
   csFrustVisObjIt (csArray<iVisibilityObject*>* vector,
-  	bool* vistest_objects_inuse)
+    bool* vistest_objects_inuse) :
+    scfImplementationType(this)
   {
-    SCF_CONSTRUCT_IBASE (0);
     csFrustVisObjIt::vector = vector;
     csFrustVisObjIt::vistest_objects_inuse = vistest_objects_inuse;
     if (vistest_objects_inuse) *vistest_objects_inuse = true;
@@ -101,7 +88,6 @@ public:
       *vistest_objects_inuse = false;
     else
       delete vector;
-    SCF_DESTRUCT_IBASE();
   }
 
   virtual iVisibilityObject* Next()
@@ -109,14 +95,14 @@ public:
     if (position == (size_t)-1) return 0;
     iVisibilityObject* vo = vector->Get (position);
     position++;
-    if (position == vector->Length ())
+    if (position == vector->GetSize ())
       position = (size_t)-1;
     return vo;
   }
 
   virtual void Reset()
   {
-    if (vector == 0 || vector->Length () < 1)
+    if (vector == 0 || vector->GetSize () < 1)
       position = (size_t)-1;
     else
       position = 0;
@@ -124,21 +110,12 @@ public:
 
   virtual bool HasNext () const
   {
-    return ((position != (size_t)-1) && position <= vector->Length ());
+    return ((position != (size_t)-1) && position <= vector->GetSize ());
   }
 };
 
-SCF_IMPLEMENT_IBASE (csFrustVisObjIt)
-  SCF_IMPLEMENTS_INTERFACE (iVisibilityObjectIterator)
-SCF_IMPLEMENT_IBASE_END
-
 //----------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csFrustVisObjectWrapper)
-  SCF_IMPLEMENTS_INTERFACE (iObjectModelListener)
-  SCF_IMPLEMENTS_INTERFACE (iMovableListener)
-SCF_IMPLEMENT_IBASE_END
-
 void csFrustVisObjectWrapper::ObjectModelChanged (iObjectModel* /*model*/)
 {
   frustvis->AddObjectToUpdateQueue (this);
@@ -172,52 +149,45 @@ public:
 //----------------------------------------------------------------------
 
 csFrustumVis::csFrustumVis (iBase *iParent) :
-	vistest_objects (256, 256),
-	visobj_vector (256, 256),
-	update_queue (151, 59)
+  scfImplementationType (this, iParent),
+  vistest_objects (256, 256),
+  visobj_vector (256, 256),
+  update_queue (151, 59)
 {
-  SCF_CONSTRUCT_IBASE (iParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
   object_reg = 0;
   kdtree = 0;
   current_vistest_nr = 1;
   vistest_objects_inuse = false;
   updating = false;
-
-  scfiEventHandler = new eiEventHandler (this);
 }
 
 csFrustumVis::~csFrustumVis ()
 {
-  if (scfiEventHandler)
+  if (object_reg)
   {
-    csRef<iEventQueue> q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+    csRef<iEventQueue> q = csQueryRegistry<iEventQueue> (object_reg);
     if (q)
-      q->RemoveListener (scfiEventHandler);
-    scfiEventHandler->DecRef ();
+      CS::RemoveWeakListener (q, weakEventHandler);
   }
 
-  while (visobj_vector.Length () > 0)
+  while (visobj_vector.GetSize () > 0)
   {
-    csFrustVisObjectWrapper* visobj_wrap = visobj_vector.Pop ();
+    csRef<csFrustVisObjectWrapper> visobj_wrap = visobj_vector.Pop ();
     iVisibilityObject* visobj = visobj_wrap->visobj;
     visobj->GetObjectModel ()->RemoveListener (
 		      (iObjectModelListener*)visobj_wrap);
     iMovable* movable = visobj->GetMovable ();
     movable->RemoveListener ((iMovableListener*)visobj_wrap);
     kdtree->RemoveObject (visobj_wrap->child);
-    visobj->DecRef ();
   }
   delete kdtree;
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csFrustumVis::HandleEvent (iEvent& ev)
 {
   if (ev.Name == CanvasResize)
   {
-    csRef<iGraphics3D> g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+    csRef<iGraphics3D> g3d = csQueryRegistry<iGraphics3D> (object_reg);
     scr_width = g3d->GetWidth ();
     scr_height = g3d->GetHeight ();
     //printf ("Got resize %dx%d!\n", scr_width, scr_height);fflush (stdout);
@@ -231,7 +201,7 @@ bool csFrustumVis::Initialize (iObjectRegistry *object_reg)
 
   delete kdtree;
 
-  csRef<iGraphics3D> g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  csRef<iGraphics3D> g3d = csQueryRegistry<iGraphics3D> (object_reg);
   if (g3d)
   {
     scr_width = g3d->GetWidth ();
@@ -245,15 +215,18 @@ bool csFrustumVis::Initialize (iObjectRegistry *object_reg)
   }
 
   kdtree = new csKDTree ();
-  csFrustVisObjectDescriptor* desc = new csFrustVisObjectDescriptor ();
+  csRef<csFrustVisObjectDescriptor> desc;
+  desc.AttachNew (new csFrustVisObjectDescriptor ());
   kdtree->SetObjectDescriptor (desc);
-  desc->DecRef ();
 
   csRef<iGraphics2D> g2d = csQueryRegistry<iGraphics2D> (object_reg);
-  CanvasResize = csevCanvasResize(object_reg, g2d);
-  csRef<iEventQueue> q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
-  if (q)
-    q->RegisterListener (scfiEventHandler, CanvasResize);
+  if (g2d)
+  {
+    CanvasResize = csevCanvasResize(object_reg, g2d);
+    csRef<iEventQueue> q = csQueryRegistry<iEventQueue> (object_reg);
+    if (q)
+      CS::RegisterWeakListener (q, this, CanvasResize, weakEventHandler);
+  }
 
   return true;
 }
@@ -267,12 +240,11 @@ void csFrustumVis::CalculateVisObjBBox (iVisibilityObject* visobj, csBox3& bbox)
   iMovable* movable = visobj->GetMovable ();
   if (movable->IsFullTransformIdentity ())
   {
-    visobj->GetObjectModel ()->GetObjectBoundingBox (bbox);
+    bbox = visobj->GetObjectModel ()->GetObjectBoundingBox ();
   }
   else
   {
-    csBox3 box;
-    visobj->GetObjectModel ()->GetObjectBoundingBox (box);
+    const csBox3& box = visobj->GetObjectModel ()->GetObjectBoundingBox ();
     csReversibleTransform trans = movable->GetFullTransform ();
     bbox.StartBoundingBox (trans.This2Other (box.GetCorner (0)));
     bbox.AddBoundingVertexSmart (trans.This2Other (box.GetCorner (1)));
@@ -289,7 +261,7 @@ void csFrustumVis::RegisterVisObject (iVisibilityObject* visobj)
 {
 #ifdef CS_DEBUG
   size_t i;
-  for (i = 0 ; i < visobj_vector.Length () ; i++)
+  for (i = 0 ; i < visobj_vector.GetSize () ; i++)
   {
     if (visobj_vector[i]->visobj == visobj)
     {
@@ -297,10 +269,9 @@ void csFrustumVis::RegisterVisObject (iVisibilityObject* visobj)
     }
   }
 #endif
-  csFrustVisObjectWrapper* visobj_wrap = new csFrustVisObjectWrapper (
-		  this);
+  csRef<csFrustVisObjectWrapper> visobj_wrap;
+  visobj_wrap.AttachNew (new csFrustVisObjectWrapper (this));
   visobj_wrap->visobj = visobj;
-  visobj->IncRef ();
   iMovable* movable = visobj->GetMovable ();
   visobj_wrap->update_number = movable->GetUpdateNumber ();
   visobj_wrap->shape_number = visobj->GetObjectModel ()->GetShapeNumber ();
@@ -330,7 +301,7 @@ void csFrustumVis::RegisterVisObject (iVisibilityObject* visobj)
 void csFrustumVis::UnregisterVisObject (iVisibilityObject* visobj)
 {
   size_t i;
-  for (i = 0 ; i < visobj_vector.Length () ; i++)
+  for (i = 0 ; i < visobj_vector.GetSize () ; i++)
   {
     csFrustVisObjectWrapper* visobj_wrap = visobj_vector[i];
     if (visobj_wrap->visobj == visobj)
@@ -338,10 +309,9 @@ void csFrustumVis::UnregisterVisObject (iVisibilityObject* visobj)
       update_queue.Delete (visobj_wrap);
       visobj->GetMovable ()->RemoveListener (
 		  (iMovableListener*)visobj_wrap);
-      visobj->GetObjectModel ()->RemoveListener (
-		  (iObjectModelListener*)visobj_wrap);
+      iObjectModel* objmodel = visobj->GetObjectModel ();
+      objmodel->RemoveListener ((iObjectModelListener*)visobj_wrap);
       kdtree->RemoveObject (visobj_wrap->child);
-      visobj->DecRef ();
 #ifdef CS_DEBUG
       // To easily recognize that the vis wrapper has been deleted:
       visobj_wrap->frustvis = (csFrustumVis*)0xdeadbeef;
@@ -1108,7 +1078,7 @@ struct CastShadows_Front2BackData
   iFrustumView* fview;
   csPlane3 planes[32];
   ShadObj* shadobjs;
-  int num_shadobjs;
+  size_t num_shadobjs;
 };
 
 static int compare_shadobj (const void* el1, const void* el2)
@@ -1216,13 +1186,13 @@ void csFrustumVis::CastShadows (iFrustumView* fview)
   // the receivers are processed.
   //======================================
 
-  data.shadobjs = new ShadObj [visobj_vector.Length () * 2];
+  data.shadobjs = new ShadObj [visobj_vector.GetSize () * 2];
   data.num_shadobjs = 0;
 
   // First check if we need to do frustum clipping.
   csFrustum* lf = fview->GetFrustumContext ()->GetLightFrustum ();
   uint32 planes_mask = 0;
-  int i;
+  size_t i;
 
   // Traverse the kd-tree to find all relevant objects.
   // @@@ What if the frustum is bigger???
@@ -1233,7 +1203,7 @@ void csFrustumVis::CastShadows (iFrustumView* fview)
     fflush (stdout);
     return;
   }
-  int i1 = lf->GetVertexCount () - 1;
+  size_t i1 = lf->GetVertexCount () - 1;
   for (i = 0 ; i < lf->GetVertexCount () ; i1 = i, i++)
   {
     planes_mask = (planes_mask<<1)|1;
