@@ -34,17 +34,16 @@
 
 #include "bindoc.h"
 
+CS_PLUGIN_NAMESPACE_BEGIN(BinDoc)
+{
+
 // =================================================
 //  csBinaryDocAttributeIterator
 // =================================================
 
-SCF_IMPLEMENT_IBASE(csBinaryDocAttributeIterator)
-  SCF_IMPLEMENTS_INTERFACE(iDocumentAttributeIterator)
-SCF_IMPLEMENT_IBASE_END
-
-csBinaryDocAttributeIterator::csBinaryDocAttributeIterator ()
+csBinaryDocAttributeIterator::csBinaryDocAttributeIterator () :
+  scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
 }
 
 void csBinaryDocAttributeIterator::SetTo (csBdNode* node,
@@ -64,7 +63,6 @@ void csBinaryDocAttributeIterator::SetTo (csBdNode* node,
 
 csBinaryDocAttributeIterator::~csBinaryDocAttributeIterator()
 {
-  SCF_DESTRUCT_IBASE();
 }
 
 bool csBinaryDocAttributeIterator::HasNext ()
@@ -151,8 +149,7 @@ const char* csBdAttr::GetValueStr (csBinaryDocument* doc) const
     }
     else
     {
-      return doc->GetInIDString (
-	csLittleEndianLong (value));
+      return doc->GetInIDString (csLittleEndian::UInt32 (value));
     }
   }
 }
@@ -161,32 +158,23 @@ const char* csBdAttr::GetValueStr (csBinaryDocument* doc) const
 //  csBinaryDocAttribute
 // =================================================
 
-void csBinaryDocAttribute::IncRef ()
-{
-  scfRefCount++;
-}
-
 void csBinaryDocAttribute::DecRef ()
 {
+  csRefTrackerAccess::TrackDecRef (scfObject, scfRefCount);
   if (scfRefCount == 1)
   {
+    // Not needed atm
+    //scfRemoveRefOwners ();
+    //if (scfParent) scfParent->DecRef();
     node->doc->RecyclePoolAttr (this);
     return;
   }
   scfRefCount--;
 }
 
-SCF_IMPLEMENT_IBASE_GETREFCOUNT(csBinaryDocAttribute)
-SCF_IMPLEMENT_IBASE_REFOWNER(csBinaryDocAttribute)
-SCF_IMPLEMENT_IBASE_REMOVE_REF_OWNERS(csBinaryDocAttribute)
-SCF_IMPLEMENT_IBASE_QUERY(csBinaryDocAttribute)
-  SCF_IMPLEMENTS_INTERFACE(iDocumentAttribute)
-SCF_IMPLEMENT_IBASE_END
-
-csBinaryDocAttribute::csBinaryDocAttribute ()
+csBinaryDocAttribute::csBinaryDocAttribute () :
+  scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
-
   vstr = 0;
   vsptr = 0;
 }
@@ -195,7 +183,6 @@ csBinaryDocAttribute::~csBinaryDocAttribute ()
 {
   CleanData ();
   delete vstr; 
-  SCF_DESTRUCT_IBASE();
 }
 
 void csBinaryDocAttribute::CleanData ()
@@ -230,7 +217,7 @@ const char* csBinaryDocAttribute::GetValue ()
 	{
   	  char buf[50];
 	  cs_snprintf (buf, sizeof (buf) - 1, "%" PRId32, 
-	    (int32)csLittleEndianLong (attrPtr->value));
+	    (int32)csLittleEndian::UInt32 (attrPtr->value));
 	  delete[] vstr; 
 	  vstr = csStrNew (buf);
 	  vsptr = attrPtr;
@@ -243,7 +230,7 @@ const char* csBinaryDocAttribute::GetValue ()
 	{
   	  char buf[50];
 	  cs_snprintf (buf, sizeof (buf) - 1,
-	    "%g", csLongToFloat (csLittleEndianLong (attrPtr->value)));
+	    "%g", csLongToFloat (csLittleEndian::UInt32 (attrPtr->value)));
 	  delete[] vstr; 
 	  vstr = csStrNew (buf);
 	  vsptr = attrPtr;
@@ -269,11 +256,11 @@ int csBinaryDocAttribute::GetValueAsInt ()
       }
     case BD_VALUE_TYPE_INT:
       {
-	return (int)((int32)csLittleEndianLong (attrPtr->value));
+	return (int)((int32)csLittleEndian::UInt32 (attrPtr->value));
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return (int)csLongToFloat (csLittleEndianLong (attrPtr->value));
+	return (int)csLongToFloat (csLittleEndian::UInt32 (attrPtr->value));
       }
     default:
       return 0;
@@ -294,11 +281,11 @@ float csBinaryDocAttribute::GetValueAsFloat ()
       }
     case BD_VALUE_TYPE_INT:
       {
-	return (float)((int32)csLittleEndianLong (attrPtr->value));
+	return (float)((int32)csLittleEndian::UInt32 (attrPtr->value));
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return csLongToFloat (csLittleEndianLong (attrPtr->value));
+	return csLongToFloat (csLittleEndian::UInt32 (attrPtr->value));
       }
     default:
       return 0.0f;
@@ -329,7 +316,7 @@ bool csBinaryDocAttribute::GetValueAsBool ()
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return (csLongToFloat (csLittleEndianLong (attrPtr->value))== 0);
+	return (csLongToFloat (csLittleEndian::UInt32 (attrPtr->value))== 0);
       }
     default:
       return false;
@@ -389,13 +376,13 @@ void csBinaryDocAttribute::SetValue (const char* val)
     {
       attrPtr->flags = (attrPtr->flags & ~BD_VALUE_TYPE_MASK) | 
 	BD_VALUE_TYPE_INT;
-      attrPtr->value = csLittleEndianLong ((long)v);
+      attrPtr->value = csLittleEndian::UInt32 ((long)v);
     }
     else if (checkFloat (val, f))
     {
       attrPtr->flags = (attrPtr->flags & ~BD_VALUE_TYPE_MASK) | 
 	BD_VALUE_TYPE_FLOAT;
-      attrPtr->value = csLittleEndianLong (csFloatToLong (f));
+      attrPtr->value = csLittleEndian::UInt32 (csFloatToLong (f));
     }
     else 
     {
@@ -414,7 +401,7 @@ void csBinaryDocAttribute::SetValueAsInt (int v)
     delete[] vstr; vstr = 0;
     attrPtr->flags = (attrPtr->flags & ~BD_VALUE_TYPE_MASK) | 
       BD_VALUE_TYPE_INT;
-    attrPtr->value = csLittleEndianLong ((long)v);
+    attrPtr->value = csLittleEndian::UInt32 ((long)v);
   }
 }
 
@@ -426,7 +413,7 @@ void csBinaryDocAttribute::SetValueAsFloat (float f)
     delete[] vstr; vstr = 0;
     attrPtr->flags = (attrPtr->flags & ~BD_VALUE_TYPE_MASK) | 
       BD_VALUE_TYPE_FLOAT;
-    attrPtr->value = csLittleEndianLong (csFloatToLong (f));
+    attrPtr->value = csLittleEndian::UInt32 (csFloatToLong (f));
   }
 }
 
@@ -447,7 +434,7 @@ void csBinaryDocAttribute::Store (csMemFile* nodesFile)
     }
     else
     {
-      diskAttr.value = csLittleEndianLong (
+      diskAttr.value = csLittleEndian::UInt32 (
 	node->doc->GetOutStringID (attrPtr->GetValueStr (node->doc)));
     }
   }
@@ -465,11 +452,11 @@ void csBinaryDocAttribute::Store (csMemFile* nodesFile)
   }
   else
   {
-    diskAttr.nameID = csLittleEndianLong (
+    diskAttr.nameID = csLittleEndian::UInt32 (
       node->doc->GetOutStringID (attrPtr->GetNameStr (node->doc)));
     // @@@ What to do if nameID > BD_ATTR_MAX_NAME_ID?
     putFlagsInName = 
-      csLittleEndianLong (diskAttr.nameID) <= BD_ATTR_MAX_NAME_ID_WITH_FLAGS;
+      csLittleEndian::UInt32 (diskAttr.nameID) <= BD_ATTR_MAX_NAME_ID_WITH_FLAGS;
   }
   if (putFlagsInName)
   {
@@ -484,13 +471,9 @@ void csBinaryDocAttribute::Store (csMemFile* nodesFile)
 //  csBinaryDocNodeIterator
 // =================================================
 
-SCF_IMPLEMENT_IBASE(csBinaryDocNodeIterator)
-  SCF_IMPLEMENTS_INTERFACE(iDocumentNodeIterator)
-SCF_IMPLEMENT_IBASE_END
-
-csBinaryDocNodeIterator::csBinaryDocNodeIterator ()
+csBinaryDocNodeIterator::csBinaryDocNodeIterator () :
+  scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   value = 0;
 }
 
@@ -523,7 +506,6 @@ void csBinaryDocNodeIterator::SetTo (csBdNode* node,
 csBinaryDocNodeIterator::~csBinaryDocNodeIterator ()
 {
   delete[] value;
-  SCF_DESTRUCT_IBASE();
 }
 
 void csBinaryDocNodeIterator::FastForward()
@@ -561,6 +543,16 @@ csRef<iDocumentNode> csBinaryDocNodeIterator::Next ()
   return csPtr<iDocumentNode> (retNode);
 }
 
+size_t csBinaryDocNodeIterator::GetNextPosition () 
+{ 
+  return pos; 
+}
+
+size_t csBinaryDocNodeIterator::GetEndPosition () 
+{ 
+  return iteratedNode->ctNum(); 
+}
+
 // =================================================
 //  csBdNode
 // =================================================
@@ -591,10 +583,10 @@ csBdNode::~csBdNode ()
   {
     delete[] vstr;
     size_t i;
-    for (i = 0; i < attrs->Length(); i++)
+    for (i = 0; i < attrs->GetSize (); i++)
       doc->FreeBdAttr (attrs->Get (i));
     delete attrs;
-    for (i = 0; i < nodes->Length(); i++)
+    for (i = 0; i < nodes->GetSize (); i++)
       doc->FreeBdNode (nodes->Get (i));
     delete nodes;
   }
@@ -647,7 +639,7 @@ const char* csBdNode::GetValueStr (csBinaryDocument* doc) const
     }
     else
     {
-      return doc->GetInIDString (value);
+      return doc->GetInIDString (csLittleEndian::UInt32 (value));
     }
   }
 }
@@ -660,9 +652,10 @@ csBdAttr* csBdNode::atGetItem(int pos)
   }
   else
   {
-    void* at =  (void*)GetAttrTab();
-    return (csBdAttr*)((uint8*)at +
-      csLittleEndianLong (*((uint32*)at + pos + 1)));
+    bdNodeAttrTab* atab = GetAttrTab();
+    const uint32 attrOffsLE = 
+      *(((uint32*)((uint8*)atab + sizeof (bdNodeAttrTab))) + pos);
+    return (csBdAttr*)((uint8*)atab + csLittleEndian::UInt32 (attrOffsLE));
   }
 }
 
@@ -671,7 +664,7 @@ void csBdNode::atSetItem (csBdAttr* item, int pos)
   if (flags & BD_NODE_MODIFIED)
   {
     attrs->Put (pos, item);
-    if (attrs->Length() != 0)
+    if (attrs->GetSize () != 0)
       flags |= BD_NODE_HAS_ATTR;
   }
 }
@@ -694,7 +687,7 @@ void csBdNode::atInsertBefore (csBdAttr* item, int pos)
   if (flags & BD_NODE_MODIFIED)
   {
     attrs->Insert (pos, item);
-    if (attrs->Length() != 0)
+    if (attrs->GetSize () != 0)
       flags |= BD_NODE_HAS_ATTR;
   }
 }
@@ -704,7 +697,7 @@ void csBdNode::atRemove (int pos)
   if (flags & BD_NODE_MODIFIED)
   {
     attrs->DeleteIndex (pos);
-    if (attrs->Length() == 0)
+    if (attrs->GetSize () == 0)
       flags &= ~BD_NODE_HAS_ATTR;
   }
 }
@@ -713,11 +706,11 @@ uint csBdNode::atNum ()
 { 
   if (flags & BD_NODE_MODIFIED)
   {
-    return (uint)attrs->Length();
+    return (uint)attrs->GetSize ();
   }
   else
   {
-    return GetAttrTab()->num; 
+    return csLittleEndian::UInt32 (GetAttrTab()->num); 
   }
 }
 
@@ -729,9 +722,10 @@ csBdNode* csBdNode::ctGetItem(int pos)
   }
   else
   {
-    void* ct = (void*)GetChildTab();
-    return (csBdNode*)((uint8*)ct +
-      csLittleEndianLong (*((uint32*)ct + pos + 1)));
+    bdNodeChildTab* ctab = GetChildTab();
+    const uint32 childOffsLE = 
+      *(((uint32*)((uint8*)ctab + sizeof (bdNodeChildTab))) + pos);
+    return (csBdNode*)((uint8*)ctab + csLittleEndian::UInt32 (childOffsLE));
   }
 }
 
@@ -740,7 +734,7 @@ void csBdNode::ctSetItem (csBdNode* item, int pos)
   if (flags & BD_NODE_MODIFIED)
   {
     nodes->Put (pos, item);
-    if (nodes->Length() != 0)
+    if (nodes->GetSize () != 0)
       flags |= BD_NODE_HAS_CHILDREN;
   }
 }
@@ -763,7 +757,7 @@ void csBdNode::ctInsertBefore (csBdNode* item, int pos)
   if (flags & BD_NODE_MODIFIED)
   {
     nodes->Insert (pos, item);
-    if (nodes->Length() != 0)
+    if (nodes->GetSize () != 0)
       flags |= BD_NODE_HAS_CHILDREN;
   }
 }
@@ -773,7 +767,7 @@ void csBdNode::ctRemove (int pos)
   if (flags & BD_NODE_MODIFIED)
   {
     nodes->DeleteIndex (pos);
-    if (nodes->Length() == 0)
+    if (nodes->GetSize () == 0)
       flags &= ~BD_NODE_HAS_CHILDREN;
   }
 }
@@ -782,11 +776,11 @@ uint csBdNode::ctNum ()
 {
   if (flags & BD_NODE_MODIFIED)
   {
-    return (uint)nodes->Length();
+    return (uint)nodes->GetSize ();
   }
   else
   {
-    return GetChildTab()->num; 
+    return csLittleEndian::UInt32 (GetChildTab()->num); 
   }
 }
 
@@ -794,31 +788,22 @@ uint csBdNode::ctNum ()
 //  csBinaryDocNode
 // =================================================
 
-void csBinaryDocNode::IncRef ()
-{
-  scfRefCount++;
-}
-
 void csBinaryDocNode::DecRef ()
 {
+  csRefTrackerAccess::TrackDecRef (scfObject, scfRefCount);
   if (scfRefCount == 1)
   {
+    // Not needed atm
+    //scfRemoveRefOwners ();
+    //if (scfParent) scfParent->DecRef();
     doc->RecyclePoolNode (this);
     return;
   }
   scfRefCount--;
 }
 
-SCF_IMPLEMENT_IBASE_GETREFCOUNT(csBinaryDocNode)
-SCF_IMPLEMENT_IBASE_REFOWNER(csBinaryDocNode)
-SCF_IMPLEMENT_IBASE_REMOVE_REF_OWNERS(csBinaryDocNode)
-SCF_IMPLEMENT_IBASE_QUERY(csBinaryDocNode)
-  SCF_IMPLEMENTS_INTERFACE(iDocumentNode)
-SCF_IMPLEMENT_IBASE_END
-
-csBinaryDocNode::csBinaryDocNode ()
+csBinaryDocNode::csBinaryDocNode () : scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   vstr = 0;
   vsptr = 0;
 }
@@ -827,7 +812,6 @@ csBinaryDocNode::~csBinaryDocNode ()
 {
   CleanData();
   delete vstr;
-  SCF_DESTRUCT_IBASE();
 }
 
 void csBinaryDocNode::SetTo (csBdNode* ptr,
@@ -884,7 +868,7 @@ const char* csBinaryDocNode::nodeValueStr (csBdNode* nodeData)
 	{
   	  char buf[50];
 	  cs_snprintf (buf, sizeof (buf) - 1, "%" PRId32, 
-	    (int32)csLittleEndianLong (nodeData->value));
+	    (int32)csLittleEndian::UInt32 (nodeData->value));
 	  delete[] vstr; 
 	  vstr = csStrNew (buf);
 	  vsptr = nodeData;
@@ -897,7 +881,7 @@ const char* csBinaryDocNode::nodeValueStr (csBdNode* nodeData)
 	{
   	  char buf[50];
 	  cs_snprintf (buf, sizeof (buf) - 1, "%g", 
-	    csLongToFloat (csLittleEndianLong (nodeData->value)));
+	    csLongToFloat (csLittleEndian::UInt32 (nodeData->value)));
 	  delete[] vstr; 
 	  vstr = csStrNew (buf);
 	  vsptr = nodeData;
@@ -923,11 +907,11 @@ int csBinaryDocNode::nodeValueInt (csBdNode* nodeData)
       }
     case BD_VALUE_TYPE_INT:
       {
-	return (int32)csLittleEndianLong (nodeData->value);
+	return (int32)csLittleEndian::UInt32 (nodeData->value);
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return (int)csLongToFloat (csLittleEndianLong (nodeData->value));
+	return (int)csLongToFloat (csLittleEndian::UInt32 (nodeData->value));
       }
     default:
       return 0;
@@ -948,11 +932,11 @@ float csBinaryDocNode::nodeValueFloat (csBdNode* nodeData)
       }
     case BD_VALUE_TYPE_INT:
       {
-	return (float)((int32)csLittleEndianLong (nodeData->value));
+	return (float)((int32)csLittleEndian::UInt32 (nodeData->value));
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return csLongToFloat (csLittleEndianLong (nodeData->value));
+	return csLongToFloat (csLittleEndian::UInt32 (nodeData->value));
       }
     default:
       return 0.0f;
@@ -1074,13 +1058,13 @@ void csBinaryDocNode::SetValue (const char* value)
     {
       nodeData->flags = (nodeData->flags & ~BD_VALUE_TYPE_MASK) | 
 	BD_VALUE_TYPE_INT;
-      nodeData->value = csLittleEndianLong (v);
+      nodeData->value = csLittleEndian::UInt32 (v);
     }
     else if (checkFloat (value, f))
     {
       nodeData->flags = (nodeData->flags & ~BD_VALUE_TYPE_MASK) | 
 	BD_VALUE_TYPE_FLOAT;
-      nodeData->value = csLittleEndianLong (csFloatToLong (f));
+      nodeData->value = csLittleEndian::UInt32 (csFloatToLong (f));
     }
     else 
     {
@@ -1098,7 +1082,7 @@ void csBinaryDocNode::SetValueAsInt (int value)
     delete[] vstr; vstr = 0;
     nodeData->flags = (nodeData->flags & ~BD_VALUE_TYPE_MASK) |
       BD_VALUE_TYPE_INT;
-    nodeData->value = csLittleEndianLong ((int32)value);
+    nodeData->value = csLittleEndian::UInt32 ((int32)value);
   }
 }
 
@@ -1109,7 +1093,7 @@ void csBinaryDocNode::SetValueAsFloat (float value)
     delete[] vstr; vstr = 0;
     nodeData->flags = (nodeData->flags & ~BD_VALUE_TYPE_MASK) | 
       BD_VALUE_TYPE_FLOAT;
-    nodeData->value = csLittleEndianLong (csFloatToLong (value));
+    nodeData->value = csLittleEndian::UInt32 (csFloatToLong (value));
   }
 }
 
@@ -1477,7 +1461,7 @@ void csBinaryDocNode::Store (csMemFile* nodesFile)
     }
     else
     {
-      diskNode.value = csLittleEndianLong (
+      diskNode.value = csLittleEndian::UInt32 (
 	doc->GetOutStringID (nodeData->vstr));
     }
   }
@@ -1528,7 +1512,7 @@ void csBinaryDocNode::Store (csMemFile* nodesFile)
     csRef<csBinaryDocAttribute> attr;
     for (i = 0; i < attrCount; i++)
     {
-      startsScratch[i] = csLittleEndianLong (
+      startsScratch[i] = csLittleEndian::UInt32 (
 	(uint32)(nodesFile->GetPos() - attrStart));
       attr.AttachNew (doc->GetPoolAttr ());
       attr->SetTo (nodeData->atGetItem (i), this);
@@ -1546,7 +1530,7 @@ void csBinaryDocNode::Store (csMemFile* nodesFile)
     csRef<csBinaryDocNode> node;
     for (i = 0; i < childCount; i++)
     {
-      startsScratch[i] = csLittleEndianLong (
+      startsScratch[i] = csLittleEndian::UInt32 (
 	(uint32)(nodesFile->GetPos() - childStart));
       node.AttachNew (doc->GetPoolNode());
       node->SetTo (nodeData->ctGetItem (i), this);
@@ -1571,19 +1555,10 @@ void csBinaryDocNode::Store (csMemFile* nodesFile)
 //  csBinaryDocument
 // =================================================
 
-SCF_IMPLEMENT_IBASE(csBinaryDocument)
-  SCF_IMPLEMENTS_INTERFACE(iDocument)
-SCF_IMPLEMENT_IBASE_END
-
-csBinaryDocument::csBinaryDocument ()
+csBinaryDocument::csBinaryDocument () : scfImplementationType (this),
+  root (0), nodePool (0), attrPool (0), attrAlloc (0), nodeAlloc (0),
+  outStrHash (0)
 {
-  SCF_CONSTRUCT_IBASE (0);
-  nodePool = 0;
-  attrPool = 0;
-  root = 0;
-  outStrHash = 0;
-  attrAlloc = 0;
-  nodeAlloc = 0;
 }
 
 csBinaryDocument::~csBinaryDocument ()
@@ -1604,7 +1579,6 @@ csBinaryDocument::~csBinaryDocument ()
   }
   delete attrAlloc;
   delete nodeAlloc;
-  SCF_DESTRUCT_IBASE();
 }
 
 csBdAttr* csBinaryDocument::AllocBdAttr ()
@@ -1774,7 +1748,7 @@ const char* csBinaryDocument::Parse (iDataBuffer* buf, bool /* collapse */)
   {
     return "Not a binary CS document";
   }
-  if (head->size != buf->GetSize())
+  if (csLittleEndian::UInt32 (head->size) != buf->GetSize())
   {
     return "Size mismatch";
   }
@@ -1789,9 +1763,10 @@ const char* csBinaryDocument::Parse (iDataBuffer* buf, bool /* collapse */)
   data = buf;
   dataStart = data->GetUint8();
 
-  inStrTabOfs =  sizeof(bdHeader) + bdDoc->ofsStr;
+  inStrTabOfs =  sizeof(bdHeader) + csLittleEndian::UInt32 (bdDoc->ofsStr);
 
-  root = (csBdNode*)((uint8*)bdDoc + bdDoc->ofsRoot);
+  root = (csBdNode*)((uint8*)bdDoc + 
+    csLittleEndian::UInt32 (bdDoc->ofsRoot));
 
   return 0;
 }
@@ -1799,9 +1774,9 @@ const char* csBinaryDocument::Parse (iDataBuffer* buf, bool /* collapse */)
 const char* csBinaryDocument::Parse (iString* str, bool collapse)
 {
   csRef<csDataBuffer> newBuffer = csPtr<csDataBuffer>
-    (new csDataBuffer(str->Length()));
+    (new csDataBuffer(str->Length ()));
   memcpy (newBuffer->GetData(), str->GetData(),
-    str->Length());
+    str->Length ());
   return Parse (newBuffer, collapse);
 }
 
@@ -1832,13 +1807,13 @@ const char* csBinaryDocument::Write (iFile* out)
     if (pad != 0)
     {
       // align to 4 byte boundary, to avoid problems
-      char null[4] = {0, 0, 0, 0};
+      static const char null[4] = {0, 0, 0, 0};
       out->Write (null, pad);
       doc.ofsStr += pad;
     }
     doc.ofsStr -= (uint32)docStart;
   }
-  doc.ofsStr = csLittleEndianLong (doc.ofsStr);
+  doc.ofsStr = csLittleEndian::UInt32 (doc.ofsStr);
   outStrTabOfs = (uint32)out->GetPos();
 
   csMemFile* outNodes = new csMemFile;
@@ -1861,17 +1836,17 @@ const char* csBinaryDocument::Write (iFile* out)
     if (pad != 0)
     {
       // align to 4 byte boundary, to avoid problems
-      char null[4] = {0, 0, 0, 0};
+      static const char null[4] = {0, 0, 0, 0};
       out->Write (null, pad);
       doc.ofsRoot += pad;
     }
     doc.ofsRoot -= (uint32)docStart;
   }
-  doc.ofsRoot = csLittleEndianLong (doc.ofsRoot);
+  doc.ofsRoot = csLittleEndian::UInt32 (doc.ofsRoot);
   out->Write (outNodes->GetData(), outNodes->GetSize());
   delete outNodes;
 
-  head.size = (uint32)out->GetSize();
+  head.size = csLittleEndian::UInt32 ((uint32)out->GetSize());
   out->SetPos (0);
   out->Write ((char*)&head, sizeof (head));
   out->Write ((char*)&doc, sizeof (doc));
@@ -1884,7 +1859,7 @@ const char* csBinaryDocument::Write (iString* str)
   csMemFile temp;
 
   const char* ret = Write (&temp);
-  str->Clear();
+  str->Empty();
   str->Append (temp.GetData(), temp.GetSize());
 
   return ret;
@@ -1911,3 +1886,6 @@ int csBinaryDocument::Changeable ()
     return CS_CHANGEABLE_NEWROOT;
   }
 }
+
+}
+CS_PLUGIN_NAMESPACE_END(BinDoc)
