@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1998-2001 by Jorrit Tyberghein
+    Copyright (C) 1998-2001,2007 by Jorrit Tyberghein
               (C) 2005 by Marten Svanfeldt
 
     This library is free software; you can redistribute it and/or
@@ -24,6 +24,7 @@
 #include "csutil/array.h"
 #include "csutil/csobject.h"
 #include "csutil/hash.h"
+#include "csutil/set.h"
 #include "csutil/nobjvec.h"
 #include "csutil/parray.h"
 #include "csutil/refarr.h"
@@ -31,9 +32,9 @@
 #include "csutil/scf_implementation.h"
 #include "csutil/stringarray.h"
 #include "csutil/weakref.h"
+#include "csutil/weakrefarr.h"
 #include "csutil/eventnames.h"
 #include "iengine/campos.h"
-#include "iengine/collectn.h"
 #include "iengine/engine.h"
 #include "iengine/renderloop.h"
 #include "igraphic/imageio.h"
@@ -53,6 +54,7 @@
 #include "ivideo/shader/shader.h"
 #include "plugins/engine/3d/halo.h"
 #include "plugins/engine/3d/meshobj.h"
+#include "plugins/engine/3d/meshfact.h"
 #include "plugins/engine/3d/region.h"
 #include "plugins/engine/3d/renderloop.h"
 #include "plugins/engine/3d/rview.h"
@@ -68,11 +70,11 @@ class csPolygon3D;
 class csRegion;
 class csRenderView;
 class csSector;
-class csSectorIt;
 class csSectorList;
 class csTextureList;
 struct iClipper2D;
 struct iConfigFile;
+struct iDocumentSystem;
 struct iLight;
 struct iMaterialWrapper;
 struct iObjectRegistry;
@@ -124,30 +126,18 @@ private:
   iLight* FetchNext ();
 };
 
-/**
- * List of collections for the engine. This class implements iCollectionList.
- */
-class csCollectionList : public scfImplementation1<csCollectionList,
-                                                   iCollectionList>
+struct csSectorPos
 {
-public:
-  /// constructor
-  csCollectionList ();
-  virtual ~csCollectionList ();
+  iSector* sector;
+  csVector3 pos;
+  csSectorPos (iSector* sector, const csVector3& pos) :
+  	sector (sector), pos (pos) { }
+};
 
-  //-- iCollectionList
-  virtual iCollection* NewCollection (const char* name);
-  virtual int GetCount () const;
-  virtual iCollection *Get (int n) const;
-  virtual int Add (iCollection *obj);
-  virtual bool Remove (iCollection *obj);
-  virtual bool Remove (int n);
-  virtual void RemoveAll ();
-  virtual int Find (iCollection *obj) const;
-  virtual iCollection *FindByName (const char *Name) const;
-
-private:
-  csRefArrayObject<iCollection> collections;
+struct csDelayedRemoveObject
+{
+  csRef<iBase> object;
+  csTicks time_to_delete;
 };
 
 /**
@@ -187,23 +177,26 @@ public:
 };
 
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
+struct csImposterUpdateQueue
+{
+  csRef<iRenderView> rview;
+  csWeakRefArray<csImposterProcTex> queue;
+};
+
 /**
  * The 3D engine.
  * This class manages all components which comprise a 3D world including
  * sectors, polygons, curves, mesh objects, etc.
  */
-class csEngine : public scfImplementationExt4<csEngine,
-                                              csObject,
-                                              iEngine,
-                                              iComponent,
-                                              iPluginConfig,
-                                              iDebugHelper>
+class csEngine : public scfImplementationExt5<csEngine, csObject,
+  iEngine, iComponent, iPluginConfig, iDebugHelper, iEventHandler>
 {
   // friends
   friend class csLight;
   friend class csLightIt;
   friend class csRenderLoop;
-  friend class csSectorIt;
   friend class csSectorList;
 
 public:
@@ -366,6 +359,8 @@ public:
   	iRegion* region = 0);
   virtual csPtr<iSectorIterator> GetNearbySectors (iSector* sector,
   	const csVector3& pos, float radius);
+  virtual csPtr<iSectorIterator> GetNearbySectors (iSector* sector,
+  	const csBox3& box);
 
   virtual void AddEngineSectorCallback (iEngineSectorCallback* cb);
   virtual void RemoveEngineSectorCallback (iEngineSectorCallback* cb);
@@ -402,9 +397,10 @@ public:
 
   virtual csPtr<iMeshWrapperIterator> GetNearbyMeshes (iSector* sector,
     const csVector3& pos, float radius, bool crossPortals = true );
-
   virtual csPtr<iMeshWrapperIterator> GetNearbyMeshes (iSector* sector,
     const csBox3& box, bool crossPortals = true );
+  virtual csPtr<iMeshWrapperIterator> GetNearbyMeshes (iSector* sector,
+    const csVector3& start, const csVector3& end, bool crossPortals = true );
 
   virtual iMeshList* GetMeshes ()
   { return &meshes; }
@@ -497,7 +493,7 @@ public:
   { return (iRenderView*)topLevelClipper; }
 
   virtual void PrecacheDraw (iRegion* region = 0);
-  virtual void Draw (iCamera* c, iClipper2D* clipper);
+  virtual void Draw (iCamera* c, iClipper2D* clipper, iMeshWrapper* mesh = 0);
 
   virtual void SetContext (iTextureHandle* ctxt);
   virtual iTextureHandle *GetContext () const;
@@ -543,15 +539,12 @@ public:
 
   virtual iSharedVariableList* GetVariableList () const;
 
-  virtual iCollectionList* GetCollections ()
-  { return &collections; }
-  
-  virtual iCollection* FindCollection (const char* name,
-  	iRegion* region = 0);
-
   virtual bool RemoveObject (iBase* object);
+  virtual void DelayedRemoveObject (csTicks delay, iBase *object);
+  virtual void RemoveDelayedRemoves (bool remove = false);
 
   virtual void DeleteAll ();
+  void DeleteAllForce ();
 
   virtual void ResetWorldSpecificSettings(); 
 
@@ -561,28 +554,11 @@ public:
   virtual bool SetOption (int id, csVariant* value);
   virtual bool GetOption (int id, csVariant* value);
 
+  // -- iEventHandler
   bool HandleEvent (iEvent &Event);
 
-  // -- iEventHandler
-  struct eiEventHandler : public scfImplementation1<
-  	eiEventHandler,iEventHandler>
-  {
-    csWeakRef<csEngine> parent;
-    eiEventHandler (csEngine* parent) : scfImplementationType (this)
-    {
-      eiEventHandler::parent = parent;
-    }
-    virtual ~eiEventHandler ()
-    {
-    }
-    virtual bool HandleEvent (iEvent& ev)
-    {
-      if (parent) return parent->HandleEvent (ev);
-      else return false;
-    }
-    CS_EVENTHANDLER_NAMES("crystalspace.engine.3d")
-    CS_EVENTHANDLER_NIL_CONSTRAINTS
-  } * scfiEventHandler;
+  CS_EVENTHANDLER_NAMES("crystalspace.engine.3d")
+  CS_EVENTHANDLER_NIL_CONSTRAINTS
 
   // -- iDebugHelper
   
@@ -656,6 +632,17 @@ public:
     return renderLoopManager;
   }
 
+  /**
+   * Add an imposter to the update queue.
+   */
+  void AddImposterToUpdateQueue (csImposterProcTex* imptex,
+      iRenderView* rview);
+
+  /**
+   * Handle imposters.
+   */
+  void HandleImposters ();
+
 private:
   // -- PRIVATE METHODS
 
@@ -668,15 +655,10 @@ private:
    */
   bool CheckConsistency ();
 
-  // Sector iteration functions
-  csSectorIt* AllocSectorIterator (iSector *sector, const csVector3 &pos, 
-    float radius);
-  void RecycleSectorIterator (csSectorIt* iterator);
-  void FreeSectorIteratorPool ();
-
   // Renderloop loading/creation
   csPtr<iRenderLoop> CreateDefaultRenderLoop ();
   void LoadDefaultRenderLoop (const char* fileName);
+  csRef<iShader> LoadShader (iDocumentSystem* docsys, const char* filename);
 
   /**
    * Setup for starting a Draw or DrawFunc.
@@ -688,26 +670,24 @@ private:
    */
   void ControlMeshes ();
 
-  /**
-   * Get a list of all objects in the given sphere.
-   */
+  void GetNearbySectorList (iSector* sector,
+    const csVector3& pos, float radius, csArray<csSectorPos>& list,
+    csSet<csPtrKey<iSector> >& visited_sectors);
+  void GetNearbySectorList (iSector* sector,
+    const csBox3& box, csArray<csSectorPos>& list,
+    csSet<csPtrKey<iSector> >& visited_sectors);
   void GetNearbyObjectList (iSector* sector,
     const csVector3& pos, float radius, csArray<iObject*>& list,
-    csArray<iSector*>& visited_sectors, bool crossPortals = true);
-
-  /**
-   * Get a list of all meshes in the given box.
-   */
+    csSet<csPtrKey<iSector> >& visited_sectors, bool crossPortals = true);
   void GetNearbyMeshList (iSector* sector,
     const csBox3& box, csArray<iMeshWrapper*>& list,
-    csArray<iSector*>& visited_sectors, bool crossPortals = true);
-
-  /**
-   * Get a list of all meshes in the given sphere.
-   */
+    csSet<csPtrKey<iSector> >& visited_sectors, bool crossPortals = true);
   void GetNearbyMeshList (iSector* sector,
     const csVector3& pos, float radius, csArray<iMeshWrapper*>& list,
-    csArray<iSector*>& visited_sectors, bool crossPortals = true);
+    csSet<csPtrKey<iSector> >& visited_sectors, bool crossPortals = true);
+  void GetNearbyMeshList (iSector* sector,
+    const csVector3& start, const csVector3& end, csArray<iMeshWrapper*>& list,
+    csSet<csPtrKey<iSector> >& visited_sectors, bool crossPortals = true);
 
   /**
    * Read configuration file (using the system driver) for all engine
@@ -768,18 +748,24 @@ public:
   /// For debugging purposes.
   csRef<iBugPlug> bugplug;
 
-  /// The shared engine instance.
-  static csEngine* currentEngine;
-  /// The shared engine instance.
-  static iEngine* currentiEngine;
   /// Remember iObjectRegistry.
-  static iObjectRegistry* objectRegistry;
+  iObjectRegistry* objectRegistry;
   /// The global material/shader string set
   csRef<iStringSet> globalStringSet;
   /// The 3D driver
   csRef<iGraphics3D> G3D;
   /// Pointer to the shader manager
   csRef<iShaderManager> shaderManager;
+  
+  /// Store virtual clock to speed up time queries.
+  csRef<iVirtualClock> virtualClock;
+
+  /// Store engine shadervar names
+  csStringID id_creation_time;
+  /// For triangle meshes.
+  csStringID colldet_id;
+  csStringID viscull_id;
+  csStringID base_id;
   /**
    * This is the Virtual File System object where all the files
    * used by the engine live. Textures, models, data, everything -
@@ -791,12 +777,32 @@ public:
   csRef<iVFS> VFS;
 
   /// Remember dimensions of display.
-  static int frameWidth, frameHeight;
+  int frameWidth, frameHeight;
+
+  /** 
+   * Config value: light ambient red value.
+   */
+  int lightAmbientRed;
+  /**
+   * Config value: light ambient green value.
+   */
+  int lightAmbientGreen;
+  /** 
+   * Config value: light ambient blue value.
+   */
+  int lightAmbientBlue;
+
+  /// Default shader to attach to all materials
+  // \todo move back to private and make accessible
+  csRef<iShader> defaultShader;
 
 private:
 
   // -- PRIVATE MEMBERS
 
+  /// Pool from which to allocate render views.
+  csRenderView::Pool rviewPool;
+
   // -- Object lists
   /**
    * List of sectors in the engine. This vector contains
@@ -805,12 +811,6 @@ private:
    */
   csSectorList sectors;
 
-  /**
-   * List of all collections in the engine. This vector contains objects
-   * of type iCollection*.
-   */
-  csCollectionList collections;
-
   /**
    * List of mesh object factories. This vector contains objects of
    * type csMeshFactoryWrapper*.
@@ -862,8 +862,8 @@ private:
   /// Array of objects that want to die next frame (iMeshWrapper*).
   csSet<csPtrKey<iMeshWrapper> > wantToDieSet;
 
-  // Sector iterator pool
-  csSectorIt* sectorItPool; //TOOD: REWORK THIS! UGLY
+  /// An array of objects to remove at a specific time.
+  csArray<csDelayedRemoveObject> delayedRemoves;
 
   /// The list of all named render priorities.
   csStringArray renderPriorities;
@@ -878,11 +878,9 @@ private:
    */
   csRef<iReporter> reporter;
 
-  /// Store virtual clock to speed up time queries.
-  csRef<iVirtualClock> virtualClock;
-
   /// Default render loop
   csRef<iRenderLoop> defaultRenderLoop;
+  csString override_renderloop;
   /// Render loop manager
   csRenderLoopManager* renderLoopManager;
   /// The graphics loader
@@ -906,9 +904,6 @@ private:
   /// Flag set when window requires resizing.
   bool resize;
 
-  /// Default shader to attach to all materials
-  csRef<iShader> defaultShader;
-
   /// 'Saveable' flag
   bool worldSaveable;
 
@@ -949,10 +944,10 @@ private:
   uint currentFrameNumber;
 
     /// Option variable: force lightmap recalculation?
-  static int lightmapCacheMode;
+  int lightmapCacheMode;
   /// Maximum lightmap dimensions
-  static int maxLightmapWidth;
-  static int maxLightmapHeight;
+  int maxLightmapWidth;
+  int maxLightmapHeight;
 
   /// Clear the Z-buffer every frame.
   bool clearZBuf;
@@ -981,9 +976,18 @@ private:
   /// Current render context (proc texture) or 0 if global.
   iTextureHandle* currentRenderContext; 
 
+  /**
+   * List of imposters that need to be rendered to texture.
+   * There is a different list for every distinct camera instance.
+   */
+  csHash<csImposterUpdateQueue,long> imposterUpdateQueue;
+
   CS_DECLARE_SYSTEM_EVENT_SHORTCUTS;
   csEventID CanvasResize;
   csEventID CanvasClose;
+  csRef<iEventHandler> weakEventHandler;
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 #endif // __CS_ENGINE_H__
