@@ -90,7 +90,7 @@ uint8 *csNovaHalo::Generate (int Size)
 
 
 csFlareHalo::csFlareHalo () 
-  : scfImplementationType (this, cshtFlare), components (0), last (0)
+: scfImplementationType (this, cshtFlare), components (0), last (0), cmp_cnt (0)
 {
 }
 
@@ -105,8 +105,17 @@ csFlareHalo::~csFlareHalo ()
     p = np;
   }
 }
+void csFlareHalo::SetComponentColor (uint component, const csVector4& color)
+{
+  csFlareComponent *pcomp = components;
+  for (size_t i = 0; i < cmp_cnt && i < component; i++)
+  {
+    pcomp = pcomp->next;
+  }
+  pcomp->color = color;
+}
 
-void csFlareHalo::AddComponent (
+uint csFlareHalo::AddComponent (
   float pos,
   float w,
   float h,
@@ -121,12 +130,14 @@ void csFlareHalo::AddComponent (
     last->next = comp;
   last = comp;
 
+  comp->color = csVector4 (1,1,1,1);
   comp->position = pos;
   comp->width = w;
   comp->height = h;
   comp->mixmode = mode;
   comp->image = image;
   if (comp->image) comp->image->IncRef ();
+  return cmp_cnt++;
 }
 
 uint8 *csFlareHalo::Generate (int /*Size*/ )
@@ -157,17 +168,17 @@ bool csLightHalo::IsVisible (iCamera* camera, csEngine* Engine, csVector3 &v)
     v.x = v.x * iz + camera->GetShiftX ();
     v.y = Engine->frameHeight - 1 - (v.y * iz + camera->GetShiftY ());
 
-    if (Engine->GetTopLevelClipper ()->GetClipper ()->IsInside (csVector2 (v.x, v.y)))
+    if (Engine->GetTopLevelClipper ()->GetClipper ()->IsInside (
+    	csVector2 (v.x, v.y)))
     {
-      csVector3 isect;
-      int polyidx = 0;
-      if(camera->GetSector ()->HitBeamPortals (
-	camera->GetTransform().GetOrigin(),
-	Light->GetFullCenter(), isect, &polyidx))
-	  return false; // hit a mesh
-      if(polyidx != -1) // double check on the above if
-	return false; // hit a polygon
-      return true;
+        csSectorHitBeamResult hbresult;
+        hbresult = camera->GetSector ()->HitBeamPortals (
+	    camera->GetTransform().GetOrigin(),	Light->GetFullCenter());
+        if(hbresult.mesh)
+            return false; // hit a mesh
+        if(hbresult.polygon_idx != -1) // double check on the above if
+            return false; // hit a polygon
+        return true;
     }
   }
 
@@ -310,7 +321,7 @@ bool csLightFlareHalo::Process (csTicks elapsed_time, iCamera* camera,
   csFlareComponent *p = flare->GetComponents ();
   while (p)
   {
-    ProcessFlareComponent (*engine, p, start, deltapos);
+    ProcessFlareComponent (engine, p, start, deltapos);
     p = p->next;
   }
 
@@ -318,7 +329,7 @@ bool csLightFlareHalo::Process (csTicks elapsed_time, iCamera* camera,
 }
 
 void csLightFlareHalo::ProcessFlareComponent (
-  csEngine const &engine,
+  csEngine* engine,
   csFlareComponent *comp,
   csVector2 const &start,
   csVector2 const &deltapos)
@@ -340,13 +351,13 @@ void csLightFlareHalo::ProcessFlareComponent (
 
   if (!comp->image)
   {
-    csEngine::currentEngine->Warn ("INTERNAL ERROR: flare used without material.");
+    engine->Warn ("INTERNAL ERROR: flare used without material.");
     return ;
   }
   iMaterial* mat = comp->image->GetMaterial ();
   if (!mat)
   {
-    csEngine::currentEngine->Warn ("INTERNAL ERROR: flare used without valid material.");
+    engine->Warn ("INTERNAL ERROR: flare used without valid material.");
     return ;
   }
 
@@ -363,10 +374,12 @@ void csLightFlareHalo::ProcessFlareComponent (
   }
 
   static uint indices[4] = {0, 1, 2, 3};
-  csVector4 colors[4] = {csVector4 (intensity), csVector4 (intensity), 
-    csVector4 (intensity), csVector4 (intensity)};
+  csVector4 c = comp->color;
+  c *= intensity;
+  csVector4 colors[4] = {c, c, 
+    c, c};
 
-  pos.y = ((float)engine.G3D->GetHeight()) - pos.y;
+  pos.y = ((float)engine->G3D->GetHeight()) - pos.y;
 
   // Create a rectangle containing the halo and clip it against screen
   csVector3 HaloPoly[4] =
@@ -387,5 +400,5 @@ void csLightFlareHalo::ProcessFlareComponent (
   mesh.texture = mat->GetTexture();
   mesh.mixmode = ((mode & CS_FX_MASK_MIXMODE) != CS_FX_ALPHA) ? mode : CS_FX_COPY;
 
-  engine.G3D->DrawSimpleMesh (mesh, csSimpleMeshScreenspace);
+  engine->G3D->DrawSimpleMesh (mesh, csSimpleMeshScreenspace);
 }
