@@ -20,21 +20,17 @@
 #include "csqint.h"
 
 #include "csgeom/frustum.h"
-#include "csgeom/polymesh.h"
+#include "csgeom/trimesh.h"
 #include "csutil/csendian.h"
 #include "csutil/csmd5.h"
-#include "csutil/debug.h"
 #include "csutil/memfile.h"
-#include "plugins/engine/3d/engine.h"
-#include "plugins/engine/3d/halo.h"
-#include "plugins/engine/3d/light.h"
-#include "plugins/engine/3d/meshobj.h"
-#include "plugins/engine/3d/sector.h"
-#include "plugins/engine/3d/portal.h"
 
-int csLight::ambient_red = CS_DEFAULT_LIGHT_LEVEL;
-int csLight::ambient_green = CS_DEFAULT_LIGHT_LEVEL;
-int csLight::ambient_blue = CS_DEFAULT_LIGHT_LEVEL;
+#include "engine.h"
+#include "halo.h"
+#include "light.h"
+#include "meshobj.h"
+#include "sector.h"
+#include "portal.h"
 
 //float csLight::influenceIntensityFraction = 256;
 #define HUGE_RADIUS 100000000
@@ -51,7 +47,7 @@ void csLight::UpdateViscullMesh ()
   // in case of a spot light. The geometry specified here should be at
   // least as big as that shape (for example, a box in case of a point light
   // would be fine).
-  csRef<iPolygonMesh> m;
+  csRef<iTriangleMesh> m;
   switch (type)
   {
     case CS_LIGHT_POINTLIGHT:
@@ -59,9 +55,8 @@ void csLight::UpdateViscullMesh ()
         object_model->box.Set (
 		-cutoffDistance, -cutoffDistance, -cutoffDistance,
 		cutoffDistance, cutoffDistance, cutoffDistance);
-        m.AttachNew (new csPolygonMeshBox (object_model->box));
-	object_model->radius.Set (
-			cutoffDistance, cutoffDistance, cutoffDistance);
+        m.AttachNew (new csTriangleMeshBox (object_model->box));
+	object_model->radius = cutoffDistance;
       }
       break;
     case CS_LIGHT_DIRECTIONAL:
@@ -71,21 +66,21 @@ void csLight::UpdateViscullMesh ()
       // @@@ TODO
       break;
   }
-  object_model->SetPolygonMeshViscull (m);
+  object_model->SetTriangleData (engine->viscull_id, m);
   object_model->ShapeChanged ();
 }
 
-csLight::csLight (
+csLight::csLight (csEngine* engine,
   float x, float y, float z,
   float d,
   float red, float green, float blue,
   csLightDynamicType dyntype) :
     scfImplementationType (this), light_id (0), color (red, green, blue),
-    specularColor (0,0,0),
+    specularColor (red, green, blue), userSpecular (false),
     halo (0), dynamicType (dyntype), type (CS_LIGHT_POINTLIGHT), 
     attenuation (CS_ATTN_LINEAR), cutoffDistance (d),
     directionalCutoffRadius (d), spotlightFalloffInner (0),
-    spotlightFalloffOuter (1), lightnr (0)
+    spotlightFalloffOuter (1), lightnr (0), engine (engine)
 {
   //movable.scfParent = (iBase*)(csObject*)this; //@@MS: Look at this?
   movable.SetLight (this);
@@ -106,12 +101,12 @@ csLight::~csLight ()
   // Copy the array because we are going to unlink the children.
   csRefArray<iSceneNode> children = movable.GetChildren ();
   size_t j;
-  for (j = 0 ; j < children.Length () ; j++)
+  for (j = 0 ; j < children.GetSize () ; j++)
     children[j]->SetParent (0);
 
   CleanupLSI ();
 
-  int i = (int)light_cb_vector.Length ()-1;
+  int i = (int)light_cb_vector.GetSize ()-1;
   while (i >= 0)
   {
     iLightCallback* cb = light_cb_vector[i];
@@ -120,7 +115,7 @@ csLight::~csLight ()
   }
 
   if (flags.Check (CS_LIGHT_ACTIVEHALO))
-    csEngine::currentEngine->RemoveHalo (this);
+    engine->RemoveHalo (this);
   delete halo;
   delete[] light_id;
 
@@ -192,7 +187,7 @@ void csLight::FindLSI ()
 {
   CleanupLSI ();
 
-  iSector* sector = GetSector ();
+  iSector* sector = GetFullSector ();
   if (!sector) return;
   const csVector3 center = GetFullCenter ();
 
@@ -300,9 +295,9 @@ const char* csLight::GenerateUniqueID ()
 
   mf.Write ("light", 5);
 
-  l = csConvertEndian ((int32)type);
+  l = csLittleEndian::Convert ((int32)type);
   mf.Write ((char*)&l, 4);
-  l = csConvertEndian ((int32)dynamicType);
+  l = csLittleEndian::Convert ((int32)dynamicType);
   mf.Write ((char*)&l, 4);
 
   iSector* sector = GetSector ();
@@ -314,23 +309,23 @@ const char* csLight::GenerateUniqueID ()
   }
 
   csVector3 center = GetCenter ();
-  l = csConvertEndian ((int32)csQint ((center.x * 1000)+.5));
+  l = csLittleEndian::Convert ((int32)csQint ((center.x * 1000)+.5));
   mf.Write ((char*)&l, 4);
-  l = csConvertEndian ((int32)csQint ((center.y * 1000)+.5));
+  l = csLittleEndian::Convert ((int32)csQint ((center.y * 1000)+.5));
   mf.Write ((char*)&l, 4);
-  l = csConvertEndian ((int32)csQint ((center.z * 1000)+.5));
+  l = csLittleEndian::Convert ((int32)csQint ((center.z * 1000)+.5));
   mf.Write ((char*)&l, 4);
 
-  l = csConvertEndian ((int32)csQint ((cutoffDistance * 1000)+.5));
+  l = csLittleEndian::Convert ((int32)csQint ((cutoffDistance * 1000)+.5));
   mf.Write ((char*)&l, 4);
 
-  l = csConvertEndian ((int32)attenuation);
+  l = csLittleEndian::Convert ((int32)attenuation);
   mf.Write ((char*)&l, 4);
-  l = csConvertEndian ((int32)csQint ((attenuationConstants.x * 1000)+.5));
+  l = csLittleEndian::Convert ((int32)csQint ((attenuationConstants.x * 1000)+.5));
   mf.Write ((char*)&l, 4);
-  l = csConvertEndian ((int32)csQint ((attenuationConstants.y * 1000)+.5));
+  l = csLittleEndian::Convert ((int32)csQint ((attenuationConstants.y * 1000)+.5));
   mf.Write ((char*)&l, 4);
-  l = csConvertEndian ((int32)csQint ((attenuationConstants.z * 1000)+.5));
+  l = csLittleEndian::Convert ((int32)csQint ((attenuationConstants.z * 1000)+.5));
   mf.Write ((char*)&l, 4);
 
   csMD5::Digest digest = csMD5::Encode (mf.GetData (), mf.GetSize ());
@@ -426,7 +421,7 @@ void csLight::OnSetPosition ()
 {
   FindLSI ();
   csVector3 pos = GetFullCenter ();
-  size_t i = light_cb_vector.Length ();
+  size_t i = light_cb_vector.GetSize ();
   while (i-- > 0)
   {
     iLightCallback* cb = light_cb_vector[i];
@@ -438,7 +433,7 @@ void csLight::OnSetPosition ()
 
 void csLight::OnSetSector (iSector *sector)
 {
-  size_t i = light_cb_vector.Length ();
+  size_t i = light_cb_vector.GetSize ();
   while (i-- > 0)
   {
     iLightCallback* cb = light_cb_vector[i];
@@ -450,7 +445,7 @@ void csLight::OnSetSector (iSector *sector)
 
 void csLight::SetColor (const csColor& col) 
 {
-  size_t i = light_cb_vector.Length ();
+  size_t i = light_cb_vector.GetSize ();
   while (i-- > 0)
   {
     iLightCallback* cb = light_cb_vector[i];
@@ -458,6 +453,7 @@ void csLight::SetColor (const csColor& col)
   }
 
   color = col; 
+  if (!userSpecular) specularColor = col;
   lightnr++;
 
   LightingInfo::GlobalIterator it(lightinginfos.GetIterator());
@@ -486,7 +482,7 @@ void csLight::SetAttenuationMode (csLightAttenuationMode a)
   attenuation = a;
   CalculateAttenuationVector();
 
-  size_t i = light_cb_vector.Length ();
+  size_t i = light_cb_vector.GetSize ();
   while (i-- > 0)
   {
     iLightCallback* cb = light_cb_vector[i];
@@ -502,7 +498,7 @@ void csLight::SetAttenuationConstants (const csVector3& attenv)
   influenceValid = false;*/
   attenuationConstants = attenv;
 
-  size_t i = light_cb_vector.Length ();
+  size_t i = light_cb_vector.GetSize ();
   while (i-- > 0)
   {
     iLightCallback* cb = light_cb_vector[i];
@@ -513,7 +509,7 @@ void csLight::SetAttenuationConstants (const csVector3& attenv)
 void csLight::SetCutoffDistance (float radius)
 {
   if (radius <= 0) return;
-  size_t i = light_cb_vector.Length ();
+  size_t i = light_cb_vector.GetSize ();
   while (i-- > 0)
   {
     iLightCallback* cb = light_cb_vector[i];
@@ -521,6 +517,8 @@ void csLight::SetCutoffDistance (float radius)
   }
   lightnr++;
   cutoffDistance = radius;
+
+  CalculateAttenuationVector();
   UpdateViscullMesh ();
 }
 
@@ -529,7 +527,7 @@ iCrossHalo *csLight::CreateCrossHalo (float intensity, float cross)
   csCrossHalo *halo = new csCrossHalo (intensity, cross);
   SetHalo (halo);
 
-  csRef<iCrossHalo> ihalo (SCF_QUERY_INTERFACE (halo, iCrossHalo));
+  csRef<iCrossHalo> ihalo (scfQueryInterface<iCrossHalo> (halo));
   return ihalo; // DecRef is ok here.
 }
 
@@ -541,7 +539,7 @@ iNovaHalo *csLight::CreateNovaHalo (
   csNovaHalo *halo = new csNovaHalo (seed, num_spokes, roundness);
   SetHalo (halo);
 
-  csRef<iNovaHalo> ihalo (SCF_QUERY_INTERFACE (halo, iNovaHalo));
+  csRef<iNovaHalo> ihalo (scfQueryInterface<iNovaHalo> (halo));
   return ihalo; // DecRef is ok here.
 }
 
@@ -550,7 +548,7 @@ iFlareHalo *csLight::CreateFlareHalo ()
   csFlareHalo *halo = new csFlareHalo ();
   SetHalo (halo);
 
-  csRef<iFlareHalo> ihalo (SCF_QUERY_INTERFACE (halo, iFlareHalo));
+  csRef<iFlareHalo> ihalo (scfQueryInterface<iFlareHalo> (halo));
   return ihalo; // DecRef is ok here.
 }
 
@@ -572,6 +570,25 @@ static void object_light_func (iMeshWrapper *mesh, iFrustumView *lview,
   cmw->InvalidateRelevantLights ();
 }
 
+iSector* csLight::GetFullSector ()
+{
+  iSector* s = GetSector ();
+  if (s) return s;
+  iSceneNode* node = (iSceneNode*)this;
+  iSceneNode* parent = node->GetParent ();
+  while (parent)
+  {
+    iSectorList* sl = parent->GetMovable ()->GetSectors ();
+    if (sl && sl->GetCount () > 0)
+    {
+      return sl->Get (0);
+    }
+
+    parent = parent->GetParent ();
+  }
+  return 0;
+}
+
 void csLight::CalculateLighting ()
 {
   csFrustumView lview;
@@ -598,11 +615,13 @@ void csLight::CalculateLighting ()
   ctxt->SetNewLightFrustum (new csFrustum (GetFullCenter ()));
   ctxt->GetLightFrustum ()->MakeInfinite ();
 
+  iSector* sect = GetFullSector ();
+  if (!sect) return;	// Do nothing.
+
   if (dynamicType == CS_LIGHT_DYNAMICTYPE_DYNAMIC)
   {
-    csRef<iMeshWrapperIterator> it = csEngine::currentEngine
-    	->GetNearbyMeshes (GetSector (), GetFullCenter (),
-	  GetCutoffDistance ());
+    csRef<iMeshWrapperIterator> it = engine->GetNearbyMeshes (
+      sect, GetFullCenter (), GetCutoffDistance ());
     while (it->HasNext ())
     {
       iMeshWrapper* m = it->Next ();
@@ -610,14 +629,14 @@ void csLight::CalculateLighting ()
       if (receiver)
       {
         receiver->CastShadows (m->GetMovable (), &lview);
-        csMeshWrapper* cmw = (csMeshWrapper*)m;
-        cmw->InvalidateRelevantLights ();
       }
+      csMeshWrapper* cmw = (csMeshWrapper*)m;
+      cmw->InvalidateRelevantLights ();
     }
   }
   else
   {
-    GetSector ()->CheckFrustum ((iFrustumView *) &lview);
+    sect->CheckFrustum ((iFrustumView *) &lview);
     lpi->FinalizeLighting ();
   }
 }
@@ -661,7 +680,7 @@ csLightList::~csLightList ()
 void csLightList::NameChanged (iObject* object, const char* oldname,
   	const char* newname)
 {
-  csRef<iLight> light = SCF_QUERY_INTERFACE (object, iLight);
+  csRef<iLight> light = scfQueryInterface<iLight> (object);
   CS_ASSERT (light != 0);
   if (oldname) lights_hash.Delete (oldname, light);
   if (newname) lights_hash.Put (newname, light);
@@ -670,7 +689,7 @@ void csLightList::NameChanged (iObject* object, const char* oldname,
 iLight *csLightList::FindByID (const char* id) const
 {
   size_t i;
-  for (i = 0; i < list.Length (); i++)
+  for (i = 0; i < list.GetSize (); i++)
   {
     iLight *l = list.Get (i);
     if (memcmp (l->GetLightID (), id, 16) == 0) return l;
@@ -713,7 +732,7 @@ bool csLightList::Remove (int n)
 void csLightList::RemoveAll ()
 {
   size_t i;
-  for (i = 0 ; i < list.Length () ; i++)
+  for (i = 0 ; i < list.GetSize () ; i++)
   {
     list[i]->QueryObject ()->RemoveNameChangeListener (listener);
     FreeLight (list[i]);
@@ -753,7 +772,7 @@ csPtr<iLightingProcessData> csLightingProcessInfo::QueryUserdata (
   scfInterfaceID id, int version)
 {
   size_t i;
-  for (i = 0 ; i < userdatas.Length () ; i++)
+  for (i = 0 ; i < userdatas.GetSize () ; i++)
   {
     iLightingProcessData* ptr = (iLightingProcessData*)(
       userdatas[i]->QueryInterface (id, version));
@@ -768,7 +787,7 @@ csPtr<iLightingProcessData> csLightingProcessInfo::QueryUserdata (
 void csLightingProcessInfo::FinalizeLighting ()
 {
   size_t i;
-  for (i = 0 ; i < userdatas.Length () ; i++)
+  for (i = 0 ; i < userdatas.GetSize () ; i++)
   {
     userdatas[i]->FinalizeLighting ();
   }
