@@ -19,17 +19,17 @@
 #ifndef __CS_LIGHT_H__
 #define __CS_LIGHT_H__
 
-#include "csutil/scf_implementation.h"
 #include "csgeom/transfrm.h"
-#include "csgeom/objmodel.h"
-#include "csutil/scf_implementation.h"
-#include "csutil/csobject.h"
+#include "cstool/objmodel.h"
 #include "csutil/cscolor.h"
-#include "csutil/weakref.h"
+#include "csutil/csobject.h"
 #include "csutil/flags.h"
-#include "csutil/nobjvec.h"
 #include "csutil/hash.h"
+#include "csutil/nobjvec.h"
 #include "csutil/refarr.h"
+#include "csutil/scf_implementation.h"
+#include "csutil/scfarray.h"
+#include "csutil/weakref.h"
 #include "iutil/selfdestruct.h"
 #include "plugins/engine/3d/lview.h"
 #include "plugins/engine/3d/halo.h"
@@ -38,6 +38,8 @@
 #include "iengine/light.h"
 #include "iengine/lightmgr.h"
 #include "iengine/viscull.h"
+#include "csgfx/shadervarcontext.h"
+
 
 class csLightMap;
 class csPolygon3D;
@@ -47,12 +49,14 @@ struct iMeshWrapper;
 struct iLightingInfo;
 struct iSector;
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 class csLightObjectModel : public scfImplementationExt0<csLightObjectModel,
                                                         csObjectModel>
 {
 public:
   csBox3 box;
-  csVector3 radius;
+  float radius;
 
   csLightObjectModel ()
     : scfImplementationType (this)
@@ -66,17 +70,25 @@ public:
   {
     bbox = box;
   }
+  virtual const csBox3& GetObjectBoundingBox ()
+  {
+    return box;
+  }
   virtual void SetObjectBoundingBox (const csBox3& bbox)
   {
     box = bbox;
   }
-  virtual void GetRadius (csVector3& rad, csVector3& cent)
+  virtual void GetRadius (float& rad, csVector3& cent)
   {
     rad = radius;
-    cent.Set (0, 0, 0);
+    cent.Set (0, 0, 0);	// @@@ FIXME!
   }
+  virtual iTerraFormer* GetTerraFormerColldet () { return 0; }
+  virtual iTerrainSystem* GetTerrainColldet () { return 0; }
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * Class that represents the influence that a certain light
  * has on a sector.
@@ -101,17 +113,22 @@ public:
 
 typedef csSet<csRef<csLightSectorInfluence> > csLightSectorInfluences;
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 /**
  * Superclass of all positional lights.
  * A light subclassing from this has a color, a position
  * and a radius.
  */
-class csLight : public scfImplementationExt4<csLight,
-                                             csObject,
-                                             iLight,
-                                             iVisibilityObject,
-					     iSceneNode,
-					     iSelfDestruct>
+class csLight : 
+  public scfImplementationExt5<csLight,
+                               csObject,
+                               iLight,
+                               iVisibilityObject,
+                               scfFakeInterface<iShaderVariableContext>,
+			       iSceneNode,
+			       iSelfDestruct>,
+  public CS::ShaderVariableContextImpl
 {
 private:
   /// ID for this light (16-byte MD5).
@@ -128,6 +145,13 @@ protected:
   csColor color;
   /// Specular color
   csColor specularColor;
+  /**
+   * Whether the user changed the specular color.
+   * This decides whether a call to SetColor() changes both the diffuse and
+   * specular color (userSpecular == false) or only diffuse 
+   * (userSpecular == true).
+   */
+  bool userSpecular;
   /// The associated halo (if not 0)
   csHalo *halo;
 
@@ -187,31 +211,18 @@ protected:
   /// List of light/sector influences.
   csLightSectorInfluences influences;
 
+  csEngine* engine;
 public:
   /// Set of flags
   csFlags flags;
 
-public:
-  /** 
-   * Config value: ambient red value.
-   */
-  static int ambient_red;
-  /**
-   * Config value: ambient green value.
-   */
-  static int ambient_green;
-  /** 
-   * Config value: ambient blue value.
-   */
-  static int ambient_blue;
-
 public:
   /**
    * Construct a light at a given position. With
    * a given radius, a given color, a given name and
    * type. The light will not have a halo by default.
    */
-  csLight (float x, float y, float z, float dist,
+  csLight (csEngine* engine, float x, float y, float z, float dist,
      float red, float green, float blue, csLightDynamicType dyntype);
 
   /**
@@ -297,6 +308,11 @@ public:
     return s;
   }
 
+  /**
+   * Get the current full sector (including parents).
+   */
+  iSector* GetFullSector ();
+
   /**
    * Set the center position.
    */
@@ -350,7 +366,7 @@ public:
   { return specularColor; }
   /// Set the specular color of this light.
   void SetSpecularColor (const csColor& col) 
-  { specularColor = col; }
+  { userSpecular = true; specularColor = col; }
 
   /**
    * Return the associated halo
@@ -388,8 +404,8 @@ public:
   { return attenuationConstants; }
 
   /**
-  * Get the the maximum distance at which the light is guranteed to shine. 
-  * Can be seen as the distance at which we turn the light of.
+  * Get the the maximum distance at which the light is guaranteed to shine.
+  * Can be seen as the distance at which we turn the light off.
   * Used for culling and selection of meshes to light, but not
   * for the lighting itself.
   */
@@ -397,8 +413,8 @@ public:
   { return cutoffDistance; }
 
   /**
-  * Set the the maximum distance at which the light is guranteed to shine. 
-  * Can be seen as the distance at which we turn the light of.
+  * Set the the maximum distance at which the light is guaranteed to shine. 
+  * Can be seen as the distance at which we turn the light off.
   * Used for culling and selection of meshes to light, but not
   * for the lighting itself.
   */
@@ -465,6 +481,11 @@ public:
     UpdateViscullMesh ();
   }
 
+  virtual iShaderVariableContext* GetSVContext()
+  {
+    return (iShaderVariableContext*)this;
+  }
+
   //----------------------------------------------------------------------
   // Light influence stuff.
   //----------------------------------------------------------------------
@@ -496,7 +517,7 @@ public:
 
   int GetLightCallbackCount () const
   {
-    return (int)light_cb_vector.Length ();
+    return (int)light_cb_vector.GetSize ();
   }
   
   iLightCallback* GetLightCallback (int idx) const
@@ -526,8 +547,8 @@ public:
   virtual iObjectModel* GetObjectModel () { return object_model; }
   virtual csFlags& GetCullerFlags () { return culler_flags; }
 
-  //--------------------- iSceneNode implementation ----------------------//
-
+  /**\name iSceneNode implementation
+   * @{ */
   virtual void SetParent (iSceneNode* parent)
   {
     csSceneNode::SetParent ((iSceneNode*)this, parent, &movable);
@@ -543,9 +564,16 @@ public:
   {
     return movable.GetChildren ();
   }
+  virtual csPtr<iSceneNodeArray> GetChildrenArray () const
+  {
+    return csPtr<iSceneNodeArray> (
+      new scfArrayWrapConst<iSceneNodeArray, csRefArray<iSceneNode> > (
+      movable.GetChildren ()));
+  }
   virtual iMeshWrapper* QueryMesh () { return 0; }
   virtual iLight* QueryLight () { return this; }
   virtual iCamera* QueryCamera () { return 0; }
+  /** @} */
 
   //--------------------- iSelfDestruct implementation -------------------//
 
@@ -572,6 +600,8 @@ public:
 
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * List of lights for a sector. This class implements iLightList.
  */
@@ -580,7 +610,7 @@ class csLightList : public scfImplementation1<csLightList,
 {
 private:
   csRefArrayObject<iLight> list;
-  csHash<iLight*,csStrKey> lights_hash;
+  csHash<iLight*, csString> lights_hash;
 
   class NameChangeListener : public scfImplementation1<NameChangeListener,
   	iObjectNameChangeListener>
@@ -618,7 +648,7 @@ public:
   /// Override FreeLight
   virtual void FreeLight (iLight*) { }
 
-  virtual int GetCount () const { return (int)list.Length (); }
+  virtual int GetCount () const { return (int)list.GetSize (); }
   virtual iLight *Get (int n) const { return list.Get (n); }
   virtual int Add (iLight *obj);
   virtual bool Remove (iLight *obj);
