@@ -1,19 +1,19 @@
 /*
-    Copyright (C) 2005-2005 by Jorrit Tyberghein
+Copyright (C) 2005-2005 by Jorrit Tyberghein
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Library General Public
+License as published by the Free Software Foundation; either
+version 2 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Library General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+You should have received a copy of the GNU Library General Public
+License along with this library; if not, write to the Free
+Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
 #include "cssysdef.h"
@@ -22,19 +22,101 @@
 #include "plugins/engine/3d/engine.h"
 #include "plugins/engine/3d/meshgen.h"
 #include "iengine/material.h"
+#include "igraphic/image.h"
 
 csMeshGeneratorGeometry::csMeshGeneratorGeometry (
-    csMeshGenerator* generator) : scfImplementationType (this),
-				  generator (generator)
+  csMeshGenerator* generator) : scfImplementationType (this),
+  generator (generator)
 {
   radius = 0.0f;
   density = 1.0f;
   total_max_dist = 0.0f;
   default_material_factor = 0.0f;
+  celldim = 0;
+  positions = 0;
+}
+
+csMeshGeneratorGeometry::~csMeshGeneratorGeometry ()
+{
+}
+
+void csMeshGeneratorGeometry::GetDensityMapFactor (float x, float z,
+                                                   float &data)
+{
+  if (density_map)
+  {
+    if (density_map.IsValid ())
+      density_map->SampleFloat (density_map_type, x, z, data); 
+    data *= density_map_factor;  
+  }
+  else data = 1.0f;
+}
+void csMeshGeneratorGeometry::SetDensityMap (iTerraFormer* map, float factor, 
+                                             const csStringID &type)
+{
+  density_map = map;
+  density_map_factor = factor;
+  density_map_type = type;
+}
+
+void csMeshGeneratorGeometry::AddPositionsFromMap (iTerraFormer* map,
+	const csBox2 &region, uint resx, uint resy, float value,
+	const csStringID & type)
+{
+  csRef<iTerraSampler> sampler = map->GetSampler (region, resx, resy);
+  float stepx = (region.MaxX () - region.MinX ())/resx;
+  float stepy = (region.MaxY () - region.MinY ())/resy;
+
+  const float* map_values = sampler->SampleFloat (type);
+  float curx = region.MinX (), cury = region.MinY ();
+  for (uint i = 0; i < resx; i++)
+  {
+    for (uint j = 0; j < resy; j++)
+    {
+      if (map_values[i*resx + j] == value)
+      {
+        AddPosition (csVector2 (curx, cury));
+      }
+      curx += stepx;
+    }
+    curx = region.MinX ();
+    cury += stepy;
+  }
+}
+
+void csMeshGeneratorGeometry::ResetManualPositions (int new_celldim)
+{
+  if (celldim == new_celldim)
+    return;
+
+  csArray<csVector2> tmp_pos;
+  for (int i = 0; i < celldim; i++)
+  {
+    for (size_t j = 0; j < positions[i].GetSize (); j++)
+    {
+      tmp_pos.Push (positions[i][j]);
+    }
+  }
+
+  delete[] positions;
+
+  celldim = new_celldim;
+  positions = new csArray<csVector2> [celldim*celldim];
+
+  for (size_t i = 0; i < tmp_pos.GetSize (); i++)
+  {
+    AddPosition (tmp_pos[i]);
+  }
+}
+
+void csMeshGeneratorGeometry::AddPosition (const csVector2 &pos)
+{
+  ResetManualPositions (generator->GetCellCount ());
+  positions[generator->GetCellId (pos)].Push (pos);
 }
 
 void csMeshGeneratorGeometry::AddDensityMaterialFactor (
-	iMaterialWrapper* material, float factor)
+  iMaterialWrapper* material, float factor)
 {
   csMGDensityMaterialFactor mf;
   mf.material = material;
@@ -50,7 +132,7 @@ static int CompareGeom (csMGGeom const& m1, csMGGeom const& m2)
 }
 
 void csMeshGeneratorGeometry::AddFactory (iMeshFactoryWrapper* factory,
-    float maxdist)
+                                          float maxdist)
 {
   csMGGeom g;
   g.factory = factory;
@@ -61,14 +143,15 @@ void csMeshGeneratorGeometry::AddFactory (iMeshFactoryWrapper* factory,
 
   csRef<iInstancingFactoryState> fs =
     scfQueryInterface<iInstancingFactoryState> (
-	factory->GetMeshObjectFactory ());
+    factory->GetMeshObjectFactory ());
   if (fs != 0)
   {
     int cell_dim = generator->GetCellCount ();
-    factories[idx].instmeshes.SetLength (cell_dim * cell_dim);
+    factories[idx].instmeshes.SetSize (cell_dim * cell_dim);
   }
 
   if (maxdist > total_max_dist) total_max_dist = maxdist;
+  
 }
 
 void csMeshGeneratorGeometry::RemoveFactory (size_t idx)
@@ -87,46 +170,46 @@ void csMeshGeneratorGeometry::SetDensity (float density)
 }
 
 csPtr<iMeshWrapper> csMeshGeneratorGeometry::AllocMesh (
-    int cidx, const csMGCell& cell, float sqdist,
-    size_t& lod, size_t& instance_id)
+  int cidx, const csMGCell& cell, float sqdist,
+  size_t& lod, size_t& instance_id)
 {
   lod = GetLODLevel (sqdist);
   if (lod == csArrayItemNotFound) return 0;
 
   csMGGeom& geom = factories[lod];
   // First check if the geometry is for instmesh.
-  if (geom.instmeshes.Length () > 0)
+  if (geom.instmeshes.GetSize () > 0)
   {
     csMGGeomInstMesh& geominst = geom.instmeshes[cidx];
     if (!geominst.instmesh)
     {
-      geominst.instmesh = csEngine::currentEngine->CreateMeshWrapper (
-	  geom.factory, 0);
+      geominst.instmesh = generator->engine->CreateMeshWrapper (
+        geom.factory, 0);
       csVector3 cell_center (
-	  (cell.box.MinX ()+cell.box.MaxX ())/2.0f,
-	  0.0f,	// @@@ FIXME? OK?
-	  (cell.box.MinY ()+cell.box.MaxY ())/2.0f);
+        (cell.box.MinX ()+cell.box.MaxX ())/2.0f,
+        0.0f,	// @@@ FIXME? OK?
+        (cell.box.MinY ()+cell.box.MaxY ())/2.0f);
       geominst.instmesh->GetMovable ()->SetPosition (generator->GetSector (),
-	  cell_center);
+        cell_center);
       geominst.instmesh->GetMovable ()->UpdateMove ();
       geominst.instmesh_state = scfQueryInterface<iInstancingMeshState> (
-	  geominst.instmesh->GetMeshObject ());
+        geominst.instmesh->GetMeshObject ());
     }
-    if (geominst.inst_setaside.Length () > 0)
+    if (geominst.inst_setaside.GetSize () > 0)
       instance_id = geominst.inst_setaside.Pop ();
     else
       instance_id = geominst.instmesh_state->AddInstance (
-	  csReversibleTransform ());
+      csReversibleTransform ());
     geominst.instmesh->IncRef ();
     return (iMeshWrapper*)geominst.instmesh;
   }
   // See if we have some mesh ready in the setaside or normal cache.
-  else if (geom.mesh_setaside.Length () > 0)
+  else if (geom.mesh_setaside.GetSize () > 0)
   {
     instance_id = csArrayItemNotFound;
     return geom.mesh_setaside.Pop ();
   }
-  else if (geom.mesh_cache.Length () > 0)
+  else if (geom.mesh_cache.GetSize () > 0)
   {
     instance_id = csArrayItemNotFound;
     return geom.mesh_cache.Pop ();
@@ -134,13 +217,13 @@ csPtr<iMeshWrapper> csMeshGeneratorGeometry::AllocMesh (
   else
   {
     instance_id = csArrayItemNotFound;
-    return csEngine::currentEngine->CreateMeshWrapper (geom.factory, 0);
+    return generator->engine->CreateMeshWrapper (geom.factory, 0);
   }
 }
 
 void csMeshGeneratorGeometry::MoveMesh (int cidx, iMeshWrapper* mesh,
-    size_t lod, size_t instance_id, const csVector3& position,
-    const csMatrix3& matrix)
+                                        size_t lod, size_t instance_id, const csVector3& position,
+                                        const csMatrix3& matrix)
 {
   if (instance_id == csArrayItemNotFound)
   {
@@ -154,20 +237,20 @@ void csMeshGeneratorGeometry::MoveMesh (int cidx, iMeshWrapper* mesh,
     csMGGeomInstMesh& geominst = geom.instmeshes[cidx];
     csVector3 meshpos = mesh->GetMovable ()->GetFullPosition ();
     csVector3 pos = position - meshpos;
-    printf ("position=%g,%g,%g    meshpos=%g,%g,%g  ->  pos=%g,%g,%g\n", position.x, position.y, position.z, meshpos.x, meshpos.y, meshpos.z, pos.x, pos.y, pos.z); fflush (stdout);
+    ////printf ("position=%g,%g,%g    meshpos=%g,%g,%g  ->  pos=%g,%g,%g\n", position.x, position.y, position.z, meshpos.x, meshpos.y, meshpos.z, pos.x, pos.y, pos.z); fflush (stdout);
     csReversibleTransform tr (matrix, pos);
     geominst.instmesh_state->MoveInstance (instance_id, tr);
   }
 }
 
 void csMeshGeneratorGeometry::SetAsideMesh (int cidx, iMeshWrapper* mesh,
-    size_t lod, size_t instance_id)
+                                            size_t lod, size_t instance_id)
 {
   csMGGeom& geom = factories[lod];
-  if (geom.instmeshes.Length () > 0)
+  if (geom.instmeshes.GetSize () > 0)
   {
     csMGGeomInstMesh& geominst = geom.instmeshes[cidx];
-    geominst.inst_setaside.Push (cidx);
+    geominst.inst_setaside.Push (instance_id);
     geom.instmesh_setaside.Add (&geominst);
   }
   else
@@ -179,10 +262,10 @@ void csMeshGeneratorGeometry::SetAsideMesh (int cidx, iMeshWrapper* mesh,
 void csMeshGeneratorGeometry::FreeSetAsideMeshes ()
 {
   size_t lod;
-  for (lod = 0 ; lod < factories.Length () ; lod++)
+  for (lod = 0 ; lod < factories.GetSize () ; lod++)
   {
     csMGGeom& geom = factories[lod];
-    while (geom.mesh_setaside.Length () > 0)
+    while (geom.mesh_setaside.GetSize () > 0)
     {
       csRef<iMeshWrapper> mesh = geom.mesh_setaside.Pop ();
       mesh->GetMovable ()->ClearSectors ();
@@ -193,10 +276,10 @@ void csMeshGeneratorGeometry::FreeSetAsideMeshes ()
     while (it.HasNext ())
     {
       csMGGeomInstMesh* geominst = it.Next ();
-      while (geominst->inst_setaside.Length () > 0)
+      while (geominst->inst_setaside.GetSize () > 0)
       {
-	size_t instance_id = geominst->inst_setaside.Pop ();
-	geominst->instmesh_state->RemoveInstance (instance_id);
+        size_t instance_id = geominst->inst_setaside.Pop ();
+        geominst->instmesh_state->RemoveInstance (instance_id);
       }
     }
     geom.instmesh_setaside.Empty ();
@@ -207,7 +290,7 @@ void csMeshGeneratorGeometry::FreeSetAsideMeshes ()
 size_t csMeshGeneratorGeometry::GetLODLevel (float sqdist)
 {
   size_t i;
-  for (i = 0 ; i < factories.Length () ; i++)
+  for (i = 0 ; i < factories.GetSize () ; i++)
   {
     csMGGeom& geom = factories[i];
     if (sqdist <= geom.sqmaxdistance)
@@ -221,46 +304,42 @@ size_t csMeshGeneratorGeometry::GetLODLevel (float sqdist)
 bool csMeshGeneratorGeometry::IsRightLOD (float sqdist, size_t current_lod)
 {
   // With only one lod level we are always right.
-  if (factories.Length () <= 1) return true;
+  if (factories.GetSize () <= 1) return true;
   if (current_lod == 0)
     return (sqdist <= factories[0].sqmaxdistance);
   else
     return (sqdist <= factories[current_lod].sqmaxdistance) &&
-      (sqdist > factories[current_lod-1].sqmaxdistance);
+    (sqdist > factories[current_lod-1].sqmaxdistance);
 }
 
 //--------------------------------------------------------------------------
 
-csMeshGenerator::csMeshGenerator() : scfImplementationType (this)
+csMeshGenerator::csMeshGenerator (csEngine* engine) : 
+  scfImplementationType (this), total_max_dist (-1.0f), 
+  use_density_scaling (false), use_alpha_scaling (false),
+  last_pos (0, 0), setup_cells (false), cell_dim (50), 
+  inuse_blocks (0), inuse_blocks_last (0), max_blocks (100),
+  engine (engine)
 {
-  max_blocks = 100;
-  cell_dim = 50;
-  use_density_scaling = false;
-  use_alpha_scaling = false;
-
-  sector = 0;
-
-  total_max_dist = -1.0f;
-
   cells = new csMGCell [cell_dim * cell_dim];
-  size_t i;
-  for (i = 0 ; i < size_t (max_blocks) ; i++)
+
+  for (size_t i = 0 ; i < size_t (max_blocks) ; i++)
     cache_blocks.Push (new csMGPositionBlock ());
-  inuse_blocks = 0;
-  inuse_blocks_last = 0;
 
   prev_cells.MakeEmpty ();
 
-  setup_cells = false;
-
-  for (i = 0 ; i < CS_GEOM_MAX_ROTATIONS ; i++)
+  for (size_t i = 0 ; i < CS_GEOM_MAX_ROTATIONS ; i++)
   {
     rotation_matrices[i] = csYRotMatrix3 (2.0f*PI * float (i)
-    	/ float (CS_GEOM_MAX_ROTATIONS));
+      / float (CS_GEOM_MAX_ROTATIONS));
   }
 
-  alpha_priority = csEngine::currentEngine->GetAlphaRenderPriority ();
-  object_priority = csEngine::currentEngine->GetObjectRenderPriority ();
+  alpha_priority = engine->GetAlphaRenderPriority ();
+  object_priority = engine->GetObjectRenderPriority ();
+  
+  csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+    engine->objectRegistry, "crystalspace.shared.stringset");
+  varTransform = strings->Request ("transform");
 }
 
 csMeshGenerator::~csMeshGenerator ()
@@ -285,7 +364,7 @@ void csMeshGenerator::SelfDestruct ()
       if (GetSector ()->GetMeshGenerator (c) == (iMeshGenerator*)this)
       {
         GetSector ()->RemoveMeshGenerator (c);
-	return;
+        return;
       }
     }
     CS_ASSERT (false);
@@ -298,7 +377,7 @@ float csMeshGenerator::GetTotalMaxDist ()
   {
     total_max_dist = 0.0f;
     size_t i;
-    for (i = 0 ; i < geometries.Length () ; i++)
+    for (i = 0 ; i < geometries.GetSize () ; i++)
     {
       float md = geometries[i]->GetTotalMaxDist ();
       if (md > total_max_dist) total_max_dist = md;
@@ -309,7 +388,7 @@ float csMeshGenerator::GetTotalMaxDist ()
 }
 
 void csMeshGenerator::SetDensityScale (float mindist, float maxdist,
-  	float maxdensityfactor)
+                                       float maxdensityfactor)
 {
   use_density_scaling = true;
   density_mindist = mindist;
@@ -339,20 +418,20 @@ void csMeshGenerator::SetCellCount (int number)
     for (z = 0 ; z < cell_dim ; z++)
       for (x = 0 ; x < cell_dim ; x++)
       {
-	int cidx = z*cell_dim + x;
-	csMGCell& cell = cells[cidx];
+        int cidx = z*cell_dim + x;
+        csMGCell& cell = cells[cidx];
         FreeMeshesInBlock (cidx, cell);
-	if (cell.block)
-	{
-	  cell.block->parent_cell = csArrayItemNotFound;
-	  cache_blocks.Push (cell.block);
-	  cell.block->next = cell.block->prev = 0;
-	  cell.block = 0;
+        if (cell.block)
+        {
+          cell.block->parent_cell = csArrayItemNotFound;
+          cache_blocks.Push (cell.block);
+          cell.block->next = cell.block->prev = 0;
+          cell.block = 0;
         }
       }
-    inuse_blocks = 0;
-    inuse_blocks_last = 0;
-    setup_cells = false;
+      inuse_blocks = 0;
+      inuse_blocks_last = 0;
+      setup_cells = false;
   }
   cell_dim = number;
   delete[] cells;
@@ -368,21 +447,21 @@ void csMeshGenerator::SetBlockCount (int number)
     for (z = 0 ; z < cell_dim ; z++)
       for (x = 0 ; x < cell_dim ; x++)
       {
-	int cidx = z*cell_dim + x;
-	csMGCell& cell = cells[cidx];
+        int cidx = z*cell_dim + x;
+        csMGCell& cell = cells[cidx];
         FreeMeshesInBlock (cidx, cell);
-	if (cell.block)
-	{
-	  cell.block->parent_cell = csArrayItemNotFound;
-	  cache_blocks.Push (cell.block);
-	  cell.block->next = cell.block->prev = 0;
-	  cell.block = 0;
+        if (cell.block)
+        {
+          cell.block->parent_cell = csArrayItemNotFound;
+          cache_blocks.Push (cell.block);
+          cell.block->next = cell.block->prev = 0;
+          cell.block = 0;
         }
       }
-    inuse_blocks = 0;
-    inuse_blocks_last = 0;
+      inuse_blocks = 0;
+      inuse_blocks_last = 0;
 
-    setup_cells = false;
+      setup_cells = false;
   }
   size_t i;
   if (number > max_blocks)
@@ -424,7 +503,7 @@ void csMeshGenerator::SetupSampleBox ()
     {
       float wx = GetWorldX (x);
       cells[idx].box.Set (wx, wz, wx + samplecellwidth_x,
-	      wz + samplecellheight_z);
+        wz + samplecellheight_z);
       // Here we need to calculate the meshe relevant for this cell (i.e.
       // meshes that intersect this cell as seen in 2D space).
       // @@@ For now we just copy the list of meshes from csMeshGenerator.
@@ -432,6 +511,16 @@ void csMeshGenerator::SetupSampleBox ()
       idx++;
     }
   }
+  for (size_t g = 0 ; g < geometries.GetSize () ; g++)
+    geometries[g]->ResetManualPositions (cell_dim);
+}
+
+int csMeshGenerator::GetCellId (const csVector2& pos)
+{
+  SetupSampleBox ();
+  int cellx = GetCellX (pos.x);
+  int cellz = GetCellZ (pos.y);
+  return cellz*cell_dim + cellx;
 }
 
 size_t csMeshGenerator::CountPositions (int cidx, csMGCell& cell)
@@ -443,7 +532,7 @@ size_t csMeshGenerator::CountPositions (int cidx, csMGCell& cell)
   float box_area = box.Area ();
 
   size_t i, j, g;
-  for (g = 0 ; g < geometries.Length () ; g++)
+  for (g = 0 ; g < geometries.GetSize () ; g++)
   {
     float density = geometries[g]->GetDensity ();
     size_t count = size_t (density * box_area);
@@ -455,16 +544,17 @@ size_t csMeshGenerator::CountPositions (int cidx, csMGCell& cell)
       csVector3 end = start;
       end.y = samplebox.MinY ();
       bool hit = false;
-      for (i = 0 ; i < cell.meshes.Length () ; i++)
+      for (i = 0 ; i < cell.meshes.GetSize () ; i++)
       {
         csHitBeamResult rc = cell.meshes[i]->HitBeam (start, end);
-	if (rc.hit)
-	{
-	  end.y = rc.isect.y + 0.0001;
-	  hit = true;
-	  break;
-	}
+        if (rc.hit)
+        {
+          end.y = rc.isect.y + 0.0001;
+          hit = true;
+          break;
+        }
       }
+      
       if (hit)
       {
         counter++;
@@ -485,11 +575,11 @@ size_t csMeshGenerator::CountAllPositions ()
       csMGCell& cell = cells[cidx];
       counter += CountPositions (cidx, cell);
     }
-  return counter;
+    return counter;
 }
 
 void csMeshGenerator::GeneratePositions (int cidx, csMGCell& cell,
-	csMGPositionBlock* block)
+                                         csMGPositionBlock* block)
 {
   random.Initialize ((unsigned int)cidx); // @@@ Consider using a better seed?
 
@@ -499,65 +589,88 @@ void csMeshGenerator::GeneratePositions (int cidx, csMGCell& cell,
   float box_area = box.Area ();
 
   size_t i, j, g;
-  for (g = 0 ; g < geometries.Length () ; g++)
+  for (g = 0 ; g < geometries.GetSize () ; g++)
   {
     csMGPosition pos;
     pos.geom_type = g;
+    size_t mpos_count = geometries[g]->GetManualPositionCount (cidx);
+
     float density = geometries[g]->GetDensity ();
-    size_t count = size_t (density * box_area);
+
+    size_t count = (mpos_count > 0)? mpos_count : size_t (density * box_area);
+
     const csArray<csMGDensityMaterialFactor>& mftable
-    	= geometries[g]->GetDensityMaterialFactors ();
+      = geometries[g]->GetDensityMaterialFactors ();
     float default_material_factor
-    	= geometries[g]->GetDefaultDensityMaterialFactor ();
-    bool do_material = mftable.Length () > 0;
+      = geometries[g]->GetDefaultDensityMaterialFactor ();
+    bool do_material = mftable.GetSize () > 0;
     for (j = 0 ; j < count ; j++)
     {
-      float x = random.Get (box.MinX (), box.MaxX ());
-      float z = random.Get (box.MinY (), box.MaxY ());
-      csVector3 start (x, samplebox.MaxY (), z);
-      csVector3 end = start;
-      end.y = samplebox.MinY ();
-      bool hit = false;
-      iMaterialWrapper* hit_material = 0;
-      for (i = 0 ; i < cell.meshes.Length () ; i++)
+      float pos_factor;
+      float x;
+      float z;
+      if (mpos_count == 0)
       {
-        csHitBeamResult rc = cell.meshes[i]->HitBeamObject (start, end,
-		do_material);
-	if (rc.hit)
-	{
-          pos.position = rc.isect;
-	  end.y = rc.isect.y + 0.0001;
-	  hit_material = rc.material;
-	  hit = true;
-	}
+        x = random.Get (box.MinX (), box.MaxX ());
+        z = random.Get (box.MinY (), box.MaxY ());
+        geometries[g]->GetDensityMapFactor (x, z, pos_factor);
       }
-      if (hit)
+      else
       {
-	if (do_material)
-	{
-	  // We use material density tables.
-	  float factor = default_material_factor;
-	  size_t mi;
-	  for (mi = 0 ; mi < mftable.Length () ; mi++)
-	    if (mftable[mi].material == hit_material)
-	    {
-	      factor = mftable[mi].factor;
-	      break;
-	    }
-	  if (factor < 0.0001) hit = false;
-	  else if (factor < 0.9999 && random.Get () > factor)
-	  {
-	    hit = false;
-	  }
-	}
-	if (hit)
-	{
-          int rot = int (random.Get (CS_GEOM_MAX_ROTATIONS));
-	  if (rot < 0) rot = 0;
-	  else if (rot >= CS_GEOM_MAX_ROTATIONS) rot = CS_GEOM_MAX_ROTATIONS-1;
-          pos.rotation = rot;
-	  pos.random = random.Get ();
-          block->positions.Push (pos);
+        csVector2 mpos = geometries[g]->GetManualPosition (cidx,j);
+        x = mpos.x; 
+        z = mpos.y;
+        pos_factor = 1.0f;
+      }
+      
+      if (!((pos_factor < 0.0001) ||
+        (pos_factor < 0.9999 && random.Get () > pos_factor)))
+      {
+        csVector3 start (x, samplebox.MaxY (), z);
+        csVector3 end = start;
+        end.y = samplebox.MinY ();
+        bool hit = false;
+        iMaterialWrapper* hit_material = 0;
+        for (i = 0 ; i < cell.meshes.GetSize () ; i++)
+        {
+          csHitBeamResult rc = cell.meshes[i]->HitBeam (start, end,
+            do_material);
+          if (rc.hit)
+          {
+            pos.position = rc.isect;
+            end.y = rc.isect.y + 0.0001;
+            hit_material = rc.material;
+            hit = true;
+          }
+        }
+        if (hit)
+        {
+          if (do_material)
+          {
+            // We use material density tables.
+            float factor = default_material_factor;
+            size_t mi;
+            for (mi = 0 ; mi < mftable.GetSize () ; mi++)
+              if (mftable[mi].material == hit_material)
+              {
+                factor = mftable[mi].factor;
+                break;
+              }
+              if (factor < 0.0001) hit = false;
+              else if (factor < 0.9999 && random.Get () > factor)
+              {
+                hit = false;
+              }
+          }
+          if (hit)
+          {
+            int rot = int (random.Get (CS_GEOM_MAX_ROTATIONS));
+            if (rot < 0) rot = 0;
+            else if (rot >= CS_GEOM_MAX_ROTATIONS) rot = CS_GEOM_MAX_ROTATIONS-1;
+            pos.rotation = rot;
+            pos.random = random.Get ();
+            block->positions.Push (pos);
+          }
         }
       }
     }
@@ -585,7 +698,7 @@ void csMeshGenerator::AllocateBlock (int cidx, csMGCell& cell)
       inuse_blocks = block;
     }
   }
-  else if (cache_blocks.Length () > 0)
+  else if (cache_blocks.GetSize () > 0)
   {
     // We need a new block and one is available in the cache.
     csMGPositionBlock* block = cache_blocks.Pop ();
@@ -627,6 +740,17 @@ void csMeshGenerator::AllocateBlock (int cidx, csMGCell& cell)
   }
 }
 
+void csMeshGenerator::SetFade (iMeshWrapper* mesh, uint mode)
+{
+  if (!mesh) return;
+  mesh->GetMeshObject ()->SetMixMode (mode);
+  iSceneNode* sn = mesh->QuerySceneNode ();
+  const csRef<iSceneNodeArray> children = sn->GetChildrenArray ();
+  size_t i;
+  for (i = 0 ; i < children->GetSize(); i++)
+    SetFade (children->Get (i)->QueryMesh (), mode);
+}
+
 void csMeshGenerator::SetFade (csMGPosition& p, float factor)
 {
   if (factor < .01)
@@ -634,38 +758,38 @@ void csMeshGenerator::SetFade (csMGPosition& p, float factor)
     if (p.last_mixmode == CS_FX_TRANSPARENT)
       return;
     p.last_mixmode = CS_FX_TRANSPARENT;
-    p.mesh->GetMeshObject ()->SetMixMode (CS_FX_TRANSPARENT);
+    SetFade (p.mesh, CS_FX_TRANSPARENT);
   }
   else if (factor < .99)
   {
     if (p.mesh->GetRenderPriority () != alpha_priority)
     {
-      p.mesh->SetRenderPriority (alpha_priority);
-      p.mesh->SetZBufMode (CS_ZBUF_TEST);
+      p.mesh->SetRenderPriorityRecursive (alpha_priority);
+      p.mesh->SetZBufModeRecursive (CS_ZBUF_TEST);
     }
     p.last_mixmode = CS_FX_SETALPHA (1.0f-factor);
-    p.mesh->GetMeshObject ()->SetMixMode (p.last_mixmode);
+    SetFade (p.mesh, p.last_mixmode);
   }
   else
   {
     if (p.last_mixmode == CS_FX_COPY)
       return;
     p.last_mixmode = CS_FX_COPY;
-    p.mesh->GetMeshObject ()->SetMixMode (CS_FX_COPY);
-    p.mesh->SetRenderPriority (object_priority);
-    p.mesh->SetZBufMode (CS_ZBUF_USE);
+    SetFade (p.mesh, CS_FX_COPY);
+    p.mesh->SetRenderPriorityRecursive (object_priority);
+    p.mesh->SetZBufModeRecursive (CS_ZBUF_USE);
   }
 }
 
 void csMeshGenerator::AllocateMeshes (int cidx, csMGCell& cell,
-    const csVector3& pos)
+                                      const csVector3& pos)
 {
   CS_ASSERT (cell.block != 0);
   CS_ASSERT (sector != 0);
   csArray<csMGPosition>& positions = cell.block->positions;
   GetTotalMaxDist ();
   size_t i;
-  for (i = 0 ; i < positions.Length () ; i++)
+  for (i = 0 ; i < positions.GetSize () ; i++)
   {
     csMGPosition& p = positions[i];
 
@@ -675,78 +799,100 @@ void csMeshGenerator::AllocateMeshes (int cidx, csMGCell& cell,
       if (!p.mesh)
       {
         // We didn't have a mesh here so we allocate one.
-	// But first we test if we have density scaling.
-	bool show = true;
+        // But first we test if we have density scaling.
+        bool show = true;
         if (use_density_scaling && sqdist > sq_density_mindist)
         {
           float dist = sqrt (sqdist);
           float factor = (density_maxdist - dist) * density_scale
-	    + density_maxfactor;
+            + density_maxfactor;
           if (factor < 0) factor = 0;
           else if (factor > 1) factor = 1;
           if (p.random > factor) show = false;
         }
 
-	if (show)
-	{
+        if (show)
+        {
           csRef<iMeshWrapper> mesh = geometries[p.geom_type]->AllocMesh (
-	      cidx, cell, sqdist, p.lod, p.instance_id);
-	  if (mesh)
-	  {
+            cidx, cell, sqdist, p.lod, p.instance_id);
+          if (mesh)
+          {
             p.mesh = mesh;
-	    p.last_mixmode = ~0;
-	    geometries[p.geom_type]->MoveMesh (cidx, mesh, p.lod,
-		p.instance_id, p.position, rotation_matrices[p.rotation]);
+            p.last_mixmode = ~0;
+            geometries[p.geom_type]->MoveMesh (cidx, mesh, p.lod,
+              p.instance_id, p.position, rotation_matrices[p.rotation]);
+
+/*
+  csRef<iImposter> imposter = SCF_QUERY_INTERFACE (mesh, iImposter);
+    imposter->SetImposterActive (true);
+  iSharedVariable *var = engine->GetVariableList()->FindByName 
+    ("Std Thing Range");
+  imposter->SetMinDistance (var);
+  var = engine->GetVariableList()->FindByName
+    ("Std Thing Angle");
+  imposter->SetRotationTolerance (var);
+*/
           }
-	}
+        }
       }
       else
       {
         // We already have a mesh but we check here if we should switch LOD level.
-	if (!geometries[p.geom_type]->IsRightLOD (sqdist, p.lod))
-	{
-	  // We need a different mesh here.
+        if (!geometries[p.geom_type]->IsRightLOD (sqdist, p.lod))
+        {
+          // We need a different mesh here.
           geometries[p.geom_type]->SetAsideMesh (cidx, p.mesh, p.lod,
-	      p.instance_id);
+            p.instance_id);
           csRef<iMeshWrapper> mesh = geometries[p.geom_type]->AllocMesh (
-	      cidx, cell, sqdist, p.lod, p.instance_id);
+            cidx, cell, sqdist, p.lod, p.instance_id);
           p.mesh = mesh;
-	  if (mesh)
-	  {
-	    p.last_mixmode = ~0;
-	    geometries[p.geom_type]->MoveMesh (cidx, mesh, p.lod, p.instance_id,
-		p.position, rotation_matrices[p.rotation]);
-	  }
-	}
+          if (mesh)
+          {
+            p.last_mixmode = ~0;
+            geometries[p.geom_type]->MoveMesh (cidx, mesh, p.lod, p.instance_id,
+              p.position, rotation_matrices[p.rotation]);
+/*
+printf("case 2\n");
+  csRef<iImposter> imposter = SCF_QUERY_INTERFACE (mesh, iImposter);
+    imposter->SetImposterActive (true);
+  iSharedVariable *var = engine->GetVariableList()->FindByName 
+    ("Std Thing Range");
+  imposter->SetMinDistance (var);
+  var = engine->GetVariableList()->FindByName
+    ("Std Thing Angle");
+  imposter->SetRotationTolerance (var);
+*/
+          }
+        }
       }
       if (p.mesh && use_alpha_scaling)
       {
-	// This is used only if we have both density scaling and
-	// alpha scaling. It is the offset to correct the distance at which
-	// the object will disappear. We can adapt alpha scaling to scale
-	// based on that new distance.
-	float correct_dist = 0;
-	float correct_sq_alpha_mindist = sq_alpha_mindist;
+        // This is used only if we have both density scaling and
+        // alpha scaling. It is the offset to correct the distance at which
+        // the object will disappear. We can adapt alpha scaling to scale
+        // based on that new distance.
+        float correct_dist = 0;
+        float correct_sq_alpha_mindist = sq_alpha_mindist;
 
 #if 0
         if (use_density_scaling && sqdist > sq_density_mindist)
         {
-	  correct_dist = (p.random - density_maxfactor) / density_scale;
-	  if (correct_dist < 0) correct_dist = 0;
-	  correct_sq_alpha_mindist = alpha_mindist-correct_dist;
-	  correct_sq_alpha_mindist *= correct_sq_alpha_mindist;
-//printf ("p.mesh='%s' sqdist=%g dist=%g correct_sq_alpha_mindist=%g correct_dist=%g p.random=%g", p.mesh->QueryObject ()->GetName (), sqdist, sqrt (sqdist), correct_sq_alpha_mindist, correct_dist, p.random); fflush (stdout);
+          correct_dist = (p.random - density_maxfactor) / density_scale;
+          if (correct_dist < 0) correct_dist = 0;
+          correct_sq_alpha_mindist = alpha_mindist-correct_dist;
+          correct_sq_alpha_mindist *= correct_sq_alpha_mindist;
+          //printf ("p.mesh='%s' sqdist=%g dist=%g correct_sq_alpha_mindist=%g correct_dist=%g p.random=%g", p.mesh->QueryObject ()->GetName (), sqdist, sqrt (sqdist), correct_sq_alpha_mindist, correct_dist, p.random); fflush (stdout);
         }
 #endif
 
-	float factor = 1.0;
+        float factor = 1.0;
         if (sqdist > correct_sq_alpha_mindist)
         {
           float dist = sqrt (sqdist);
           factor = (alpha_maxdist-correct_dist - dist) * alpha_scale;
-	}
-	//printf (" -> factor=%g\n", factor); fflush (stdout);
-	SetFade (p, factor);
+        }
+        //printf (" -> factor=%g\n", factor); fflush (stdout);
+        SetFade (p, factor);
       }
     }
     else
@@ -754,7 +900,7 @@ void csMeshGenerator::AllocateMeshes (int cidx, csMGCell& cell,
       if (p.mesh)
       {
         geometries[p.geom_type]->SetAsideMesh (cidx, p.mesh, p.lod,
-	    p.instance_id);
+          p.instance_id);
         p.mesh = 0;
       }
     }
@@ -763,9 +909,14 @@ void csMeshGenerator::AllocateMeshes (int cidx, csMGCell& cell,
 
 void csMeshGenerator::AllocateBlocks (const csVector3& pos)
 {
+  //if (setup_cells)
+    //if (pos.x == last_pos.x && pos.z == last_pos.y)
+      //return;
+  last_pos.x = pos.x;
+  last_pos.y = pos.z;
   SetupSampleBox ();
   //printf ("positions=%d\n", CountAllPositions ()); fflush (stdout);
-  
+
   int cellx = GetCellX (pos.x);
   int cellz = GetCellZ (pos.z);
 
@@ -800,9 +951,9 @@ void csMeshGenerator::AllocateBlocks (const csVector3& pos)
     for (z = prev_cells.ymin ; z < prev_cells.ymax ; z++)
       for (x = prev_cells.xmin ; x < prev_cells.xmax ; x++)
         if (!cur_cells.Contains (x, z))
-	{
-	  int cidx = z*cell_dim + x;
-	  csMGCell& cell = cells[cidx];
+        {
+          int cidx = z*cell_dim + x;
+          csMGCell& cell = cells[cidx];
           FreeMeshesInBlock (cidx, cell);
         }
   }
@@ -822,12 +973,12 @@ void csMeshGenerator::AllocateBlocks (const csVector3& pos)
       if (sqdist < sqmd)
       {
         total_needed++;
-	if (total_needed >= max_blocks)
-	{
-	  csEngine::currentEngine->Error (
-	  	"The mesh generator need more blocks then %d!", max_blocks);
-	  return;	// @@@ What to do here???
-	}
+        if (total_needed >= max_blocks)
+        {
+          engine->Error (
+            "The mesh generator needs more blocks than %d!", max_blocks);
+          return;	// @@@ What to do here???
+        }
         AllocateBlock (cidx, cell);
         AllocateMeshes (cidx, cell, pos);
       }
@@ -843,7 +994,7 @@ void csMeshGenerator::AllocateBlocks (const csVector3& pos)
 
   // Now really free the meshes we didn't reuse.
   size_t i;
-  for (i = 0 ; i < geometries.Length () ; i++)
+  for (i = 0 ; i < geometries.GetSize () ; i++)
     geometries[i]->FreeSetAsideMeshes ();
 }
 
@@ -853,14 +1004,14 @@ void csMeshGenerator::FreeMeshesInBlock (int cidx, csMGCell& cell)
   {
     csArray<csMGPosition>& positions = cell.block->positions;
     size_t i;
-    for (i = 0 ; i < positions.Length () ; i++)
+    for (i = 0 ; i < positions.GetSize () ; i++)
     {
       if (positions[i].mesh)
       {
         CS_ASSERT (positions[i].geom_type >= 0);
-        CS_ASSERT (positions[i].geom_type < geometries.Length ());
+        CS_ASSERT (positions[i].geom_type < geometries.GetSize ());
         geometries[positions[i].geom_type]->SetAsideMesh (cidx,
-	    positions[i].mesh, positions[i].lod, positions[i].instance_id);
+          positions[i].mesh, positions[i].lod, positions[i].instance_id);
         positions[i].mesh = 0;
       }
     }
