@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000-2004 by Jorrit Tyberghein
+    Copyright (C) 2000-2007 by Jorrit Tyberghein
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -18,21 +18,22 @@
 #include "cssysdef.h"
 #include "csqsqrt.h"
 #include "csgeom/sphere.h"
-#include "igeom/objmodel.h"
+#include "cstool/rviewclipper.h"
+#include "imesh/objmodel.h"
 #include "igeom/clip2d.h"
 #include "plugins/engine/3d/sector.h"
 #include "plugins/engine/3d/meshobj.h"
+#include "plugins/engine/3d/meshfact.h"
 #include "plugins/engine/3d/meshlod.h"
 #include "plugins/engine/3d/light.h"
 #include "plugins/engine/3d/engine.h"
 #include "iengine/portal.h"
-#include "csutil/debug.h"
 #include "iengine/rview.h"
 #include "ivideo/graph3d.h"
+#include "ivideo/rendermesh.h"
 
 
 CS_LEAKGUARD_IMPLEMENT (csMeshWrapper);
-CS_LEAKGUARD_IMPLEMENT (csMeshFactoryWrapper);
 
 // ---------------------------------------------------------------------------
 
@@ -62,12 +63,12 @@ public:
   virtual void AppendShadows (iMovable* movable, iShadowBlockList* shadows,
   	const csVector3& origin)
   {
-    const csRefArray<iSceneNode>& c = static_lod_mesh->QuerySceneNode ()
-    	->GetChildren ();
-    size_t i = c.Length ();
+    const csRef<iSceneNodeArray> c = static_lod_mesh->QuerySceneNode ()
+      ->GetChildrenArray ();
+    size_t i = c->GetSize ();
     while (i-- > 0)
     {
-      iMeshWrapper* child = c[i]->QueryMesh ();
+      iMeshWrapper* child = c->Get (i)->QueryMesh ();
       if (child && child->GetShadowCaster ())
       {
         child->GetShadowCaster ()->AppendShadows (movable, shadows, origin);
@@ -99,13 +100,13 @@ public:
 
   virtual void CastShadows (iMovable* movable, iFrustumView* fview)
   {
-    const csRefArray<iSceneNode>& c = static_lod_mesh->QuerySceneNode ()
-    	->GetChildren ();
-    size_t cnt = c.Length ();
+    const csRef<iSceneNodeArray> c = static_lod_mesh->QuerySceneNode ()
+      ->GetChildrenArray ();
+    size_t cnt = c->GetSize ();
     size_t i;
     for (i = 0 ; i < cnt ; i++)
     {
-      iMeshWrapper* child = c[i]->QueryMesh ();
+      iMeshWrapper* child = c->Get (i)->QueryMesh ();
       if (child && child->GetShadowReceiver ())
         child->GetShadowReceiver ()->CastShadows (movable, fview);
     }
@@ -117,16 +118,14 @@ public:
 // csMeshWrapper
 // ---------------------------------------------------------------------------
 
-csMeshWrapper::csMeshWrapper (iMeshObject *meshobj) 
-  : scfImplementationType (this)
+csMeshWrapper::csMeshWrapper (csEngine* engine, iMeshObject *meshobj) 
+  : scfImplementationType (this), engine (engine)
 {
-  DG_TYPE (this, "csMeshWrapper");
-
 //  movable.scfParent = this; //@TODO: CHECK THIS
   wor_bbox_movablenr = -1;
   movable.SetMeshWrapper (this);
 
-  render_priority = csEngine::currentEngine->GetObjectRenderPriority ();
+  render_priority = engine->GetObjectRenderPriority ();
 
   last_anim_time = 0;
 
@@ -135,8 +134,8 @@ csMeshWrapper::csMeshWrapper (iMeshObject *meshobj)
   csMeshWrapper::meshobj = meshobj;
   if (meshobj)
   {
-    light_info = SCF_QUERY_INTERFACE (meshobj, iLightingInfo);
-    portal_container = SCF_QUERY_INTERFACE (meshobj, iPortalContainer);
+    light_info = scfQueryInterface<iLightingInfo> (meshobj);
+    portal_container = scfQueryInterface<iPortalContainer> (meshobj);
     // Only if we have a parent can it possibly be useful to call
     // AddToSectorPortalLists. Because if we don't have a parent yet then
     // we cannot have a sector either. If we have a parent then the parent
@@ -146,7 +145,7 @@ csMeshWrapper::csMeshWrapper (iMeshObject *meshobj)
   }
   factory = 0;
   zbufMode = CS_ZBUF_USE;
-  imposter_active = false;
+  //imposter_active = false;
   imposter_mesh = 0;
   cast_hardware_shadow = true;
   draw_after_fancy_stuff = false;
@@ -161,11 +160,17 @@ csMeshWrapper::csMeshWrapper (iMeshObject *meshobj)
 
   last_camera = 0;
   last_frame_number = 0;
+
+  // Set creation time on the mesh
+  csRef<csShaderVariable> sv_creation_time;
+  sv_creation_time.AttachNew(new csShaderVariable(engine->id_creation_time));
+  sv_creation_time->SetValue((float)engine->virtualClock->GetCurrentTicks() / 1000.0f);
+  GetSVContext()->AddVariable(sv_creation_time);
 }
 
 void csMeshWrapper::SelfDestruct ()
 {
-  csEngine::currentEngine->GetMeshes ()->Remove ((iMeshWrapper*)this);
+  engine->GetMeshes ()->Remove (static_cast<iMeshWrapper*> (this));
 }
 
 iShadowReceiver* csMeshWrapper::GetShadowReceiver ()
@@ -182,7 +187,7 @@ iShadowReceiver* csMeshWrapper::GetShadowReceiver ()
 
     if (!meshobj) return 0;
     shadow_receiver_valid = true;
-    shadow_receiver = SCF_QUERY_INTERFACE (meshobj, iShadowReceiver);
+    shadow_receiver = scfQueryInterface<iShadowReceiver> (meshobj);
   }
   return shadow_receiver;
 }
@@ -201,7 +206,7 @@ iShadowCaster* csMeshWrapper::GetShadowCaster ()
 
     if (!meshobj) return 0;
     shadow_caster_valid = true;
-    shadow_caster = SCF_QUERY_INTERFACE (meshobj, iShadowCaster);
+    shadow_caster = scfQueryInterface<iShadowCaster> (meshobj);
   }
   return shadow_caster;
 }
@@ -218,7 +223,7 @@ void csMeshWrapper::AddToSectorPortalLists ()
     for (i = 0; i < sectors->GetCount (); i++)
     {
       iSector *ss = sectors->Get (i);
-      if (ss) ss->RegisterPortalMesh ((iMeshWrapper*)this);
+      if (ss) ((csSector*)ss)->RegisterPortalMesh ((iMeshWrapper*)this);
     }
   }
 }
@@ -230,7 +235,7 @@ void csMeshWrapper::ClearFromSectorPortalLists (iSector* sector)
     int i;
     if (sector)
     {
-      sector->UnregisterPortalMesh ((iMeshWrapper*)this);
+      ((csSector*)sector)->UnregisterPortalMesh ((iMeshWrapper*)this);
     }
     else
     {
@@ -242,7 +247,7 @@ void csMeshWrapper::ClearFromSectorPortalLists (iSector* sector)
       for (i = 0; i < sectors->GetCount (); i++)
       {
         iSector *ss = sectors->Get (i);
-        if (ss) ss->UnregisterPortalMesh ((iMeshWrapper*)this);
+        if (ss) ((csSector*)ss)->UnregisterPortalMesh ((iMeshWrapper*)this);
       }
     }
   }
@@ -260,8 +265,8 @@ void csMeshWrapper::SetMeshObject (iMeshObject *meshobj)
 
   if (meshobj)
   {
-    light_info = SCF_QUERY_INTERFACE (meshobj, iLightingInfo);
-    portal_container = SCF_QUERY_INTERFACE (meshobj, iPortalContainer);
+    light_info = scfQueryInterface<iLightingInfo> (meshobj);
+    portal_container = scfQueryInterface<iPortalContainer> (meshobj);
     AddToSectorPortalLists ();
   }
   else
@@ -276,7 +281,7 @@ csMeshWrapper::~csMeshWrapper ()
   // Copy the array because we are going to unlink the children.
   csRefArray<iSceneNode> children = movable.GetChildren ();
   size_t i;
-  for (i = 0 ; i < children.Length () ; i++)
+  for (i = 0 ; i < children.GetSize () ; i++)
     children[i]->SetParent (0);
   delete imposter_mesh;
   ClearFromSectorPortalLists ();
@@ -311,14 +316,15 @@ void csMeshWrapper::MoveToSector (iSector *s)
   if (!movable.GetParent ()) s->GetMeshes ()->Add ((iMeshWrapper*)this);
   // If we are a portal container then we have to register ourselves
   // to the sector.
-  if (portal_container) s->RegisterPortalMesh ((iMeshWrapper*)this);
+  if (portal_container)
+    ((csSector*)s)->RegisterPortalMesh ((iMeshWrapper*)this);
 
   // Fire the new mesh callbacks in the sector.
   ((csSector*)s)->FireNewMesh ((iMeshWrapper*)this);
 
   const csRefArray<iSceneNode>& children = movable.GetChildren ();
   size_t i;
-  for (i = 0; i < children.Length (); i++)
+  for (i = 0; i < children.GetSize (); i++)
   {
     iMeshWrapper* spr = children[i]->QueryMesh ();
     // @@@ What for other types of objects like lights and camera???
@@ -346,7 +352,7 @@ void csMeshWrapper::RemoveFromSectors (iSector* sector)
   ClearFromSectorPortalLists (sector);
   const csRefArray<iSceneNode>& children = movable.GetChildren ();
   size_t i;
-  for (i = 0; i < children.Length (); i++)
+  for (i = 0; i < children.GetSize (); i++)
   {
     iMeshWrapper* spr = children[i]->QueryMesh ();
     // @@@ What to do in case of light!
@@ -386,7 +392,7 @@ void csMeshWrapper::SetFlagsRecursive (uint32 mask, uint32 value)
   flags.Set (mask, value);
   const csRefArray<iSceneNode>& children = movable.GetChildren ();
   size_t i;
-  for (i = 0 ; i < children.Length () ; i++)
+  for (i = 0 ; i < children.GetSize () ; i++)
   {
     iMeshWrapper* mesh = children[i]->QueryMesh ();
     if (mesh)
@@ -399,7 +405,7 @@ void csMeshWrapper::SetZBufModeRecursive (csZBufMode mode)
   SetZBufMode (mode);
   const csRefArray<iSceneNode>& children = movable.GetChildren ();
   size_t i;
-  for (i = 0 ; i < children.Length () ; i++)
+  for (i = 0 ; i < children.GetSize () ; i++)
   {
     iMeshWrapper* mesh = children[i]->QueryMesh ();
     if (mesh)
@@ -412,7 +418,7 @@ void csMeshWrapper::SetRenderPriorityRecursive (long rp)
   SetRenderPriority (rp);
   const csRefArray<iSceneNode>& children = movable.GetChildren ();
   size_t i;
-  for (i = 0 ; i < children.Length () ; i++)
+  for (i = 0 ; i < children.GetSize () ; i++)
   {
     iMeshWrapper* mesh = children[i]->QueryMesh ();
     if (mesh)
@@ -457,7 +463,8 @@ static int compare_light (const void *p1, const void *p2)
   return 0;
 }
 
-const csArray<iLightSectorInfluence*>& csMeshWrapper::GetRelevantLights (
+const csArray<iLightSectorInfluence*>& csMeshWrapper::GetRelevantLights 
+(
     	int maxLights, bool desireSorting)
 {
   bool always_update = relevant_lights_flags.Check (
@@ -470,7 +477,7 @@ const csArray<iLightSectorInfluence*>& csMeshWrapper::GetRelevantLights (
       // Object didn't move. Now check lights (moved or destroyed).
       bool relevant = true;
       size_t i;
-      for (i = 0 ; i < relevant_lights.Length () ; i++)
+      for (i = 0 ; i < relevant_lights.GetSize () ; i++)
       {
 	if (!relevant_lights_ref[i].lsi)
 	{
@@ -500,8 +507,8 @@ const csArray<iLightSectorInfluence*>& csMeshWrapper::GetRelevantLights (
     csBox3 box;
     GetFullBBox (box);
 
-    if (relevant_lights_max > relevant_lights.Length ())
-      relevant_lights.SetLength (relevant_lights_max);
+    if (relevant_lights_max > relevant_lights.GetSize ())
+      relevant_lights.SetSize (relevant_lights_max);
 
     iSector *sect = movable_sectors->Get (0);
     csVector3 pos = movable.GetFullPosition ();
@@ -557,8 +564,8 @@ const csArray<iLightSectorInfluence*>& csMeshWrapper::GetRelevantLights (
     }
     if (cnt > relevant_lights_max) cnt = relevant_lights_max;
     if (maxLights != -1 && int (cnt) > maxLights) cnt = maxLights;
-    relevant_lights.SetLength (cnt);
-    relevant_lights_ref.SetLength (cnt);
+    relevant_lights.SetSize (cnt);
+    relevant_lights_ref.SetSize (cnt);
     size_t i;
     for (i = 0 ; i < cnt ; i++)
     {
@@ -582,13 +589,24 @@ const csArray<iLightSectorInfluence*>& csMeshWrapper::GetRelevantLights (
 csRenderMesh** csMeshWrapper::GetRenderMeshes (int& n, iRenderView* rview, 
 					       uint32 frustum_mask)
 {
-  //if (imposter_active && CheckImposterRelevant (rview))
-    //if (DrawImposter (rview))
-      //return;
+  if (factory)
+  {
+    csMeshFactoryWrapper* factwrap = static_cast<csMeshFactoryWrapper*> (
+  	factory);
+    if (factwrap->imposter_active && CheckImposterRelevant (rview))
+    {
+      csRenderMesh** imposter = GetImposter (rview);
+      if (imposter)
+      {
+        n = 1;
+        return imposter;
+      }
+    }
+  }
 
   // Callback are traversed in reverse order so that they can safely
   // delete themselves.
-  size_t i = draw_cb_vector.Length ();
+  size_t i = draw_cb_vector.GetSize ();
   while (i > 0)
   {
     i--;
@@ -626,9 +644,10 @@ csRenderMesh** csMeshWrapper::GetRenderMeshes (int& n, iRenderView* rview,
     csrview->SetCsRenderContext (ctxt);
   }
 
-  csTicks lt = csEngine::currentEngine->GetLastAnimationTime ();
-  meshobj->NextFrame (lt, movable.GetPosition ());
-    
+  csTicks lt = engine->GetLastAnimationTime ();
+  meshobj->NextFrame (lt, movable.GetPosition (), 
+    rview->GetCurrentFrameNumber ());
+
   csMeshWrapper *meshwrap = this;
   last_anim_time = lt;
   csMeshWrapper* lastparent = meshwrap;
@@ -655,6 +674,97 @@ csRenderMesh** csMeshWrapper::GetRenderMeshes (int& n, iRenderView* rview,
   return rmeshes;
 }
 
+void csMeshWrapper::AddExtraRenderMesh(csRenderMesh* renderMesh, long priority,
+        csZBufMode zBufMode)
+{
+  ExtraRenderMeshData data;
+  extraRenderMeshes.Push(renderMesh);
+
+  data.priority = priority;
+  data.zBufMode = zBufMode;
+  extraRenderMeshData.Push(data);
+}
+
+csRenderMesh** csMeshWrapper::GetExtraRenderMeshes (size_t& num, 
+                    iRenderView* rview, uint32 frustum_mask)
+{
+  // Here we check the CS_ENTITY_NOCLIP flag. If that flag is set
+  // we will only render the object once in a give frame/camera combination.
+  // So if multiple portals arrive in a sector containing this object the
+  // object will be rendered at the first portal and not clipped to that
+  // portal (as is usually the case).
+  csRenderContext* old_ctxt = 0;
+
+  if (flags.Check (CS_ENTITY_NOCLIP))
+  {
+    csRenderView* csrview = (csRenderView*)rview;
+    csRenderContext* ctxt = csrview->GetCsRenderContext ();
+
+    if (last_frame_number == rview->GetCurrentFrameNumber () &&
+    	last_camera == ctxt->icamera)
+    {
+      num = 0;
+      return 0;
+    }
+    last_camera = ctxt->icamera;
+    last_frame_number = rview->GetCurrentFrameNumber ();
+    old_ctxt = ctxt;
+    // Go back to top-level context.
+    while (ctxt->previous) ctxt = ctxt->previous;
+    csrview->SetCsRenderContext (ctxt);
+  }
+
+  int clip_portal, clip_plane, clip_z_plane;
+  CS::RenderViewClipper::CalculateClipSettings (rview->GetRenderContext (),
+      frustum_mask, clip_portal, clip_plane, clip_z_plane);
+
+  iCamera* pCamera = rview->GetCamera();
+  const csReversibleTransform& o2wt = movable.GetFullTransform();
+  const csVector3& wo = o2wt.GetOrigin();
+  num = extraRenderMeshes.GetSize();
+  for (size_t a = 0; a < num; a++)
+  {
+    extraRenderMeshes[a]->clip_portal = clip_portal;
+    extraRenderMeshes[a]->clip_plane = clip_plane;
+    extraRenderMeshes[a]->clip_z_plane = clip_z_plane;
+    extraRenderMeshes[a]->do_mirror = pCamera->IsMirrored();
+    extraRenderMeshes[a]->worldspace_origin = wo;
+    extraRenderMeshes[a]->object2world = o2wt;
+  }
+
+  return extraRenderMeshes.GetArray();
+}
+
+long csMeshWrapper::GetExtraRenderMeshPriority(size_t idx) const
+{
+    return extraRenderMeshData[idx].priority;
+}
+
+csZBufMode csMeshWrapper::GetExtraRenderMeshZBufMode(size_t idx) const
+{
+    return extraRenderMeshData[idx].zBufMode;
+}
+
+void csMeshWrapper::RemoveExtraRenderMesh(csRenderMesh* renderMesh)
+{
+    size_t len = extraRenderMeshes.GetSize();
+    for (size_t a=0; a<len; ++a)
+    {
+        if (extraRenderMeshes[a] != renderMesh)
+            continue;
+
+        // copy last value in list over top of the one we're removing
+        extraRenderMeshes[a] = extraRenderMeshes[len-1];
+        extraRenderMeshData[a] = extraRenderMeshData[len-1];
+
+        // remove the last one
+        extraRenderMeshes.DeleteIndexFast(len-1);
+        extraRenderMeshData.DeleteIndexFast(len-1);
+
+        return;
+    }
+}
+
 //----- Min/Max Distance Range ----------------------------------------------
 
 void csMeshWrapper::ClearMinVariable ()
@@ -739,6 +849,10 @@ void csMeshWrapper::SetParent (iSceneNode* parent)
 
   ClearFromSectorPortalLists ();
 
+  // If we are setting a parent then we clear the sectors of this object.
+  if (parent)
+    movable.ClearSectors ();
+
 #if 0
   if (!movable.GetParent ())
   {
@@ -830,41 +944,29 @@ void csMeshWrapper::AddMeshToStaticLOD (int lod, iMeshWrapper* mesh)
 
 bool csMeshWrapper::CheckImposterRelevant (iRenderView *rview)
 {
+  if (!factory) return false;
+  csMeshFactoryWrapper* factwrap = static_cast<csMeshFactoryWrapper*> (
+  	factory);
   float wor_sq_dist = GetSquaredDistance (rview);
-  float dist = min_imposter_distance->Get ();
+  float dist = factwrap->min_imposter_distance->Get ();
   return (wor_sq_dist > dist*dist);
 }
 
-bool csMeshWrapper::DrawImposter (iRenderView *rview)
+csRenderMesh** csMeshWrapper::GetImposter (iRenderView *rview)
 {
-  // Check for imposter existence.  If not, create it.
-  if (!imposter_mesh)
-  {
-    return false;
-  }
+  csMeshFactoryWrapper* factwrap = static_cast<csMeshFactoryWrapper*> (
+  	factory);
+  csImposterFactory* imposter_factory = factwrap->GetImposterFactory ();
+  if (!imposter_factory) return 0;
 
-  // Check for imposter already ready
-  if (!imposter_mesh->GetImposterReady ())
-    return false;
+  imposter_mesh = imposter_factory->GetImposterMesh (this,
+      imposter_mesh, rview);
+  if (!imposter_mesh) return 0;
+  if (!imposter_mesh->GetImposterReady (rview))
+    return 0;
 
-  // Check for too much camera movement since last imposter render
-  if (!imposter_mesh->CheckIncidenceAngle (rview,
-	imposter_rotation_tolerance->Get ()))
-    return false;
-
-  // Else draw imposter as-is.
-  imposter_mesh->Draw (rview);
-  return true;
-}
-
-void csMeshWrapper::SetImposterActive (bool flag)
-{
-  imposter_active = flag;
-  if (flag)
-  {
-    imposter_mesh = new csImposterMesh (this);
-    imposter_mesh->SetImposterReady (false);
-  }
+  // Get imposter rendermesh
+  return imposter_mesh->GetRenderMesh (rview);
 }
 
 csHitBeamResult csMeshWrapper::HitBeamOutline (
@@ -876,14 +978,6 @@ csHitBeamResult csMeshWrapper::HitBeamOutline (
   return rc;
 }
 
-bool csMeshWrapper::HitBeamOutline (
-  const csVector3 &start,
-  const csVector3 &end,
-  csVector3 &isect,
-  float *pr)
-{
-  return meshobj->HitBeamOutline (start, end, isect, pr);
-}
 
 csHitBeamResult csMeshWrapper::HitBeamObject (
   const csVector3 &start,
@@ -897,30 +991,14 @@ csHitBeamResult csMeshWrapper::HitBeamObject (
   return rc;
 }
 
-bool csMeshWrapper::HitBeamObject (
-  const csVector3 &start,
-  const csVector3 &end,
-  csVector3 &isect,
-  float *pr, int* polygon_idx)
-{
-  return meshobj->HitBeamObject (start, end, isect, pr, polygon_idx);
-}
 
 csHitBeamResult csMeshWrapper::HitBeam (
   const csVector3 &start,
-  const csVector3 &end)
+  const csVector3 &end,
+  bool do_material)
 {
   csHitBeamResult rc;
-  rc.hit = HitBeam (start, end, rc.isect, &rc.r);
-  return rc;
-}
 
-bool csMeshWrapper::HitBeam (
-  const csVector3 &start,
-  const csVector3 &end,
-  csVector3 &isect,
-  float *pr)
-{
   csVector3 startObj;
   csVector3 endObj;
   csReversibleTransform trans;
@@ -935,14 +1013,22 @@ bool csMeshWrapper::HitBeam (
     startObj = trans.Other2This (start);
     endObj = trans.Other2This (end);
   }
-  bool rc = false;
-  if (HitBeamBBox (startObj, endObj, isect, 0) > -1)
+
+  if (HitBeamBBox (startObj, endObj).facehit > -1)
   {
-    rc = HitBeamOutline (startObj, endObj, isect, pr);
-    if (rc)
+    if (do_material)
+    {
+      rc.hit = meshobj->HitBeamObject (startObj, endObj, rc.isect, &rc.r, 0, &rc.material);
+    }
+    else
+    {
+      rc = HitBeamOutline (startObj, endObj);
+    }
+
+    if (rc.hit)
     {
       if (!movable.IsFullTransformIdentity ())
-        isect = trans.This2Other (isect);
+        rc.isect = trans.This2Other (rc.isect);
     }
   }
 
@@ -955,7 +1041,7 @@ void csMeshWrapper::HardTransform (const csReversibleTransform &t)
 
   const csRefArray<iSceneNode>& children = movable.GetChildren ();
   size_t i;
-  for (i = 0 ; i < children.Length () ; i++)
+  for (i = 0 ; i < children.GetSize () ; i++)
   {
     iMeshWrapper* mesh = children[i]->QueryMesh ();
     if (mesh)
@@ -963,70 +1049,59 @@ void csMeshWrapper::HardTransform (const csReversibleTransform &t)
   }
 }
 
-csEllipsoid csMeshWrapper::GetRadius () const
+csSphere csMeshWrapper::GetRadius () const
 {
-  csEllipsoid e;
-  GetRadius (e.GetRadius (), e.GetCenter ());
-  return e;
-}
+  csVector3 cent;
+  float rad;
 
-void csMeshWrapper::GetRadius (csVector3 &rad, csVector3 &cent) const
-{
   meshobj->GetObjectModel ()->GetRadius (rad, cent);
   const csRefArray<iSceneNode>& children = movable.GetChildren ();
-  if (children.Length () > 0)
-  {
-    float max_radius = rad.x;
-    if (max_radius < rad.y) max_radius = rad.y;
-    if (max_radius < rad.z) max_radius = rad.z;
 
-    csSphere sphere (cent, max_radius);
+  csSphere sphere (cent, rad);
+
+  if (children.GetSize () > 0)
+  {    
     size_t i;
-    for (i = 0; i < children.Length (); i++)
+    for (i = 0; i < children.GetSize (); i++)
     {
       iMeshWrapper *spr = children[i]->QueryMesh ();
       if (spr)
       {
-        csVector3 childrad, childcent;
-        spr->GetRadius (childrad, childcent);
-
-        float child_max_radius = childrad.x;
-        if (child_max_radius < childrad.y) child_max_radius = childrad.y;
-        if (child_max_radius < childrad.z) child_max_radius = childrad.z;
-
-        csSphere childsphere (childcent, child_max_radius);
+        csSphere childsphere = spr->GetRadius ();
 
         // @@@ Is this the right transform?
         childsphere *= spr->GetMovable ()->GetTransform ();
         sphere += childsphere;
       }
     }
-
-    rad.Set (sphere.GetRadius (), sphere.GetRadius (), sphere.GetRadius ());
-    cent.Set (sphere.GetCenter ());
   }
+
+  return sphere;
 }
 
 float csMeshWrapper::GetSquaredDistance (iRenderView *rview)
 {
-  iCamera* camera = rview->GetCamera ();
-  // calculate distance from camera to mesh
-  csBox3 obox;
-  GetObjectModel ()->GetObjectBoundingBox (obox);
+  csVector3 cam_origin = rview->GetCamera ()->GetTransform ().GetOrigin ();
+  return GetSquaredDistance (cam_origin);
+}
+
+float csMeshWrapper::GetSquaredDistance (const csVector3& pos)
+{
+  // calculate distance from pos to mesh
+  const csBox3& obox = GetObjectModel ()->GetObjectBoundingBox ();
   csVector3 obj_center = (obox.Min () + obox.Max ()) / 2;
   csVector3 wor_center;
   if (movable.IsFullTransformIdentity ())
     wor_center = obj_center;
   else
     wor_center = movable.GetFullTransform ().This2Other (obj_center);
-  csVector3 cam_origin = camera->GetTransform ().GetOrigin ();
-  float wor_sq_dist = csSquaredDist::PointPoint (cam_origin, wor_center);
+  float wor_sq_dist = csSquaredDist::PointPoint (pos, wor_center);
   return wor_sq_dist;
 }
 
 void csMeshWrapper::GetFullBBox (csBox3& box)
 {
-  GetObjectModel ()->GetObjectBoundingBox (box);
+  box = GetObjectModel ()->GetObjectBoundingBox ();
   csMovable* mov = &movable;
   while (mov)
   {
@@ -1053,24 +1128,21 @@ void csMeshWrapper::PlaceMesh ()
   iSectorList *movable_sectors = movable.GetSectors ();
   if (movable_sectors->GetCount () == 0) return ; // Do nothing
   csSphere sphere;
-  csVector3 radius;
+  float radius;
   GetObjectModel ()->GetRadius (radius, sphere.GetCenter ());
 
   iSector *sector = movable_sectors->Get (0);
   movable.SetSector (sector);       // Make sure all other sectors are removed
 
   // Transform the sphere from object to world space.
-  float max_radius = radius.x;
-  if (max_radius < radius.y) max_radius = radius.y;
-  if (max_radius < radius.z) max_radius = radius.z;
-  sphere.SetRadius (max_radius);
+  sphere.SetRadius (radius);
   if (!movable.IsFullTransformIdentity ())
     sphere = movable.GetFullTransform ().This2Other (sphere);
-  max_radius = sphere.GetRadius ();
-  float max_sq_radius = max_radius * max_radius;
+  radius = sphere.GetRadius ();
+  float max_sq_radius = radius * radius;
 
-  csRef<iMeshWrapperIterator> it = csEngine::currentEngine
-  	->GetNearbyMeshes (sector, sphere.GetCenter (), max_radius, true);
+  csRef<iMeshWrapperIterator> it = 
+    engine->GetNearbyMeshes (sector, sphere.GetCenter (), radius, true);
 
   int j;
   while (it->HasNext ())
@@ -1109,26 +1181,13 @@ csHitBeamResult csMeshWrapper::HitBeamBBox (
   const csVector3 &end)
 {
   csHitBeamResult rc;
-  csBox3 b;
-  GetObjectModel ()->GetObjectBoundingBox (b);
+  const csBox3& b = GetObjectModel ()->GetObjectBoundingBox ();
 
   csSegment3 seg (start, end);
   rc.facehit = csIntersect3::BoxSegment (b, seg, rc.isect, &rc.r);
   return rc;
 }
 
-int csMeshWrapper::HitBeamBBox (
-  const csVector3 &start,
-  const csVector3 &end,
-  csVector3 &isect,
-  float *pr)
-{
-  csBox3 b;
-  GetObjectModel ()->GetObjectBoundingBox (b);
-
-  csSegment3 seg (start, end);
-  return csIntersect3::BoxSegment (b, seg, isect, pr);
-}
 
 const csBox3& csMeshWrapper::GetWorldBoundingBox ()
 {
@@ -1137,11 +1196,10 @@ const csBox3& csMeshWrapper::GetWorldBoundingBox ()
     wor_bbox_movablenr = movable.GetUpdateNumber ();
 
     if (movable.IsFullTransformIdentity ())
-      GetObjectModel ()->GetObjectBoundingBox (wor_bbox);
+      wor_bbox = GetObjectModel ()->GetObjectBoundingBox ();
     else
     {
-      csBox3 obj_bbox;
-      GetObjectModel ()->GetObjectBoundingBox (obj_bbox);
+      const csBox3& obj_bbox = GetObjectModel ()->GetObjectBoundingBox ();
 
       // @@@ Maybe it would be better to really calculate the bounding box
       // here instead of just transforming the object space bounding box?
@@ -1159,16 +1217,11 @@ const csBox3& csMeshWrapper::GetWorldBoundingBox ()
   return wor_bbox;
 }
 
-void csMeshWrapper::GetWorldBoundingBox (csBox3 &cbox)
-{
-  cbox = GetWorldBoundingBox ();
-}
-
 csBox3 csMeshWrapper::GetTransformedBoundingBox (
   const csReversibleTransform &trans)
 {
-  csBox3 cbox, box;
-  GetObjectModel ()->GetObjectBoundingBox (box);
+  csBox3 cbox;
+  const csBox3& box = GetObjectModel ()->GetObjectBoundingBox ();
   cbox.StartBoundingBox (trans * box.GetCorner (0));
   cbox.AddBoundingVertexSmart (trans * box.GetCorner (1));
   cbox.AddBoundingVertexSmart (trans * box.GetCorner (2));
@@ -1180,60 +1233,50 @@ csBox3 csMeshWrapper::GetTransformedBoundingBox (
   return cbox;
 }
 
-void csMeshWrapper::GetTransformedBoundingBox (
-  const csReversibleTransform &trans,
-  csBox3 &cbox)
-{
-  cbox = GetTransformedBoundingBox (trans);
-}
 
 csScreenBoxResult csMeshWrapper::GetScreenBoundingBox (iCamera *camera)
 {
   csScreenBoxResult rc;
-  rc.distance = GetScreenBoundingBox (camera, rc.sbox, rc.cbox);
-  return rc;
-}
 
-float csMeshWrapper::GetScreenBoundingBox (
-  iCamera *camera,
-  csBox2 &sbox,
-  csBox3 &cbox)
-{
   csVector2 oneCorner;
   csReversibleTransform tr_o2c = camera->GetTransform ();
   if (!movable.IsFullTransformIdentity ())
     tr_o2c /= movable.GetFullTransform ();
-  GetTransformedBoundingBox (tr_o2c, cbox);
+  
+  rc.cbox = GetTransformedBoundingBox (tr_o2c);
 
   // if the entire bounding box is behind the camera, we're done
-  if ((cbox.MinZ () < 0) && (cbox.MaxZ () < 0))
+  if ((rc.cbox.MinZ () < 0) && (rc.cbox.MaxZ () < 0))
   {
-    return -1;
+    rc.distance = -1;
+    return rc;
   }
 
   // Transform from camera to screen space.
-  if (cbox.MinZ () <= 0)
+  if (rc.cbox.MinZ () <= 0)
   {
     // Mesh is very close to camera.
     // Just return a maximum bounding box.
-    sbox.Set (-10000, -10000, 10000, 10000);
+    rc.sbox.Set (-10000, -10000, 10000, 10000);
   }
   else
   {
-    camera->Perspective (cbox.Max (), oneCorner);
-    sbox.StartBoundingBox (oneCorner);
+    oneCorner = camera->Perspective (rc.cbox.Max ());
+    rc.sbox.StartBoundingBox (oneCorner);
 
-    csVector3 v (cbox.MinX (), cbox.MinY (), cbox.MaxZ ());
-    camera->Perspective (v, oneCorner);
-    sbox.AddBoundingVertexSmart (oneCorner);
-    camera->Perspective (cbox.Min (), oneCorner);
-    sbox.AddBoundingVertexSmart (oneCorner);
-    v.Set (cbox.MaxX (), cbox.MaxY (), cbox.MinZ ());
-    camera->Perspective (v, oneCorner);
-    sbox.AddBoundingVertexSmart (oneCorner);
+    csVector3 v (rc.cbox.MinX (), rc.cbox.MinY (), rc.cbox.MaxZ ());
+    oneCorner = camera->Perspective (v);
+    rc.sbox.AddBoundingVertexSmart (oneCorner);
+    oneCorner = camera->Perspective (rc.cbox.Min ());
+    rc.sbox.AddBoundingVertexSmart (oneCorner);
+    v.Set (rc.cbox.MaxX (), rc.cbox.MaxY (), rc.cbox.MinZ ());
+    oneCorner = camera->Perspective (v);
+    rc.sbox.AddBoundingVertexSmart (oneCorner);
   }
 
-  return cbox.MaxZ ();
+  rc.distance = rc.cbox.MaxZ ();
+  
+  return rc;
 }
 
 iMeshWrapper* csMeshWrapper::FindChildByName (const char* name)
@@ -1244,7 +1287,7 @@ iMeshWrapper* csMeshWrapper::FindChildByName (const char* name)
   char const* p = strchr (name, ':');
   if (!p)
   {
-    for (i = 0 ; i < children.Length () ; i++)
+    for (i = 0 ; i < children.GetSize () ; i++)
     {
       iMeshWrapper* m = children[i]->QueryMesh ();
       if (m && !strcmp (name, m->QueryObject ()->GetName ()))
@@ -1257,7 +1300,7 @@ iMeshWrapper* csMeshWrapper::FindChildByName (const char* name)
   csString firstName;
   firstName.Append (name, firstsize);
 
-  for (i = 0 ; i < children.Length () ; i++)
+  for (i = 0 ; i < children.GetSize () ; i++)
   {
     iMeshWrapper* m = children[i]->QueryMesh ();
     if (m && !strcmp (firstName, m->QueryObject ()->GetName ()))
@@ -1268,191 +1311,6 @@ iMeshWrapper* csMeshWrapper::FindChildByName (const char* name)
   return 0;
 }
 
-// ---------------------------------------------------------------------------
-// csMeshFactoryWrapper
-// ---------------------------------------------------------------------------
-
-
-csMeshFactoryWrapper::csMeshFactoryWrapper (iMeshObjectFactory *meshFact)
-  : scfImplementationType (this), meshFact (meshFact), parent (0),
-  zbufMode (CS_ZBUF_USE)
-{
-  children.SetMeshFactory (this);
-
-  render_priority = csEngine::currentEngine->GetObjectRenderPriority ();
-}
-
-csMeshFactoryWrapper::csMeshFactoryWrapper ()
-  : scfImplementationType (this), parent (0), zbufMode (CS_ZBUF_USE)
-{
-  children.SetMeshFactory (this);
-
-  render_priority = csEngine::currentEngine->GetObjectRenderPriority ();
-}
-
-csMeshFactoryWrapper::~csMeshFactoryWrapper ()
-{
-  // This line MUST be here to ensure that the children are not
-  // removed after the destructor has already finished.
-  children.RemoveAll ();
-}
-
-void csMeshFactoryWrapper::SelfDestruct ()
-{
-  csEngine::currentEngine->GetMeshFactories ()->Remove (
-  	(iMeshFactoryWrapper*)this);
-}
-
-void csMeshFactoryWrapper::SetZBufModeRecursive (csZBufMode mode)
-{
-  SetZBufMode (mode);
-  const iMeshFactoryList* ml = &children;
-  if (!ml) return;
-  int i;
-  for (i = 0 ; i < ml->GetCount () ; i++)
-    ml->Get (i)->SetZBufModeRecursive (mode);
-}
-
-void csMeshFactoryWrapper::SetRenderPriorityRecursive (long rp)
-{
-  SetRenderPriority (rp);
-  const iMeshFactoryList* ml = &children;
-  if (!ml) return;
-  int i;
-  for (i = 0 ; i < ml->GetCount () ; i++)
-    ml->Get (i)->SetRenderPriorityRecursive (rp);
-}
-
-void csMeshFactoryWrapper::SetRenderPriority (long rp)
-{
-  render_priority = rp;
-}
-
-void csMeshFactoryWrapper::SetMeshObjectFactory (iMeshObjectFactory *meshFact)
-{
-  csMeshFactoryWrapper::meshFact = meshFact;
-}
-
-iMeshWrapper *csMeshFactoryWrapper::CreateMeshWrapper ()
-{
-  csRef<iMeshObject> basemesh = meshFact->NewInstance ();
-  iMeshWrapper *mesh = (iMeshWrapper*)(new csMeshWrapper (basemesh));
-  basemesh->SetMeshWrapper (mesh);
-
-  if (GetName ()) mesh->QueryObject ()->SetName (GetName ());
-  mesh->SetFactory (this);
-  mesh->SetRenderPriority (render_priority);
-  mesh->SetZBufMode (zbufMode);
-
-  if (static_lod)
-  {
-    iLODControl* lod = mesh->CreateStaticLOD ();
-    iSharedVariable* varm, * vara;
-    static_lod->GetLOD (varm, vara);
-    if (varm)
-    {
-      lod->SetLOD (varm, vara);
-    }
-    else
-    {
-      float m, a;
-      static_lod->GetLOD (m, a);
-      lod->SetLOD (m, a);
-    }
-  }
-
-  int i;
-  for (i = 0; i < children.GetCount (); i++)
-  {
-    iMeshFactoryWrapper *childfact = children.Get (i);
-    iMeshWrapper *child = childfact->CreateMeshWrapper ();
-    child->QuerySceneNode ()->SetParent (mesh->QuerySceneNode ());
-    child->GetMovable ()->SetTransform (childfact->GetTransform ());
-    child->GetMovable ()->UpdateMove ();
-
-    if (static_lod)
-    {
-      // We have static lod so we need to put the child in the right
-      // lod level.
-      int l;
-      for (l = 0 ; l < static_lod->GetLODCount () ; l++)
-      {
-        csArray<iMeshFactoryWrapper*>& facts_for_lod =
-      	  static_lod->GetMeshesForLOD (l);
-        size_t j;
-	for (j = 0 ; j < facts_for_lod.Length () ; j++)
-	{
-	  if (facts_for_lod[j] == childfact)
-	    mesh->AddMeshToStaticLOD (l, child);
-	}
-      }
-    }
-
-    child->DecRef ();
-  }
-
-  return mesh;
-}
-
-void csMeshFactoryWrapper::HardTransform (const csReversibleTransform &t)
-{
-  meshFact->HardTransform (t);
-}
-
-iLODControl* csMeshFactoryWrapper::CreateStaticLOD ()
-{
-  static_lod = csPtr<csStaticLODFactoryMesh> (new csStaticLODFactoryMesh ());
-  return static_lod;
-}
-
-void csMeshFactoryWrapper::DestroyStaticLOD ()
-{
-  static_lod = 0;
-}
-
-iLODControl* csMeshFactoryWrapper::GetStaticLOD ()
-{
-  return (iLODControl*)static_lod;
-}
-
-void csMeshFactoryWrapper::SetStaticLOD (float m, float a)
-{
-  if (static_lod) static_lod->SetLOD (m, a);
-}
-
-void csMeshFactoryWrapper::GetStaticLOD (float& m, float& a) const
-{
-  if (static_lod)
-    static_lod->GetLOD (m, a);
-  else
-  {
-    m = 0;
-    a = 0;
-  }
-}
-
-void csMeshFactoryWrapper::RemoveFactoryFromStaticLOD (
-	iMeshFactoryWrapper* fact)
-{
-  if (!static_lod) return;	// No static lod, nothing to do here.
-  int lod;
-  for (lod = 0 ; lod < static_lod->GetLODCount () ; lod++)
-  {
-    csArray<iMeshFactoryWrapper*>& meshes_for_lod =
-    	static_lod->GetMeshesForLOD (lod);
-    meshes_for_lod.Delete (fact);
-  }
-}
-
-void csMeshFactoryWrapper::AddFactoryToStaticLOD (int lod,
-	iMeshFactoryWrapper* fact)
-{
-  if (!static_lod) return;	// No static lod, nothing to do here.
-  csArray<iMeshFactoryWrapper*>& meshes_for_lod =
-  	static_lod->GetMeshesForLOD (lod);
-  meshes_for_lod.Push (fact);
-}
-
 //--------------------------------------------------------------------------
 // csMeshList
 //--------------------------------------------------------------------------
@@ -1471,7 +1329,7 @@ csMeshList::~csMeshList ()
 void csMeshList::NameChanged (iObject* object, const char* oldname,
   	const char* newname)
 {
-  csRef<iMeshWrapper> mesh = SCF_QUERY_INTERFACE (object, iMeshWrapper);
+  csRef<iMeshWrapper> mesh = scfQueryInterface<iMeshWrapper> (object);
   CS_ASSERT (mesh != 0);
   if (oldname) meshes_hash.Delete (oldname, mesh);
   if (newname) meshes_hash.Put (newname, mesh);
@@ -1486,7 +1344,7 @@ iMeshWrapper* csMeshList::FindByNameWithChild (const char *Name) const
   csString firstName;
   firstName.Append (Name, firstsize);
 
-  iMeshWrapper* m = meshes_hash.Get (csStrKey (firstName), 0);
+  iMeshWrapper* m = meshes_hash.Get (firstName, 0);
   if (!m) return 0;
   return m->FindChildByName (p+1);
 }
@@ -1527,7 +1385,7 @@ bool csMeshList::Remove (int n)
 void csMeshList::RemoveAll ()
 {
   size_t i;
-  for (i = 0 ; i < list.Length () ; i++)
+  for (i = 0 ; i < list.GetSize () ; i++)
   {
     list[i]->QueryObject ()->RemoveNameChangeListener (listener);
     FreeMesh (list[i]);
@@ -1616,99 +1474,3 @@ void csMeshMeshList::FreeMesh (iMeshWrapper* item)
 }
 #endif
 
-//--------------------------------------------------------------------------
-// csMeshFactoryList
-//--------------------------------------------------------------------------
-csMeshFactoryList::csMeshFactoryList ()
-  : scfImplementationType (this), list (64, 64)
-{
-  listener.AttachNew (new NameChangeListener (this));
-}
-
-csMeshFactoryList::~csMeshFactoryList ()
-{
-  RemoveAll ();
-}
-
-void csMeshFactoryList::NameChanged (iObject* object, const char* oldname,
-  	const char* newname)
-{
-  csRef<iMeshFactoryWrapper> mesh = SCF_QUERY_INTERFACE (object,
-    iMeshFactoryWrapper);
-  CS_ASSERT (mesh != 0);
-  if (oldname) factories_hash.Delete (oldname, mesh);
-  if (newname) factories_hash.Put (newname, mesh);
-}
-
-int csMeshFactoryList::Add (iMeshFactoryWrapper *obj)
-{
-  PrepareFactory (obj);
-  const char* name = obj->QueryObject ()->GetName ();
-  if (name)
-    factories_hash.Put (name, obj);
-  obj->QueryObject ()->AddNameChangeListener (listener);
-  return (int)list.Push (obj);
-}
-
-bool csMeshFactoryList::Remove (iMeshFactoryWrapper *obj)
-{
-  FreeFactory (obj);
-  const char* name = obj->QueryObject ()->GetName ();
-  if (name)
-    factories_hash.Delete (name, obj);
-  list.Delete (obj);
-  obj->QueryObject ()->RemoveNameChangeListener (listener);
-  return true;
-}
-
-bool csMeshFactoryList::Remove (int n)
-{
-  return Remove (Get (n));
-}
-
-void csMeshFactoryList::RemoveAll ()
-{
-  size_t i;
-  for (i = 0 ; i < list.Length () ; i++)
-  {
-    list[i]->QueryObject ()->RemoveNameChangeListener (listener);
-    FreeFactory (list[i]);
-  }
-  factories_hash.DeleteAll ();
-  list.DeleteAll ();
-}
-
-int csMeshFactoryList::Find (iMeshFactoryWrapper *obj) const
-{
-  return (int)list.Find (obj);
-}
-
-iMeshFactoryWrapper *csMeshFactoryList::FindByName (
-  const char *Name) const
-{
-  if (!Name) return 0;
-  return factories_hash.Get (Name, 0);
-}
-
-//--------------------------------------------------------------------------
-// csMeshFactoryFactoryList
-//--------------------------------------------------------------------------
-void csMeshFactoryFactoryList::PrepareFactory (iMeshFactoryWrapper* child)
-{
-  CS_ASSERT (meshfact != 0);
-  csMeshFactoryList::PrepareFactory (child);
-
-  // unlink the factory from another possible parent.
-  if (child->GetParentContainer ())
-    child->GetParentContainer ()->GetChildren ()->Remove (child);
-
-  child->SetParentContainer (meshfact);
-}
-
-void csMeshFactoryFactoryList::FreeFactory (iMeshFactoryWrapper* item)
-{
-  CS_ASSERT (meshfact != 0);
-  item->SetParentContainer (0);
-  meshfact->RemoveFactoryFromStaticLOD (item);
-  csMeshFactoryList::FreeFactory (item);
-}
