@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000-2006 by Jorrit Tyberghein
+    Copyright (C) 2000-2007 by Jorrit Tyberghein
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -31,11 +31,6 @@
 #include "csutil/hash.h"
 #include "iutil/selfdestruct.h"
 #include "csgfx/shadervarcontext.h"
-#include "plugins/engine/3d/movable.h"
-#include "plugins/engine/3d/impmesh.h"
-#include "plugins/engine/3d/meshlod.h"
-#include "plugins/engine/3d/scenenode.h"
-#include "plugins/engine/3d/light.h"
 #include "imesh/object.h"
 #include "imesh/lighting.h"
 #include "iengine/mesh.h"
@@ -45,13 +40,19 @@
 #include "ivideo/graph3d.h"
 #include "ivideo/shader/shader.h"
 
+#include "movable.h"
+#include "impmesh.h"
+#include "meshlod.h"
+#include "scenenode.h"
+#include "light.h"
+
 struct iMeshWrapper;
-struct iRenderView;
 struct iMovable;
+struct iRenderView;
 struct iSharedVariable;
-class csMeshWrapper;
-class csMeshFactoryWrapper;
+class csEngine;
 class csLight;
+class csMeshWrapper;
 
 /**
  * General list of meshes.
@@ -60,7 +61,7 @@ class csMeshList : public scfImplementation1<csMeshList, iMeshList>
 {
 private:
   csRefArrayObject<iMeshWrapper> list;
-  csHash<iMeshWrapper*,csStrKey> meshes_hash;
+  csHash<iMeshWrapper*, csString> meshes_hash;
 
   class NameChangeListener : public scfImplementation1<NameChangeListener,
   	iObjectNameChangeListener>
@@ -101,7 +102,7 @@ public:
   /// Override FreeMesh
   virtual void FreeMesh (iMeshWrapper*) { }
 
-  virtual int GetCount () const { return (int)list.Length () ; }
+  virtual int GetCount () const { return (int)list.GetSize () ; }
   virtual iMeshWrapper *Get (int n) const { return list.Get (n); }
   virtual int Add (iMeshWrapper *obj);
   virtual bool Remove (iMeshWrapper *obj);
@@ -111,77 +112,6 @@ public:
   virtual iMeshWrapper *FindByName (const char *Name) const;
 };
 
-/**
- * A list of mesh factories.
- */
-class csMeshFactoryList : public scfImplementation1<csMeshFactoryList,
-	iMeshFactoryList>
-{
-private:
-  csRefArrayObject<iMeshFactoryWrapper> list;
-  csHash<iMeshFactoryWrapper*,csStrKey>
-  	factories_hash;
-
-  class NameChangeListener : public scfImplementation1<NameChangeListener,
-  	iObjectNameChangeListener>
-  {
-  private:
-    csWeakRef<csMeshFactoryList> list;
-
-  public:
-    NameChangeListener (csMeshFactoryList* list) : scfImplementationType (this),
-  	  list (list)
-    {
-    }
-    virtual ~NameChangeListener () { }
-
-    virtual void NameChanged (iObject* obj, const char* oldname,
-  	  const char* newname)
-    {
-      if (list)
-        list->NameChanged (obj, oldname, newname);
-    }
-  };
-  csRef<NameChangeListener> listener;
-
-
-public:
-
-  void NameChanged (iObject* object, const char* oldname,
-  	const char* newname);
-
-  /// constructor
-  csMeshFactoryList ();
-  virtual ~csMeshFactoryList ();
-  virtual void PrepareFactory (iMeshFactoryWrapper*) { }
-  virtual void FreeFactory (iMeshFactoryWrapper*) { }
-
-  virtual int GetCount () const { return (int)list.Length (); }
-  virtual iMeshFactoryWrapper *Get (int n) const { return list.Get (n); }
-  virtual int Add (iMeshFactoryWrapper *obj);
-  virtual bool Remove (iMeshFactoryWrapper *obj);
-  virtual bool Remove (int n);
-  virtual void RemoveAll ();
-  virtual int Find (iMeshFactoryWrapper *obj) const;
-  virtual iMeshFactoryWrapper *FindByName (const char *Name) const;
-};
-
-/**
- * Subclass of csMeshFactoryList to hold the children of another mesh factory.
- */
-class csMeshFactoryFactoryList : public csMeshFactoryList
-{
-private:
-  csMeshFactoryWrapper* meshfact;
-
-public:
-  csMeshFactoryFactoryList () : meshfact (0) {}
-  virtual ~csMeshFactoryFactoryList () { RemoveAll (); }
-  void SetMeshFactory (csMeshFactoryWrapper* m) { meshfact = m; }
-  virtual void PrepareFactory (iMeshFactoryWrapper* item);
-  virtual void FreeFactory (iMeshFactoryWrapper* item);
-};
-
 struct LSIAndDist
 {
   csLightSectorInfluence* lsi;
@@ -190,15 +120,22 @@ struct LSIAndDist
   float influence;
 };
 
+struct ExtraRenderMeshData
+{
+    long            priority;
+    csZBufMode      zBufMode;
+};
+
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 /**
  * The holder class for all implementations of iMeshObject.
  */
-class csMeshWrapper : public scfImplementationExt6<csMeshWrapper,
+class csMeshWrapper : public scfImplementationExt5<csMeshWrapper,
                                                    csObject,
                                                    iMeshWrapper,
                                                    iShaderVariableContext,
                                                    iVisibilityObject,
-                                                   iImposter,
 						   iSceneNode,
 						   iSelfDestruct>
 {
@@ -237,6 +174,11 @@ protected:
   /// For NR: should we draw last
   bool draw_after_fancy_stuff;
 
+  // Used to store extra rendermeshes that something might attach to this
+  // mesh (ie, for decals)
+  csDirtyAccessArray<csRenderMesh*> extraRenderMeshes;
+  csArray<ExtraRenderMeshData> extraRenderMeshData;
+
   /**
    * This value indicates the last time that was used to do animation.
    * If 0 then we haven't done animation yet. We compare this value
@@ -263,6 +205,7 @@ protected:
   csShaderVariableContext svcontext;
   csRef<iShaderVariableContext> factorySVC;
 
+  csEngine* engine;
 private:
   /// Mesh object corresponding with this csMeshWrapper.
   csRef<iMeshObject> meshobj;
@@ -301,12 +244,6 @@ private:
   /// Z-buf mode to use for drawing this object.
   csZBufMode zbufMode;
 
-  /// Flag indicating whether this mesh should try to imposter or not.
-  bool imposter_active;
-  /// Imposter Threshold Range.
-  csRef<iSharedVariable> min_imposter_distance;
-  /// Imposter Redo Threshold angle change.
-  csRef<iSharedVariable> imposter_rotation_tolerance;
   csImposterMesh *imposter_mesh;
 
   /**
@@ -383,7 +320,7 @@ protected:
 
 public:
   /// Constructor.
-  csMeshWrapper (iMeshObject* meshobj = 0);
+  csMeshWrapper (csEngine* engine, iMeshObject* meshobj = 0);
 
   /// Set the mesh factory.
   virtual void SetFactory (iMeshFactoryWrapper* factory)
@@ -476,7 +413,7 @@ public:
 
   virtual int GetDrawCallbackCount () const
   {
-    return (int)draw_cb_vector.Length ();
+    return (int)draw_cb_vector.GetSize ();
   }
 
   virtual iMeshDrawCallback* GetDrawCallback (int idx) const
@@ -516,8 +453,37 @@ public:
    * Draw the zpass for the object.  If this object doesn't use lighting
    * then it can be drawn fully here.
    */
-  csRenderMesh** GetRenderMeshes (int& num, iRenderView* rview,
+  virtual csRenderMesh** GetRenderMeshes (int& num, iRenderView* rview,
   	uint32 frustum_mask);
+  /**
+   * Adds a render mesh to the list of extra render meshes.
+   * This list is used for special cases (like decals) where additional
+   * things need to be renderered for the mesh in an abstract way.
+   */
+  virtual void AddExtraRenderMesh(csRenderMesh* renderMesh, long priority, 
+          csZBufMode zBufMode);
+
+  /**
+   * Grabs any additional render meshes this mesh might have on top
+   * of the normal rendermeshes through GetRenderMeshes.
+   */
+  virtual csRenderMesh** GetExtraRenderMeshes (size_t& num, iRenderView* rview,
+    uint32 frustum_mask);
+
+  /** 
+   * Gets the priority of a specific extra rendermesh.
+   */
+  virtual long GetExtraRenderMeshPriority(size_t idx) const;
+
+  /**
+   * Gets the z-buffer mode of a specific extra rendermesh
+   */
+  virtual csZBufMode GetExtraRenderMeshZBufMode(size_t idx) const;
+
+  /**
+   * Deletes a specific extra rendermesh
+   */
+  virtual void RemoveExtraRenderMesh(csRenderMesh* renderMesh);
 
   /**
    * Do a hard transform of this object.
@@ -530,69 +496,18 @@ public:
    */
   virtual void HardTransform (const csReversibleTransform& t);
 
-  //---------- iImposter Functions -----------------//
-
-  /// Set true if this Mesh should use Impostering.
-  virtual void SetImposterActive (bool flag);
-
-  /**
-   * Determine if this mesh is using Impostering
-   * (not if Imposter is being drawn, but simply considered).
-   */
-  virtual bool GetImposterActive () const { return imposter_active; }
-
-  /**
-   * Minimum Imposter Distance is the distance from camera 
-   * beyond which imposter is used. Imposter gets a 
-   * ptr here because value is a shared variable 
-   * which can be changed at runtime for many objects.
-   */
-  virtual void SetMinDistance (iSharedVariable* dist)
-  { min_imposter_distance = dist; }
-
-  /** 
-   * Rotation Tolerance is the maximum allowable 
-   * angle difference between when the imposter was 
-   * created and the current position of the camera.
-   * Angle greater than this triggers a re-render of
-   * the imposter.
-   */
-  virtual void SetRotationTolerance (iSharedVariable* angle)
-  { imposter_rotation_tolerance = angle; }
-
-  /**
-   * Tells the object to create its proctex and polygon
-   * for use by main render process later, relative to
-   * the specified Point Of View.
-   */
-  virtual void CreateImposter (csReversibleTransform& /*pov*/)
-  { /* implement later */ }
-
-  /**
-   * Renders the imposter on the screen.
-   */
-  bool DrawImposter (iRenderView *rview);
-
-  /// Determine if imposter or true rendering will be used.
-  virtual bool WouldUseImposter (csReversibleTransform& /*pov*/) const
-  { /* implement later */ return false; }
-
   /// This is the function to check distances.  Fn above may not be needed.
   bool CheckImposterRelevant (iRenderView *rview);
-  
+
+  /**
+   * Gets the imposters rendermesh
+   */
+  csRenderMesh** GetImposter (iRenderView *rview);
+
   //---------- Bounding volume and beam functions -----------------//
 
-  virtual void GetRadius (csVector3& rad, csVector3& cent) const;
-  virtual csEllipsoid GetRadius () const;
-
-  virtual int HitBeamBBox (const csVector3& start, const csVector3& end,
-         csVector3& isect, float* pr);
-  virtual bool HitBeamOutline (const csVector3& start, const csVector3& end,
-         csVector3& isect, float* pr);
-  virtual bool HitBeamObject (const csVector3& start, const csVector3& end,
-  	csVector3& isect, float* pr, int* polygon_idx = 0);
-  virtual bool HitBeam (const csVector3& start, const csVector3& end,
-  	csVector3& isect, float* pr);
+  virtual csSphere GetRadius () const;
+  
   virtual csHitBeamResult HitBeamBBox (const csVector3& start,
   	const csVector3& end);
   virtual csHitBeamResult HitBeamOutline (const csVector3& start,
@@ -600,12 +515,16 @@ public:
   virtual csHitBeamResult HitBeamObject (const csVector3& start,
   	const csVector3& end, bool do_material = false);
   virtual csHitBeamResult HitBeam (const csVector3& start,
-  	const csVector3& end);
+  	const csVector3& end, bool do_material = false);
 
   /**
    * Calculate the squared distance between the camera and the object.
    */
   float GetSquaredDistance (iRenderView *rview);
+  /**
+   * Calculate the squared distance between a position and the object.
+   */
+  float GetSquaredDistance (const csVector3& pos);
 
   /**
    * This routine will find out in which sectors a mesh object
@@ -621,18 +540,13 @@ public:
    */
   virtual void PlaceMesh ();
 
-  virtual void GetWorldBoundingBox (csBox3& cbox);
-  virtual void GetTransformedBoundingBox (const csReversibleTransform& trans,
-  	csBox3& cbox);
-  virtual float GetScreenBoundingBox (iCamera *camera, csBox2& sbox,
-  	csBox3& cbox);
   virtual const csBox3& GetWorldBoundingBox ();
   virtual csBox3 GetTransformedBoundingBox (const csReversibleTransform& trans);
   virtual csScreenBoxResult GetScreenBoundingBox (iCamera *camera);
 
   //--------------------- SCF stuff follows ------------------------------//
-  //=================== iShaderVariableContext ================//
-
+  /**\name iShaderVariableContext implementation
+   * @{ */
   /// Add a variable to this context
   void AddVariable (csShaderVariable *variable)
   { svcontext.AddVariable (variable); }
@@ -657,7 +571,7 @@ public:
    * Push the variables of this context onto the variable stacks
    * supplied in the "stacks" argument
    */
-  void PushVariables (csShaderVarStack &stacks) const
+  void PushVariables (iShaderVarStack* stacks) const
   { 
     if (factorySVC.IsValid()) factorySVC->PushVariables (stacks);
     svcontext.PushVariables (stacks); 
@@ -672,13 +586,20 @@ public:
   void ReplaceVariable (csShaderVariable *variable)
   { svcontext.ReplaceVariable (variable); }
   void Clear () { svcontext.Clear(); }
+  
+  bool RemoveVariable (csShaderVariable* variable)
+  { 
+    // @@@ Also remove from factory?
+    return svcontext.RemoveVariable (variable); 
+  }
+  /** @} */
 
   //--------------------- iSelfDestruct implementation -------------------//
 
   virtual void SelfDestruct ();
 
-  //--------------------- iSceneNode implementation ----------------------//
-
+  /**\name iSceneNode implementation
+   * @{ */
   virtual void SetParent (iSceneNode* parent);
   virtual iSceneNode* GetParent () const
   {
@@ -694,6 +615,13 @@ public:
   virtual iMeshWrapper* QueryMesh () { return this; }
   virtual iLight* QueryLight () { return 0; }
   virtual iCamera* QueryCamera () { return 0; }
+  virtual csPtr<iSceneNodeArray> GetChildrenArray () const
+  {
+    return csPtr<iSceneNodeArray> (
+      new scfArrayWrapConst<iSceneNodeArray, csRefArray<iSceneNode> > (
+      movable.GetChildren ()));
+  }
+  /** @} */
 
   //--------------------- iMeshWrapper implementation --------------------//
 
@@ -739,152 +667,6 @@ public:
   }
 };
 
-/**
- * The holder class for all implementations of iMeshObjectFactory.
- */
-class csMeshFactoryWrapper : public scfImplementationExt3<csMeshFactoryWrapper,
-                                                          csObject, 
-                                                          iMeshFactoryWrapper,
-                                                          iShaderVariableContext,
-							  iSelfDestruct>
-{
-private:
-  /// Mesh object factory corresponding with this csMeshFactoryWrapper.
-  csRef<iMeshObjectFactory> meshFact;
-
-  /// Optional parent of this object (can be 0).
-  iMeshFactoryWrapper* parent;
-
-  /// Optional relative transform to parent.
-  csReversibleTransform transform;
-
-  /// Children of this object (other instances of iMeshFactoryWrapper).
-  csMeshFactoryFactoryList children;
-
-  /**
-   * Optional LOD control that will turn a hierarchical mesh in a
-   * mesh that supports static LOD.
-   */
-  csRef<csStaticLODFactoryMesh> static_lod;
-
-  /// Suggestion for new children created from factory.
-  long render_priority;
-  /// Suggestion for new children created from factory.
-  csZBufMode zbufMode;
-
-  csShaderVariableContext svcontext;
-
-public:
-  /// Constructor.
-  csMeshFactoryWrapper (iMeshObjectFactory* meshFact);
-  /// Constructor.
-  csMeshFactoryWrapper ();
-  /// Destructor.
-  virtual ~csMeshFactoryWrapper ();
-
-  CS_LEAKGUARD_DECLARE (csMeshFactoryWrapper);
-
-  /// Set the mesh object factory.
-  void SetMeshObjectFactory (iMeshObjectFactory* meshFact);
-
-  /// Get the mesh object factory.
-  iMeshObjectFactory* GetMeshObjectFactory () const
-  {
-    return meshFact;
-  }
-
-  /**
-   * Create a new mesh object for this template.
-   */
-  iMeshWrapper* CreateMeshWrapper ();
-
-  /**
-   * Do a hard transform of this factory.
-   * This transformation and the original coordinates are not
-   * remembered but the object space coordinates are directly
-   * computed (world space coordinates are set to the object space
-   * coordinates by this routine). Note that some implementations
-   * of mesh objects will not change the orientation of the object but
-   * only the position.
-   */
-  void HardTransform (const csReversibleTransform& t);
-
-  /// This function is called for every child that is added
-  void PrepareChild (iMeshFactoryWrapper* child);
-  /// This function is called for every child that is removed
-  void UnprepareChild (iMeshFactoryWrapper* child);
-
-  /**
-   * Get optional relative transform (relative to parent).
-   */
-  csReversibleTransform& GetTransform () { return transform; }
-
-  /**
-   * Set optional relative transform (relative to parent).
-   */
-  void SetTransform (const csReversibleTransform& tr) { transform = tr; }
-
-  virtual iMeshFactoryWrapper* GetParentContainer () const { return parent; }
-  virtual void SetParentContainer (iMeshFactoryWrapper *p) { parent = p; }
-  virtual iMeshFactoryList* GetChildren (){ return &children; }
-
-  virtual iObject* QueryObject () { return this; }
-
-  virtual iShaderVariableContext* GetSVContext()
-  { return CS_STATIC_CAST(iShaderVariableContext*, this); }
-
-  // Static LOD methods.
-  iLODControl* CreateStaticLOD ();
-  void DestroyStaticLOD ();
-  iLODControl* GetStaticLOD ();
-  void SetStaticLOD (float m, float a);
-  void GetStaticLOD (float& m, float& a) const;
-  void RemoveFactoryFromStaticLOD (iMeshFactoryWrapper* mesh);
-  void AddFactoryToStaticLOD (int lod, iMeshFactoryWrapper* mesh);
-
-  // Flags that are used for children.
-  void SetZBufMode (csZBufMode mode) { zbufMode = mode; }
-  csZBufMode GetZBufMode () const { return zbufMode; }
-  void SetZBufModeRecursive (csZBufMode mode);
-  void SetRenderPriority (long rp);
-  long GetRenderPriority () const
-  {
-    return render_priority;
-  }
-  void SetRenderPriorityRecursive (long rp);
-
-  //--------------------- iSelfDestruct implementation -------------------//
-
-  virtual void SelfDestruct ();
-
-  //=================== iShaderVariableContext ================//
-
-  /// Add a variable to this context
-  void AddVariable (csShaderVariable *variable)
-  { svcontext.AddVariable (variable); }
-
-  /// Get a named variable from this context
-  csShaderVariable* GetVariable (csStringID name) const
-  { return svcontext.GetVariable (name); }
-
-  /// Get Array of all ShaderVariables
-  const csRefArray<csShaderVariable>& GetShaderVariables () const
-  { return svcontext.GetShaderVariables (); }
-
-  /**
-   * Push the variables of this context onto the variable stacks
-   * supplied in the "stacks" argument
-   */
-  void PushVariables (csShaderVarStack &stacks) const
-  { svcontext.PushVariables (stacks); }
-
-  bool IsEmpty () const 
-  { return svcontext.IsEmpty(); }
-
-  void ReplaceVariable (csShaderVariable *variable)
-  { svcontext.ReplaceVariable (variable); }
-  void Clear () { svcontext.Clear(); }
-
-};
+#include "csutil/win32/msvc_deprecated_warn_on.h"
 
 #endif // __CS_MESHOBJ_H__
