@@ -171,7 +171,7 @@ public:
   }
   virtual int GetPortalCallbackCount () const
   {
-    return (int)portal_cb_vector.Length ();
+    return (int)portal_cb_vector.GetSize ();
   }
   virtual iPortalCallback* GetPortalCallback (int idx) const;
 
@@ -183,7 +183,7 @@ public:
   }
   virtual int GetMissingSectorCallbackCount () const
   {
-    return (int)sector_cb_vector.Length ();
+    return (int)sector_cb_vector.GetSize ();
   }
   virtual iPortalCallback* GetMissingSectorCallback (int idx) const;
 
@@ -277,7 +277,7 @@ public:
    */
   virtual iMeshWrapper* HitBeamPortals (const csReversibleTransform& t,
 	const csVector3& start, const csVector3& end,
-  	csVector3& isect, int* polygon_idx);
+  	csVector3& isect, int* polygon_idx, iSector** final_sector = 0);
 
   /**
    * Check if the destination sector is 0 and if so call
