@@ -22,7 +22,7 @@
 #include "csgeom/poly3d.h"
 #include "csgeom/poly2d.h"
 #include "csgeom/transfrm.h"
-#include "igeom/objmodel.h"
+#include "imesh/objmodel.h"
 #include "iengine/movable.h"
 #include "iengine/rview.h"
 #include "iengine/camera.h"
@@ -30,6 +30,70 @@
 #include "plugins/engine/3d/portalcontainer.h"
 #include "plugins/engine/3d/rview.h"
 #include "plugins/engine/3d/meshobj.h"
+#include "plugins/engine/3d/engine.h"
+#include "cstool/rviewclipper.h"
+
+// ---------------------------------------------------------------------------
+// csPortalContainerTriMeshHelper
+// ---------------------------------------------------------------------------
+
+void csPortalContainerTriMeshHelper::SetPortalContainer (csPortalContainer* pc)
+{
+  parent = pc;
+  data_nr = pc->GetDataNumber ()-1;
+}
+
+void csPortalContainerTriMeshHelper::Setup ()
+{
+  parent->Prepare ();
+  if (data_nr != parent->GetDataNumber () || !vertices)
+  {
+    data_nr = parent->GetDataNumber ();
+    Cleanup ();
+
+    vertices = parent->GetVertices ();
+    // Count number of needed triangles.
+    num_tri = 0;
+
+    size_t i;
+    const csRefArray<csPortal>& portals = parent->GetPortals ();
+    for (i = 0 ; i < portals.GetSize () ; i++)
+    {
+      csPortal *p = portals[i];
+      if (p->flags.CheckAll (poly_flag))
+        num_tri += p->GetVertexIndicesCount ()-2;
+    }
+
+    if (num_tri)
+    {
+      triangles = new csTriangle[num_tri];
+      num_tri = 0;
+      for (i = 0 ; i < portals.GetSize () ; i++)
+      {
+        csPortal *p = portals[i];
+        if (p->flags.CheckAll (poly_flag))
+        {
+          csDirtyAccessArray<int>& vi = p->GetVertexIndices ();
+          size_t j;
+          for (j = 1 ; j < (size_t)p->GetVertexIndicesCount ()-1 ; j++)
+          {
+	    triangles[num_tri].a = vi[0];
+	    triangles[num_tri].b = vi[j];
+	    triangles[num_tri].c = vi[j+1];
+            num_tri++;
+          }
+        }
+      }
+    }
+  }
+}
+
+void csPortalContainerTriMeshHelper::Cleanup ()
+{
+  vertices = 0;
+  delete[] triangles;
+  triangles = 0;
+}
 
 // ---------------------------------------------------------------------------
 // csPortalContainerPolyMeshHelper
@@ -55,7 +119,7 @@ void csPortalContainerPolyMeshHelper::Setup ()
 
     size_t i;
     const csRefArray<csPortal>& portals = parent->GetPortals ();
-    for (i = 0 ; i < portals.Length () ; i++)
+    for (i = 0 ; i < portals.GetSize () ; i++)
     {
       csPortal *p = portals[i];
       if (p->flags.CheckAll (poly_flag)) num_poly++;
@@ -65,13 +129,13 @@ void csPortalContainerPolyMeshHelper::Setup ()
     {
       polygons = new csMeshedPolygon[num_poly];
       num_poly = 0;
-      for (i = 0 ; i < portals.Length () ; i++)
+      for (i = 0 ; i < portals.GetSize () ; i++)
       {
         csPortal *p = portals[i];
         if (p->flags.CheckAll (poly_flag))
         {
 	  csDirtyAccessArray<int>& vidx = p->GetVertexIndices ();
-          polygons[num_poly].num_vertices = (int)vidx.Length ();
+          polygons[num_poly].num_vertices = (int)vidx.GetSize ();
           polygons[num_poly].vertices = vidx.GetArray ();
           num_poly++;
         }
@@ -96,15 +160,27 @@ void csPortalContainerPolyMeshHelper::Cleanup ()
 
 csPortalContainer::csPortalContainer (iEngine* engine,
 	iObjectRegistry *object_reg) :
-	scfImplementationType (this, engine),
-	scfiPolygonMesh (0),
-	scfiPolygonMeshCD (CS_PORTAL_COLLDET),
-	scfiPolygonMeshLOD (CS_PORTAL_VISCULL)
+	scfImplementationType (this, engine)
 {
-  SetPolygonMeshBase (&scfiPolygonMesh);
-  SetPolygonMeshColldet (&scfiPolygonMeshCD);
-  SetPolygonMeshViscull (&scfiPolygonMeshLOD);
-  SetPolygonMeshShadows (&scfiPolygonMeshLOD);
+  polygonMesh.AttachNew (new csPortalContainerPolyMeshHelper (0));
+  polygonMeshCD.AttachNew (new csPortalContainerPolyMeshHelper (CS_PORTAL_COLLDET));
+  polygonMeshLOD.AttachNew (new csPortalContainerPolyMeshHelper (CS_PORTAL_VISCULL));
+  SetPolygonMeshBase (polygonMesh);
+  SetPolygonMeshColldet (polygonMeshCD);
+  SetPolygonMeshViscull (polygonMeshLOD);
+  SetPolygonMeshShadows (polygonMeshLOD);
+
+  csRef<csPortalContainerTriMeshHelper> trimesh;
+  trimesh.AttachNew (new csPortalContainerTriMeshHelper (0));
+  trimesh->SetPortalContainer (this);
+  csEngine* e = static_cast<csEngine*> (engine);
+  SetTriangleData (e->base_id, trimesh);
+  trimesh.AttachNew (new csPortalContainerTriMeshHelper (CS_PORTAL_COLLDET));
+  trimesh->SetPortalContainer (this);
+  SetTriangleData (e->colldet_id, trimesh);
+  trimesh.AttachNew (new csPortalContainerTriMeshHelper (CS_PORTAL_VISCULL));
+  trimesh->SetPortalContainer (this);
+  SetTriangleData (e->viscull_id, trimesh);
 
   prepared = false;
   data_nr = 0;
@@ -113,18 +189,16 @@ csPortalContainer::csPortalContainer (iEngine* engine,
 
   meshwrapper = 0;
 
-  scfiPolygonMesh.SetPortalContainer (this);
-  scfiPolygonMeshCD.SetPortalContainer (this);
-  scfiPolygonMeshLOD.SetPortalContainer (this);
+  polygonMesh->SetPortalContainer (this);
+  polygonMeshCD->SetPortalContainer (this);
+  polygonMeshLOD->SetPortalContainer (this);
 
-  shader_man = CS_QUERY_REGISTRY (object_reg, iShaderManager);
+  shader_man = csQueryRegistry<iShaderManager> (object_reg);
   fog_shader = shader_man->GetShader ("std_lighting_portal");
 
-  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg,
-    "crystalspace.shared.stringset", iStringSet);
-  fogplane_name = strings->Request ("fogplane");
-  fogdensity_name = strings->Request ("fog density");
-  fogcolor_name = strings->Request ("fog color");
+  fogplane_name = e->globalStringSet->Request ("fogplane");
+  fogdensity_name = e->globalStringSet->Request ("fog density");
+  fogcolor_name = e->globalStringSet->Request ("fog color");
 }
 
 csPortalContainer::~csPortalContainer ()
@@ -169,13 +243,13 @@ void csPortalContainer::Prepare ()
   data_nr++;
   csCompressVertex* vt = csVector3Array::CompressVertices (vertices);
   size_t i;
-  for (i = 0 ; i < portals.Length () ; i++)
+  for (i = 0 ; i < portals.GetSize () ; i++)
   {
     csPortal* prt = portals[i];
     size_t j;
     csArray<int>& vidx = prt->GetVertexIndices ();
     csPoly3D poly;
-    for (j = 0 ; j < vidx.Length () ; j++)
+    for (j = 0 ; j < vidx.GetSize () ; j++)
     {
       if (vt) vidx[j] = (int)vt[vidx[j]].new_idx;
       poly.AddVertex (vertices[vidx[j]]);
@@ -184,11 +258,10 @@ void csPortalContainer::Prepare ()
   }
   delete[] vt;
   object_bbox.StartBoundingBox ();
-  for (i = 0 ; i < vertices.Length () ; i++)
+  for (i = 0 ; i < vertices.GetSize () ; i++)
     object_bbox.AddBoundingVertex (vertices[i]);
 
-  object_radius = object_bbox.Max () - object_bbox.GetCenter ();
-  max_object_radius = csQsqrt (csSquaredDist::PointPoint (
+  object_radius = csQsqrt (csSquaredDist::PointPoint (
   	object_bbox.Max (), object_bbox.Min ())) * 0.5f;
 
 }
@@ -236,11 +309,11 @@ void csPortalContainer::ObjectToWorld (const csMovable& movable,
   movable_nr = movable.GetUpdateNumber ();
   movable_identity = movable.IsFullTransformIdentity ();
   size_t i;
-  world_vertices.SetLength (vertices.Length ());
+  world_vertices.SetSize (vertices.GetSize ());
   if (movable_identity)
   {
     world_vertices = vertices;
-    for (i = 0 ; i < portals.Length () ; i++)
+    for (i = 0 ; i < portals.GetSize () ; i++)
     {
       csPortal* prt = portals[i];
       csPlane3 wp (prt->GetIntObjectPlane ());
@@ -249,9 +322,9 @@ void csPortalContainer::ObjectToWorld (const csMovable& movable,
   }
   else
   {
-    for (i = 0 ; i < vertices.Length () ; i++)
+    for (i = 0 ; i < vertices.GetSize () ; i++)
       world_vertices[i] = movtrans.This2Other (vertices[i]);
-    for (i = 0 ; i < portals.Length () ; i++)
+    for (i = 0 ; i < portals.GetSize () ; i++)
     {
       csPortal* prt = portals[i];
       csPlane3 p;
@@ -266,12 +339,12 @@ void csPortalContainer::ObjectToWorld (const csMovable& movable,
 void csPortalContainer::WorldToCamera (iCamera*,
 	const csReversibleTransform& camtrans)
 {
-  camera_vertices.SetLength (world_vertices.Length ());
+  camera_vertices.SetSize (world_vertices.GetSize ());
   size_t i;
-  for (i = 0 ; i < world_vertices.Length () ; i++)
+  for (i = 0 ; i < world_vertices.GetSize () ; i++)
     camera_vertices[i] = camtrans.Other2This (world_vertices[i]);
   camera_planes.Empty ();
-  for (i = 0 ; i < portals.Length () ; i++)
+  for (i = 0 ; i < portals.GetSize () ; i++)
   {
     csPortal* prt = portals[i];
     csPlane3 p;
@@ -305,7 +378,7 @@ bool csPortalContainer::ClipToPlane (
   cnt_vis = 0;
   csPortal* portal = portals[portal_idx];
   csDirtyAccessArray<int>& vt = portal->GetVertexIndices ();
-  num_vertices = (int)vt.Length ();
+  num_vertices = (int)vt.GetSize ();
   for (i = 0; i < num_vertices; i++)
     if (camera_vertices[vt[i]].z >= 0)
     {
@@ -728,8 +801,12 @@ void csPortalContainer::DrawOnePortal (
     keep_camera_z = camera_vertices[po->GetVertexIndices ()[0]].z;
   }
   // First call OpenPortal().
+  csFlags g3d_flags = 0;
+  if (po->flags.Check(CS_PORTAL_FLOAT)) g3d_flags.Set(CS_OPENPORTAL_FLOAT);
+  if (po->flags.Check(CS_PORTAL_ZFILL)) g3d_flags.Set(CS_OPENPORTAL_ZFILL);
+  if (po->flags.Check(CS_PORTAL_MIRROR)) g3d_flags.Set(CS_OPENPORTAL_MIRROR);
   g3d->OpenPortal (poly.GetVertexCount(), poly.GetVertices(),
-      camera_plane, po->flags);
+      camera_plane, g3d_flags);
 
   // Draw through the portal. This can fail.
   // Drawing through a portal can fail because we have reached
@@ -740,10 +817,10 @@ void csPortalContainer::DrawOnePortal (
   {
     csSimpleRenderMesh mesh;
     mesh.meshtype = CS_MESHTYPE_TRIANGLEFAN;
-    mesh.indexCount = (uint)po->GetVertexIndices ().Length ();
+    mesh.indexCount = (uint)po->GetVertexIndices ().GetSize ();
     // @@@ Weirdo overloads approaching, captain!
     mesh.indices = (const uint*)(int*)po->GetVertexIndices ().GetArray ();
-    mesh.vertexCount = (uint)vertices.Length ();
+    mesh.vertexCount = (uint)vertices.GetSize ();
     mesh.vertices = vertices.GetArray ();
     mesh.texcoords = 0;
     mesh.texture = 0;
@@ -755,10 +832,16 @@ void csPortalContainer::DrawOnePortal (
     // @@@ Hackish...
     csShaderVariableContext varContext;
     const csRefArray<csShaderVariable>& globVars = shader_man->GetShaderVariables ();
-    for (uint i = 0; i < globVars.Length (); i++)
+    for (size_t i = 0; i < globVars.GetSize (); i++)
     {
       varContext.AddVariable (globVars[i]);
     }
+    const csRefArray<csShaderVariable>& sectorVars =
+      rview->GetThisSector()->GetSVContext()->GetShaderVariables();
+    for (size_t i = 0; i < sectorVars.GetSize (); i++)
+    {
+      varContext.AddVariable (sectorVars[i]);
+    }
     csVector4 fogPlane;
     iPortal *lastPortal = rview->GetLastPortal();
     if(lastPortal)
@@ -774,13 +857,6 @@ void csPortalContainer::DrawOnePortal (
     }
     varContext.GetVariableAdd (fogplane_name)->SetValue (fogPlane);
 
-    iSector *sector = rview->GetThisSector();
-    varContext.GetVariableAdd (fogdensity_name)->SetValue (sector->GetFog()->density);
-    varContext.GetVariableAdd (fogcolor_name)->SetValue (csVector3 (sector->GetFog()->red,
-      sector->GetFog()->green,
-      sector->GetFog()->blue));
-
-
     mesh.dynDomain = &varContext;
     // @@@ Could be used for z-fill and stuff, while we're at it?
     mesh.z_buf_mode = CS_ZBUF_TEST;
@@ -798,7 +874,7 @@ void csPortalContainer::CastShadows (iMovable* movable, iFrustumView* fview)
   CheckMovable ();
 
   size_t i;
-  for (i = 0 ; i < portals.Length () ; i++)
+  for (i = 0 ; i < portals.GetSize () ; i++)
   {
     csPortal *p = portals[i];
     p->CastShadows (movable, fview);
@@ -824,7 +900,7 @@ bool csPortalContainer::ExtraVisTest (iRenderView* rview,
 
   csSphere sphere, world_sphere;
   sphere.SetCenter (object_bbox.GetCenter ());
-  sphere.SetRadius (max_object_radius);
+  sphere.SetRadius (object_radius);
 
   tr_o2c = camtrans;
   if (!movable_identity)
@@ -840,8 +916,8 @@ bool csPortalContainer::ExtraVisTest (iRenderView* rview,
   csSphere cam_sphere = tr_o2c.Other2This (sphere);
   camera_origin = cam_sphere.GetCenter ();
 
-  return csrview->ClipBSphere (cam_sphere, world_sphere, clip_portal,
-      clip_plane, clip_z_plane);
+  return CS::RenderViewClipper::CullBSphere (csrview->GetRenderContext (),
+      cam_sphere, world_sphere, clip_portal, clip_plane, clip_z_plane);
 }
 
 bool csPortalContainer::Draw (iRenderView* rview, iMovable* /*movable*/,
@@ -875,7 +951,7 @@ bool csPortalContainer::Draw (iRenderView* rview, iMovable* /*movable*/,
   size_t i;
   if (clip_plane || clip_portal || clip_z_plane || do_portal_plane || farplane)
   {
-    for (i = 0 ; i < portals.Length () ; i++)
+    for (i = 0 ; i < portals.GetSize () ; i++)
     {
       csVector3 *verts;
       int num_verts;
@@ -902,12 +978,12 @@ bool csPortalContainer::Draw (iRenderView* rview, iMovable* /*movable*/,
   }
   else
   {
-    for (i = 0 ; i < portals.Length () ; i++)
+    for (i = 0 ; i < portals.GetSize () ; i++)
     {
       csPoly2D poly;
       csPortal* prt = portals[i];
       csDirtyAccessArray<int>& vt = prt->GetVertexIndices ();
-      int num_vertices = (int)vt.Length ();
+      int num_vertices = (int)vt.GetSize ();
       int j;
       for (j = 0 ; j < num_vertices ; j++)
         AddPerspective (&poly, camera_vertices[vt[j]], fov, shift_x, shift_y);
@@ -921,13 +997,13 @@ bool csPortalContainer::Draw (iRenderView* rview, iMovable* /*movable*/,
 void csPortalContainer::HardTransform (const csReversibleTransform& t)
 {
   size_t i;
-  world_vertices.SetLength (vertices.Length ());
-  for (i = 0 ; i < vertices.Length () ; i++)
+  world_vertices.SetSize (vertices.GetSize ());
+  for (i = 0 ; i < vertices.GetSize () ; i++)
   {
     vertices[i] = t.This2Other (vertices[i]);
     world_vertices[i] = vertices[i];
   }
-  for (i = 0 ; i < portals.Length () ; i++)
+  for (i = 0 ; i < portals.GetSize () ; i++)
   {
     csPortal* prt = portals[i];
     csPlane3 new_plane;
@@ -947,7 +1023,7 @@ bool csPortalContainer::HitBeamOutline (const csVector3& start,
 {
   Prepare ();
   size_t i;
-  for (i = 0; i < portals.Length (); i++)
+  for (i = 0; i < portals.GetSize (); i++)
   {
     csPortal *p = portals[i];
     if (p->IntersectSegment (start, end, isect, pr))
@@ -969,7 +1045,7 @@ bool csPortalContainer::HitBeamObject (const csVector3& start,
   float best_r = 2000000000.;
   int best_p = -1;
 
-  for (i = 0; i < portals.Length (); i++)
+  for (i = 0; i < portals.GetSize (); i++)
   {
     csPortal *p = portals[i];
     float r;
@@ -990,7 +1066,7 @@ bool csPortalContainer::HitBeamObject (const csVector3& start,
   return best_p != -1;
 }
 
-void csPortalContainer::GetRadius (csVector3& radius, csVector3& center)
+void csPortalContainer::GetRadius (float& radius, csVector3& center)
 {
   Prepare ();
   center = object_bbox.GetCenter ();
