@@ -21,6 +21,7 @@
 #define __CS_PORTALCONTAINER_H__
 
 #include "csgeom/pmtools.h"
+#include "csgeom/trimeshtools.h"
 #include "csgeom/vector3.h"
 #include "cstool/meshobjtmpl.h"
 #include "cstool/rendermeshholder.h"
@@ -65,7 +66,7 @@ public:
   virtual int GetVertexCount ()
   {
     Setup ();
-    return (int)vertices->Length ();
+    return (int)vertices->GetSize ();
   }
   virtual csVector3* GetVertices ()
   {
@@ -119,6 +120,71 @@ private:
   }
 };
 
+/**
+ * A helper class for iTriangleMesh implementations used by csPortalContainer.
+ */
+class csPortalContainerTriMeshHelper : 
+  public scfImplementation1<csPortalContainerTriMeshHelper,
+                            iTriangleMesh>
+{
+public:
+
+  /**
+   * Make a polygon mesh helper which will accept polygons which match
+   * with the given flag (one of CS_POLY_COLLDET or CS_POLY_VISCULL).
+   */
+  csPortalContainerTriMeshHelper (uint32 flag) 
+    : scfImplementationType (this), vertices (0),
+      poly_flag (flag), triangles (0)
+  {
+  }
+  virtual ~csPortalContainerTriMeshHelper ()
+  {
+    Cleanup ();
+  }
+
+  void Setup ();
+  void SetPortalContainer (csPortalContainer* pc);
+
+  virtual size_t GetVertexCount ()
+  {
+    Setup ();
+    return vertices->GetSize ();
+  }
+  virtual csVector3* GetVertices ()
+  {
+    Setup ();
+    return vertices->GetArray ();
+  }
+  virtual size_t GetTriangleCount ()
+  {
+    Setup ();
+    return num_tri;
+  }
+  virtual csTriangle* GetTriangles ()
+  {
+    Setup ();
+    return triangles;
+  }
+  virtual void Lock () { }
+  virtual void Unlock () { }
+ 
+  virtual csFlags& GetFlags () { return flags;  }
+  virtual uint32 GetChangeNumber() const { return 0; }
+
+  void Cleanup ();
+
+private:
+  csPortalContainer* parent;
+  uint32 data_nr;		// To see if the portal container has changed.
+  // Array of vertices from portal container.
+  csDirtyAccessArray<csVector3>* vertices;
+  size_t num_tri;		// Total number of triangles.
+  uint32 poly_flag;		// Polygons must match with this flag.
+  csFlags flags;
+  csTriangle* triangles;
+};
+
 /**
  * This is a container class for portals.
  */
@@ -136,8 +202,7 @@ private:
   // Object space data.
   csDirtyAccessArray<csVector3> vertices;
   csBox3 object_bbox;
-  csVector3 object_radius;
-  float max_object_radius;
+  float object_radius;
 
   csRef<iShaderManager> shader_man;
   csRef<iShader> fog_shader;
@@ -205,11 +270,11 @@ public:
 
 
   //-------------------- iPolygonMesh interface implementation ---------------
-  csPortalContainerPolyMeshHelper scfiPolygonMesh;
+  csRef<csPortalContainerPolyMeshHelper> polygonMesh;
   //------------------- CD iPolygonMesh implementation ---------------
-  csPortalContainerPolyMeshHelper scfiPolygonMeshCD;
+  csRef<csPortalContainerPolyMeshHelper> polygonMeshCD;
   //------------------- LOD iPolygonMesh implementation ---------------
-  csPortalContainerPolyMeshHelper scfiPolygonMeshLOD;
+  csRef<csPortalContainerPolyMeshHelper> polygonMeshLOD;
 
   //-------------------For iShadowReceiver ----------------------------//
   virtual void CastShadows (iMovable* movable, iFrustumView* fview);
@@ -217,7 +282,7 @@ public:
   //-------------------For iPortalContainer ----------------------------//
   virtual iPortal* CreatePortal (csVector3* vertices, int num);
   virtual void RemovePortal (iPortal* portal);
-  virtual int GetPortalCount () const { return (int)portals.Length () ; }
+  virtual int GetPortalCount () const { return (int)portals.GetSize () ; }
   virtual iPortal* GetPortal (int idx) const { return (iPortal*)portals[idx]; }
   virtual void Draw (iRenderView* rview)
   {
@@ -245,12 +310,17 @@ public:
     Prepare ();
     bbox = object_bbox;
   }
+  virtual const csBox3& GetObjectBoundingBox ()
+  {
+    Prepare ();
+    return object_bbox;
+  }
   virtual void SetObjectBoundingBox (const csBox3& bbox)
   {
     object_bbox = bbox;
     ShapeChanged ();
   }
-  virtual void GetRadius (csVector3& radius, csVector3& center);
+  virtual void GetRadius (float& radius, csVector3& center);
 };
 
 #endif // __CS_PORTALCONTAINER_H__
