@@ -23,7 +23,7 @@
 #include "csgeom/kdtree.h"
 #include "csutil/csppulse.h"
 #include "csutil/csstring.h"
-#include "csutil/debug.h"
+#include "cstool/csview.h"
 #include "iengine/portal.h"
 #include "iengine/rview.h"
 #include "igeom/clip2d.h"
@@ -45,6 +45,7 @@
 #include "plugins/engine/3d/rview.h"
 #include "plugins/engine/3d/sector.h"
 #include "plugins/engine/3d/meshgen.h"
+#include "plugins/engine/3d/meshobj.h"
 
 //---------------------------------------------------------------------------
 csSectorLightList::csSectorLightList ()
@@ -117,11 +118,8 @@ void csSectorMeshList::FreeMesh (iMeshWrapper* item)
 //---------------------------------------------------------------------------
 
 csSector::csSector (csEngine *engine) :
-  scfImplementationType (this)
+  scfImplementationType (this), engine (engine)
 {
-  DG_TYPE (this, "csSector");
-  csSector::engine = engine;
-  fog.enabled = false;
   drawBusy = 0;
   dynamicAmbientLightColor.Set (0,0,0);
   dynamicAmbientLightVersion = (uint)~0;
@@ -131,6 +129,24 @@ csSector::csSector (csEngine *engine) :
   currentVisibilityNumber = 0;
   renderloop = 0;
   use_lightculling = false;
+  single_mesh = 0;
+  relevant_lights_dirty = true;
+
+  SetupSVNames();
+  svDynamicAmbient.AttachNew (new csShaderVariable (SVNames().dynamicAmbient));
+  svDynamicAmbient->SetValue (dynamicAmbientLightColor);
+  AddVariable (svDynamicAmbient);
+  svFogColor.AttachNew (new csShaderVariable (SVNames().fogColor));
+  AddVariable (svFogColor);
+  svFogMode.AttachNew (new csShaderVariable (SVNames().fogMode));
+  AddVariable (svFogMode);
+  svFogStart.AttachNew (new csShaderVariable (SVNames().fogStart));
+  AddVariable (svFogStart);
+  svFogEnd.AttachNew (new csShaderVariable (SVNames().fogEnd));
+  AddVariable (svFogEnd);
+  svFogDensity.AttachNew (new csShaderVariable (SVNames().fogDensity));
+  AddVariable (svFogDensity);
+  UpdateFogSVs();
 }
 
 csSector::~csSector ()
@@ -141,21 +157,21 @@ csSector::~csSector ()
 
 void csSector::SelfDestruct ()
 {
-  csEngine::currentEngine->GetSectors ()->Remove ((iSector*)this);
+  engine->GetSectors ()->Remove ((iSector*)this);
 }
 
 void csSector::RegisterLightToCuller (csLight* light)
 {
   light->UseAsCullingObject ();
-  csRef<iVisibilityObject> vo = SCF_QUERY_INTERFACE (light,
-        iVisibilityObject);
+  csRef<iVisibilityObject> vo = 
+        scfQueryInterface<iVisibilityObject> (light);
   culler->RegisterVisObject (vo);
 }
 
 void csSector::UnregisterLightToCuller (csLight* light)
 {
-  csRef<iVisibilityObject> vo = SCF_QUERY_INTERFACE (light,
-        iVisibilityObject);
+  csRef<iVisibilityObject> vo = 
+        scfQueryInterface<iVisibilityObject> (light);
   culler->UnregisterVisObject (vo);
   light->StopUsingAsCullingObject ();
 }
@@ -197,7 +213,7 @@ void csSector::RemoveLightVisibleCallback (iLightVisibleCallback* cb)
 
 void csSector::FireLightVisibleCallbacks (iLight* light)
 {
-  size_t i = lightVisibleCallbackList.Length ();
+  size_t i = lightVisibleCallbackList.GetSize ();
   while (i > 0)
   {
     i--;
@@ -208,14 +224,14 @@ void csSector::FireLightVisibleCallbacks (iLight* light)
 void csSector::UnlinkObjects ()
 {
   int i;
-  for (i = 0; i < meshes.GetCount (); i++)
+  for (i = meshes.GetCount()-1; i >= 0; i--)
   {
     iMeshWrapper* m = meshes.Get (i);
     iSectorList* sl = m->GetMovable ()->GetSectors ();
     sl->Remove ((iSector*)this);
     m->GetMovable ()->UpdateMove ();
   }
-  for (i = 0; i < lights.GetCount (); i++)
+  for (i = lights.GetCount()-1; i >= 0; i--)
   {
     iLight* l = lights.Get (i);
     iSectorList* sl = l->GetMovable ()->GetSectors ();
@@ -231,14 +247,14 @@ void csSector::RegisterEntireMeshToCuller (iMeshWrapper* mesh)
   csMeshWrapper* cmesh = (csMeshWrapper*)mesh;
   if (cmesh->SomeParentHasStaticLOD ()) return;
 
-  csRef<iVisibilityObject> vo = SCF_QUERY_INTERFACE (mesh,
-        iVisibilityObject);
+  csRef<iVisibilityObject> vo = 
+        scfQueryInterface<iVisibilityObject> (mesh);
   culler->RegisterVisObject (vo);
 
   if (cmesh->GetStaticLODMesh ()) return;
   size_t i;
   const csRefArray<iSceneNode>& ml = cmesh->GetChildren ();
-  for (i = 0 ; i < ml.Length () ; i++)
+  for (i = 0 ; i < ml.GetSize () ; i++)
   {
     iMeshWrapper* child = ml[i]->QueryMesh ();
     if (child)
@@ -251,15 +267,15 @@ void csSector::RegisterMeshToCuller (iMeshWrapper* mesh)
   csMeshWrapper* cmesh = (csMeshWrapper*)mesh;
   if (cmesh->SomeParentHasStaticLOD ()) return;
 
-  csRef<iVisibilityObject> vo = SCF_QUERY_INTERFACE (mesh,
-        iVisibilityObject);
+  csRef<iVisibilityObject> vo = 
+        scfQueryInterface<iVisibilityObject> (mesh);
   culler->RegisterVisObject (vo);
 }
 
 void csSector::UnregisterMeshToCuller (iMeshWrapper* mesh)
 {
-  csRef<iVisibilityObject> vo = SCF_QUERY_INTERFACE (mesh,
-        iVisibilityObject);
+  csRef<iVisibilityObject> vo = 
+        scfQueryInterface<iVisibilityObject> (mesh);
   culler->UnregisterVisObject (vo);
 }
 
@@ -271,7 +287,7 @@ void csSector::PrepareMesh (iMeshWrapper *mesh)
   if (culler) RegisterMeshToCuller (mesh);
   size_t i;
   const csRefArray<iSceneNode>& ml = ((csMeshWrapper*)mesh)->GetChildren ();
-  for (i = 0 ; i < ml.Length () ; i++)
+  for (i = 0 ; i < ml.GetSize () ; i++)
   {
     iMeshWrapper* child = ml[i]->QueryMesh ();
     if (child)
@@ -286,7 +302,7 @@ void csSector::UnprepareMesh (iMeshWrapper *mesh)
   if (culler) UnregisterMeshToCuller (mesh);
   size_t i;
   const csRefArray<iSceneNode>& ml = ((csMeshWrapper*)mesh)->GetChildren ();
-  for (i = 0 ; i < ml.Length () ; i++)
+  for (i = 0 ; i < ml.GetSize () ; i++)
   {
     iMeshWrapper* child = ml[i]->QueryMesh ();
     if (child)
@@ -302,7 +318,7 @@ void csSector::RelinkMesh (iMeshWrapper *mesh)
 
   size_t i;
   const csRefArray<iSceneNode>& ml = ((csMeshWrapper*)mesh)->GetChildren ();
-  for (i = 0 ; i < ml.Length () ; i++)
+  for (i = 0 ; i < ml.GetSize () ; i++)
   {
     iMeshWrapper* child = ml[i]->QueryMesh ();
     if (child)
@@ -310,6 +326,47 @@ void csSector::RelinkMesh (iMeshWrapper *mesh)
   }
 }
 
+void csSector::PrecacheDraw ()
+{
+  GetVisibilityCuller ()->PrecacheCulling ();
+
+  // First calculate the box of all objects in the level.
+  csBox3 box;
+  box.StartBoundingBox ();
+  int i;
+  for (i = 0; i < meshes.GetCount (); i++)
+  {
+    iMeshWrapper* m = meshes.Get (i);
+    const csBox3& mesh_box = m->GetWorldBoundingBox ();
+    box += mesh_box;
+  }
+
+  // Try to position our camera somewhere above the bounding
+  // box of the sector so we see as much as possible.
+  csVector3 pos = box.GetCenter ();
+  pos.y = box.MaxY () + (box.MaxY () - box.MinY ());
+  csVector3 lookat = pos;
+  lookat.y = box.MinY ();
+
+  csRef<iGraphics3D> g3d = csQueryRegistry<iGraphics3D> (
+      engine->objectRegistry);
+  csRef<csView> view;
+  view.AttachNew (new csView (engine, g3d));
+  iGraphics2D* g2d = g3d->GetDriver2D ();
+  view->SetRectangle (0, 0, g2d->GetWidth (), g2d->GetHeight ());
+
+  iCamera* camera = view->GetCamera ();
+  camera->SetSector (this);
+  camera->GetTransform ().SetOrigin (pos);
+  camera->GetTransform ().LookAt (lookat-pos, csVector3 (0, 0, 1));
+
+  // @@@ Ideally we would want to disable visibility culling
+  // here so that all objects are visible.
+  g3d->BeginDraw (CSDRAW_3DGRAPHICS);
+  view->Draw ();
+  g3d->FinishDraw ();
+}
+
 //----------------------------------------------------------------------
 
 bool csSector::SetVisibilityCullerPlugin (const char *plugname,
@@ -329,8 +386,8 @@ bool csSector::SetVisibilityCullerPlugin (const char *plugname,
   culler = 0;
 
   // Load the culler plugin.
-  csRef<iPluginManager> plugmgr = CS_QUERY_REGISTRY (csEngine::objectRegistry,
-  	iPluginManager);
+  csRef<iPluginManager> plugmgr = 
+  	csQueryRegistry<iPluginManager> (engine->objectRegistry);
   culler = CS_LOAD_PLUGIN (plugmgr, plugname, iVisibilityCuller);
 
   if (!culler)
@@ -341,7 +398,7 @@ bool csSector::SetVisibilityCullerPlugin (const char *plugname,
   const char* err = culler->ParseCullerParameters (culler_params);
   if (err)
   {
-    csEngine::currentEngine->Error ("Error loading visibility culler: %s!",
+    engine->Error ("Error loading visibility culler: %s!",
     	err);
     return false;
   }
@@ -407,7 +464,7 @@ public:
     ObjectVisible (cmesh, frustum_mask);
     size_t i;
     const csRefArray<iSceneNode>& children = cmesh->GetChildren ();
-    for (i = 0 ; i < children.Length () ; i++)
+    for (i = 0 ; i < children.GetSize () ; i++)
     {
       iMeshWrapper* child = children[i]->QueryMesh ();
       // @@@ Traverse too in case there are lights/cameras?
@@ -437,7 +494,7 @@ public:
       float lod = static_lod->GetLODValue (distance);
       csArray<iMeshWrapper*>& meshes = static_lod->GetMeshesForLOD (lod);
       size_t i;
-      for (i = 0 ; i < meshes.Length () ; i++)
+      for (i = 0 ; i < meshes.GetSize () ; i++)
         MarkMeshAndChildrenVisible (meshes[i], frustum_mask);
     }
 
@@ -445,15 +502,27 @@ public:
     csRenderMesh** meshes = cmesh->GetRenderMeshes (num, rview, frustum_mask);
     CS_ASSERT(!((num != 0) && (meshes == 0)));
 #ifdef CS_DEBUG
-    int i;
-    for (i = 0 ; i < num ; i++)
+    for (int i = 0 ; i < num ; i++)
       meshes[i]->db_mesh_name = cmesh->GetName ();
 #endif
     if (num > 0)
     {
       privMeshlist->AddRenderMeshes (meshes, num,
       	cmesh->csMeshWrapper::GetRenderPriority (),
-	cmesh->csMeshWrapper::GetZBufMode (), (iMeshWrapper*)cmesh);
+      	cmesh->csMeshWrapper::GetZBufMode (), (iMeshWrapper*)cmesh);
+
+      // get extra render meshes
+      size_t numExtra = 0;
+      csRenderMesh** extraMeshes = cmesh->GetExtraRenderMeshes (numExtra, rview,
+                                            frustum_mask);
+      CS_ASSERT(!((numExtra != 0) && (extraMeshes == 0)));
+      for (size_t i = 0; i < numExtra; ++i)
+      {
+          privMeshlist->AddRenderMeshes (&extraMeshes[i], 1,
+                  cmesh->csMeshWrapper::GetExtraRenderMeshPriority(i),
+                  cmesh->csMeshWrapper::GetExtraRenderMeshZBufMode(i),
+                  (iMeshWrapper*)cmesh);
+      }
     }
   }
 
@@ -467,7 +536,7 @@ public:
     }
     else
     {
-      csRef<iLight> light = SCF_QUERY_INTERFACE (visobj, iLight);
+      csRef<iLight> light = scfQueryInterface<iLight> (visobj);
       if (light)
       {
         csSector* csector = (csSector*)sector;
@@ -509,7 +578,7 @@ csRenderMeshList *csSector::GetVisibleMeshes (iRenderView *rview)
   size_t i;
   uint32 cur_framenr = engine->GetCurrentFrameNumber ();
   uint32 cur_context_id = rview->GetRenderContext ()->context_id;
-  for (i = 0 ; i < visibleMeshCache.Length () ; i++)
+  for (i = 0 ; i < visibleMeshCache.GetSize () ; i++)
   {
     visibleMeshCacheHolder& entry = visibleMeshCache[i];
     if (entry.cachedFrameNumber == cur_framenr &&
@@ -520,7 +589,7 @@ csRenderMeshList *csSector::GetVisibleMeshes (iRenderView *rview)
   }
 
   //try to find a spot to do a new cache
-  for (i = 0 ; i < visibleMeshCache.Length () ; i++)
+  for (i = 0 ; i < visibleMeshCache.GetSize () ; i++)
   {
     visibleMeshCacheHolder& entry = visibleMeshCache[i];
     if (entry.cachedFrameNumber != cur_framenr)
@@ -528,7 +597,11 @@ csRenderMeshList *csSector::GetVisibleMeshes (iRenderView *rview)
       //use this slot
       entry.meshList->Empty ();
       GetVisMeshCb ()->Setup (entry.meshList, rview, (iSector*)this);
-      GetVisibilityCuller()->VisTest (rview, GetVisMeshCb ());
+
+      if (single_mesh)
+	GetVisMeshCb ()->ObjectVisible ((csMeshWrapper*)single_mesh, ~0);
+      else
+        GetVisibilityCuller()->VisTest (rview, GetVisMeshCb ());
 
       entry.cachedFrameNumber = cur_framenr;
       entry.cached_context_id = cur_context_id;
@@ -543,24 +616,27 @@ csRenderMeshList *csSector::GetVisibleMeshes (iRenderView *rview)
   holder.meshList = new csRenderMeshList (engine);
   usedMeshLists.Push (holder.meshList);
   GetVisMeshCb ()->Setup (holder.meshList, rview, (iSector*)this);
-  GetVisibilityCuller()->VisTest (rview, GetVisMeshCb ());
+  if (single_mesh)
+    GetVisMeshCb ()->ObjectVisible ((csMeshWrapper*)single_mesh, ~0);
+  else
+    GetVisibilityCuller()->VisTest (rview, GetVisMeshCb ());
   visibleMeshCache.Push (holder);
   return holder.meshList;
 }
 
 
-iMeshWrapper* csSector::HitBeamPortals (
+csSectorHitBeamResult csSector::HitBeamPortals (
   const csVector3 &start,
-  const csVector3 &end,
-  csVector3 &isect,
-  int* polygon_idx)
+  const csVector3 &end)
 {
-  iMeshWrapper* mesh = 0;
-  int p = IntersectSegment (start, end, isect, 0, false,
-		  &mesh);
+  csSectorHitBeamResult rc;
+  rc.mesh = 0;
+  rc.final_sector = static_cast<iSector*> (this);
+  int p = IntersectSegment (start, end, rc.isect, 0, false,
+		  &rc.mesh);
   if (p != -1)
   {
-    iPortalContainer* portals = mesh->GetPortalContainer ();
+    iPortalContainer* portals = rc.mesh->GetPortalContainer ();
     if (portals)
     {
       // There are portals.
@@ -568,35 +644,33 @@ iMeshWrapper* csSector::HitBeamPortals (
       if (po)
       {
 	drawBusy++;
-	csVector3 new_start = isect;
-	mesh = po->HitBeamPortals (mesh->GetMovable ()->GetFullTransform (),
-		      new_start, end, isect, &p);
+	csVector3 new_start = rc.isect;
+	rc.mesh = po->HitBeamPortals (rc.mesh->GetMovable ()
+		->GetFullTransform (),
+		new_start, end, rc.isect, &p, &rc.final_sector);
 	drawBusy--;
       }
     }
   }
-  if (polygon_idx) *polygon_idx = p;
-  return mesh;
+  rc.polygon_idx = p;
+  return rc;
 }
 
-iMeshWrapper *csSector::HitBeam (
+csSectorHitBeamResult csSector::HitBeam (
   const csVector3 &start,
   const csVector3 &end,
-  csVector3 &isect,
-  int *polygonPtr,
   bool accurate)
 {
   GetVisibilityCuller ();
   float r;
-  iMeshWrapper* mesh = 0;
-  int poly = -1;
-  bool rc = culler->IntersectSegment (start, end, isect, &r, &mesh, &poly,
-  	accurate);
-  if (polygonPtr) *polygonPtr = poly;
-  if (rc)
-    return mesh;
-  else
-    return 0;
+  csSectorHitBeamResult rc;
+  rc.mesh = 0;
+  rc.polygon_idx = -1;
+  rc.final_sector = 0;
+  bool result = culler->IntersectSegment (start, end, rc.isect, &r, &rc.mesh,
+  	&rc.polygon_idx, accurate);
+  if (!result) rc.mesh = 0;
+  return rc;
 }
 
 int csSector::IntersectSegment (
@@ -738,15 +812,15 @@ iSector *csSector::FollowSegment (
 
 void csSector::PrepareDraw (iRenderView *rview)
 {
-  if (csEngine::currentEngine->bugplug)
-    csEngine::currentEngine->bugplug->AddCounter ("Sector Count", 1);
+  if (engine->bugplug)
+    engine->bugplug->AddCounter ("Sector Count", 1);
 
   // Make sure the visibility culler is loaded.
   GetVisibilityCuller ();
   csRenderView* csrview = (csRenderView*)rview;
   csrview->SetThisSector ((iSector*)this);
 
-  size_t i = sectorCallbackList.Length ();
+  size_t i = sectorCallbackList.GetSize ();
   while (i > 0)
   {
     i--;
@@ -756,14 +830,14 @@ void csSector::PrepareDraw (iRenderView *rview)
 
   // Mesh generators.
   const csVector3& pos = rview->GetCamera ()->GetTransform ().GetOrigin ();
-  for (i = 0 ; i < meshGenerators.Length () ; i++)
+  for (i = 0 ; i < meshGenerators.GetSize () ; i++)
   {
     meshGenerators[i]->AllocateBlocks (pos);
   }
 
   // CS_ENTITY_CAMERA meshes have to be moved to right position first.
   const csArray<iMeshWrapper*>& cm = cameraMeshes;
-  for (i = 0 ; i < cm.Length () ; i++)
+  for (i = 0 ; i < cm.GetSize () ; i++)
   {
     iMeshWrapper* m = cm.Get (i);
     if (m->GetFlags ().Check (CS_ENTITY_CAMERA))
@@ -922,7 +996,7 @@ void csSector::RemoveSectorMeshCallback (iSectorMeshCallback* cb)
 
 void csSector::FireNewMesh (iMeshWrapper* mesh)
 {
-  size_t i = sectorMeshCallbackList.Length ();
+  size_t i = sectorMeshCallbackList.GetSize ();
   while (i > 0)
   {
     i--;
@@ -932,7 +1006,7 @@ void csSector::FireNewMesh (iMeshWrapper* mesh)
 
 void csSector::FireRemoveMesh (iMeshWrapper* mesh)
 {
-  size_t i = sectorMeshCallbackList.Length ();
+  size_t i = sectorMeshCallbackList.GetSize ();
   while (i > 0)
   {
     i--;
@@ -942,7 +1016,7 @@ void csSector::FireRemoveMesh (iMeshWrapper* mesh)
 
 void csSector::CheckFrustum (iFrustumView *lview)
 {
-  int i = (int)sectorCallbackList.Length ()-1;
+  int i = (int)sectorCallbackList.GetSize ()-1;
   while (i >= 0)
   {
     iSectorCallback* cb = sectorCallbackList.Get (i);
@@ -993,6 +1067,7 @@ void csSector::SetDynamicAmbientLight (const csColor& color)
 {
   dynamicAmbientLightColor = color;
   dynamicAmbientLightVersion++;
+  svDynamicAmbient->SetValue (dynamicAmbientLightColor);
 }
 
 void csSector::CalculateSectorBBox (csBox3 &bbox, bool do_meshes) const
@@ -1006,9 +1081,8 @@ void csSector::CalculateSectorBBox (csBox3 &bbox, bool do_meshes) const
     for (i = 0; i < meshes.GetCount (); i++)
     {
       iMeshWrapper *mesh = meshes.Get (i);
-      mesh->GetTransformedBoundingBox (
-          mesh->GetMovable ()->GetTransform (),
-          b);
+      b = mesh->GetTransformedBoundingBox (
+          mesh->GetMovable ()->GetTransform ());
       bbox += b;
     }
   }
@@ -1023,27 +1097,54 @@ void csSector::CleanupLSI ()
     ((csLight*)inf->light)->RemoveLSI (inf);
   }
   influences.Empty ();
+  relevant_lights_dirty = true;
 }
 
 void csSector::AddLSI (csLightSectorInfluence* inf)
 {
   influences.Add (inf);
+  relevant_lights_dirty = true;
 }
 
 void csSector::RemoveLSI (csLightSectorInfluence* inf)
 {
   influences.Delete (inf);
+  relevant_lights_dirty = true;
+}
+
+const csArray<iLightSectorInfluence*>& csSector::GetRelevantLights (
+  	int maxLights, bool desireSorting)
+{
+  if (relevant_lights_dirty)
+  {
+    if (maxLights != -1)
+      relevant_lights.SetSize (maxLights);
+    relevant_lights.Empty ();
+    csLightSectorInfluences::GlobalIterator it = influences.GetIterator ();
+    size_t cnt = 0;
+    while (it.HasNext ())
+    {
+      csLightSectorInfluence* inf = it.Next ();
+      relevant_lights.Push (inf);
+      cnt++;
+      if (maxLights != -1 && cnt >= (size_t)maxLights)
+	break;
+    }
+
+    relevant_lights_dirty = false;
+  }
+  return relevant_lights;
 }
 
 //---------------------------------------------------------------------------
 
 iMeshGenerator* csSector::CreateMeshGenerator (const char* name)
 {
-  csMeshGenerator* meshgen = new csMeshGenerator ();
+  csRef<csMeshGenerator> meshgen;
+  meshgen.AttachNew (new csMeshGenerator (engine));
   meshgen->SetSector (this);
   meshgen->QueryObject ()->SetName (name);
   meshGenerators.Push (meshgen);
-  meshgen->DecRef ();
   return (iMeshGenerator*)meshgen;
 }
 
@@ -1074,11 +1175,42 @@ void csSector::UnregisterPortalMesh (iMeshWrapper* mesh)
   portalMeshes.Delete (mesh);
 }
 
+CS_IMPLEMENT_STATIC_CLASSVAR_REF(csSector, svNames, SVNames,
+                                 csSector::SVNamesHolder, ());
+
+void csSector::UpdateFogSVs ()
+{
+  svFogColor->SetValue (fog.color);
+  svFogMode->SetValue (int (fog.mode));
+  svFogStart->SetValue (fog.start);
+  svFogEnd->SetValue (fog.end);
+  svFogDensity->SetValue (fog.density);
+}
+
+void csSector::SetupSVNames()
+{
+  if (SVNames().dynamicAmbient == csInvalidStringID)
+  {
+    SVNames().dynamicAmbient = CS::ShaderVarName (engine->globalStringSet,
+      "dynamic ambient");
+    SVNames().fogColor = CS::ShaderVarName (engine->globalStringSet,
+      "fog color");
+    SVNames().fogMode = CS::ShaderVarName (engine->globalStringSet,
+      "fog mode");
+    SVNames().fogStart = CS::ShaderVarName (engine->globalStringSet,
+      "fog start");
+    SVNames().fogEnd = CS::ShaderVarName (engine->globalStringSet,
+      "fog end");
+    SVNames().fogDensity = CS::ShaderVarName (engine->globalStringSet,
+      "fog density");
+  }
+}
+
 //---------------------------------------------------------------------------
 
 
-csSectorList::csSectorList ()
-  : scfImplementationType (this)
+csSectorList::csSectorList (csEngine* engine)
+  : scfImplementationType (this), engine (engine)
 {
   listener.AttachNew (new NameChangeListener (this));
 }
@@ -1091,7 +1223,7 @@ csSectorList::~csSectorList ()
 void csSectorList::NameChanged (iObject* object, const char* oldname,
   	const char* newname)
 {
-  csRef<iSector> sector = SCF_QUERY_INTERFACE (object, iSector);
+  csRef<iSector> sector = scfQueryInterface<iSector> (object);
   CS_ASSERT (sector != 0);
   if (oldname) sectors_hash.Delete (oldname, sector);
   if (newname) sectors_hash.Put (newname, sector);
@@ -1115,7 +1247,7 @@ int csSectorList::Add (iSector *obj)
 
 bool csSectorList::Remove (iSector *obj)
 {
-  csEngine::currentEngine->FireRemoveSector (obj);
+  engine->FireRemoveSector (obj);
   FreeSector (obj);
   const char* name = obj->QueryObject ()->GetName ();
   if (name)
@@ -1138,7 +1270,7 @@ bool csSectorList::Remove (int n)
 void csSectorList::RemoveAll ()
 {
   size_t i;
-  for (i = 0 ; i < list.Length () ; i++)
+  for (i = 0 ; i < list.GetSize () ; i++)
   {
     list[i]->QueryObject ()->RemoveNameChangeListener (listener);
     FreeSector (list[i]);
