@@ -40,6 +40,7 @@
 #include "ivideo/shader/shader.h"
 #include "plugins/engine/3d/light.h"
 #include "plugins/engine/3d/meshobj.h"
+#include "plugins/engine/3d/meshgen.h"
 
 class csEngine;
 class csProgressPulse;
@@ -97,14 +98,18 @@ private:
   csSector* sector;
 };
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 /**
  * A sector is a container for objects. It is one of
  * the base classes for the portal engine.
  */
-class csSector : public scfImplementationExt2<csSector, 
+class csSector : public scfImplementationExt3<csSector, 
                                               csObject,
                                               iSector,
-					      iSelfDestruct>
+					      iSelfDestruct,
+                                              scfFakeInterface<iShaderVariableContext> >,
+                 public CS::ShaderVariableContextImpl
 {
   // Friends
   friend class csEngine;
@@ -118,14 +123,21 @@ public:
    */
   csSector (csEngine*);
 
+  /**
+   * Set single mesh. This is used to render only a single mesh out
+   * of the sector. Set to 0 to go back to default behaviour of drawing
+   * all visible meshes.
+   */
+  void SetSingleMesh (iMeshWrapper* mesh) { single_mesh = mesh; }
 
-  //-- iSector 
-  
+  /**\name iSector 
+   * @{ */
   virtual iObject *QueryObject ()
   { return this; }
+  /** @} */
 
-  // -- Mesh handling
-
+  /**\name Mesh handling
+   * @{ */
   virtual iMeshList* GetMeshes ()
   { return &meshes; }
 
@@ -134,16 +146,17 @@ public:
   virtual const csSet<csPtrKey<iMeshWrapper> >& GetPortalMeshes () const
   { return portalMeshes; }
 
-  virtual void RegisterPortalMesh (iMeshWrapper* mesh);
-  virtual void UnregisterPortalMesh (iMeshWrapper* mesh);
+  void RegisterPortalMesh (iMeshWrapper* mesh);
+  void UnregisterPortalMesh (iMeshWrapper* mesh);
 
   virtual void UnlinkObjects ();
 
   virtual void AddSectorMeshCallback (iSectorMeshCallback* cb);
   virtual void RemoveSectorMeshCallback (iSectorMeshCallback* cb);
+  /** @} */
 
-  // -- Drawing related
-  
+  /**\name Drawing related
+   * @{ */
   virtual void Draw (iRenderView* rview);
 
   virtual void PrepareDraw (iRenderView* rview);
@@ -159,29 +172,38 @@ public:
   { renderloop = rl; }
   virtual iRenderLoop* GetRenderLoop ()
   { return renderloop; }
+  /** @} */
 
-  // -- Fog handling
-
+  /**\name Fog handling
+   * @{ */
   virtual bool HasFog () const
-  { return fog.enabled; }
+  { return fog.mode != CS_FOG_MODE_NONE; }
   
-  virtual csFog *GetFog () const
-  { return &fog; }
+  virtual const csFog& GetFog () const
+  { return fog; }
   
   virtual void SetFog (float density, const csColor& color)
   {
-    fog.enabled = true;
+    fog.mode = CS_FOG_MODE_CRYSTALSPACE;
     fog.density = density;
-    fog.red = color.red;
-    fog.green = color.green;
-    fog.blue = color.blue;
+    fog.color = color;
+    UpdateFogSVs ();
   }
- 
+  virtual void SetFog (const csFog& fog)
+  { 
+    this->fog = fog; 
+    UpdateFogSVs ();
+  }
+
   virtual void DisableFog ()
-  { fog.enabled = false; }
-
-  // -- Light handling
+  { 
+    fog.mode = CS_FOG_MODE_NONE; 
+    UpdateFogSVs ();
+  }
+  /** @} */
 
+  /**\name Light handling
+   * @{ */
   virtual iLightList* GetLights ()
   { return &lights; }
 
@@ -198,9 +220,10 @@ public:
 
   virtual uint GetDynamicAmbientVersion () const
   { return dynamicAmbientLightVersion; }
+  /** @} */
 
-  // -- Visculling
-
+  /**\name Visculling
+   * @{ */
   virtual void CalculateSectorBBox (csBox3& bbox,
     bool do_meshes) const;
 
@@ -211,17 +234,19 @@ public:
 
   virtual void CheckFrustum (iFrustumView* lview);
   
-  virtual iMeshWrapper* HitBeamPortals (const csVector3& start,
-  	const csVector3& end, csVector3& isect, int* polygon_idx);
 
-  virtual iMeshWrapper* HitBeam (const csVector3& start, const csVector3& end,
-    csVector3& intersect, int* polygon_idx, bool accurate = false);
+  virtual csSectorHitBeamResult HitBeamPortals (const csVector3& start,
+  	const csVector3& end);
+
+  virtual csSectorHitBeamResult HitBeam (const csVector3& start,
+  	const csVector3& end, bool accurate = false);
 
   virtual iSector* FollowSegment (csReversibleTransform& t,
     csVector3& new_position, bool& mirror, bool only_portals = false);
+  /** @} */
 
-  // -- Various  
-
+  /**\name Callbacks
+   * @{ */
   virtual void SetSectorCallback (iSectorCallback* cb)
   { sectorCallbackList.Push (cb); }
 
@@ -229,11 +254,14 @@ public:
   { sectorCallbackList.Delete (cb); }
 
   virtual int GetSectorCallbackCount () const 
-  { return (int) sectorCallbackList.Length (); }
+  { return (int) sectorCallbackList.GetSize (); }
 
   virtual iSectorCallback* GetSectorCallback (int idx) const
   { return sectorCallbackList.Get (idx); }
+  /** @} */
 
+  /**\name Lighting
+   * @{ */
   virtual void SetLightCulling (bool enable);
   virtual bool IsLightCullingEnabled () const { return use_lightculling; }
   virtual void AddLightVisibleCallback (iLightVisibleCallback* cb);
@@ -255,23 +283,38 @@ public:
   void AddLSI (csLightSectorInfluence* inf);
   void RemoveLSI (csLightSectorInfluence* inf);
   const csLightSectorInfluences& GetLSI () const { return influences; }
+  /**
+   * Get the array of relevant lights for this sector.
+   */
+  const csArray<iLightSectorInfluence*>& GetRelevantLights (
+  	int maxLights, bool desireSorting);
+  /** @} */
 
+  /**\name Mesh generators
+   * @{ */
   iMeshGenerator* CreateMeshGenerator (const char* name);
   size_t GetMeshGeneratorCount () const
   {
-    return meshGenerators.Length ();
+    return meshGenerators.GetSize ();
   }
   iMeshGenerator* GetMeshGenerator (size_t idx)
   {
-    return (iMeshGenerator*)meshGenerators[idx];
+    return (iMeshGenerator*)(csMeshGenerator*)meshGenerators[idx];
   }
   iMeshGenerator* GetMeshGeneratorByName (const char* name);
   void RemoveMeshGenerator (size_t idx);
   void RemoveMeshGenerators ();
+  /** @} */
 
-  //--------------------- iSelfDestruct implementation -------------------//
-
+  /**\name iSelfDestruct implementation
+   * @{ */
   virtual void SelfDestruct ();
+  /** @} */
+
+  virtual iShaderVariableContext* GetSVContext()
+  { return static_cast<iShaderVariableContext*> (this); }
+
+  virtual void PrecacheDraw ();
 
 private:
   // -- PRIVATE METHODS
@@ -364,6 +407,10 @@ private:
   void FireNewMesh (iMeshWrapper* mesh);
   void FireRemoveMesh (iMeshWrapper* mesh);
 
+  /// Update shader vars with fog information
+  void UpdateFogSVs ();
+
+  void SetupSVNames();
 private:
   // PRIVATE MEMBERS
 
@@ -424,10 +471,16 @@ private:
   iRenderLoop* renderloop;
 
   /// Fog information.
-  mutable csFog fog;
+  csFog fog;
 
   /// List of light/sector influences.
   csLightSectorInfluences influences;
+  /**
+   * Array of lights affecting this mesh object. This is calculated
+   * by the csLightManager class.
+   */
+  csDirtyAccessArray<iLightSectorInfluence*> relevant_lights;
+  bool relevant_lights_dirty;
 
   /**
    * The visibility culler for this sector or 0 if none.
@@ -469,14 +522,40 @@ private:
    * use.
    */
   int drawBusy;
+
+  /**
+   * If this is not 0 then we're drawing only a specific mesh and
+   * not all meshes.
+   */
+  iMeshWrapper* single_mesh;
+
+  /// Shader variable names
+  struct SVNamesHolder
+  {
+    CS::ShaderVarName dynamicAmbient;
+    CS::ShaderVarName fogColor;
+    CS::ShaderVarName fogMode;
+    CS::ShaderVarName fogStart;
+    CS::ShaderVarName fogEnd;
+    CS::ShaderVarName fogDensity;
+  };
+  CS_DECLARE_STATIC_CLASSVAR_REF(svNames, SVNames, SVNamesHolder);
+  csRef<csShaderVariable> svDynamicAmbient;
+  csRef<csShaderVariable> svFogColor;
+  csRef<csShaderVariable> svFogMode;
+  csRef<csShaderVariable> svFogStart;
+  csRef<csShaderVariable> svFogEnd;
+  csRef<csShaderVariable> svFogDensity;
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /// List of 3D engine sectors.
 class csSectorList : public scfImplementation1<csSectorList, iSectorList>
 {
 private:
   csRefArrayObject<iSector> list;
-  csHash<iSector*,csStrKey> sectors_hash;
+  csHash<iSector*, csString> sectors_hash;
 
   class NameChangeListener : public scfImplementation1<NameChangeListener,
   	iObjectNameChangeListener>
@@ -500,20 +579,20 @@ private:
   };
   csRef<NameChangeListener> listener;
 
-
+  csEngine* engine;
 public:
   void NameChanged (iObject* object, const char* oldname,
   	const char* newname);
 
   /// constructor
-  csSectorList ();
+  csSectorList (csEngine* engine);
   /// destructor
   virtual ~csSectorList ();
 
   /// Override FreeSector.
   virtual void FreeSector (iSector* item);
 
-  virtual int GetCount () const { return (int)list.Length (); }
+  virtual int GetCount () const { return (int)list.GetSize (); }
   virtual iSector *Get (int n) const { return list.Get (n); }
   virtual int Add (iSector *obj);
   virtual bool Remove (iSector *obj);
