@@ -27,14 +27,13 @@ CS_LEAKGUARD_IMPLEMENT (csSharedVariable);
 void csSharedVariable::FireListeners ()
 {
   size_t i;
-  for (i = 0 ; i < listeners.Length () ; i++)
+  for (i = 0 ; i < listeners.GetSize () ; i++)
     listeners[i]->VariableChanged (this);
 }
 
 void csSharedVariable::SelfDestruct ()
 {
-  csEngine::currentEngine->GetVariableList ()->Remove (
-  	(iSharedVariable*)this);
+  variables->Remove (static_cast<iSharedVariable*> (this));
 }
 
 //-----------------------------------------------------------------------------
@@ -91,9 +90,9 @@ iSharedVariable *csSharedVariableList::FindByName (
   return list.FindByName (Name);
 }
 
-csPtr<iSharedVariable> csSharedVariableList::New() const
+csPtr<iSharedVariable> csSharedVariableList::New()
 {
-  csSharedVariable *New = new csSharedVariable;
+  csSharedVariable *New = new csSharedVariable (this);
   return csPtr<iSharedVariable> (New);
 }
 
