@@ -30,11 +30,13 @@
 #include "iengine/rendersteps/icontainer.h"
 #include "iengine/rendersteps/ilightiter.h"
 #include "iengine/viscull.h"
-#include "igeom/objmodel.h"
+#include "imesh/objmodel.h"
 #include "igeom/polymesh.h"
+#include "igeom/trimesh.h"
 #include "csutil/hash.h"
 #include "csutil/csstring.h"
 #include "csutil/strhash.h"
+#include "csutil/scf_implementation.h"
 #include "csutil/dirtyaccessarray.h"
 #include "csutil/weakref.h"
 #include "csgfx/shadervarcontext.h"
@@ -45,13 +47,17 @@
 class csStencilShadowStep;
 class csStencilShadowType;
 
-class csStencilShadowCacheEntry : public iObjectModelListener				  
+class csStencilShadowCacheEntry :
+  public scfImplementation1<csStencilShadowCacheEntry, iObjectModelListener>
 {
 private:
   csStencilShadowStep* parent;
   iObjectModel* model;
   iMeshWrapper* meshWrapper;
 
+  // If true we use the new triangle mesh system.
+  bool use_trimesh;
+
   struct csLightCacheEntry 
   {
     iLight* light;
@@ -73,8 +79,8 @@ private:
     EdgeInfo() : a(0), b(0), norm(0), ind_a(0), ind_b(0) {}
   };
 
-  int vertex_count, triangle_count;
-  int edge_count;
+  size_t vertex_count, triangle_count;
+  size_t edge_count;
   csDirtyAccessArray<csVector3> face_normals;
   csDirtyAccessArray<int> edge_indices;
   csArray<csVector3> edge_midpoints;
@@ -82,7 +88,7 @@ private:
 
   // Mesh that was created when the original shadow mesh was auto-closed.
   // Kept so that a new one isn't alloced every time.
-  csStencilPolygonMesh* closedMesh;
+  csStencilTriangleMesh* closedMesh;
 
   bool enable_caps;
 
@@ -93,7 +99,6 @@ private:
     csHash<EdgeInfo*>& edge_stack, int& NextEdge, int& TriIndex);
   void HandleEdge (EdgeInfo* e, csHash<EdgeInfo*>& edge_stack);
 public:
-  SCF_DECLARE_IBASE;
   csRef<csRenderBufferHolder> bufferHolder;
 
   csStencilShadowCacheEntry (csStencilShadowStep* parent, 
@@ -112,9 +117,10 @@ public:
   void UpdateBuffers() ;
 };
 
-class csStencilShadowStep : public iRenderStep,
-			    public iLightRenderStep,
-			    public iRenderStepContainer
+class csStencilShadowStep : 
+  public scfImplementation4<csStencilShadowStep,
+    iRenderStep, iLightRenderStep, iRenderStepContainer,
+    iVisibilityCullerListener>
 {
 private:
   friend class csStencilShadowCacheEntry;
@@ -124,6 +130,10 @@ private:
   csWeakRef<iShaderManager> shmgr;
   csRef<csStencilShadowType> type;
 
+  // ID's for the triangle mesh system.
+  csStringID base_id;
+  csStringID shadows_id;
+
   bool enableShadows;
   csRefArray<iLightRenderStep> steps;
 
@@ -137,17 +147,18 @@ private:
   void Report (int severity, const char* msg, ...);
 
 public:
-  SCF_DECLARE_IBASE;
-
   csStencilShadowStep (csStencilShadowType* type);
   virtual ~csStencilShadowStep ();
 
   bool Initialize (iObjectRegistry* objreg);
+
+  csStringID GetBaseID () const { return base_id; }
+  csStringID GetShadowsID () const { return shadows_id; }
   
   void Perform (iRenderView* rview, iSector* sector,
-    csShaderVarStack &stacks);
+    iShaderVarStack* stacks);
   void Perform (iRenderView* rview, iSector* sector, iLight* light,
-    csShaderVarStack &stacks);
+    iShaderVarStack* stacks);
 
   virtual size_t AddStep (iRenderStep* step);
   virtual bool DeleteStep (iRenderStep* step);
@@ -155,27 +166,17 @@ public:
   virtual size_t Find (iRenderStep* step) const;
   virtual size_t GetStepCount () const;
 
-  struct ShadowDrawVisCallback : public iVisibilityCullerListener
-  {
-    csStencilShadowStep* parent;
-
-    SCF_DECLARE_IBASE;
-    ShadowDrawVisCallback ();
-    virtual ~ShadowDrawVisCallback ();
-
-    virtual void ObjectVisible (iVisibilityObject *visobject, 
-      iMeshWrapper *mesh, uint32 frustum_mask);
-  } shadowDrawVisCallback;
-  friend struct ShadowDrawVisCallback;
+  virtual void ObjectVisible (iVisibilityObject *visobject, 
+    iMeshWrapper *mesh, uint32 frustum_mask);
 };
 
-class csStencilShadowFactory : public iRenderStepFactory
+class csStencilShadowFactory :
+  public scfImplementation1<csStencilShadowFactory, iRenderStepFactory>
 {
   iObjectRegistry* object_reg;
   csRef<csStencilShadowType> type;
-public:
-  SCF_DECLARE_IBASE;
 
+public:
   csStencilShadowFactory (iObjectRegistry* object_reg,
       csStencilShadowType* type);
   virtual ~csStencilShadowFactory ();
@@ -184,7 +185,8 @@ public:
 
 };
 
-class csStencilShadowType : public csBaseRenderStepType
+class csStencilShadowType :
+  public scfImplementationExt0<csStencilShadowType, csBaseRenderStepType>
 {
   csRef<iShader> shadow;
   bool shadowLoaded;
@@ -199,7 +201,8 @@ public:
   iShader* GetShadow ();
 };
 
-class csStencilShadowLoader : public csBaseRenderStepLoader
+class csStencilShadowLoader :
+  public scfImplementationExt0<csStencilShadowLoader, csBaseRenderStepLoader>
 {
   csRenderStepParser rsp;
 
