@@ -25,10 +25,12 @@
 #include "csgeom/sphere.h"
 #include "csgeom/transfrm.h"
 #include "csgeom/vector4.h"
+#include "cstool/rviewclipper.h"
 #include "csgfx/renderbuffer.h"
 #include "csutil/dirtyaccessarray.h"
 #include "csutil/flags.h"
 #include "csutil/scf.h"
+#include "csutil/scfarray.h"
 #include "csutil/xmltiny.h"
 
 #include "iengine/camera.h"
@@ -54,25 +56,15 @@
 #define Z_PASS 0
 #define Z_FAIL 1
 
-#include "polymesh.h"
+#include "trimesh.h"
 #include "stencil2.h"
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csStencil2ShadowStep)
-SCF_IMPLEMENTS_INTERFACE (iRenderStep)
-SCF_IMPLEMENTS_INTERFACE (iLightRenderStep)
-SCF_IMPLEMENTS_INTERFACE (iRenderStepContainer)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csStencil2ShadowCacheEntry)
-SCF_IMPLEMENTS_INTERFACE (iObjectModelListener)
-SCF_IMPLEMENT_IBASE_END
-
 csStencil2ShadowCacheEntry::csStencil2ShadowCacheEntry (
-  csStencil2ShadowStep* parent, iMeshWrapper* mesh)
+  csStencil2ShadowStep* parent, iMeshWrapper* mesh) :
+  scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   shadow_vertex_buffer = 0;
   shadow_normal_buffer = 0;
   active_index_buffer = 0;
@@ -93,21 +85,21 @@ csStencil2ShadowCacheEntry::csStencil2ShadowCacheEntry (
 
   csRef<iObjectModel> model = mesh->GetMeshObject ()->GetObjectModel ();
   model->AddListener (this);
+  use_trimesh = model->IsTriangleDataSet (parent->GetBaseID ());
   ObjectModelChanged (model);
 }
 
 csStencil2ShadowCacheEntry::~csStencil2ShadowCacheEntry ()
 {
   delete closedMesh;
-  SCF_DESTRUCT_IBASE();
 }
 
 void csStencil2ShadowCacheEntry::UpdateRenderBuffers(
 	csArray<csVector3> & shadow_vertices,
 	csArray<int> & shadow_indeces)
 {
-  int vertex_count = (int)shadow_vertices.Length();
-  int index_count = (int)shadow_indeces.Length();
+  int vertex_count = (int)shadow_vertices.GetSize ();
+  int index_count = (int)shadow_indeces.GetSize ();
 
   shadow_vertex_buffer = csRenderBuffer::CreateRenderBuffer (
     vertex_count, CS_BUF_DYNAMIC,
@@ -129,38 +121,44 @@ void csStencil2ShadowCacheEntry::ObjectModelChanged (iObjectModel* model)
 
   meshShadows = false;
 
-  csRef<iPolygonMesh> mesh = model->GetPolygonMeshShadows ();
-  if (mesh && mesh->GetPolygonCount () > 0)
+  // Try to get a MeshShadow polygonmesh
+  csRef<iTriangleMesh> trimesh;
+  if (use_trimesh)
   {
-    if (closedMesh == 0)
-      closedMesh = new csStencil2PolygonMesh ();
-    closedMesh->CopyFrom (mesh);
-
-    /* This is really hard method to close object ... must use better one
-
-    const csFlags& meshFlags = mesh->GetFlags ();
-    if (meshFlags.Check (CS_POLYMESH_NOTCLOSED) || (!meshFlags.Check (CS_POLYMESH_CLOSED) && 
-    !csPolygonMeshTools::IsMeshClosed (mesh)))
+    if (model->IsTriangleDataSet (parent->GetShadowsID ()))
+      trimesh = model->GetTriangleData (parent->GetShadowsID ());
+    else
+      trimesh = model->GetTriangleData (parent->GetBaseID ());
+    if (trimesh && trimesh->GetTriangleCount () <= 0)
     {
-    csArray<csMeshedPolygon> newPolys;
-    int* vertidx;
-    int vertidx_len;
-    csPolygonMeshTools::CloseMesh (mesh, newPolys, vertidx, vertidx_len);
-    closedMesh->AddPolys (newPolys, vertidx);
-    }
-    */
-
-    if (!CalculateEdges())
-    {
-      if (closedMesh)
-      {
-        delete closedMesh;
-        closedMesh = 0;
-      }
-      return;
+      trimesh = 0;
     }
   }
   else
+  {
+    iPolygonMesh* mesh = model->GetPolygonMeshShadows ();
+    if (mesh)
+      trimesh.AttachNew (new csTriangleMeshPolyMesh (mesh));
+  }
+
+  if (!trimesh) return;	// No shadow casting for this object.
+
+  if (closedMesh == 0)
+    closedMesh = new csStencil2TriangleMesh ();
+  closedMesh->CopyFrom (trimesh);
+
+  /* This is really hard method to close object ... must use better one
+  const csFlags& meshFlags = trimesh->GetFlags ();
+  if (meshFlags.Check (CS_TRIMESH_NOTCLOSED) || (!meshFlags.Check (CS_TRIMESH_CLOSED) && 
+    !csTriangleMeshTools::IsMeshClosed (trimesh)))
+  {
+    csArray<csTriangle> newTris;
+    csTriangleMeshTools::CloseMesh (trimesh, newTris);
+    closedMesh->AddTris (newTris);
+  }
+  */
+
+  if (!CalculateEdges())
   {
     if (closedMesh)
     {
@@ -210,7 +208,7 @@ bool csStencil2ShadowCacheEntry::CalculateEdges()
   bool result = true;
 
   int errors_count = 0;
-  for (i = 0; i < edges.Length(); i++)
+  for (i = 0; i < edges.GetSize (); i++)
   {
     if ((edges[i]->face_2 == -1) || (edges[i]->face_1 == edges[i]->face_2))
     {
@@ -218,7 +216,7 @@ bool csStencil2ShadowCacheEntry::CalculateEdges()
       result = false;
       errors_count++;
       /*
-      for (j = 0; j < edges.Length(); j++)
+      for (j = 0; j < edges.GetSize (); j++)
       {
       if ((((edges[j]->v1 == edges[i]->v1) && (edges[j]->v2 == edges[i]->v2)) ||
       ((edges[j]->v1 == edges[i]->v2) && (edges[j]->v2 == edges[i]->v1))) && (i != j))
@@ -241,7 +239,7 @@ void csStencil2ShadowCacheEntry::AddEdge(int index_v1, int index_v2, int face_in
 {
   size_t i;
   bool found = false;
-  for (i = 0; i < edges.Length(); i++)
+  for (i = 0; i < edges.GetSize (); i++)
   {
     if ((((edges[i]->v1 == index_v1) && (edges[i]->v2 == index_v2)) || 
       ((edges[i]->v1 == index_v2) && (edges[i]->v2 == index_v1))) && 
@@ -304,9 +302,9 @@ bool csStencil2ShadowCacheEntry::GetShadow(
   if (extrusion || back_cap)
   {
     //first calculate silhouette 
-    silhouette_edges.SetLength(0);
-    silhouette_edges.SetCapacity (edges.Length ());
-    for (i = 0; i < edges.Length(); i++)
+    silhouette_edges.SetSize (0);
+    silhouette_edges.SetCapacity (edges.GetSize ());
+    for (i = 0; i < edges.GetSize (); i++)
     {
       //if (edges[i]->face_2 > -1)
       //{
@@ -320,12 +318,12 @@ bool csStencil2ShadowCacheEntry::GetShadow(
 
     if (extrusion) // building indexed triangles for shadow sides (using quads should be wiser!)
     {
-      shadow_vertices.SetMinimalCapacity (shadow_vertices.Length () +
-      	silhouette_edges.Length () * 4);
-      shadow_indeces.SetMinimalCapacity (shadow_indeces.Length () +
-      	silhouette_edges.Length () * 6);
+      shadow_vertices.SetMinimalCapacity (shadow_vertices.GetSize () +
+      	silhouette_edges.GetSize () * 4);
+      shadow_indeces.SetMinimalCapacity (shadow_indeces.GetSize () +
+      	silhouette_edges.GetSize () * 6);
 
-      for (i = 0; i < silhouette_edges.Length(); i++ )
+      for (i = 0; i < silhouette_edges.GetSize (); i++ )
       {
         int index = silhouette_edges[i];
         csVector3 v0 = vertices[edges[index]->v1];
@@ -361,18 +359,18 @@ bool csStencil2ShadowCacheEntry::GetShadow(
     }
 
     // optimization for dark cap - building dark cap from silhouette edges
-    if (back_cap && (silhouette_edges.Length() > 0)) 
+    if (back_cap && (silhouette_edges.GetSize () > 0)) 
     {
       csVector3 first_point = ((vertices[edges[silhouette_edges[0]]->v1]
       	- light_pos))*shadow_length
         + vertices[edges[silhouette_edges[0]]->v1];
 
-      shadow_vertices.SetMinimalCapacity (shadow_vertices.Length () +
-      	silhouette_edges.Length () * 3);
-      shadow_indeces.SetMinimalCapacity (shadow_indeces.Length () +
-      	silhouette_edges.Length () * 3);
+      shadow_vertices.SetMinimalCapacity (shadow_vertices.GetSize () +
+      	silhouette_edges.GetSize () * 3);
+      shadow_indeces.SetMinimalCapacity (shadow_indeces.GetSize () +
+      	silhouette_edges.GetSize () * 3);
 
-      for (i = 1; i < silhouette_edges.Length(); i++)
+      for (i = 1; i < silhouette_edges.GetSize (); i++)
       {
         shadow_vertices.Push(first_point);
 
@@ -399,7 +397,7 @@ bool csStencil2ShadowCacheEntry::GetShadow(
     }
   }
 
-  return shadow_vertices.Length() && shadow_indeces.Length();
+  return shadow_vertices.GetSize () && shadow_indeces.GetSize ();
 }
 
 
@@ -407,17 +405,14 @@ bool csStencil2ShadowCacheEntry::GetShadow(
 
 
 csStencil2ShadowStep::csStencil2ShadowStep (csStencil2ShadowType* type) :  
-shadowDrawVisCallback ()
+  scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   csStencil2ShadowStep::type = type;
-  shadowDrawVisCallback.parent = this;
   enableShadows = false;
 }
 
 csStencil2ShadowStep::~csStencil2ShadowStep ()
 {
-  SCF_DESTRUCT_IBASE();
 }
 
 void csStencil2ShadowStep::Report (int severity, const char* msg, ...)
@@ -433,8 +428,8 @@ void csStencil2ShadowStep::Report (int severity, const char* msg, ...)
 bool csStencil2ShadowStep::Initialize (iObjectRegistry* objreg)
 {
   object_reg = objreg;
-  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
-  shmgr = CS_QUERY_REGISTRY (object_reg, iShaderManager);
+  g3d = csQueryRegistry<iGraphics3D> (object_reg);
+  shmgr = csQueryRegistry<iShaderManager> (object_reg);
 
   const csGraphics3DCaps* caps = g3d->GetCaps();
   enableShadows = caps->StencilShadows;
@@ -444,14 +439,16 @@ bool csStencil2ShadowStep::Initialize (iObjectRegistry* objreg)
       "Renderer does not support stencil shadows");
   }
 
-  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg,
-    "crystalspace.shared.stringset", iStringSet);
+  csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet>
+    (object_reg, "crystalspace.shared.stringset");
+  base_id = strings->Request ("base");
+  shadows_id = strings->Request ("shadows");
 
   return true;
 }
 
 void csStencil2ShadowStep::Perform (iRenderView* /*rview*/, iSector* /*sector*/,
-                                    csShaderVarStack& /*stacks*/)
+                                    iShaderVarStack* /*stacks*/)
 {
   /// TODO: Report error (no light)
   return;
@@ -459,7 +456,7 @@ void csStencil2ShadowStep::Perform (iRenderView* /*rview*/, iSector* /*sector*/,
 
 void csStencil2ShadowStep::ModelInFrustum(
 	csVector3 &light_pos, float shadow_length, csPlane3* frustum_planes, 
-        uint32& frustum_mask, csBox3 &model_bounding_box,
+        uint32& frustum_mask, const csBox3 &model_bounding_box,
 	bool & front_cap_in_frustum, bool & extrusion_in_frustum,
 	bool & back_cap_in_frustum)
 {
@@ -496,7 +493,7 @@ void csStencil2ShadowStep::ModelInFrustum(
     {
       bool behind_plane = true;
       size_t v = 0;
-      while ((v < projected_points.Length()) && behind_plane) 
+      while ((v < projected_points.GetSize ()) && behind_plane) 
       {
         behind_plane = (frustum_planes[i].Classify(projected_points[v]) < 0);
         v++;
@@ -520,7 +517,7 @@ void csStencil2ShadowStep::ModelInFrustum(
     {
       bool behind_plane = true;
       size_t v = 0;
-      while ((v < projected_points.Length()) && behind_plane) 
+      while ((v < projected_points.GetSize ()) && behind_plane) 
       {
         behind_plane = (frustum_planes[i].Classify(projected_points[v]) < 0);
         v++;
@@ -540,7 +537,7 @@ void csStencil2ShadowStep::ModelInFrustum(
 
 int csStencil2ShadowStep::CalculateShadowMethod(
 	iRenderView *rview, csVector3 &light_pos, 
-        const csReversibleTransform &t, csBox3 &model_bounding_box)
+        const csReversibleTransform &t, const csBox3 &model_bounding_box)
 {
   float lx, rx, ty, dy;
   rview->GetFrustum(lx, rx, ty, dy);
@@ -579,7 +576,7 @@ int csStencil2ShadowStep::CalculateShadowMethod(
   size_t i;
   if ((light_dir*forward_view_vector) > 0)
   {
-    for (i = 0; i < oclusion_pyramid.Length() ; i++)
+    for (i = 0; i < oclusion_pyramid.GetSize () ; i++)
     {
       oclusion_pyramid[i].Invert();
     }
@@ -587,7 +584,7 @@ int csStencil2ShadowStep::CalculateShadowMethod(
 
   oclusion_pyramid.Push(csPlane3(light_dir, -(light_dir*light_pos)));
 
-  for (i = 0; i < oclusion_pyramid.Length(); i++) 
+  for (i = 0; i < oclusion_pyramid.GetSize (); i++) 
   {
     bool behind_plane = true;
     int v = 0;
@@ -616,7 +613,10 @@ void csStencil2ShadowStep::DrawShadow(
 	csArray<int> & shadow_indeces, 
 	iShader* shader, size_t shaderTicket, size_t /*pass*/)
 {
-  if (!cache_entry->MeshCastsShadow() || !cache_entry->ShadowCaps()) return;
+  if (!cache_entry->MeshCastsShadow() || 
+    !cache_entry->ShadowCaps() ||
+    mesh->GetFlags ().Check (CS_ENTITY_NOSHADOWS)) 
+    return;
 
   iCamera* camera = rview->GetCamera ();  
 
@@ -630,7 +630,7 @@ void csStencil2ShadowStep::DrawShadow(
   rmesh.buffers = cache_entry->bufferHolder;
   rmesh.meshtype = CS_MESHTYPE_TRIANGLES;
   rmesh.indexstart = 0;
-  rmesh.indexend = (uint)shadow_indeces.Length();
+  rmesh.indexend = (uint)shadow_indeces.GetSize ();
 
   cache_entry->UpdateRenderBuffers(shadow_vertices, shadow_indeces);
 
@@ -638,9 +638,9 @@ void csStencil2ShadowStep::DrawShadow(
 
   cache_entry->UpdateBuffers();
 
-  static csShaderVarStack stacks; // @@@ use STATIC macros
+  csRef<iShaderVarStack> stacks;
+  stacks.AttachNew (new scfArray<iShaderVarStack>);
 
-  stacks.Empty ();
   shmgr->PushVariables (stacks);
   g3d->SetWorldToCamera (camera->GetTransform ().GetInverse ());
   shader->SetupPass (shaderTicket, &rmesh, rmesh, stacks);
@@ -665,12 +665,12 @@ void csStencil2ShadowStep::DrawShadow(
 }
 
 void csStencil2ShadowStep::Perform (iRenderView* rview, iSector* sector,
-                                    iLight* light, csShaderVarStack &stacks)
+                                    iLight* light, iShaderVarStack* stacks)
 {
   iShader* shadow;
   if (!enableShadows || ((shadow = type->GetShadow ()) == 0))
   {
-    for (size_t i = 0; i < steps.Length (); i++)
+    for (size_t i = 0; i < steps.GetSize (); i++)
     {
       steps[i]->Perform (rview, sector, light, stacks);
     }
@@ -681,9 +681,9 @@ void csStencil2ShadowStep::Perform (iRenderView* rview, iSector* sector,
   csSphere lightSphere (light_pos, light->GetCutoffDistance ());
   csRef<iVisibilityCuller> culler = sector->GetVisibilityCuller ();
   shadowMeshes.Truncate (0);
-  culler->VisTest (lightSphere, &shadowDrawVisCallback);
+  culler->VisTest (lightSphere, this);
   size_t numShadowMeshes;
-  if ((numShadowMeshes = shadowMeshes.Length ()) > 0)
+  if ((numShadowMeshes = shadowMeshes.GetSize ()) > 0)
   {
     g3d->SetZMode (CS_ZBUF_TEST);
     g3d->SetShadowState (CS_SHADOW_VOLUME_BEGIN);
@@ -697,7 +697,8 @@ void csStencil2ShadowStep::Perform (iRenderView* rview, iSector* sector,
       {
         iMeshWrapper*& sp = shadowMeshes[m];
 
-        csRef<csStencil2ShadowCacheEntry> shadowCacheEntry = shadowcache.Get (sp, 0);
+        csRef<csStencil2ShadowCacheEntry> shadowCacheEntry = 
+	  shadowcache.Get (sp, (csStencil2ShadowCacheEntry*)0);
 
         if (!shadowCacheEntry) 
         {
@@ -715,16 +716,16 @@ void csStencil2ShadowStep::Perform (iRenderView* rview, iSector* sector,
           continue;
         }
 
-        csBox3 model_bounding_box;
-        sp->GetMeshObject()->GetObjectModel()
-		->GetObjectBoundingBox(model_bounding_box);
+        const csBox3& model_bounding_box = sp->GetMeshObject()->GetObjectModel()
+		->GetObjectBoundingBox();
 
         const csReversibleTransform& tf = sp->GetMovable ()->GetTransform ();
 
         csPlane3 frustum_planes[6];
         uint32 frustum_mask;
         csReversibleTransform tr_o2c = rview->GetCamera()->GetTransform()/tf;
-        rview->SetupClipPlanes(tr_o2c, frustum_planes, frustum_mask);
+	CS::RenderViewClipper::SetupClipPlanes (rview->GetRenderContext (),
+	    tr_o2c, frustum_planes, frustum_mask);
 
         float shadow_length = 100;//(light->GetInfluenceRadius() + maxRadius);
         csVector3 light_pos2object = tf.Other2This(light_pos);
@@ -772,7 +773,7 @@ void csStencil2ShadowStep::Perform (iRenderView* rview, iSector* sector,
 
     g3d->SetShadowState (CS_SHADOW_VOLUME_USE);
 
-    for (size_t i = 0; i < steps.Length (); i++)
+    for (size_t i = 0; i < steps.GetSize (); i++)
     {
       steps[i]->Perform (rview, sector, light, stacks);
     }
@@ -784,7 +785,7 @@ void csStencil2ShadowStep::Perform (iRenderView* rview, iSector* sector,
 size_t csStencil2ShadowStep::AddStep (iRenderStep* step)
 {
   csRef<iLightRenderStep> lrs = 
-    SCF_QUERY_INTERFACE (step, iLightRenderStep);
+    scfQueryInterface<iLightRenderStep> (step);
   if (!lrs) return csArrayItemNotFound;
   return steps.Push (lrs);
 }
@@ -792,7 +793,7 @@ size_t csStencil2ShadowStep::AddStep (iRenderStep* step)
 bool csStencil2ShadowStep::DeleteStep (iRenderStep* step)
 {
   csRef<iLightRenderStep> lrs = 
-    SCF_QUERY_INTERFACE (step, iLightRenderStep);
+    scfQueryInterface<iLightRenderStep> (step);
   if (!lrs) return false;
   return steps.Delete(lrs);
 }
@@ -805,53 +806,34 @@ iRenderStep* csStencil2ShadowStep::GetStep (size_t n) const
 size_t csStencil2ShadowStep::Find (iRenderStep* step) const
 {
   csRef<iLightRenderStep> lrs = 
-    SCF_QUERY_INTERFACE (step, iLightRenderStep);
+    scfQueryInterface<iLightRenderStep> (step);
   if (!lrs) return csArrayItemNotFound;
   return steps.Find(lrs);
 }
 
 size_t csStencil2ShadowStep::GetStepCount () const
 {
-  return steps.Length();
+  return steps.GetSize ();
 }
 
-SCF_IMPLEMENT_IBASE(csStencil2ShadowStep::ShadowDrawVisCallback)
-SCF_IMPLEMENTS_INTERFACE(iVisibilityCullerListener)
-SCF_IMPLEMENT_IBASE_END
-
-csStencil2ShadowStep::ShadowDrawVisCallback::ShadowDrawVisCallback ()
-{
-  SCF_CONSTRUCT_IBASE(0);
-}
-
-csStencil2ShadowStep::ShadowDrawVisCallback::~ShadowDrawVisCallback ()
-{
-  SCF_DESTRUCT_IBASE();
-}
-
-void csStencil2ShadowStep::ShadowDrawVisCallback::ObjectVisible (
+void csStencil2ShadowStep::ObjectVisible (
   iVisibilityObject* /*visobject*/, iMeshWrapper *mesh, uint32 /*frustum_mask*/)
 {
-  parent->shadowMeshes.Push (mesh);
+  shadowMeshes.Push (mesh);
 }
 
 //---------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csStencil2ShadowFactory);
-SCF_IMPLEMENTS_INTERFACE (iRenderStepFactory);
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 csStencil2ShadowFactory::csStencil2ShadowFactory (iObjectRegistry* object_reg,
-                                                  csStencil2ShadowType* type)
+  csStencil2ShadowType* type) :
+  scfImplementationType (this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   csStencil2ShadowFactory::object_reg = object_reg;
   csStencil2ShadowFactory::type = type;
 }
 
 csStencil2ShadowFactory::~csStencil2ShadowFactory ()
 {
-  SCF_DESTRUCT_IBASE();
 }
 
 csPtr<iRenderStep> csStencil2ShadowFactory::Create ()
@@ -865,7 +847,8 @@ csPtr<iRenderStep> csStencil2ShadowFactory::Create ()
 
 SCF_IMPLEMENT_FACTORY(csStencil2ShadowType)
 
-csStencil2ShadowType::csStencil2ShadowType (iBase *p) : csBaseRenderStepType (p)
+csStencil2ShadowType::csStencil2ShadowType (iBase *p) :
+  scfImplementationType (this, p)
 {
   shadowLoaded = false;
 }
@@ -896,34 +879,19 @@ iShader* csStencil2ShadowType::GetShadow ()
   {
     shadowLoaded = true;
 
-    csRef<iPluginManager> plugin_mgr (
-      CS_QUERY_REGISTRY (object_reg, iPluginManager));
-
     // Load the shadow vertex program 
-    csRef<iShaderManager> shmgr = CS_QUERY_REGISTRY (object_reg,
-      iShaderManager);
-
-    if (!shmgr) 
-    {
-      shmgr = CS_LOAD_PLUGIN (plugin_mgr,
-        "crystalspace.graphics3d.shadermanager",
-        iShaderManager);
-    }
-
-    if (!shmgr) 
-    {
-      Report (CS_REPORTER_SEVERITY_ERROR, "Unable to retrieve shader manager!");
-      return 0;
-    }
+    csRef<iShaderManager> shmgr = csQueryRegistryOrLoad<iShaderManager> (
+    	object_reg, "crystalspace.graphics3d.shadermanager");
+    if (!shmgr) return 0;
 
     csRef<iShaderCompiler> shcom (shmgr->GetCompiler ("XMLShader"));
 
-    csRef<iVFS> vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+    csRef<iVFS> vfs = csQueryRegistry<iVFS> (object_reg);
     //csRef<iDataBuffer> buf = vfs->ReadFile ("/shader/shadow.xml");
     csRef<iDataBuffer> buf = vfs->ReadFile ("/shader/shadow2.xml");
     //csRef<iDataBuffer> buf = vfs->ReadFile ("/shader/shadowdebug.xml");
     csRef<iDocumentSystem> docsys (
-      CS_QUERY_REGISTRY(object_reg, iDocumentSystem));
+      csQueryRegistry<iDocumentSystem> (object_reg));
     if (docsys == 0)
     {
       docsys.AttachNew (new csTinyDocumentSystem ());
@@ -932,7 +900,9 @@ iShader* csStencil2ShadowType::GetShadow ()
     csRef<iDocument> shaderDoc = docsys->CreateDocument ();
     shaderDoc->Parse (buf, true);
 
-    shadow = shcom->CompileShader (shaderDoc->GetRoot ()->GetNode ("shader"));
+    // @@@ TODO: Try to get a right ldr_context here???
+    shadow = shcom->CompileShader (0,
+	shaderDoc->GetRoot ()->GetNode ("shader"));
 
     if (!shadow)
     {
@@ -948,8 +918,8 @@ iShader* csStencil2ShadowType::GetShadow ()
 
 SCF_IMPLEMENT_FACTORY(csStencil2ShadowLoader)
 
-csStencil2ShadowLoader::csStencil2ShadowLoader (iBase *p) 
-: csBaseRenderStepLoader (p)
+csStencil2ShadowLoader::csStencil2ShadowLoader (iBase *p) :
+  scfImplementationType (this, p)
 {
   InitTokenTable (tokens);
 }
@@ -975,8 +945,8 @@ csPtr<iBase> csStencil2ShadowLoader::Parse (iDocumentNode* node,
 					    iLoaderContext* /*ldr_context*/,
                                             iBase* /*context*/)
 {
-  csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (object_reg,
-    iPluginManager));
+  csRef<iPluginManager> plugin_mgr (
+    csQueryRegistry<iPluginManager> (object_reg));
   csRef<iRenderStepType> type (CS_LOAD_PLUGIN (plugin_mgr,
     "crystalspace.renderloop.step.shadow.stencil2.type", 
     iRenderStepType));
@@ -985,7 +955,7 @@ csPtr<iBase> csStencil2ShadowLoader::Parse (iDocumentNode* node,
   csRef<iRenderStep> step = factory->Create ();
 
   csRef<iRenderStepContainer> steps =
-    SCF_QUERY_INTERFACE (step, iRenderStepContainer);
+    scfQueryInterface<iRenderStepContainer> (step);
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
