@@ -29,6 +29,7 @@
 #include "csutil/randomgen.h"
 #include "csutil/weakref.h"
 #include "csutil/event.h"
+#include "csutil/eventnames.h"
 #include "iutil/objreg.h"
 #include "iutil/event.h"
 #include "iutil/eventq.h"
@@ -54,41 +55,30 @@ CS_IMPLEMENT_PLUGIN
 
 //---------------------------------------------------------------------------
 
+CS_PLUGIN_NAMESPACE_BEGIN(EngSeq)
+{
+
 /**
  * The superclass of all sequence operations.
  */
-class OpStandard : public iSequenceOperation
+class OpStandard : 
+  public scfImplementation1<OpStandard, iSequenceOperation>
 {
-protected:
-  virtual ~OpStandard() { SCF_DESTRUCT_IBASE(); }
-
 public:
-  SCF_DECLARE_IBASE;
-  OpStandard () { SCF_CONSTRUCT_IBASE (0); }
+  OpStandard () : scfImplementationType (this) { }
   virtual void CleanupSequences () { }
 };
 
-SCF_IMPLEMENT_IBASE (OpStandard)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceOperation)
-SCF_IMPLEMENT_IBASE_END
-
 /**
  * The superclass of all sequence conditions.
  */
-class CondStandard : public iSequenceCondition
+class CondStandard : 
+  public scfImplementation1<CondStandard, iSequenceCondition>
 {
-protected:
-  virtual ~CondStandard() { SCF_DESTRUCT_IBASE(); }
-
 public:
-  SCF_DECLARE_IBASE;
-  CondStandard () { SCF_CONSTRUCT_IBASE (0); }
+  CondStandard () : scfImplementationType (this) { }
 };
 
-SCF_IMPLEMENT_IBASE (CondStandard)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceCondition)
-SCF_IMPLEMENT_IBASE_END
-
 //---------------------------------------------------------------------------
 
 /**
@@ -106,7 +96,7 @@ public:
   OpSetFog (iParameterESM* sectorpar, const csColor& color, float density)
   {
     if (sectorpar->IsConstant ())
-      sector = SCF_QUERY_INTERFACE (sectorpar->GetValue (), iSector);
+      sector = scfQueryInterface<iSector> (sectorpar->GetValue ());
     else
       OpSetFog::sectorpar = sectorpar;
     OpSetFog::color = color;
@@ -116,7 +106,7 @@ public:
   virtual void Do (csTicks /*dt*/, iBase* params)
   {
     if (sectorpar)
-      sector = SCF_QUERY_INTERFACE (sectorpar->GetValue (params), iSector);
+      sector = scfQueryInterface<iSector> (sectorpar->GetValue (params));
     if (density < 0.001)
       sector->DisableFog ();
     else
@@ -126,18 +116,18 @@ public:
   }
 };
 
-class FadeFogInfo : public iSequenceTimedOperation
+class FadeFogInfo : 
+  public scfImplementation1<FadeFogInfo, iSequenceTimedOperation>
 {
 public:
   csRef<iSector> sector;
   csColor start_col, end_col;
   float start_density, end_density;
 
-  SCF_DECLARE_IBASE;
-  FadeFogInfo ()
-  { SCF_CONSTRUCT_IBASE (0); }
+  FadeFogInfo () : scfImplementationType (this)
+  { }
   virtual ~FadeFogInfo ()
-  { SCF_DESTRUCT_IBASE(); }
+  { }
 
   virtual void Do (float time, iBase*)
   {
@@ -155,10 +145,6 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (FadeFogInfo)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceTimedOperation)
-SCF_IMPLEMENT_IBASE_END
-
 /**
  * Fade fog operation.
  */
@@ -171,36 +157,33 @@ private:
   float end_density;
   csTicks duration;
   iEngineSequenceManager* eseqmgr;
+  uint sequence_id;
 
 public:
   OpFadeFog (iParameterESM* sectorpar, const csColor& color, float density,
-  	csTicks duration, iEngineSequenceManager* eseqmgr)
+  	csTicks duration, iEngineSequenceManager* eseqmgr, uint sequence_id) :
+    end_col (color), end_density (density), duration (duration),
+    eseqmgr (eseqmgr), sequence_id (sequence_id)
   {
     if (sectorpar->IsConstant ())
-      sector = SCF_QUERY_INTERFACE (sectorpar->GetValue (), iSector);
+      sector = scfQueryInterface<iSector> (sectorpar->GetValue ());
     else
       OpFadeFog::sectorpar = sectorpar;
-    OpFadeFog::end_col = color;
-    OpFadeFog::end_density = density;
-    OpFadeFog::duration = duration;
-    OpFadeFog::eseqmgr = eseqmgr;
   }
 
   virtual void Do (csTicks dt, iBase* params)
   {
     if (sectorpar)
-      sector = SCF_QUERY_INTERFACE (sectorpar->GetValue (params), iSector);
-    csFog* fog = sector->GetFog ();
+      sector = scfQueryInterface<iSector> (sectorpar->GetValue (params));
+    const csFog& fog = sector->GetFog ();
 
     FadeFogInfo* fi = new FadeFogInfo ();
-    fi->start_col.red = fog->red;
-    fi->start_col.green = fog->green;
-    fi->start_col.blue = fog->blue;
-    fi->start_density = fog->density;
+    fi->start_col = fog.color;
     fi->end_col = end_col;
+    fi->start_density = fog.density;
     fi->end_density = end_density;
     fi->sector = sector;
-    eseqmgr->FireTimedOperation (dt, duration, fi);
+    eseqmgr->FireTimedOperation (dt, duration, fi, 0, sequence_id);
     fi->DecRef ();
     if (sectorpar)
       sector = 0;
@@ -330,20 +313,20 @@ public:
     if (meshpar)
     {
       if (meshpar->IsConstant ())
-        mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (), iMeshWrapper);
+        mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue ());
       else
         OpSetMaterial::meshpar = meshpar;
     }
     if (polygonpar)
     {
       if (polygonpar->IsConstant ())
-        polygon = SCF_QUERY_INTERFACE (polygonpar->GetValue (), iPolygonHandle);
+        polygon = scfQueryInterface<iPolygonHandle> (polygonpar->GetValue ());
       else
         OpSetMaterial::polygonpar = polygonpar;
     }
     if (materialpar->IsConstant ())
-      material = SCF_QUERY_INTERFACE (
-      	materialpar->GetValue (), iMaterialWrapper);
+      material = scfQueryInterface<iMaterialWrapper> (
+      	materialpar->GetValue ());
     else
       OpSetMaterial::materialpar = materialpar;
   }
@@ -351,13 +334,13 @@ public:
   virtual void Do (csTicks /*dt*/, iBase* params)
   {
     if (materialpar)
-      material = SCF_QUERY_INTERFACE (
-      	materialpar->GetValue (params), iMaterialWrapper);
+      material = scfQueryInterface<iMaterialWrapper> (
+      	materialpar->GetValue (params));
     if (polygon || polygonpar)
     {
       if (polygonpar)
-        polygon = SCF_QUERY_INTERFACE (polygonpar->GetValue (params),
-		iPolygonHandle);
+        polygon = 
+		scfQueryInterface<iPolygonHandle> (polygonpar->GetValue (params));
       int poly_idx = polygon->GetIndex ();
       iThingFactoryState* tfs = polygon->GetThingFactoryState ();
       if (tfs)
@@ -370,7 +353,7 @@ public:
     else
     {
       if (meshpar)
-        mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (params), iMeshWrapper);
+        mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue (params));
       mesh->GetMeshObject ()->SetMaterialWrapper (material);
       if (meshpar)
         mesh = 0;
@@ -396,7 +379,7 @@ public:
   OpSetLight (iParameterESM* lightpar, const csColor& color)
   {
     if (lightpar->IsConstant ())
-      light = SCF_QUERY_INTERFACE (lightpar->GetValue (), iLight);
+      light = scfQueryInterface<iLight> (lightpar->GetValue ());
     else
       OpSetLight::lightpar = lightpar;
     OpSetLight::color = color;
@@ -405,24 +388,24 @@ public:
   virtual void Do (csTicks /*dt*/, iBase* params)
   {
     if (lightpar)
-      light = SCF_QUERY_INTERFACE (lightpar->GetValue (params), iLight);
+      light = scfQueryInterface<iLight> (lightpar->GetValue (params));
     light->SetColor (color);
     if (lightpar)
       light = 0;
   }
 };
 
-class FadeLightInfo : public iSequenceTimedOperation
+class FadeLightInfo : 
+  public scfImplementation1<FadeLightInfo, iSequenceTimedOperation>
 {
 public:
   csRef<iLight> light;
   csColor start_col, end_col;
 
-  SCF_DECLARE_IBASE;
-  FadeLightInfo ()
-  { SCF_CONSTRUCT_IBASE (0); }
+  FadeLightInfo () : scfImplementationType (this)
+  { }
   virtual ~FadeLightInfo ()
-  { SCF_DESTRUCT_IBASE(); }
+  { }
 
   virtual void Do (float time, iBase*)
   {
@@ -434,10 +417,6 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (FadeLightInfo)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceTimedOperation)
-SCF_IMPLEMENT_IBASE_END
-
 /**
  * Fade light operation.
  */
@@ -449,29 +428,30 @@ private:
   csColor end_col;
   csTicks duration;
   iEngineSequenceManager* eseqmgr;
+  uint sequence_id;
 
 public:
   OpFadeLight (iParameterESM* lightpar, const csColor& color,
-  	csTicks duration, iEngineSequenceManager* eseqmgr)
+  	csTicks duration, iEngineSequenceManager* eseqmgr,
+	uint sequence_id) :
+    end_col (color), duration (duration), eseqmgr (eseqmgr),
+    sequence_id (sequence_id)
   {
     if (lightpar->IsConstant ())
-      light = SCF_QUERY_INTERFACE (lightpar->GetValue (), iLight);
+      light = scfQueryInterface<iLight> (lightpar->GetValue ());
     else
       OpFadeLight::lightpar = lightpar;
-    OpFadeLight::end_col = color;
-    OpFadeLight::duration = duration;
-    OpFadeLight::eseqmgr = eseqmgr;
   }
 
   virtual void Do (csTicks dt, iBase* params)
   {
     if (lightpar)
-      light = SCF_QUERY_INTERFACE (lightpar->GetValue (params), iLight);
+      light = scfQueryInterface<iLight> (lightpar->GetValue (params));
     FadeLightInfo* fl = new FadeLightInfo ();
     fl->light = light;
     fl->start_col = light->GetColor ();
     fl->end_col = end_col;
-    eseqmgr->FireTimedOperation (dt, duration, fl);
+    eseqmgr->FireTimedOperation (dt, duration, fl, 0, sequence_id);
     fl->DecRef ();
     if (lightpar)
       light = 0;
@@ -496,7 +476,7 @@ public:
   OpSetAmbientLight (iParameterESM* sectorpar, const csColor& color,
 		     iSharedVariable *varcolor)
   {
-    sector = SCF_QUERY_INTERFACE (sectorpar->GetValue (), iSector);
+    sector = scfQueryInterface<iSector> (sectorpar->GetValue ());
     if (varcolor)
       colorvar = varcolor;
     else
@@ -512,17 +492,17 @@ public:
   }
 };
 
-class FadeAmbientLightInfo : public iSequenceTimedOperation
+class FadeAmbientLightInfo : 
+  public scfImplementation1<FadeAmbientLightInfo, iSequenceTimedOperation>
 {
 public:
   csRef<iSector> sector;
   csColor start_col, end_col;
 
-  SCF_DECLARE_IBASE;
-  FadeAmbientLightInfo ()
-  { SCF_CONSTRUCT_IBASE (0); }
+  FadeAmbientLightInfo () : scfImplementationType (this)
+  { }
   virtual ~FadeAmbientLightInfo ()
-  { SCF_DESTRUCT_IBASE(); }
+  { }
 
   virtual void Do (float time, iBase*)
   {
@@ -534,10 +514,6 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (FadeAmbientLightInfo)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceTimedOperation)
-SCF_IMPLEMENT_IBASE_END
-
 /**
  * Fade light operation.
  */
@@ -548,15 +524,16 @@ private:
   csColor end_col;
   csTicks duration;
   iEngineSequenceManager* eseqmgr;
+  uint sequence_id;
 
 public:
   OpFadeAmbientLight (iParameterESM* sectorpar, const csColor& color,
-  	csTicks duration, iEngineSequenceManager* eseqmgr)
+  	csTicks duration, iEngineSequenceManager* eseqmgr,
+	uint sequence_id) :
+    end_col (color), duration (duration), eseqmgr (eseqmgr),
+    sequence_id (sequence_id)
   {
-    sector = SCF_QUERY_INTERFACE (sectorpar->GetValue (), iSector);
-    OpFadeAmbientLight::end_col = color;
-    OpFadeAmbientLight::duration = duration;
-    OpFadeAmbientLight::eseqmgr = eseqmgr;
+    sector = scfQueryInterface<iSector> (sectorpar->GetValue ());
   }
 
   virtual void Do (csTicks dt, iBase* /*params*/)
@@ -565,7 +542,7 @@ public:
     fl->sector = sector;
     fl->start_col = sector->GetDynamicAmbientLight ();
     fl->end_col = end_col;
-    eseqmgr->FireTimedOperation (dt, duration, fl);
+    eseqmgr->FireTimedOperation (dt, duration, fl, 0, sequence_id);
     fl->DecRef ();
   }
 };
@@ -586,7 +563,7 @@ public:
   OpSetMeshColor (iParameterESM* meshpar, const csColor& color)
   {
     if (meshpar->IsConstant ())
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue ());
     else
       OpSetMeshColor::meshpar = meshpar;
     OpSetMeshColor::color = color;
@@ -595,25 +572,25 @@ public:
   virtual void Do (csTicks /*dt*/, iBase* params)
   {
     if (meshpar)
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (params), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue (params));
     mesh->GetMeshObject ()->SetColor (color);
     if (meshpar)
       mesh = 0;
   }
 };
 
-class FadeMeshColorInfo : public iSequenceTimedOperation
+class FadeMeshColorInfo : 
+  public scfImplementation1<FadeMeshColorInfo, iSequenceTimedOperation>
 {
 public:
   csRef<iMeshWrapper> mesh;
   csColor start_col, end_col;
   float start_density, end_density;
 
-  SCF_DECLARE_IBASE;
-  FadeMeshColorInfo ()
-  { SCF_CONSTRUCT_IBASE (0); }
+  FadeMeshColorInfo () : scfImplementationType (this)
+  { }
   virtual ~FadeMeshColorInfo ()
-  { SCF_DESTRUCT_IBASE(); }
+  { }
 
   virtual void Do (float time, iBase*)
   {
@@ -625,10 +602,6 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (FadeMeshColorInfo)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceTimedOperation)
-SCF_IMPLEMENT_IBASE_END
-
 /**
  * Fade mesh operation.
  */
@@ -640,29 +613,30 @@ private:
   csColor end_col;
   csTicks duration;
   iEngineSequenceManager* eseqmgr;
+  uint sequence_id;
 
 public:
   OpFadeMeshColor (iParameterESM* meshpar, const csColor& color,
-  	csTicks duration, iEngineSequenceManager* eseqmgr)
+  	csTicks duration, iEngineSequenceManager* eseqmgr,
+	uint sequence_id) :
+    end_col (color), duration (duration), eseqmgr (eseqmgr),
+    sequence_id (sequence_id)
   {
     if (meshpar->IsConstant ())
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue ());
     else
       OpFadeMeshColor::meshpar = meshpar;
-    OpFadeMeshColor::end_col = color;
-    OpFadeMeshColor::duration = duration;
-    OpFadeMeshColor::eseqmgr = eseqmgr;
   }
 
   virtual void Do (csTicks dt, iBase* params)
   {
     if (meshpar)
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (params), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue (params));
     FadeMeshColorInfo* fm = new FadeMeshColorInfo ();
     fm->mesh = mesh;
     mesh->GetMeshObject ()->GetColor (fm->start_col);
     fm->end_col = end_col;
-    eseqmgr->FireTimedOperation (dt, duration, fm);
+    eseqmgr->FireTimedOperation (dt, duration, fm, 0, sequence_id);
     fm->DecRef ();
     if (meshpar)
       mesh = 0;
@@ -671,7 +645,8 @@ public:
 
 //---------------------------------------------------------------------------
 
-class RotateInfo : public iSequenceTimedOperation
+class RotateInfo : 
+  public scfImplementation1<RotateInfo, iSequenceTimedOperation>
 {
 public:
   csRef<iMovable> movable;
@@ -680,11 +655,10 @@ public:
   csVector3 offset;
   csReversibleTransform start_transform;
 
-  SCF_DECLARE_IBASE;
-  RotateInfo ()
-  { SCF_CONSTRUCT_IBASE (0); }
+  RotateInfo () : scfImplementationType (this)
+  { }
   virtual ~RotateInfo ()
-  { SCF_DESTRUCT_IBASE(); }
+  { }
 
   virtual void Do (float time, iBase*)
   {
@@ -739,10 +713,6 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (RotateInfo)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceTimedOperation)
-SCF_IMPLEMENT_IBASE_END
-
 /**
  * Rotate operation.
  */
@@ -757,6 +727,7 @@ private:
   csVector3 offset;
   csTicks duration;
   iEngineSequenceManager* eseqmgr;
+  uint sequence_id;
 
 public:
   OpRotate (iParameterESM* meshpar,
@@ -764,34 +735,30 @@ public:
   	int axis2, float tot_angle2,
   	int axis3, float tot_angle3,
 	const csVector3& offset,
-  	csTicks duration, iEngineSequenceManager* eseqmgr)
+  	csTicks duration, iEngineSequenceManager* eseqmgr,
+	uint sequence_id) :
+    axis1 (axis1), axis2 (axis2), axis3 (axis3),
+    tot_angle1 (tot_angle1), tot_angle2 (tot_angle2), tot_angle3 (tot_angle3),
+    offset (offset), duration (duration),
+    eseqmgr (eseqmgr), sequence_id (sequence_id)
   {
     if (meshpar->IsConstant ())
     {
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue ());
       if (!mesh)
-        light = SCF_QUERY_INTERFACE (meshpar->GetValue (), iLight);
+        light = scfQueryInterface<iLight> (meshpar->GetValue ());
     }
     else
       OpRotate::meshpar = meshpar;
-    OpRotate::axis1 = axis1;
-    OpRotate::tot_angle1 = tot_angle1;
-    OpRotate::axis2 = axis2;
-    OpRotate::tot_angle2 = tot_angle2;
-    OpRotate::axis3 = axis3;
-    OpRotate::tot_angle3 = tot_angle3;
-    OpRotate::offset = offset;
-    OpRotate::duration = duration;
-    OpRotate::eseqmgr = eseqmgr;
   }
 
   virtual void Do (csTicks dt, iBase* params)
   {
     if (meshpar)
     {
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (params), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue (params));
       if (!mesh)
-        light = SCF_QUERY_INTERFACE (meshpar->GetValue (), iLight);
+        light = scfQueryInterface<iLight> (meshpar->GetValue ());
     }
     iMovable* movable = NULL;
     if (mesh)
@@ -810,7 +777,7 @@ public:
       ri->tot_angle2 = tot_angle2;
       ri->tot_angle3 = tot_angle3;
       ri->offset = offset;
-      eseqmgr->FireTimedOperation (dt, duration, ri);
+      eseqmgr->FireTimedOperation (dt, duration, ri, 0, sequence_id);
       ri->DecRef ();
     }
 
@@ -824,18 +791,18 @@ public:
 
 //---------------------------------------------------------------------------
 
-class MoveLightInfo : public iSequenceTimedOperation
+class MoveLightInfo : 
+  public scfImplementation1<MoveLightInfo, iSequenceTimedOperation>
 {
 public:
   csRef<iLight> light;
   csVector3 start_pos;
   csVector3 offset;
 
-  SCF_DECLARE_IBASE;
-  MoveLightInfo ()
-  { SCF_CONSTRUCT_IBASE (0); }
+  MoveLightInfo () : scfImplementationType (this)
+  { }
   virtual ~MoveLightInfo ()
-  { SCF_DESTRUCT_IBASE(); }
+  { }
 
   virtual void Do (float time, iBase*)
   {
@@ -844,22 +811,18 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (MoveLightInfo)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceTimedOperation)
-SCF_IMPLEMENT_IBASE_END
-
-class MoveInfo : public iSequenceTimedOperation
+class MoveInfo : 
+  public scfImplementation1<MoveInfo, iSequenceTimedOperation>
 {
 public:
   csRef<iMeshWrapper> mesh;
   csVector3 start_pos;
   csVector3 offset;
 
-  SCF_DECLARE_IBASE;
-  MoveInfo ()
-  { SCF_CONSTRUCT_IBASE (0); }
+  MoveInfo () : scfImplementationType (this)
+  { }
   virtual ~MoveInfo ()
-  { SCF_DESTRUCT_IBASE(); }
+  { }
 
   virtual void Do (float time, iBase*)
   {
@@ -869,10 +832,6 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (MoveInfo)
-  SCF_IMPLEMENTS_INTERFACE (iSequenceTimedOperation)
-SCF_IMPLEMENT_IBASE_END
-
 /**
  * Move operation.
  */
@@ -885,34 +844,35 @@ private:
   csVector3 offset;
   csTicks duration;
   iEngineSequenceManager* eseqmgr;
+  uint sequence_id;
 
 public:
   OpMove (iParameterESM* meshpar,
 	const csVector3& offset,
-  	csTicks duration, iEngineSequenceManager* eseqmgr)
+  	csTicks duration, iEngineSequenceManager* eseqmgr,
+	uint sequence_id) :
+    offset (offset), duration (duration), eseqmgr (eseqmgr),
+    sequence_id (sequence_id)
   {
     if (meshpar->IsConstant ())
     {
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue ());
       if (!mesh)
-        light = SCF_QUERY_INTERFACE (meshpar->GetValue (), iLight);
+        light = scfQueryInterface<iLight> (meshpar->GetValue ());
     }
     else
     {
       OpMove::meshpar = meshpar;
     }
-    OpMove::offset = offset;
-    OpMove::duration = duration;
-    OpMove::eseqmgr = eseqmgr;
   }
 
   virtual void Do (csTicks dt, iBase* params)
   {
     if (meshpar)
     {
-      mesh = SCF_QUERY_INTERFACE (meshpar->GetValue (params), iMeshWrapper);
+      mesh = scfQueryInterface<iMeshWrapper> (meshpar->GetValue (params));
       if (!mesh)
-        light = SCF_QUERY_INTERFACE (meshpar->GetValue (params), iLight);
+        light = scfQueryInterface<iLight> (meshpar->GetValue (params));
     }
     if (mesh)
     {
@@ -921,7 +881,7 @@ public:
       mi->mesh = mesh;
       mi->start_pos = movable->GetTransform ().GetOrigin ();
       mi->offset = offset;
-      eseqmgr->FireTimedOperation (dt, duration, mi);
+      eseqmgr->FireTimedOperation (dt, duration, mi, 0, sequence_id);
       mi->DecRef ();
     }
     else if (light)
@@ -930,7 +890,7 @@ public:
       mi->light = light;
       mi->start_pos = light->GetCenter ();
       mi->offset = offset;
-      eseqmgr->FireTimedOperation (dt, duration, mi);
+      eseqmgr->FireTimedOperation (dt, duration, mi, 0, sequence_id);
       mi->DecRef ();
     }
     if (meshpar)
@@ -990,7 +950,7 @@ public:
   OpTriggerState (iParameterESM* triggerpar, bool en)
   {
     if (triggerpar->IsConstant ())
-      trigger = SCF_QUERY_INTERFACE (triggerpar->GetValue (), iSequenceTrigger);
+      trigger = scfQueryInterface<iSequenceTrigger> (triggerpar->GetValue ());
     else
       OpTriggerState::triggerpar = triggerpar;
     OpTriggerState::en = en;
@@ -999,8 +959,8 @@ public:
   virtual void Do (csTicks /*dt*/, iBase* params)
   {
     if (triggerpar)
-      trigger = SCF_QUERY_INTERFACE (
-      	triggerpar->GetValue (params), iSequenceTrigger);
+      trigger = scfQueryInterface<iSequenceTrigger> (
+      	triggerpar->GetValue (params));
     if (!trigger) return;
     trigger->SetEnabled (en);
     if (triggerpar)
@@ -1024,7 +984,7 @@ public:
   OpCheckTrigger (iParameterESM* triggerpar, csTicks delay)
   {
     if (triggerpar->IsConstant ())
-      trigger = SCF_QUERY_INTERFACE (triggerpar->GetValue (), iSequenceTrigger);
+      trigger = scfQueryInterface<iSequenceTrigger> (triggerpar->GetValue ());
     else
       OpCheckTrigger::triggerpar = triggerpar;
     OpCheckTrigger::delay = delay;
@@ -1033,8 +993,8 @@ public:
   virtual void Do (csTicks /*dt*/, iBase* params)
   {
     if (triggerpar)
-      trigger = SCF_QUERY_INTERFACE (
-      	triggerpar->GetValue (params), iSequenceTrigger);
+      trigger = scfQueryInterface<iSequenceTrigger> (
+        triggerpar->GetValue (params));
     if (!trigger) return;
     trigger->TestConditions (delay);
     if (triggerpar)
@@ -1057,7 +1017,7 @@ public:
   CondTestTrigger (iParameterESM* triggerpar)
   {
     if (triggerpar->IsConstant ())
-      trigger = SCF_QUERY_INTERFACE (triggerpar->GetValue (), iSequenceTrigger);
+      trigger = scfQueryInterface<iSequenceTrigger> (triggerpar->GetValue ());
     else
       CondTestTrigger::triggerpar = triggerpar;
   }
@@ -1065,8 +1025,8 @@ public:
   virtual bool Condition (csTicks /*dt*/, iBase* params)
   {
     if (triggerpar)
-      trigger = SCF_QUERY_INTERFACE (
-      	triggerpar->GetValue (params), iSequenceTrigger);
+      trigger = scfQueryInterface<iSequenceTrigger> (
+        triggerpar->GetValue (params));
     if (!trigger) return false;
     bool rc = trigger->CheckState ();
     if (triggerpar)
@@ -1077,28 +1037,24 @@ public:
 
 //---------------------------------------------------------------------------
 
-class esmPar : public iParameterESM
+class esmPar : public scfImplementation1<esmPar, iParameterESM>
 {
 private:
   size_t idx;
 
 public:
-  SCF_DECLARE_IBASE;
-  esmPar (size_t idx)
+  esmPar (size_t idx) : scfImplementationType (this)
   {
-    SCF_CONSTRUCT_IBASE (0);
     esmPar::idx = idx;
   }
   virtual ~esmPar ()
   {
-    SCF_DESTRUCT_IBASE();
   }
   virtual iBase* GetValue (iBase* params = 0) const
   {
-    // The following cast is in theory unsafe but in this case it is
-    // ok since there is no way some other 'params' instance could get
-    // here.
-    iEngineSequenceParameters* par = (iEngineSequenceParameters*)params;
+    if (!params) return 0;
+    csRef<iEngineSequenceParameters> par = 
+      scfQueryInterface<iEngineSequenceParameters> (params);
     return par->GetParameter (idx);
   }
   virtual bool IsConstant () const
@@ -1107,25 +1063,18 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (esmPar)
-  SCF_IMPLEMENTS_INTERFACE (iParameterESM)
-SCF_IMPLEMENT_IBASE_END
-
-class constantPar : public iParameterESM
+class constantPar : public scfImplementation1<constantPar, iParameterESM>
 {
 private:
   csWeakRef<iBase> value;
 
 public:
-  SCF_DECLARE_IBASE;
-  constantPar (iBase* value)
+  constantPar (iBase* value) : scfImplementationType (this)
   {
-    SCF_CONSTRUCT_IBASE (0);
     constantPar::value = value;
   }
   virtual ~constantPar ()
   {
-    SCF_DESTRUCT_IBASE();
   }
   virtual iBase* GetValue (iBase* params = 0) const
   {
@@ -1138,16 +1087,8 @@ public:
   }
 };
 
-SCF_IMPLEMENT_IBASE (constantPar)
-  SCF_IMPLEMENTS_INTERFACE (iParameterESM)
-SCF_IMPLEMENT_IBASE_END
-
 //---------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csEngineSequenceParameters)
-  SCF_IMPLEMENTS_INTERFACE (iEngineSequenceParameters)
-SCF_IMPLEMENT_IBASE_END
-
 csPtr<iParameterESM> csEngineSequenceParameters::CreateParameterESM (
 	const char* name)
 {
@@ -1167,13 +1108,14 @@ csSequenceWrapper::csSequenceWrapper (csEngineSequenceManager* eseqmgr,
 
 csSequenceWrapper::~csSequenceWrapper ()
 {
+  eseqmgr->DestroyTimedOperations (sequence_id);
   if (eseqmgr->GetSequenceManager ())
     eseqmgr->GetSequenceManager ()->DestroySequenceOperations (sequence_id);
 }
 
 void csSequenceWrapper::SelfDestruct ()
 {
-  eseqmgr->RemoveSequence ((iSequenceWrapper*)this);
+  eseqmgr->RemoveSequence (static_cast<iSequenceWrapper*> (this));
 }
 
 iEngineSequenceParameters* csSequenceWrapper::CreateBaseParameterBlock ()
@@ -1264,7 +1206,8 @@ void csSequenceWrapper::AddOperationSetLight (csTicks time,
 void csSequenceWrapper::AddOperationFadeLight (csTicks time,
 	iParameterESM* light, const csColor& color, csTicks duration)
 {
-  OpFadeLight* op = new OpFadeLight (light, color, duration, eseqmgr);
+  OpFadeLight* op = new OpFadeLight (light, color, duration, eseqmgr,
+      sequence_id);
   sequence->AddOperation (time, op, 0, sequence_id);
   op->DecRef ();
 }
@@ -1281,7 +1224,7 @@ void csSequenceWrapper::AddOperationFadeAmbient (csTicks time,
 	iParameterESM* sector, const csColor& color, csTicks duration)
 {
   OpFadeAmbientLight* op = new OpFadeAmbientLight (sector, color, duration,
-  	eseqmgr);
+  	eseqmgr, sequence_id);
   sequence->AddOperation (time, op, 0, sequence_id);
   op->DecRef ();
 }
@@ -1304,7 +1247,8 @@ void csSequenceWrapper::AddOperationSetMeshColor (csTicks time,
 void csSequenceWrapper::AddOperationFadeMeshColor (csTicks time,
 	iParameterESM* mesh, const csColor& color, csTicks duration)
 {
-  OpFadeMeshColor* op = new OpFadeMeshColor (mesh, color, duration, eseqmgr);
+  OpFadeMeshColor* op = new OpFadeMeshColor (mesh, color, duration, eseqmgr,
+      sequence_id);
   sequence->AddOperation (time, op, 0, sequence_id);
   op->DecRef ();
 }
@@ -1321,7 +1265,8 @@ void csSequenceWrapper::AddOperationFadeFog (csTicks time,
 	iParameterESM* sector, const csColor& color, float density,
 	csTicks duration)
 {
-  OpFadeFog* op = new OpFadeFog (sector, color, density, duration, eseqmgr);
+  OpFadeFog* op = new OpFadeFog (sector, color, density, duration, eseqmgr,
+      sequence_id);
   sequence->AddOperation (time, op, 0, sequence_id);
   op->DecRef ();
 }
@@ -1335,7 +1280,7 @@ void csSequenceWrapper::AddOperationRotateDuration (csTicks time,
 {
   OpRotate* op = new OpRotate (mesh,
   	axis1, tot_angle1, axis2, tot_angle2, axis3, tot_angle3,
-	offset, duration, eseqmgr);
+	offset, duration, eseqmgr, sequence_id);
   sequence->AddOperation (time, op, 0, sequence_id);
   op->DecRef ();
 }
@@ -1345,7 +1290,7 @@ void csSequenceWrapper::AddOperationMoveDuration (csTicks time,
 	csTicks duration)
 {
   OpMove* op = new OpMove (mesh, offset,
-	duration, eseqmgr);
+	duration, eseqmgr, sequence_id);
   sequence->AddOperation (time, op, 0, sequence_id);
   op->DecRef ();
 }
@@ -1446,7 +1391,7 @@ public:
 
   virtual void Traverse (iSector* /*sector*/, iBase* context)
   {
-    csRef<iRenderView> rview (SCF_QUERY_INTERFACE (context, iRenderView));
+    csRef<iRenderView> rview (scfQueryInterface<iRenderView> (context));
     if (rview)
     {
       uint32 global_framenr = trigger->GetEngineSequenceManager ()
@@ -1485,7 +1430,8 @@ public:
 /**
  * Callback that will activate trigger when light crosses threshold value.
  */
-class csTriggerLightCallback : public iLightCallback
+class csTriggerLightCallback : 
+  public scfImplementation1<csTriggerLightCallback, iLightCallback>
 {
 private:
   csSequenceTrigger* trigger;
@@ -1494,12 +1440,9 @@ private:
   unsigned int framenr;
 
 public:
-  SCF_DECLARE_IBASE;
-
   csTriggerLightCallback (csSequenceTrigger* trigger,
-	int oper, const csColor& col)
+    int oper, const csColor& col) : scfImplementationType (this)
   {
-    SCF_CONSTRUCT_IBASE (0);
     csTriggerLightCallback::trigger = trigger;
     operation = oper;
     trigger_color = col;
@@ -1507,7 +1450,6 @@ public:
   }
   virtual ~csTriggerLightCallback ()
   {
-    SCF_DESTRUCT_IBASE();
   }
 
   float AverageColor (const csColor& col)
@@ -1543,10 +1485,6 @@ public:
   virtual void OnAttenuationChange (iLight* /*light*/, int /*newatt*/) { }
 };
 
-SCF_IMPLEMENT_IBASE (csTriggerLightCallback)
-  SCF_IMPLEMENTS_INTERFACE (iLightCallback)
-SCF_IMPLEMENT_IBASE_END
-
 //---------------------------------------------------------------------------
 
 /**
@@ -1622,7 +1560,7 @@ csSequenceTrigger::~csSequenceTrigger ()
 
 void csSequenceTrigger::SelfDestruct ()
 {
-  eseqmgr->RemoveTrigger ((iSequenceTrigger*)this);
+  eseqmgr->RemoveTrigger (static_cast<iSequenceTrigger*> (this));
 }
 
 void csSequenceTrigger::AddConditionInSector (iSector* sector,
@@ -1727,7 +1665,7 @@ void csSequenceTrigger::Fire ()
       last_trigger_state = true;
       // Only fire if trigger is enabled. Otherwise we are only
       // doing the test.
-      csSequenceWrapper* wf = (csSequenceWrapper*)fire_sequence;
+      csSequenceWrapper* wf = static_cast<csSequenceWrapper*> (fire_sequence);
       eseqmgr->GetSequenceManager ()->RunSequence (fire_delay,
     	  fire_sequence->GetSequence (), params, wf->GetSequenceID ());
       enabled = false;
@@ -1766,7 +1704,7 @@ void csSequenceTrigger::Fire ()
 
 void csSequenceTrigger::ForceFire (bool now)
 {
-  csSequenceWrapper* wf = (csSequenceWrapper*)fire_sequence;
+  csSequenceWrapper* wf = static_cast<csSequenceWrapper*> (fire_sequence);
   eseqmgr->GetSequenceManager ()->RunSequence (now ? 0 : fire_delay,
     	  fire_sequence->GetSequence (), params, wf->GetSequenceID ());
 }
@@ -1828,24 +1766,9 @@ void csSequenceTrigger::TestConditions (csTicks delay)
 SCF_IMPLEMENT_FACTORY (csEngineSequenceManager)
 
 
-SCF_IMPLEMENT_IBASE (csEngineSequenceManager)
-  SCF_IMPLEMENTS_INTERFACE (iEngineSequenceManager)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csEngineSequenceManager::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csEngineSequenceManager::EventHandler)
-  SCF_IMPLEMENTS_INTERFACE (iEventHandler)
-SCF_IMPLEMENT_IBASE_END
-
-csEngineSequenceManager::csEngineSequenceManager (iBase *iParent)
+csEngineSequenceManager::csEngineSequenceManager (iBase *iParent) :
+  scfImplementationType (this, iParent)
 {
-  SCF_CONSTRUCT_IBASE (iParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
-  scfiEventHandler = 0;
   object_reg = 0;
   global_framenr = 1;
   debug_eseqmgr = this;//@@@@@@@@@@@@@
@@ -1853,33 +1776,29 @@ csEngineSequenceManager::csEngineSequenceManager (iBase *iParent)
 
 csEngineSequenceManager::~csEngineSequenceManager ()
 {
-  if (scfiEventHandler)
+  if (eventHandler.IsValid())
   {
-    csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+    csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
     if (q != 0)
-      q->RemoveListener (scfiEventHandler);
-    scfiEventHandler->DecRef ();
+      q->RemoveListener (eventHandler);
   }
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_IBASE();
 }
 
 bool csEngineSequenceManager::Initialize (iObjectRegistry *r)
 {
   object_reg = r;
-  if (!scfiEventHandler)
-    scfiEventHandler = new EventHandler (this);
+  eventHandler.AttachNew (new EventHandler (this));
   PostProcess = csevPostProcess (object_reg);
   MouseEvent = csevMouseEvent (object_reg);
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
   if (q != 0)
   {
     csEventID events[3] = { PostProcess, MouseEvent, CS_EVENTLIST_END };
-    q->RegisterListener (scfiEventHandler, events);
+    q->RegisterListener (eventHandler, events);
   }
 
-  csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (object_reg,
-  	iPluginManager));
+  csRef<iPluginManager> plugin_mgr (
+  	csQueryRegistry<iPluginManager> (object_reg));
   seqmgr = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.utilities.sequence",
   	iSequenceManager);
   if (!seqmgr)
@@ -1914,7 +1833,7 @@ bool csEngineSequenceManager::HandleEvent (iEvent &event)
     global_framenr++;
 
     csTicks curtime = seqmgr->GetMainTime () + seqmgr->GetDeltaTime ();
-    size_t i = timed_operations.Length ();
+    size_t i = timed_operations.GetSize ();
     while (i-- > 0)
     {
       csTimedOperation* op = timed_operations[i];
@@ -1938,25 +1857,25 @@ bool csEngineSequenceManager::HandleEvent (iEvent &event)
     int mouse_x = csMouseEventHelper::GetX(&event);
     int mouse_y = csMouseEventHelper::GetY(&event);
     iCamera* camera = cameracatcher->camera;
-    if (camera != 0 && mesh_triggers.Length () > 0)
+    if (camera != 0 && mesh_triggers.GetSize () > 0)
     {
       csVector3 v;
       // Setup perspective vertex, invert mouse Y axis.
       csVector2 p (mouse_x, camera->GetShiftY() * 2 - mouse_y);
 
-      camera->InvPerspective (p, 1, v);
+      v = camera->InvPerspective (p, 1);
       csVector3 vw = camera->GetTransform ().This2Other (v);
 
       iSector* sector = camera->GetSector ();
       if (!sector) return false;
       csVector3 origin = camera->GetTransform ().GetO2TTranslation ();
-      csVector3 isect, end = origin + (vw - origin) * 120;
-
-      int polyidx = -1;
-      iMeshWrapper* sel = sector->HitBeam (origin, end, isect, &polyidx);
+      csVector3 end = origin + (vw - origin) * 120;
+     
+      csSectorHitBeamResult hitBeamResult = sector->HitBeam (origin, end);
+      iMeshWrapper * sel = hitBeamResult.mesh;
 
       size_t i;
-      for (i = 0 ; i < mesh_triggers.Length () ; i++)
+      for (i = 0 ; i < mesh_triggers.GetSize () ; i++)
       {
 	if (mesh_triggers[i]->GetClickMesh () == sel)
 	  mesh_triggers[i]->Fire ();
@@ -1987,8 +1906,8 @@ csPtr<iSequenceTrigger> csEngineSequenceManager::CreateTrigger (
 {
   csSequenceTrigger* trig = new csSequenceTrigger (this);
   trig->SetName (name);
-  triggers.Push ((iSequenceTrigger*)trig);
-  return (iSequenceTrigger*)trig;
+  triggers.Push (static_cast<iSequenceTrigger*> (trig));
+  return static_cast<iSequenceTrigger*> (trig);
 }
 
 csPtr<iParameterESM> csEngineSequenceManager::CreateParameterESM (iBase* value)
@@ -1998,7 +1917,7 @@ csPtr<iParameterESM> csEngineSequenceManager::CreateParameterESM (iBase* value)
 
 void csEngineSequenceManager::RemoveTrigger (iSequenceTrigger* trigger)
 {
-  mesh_triggers.Delete ((csSequenceTrigger*)trigger);
+  mesh_triggers.Delete (static_cast<csSequenceTrigger*> (trigger));
   triggers.Delete (trigger);
 }
 
@@ -2010,7 +1929,7 @@ void csEngineSequenceManager::RemoveTriggers ()
 
 size_t csEngineSequenceManager::GetTriggerCount () const
 {
-  return triggers.Length ();
+  return triggers.GetSize ();
 }
 
 iSequenceTrigger* csEngineSequenceManager::GetTrigger (size_t idx) const
@@ -2022,7 +1941,7 @@ iSequenceTrigger* csEngineSequenceManager::FindTriggerByName (
 	const char* name) const
 {
   size_t i;
-  for (i = 0 ; i < triggers.Length () ; i++)
+  for (i = 0 ; i < triggers.GetSize () ; i++)
   {
     if (!strcmp (name, triggers[i]->QueryObject ()->GetName ()))
       return triggers[i];
@@ -2049,8 +1968,8 @@ csPtr<iSequenceWrapper> csEngineSequenceManager::CreateSequence (
   csSequenceWrapper* seqwrap = new csSequenceWrapper (this, seq,
       seqmgr->GetUniqueID ());
   seqwrap->SetName (name);
-  sequences.Push ((iSequenceWrapper*)seqwrap);
-  return (iSequenceWrapper*)seqwrap;
+  sequences.Push (static_cast<iSequenceWrapper*> (seqwrap));
+  return static_cast<iSequenceWrapper*> (seqwrap);
 }
 
 void csEngineSequenceManager::RemoveSequence (iSequenceWrapper* seq)
@@ -2065,7 +1984,7 @@ void csEngineSequenceManager::RemoveSequences ()
 
 size_t csEngineSequenceManager::GetSequenceCount () const
 {
-  return sequences.Length ();
+  return sequences.GetSize ();
 }
 
 iSequenceWrapper* csEngineSequenceManager::GetSequence (size_t idx) const
@@ -2077,7 +1996,7 @@ iSequenceWrapper* csEngineSequenceManager::FindSequenceByName (
 	const char* name) const
 {
   size_t i;
-  for (i = 0 ; i < sequences.Length () ; i++)
+  for (i = 0 ; i < sequences.GetSize () ; i++)
   {
     if (!strcmp (name, sequences[i]->QueryObject ()->GetName ()))
       return sequences[i];
@@ -2091,7 +2010,7 @@ bool csEngineSequenceManager::RunSequenceByName (
   iSequenceWrapper *seq = FindSequenceByName(name);
   if (seq)
   {
-    csSequenceWrapper* wf = (csSequenceWrapper*)seq;
+    csSequenceWrapper* wf = static_cast<csSequenceWrapper*> (seq);
     seqmgr->RunSequence (delay, seq->GetSequence (), 0,
 	wf->GetSequenceID ());
     return true;
@@ -2100,7 +2019,8 @@ bool csEngineSequenceManager::RunSequenceByName (
 }
 
 void csEngineSequenceManager::FireTimedOperation (csTicks delta,
-	csTicks duration, iSequenceTimedOperation* op, iBase* params)
+	csTicks duration, iSequenceTimedOperation* op, iBase* params,
+	uint sequence_id)
 {
   csTicks curtime = seqmgr->GetMainTime ();
   if (delta >= duration)
@@ -2109,7 +2029,8 @@ void csEngineSequenceManager::FireTimedOperation (csTicks delta,
     return;	// Already done.
   }
 
-  csTimedOperation* top = new csTimedOperation (op, params);
+  csTimedOperation* top = new csTimedOperation (op, params,
+      sequence_id);
   top->start = curtime-delta;
   top->end = top->start + duration;
 
@@ -2117,4 +2038,22 @@ void csEngineSequenceManager::FireTimedOperation (csTicks delta,
   top->DecRef ();
 }
 
+void csEngineSequenceManager::DestroyTimedOperations (uint sequence_id)
+{
+  size_t i = 0;
+  while (i < timed_operations.GetSize ())
+  {
+    csTimedOperation* top = timed_operations[i];
+    if (top->GetSequenceID () == sequence_id)
+    {
+      timed_operations.DeleteIndex (i);
+    }
+    else
+    {
+      i++;
+    }
+  }
+}
 
+}
+CS_PLUGIN_NAMESPACE_END(EngSeq)
