@@ -22,34 +22,43 @@
 
 #include "csutil/cfgfile.h"
 #include "csutil/parray.h"
-#include "csutil/scopedmutexlock.h"
+#include "csutil/scf_implementation.h"
+#include "csutil/threading/mutex.h"
 #include "csutil/stringarray.h"
 #include "iutil/vfs.h"
 #include "iutil/eventh.h"
 #include "iutil/comp.h"		  
 
-class VfsNode;
-class csVFS;
 struct iConfigFile;
 
+namespace cspluginVFS
+{
+
+class VfsNode;
+class csVFS;
+
 /// A replacement for standard-C FILE type in the virtual file space
-class csFile : public iFile
+class csFile : public scfImplementation1<csFile, iFile>
 {
 protected:
   // Index into parent node RPath
-  int Index;
+  size_t Index;
   // File node
   VfsNode *Node;
   // Filename in VFS
   char *Name;
   // File size (initialized in constructor)
   size_t Size;
-  // last error code
+  // Last error code
   int Error;
+  // Verbosity flags.
+  unsigned int verbosity;
 
   // The constructor for csFile
-  csFile (int Mode, VfsNode *ParentNode, int RIndex,
-  	const char *NameSuffix);
+  csFile (int Mode, VfsNode *ParentNode, size_t RIndex,
+	  const char *NameSuffix, unsigned int verbosity);
+  // Is verbosity enabled for a mode or set of modes?
+  bool IsVerbose(unsigned int mask) const { return (verbosity & mask) != 0; }
 
 public:
   /// Instead of fclose() do "delete file" or file->DecRef ()
@@ -85,15 +94,14 @@ protected:
  * Space engine to access the files. This gives unified control over the
  * way how files are found, read and written. VFS gives the following
  * goodies over the standard file i/o functions:
- * <ul>
- * <li>Multiple search directories. Several "real" directories can be
- *     collected together into one "virtual" directory.
- * <li>Directories can be mapped to "real" directories as well as to
- *     archives (.zip files). Files are compressed/decompressed
- *     transparently for clients.
- * <li>The Virtual File System is unique across all operating systems
- *     Crystal Space supports, no matter of features of the underlying OS.
- * </ul>
+ * - Multiple search directories. Several "real" directories can be
+ *   collected together into one "virtual" directory.
+ * - Directories can be mapped to "real" directories as well as to
+ *   archives (.zip files). Files are compressed/decompressed
+ *   transparently for clients.
+ * - The Virtual File System is unique across all operating systems
+ *   Crystal Space supports, no matter of features of the underlying OS.
+ *
  * This class has only most basic features of a real filesystem: file
  * reading and writing (no simultaneous read and write mode are allowed
  * because it would be rather complex to implement it for archives).
@@ -104,14 +112,14 @@ protected:
  * only a list of file names; no fancy things like file size, time etc
  * (file size can be determined after opening it for reading).
  */
-class csVFS : public iVFS
+class csVFS : public scfImplementation2<csVFS, iVFS, iComponent>
 {
 private:
-  /// Mutex to make VFS thread-safe.
-  csRef<csMutex> mutex;
-
   friend class VfsNode;
 
+  /// Mutex to make VFS thread-safe.
+  mutable CS::Threading::RecursiveMutex mutex;
+
   // A vector of VFS nodes
   class VfsVector : public csPDelArray<VfsNode>
   {
@@ -119,7 +127,7 @@ private:
     static int Compare (VfsNode* const&, VfsNode* const&);
   } NodeList;
 
-  // Current working directory (in fact, the automaticaly-added prefix path)
+  // Current working directory (in fact, the automatically-added prefix path)
   // NOTE: cwd ALWAYS ends in '/'!
   char *cwd;
   // The installation directory (the value of $@)
@@ -129,25 +137,39 @@ private:
   // Full path of the directory containing the application executable or
   // the Cocoa application wrapper (the value of $^)
   char *appdir;
-  // Current node
-  const VfsNode *cnode;
-  // The current directory minus current node (cnode suffix)
-  char cnsufx [VFS_MAX_PATH_LEN + 1];
   // The initialization file
   csConfigFile config;
   // Directory stack (used in PushDir () and PopDir ())
   csStringArray dirstack;
-  // The pointer to system driver interface
+  // Reference to the object registry.
   iObjectRegistry *object_reg;
+  // ChDirAuto() may need to generate unique temporary names for mount points.
+  // It uses a counter to do so.
+  int auto_name_counter;
+  // Verbosity flags.
+  unsigned int verbosity;
 
 public:
-  SCF_DECLARE_IBASE;
+  enum
+  {
+    VERBOSITY_NONE  = 0,
+    VERBOSITY_DEBUG = 1 << 0,
+    VERBOSITY_SCAN  = 1 << 1,
+    VERBOSITY_MOUNT = 1 << 2,
+    VERBOSITY_ALL  = ~0
+  };
 
+public:
   /// Initialize VFS by reading contents of given INI file
   csVFS (iBase *iParent);
   /// Virtual File System destructor
   virtual ~csVFS ();
 
+  /// Is verbosity enabled for a mode or set of modes?
+  bool IsVerbose(unsigned int mask) const { return (verbosity & mask) != 0; }
+  /// Get verbosity flags.
+  unsigned int GetVerbosity() const { return verbosity; }
+
   /// Set current working directory
   virtual bool ChDir (const char *Path);
   /// Get current working directory
@@ -161,7 +183,7 @@ public:
 
   /**
    * Expand given virtual path, interpret all "." and ".."'s relative to
-   * 'currend virtual directory'. Return a new iString object.
+   * 'current virtual directory'. Return a new iString object.
    * If IsDir is true, expanded path ends in an '/', otherwise no.
    */
   virtual csPtr<iDataBuffer> ExpandPath (
@@ -191,6 +213,10 @@ public:
   /// Close all opened archives, free temporary storage etc.
   virtual bool Sync ();
 
+  /// Create or add a symbolic link
+  virtual bool SymbolicLink(const char *Target, const char *Link = 0, 
+    int priority = 0);
+
   /// Mount an VFS path on a "real-world-filesystem" path
   virtual bool Mount (const char *VirtualPath, const char *RealPath);
   /// Unmount an VFS path; if RealPath is 0, entire VirtualPath is unmounted
@@ -201,6 +227,12 @@ public:
 
   /// Save current configuration back into configuration file
   virtual bool SaveMounts (const char *FileName);
+  /// Load a configuration file
+  virtual bool LoadMountsFromFile (iConfigFile* file);
+
+  /// Auto-mount ChDir.
+  virtual bool ChDirAuto (const char* path, const csStringArray* paths = 0,
+  	const char* vfspath = 0, const char* filename = 0);
 
   /// Initialize the Virtual File System
   virtual bool Initialize (iObjectRegistry *object_reg);
@@ -221,12 +253,11 @@ public:
    */
   virtual csPtr<iDataBuffer> GetRealPath (const char *FileName);
 
-  struct eiComponent : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE(csVFS);
-    virtual bool Initialize (iObjectRegistry* p)
-    { return scfParent->Initialize(p); }
-  } scfiComponent;
+  /// Get all current virtual mount paths
+  virtual csRef<iStringArray> GetMounts ();
+
+  /// Get the real paths associated with a mount
+  virtual csRef<iStringArray> GetRealMountPaths (const char *VirtualPath);
 
 private:
   /// Same as ExpandPath() but with less overhead
@@ -245,6 +276,33 @@ private:
   /// Common routine for many functions
   bool PreparePath (const char *Path, bool IsDir, VfsNode *&Node,
     char *Suffix, size_t SuffixSize) const;
+
+  /**
+   * Check if a virtual path represents an actual physical mount point.  Note
+   * that there are cases where the virtual path itself may be valid yet not in
+   * fact point at a physical mount point (i.e. not be represented by a
+   * VfsNode).  This can occur, for example, in cases such as the following:
+   *
+   * VFS.Mount.lib/textures = textures.zip
+   * VFS.Mount.lib/materials = materials.zip
+   *
+   * The virtual directory "/lib" is a valid path, yet it has no VfsNode
+   * representation because there is nothing actually mounted at "/lib", thus
+   * false will be returned. On the other hand, the virtual directories
+   * "/lib/textures" and "/lib/materials" are both valid and represented by
+   * physical mount points, so true will be returned.
+   */
+  bool CheckIfMounted(char const* virtual_path) const;
+
+  /**
+   * Helper for ChDirAuto(). Checks if dir is mounted and invokes ChDir() if it
+   * is and returns true. If filename is specified, then ensures that it exists
+   * in dir. If filename is not present, returns false and does not invoke
+   * ChDir().
+   */
+  bool TryChDirAuto(char const* dir, char const* filename);
 };
 
+} // namespace cspluginVFS
+
 #endif // __CS_VFS_H__
