@@ -23,6 +23,7 @@
 #include "csutil/csuctransform.h"
 #include "csutil/csendian.h"
 #include "iutil/vfs.h"
+#include "csutil/csstring.h"
 #include "csutil/util.h"
 #include "csutil/databuf.h"
 #include "csutil/dirtyaccessarray.h"
@@ -31,7 +32,7 @@
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginCSfont
+CS_PLUGIN_NAMESPACE_BEGIN(csFont)
 {
 
 #include "font_police.h"
@@ -46,10 +47,10 @@ struct csFontDef
   const char* Name;
   int Height;
   int Baseline;
-  uint8 *FontBitmap;
+  const uint8 *FontBitmap;
   size_t bitmapSize;
-  uint8 *IndividualWidth;
-  csDefaultFont::CharRange* ranges;
+  const uint8 *IndividualWidth;
+  const csDefaultFont::CharRange* ranges;
   int underline_position;
   int underline_thickness;
   int text_height;
@@ -128,9 +129,9 @@ csPtr<iFont> csDefaultFontServer::LoadFont (const char *filename, float /*size*/
 	  numRanges++;
 	}
 	csDirtyAccessArray<csBitmapMetrics> bMetrics;
-	bMetrics.SetLength (numGlyphs);
+	bMetrics.SetSize (numGlyphs);
 	csDirtyAccessArray<csGlyphMetrics> gMetrics;
-	gMetrics.SetLength (numGlyphs);
+	gMetrics.SetSize (numGlyphs);
 
 	for (int j = 0; j < numGlyphs; j++)
 	{
@@ -191,7 +192,7 @@ void csDefaultFontServer::NotifyDelete (csDefaultFont *font)
 
 csDefaultFont *csDefaultFontServer::ReadFontFile(const char *file)
 {
-  csRef<iVFS> VFS (CS_QUERY_REGISTRY (object_reg, iVFS));
+  csRef<iVFS> VFS (csQueryRegistry<iVFS> (object_reg));
   csRef<iDataBuffer> fntfile (VFS->ReadFile (file, false));
   if (!fntfile)
   {
@@ -338,9 +339,11 @@ error:
       int numRanges = 0;
       do
       {
-	ranges.GetExtend (numRanges).startChar = csGetLittleEndianLong (binary);
+	ranges.GetExtend (numRanges).startChar = 
+          csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	binary += 4;
-	ranges[numRanges].charCount = csGetLittleEndianLong (binary);
+	ranges[numRanges].charCount = 
+          csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	binary += 4;
 	numGlyphs += ranges[numRanges].charCount;
 	numRanges++;
@@ -357,7 +360,7 @@ error:
 	  fontdef.Name);
 	return 0;
       }
-      ranges.SetLength (2);
+      ranges.SetSize (2);
       ranges[0].startChar = fontdefFirst;
       ranges[0].charCount = fontdefGlyphs;
       ranges[1].startChar = 0;
@@ -367,21 +370,25 @@ error:
 
     size_t bitmapSize = 0;
     size_t alphaSize = 0;
-    bMetrics.SetLength (numGlyphs);
-    aMetrics.SetLength (numGlyphs);
+    bMetrics.SetSize (numGlyphs);
+    aMetrics.SetSize (numGlyphs);
     if (hasBitmapMetrics)
     {
       int j;
       for (j = 0; j < numGlyphs; j++)
       {
 	memset (&(bMetrics[j]), 0, sizeof (csBitmapMetrics));
-	bMetrics[j].width = csGetLittleEndianLong (binary);
+	bMetrics[j].width = 
+          csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	binary += 4;
-	bMetrics[j].height = csGetLittleEndianLong (binary);
+	bMetrics[j].height = 
+          csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	binary += 4;
-	bMetrics[j].left = csGetLittleEndianLong (binary);
+	bMetrics[j].left = 
+          csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	binary += 4;
-	bMetrics[j].top = csGetLittleEndianLong (binary);
+	bMetrics[j].top = 
+          csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	binary += 4;
 	bitmapSize += ((bMetrics[j].width + 7) / 8) * bMetrics[j].height;
       }
@@ -390,13 +397,17 @@ error:
 	for (j = 0; j < numGlyphs; j++)
 	{
 	  memset (&(aMetrics[j]), 0, sizeof (csBitmapMetrics));
-	  aMetrics[j].width = csGetLittleEndianLong (binary);
+	  aMetrics[j].width = 
+            csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	  binary += 4;
-	  aMetrics[j].height = csGetLittleEndianLong (binary);
+	  aMetrics[j].height = 
+            csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	  binary += 4;
-	  aMetrics[j].left = csGetLittleEndianLong (binary);
+	  aMetrics[j].left = 
+            csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	  binary += 4;
-	  aMetrics[j].top = csGetLittleEndianLong (binary);
+	  aMetrics[j].top = 
+            csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	  binary += 4;
 	  alphaSize += bMetrics[j].width * bMetrics[j].height;
 	}
@@ -421,13 +432,14 @@ error:
       binary += numGlyphs;
     }
 
-    gMetrics.SetLength (numGlyphs);
+    gMetrics.SetSize (numGlyphs);
     if (hasGlyphAdvance)
     {
       for (int j = 0; j < numGlyphs; j++)
       {
 	memset (&(gMetrics[j]), 0, sizeof (csGlyphMetrics));
-	gMetrics[j].advance = csGetLittleEndianLong (binary);
+	gMetrics[j].advance = 
+          csLittleEndian::Convert (csGetFromAddress::UInt32 (binary));
 	binary += 4;
       }
     }
@@ -470,9 +482,9 @@ error:
 
     uint8* individualWidths = (uint8*)binary;
     csDirtyAccessArray<csBitmapMetrics> bMetrics;
-    bMetrics.SetLength (fontdefGlyphs);
+    bMetrics.SetSize (fontdefGlyphs);
     csDirtyAccessArray<csGlyphMetrics> gMetrics;
-    gMetrics.SetLength (fontdefGlyphs);
+    gMetrics.SetSize (fontdefGlyphs);
     for (int j = 0; j < fontdefGlyphs; j++)
     {
       bMetrics[j].width = individualWidths[j];
@@ -521,7 +533,7 @@ error:
 //--//--//--//--//--//--//--//--//--//--//--//--//--//- The font object -//--//
 
 csDefaultFont::csDefaultFont (csDefaultFontServer *parent, const char *name, 
-			      CharRange* glyphRanges, int height, 
+			      const CharRange* glyphRanges, int height, 
 			      int ascent, int descent,
 			      int text_height, int underline_position,
 			      int underline_thickness,
@@ -592,7 +604,7 @@ csDefaultFont::csDefaultFont (csDefaultFontServer *parent, const char *name,
 
 csDefaultFont::~csDefaultFont ()
 {
-  size_t i = DeleteCallbacks.Length();
+  size_t i = DeleteCallbacks.GetSize ();
   while (i-- > 0)
   {
     iFontDeleteNotify* delnot = DeleteCallbacks[i];
@@ -813,4 +825,5 @@ int csDefaultFont::GetUnderlineThickness ()
   return UnderlineThickness;
 }
 
-} // namespace cspluginCSfont
+}
+CS_PLUGIN_NAMESPACE_END(csFont)
