@@ -23,17 +23,19 @@
 #include "csqint.h"
 #include "csutil/csuctransform.h"
 #include "csutil/util.h"
-#include "iutil/plugin.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "iutil/objreg.h"
+#include "csutil/set.h"
 #include "iutil/databuff.h"
+#include "iutil/eventh.h"
+#include "iutil/objreg.h"
+#include "iutil/plugin.h"
+#include "iutil/stringarray.h"
 #include "ivaria/reporter.h"
+
 #include "fontplex.h"
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginFontplex
+CS_PLUGIN_NAMESPACE_BEGIN(FontPlex)
 {
 
 //---------------------------------------------------------------------------
@@ -87,7 +89,7 @@ iFont* csFontLoadOrderEntry::GetFont (csFontPlexer* parent)
 void csFontLoaderOrder::AppendSmart (const csFontLoaderOrder& other)
 {
   size_t i;
-  for (i = 0; i < other.Length (); i++)
+  for (i = 0; i < other.GetSize (); i++)
   {
     PushSmart (other[i]);
   }
@@ -124,13 +126,14 @@ csFontServerMultiplexer::~csFontServerMultiplexer ()
 {
 }
 
+#define FSCLASSPREFIX "crystalspace.font.server."
+#define FONTPLEX_CLASSNAME FSCLASSPREFIX "multiplexer"
+#define CSFONT_CLASSNAME FSCLASSPREFIX "default"
+
 bool csFontServerMultiplexer::Initialize (iObjectRegistry *object_reg)
 {
   csFontServerMultiplexer::object_reg = object_reg;
 
-  csRef<iPluginManager> plugin_mgr = CS_QUERY_REGISTRY (object_reg,
-    iPluginManager);
-
   config.AddConfig(object_reg, "config/fontplex.cfg");
   fontset = config->GetStr ("Fontplex.Settings.FontSet", 0);
   csString mapKey;
@@ -138,11 +141,12 @@ bool csFontServerMultiplexer::Initialize (iObjectRegistry *object_reg)
   if (fontset) mapKey << fontset << '.';
   
   csRef<iConfigIterator> mapEnum (config->Enumerate (mapKey));
-  while (mapEnum->Next ())
+  while (mapEnum->HasNext ())
   {
+    mapEnum->Next();
     const char* pluginName = mapEnum->GetStr ();
-    csRef<iFontServer> fs = CS_QUERY_PLUGIN_CLASS (plugin_mgr, pluginName, 
-      iFontServer);
+    csRef<iFontServer> fs = csLoadPluginCheck<iFontServer> (object_reg, 
+      pluginName);
 
     if (fs)
     {
@@ -155,13 +159,15 @@ bool csFontServerMultiplexer::Initialize (iObjectRegistry *object_reg)
   }
 
   // Query the auxiliary font servers in turn
+  csSet<csString> usedPlugins;
+  usedPlugins.Add (FONTPLEX_CLASSNAME);
   csString tag;
   int idx;
   int errorcount = 0;
   for (idx = 1; ; idx++)
   {
     tag.Format ("iFontServer.%d", idx);
-    csRef<iBase> b (CS_QUERY_REGISTRY_TAG (object_reg, tag));
+    csRef<iBase> b (csQueryRegistryTag (object_reg, tag));
     if (!b) 
     {
       // in cases where just one entry in the server list doesn't work
@@ -172,12 +178,37 @@ bool csFontServerMultiplexer::Initialize (iObjectRegistry *object_reg)
     else
     {
       errorcount = 0;	
-      csRef<iFontServer> fs (SCF_QUERY_INTERFACE (b, iFontServer));
+      csRef<iFactory> fact = scfQueryInterface<iFactory> (b);
+      const char* classId = fact->QueryClassID ();
+      if (usedPlugins.Contains (classId)) continue;
+      csRef<iFontServer> fs (scfQueryInterface<iFontServer> (b));
+      if (!fs.IsValid()) continue;
       fs->SetWarnOnError (false);
       fontservers.Push (fs);
     }
   }
-  if (!fontservers.Length ())
+
+  // Small hack: forcibly add csfont as the last server
+  usedPlugins.Add (CSFONT_CLASSNAME);
+  csRef<iStringArray> scfClassList = 
+    iSCF::SCF->QueryClassList (FSCLASSPREFIX);
+  for (size_t i = 0; i < scfClassList->GetSize(); i++)
+  {
+    const char* scfClass = scfClassList->Get (i);
+    if (!usedPlugins.Contains (scfClass))
+    {
+      csRef<iFontServer> fs = csLoadPluginCheck<iFontServer> (object_reg, 
+        scfClass);
+      if (fs.IsValid()) fontservers.Push (fs);
+    }
+  }
+  {
+    csRef<iFontServer> fs = csLoadPluginCheck<iFontServer> (object_reg, 
+      CSFONT_CLASSNAME);
+    if (fs.IsValid()) fontservers.Push (fs);
+  }
+
+  if (!fontservers.GetSize ())
   {
     csReport (object_reg, CS_REPORTER_SEVERITY_WARNING,
         "crystalspace.font.fontplex",
@@ -250,12 +281,13 @@ csPtr<iFont> csFontServerMultiplexer::LoadFont (const char *filename,
 
   csRef<csFontPlexer> newFont;
   newFont.AttachNew (new csFontPlexer (this, fontid, size, order));
+  loadedFonts.Put (fontid, newFont);
 
   // The first font that could be loaded is the "primary" font.
   iFont* primary = 0;
   size_t i;
   bool wasFallback = false;
-  for (i = 0; i < order->Length (); i++)
+  for (i = 0; i < order->GetSize (); i++)
   {
     csFontLoadOrderEntry& orderEntry = (*order)[i];
     if ((i > 0) && !wasFallback && orderEntry.fallback)
@@ -272,13 +304,12 @@ csPtr<iFont> csFontServerMultiplexer::LoadFont (const char *filename,
   if (primary == 0)
   {
     // Not a single font in the substitution list could be loaded?...
-    delete order;
+    // Note: order is deleted when newFont gets released.
     return 0;
   }
   else
   {
     newFont->primaryFont = primary;
-    loadedFonts.Put (fontid, newFont);
     return csPtr<iFont> (newFont);
   }
 }
@@ -333,8 +364,8 @@ void csFontServerMultiplexer::ParseFontLoaderOrder (
 
 csPtr<iFontServer> csFontServerMultiplexer::ResolveFontServer (const char* name)
 {
-  csRef<iPluginManager> plugin_mgr = CS_QUERY_REGISTRY (object_reg,
-    iPluginManager);
+  csRef<iPluginManager> plugin_mgr = 
+    csQueryRegistry<iPluginManager> (object_reg);
 
   csRef<iFontServer> fs;
   if (iSCF::SCF->ClassRegistered (name))
@@ -343,7 +374,7 @@ csPtr<iFontServer> csFontServerMultiplexer::ResolveFontServer (const char* name)
   }
   if (fs == 0)
   {
-    csHash<FontServerMapEntry, csStrKey>::Iterator it = 
+    csHash<FontServerMapEntry, csString>::Iterator it = 
       fontServerMap.GetIterator (name);
 
     while (it.HasNext ())
@@ -361,11 +392,7 @@ csPtr<iFontServer> csFontServerMultiplexer::ResolveFontServer (const char* name)
     csString plugName;
     plugName << "crystalspace.font.server." << name;
 
-    fs = CS_QUERY_PLUGIN_CLASS (plugin_mgr, plugName, iFontServer);
-    if (fs == 0)
-    {
-      fs = CS_LOAD_PLUGIN (plugin_mgr, plugName, iFontServer);
-    }
+    fs = csLoadPluginCheck<iFontServer> (plugin_mgr, plugName);
   }
   return csPtr<iFontServer> (fs);
 }
@@ -389,7 +416,7 @@ csFontPlexer::~csFontPlexer ()
 
   delete order;
 
-  size_t i = DeleteCallbacks.Length ();
+  size_t i = DeleteCallbacks.GetSize ();
   while (i-- > 0)
   {
     iFontDeleteNotify* delnot = DeleteCallbacks[i];
@@ -411,7 +438,7 @@ bool csFontPlexer::GetGlyphMetrics (utf32_char c, csGlyphMetrics& metrics)
 {
   iFont* font;
   size_t i;
-  for (i = 0; i < order->Length (); i++)
+  for (i = 0; i < order->GetSize (); i++)
   {
     if ((font = (*order)[i].GetFont (this)))
     {
@@ -427,7 +454,7 @@ csPtr<iDataBuffer> csFontPlexer::GetGlyphBitmap (utf32_char c,
 {
   iFont* font;
   size_t i;
-  for (i = 0; i < order->Length (); i++)
+  for (i = 0; i < order->GetSize (); i++)
   {
     if ((font = (*order)[i].GetFont (this)))
     {
@@ -447,7 +474,7 @@ csPtr<iDataBuffer> csFontPlexer::GetGlyphAlphaBitmap (utf32_char c,
 {
   iFont* font;
   size_t i;
-  for (i = 0; i < order->Length (); i++)
+  for (i = 0; i < order->GetSize (); i++)
   {
     if ((font = (*order)[i].GetFont (this)))
     {
@@ -487,7 +514,7 @@ void csFontPlexer::GetDimensions (const char *text, int &oW, int &oH, int &desc)
     csGlyphMetrics gMetrics = defMetrics;
     iFont* font;
     size_t i;
-    for (i = 0; i < order->Length (); i++)
+    for (i = 0; i < order->GetSize (); i++)
     {
       if ((font = (*order)[i].GetFont (this)))
       {
@@ -538,7 +565,7 @@ int csFontPlexer::GetLength (const char *text, int maxwidth)
     csGlyphMetrics gMetrics = defMetrics;
     iFont* font;
     size_t i;
-    for (i = 0; i < order->Length (); i++)
+    for (i = 0; i < order->GetSize (); i++)
     {
       if ((font = (*order)[i].GetFont (this)))
       {
@@ -565,7 +592,7 @@ void csFontPlexer::AddDeleteCallback (iFontDeleteNotify* func)
 
 bool csFontPlexer::RemoveDeleteCallback (iFontDeleteNotify* func)
 {
-  size_t i = DeleteCallbacks.Length ();
+  size_t i = DeleteCallbacks.GetSize ();
   while (i-- > 0)
   {
     iFontDeleteNotify* delnot = DeleteCallbacks[i];
@@ -592,7 +619,7 @@ bool csFontPlexer::HasGlyph (utf32_char c)
 {
   iFont* font;
   size_t i;
-  for (i = 0; i < order->Length (); i++)
+  for (i = 0; i < order->GetSize (); i++)
   {
     if ((font = (*order)[i].GetFont (this)))
     {
@@ -617,4 +644,5 @@ int csFontPlexer::GetUnderlineThickness ()
   return primaryFont->GetUnderlineThickness();
 }
 
-} // namespace cspluginFontplex
+} 
+CS_PLUGIN_NAMESPACE_END(FontPlex)
