@@ -25,12 +25,12 @@
 #include "csgeom/poly3d.h"
 #include "csgeom/polypool.h"
 #include "csgeom/sphere.h"
+#include "cstool/rviewclipper.h"
 #include "csgfx/renderbuffer.h"
 #include "csutil/array.h"
 #include "csutil/csendian.h"
 #include "csutil/csmd5.h"
 #include "csutil/csstring.h"
-#include "csutil/debug.h"
 #include "csutil/dirtyaccessarray.h"
 #include "csutil/memfile.h"
 #include "iengine/camera.h"
@@ -65,64 +65,8 @@ CS_IMPLEMENT_PLUGIN
 
 //---------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE(csBezierMesh)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iBezierState)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iBezierFactoryState)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iLightingInfo)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iObjectModel)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iShadowCaster)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iShadowReceiver)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iMeshObject)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iMeshObjectFactory)
-  {
-    static scfInterfaceID iPolygonMesh_scfID = (scfInterfaceID)-1;		
-    if (iPolygonMesh_scfID == (scfInterfaceID)-1)				
-      iPolygonMesh_scfID = iSCF::SCF->GetInterfaceID ("iPolygonMesh");		
-    if (iInterfaceID == iPolygonMesh_scfID &&				
-      scfCompatibleVersion(iVersion, scfInterfaceTraits<iPolygonMesh>::GetVersion()))
-    {
-#ifdef CS_DEBUG
-      csPrintf ("Deprecated feature use: iPolygonMesh queried from Bezier "
-	"object; use iObjectModel->GetPolygonMeshColldet() instead.\n");
-#endif
-      iPolygonMesh* Object = scfiObjectModel.GetPolygonMeshColldet();
-      (Object)->IncRef ();						
-      return CS_STATIC_CAST(iPolygonMesh*, Object);				
-    }
-  }
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::BezierState)
-  SCF_IMPLEMENTS_INTERFACE(iBezierState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::BezierFactoryState)
-  SCF_IMPLEMENTS_INTERFACE(iBezierFactoryState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::LightingInfo)
-  SCF_IMPLEMENTS_INTERFACE(iLightingInfo)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::ObjectModel)
-  SCF_IMPLEMENTS_INTERFACE(iObjectModel)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::ShadowCaster)
-  SCF_IMPLEMENTS_INTERFACE(iShadowCaster)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::ShadowReceiver)
-  SCF_IMPLEMENTS_INTERFACE(iShadowReceiver)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::MeshObject)
-  SCF_IMPLEMENTS_INTERFACE(iMeshObject)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMesh::MeshObjectFactory)
-  SCF_IMPLEMENTS_INTERFACE(iMeshObjectFactory)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
+CS_PLUGIN_NAMESPACE_BEGIN(Bezier)
+{
 
 int csBezierMesh:: last_thing_id = 0;
 
@@ -169,27 +113,23 @@ csStringID csBezierMesh::color_name = csInvalidStringID;
 csStringID csBezierMesh::index_name = csInvalidStringID;
 
 csBezierMesh::csBezierMesh (iBase *parent, csBezierMeshObjectType* thing_type) :
-  curves (4, 16)
+  scfImplementationType (this, parent), curves (4, 16)
 {
-  SCF_CONSTRUCT_IBASE (parent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiBezierState);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiBezierFactoryState);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiLightingInfo);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiShadowCaster);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiShadowReceiver);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiMeshObject);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiMeshObjectFactory);
-
   static_data = new csBezierMeshStatic (
-  	thing_type, &scfiBezierFactoryState);
+  	thing_type, (iBezierFactoryState*)this);
 
-  scfiPolygonMesh.SetThing (this);
-  scfiPolygonMeshLOD.SetThing (this);
-  scfiObjectModel.SetPolygonMeshBase (&scfiPolygonMesh);
-  scfiObjectModel.SetPolygonMeshColldet (&scfiPolygonMesh);
-  scfiObjectModel.SetPolygonMeshViscull (&scfiPolygonMeshLOD);
-  scfiObjectModel.SetPolygonMeshShadows (&scfiPolygonMeshLOD);
+  polygonMesh.AttachNew (new BezierPolyMeshHelper ());
+  polygonMeshLOD.AttachNew (new BezierPolyMeshHelper ());
+  polygonMesh->SetThing (this);
+  polygonMeshLOD->SetThing (this);
+  SetPolygonMeshBase (polygonMesh);
+  SetPolygonMeshColldet (polygonMesh);
+  SetPolygonMeshViscull (polygonMeshLOD);
+  SetPolygonMeshShadows (polygonMeshLOD);
+  csRef<BezierTriMeshHelper> trimesh;
+  trimesh.AttachNew (new BezierTriMeshHelper ());
+  trimesh->SetThing (this);
+  SetTriangleData (thing_type->base_id, trimesh);
 
   last_thing_id++;
   thing_id = last_thing_id;
@@ -220,8 +160,8 @@ csBezierMesh::csBezierMesh (iBase *parent, csBezierMeshObjectType* thing_type) :
   dynamic_ambient_version = 0;
 
   csRef<iStringSet> strings;
-  strings = CS_QUERY_REGISTRY_TAG_INTERFACE (thing_type->object_reg,
-    "crystalspace.shared.stringset", iStringSet);
+  strings = csQueryRegistryTagInterface<iStringSet>
+    (thing_type->object_reg, "crystalspace.shared.stringset");
 
   if ((vertex_name == csInvalidStringID) ||
     (texel_name == csInvalidStringID) ||
@@ -238,28 +178,15 @@ csBezierMesh::csBezierMesh (iBase *parent, csBezierMeshObjectType* thing_type) :
 csBezierMesh::~csBezierMesh ()
 {
   delete static_data;
-
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiBezierState);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiBezierFactoryState);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiLightingInfo);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiShadowCaster);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiShadowReceiver);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiMeshObject);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiMeshObjectFactory);
-  SCF_DESTRUCT_IBASE ();
 }
 
 char* csBezierMesh::GenerateCacheName ()
 {
-  csBox3 b;
-  GetBoundingBox (b);
-
   csMemFile mf;
   int32 l;
   l = csLittleEndian::Convert ((int32)static_data->num_curve_vertices);
   mf.Write ((char*)&l, 4);
-  l = csLittleEndian::Convert ((int32)curves.Length ());
+  l = csLittleEndian::Convert ((int32)curves.GetSize ());
   mf.Write ((char*)&l, 4);
 
   if (logparent)
@@ -294,9 +221,9 @@ void csBezierMesh::LightDisconnect (iLight* light)
   MarkLightmapsDirty ();
   size_t i;
   int dt = light->GetDynamicType ();
-  for (i = 0; i < curves.Length (); i++)
+  for (i = 0; i < curves.GetSize (); i++)
   {
-    csCurve *c = GetCurve ((int)i);
+    csCurve *c = curves[i];
     if (dt == CS_LIGHT_DYNAMICTYPE_DYNAMIC)
       c->DynamicLightDisconnect (light);
     else
@@ -308,9 +235,9 @@ void csBezierMesh::DisconnectAllLights ()
 {
   MarkLightmapsDirty ();
   size_t i;
-  for (i = 0; i < curves.Length (); i++)
+  for (i = 0; i < curves.GetSize (); i++)
   {
-    csCurve *c = GetCurve ((int)i);
+    csCurve *c = curves[i];
     c->DisconnectAllLights ();
   }
 }
@@ -344,18 +271,18 @@ void csBezierMesh::Prepare ()
 
   prepared = true;
 
-  scfiPolygonMeshLOD.Cleanup ();
-  scfiPolygonMesh.Cleanup ();
+  polygonMeshLOD->Cleanup ();
+  polygonMesh->Cleanup ();
 
   static_data_nr = static_data->static_data_nr;
 
   if (cached_movable) movablenr = cached_movable->GetUpdateNumber ()-1;
   else movablenr--;
 
-  scfiObjectModel.ShapeChanged ();
+  ShapeChanged ();
 }
 
-int csBezierMesh::AddCurveVertex (const csVector3 &v, const csVector2 &t)
+size_t csBezierMesh::AddCurveVertex (const csVector3 &v, const csVector2 &t)
 {
   if (!static_data->curve_vertices)
   {
@@ -419,20 +346,15 @@ void csBezierMesh::InvalidateThing ()
   prepared = false;
   static_data->obj_bbox_valid = false;
 
-  scfiPolygonMeshLOD.Cleanup ();
-  scfiPolygonMesh.Cleanup ();
-  scfiObjectModel.ShapeChanged ();
-}
-
-iPolygonMesh* csBezierMesh::GetWriteObject ()
-{
-  return &scfiPolygonMeshLOD;
+  polygonMeshLOD->Cleanup ();
+  polygonMesh->Cleanup ();
+  ShapeChanged ();
 }
 
 csCurve *csBezierMesh::GetCurve (char *name) const
 {
   size_t i;
-  for (i = 0 ; i < curves.Length () ; i++)
+  for (i = 0 ; i < curves.GetSize () ; i++)
   {
     const char* n = curves[i]->GetName ();
     if (n && !strcmp (n, name))
@@ -454,12 +376,12 @@ iCurve *csBezierMesh::CreateCurve ()
   csCurve *c = new csBezierCurve (static_data->thing_type);
   c->SetParentThing (this);
   AddCurve (c);
-  return &(c->scfiCurve);
+  return c;
 }
 
 int csBezierMesh::FindCurveIndex (iCurve *curve) const
 {
-  return (int)curves.Find (curve->GetOriginalObject ());
+  return (int)curves.Find (static_cast<csCurve*> (curve));
 }
 
 void csBezierMesh::RemoveCurve (int idx)
@@ -487,9 +409,9 @@ void csBezierMesh::HardTransform (const csReversibleTransform &t)
       	static_data->curve_vertices[i]);
 
   curves_transf_ok = false;
-  for (i = 0; i < curves.Length (); i++)
+  for (i = 0; i < curves.GetSize (); i++)
   {
-    csCurve *c = GetCurve ((int)i);
+    csCurve *c = curves[i];
     c->HardTransform (t);
   }
 }
@@ -503,8 +425,7 @@ void csBezierMesh::GetTransformedBoundingBox (
   // @@@ Shouldn't we try to cache this depending on camera/movable number?
 
   // Similar to what happens in csSprite3D.
-  csBox3 box;
-  GetBoundingBox (box);
+  const csBox3& box = GetBoundingBox ();
   cbox.StartBoundingBox (trans * box.GetCorner (0));
   cbox.AddBoundingVertexSmart (trans * box.GetCorner (1));
   cbox.AddBoundingVertexSmart (trans * box.GetCorner (2));
@@ -582,13 +503,13 @@ void csBezierMesh::AppendShadows (
   WorUpdate ();
 
   iShadowBlock *list = shadows->NewShadowBlock (0);
-      //@@@polygons.Length ());
+      //@@@polygons.GetSize ());
 (void)list;
 #if 0
   csFrustum *frust;
   int i, j;
   bool cw = true;                   //@@@ Use mirroring parameter here!
-  for (i = 0; i < static_data->static_polygons.Length (); i++)
+  for (i = 0; i < static_data->static_polygons.GetSize (); i++)
   {
     sp = static_data->static_polygons.Get (i);
     if (sp->GetPortal ()) continue;  // No portals
@@ -613,54 +534,48 @@ void csBezierMesh::AppendShadows (
 #endif
 }
 
-void csBezierMesh::GetRadius (csVector3 &rad, csVector3 &cent)
+void csBezierMesh::GetRadius (float &rad, csVector3 &cent)
 {
-  csBox3 b;
-  GetBoundingBox (b);
+  const csBox3& b = GetBoundingBox ();
   rad = static_data->obj_radius;
   cent = b.GetCenter ();
 }
 
-void csBezierMesh::GetBoundingBox (csBox3 &box)
+const csBox3& csBezierMesh::GetBoundingBox ()
 {
   int i;
 
   if (static_data->obj_bbox_valid)
-  {
-    box = static_data->obj_bbox;
-    return ;
-  }
+    return static_data->obj_bbox;
 
   static_data->obj_bbox_valid = true;
 
   if (static_data->num_curve_vertices == 0)
   {
     static_data->obj_bbox.Set (0, 0, 0, 0, 0, 0);
-    box = static_data->obj_bbox;
-    return ;
+    return static_data->obj_bbox;
   }
 
   if (static_data->num_curve_vertices > 0)
   {
     static_data->obj_bbox.StartBoundingBox (static_data->curve_vertices[0]);
-
     for (i = 1 ; i < static_data->num_curve_vertices ; i++)
     {
-      static_data->obj_bbox.AddBoundingVertexSmart (static_data->curve_vertices[i]);
+      static_data->obj_bbox.AddBoundingVertexSmart (
+	  static_data->curve_vertices[i]);
     }
   }
 
-  static_data->obj_radius = (static_data->obj_bbox.Max () - static_data->obj_bbox.Min ()) * 0.5f;
-  static_data->max_obj_radius = csQsqrt (csSquaredDist::PointPoint (
+  static_data->obj_radius = csQsqrt (csSquaredDist::PointPoint (
   	static_data->obj_bbox.Max (), static_data->obj_bbox.Min ())) * 0.5f;
-  box = static_data->obj_bbox;
+  return static_data->obj_bbox;
 }
 
 void csBezierMesh::SetBoundingBox (const csBox3& b)
 {
   static_data->obj_bbox_valid = true;
   static_data->obj_bbox = b;
-  scfiObjectModel.ShapeChanged ();
+  ShapeChanged ();
 }
 
 void csBezierMesh::GetBoundingBox (iMovable *movable, csBox3 &box)
@@ -668,7 +583,7 @@ void csBezierMesh::GetBoundingBox (iMovable *movable, csBox3 &box)
   if (wor_bbox_movablenr != movable->GetUpdateNumber ())
   {
     // First make sure obj_bbox is valid.
-    GetBoundingBox (box);
+    box = GetBoundingBox ();
     wor_bbox_movablenr = movable->GetUpdateNumber ();
     csBox3& obj_bbox = static_data->obj_bbox;
 
@@ -697,22 +612,6 @@ void csBezierMesh::GetBoundingBox (iMovable *movable, csBox3 &box)
 
 //-------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE(csBezierMesh::PolyMeshLOD)
-  SCF_IMPLEMENTS_INTERFACE(iPolygonMesh)
-SCF_IMPLEMENT_IBASE_END
-
-csBezierMesh::PolyMeshLOD::PolyMeshLOD () : BezierPolyMeshHelper ()
-{
-  SCF_CONSTRUCT_IBASE (0);
-}
-
-csBezierMesh::PolyMeshLOD::~PolyMeshLOD ()
-{
-  SCF_DESTRUCT_IBASE ();
-}
-
-//-------------------------------------------------------------------------
-
 void BezierPolyMeshHelper::Setup ()
 {
   if (polygons)
@@ -800,6 +699,75 @@ void BezierPolyMeshHelper::Cleanup ()
 
 //-------------------------------------------------------------------------
 
+void BezierTriMeshHelper::Setup ()
+{
+  if (triangles)
+  {
+    // Already set up.
+    return ;
+  }
+
+  triangles = 0;
+  vertices = 0;
+
+  // Count the number of needed polygons and vertices.
+  num_verts = 0;
+  num_tri = 0;
+
+  size_t i, j;
+
+  // Check curves.
+  for (i = 0; i < (size_t)thing->GetCurveCount (); i++)
+  {
+    csCurve *c = thing->curves.Get (i);
+    csCurveTesselated *tess = c->Tesselate (1000);    // @@@ High quality?
+    num_tri += tess->GetTriangleCount ();
+    num_verts += tess->GetVertexCount ();
+  }
+
+  if (!num_verts || !num_tri) return;
+
+  // Allocate the arrays and the copy the data.
+  vertices = new csVector3[num_verts];
+  triangles = new csTriangle[num_tri];
+
+  num_verts = 0;
+  num_tri = 0;
+  for (i = 0; i < (size_t)thing->GetCurveCount (); i++)
+  {
+    csCurve *c = thing->curves.Get (i);
+    csCurveTesselated *tess = c->Tesselate (1000);  // @@@ High quality?
+    csTriangle *tris = tess->GetTriangles ();
+    size_t tri_count = tess->GetTriangleCount ();
+    for (j = 0; j < tri_count; j++)
+    {
+      triangles[num_tri].a = 
+      // Adjust indices to skip the original polygon set vertices and
+      // preceeding curves.
+      triangles[num_tri].a = tris[j].a + num_verts;
+      triangles[num_tri].b = tris[j].b + num_verts;
+      triangles[num_tri].c = tris[j].c + num_verts;
+      num_tri++;
+    }
+
+    csVector3 *vts = tess->GetVertices ();
+    size_t num_vt = tess->GetVertexCount ();
+    memcpy (vertices + num_verts, vts, sizeof (csVector3) * num_vt);
+    num_verts += num_vt;
+  }
+}
+
+void BezierTriMeshHelper::Cleanup ()
+{
+  delete[] triangles;
+  triangles = 0;
+
+  delete[] vertices;
+  vertices = 0;
+}
+
+//-------------------------------------------------------------------------
+
 void csBezierMesh::UpdateCurveTransform (const csReversibleTransform &movtrans)
 {
   if (GetCurveCount () == 0) return ;
@@ -865,14 +833,15 @@ csRenderMesh** csBezierMesh::GetRenderMeshes (int &n, iRenderView* rview,
   obj_cam /= movtrans;
  
   int clip_portal, clip_plane, clip_z_plane;
-  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
-      clip_z_plane);
+  CS::RenderViewClipper::CalculateClipSettings (rview->GetRenderContext (),
+      frustum_mask, clip_portal, clip_plane, clip_z_plane);
 
   const uint currentFrame = rview->GetCurrentFrameNumber();
   bool listCreated;
   csDirtyAccessArray<csRenderMesh*>& meshes = rmListHolder.GetUnusedData (
     listCreated, currentFrame);
-  meshes.SetLength (GetCurveCount(), 0);
+  meshes.Empty();
+  meshes.SetCapacity (GetCurveCount());
 
   iSector* s = movable->GetSectors ()->Get (0);
   csColor ambient = s->GetDynamicAmbientLight ();
@@ -882,12 +851,20 @@ csRenderMesh** csBezierMesh::GetRenderMeshes (int &n, iRenderView* rview,
     update_ambient = true;
     dynamic_ambient_version = s->GetDynamicAmbientVersion ();
   }
-  csCurve *c;
+  
   for (i = 0; i < GetCurveCount (); i++)
   {
+    csCurve* c = curves.Get (i);
+
+    // First get a bounding box in camera space.
+    csBox3 cbox;
+    csBox2 sbox;
+    if (c->GetScreenBoundingBox (obj_cam, icam, cbox, sbox) < 0)
+      continue;                                 // Not visible.
+
     bool meshCreated;
     csRenderMesh*& rm = rmHolder.GetUnusedMesh (meshCreated, currentFrame);
-    meshes[i] = rm;
+    meshes.Push (rm);
     if (meshCreated)
     {
       rm->buffers.AttachNew (new csRenderBufferHolder);
@@ -900,14 +877,6 @@ csRenderMesh** csBezierMesh::GetRenderMeshes (int &n, iRenderView* rview,
     rm->do_mirror = icam->IsMirrored ();
     rm->object2world = movtrans;
 
-    c = curves.Get (i);
-
-    // First get a bounding box in camera space.
-    csBox3 cbox;
-    csBox2 sbox;
-    if (c->GetScreenBoundingBox (obj_cam, icam, cbox, sbox) < 0)
-      continue;                                 // Not visible.
-
     // If we have a dirty lightmap recombine the curves and the shadow maps.
     bool updated_lm = c->RecalculateDynamicLights ();
 
@@ -961,13 +930,13 @@ csRenderMesh** csBezierMesh::GetRenderMeshes (int &n, iRenderView* rview,
     rm->material = c->GetMaterial ();
   }
 
-  n = (int)meshes.Length();
+  n = (int)meshes.GetSize ();
   return meshes.GetArray();
 }
 
 //----------------------------------------------------------------------
 
-void csBezierMesh::CastShadows (iFrustumView *lview, iMovable *movable)
+void csBezierMesh::CastShadows (iMovable *movable, iFrustumView *lview)
 {
   Prepare ();
   //@@@ Ok?
@@ -980,7 +949,7 @@ void csBezierMesh::CastShadows (iFrustumView *lview, iMovable *movable)
   iLightingProcessInfo* lpi = (iLightingProcessInfo*)fvud;
   bool dyn = lpi->IsDynamic ();
 
-  lpi->GetLight ()->AddAffectedLightingInfo (&scfiLightingInfo);
+  lpi->GetLight ()->AddAffectedLightingInfo ((iLightingInfo*)this);
 
   for (i = 0; i < GetCurveCount (); i++)
   {
@@ -1087,7 +1056,7 @@ void csBezierMesh::Merge (csBezierMesh *other)
     AddCurveVertex (other->static_data->GetCurveVertex (i),
     	other->static_data->GetCurveTexel (i));
 
-  while (other->curves.Length () > 0)
+  while (other->curves.GetSize () > 0)
   {
     csCurve *c = other->curves.Extract (0);
     AddCurve (c);
@@ -1107,9 +1076,8 @@ void csBezierMesh::MergeTemplate (
   static_data->curves_scale = tpl->GetCurvesScale ();
 
   //@@@ TEMPORARY
-  csRef<iBezierState> ith = SCF_QUERY_INTERFACE (tpl, iBezierState);
-  ParentTemplate = ((csBezierMesh::BezierState*)(iBezierState*)ith)
-  	->GetPrivateObject ();
+  csRef<iBezierState> ith = scfQueryInterface<iBezierState> (tpl);
+  ParentTemplate = (csBezierMesh*)(iBezierState*)ith;
 
   for (i = 0; i < tpl->GetCurveVertexCount (); i++)
   {
@@ -1134,60 +1102,23 @@ void csBezierMesh::MergeTemplate (
   }
 }
 
-//---------------------------------------------------------------------------
-iCurve *csBezierMesh::BezierFactoryState::GetCurve (int idx) const
+csPtr<iMeshObject> csBezierMesh::NewInstance ()
 {
-  csCurve *c = scfParent->GetCurve (idx);
-  return &(c->scfiCurve);
-}
-
-//---------------------------------------------------------------------------
-iMeshObjectFactory *csBezierMesh::MeshObject::GetFactory () const
-{
-  if (!scfParent->ParentTemplate) return 0;
-  return &scfParent->ParentTemplate->scfiMeshObjectFactory;
-}
-
-//---------------------------------------------------------------------------
-csPtr<iMeshObject> csBezierMesh::MeshObjectFactory::NewInstance ()
-{
-  csBezierMesh *thing = new csBezierMesh (scfParent,
-  	scfParent->static_data->thing_type);
-  thing->MergeTemplate (&(scfParent->scfiBezierFactoryState), 0);
-  return csPtr<iMeshObject> (&thing->scfiMeshObject);
+  csBezierMesh *thing = new csBezierMesh (this,
+  	static_data->thing_type);
+  thing->MergeTemplate ((iBezierFactoryState*)this, 0);
+  return csPtr<iMeshObject> ((iMeshObject*)thing);
 }
 
 //---------------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE(csBezierMesh::PolyMesh)
-  SCF_IMPLEMENTS_INTERFACE(iPolygonMesh)
-SCF_IMPLEMENT_IBASE_END
-
-//---------------------------------------------------------------------------
-
-SCF_IMPLEMENT_IBASE(csBezierMeshObjectType)
-  SCF_IMPLEMENTS_INTERFACE(iMeshObjectType)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iComponent)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(iPluginConfig)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMeshObjectType::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE(iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csBezierMeshObjectType::eiPluginConfig)
-  SCF_IMPLEMENTS_INTERFACE(iPluginConfig)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
 
 SCF_IMPLEMENT_FACTORY (csBezierMeshObjectType)
 
 
 csBezierMeshObjectType::csBezierMeshObjectType (
-  iBase *pParent)
+  iBase *pParent) : scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiPluginConfig);
   lightpatch_pool = 0;
   do_verbose = false;
 }
@@ -1195,24 +1126,23 @@ csBezierMeshObjectType::csBezierMeshObjectType (
 csBezierMeshObjectType::~csBezierMeshObjectType ()
 {
   delete lightpatch_pool;
-
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiPluginConfig);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csBezierMeshObjectType::Initialize (iObjectRegistry *object_reg)
 {
   csBezierMeshObjectType::object_reg = object_reg;
-  csRef<iEngine> e = CS_QUERY_REGISTRY (object_reg, iEngine);
+  csRef<iStringSet> strset = csQueryRegistryTagInterface<iStringSet> (
+      object_reg, "crystalspace.shared.stringset");
+  base_id = strset->Request ("base");
+  csRef<iEngine> e = csQueryRegistry<iEngine> (object_reg);
   engine = e;	// We don't want a real ref here to avoid circular refs.
-  csRef<iGraphics3D> g = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
+  csRef<iGraphics3D> g = csQueryRegistry<iGraphics3D> (object_reg);
   G3D = g;
 
   lightpatch_pool = new csBezierLightPatchPool ();
 
   csRef<iVerbosityManager> verbosemgr (
-    CS_QUERY_REGISTRY (object_reg, iVerbosityManager));
+    csQueryRegistry<iVerbosityManager> (object_reg));
   if (verbosemgr) 
     do_verbose = verbosemgr->Enabled ("bezier");
 
@@ -1228,33 +1158,24 @@ void csBezierMeshObjectType::Clear ()
 csPtr<iMeshObjectFactory> csBezierMeshObjectType::NewFactory ()
 {
   csBezierMesh *cm = new csBezierMesh (this, this);
-  csRef<iMeshObjectFactory> ifact (SCF_QUERY_INTERFACE (
-      cm, iMeshObjectFactory));
+  csRef<iMeshObjectFactory> ifact (
+    scfQueryInterface<iMeshObjectFactory> (cm));
   cm->DecRef ();
   return csPtr<iMeshObjectFactory> (ifact);
 }
 
+void csBezierMeshObjectType::ReportV (int severity, const char *description, 
+                                      va_list args)
+{
+  csReportV (object_reg, severity, "crystalspace.mesh.object.bezier",
+    description, args);
+}
+
 void csBezierMeshObjectType::Warn (const char *description, ...)
 {
   va_list arg;
   va_start (arg, description);
-
-  csRef<iReporter> Reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-
-  if (Reporter)
-  {
-    Reporter->ReportV (
-        CS_REPORTER_SEVERITY_WARNING,
-        "crystalspace.engine.warning",
-        description,
-        arg);
-  }
-  else
-  {
-    csPrintfV (description, arg);
-    csPrintf ("\n");
-  }
-
+  ReportV (CS_REPORTER_SEVERITY_WARNING, description, arg);
   va_end (arg);
 }
 
@@ -1262,23 +1183,7 @@ void csBezierMeshObjectType::Bug (const char *description, ...)
 {
   va_list arg;
   va_start (arg, description);
-
-  csRef<iReporter> Reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-
-  if (Reporter)
-  {
-    Reporter->ReportV (
-        CS_REPORTER_SEVERITY_BUG,
-        "crystalspace.engine.warning",
-        description,
-        arg);
-  }
-  else
-  {
-    csPrintfV (description, arg);
-    csPrintf ("\n");
-  }
-
+  ReportV (CS_REPORTER_SEVERITY_BUG, description, arg);
   va_end (arg);
 }
 
@@ -1286,23 +1191,7 @@ void csBezierMeshObjectType::Error (const char *description, ...)
 {
   va_list arg;
   va_start (arg, description);
-
-  csRef<iReporter> Reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-
-  if (Reporter)
-  {
-    Reporter->ReportV (
-        CS_REPORTER_SEVERITY_ERROR,
-        "crystalspace.engine.warning",
-        description,
-        arg);
-  }
-  else
-  {
-    csPrintfV (description, arg);
-    csPrintf ("\n");
-  }
-
+  ReportV (CS_REPORTER_SEVERITY_ERROR, description, arg);
   va_end (arg);
 }
 
@@ -1310,23 +1199,7 @@ void csBezierMeshObjectType::Notify (const char *description, ...)
 {
   va_list arg;
   va_start (arg, description);
-
-  csRef<iReporter> Reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-
-  if (Reporter)
-  {
-    Reporter->ReportV (
-        CS_REPORTER_SEVERITY_NOTIFY,
-        "crystalspace.engine.warning",
-        description,
-        arg);
-  }
-  else
-  {
-    csPrintfV (description, arg);
-    csPrintf ("\n");
-  }
-
+  ReportV (CS_REPORTER_SEVERITY_NOTIFY, description, arg);
   va_end (arg);
 }
 
@@ -1343,7 +1216,7 @@ const int NUM_OPTIONS =
     sizeof (config_options[0])
   );
 
-bool csBezierMeshObjectType::eiPluginConfig::SetOption (int id, csVariant *value)
+bool csBezierMeshObjectType::SetOption (int id, csVariant *value)
 {
   switch (id)
   {
@@ -1357,7 +1230,7 @@ bool csBezierMeshObjectType::eiPluginConfig::SetOption (int id, csVariant *value
   return true;
 }
 
-bool csBezierMeshObjectType::eiPluginConfig::GetOption (int id, csVariant *value)
+bool csBezierMeshObjectType::GetOption (int id, csVariant *value)
 {
   switch (id)
   {
@@ -1368,7 +1241,7 @@ bool csBezierMeshObjectType::eiPluginConfig::GetOption (int id, csVariant *value
   return true;
 }
 
-bool csBezierMeshObjectType::eiPluginConfig::GetOptionDescription (
+bool csBezierMeshObjectType::GetOptionDescription (
   int idx,
   csOptionDescription *option)
 {
@@ -1379,3 +1252,5 @@ bool csBezierMeshObjectType::eiPluginConfig::GetOptionDescription (
 
 //---------------------------------------------------------------------------
 
+}
+CS_PLUGIN_NAMESPACE_END(Bezier)
