@@ -19,6 +19,7 @@
 #include "cssysdef.h"
 #include "csgeom/math3d.h"
 #include "csutil/util.h"
+#include "csutil/event.h"
 #include "iutil/document.h"
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
@@ -27,30 +28,8 @@
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csGenmeshAnimationControlType)
-  SCF_IMPLEMENTS_INTERFACE (iGenMeshAnimationControlType)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csGenmeshAnimationControlType::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csGenmeshAnimationControlType)
 
-SCF_IMPLEMENT_IBASE (csGenmeshAnimationControlFactory)
-  SCF_IMPLEMENTS_INTERFACE (iGenMeshAnimationControlFactory)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csGenmeshAnimationControl)
-  SCF_IMPLEMENTS_INTERFACE (iGenMeshAnimationControl)
-  SCF_IMPLEMENTS_INTERFACE (iGenMeshAnimationControlState)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csGenmeshAnimationControlType::EventHandler)
-  SCF_IMPLEMENTS_INTERFACE (iEventHandler)
-SCF_IMPLEMENT_IBASE_END
-
 //-------------------------------------------------------------------------
 
 csAnimControlGroup::csAnimControlGroup (const char* name)
@@ -109,7 +88,7 @@ bool csAnimControlRunnable::Do (csTicks current, bool& stop)
   //-------
   // Perform all running colors.
   //-------
-  size_t i = colors.Length ();
+  size_t i = colors.GetSize ();
   while (i > 0)
   {
     i--;
@@ -130,7 +109,7 @@ bool csAnimControlRunnable::Do (csTicks current, bool& stop)
   //-------
   // Perform all running moves.
   //-------
-  i = moves.Length ();
+  i = moves.GetSize ();
   while (i > 0)
   {
     i--;
@@ -151,7 +130,7 @@ bool csAnimControlRunnable::Do (csTicks current, bool& stop)
   //-------
   // Perform all running scales.
   //-------
-  i = scales.Length ();
+  i = scales.GetSize ();
   while (i > 0)
   {
     i--;
@@ -179,7 +158,7 @@ bool csAnimControlRunnable::Do (csTicks current, bool& stop)
   //-------
   // Perform all running rotates.
   //-------
-  i = rotates.Length ();
+  i = rotates.GetSize ();
   while (i > 0)
   {
     i--;
@@ -299,7 +278,7 @@ bool csAnimControlRunnable::Do (csTicks current, bool& stop)
 	  ac_color_execution m;
 	  m.final = current + inst.color.duration;
 	  m.group = groups[inst.color.group_id];
-	  csColor4 current_col = m.group->GetColor ();
+	  csColor current_col = m.group->GetColor ();
 	  m.final_color.Set (inst.color.red, inst.color.green,
 	  	inst.color.blue);
 	  if (inst.color.duration == 0)
@@ -345,9 +324,9 @@ bool csAnimControlRunnable::Do (csTicks current, bool& stop)
 //-------------------------------------------------------------------------
 
 csGenmeshAnimationControl::csGenmeshAnimationControl (
-	csGenmeshAnimationControlFactory* fact)
+  csGenmeshAnimationControlFactory* fact) :
+  scfImplementationType(this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   factory = fact;
   num_animated_verts = 0;
   animated_verts = 0;
@@ -371,8 +350,6 @@ csGenmeshAnimationControl::~csGenmeshAnimationControl ()
 {
   delete[] animated_verts;
   delete[] animated_colors;
-
-  SCF_DESTRUCT_IBASE ();
 }
 
 void csGenmeshAnimationControl::UpdateAnimation (csTicks current,
@@ -385,16 +362,16 @@ void csGenmeshAnimationControl::UpdateAnimation (csTicks current,
   if (current != last_update_time)
   {
     last_update_time = current;
-    size_t i = running_scripts.Length ();
+    size_t i = running_animations.GetSize ();
     while (i > 0)
     {
       i--;
       bool stop = false;
-      if (running_scripts[i]->Do (current, stop))
+      if (running_animations[i]->Do (current, stop))
         mod = true;
       if (stop)
       {
-        running_scripts.DeleteIndexFast (i);
+        running_animations.DeleteIndexFast (i);
       }
     }
   }
@@ -446,30 +423,30 @@ const csVector3* csGenmeshAnimationControl::UpdateVertices (csTicks current,
     const csPDelArray<csAnimControlGroup>& groups = factory->GetGroups ();
     size_t i;
 
-    if (groups.Length () > group_transforms.Length ())
-      group_transforms.SetLength (groups.Length ());
+    if (groups.GetSize () > group_transforms.GetSize ())
+      group_transforms.SetSize (groups.GetSize ());
 
     if (factory->HasHierarchicalGroups ())
-      for (i = 0 ; i < groups.Length () ; i++)
+      for (i = 0 ; i < groups.GetSize () ; i++)
         group_transforms[i] = groups[i]->GetFullTransform ();
     else
-      for (i = 0 ; i < groups.Length () ; i++)
+      for (i = 0 ; i < groups.GetSize () ; i++)
         group_transforms[i] = groups[i]->GetTransform ();
 
     const csArray<csArray<ac_group_data> >& groups_vertices = factory
     	->GetGroupsVerticesMapping ();
     for (i = 0 ; i < (size_t)num_verts ; i++)
     {
-      if (i >= groups_vertices.Length ())
+      if (i >= groups_vertices.GetSize ())
         animated_verts[i] = verts[i];
       else
       {
         const csArray<ac_group_data>& vtgr = groups_vertices[i];
-	if (vtgr.Length () == 0)
+	if (vtgr.GetSize () == 0)
 	{
 	  animated_verts[i] = verts[i];
 	}
-        else if (vtgr.Length () == 1)
+        else if (vtgr.GetSize () == 1)
 	{
 	  csReversibleTransform& transform = group_transforms[vtgr[0].idx];
 	  animated_verts[i] = transform.Other2This (verts[i]);
@@ -480,7 +457,7 @@ const csVector3* csGenmeshAnimationControl::UpdateVertices (csTicks current,
 	  float total_weight = vtgr[0].weight;
 	  csVector3 orig = vtgr[0].weight * transform.Other2This (verts[i]);
 	  size_t j;
-	  for (j = 1 ; j < vtgr.Length () ; j++)
+	  for (j = 1 ; j < vtgr.GetSize () ; j++)
 	  {
 	    csReversibleTransform& transform2 = group_transforms[vtgr[j].idx];
 	    total_weight += vtgr[j].weight;
@@ -525,30 +502,30 @@ const csColor4* csGenmeshAnimationControl::UpdateColors (csTicks current,
     const csPDelArray<csAnimControlGroup>& groups = factory->GetGroups ();
     size_t i;
 
-    if (groups.Length () > group_colors.Length ())
-      group_colors.SetLength (groups.Length ());
+    if (groups.GetSize () > group_colors.GetSize ())
+      group_colors.SetSize (groups.GetSize ());
 
     if (factory->HasHierarchicalGroups ())
-      for (i = 0 ; i < groups.Length () ; i++)
+      for (i = 0 ; i < groups.GetSize () ; i++)
         group_colors[i] = groups[i]->GetFullColor ();
     else
-      for (i = 0 ; i < groups.Length () ; i++)
+      for (i = 0 ; i < groups.GetSize () ; i++)
         group_colors[i] = groups[i]->GetColor ();
 
     const csArray<csArray<ac_group_data> >& gc = factory
     	->GetGroupsColorsMapping ();
     for (i = 0 ; i < (size_t)num_colors ; i++)
     {
-      if (i >= gc.Length ())
+      if (i >= gc.GetSize ())
         animated_colors[i] = colors[i];
       else
       {
         const csArray<ac_group_data>& vtgr = gc[i];
-	if (vtgr.Length () == 0)
+	if (vtgr.GetSize () == 0)
 	{
 	  animated_colors[i] = colors[i];
 	}
-        else if (vtgr.Length () == 1)
+        else if (vtgr.GetSize () == 1)
 	{
 	  csColor4& color = group_colors[vtgr[0].idx];
 	  animated_colors[i] = color * colors[i];
@@ -559,7 +536,7 @@ const csColor4* csGenmeshAnimationControl::UpdateColors (csTicks current,
 	  float total_weight = vtgr[0].weight;
 	  csColor4 orig = vtgr[0].weight * color * colors[i];
 	  size_t j;
-	  for (j = 1 ; j < vtgr.Length () ; j++)
+	  for (j = 1 ; j < vtgr.GetSize () ; j++)
 	  {
 	    csColor4& color2 = group_colors[vtgr[j].idx];
 	    total_weight += vtgr[j].weight;
@@ -580,16 +557,16 @@ bool csGenmeshAnimationControl::Execute (const char* scriptname)
   if (!script) return false;
   csAnimControlRunnable* runnable = new csAnimControlRunnable (script,
       factory);
-  running_scripts.Push (runnable);
+  running_animations.Push (runnable);
   return true;
 }
 
 //-------------------------------------------------------------------------
 
 csGenmeshAnimationControlFactory::csGenmeshAnimationControlFactory (
-	csGenmeshAnimationControlType* type, iObjectRegistry* object_reg)
+  csGenmeshAnimationControlType* type, iObjectRegistry* object_reg) :
+  scfImplementationType(this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   csGenmeshAnimationControlFactory::type = type;
   csGenmeshAnimationControlFactory::object_reg = object_reg;
   InitTokenTable (xmltokens);
@@ -603,7 +580,6 @@ csGenmeshAnimationControlFactory::csGenmeshAnimationControlFactory (
 
 csGenmeshAnimationControlFactory::~csGenmeshAnimationControlFactory ()
 {
-  SCF_DESTRUCT_IBASE ();
 }
 
 csPtr<iGenMeshAnimationControl> csGenmeshAnimationControlFactory::
@@ -611,7 +587,7 @@ csPtr<iGenMeshAnimationControl> csGenmeshAnimationControlFactory::
 {
   csGenmeshAnimationControl* ctrl = new csGenmeshAnimationControl (this);
   size_t i;
-  for (i = 0 ; i < autorun_scripts.Length () ; i++)
+  for (i = 0 ; i < autorun_scripts.GetSize () ; i++)
     ctrl->Execute (autorun_scripts[i]);
   return csPtr<iGenMeshAnimationControl> (ctrl);
 }
@@ -619,12 +595,12 @@ csPtr<iGenMeshAnimationControl> csGenmeshAnimationControlFactory::
 void csGenmeshAnimationControlFactory::UpdateGroupsMapping ()
 {
   size_t i;
-  for (i = 0 ; i < groups.Length () ; i++)
+  for (i = 0 ; i < groups.GetSize () ; i++)
   {
     csAnimControlGroup* g = groups[i];
     const csArray<ac_vertex_data>& vtdata = g->GetVertexData ();
     size_t j;
-    for (j = 0 ; j < vtdata.Length () ; j++)
+    for (j = 0 ; j < vtdata.GetSize () ; j++)
     {
       if (vtdata[j].weight > SMALL_EPSILON)
       {
@@ -652,7 +628,7 @@ csAnimControlScript* csGenmeshAnimationControlFactory::FindScript (
 	const char* scriptname) const
 {
   size_t i;
-  for (i = 0 ; i < scripts.Length () ; i++)
+  for (i = 0 ; i < scripts.GetSize () ; i++)
     if (strcmp (scripts[i]->GetName (), scriptname) == 0)
       return scripts[i];
   return 0;
@@ -662,7 +638,7 @@ csAnimControlGroup* csGenmeshAnimationControlFactory::FindGroup (
 	const char* groupname) const
 {
   size_t i;
-  for (i = 0 ; i < groups.Length () ; i++)
+  for (i = 0 ; i < groups.GetSize () ; i++)
     if (strcmp (groups[i]->GetName (), groupname) == 0)
       return groups[i];
   return 0;
@@ -672,7 +648,7 @@ size_t csGenmeshAnimationControlFactory::FindGroupIndex (
 	const char* groupname) const
 {
   size_t i;
-  for (i = 0 ; i < groups.Length () ; i++)
+  for (i = 0 ; i < groups.GetSize () ; i++)
     if (strcmp (groups[i]->GetName (), groupname) == 0)
       return i;
   return (size_t)~0;
@@ -940,7 +916,7 @@ const char* csGenmeshAnimationControlFactory::Load (iDocumentNode* node)
 
 const char* csGenmeshAnimationControlFactory::Save (iDocumentNode* parent)
 {
-  csRef<iFactory> plugin = SCF_QUERY_INTERFACE(type, iFactory);
+  csRef<iFactory> plugin = scfQueryInterface<iFactory> (type);
   if (!plugin) return "Couldn't get Class ID";
   parent->SetAttribute("plugin", plugin->QueryClassID());
   return "Not implemented yet!";
@@ -949,37 +925,31 @@ const char* csGenmeshAnimationControlFactory::Save (iDocumentNode* parent)
 //-------------------------------------------------------------------------
 
 csGenmeshAnimationControlType::csGenmeshAnimationControlType (
-	iBase* pParent)
+  iBase* pParent) :
+  scfImplementationType(this, pParent), object_reg(0)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
-  scfiEventHandler = 0;
 }
 
 csGenmeshAnimationControlType::~csGenmeshAnimationControlType ()
 {
-  if (scfiEventHandler)
+  if (weakEventHandler)
   {
-    csRef<iEventQueue> q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+    csRef<iEventQueue> q = csQueryRegistry<iEventQueue> (object_reg);
     if (q)
-      q->RemoveListener (scfiEventHandler);
-    scfiEventHandler->DecRef ();
+      CS::RemoveWeakListener (q, weakEventHandler);
   }
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csGenmeshAnimationControlType::Initialize (iObjectRegistry* object_reg)
 {
-  csGenmeshAnimationControlType::object_reg = object_reg;
+  this->object_reg = object_reg;
   Frame = csevFrame (object_reg);
   PreProcess = csevPreProcess (object_reg);
-  scfiEventHandler = new EventHandler (this);
-  csRef<iEventQueue> q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
+  csRef<iEventQueue> q = csQueryRegistry<iEventQueue> (object_reg);
   // \todo It looks like @csGenmeshAnimationControlType doesn't actually handle any events.  So why does it register an event listener?
   if (q != 0) {
     csEventID events[] = { Frame, PreProcess, CS_EVENTLIST_END };
-    q->RegisterListener (scfiEventHandler, events);
+    CS::RegisterWeakListener (q, this, events, weakEventHandler);
   }
   return true;
 }
