@@ -19,7 +19,8 @@
 #ifndef __CS_GENMESH_H__
 #define __CS_GENMESH_H__
 
-#include "csgeom/objmodel.h"
+#include "cstool/objmodel.h"
+#include "csgeom/box.h"
 #include "csgeom/transfrm.h"
 #include "csgeom/vector3.h"
 #include "csgeom/vector4.h"
@@ -27,13 +28,14 @@
 #include "csgfx/shadervarcontext.h"
 #include "cstool/rendermeshholder.h"
 #include "cstool/userrndbuf.h"
-#include "csutil/cscolor.h"
+#include "csutil/cscolor.h" 
 #include "csutil/dirtyaccessarray.h"
 #include "csutil/flags.h"
 #include "csutil/hash.h"
 #include "csutil/leakguard.h"
 #include "csutil/refarr.h"
 #include "csutil/parray.h"
+#include "csutil/pooledscfclass.h"
 #include "csutil/weakref.h"
 #include "iengine/light.h"
 #include "iengine/lightmgr.h"
@@ -45,16 +47,16 @@
 #include "iutil/comp.h"
 #include "iutil/eventh.h"
 #include "iutil/virtclk.h"
+#include "iutil/strset.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/rendermesh.h"
 #include "ivideo/rndbuf.h"
 
+#include "submeshes.h"
+
 class csBSPTree;
 class csColor;
 class csColor4;
-class csGenmeshMeshObject;
-class csGenmeshMeshObjectFactory;
-class csGenmeshMeshObjectType;
 class csPolygonMesh;
 struct iCacheManager;
 struct iEngine;
@@ -63,6 +65,13 @@ struct iMovable;
 struct iObjectRegistry;
 struct iShadowBlockList;
 
+CS_PLUGIN_NAMESPACE_BEGIN(Genmesh)
+{
+
+class csGenmeshMeshObject;
+class csGenmeshMeshObjectFactory;
+class csGenmeshMeshObjectType;
+
 /**
  * An array giving shadow information for a pseudo-dynamic light.
  */
@@ -81,30 +90,65 @@ public:
   }
 };
 
-struct csGenmeshSubMesh
+class MergedSVContext : 
+  public scfImplementationPooled<scfImplementation1<MergedSVContext,
+                                                    iShaderVariableContext> >
 {
-  csRef<iRenderBuffer> index_buffer;
-  csRef<iMaterialWrapper> material;
-  csRenderMeshHolder rmHolder;
-  csRef<csRenderBufferHolder> bufferHolder;
-  int tricount;
+  // context1 has precedence
+  iShaderVariableContext* context1;
+  iShaderVariableContext* context2;
+public:
+  MergedSVContext (iShaderVariableContext* context1, 
+    iShaderVariableContext* context2) : scfPooledImplementationType (this),
+    context1 (context1), context2 (context2)
+  { }
 
-  // Override mixmode from parent.
-  bool override_mixmode;
-  uint MixMode;
+  void AddVariable (csShaderVariable *variable)
+  { }
+  csShaderVariable* GetVariable (csStringID name) const
+  { 
+    csShaderVariable* sv = context1->GetVariable (name); 
+    if (sv == 0)
+      sv = context2->GetVariable (name);
+    return sv;
+  }
+  const csRefArray<csShaderVariable>& GetShaderVariables () const
+  { 
+    return context1->GetShaderVariables();
+  }
+  void PushVariables (iShaderVarStack* stacks) const
+  { 
+    context2->PushVariables (stacks);
+    context1->PushVariables (stacks);
+  }
+
+  bool IsEmpty () const { return context1->IsEmpty() && context2->IsEmpty(); }
+
+  void ReplaceVariable (csShaderVariable *variable) { }
+  void Clear () { }
+  bool RemoveVariable  (csShaderVariable *) { return false; }
 };
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 /**
  * Genmesh version of mesh object.
  */
-class csGenmeshMeshObject : public iMeshObject
+class csGenmeshMeshObject : public scfImplementation5<csGenmeshMeshObject, 
+						      iMeshObject,
+						      iLightingInfo,
+						      iShadowCaster,
+						      iShadowReceiver,
+						      iGeneralMeshState>
 {
 private:
+
   csRenderMeshHolder rmHolder;
   csRef<csShaderVariableContext> svcontext;
   csRef<csRenderBufferHolder> bufferHolder;
   csWeakRef<iGraphics3D> g3d;
   bool mesh_colors_dirty_flag;
+  bool mesh_user_rb_dirty_flag;
 
   uint buffers_version;
   csRef<iRenderBuffer> sorted_index_buffer;	// Only if factory back2front
@@ -116,11 +160,24 @@ private:
   csRef<iRenderBuffer> texel_buffer;
   csRef<iRenderBuffer> normal_buffer;
 
+  size_t factory_user_rb_state;
+
   csRef<iRenderBuffer> color_buffer;
   iMovable* lighting_movable;
 
-  csPDelArray<csGenmeshSubMesh> subMeshes;
   csDirtyAccessArray<csRenderMesh*> renderMeshes;
+  mutable SubMeshProxiesContainer subMeshes;
+  mutable uint factorySubMeshesChangeNum;
+  void UpdateSubMeshProxies () const;
+  struct LegacySubmesh
+  {
+    csRef<iRenderBuffer> indexbuffer;
+    csRef<iMaterialWrapper> material;
+    uint mixmode;
+    csRenderMeshHolder rmHolder;
+    csRef<csRenderBufferHolder> bufferHolder;
+  };
+  csArray<LegacySubmesh> legacySubmeshes;
 
   csUserRenderBufferManager userBuffers;
   csArray<csStringID> user_buffer_names;
@@ -130,7 +187,7 @@ private:
   csRef<iMaterialWrapper> material;
   bool material_needs_visit;
   uint MixMode;
-  iMeshObjectDrawCallback* vis_cb;
+  csRef<iMeshObjectDrawCallback> vis_cb;
   bool do_lighting;
   bool do_manual_colors;
   csColor4 base_color;
@@ -210,25 +267,39 @@ public:
   /// Destructor.
   virtual ~csGenmeshMeshObject ();
 
-  void ClearSubMeshes ();
-  void AddSubMesh (unsigned int *triangles,
-    int tricount, iMaterialWrapper *material, bool do_mixmode, uint mixmode);
-
+  /**\name iMeshObject implementation
+   * @{ */
   void SetMixMode (uint mode)
   {
     MixMode = mode;
   }
   uint GetMixMode () const { return MixMode; }
+  const csColor& GetColor () const { return base_color; }
+  /** @} */
+  
+  /**\name iGeneralMeshState implementation
+   * @{ */
   void SetLighting (bool l) { do_lighting = l; }
   bool IsLighting () const { return do_lighting; }
-  const csColor& GetColor () const { return base_color; }
   void SetManualColors (bool m) { do_manual_colors = m; }
   bool IsManualColors () const { return do_manual_colors; }
   void GetObjectBoundingBox (csBox3& bbox);
+  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  void GetRadius (csVector3& rad, csVector3& cent);
+  void GetRadius (float& rad, csVector3& cent);
   void SetShadowCasting (bool m) { do_shadows = m; }
+  bool IsShadowCasting () const { return do_shadows; }
   void SetShadowReceiving (bool m) { do_shadow_rec = m; }
+  bool IsShadowReceiving () const { return do_shadow_rec; }
+  iGeneralMeshSubMesh* FindSubMesh (const char* name) const; 
+  void AddSubMesh (unsigned int *triangles, int tricount, 
+    iMaterialWrapper *material, uint mixmode);
+  void AddSubMesh (unsigned int *triangles, int tricount, 
+    iMaterialWrapper *material)
+  {
+    AddSubMesh (triangles, tricount, material, (uint)~0);
+  }
+  /** @} */
 
   iVirtualClock* vc;
   csRef<iGenMeshAnimationControl> anim_ctrl;
@@ -251,8 +322,16 @@ public:
 
   bool AddRenderBuffer (const char *name, iRenderBuffer* buffer);
   bool RemoveRenderBuffer (const char *name);
+  int GetRenderBufferCount () const
+  {
+    return (int)this->user_buffer_names.GetSize ();
+  }
+  csRef<iRenderBuffer> GetRenderBuffer (int index); 
+  csRef<iString> GetRenderBufferName (int index) const;
 
-  //----------------------- Shadow and lighting system ----------------------
+
+  /**\name Shadow and lighting system
+   * @{ */
   char* GenerateCacheName ();
   void InitializeDefault (bool clear);
   bool ReadFromCache (iCacheManager* cache_mgr);
@@ -265,22 +344,17 @@ public:
   void LightChanged (iLight* light);
   void LightDisconnect (iLight* light);
   void DisconnectAllLights ();
+  /** @} */
 
-  //----------------------- iMeshObject implementation ----------------------
-  SCF_DECLARE_IBASE;
-
-  virtual iMeshObjectFactory* GetFactory () const
-  {
-    return (iMeshObjectFactory*)factory;
-  }
+  /**\name iMeshObject implementation
+   * @{ */
+  virtual iMeshObjectFactory* GetFactory () const;
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> Clone () { return 0; }
   virtual csRenderMesh** GetRenderMeshes (int &n, iRenderView* rview, 
     iMovable* movable, uint32 frustum_mask);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* cb)
   {
-    if (cb) cb->IncRef ();
-    if (vis_cb) vis_cb->DecRef ();
     vis_cb = cb;
   }
   virtual iMeshObjectDrawCallback* GetVisibleCallback () const
@@ -288,7 +362,7 @@ public:
     return vis_cb;
   }
   virtual void NextFrame (csTicks /*current_time*/,
-  	const csVector3& /*pos*/) { }
+  	const csVector3& /*pos*/, uint /*currentFrame*/) { }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
   virtual bool HitBeamOutline (const csVector3& start, const csVector3& end,
@@ -320,207 +394,52 @@ public:
    */
   virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) { }
 
-  //------------------------- iLightingInfo interface -------------------------
-  struct LightingInfo : public iLightingInfo
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObject);
-    virtual void InitializeDefault (bool clear)
-    {
-      scfParent->InitializeDefault (clear);
-    }
-    virtual bool ReadFromCache (iCacheManager* cache_mgr)
-    {
-      return scfParent->ReadFromCache (cache_mgr);
-    }
-    virtual bool WriteToCache (iCacheManager* cache_mgr)
-    {
-      return scfParent->WriteToCache (cache_mgr);
-    }
-    virtual void PrepareLighting ()
-    {
-      scfParent->PrepareLighting ();
-    }
-    virtual void LightChanged (iLight* light)
-    {
-      scfParent->LightChanged (light);
-    }
-    virtual void LightDisconnect (iLight* light)
-    {
-      scfParent->LightDisconnect (light);
-    }
-    virtual void DisconnectAllLights ()
-    {
-      scfParent->DisconnectAllLights ();
-    }
-  } scfiLightingInfo;
-  friend struct LightingInfo;
+  virtual void BuildDecal(const csVector3* pos, float decalRadius,
+          iDecalBuilder* decalBuilder);
+  /** @} */
 
-  //-------------------- iShadowCaster interface implementation ----------
-  struct ShadowCaster : public iShadowCaster
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObject);
-    virtual void AppendShadows (iMovable* movable, iShadowBlockList* shadows,
-    	const csVector3& origin)
-    {
-      scfParent->AppendShadows (movable, shadows, origin);
-    }
-  } scfiShadowCaster;
-  friend struct ShadowCaster;
-
-  //-------------------- iShadowReceiver interface implementation ----------
-  struct ShadowReceiver : public iShadowReceiver
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObject);
-    virtual void CastShadows (iMovable* movable, iFrustumView* fview)
-    {
-      scfParent->CastShadows (movable, fview);
-    }
-  } scfiShadowReceiver;
-  friend struct ShadowReceiver;
-
-  //------------------------- iGeneralMeshState implementation ----------------
-  class GeneralMeshState : public iGeneralMeshState
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObject);
-    virtual void SetMaterialWrapper (iMaterialWrapper* material)
-    { scfParent->SetMaterialWrapper (material); }
-    virtual iMaterialWrapper* GetMaterialWrapper () const
-    { return scfParent->material; }
-    virtual void SetMixMode (uint mode) { scfParent->MixMode = mode; }
-    virtual uint GetMixMode () const { return scfParent->MixMode; }
-    virtual void SetLighting (bool l) { scfParent->SetLighting (l); }
-    virtual bool IsLighting () const { return scfParent->IsLighting (); }
-    virtual void SetColor (const csColor& col) { scfParent->SetColor (col); }
-    virtual const csColor& GetColor () const { return scfParent->GetColor (); }
-    virtual void SetManualColors (bool m)
-    {
-      scfParent->SetManualColors (m);
-    }
-    virtual bool IsManualColors () const
-    {
-      return scfParent->IsManualColors ();
-    }
-    virtual void SetShadowCasting (bool m)
-    {
-      scfParent->do_shadows = m;
-    }
-    virtual bool IsShadowCasting () const
-    {
-      return scfParent->do_shadows;
-    }
-    virtual void SetShadowReceiving (bool m)
-    {
-      scfParent->do_shadow_rec = m;
-    }
-    virtual bool IsShadowReceiving () const
-    {
-      return scfParent->do_shadow_rec;
-    }
-    virtual void SetAnimationControl (iGenMeshAnimationControl* anim_ctrl)
-    {
-      scfParent->SetAnimationControl (anim_ctrl);
-    }
-    virtual iGenMeshAnimationControl* GetAnimationControl () const
-    {
-      return scfParent->GetAnimationControl ();
-    }
-    virtual void ClearSubMeshes ()
-    {
-      scfParent->ClearSubMeshes ();
-    }
-    virtual void AddSubMesh (unsigned int *triangles,
-      int tricount,
-      iMaterialWrapper *material)
-    {
-      scfParent->AddSubMesh (triangles, tricount, material, false, CS_FX_COPY);
-    }
-    virtual void AddSubMesh (unsigned int *triangles,
-      int tricount, iMaterialWrapper *material, uint mixmode)
-    {
-      scfParent->AddSubMesh (triangles, tricount, material, true, mixmode);
-    }
-    virtual bool AddRenderBuffer (const char *name, iRenderBuffer* buffer)
-    { return scfParent->AddRenderBuffer (name, buffer); }
-    virtual bool RemoveRenderBuffer (const char *name)
-    { return scfParent->RemoveRenderBuffer (name); }
-  } scfiGeneralMeshState;
-  friend class GeneralMeshState;
-
-  //------------------ iPolygonMesh interface implementation ----------------//
-  struct PolyMesh : public iPolygonMesh
-  {
-  private:
-    csFlags flags;
-
-  public:
-    SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObject);
-
-    virtual int GetVertexCount ();
-    virtual csVector3* GetVertices ();
-    virtual int GetPolygonCount ();
-    virtual csMeshedPolygon* GetPolygons ();
-    virtual int GetTriangleCount ();
-    virtual csTriangle* GetTriangles ();
-    virtual void Lock () { }
-    virtual void Unlock () { }
-    
-    virtual csFlags& GetFlags () { return flags;  }
-    virtual uint32 GetChangeNumber() const { return 0; }
-
-    PolyMesh ()
-    {
-      flags.Set (CS_POLYMESH_TRIANGLEMESH);
-    }
-    virtual ~PolyMesh () { }
-  } scfiPolygonMesh;
-  friend struct PolyMesh;
-
-  class eiRenderBufferAccessor : public iRenderBufferAccessor
+  class RenderBufferAccessor : 
+    public scfImplementation1<RenderBufferAccessor, iRenderBufferAccessor>
   {
   public:
-    CS_LEAKGUARD_DECLARE (eiRenderBufferAccessor);
-    SCF_DECLARE_IBASE;
-    csGenmeshMeshObject* parent;
-    virtual ~eiRenderBufferAccessor ()
+    CS_LEAKGUARD_DECLARE (RenderBufferAccessor);
+    csWeakRef<csGenmeshMeshObject> parent;
+    virtual ~RenderBufferAccessor () { }
+    RenderBufferAccessor (csGenmeshMeshObject* parent)
+    	: scfImplementationType (this)
     {
-      SCF_DESTRUCT_IBASE ();
-    }
-    eiRenderBufferAccessor (csGenmeshMeshObject* parent)
-    {
-      SCF_CONSTRUCT_IBASE (0);
-      eiRenderBufferAccessor::parent = parent;
+      this->parent = parent;
     }
     virtual void PreGetBuffer (csRenderBufferHolder* holder,
     	csRenderBufferName buffer)
     {
-      parent->PreGetBuffer (holder, buffer);
+      if (parent) parent->PreGetBuffer (holder, buffer);
     }
-  } *scfiRenderBufferAccessor;
-  friend class eiRenderBufferAccessor;
+  };
+  csRef<RenderBufferAccessor> renderBufferAccessor;
+  friend class RenderBufferAccessor;
 
   void PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer);
 
   //------------------ iShaderVariableAccessor implementation ------------
-  class eiShaderVariableAccessor : public iShaderVariableAccessor
+  class ShaderVariableAccessor : 
+    public scfImplementation1<ShaderVariableAccessor, iShaderVariableAccessor>
   {
   public:
-    SCF_DECLARE_IBASE;
-    csGenmeshMeshObject* parent;
-    virtual ~eiShaderVariableAccessor ()
+    csWeakRef<csGenmeshMeshObject> parent;
+    virtual ~ShaderVariableAccessor () { }
+    ShaderVariableAccessor (csGenmeshMeshObject* parent)
+    	: scfImplementationType (this)
     {
-      SCF_DESTRUCT_IBASE ();
-    }
-    eiShaderVariableAccessor (csGenmeshMeshObject* parent)
-    {
-      SCF_CONSTRUCT_IBASE (0);
-      eiShaderVariableAccessor::parent = parent;
+      this->parent = parent;
     }
     virtual void PreGetValue (csShaderVariable* variable)
     {
-      parent->PreGetShaderVariableValue (variable);
+      if (parent) parent->PreGetShaderVariableValue (variable);
     }
-  } *scfiShaderVariableAccessor;
-  friend class eiShaderVariableAccessor;
+  };
+  csRef<ShaderVariableAccessor> shaderVariableAccessor;
+  friend class ShaderVariableAccessor;
 
   void PreGetShaderVariableValue (csShaderVariable* variable);
 };
@@ -528,14 +447,17 @@ public:
 /**
  * Factory for general meshes.
  */
-class csGenmeshMeshObjectFactory : public iMeshObjectFactory
+class csGenmeshMeshObjectFactory : 
+  public scfImplementationExt2<csGenmeshMeshObjectFactory,
+                               csObjectModel,
+                               iMeshObjectFactory,
+                               iGeneralFactoryState>
 {
-private:
+public:
   csRef<iMaterialWrapper> material;
   csDirtyAccessArray<csVector3> mesh_vertices;
   csDirtyAccessArray<csVector2> mesh_texels;
   csDirtyAccessArray<csVector3> mesh_normals;
-  csDirtyAccessArray<csColor4> mesh_colors;
 
   bool autonormals;
   bool autonormals_compress;
@@ -564,7 +486,7 @@ private:
   csUserRenderBufferManager userBuffers;
   csArray<csStringID> user_buffer_names;
 
-  csPDelArray<csGenmeshSubMesh> subMeshes;
+  SubMeshesContainer subMeshes;
 
   uint default_mixmode;
   bool default_lighting;
@@ -573,7 +495,9 @@ private:
   bool default_shadowcasting;
   bool default_shadowreceiving;
 
-  csVector3 radius;
+  size_t user_buffer_change;
+
+  float radius;
   csBox3 object_bbox;
   bool object_bbox_valid;
   bool initialized;
@@ -596,7 +520,8 @@ private:
    * Setup this factory. This function will check if setup is needed.
    */
   void SetupFactory ();
-
+private:
+  csDirtyAccessArray<csColor4> mesh_colors;
 public:
   CS_LEAKGUARD_DECLARE (csGenmeshMeshObjectFactory);
 
@@ -609,19 +534,19 @@ public:
 
   iObjectRegistry* object_reg;
   iMeshFactoryWrapper* logparent;
-  iMeshObjectType* genmesh_type;
+  csRef<csGenmeshMeshObjectType> genmesh_type;
   csRef<iLightManager> light_mgr;
   csFlags flags;
 
   iEngine* engine;
 
-  const csPDelArray<csGenmeshSubMesh>& GetSubMeshes () const
+  SubMeshesContainer& GetSubMeshes ()
   {
     return subMeshes;
   }
 
   /// Constructor.
-  csGenmeshMeshObjectFactory (iMeshObjectType *pParent,
+  csGenmeshMeshObjectFactory (csGenmeshMeshObjectType* pParent,
   	iObjectRegistry* object_reg);
 
   /// Destructor.
@@ -630,16 +555,17 @@ public:
   /// Do full bright.
   bool DoFullBright () const { return do_fullbright; }
 
-  void SetMaterialWrapper (iMaterialWrapper* material)
+  bool SetMaterialWrapper (iMaterialWrapper* material)
   {
     csGenmeshMeshObjectFactory::material = material;
+    return true;
   }
   iMaterialWrapper* GetMaterialWrapper () const { return material; }
   void AddVertex (const csVector3& v,
       const csVector2& uv, const csVector3& normal,
       const csColor4& color);
   void SetVertexCount (int n);
-  int GetVertexCount () const { return (int)mesh_vertices.Length (); }
+  int GetVertexCount () const { return (int)mesh_vertices.GetSize (); }
   csVector3* GetVertices ()
   {
     SetupFactory ();
@@ -655,16 +581,25 @@ public:
     SetupFactory ();
     return mesh_normals.GetArray ();
   }
+  csColor4* GetColors (bool ensureValid)
+  {
+    if (ensureValid && (mesh_colors.GetSize() == 0))
+    {
+      mesh_colors.SetCapacity (mesh_vertices.Capacity());
+      mesh_colors.SetSize (mesh_vertices.GetSize(), csColor4 (0, 0, 0, 1));
+    }
+    return mesh_colors.GetArray();
+  }
   csColor4* GetColors ()
   {
     SetupFactory ();
-    return mesh_colors.GetArray ();
+    return GetColors (true);
   }
 
   void AddTriangle (const csTriangle& tri);
   void SetTriangleCount (int n);
 
-  int GetTriangleCount () const { return (int)mesh_triangles.Length (); }
+  int GetTriangleCount () const { return (int)mesh_triangles.GetSize (); }
   csTriangle* GetTriangles ()
   {
     SetupFactory ();
@@ -673,8 +608,11 @@ public:
 
   void Invalidate ();
   void CalculateNormals (bool compress);
+  void DisableAutoNormals ()
+  { autonormals = false; }
   void Compress ();
   void GenerateBox (const csBox3& box);
+  void GenerateCapsule (float l, float r, uint sides);
   void GenerateSphere (const csEllipsoid& ellips, int rim_vertices,
       	bool cyl_mapping = false, bool toponly = false,
 	bool reversed = false);
@@ -685,6 +623,13 @@ public:
 
   bool AddRenderBuffer (const char *name, iRenderBuffer* buffer);
   bool RemoveRenderBuffer (const char *name);
+  int GetRenderBufferCount () const
+  {
+    return (int)this->user_buffer_names.GetSize ();
+  }
+  csRef<iRenderBuffer> GetRenderBuffer (int index); 
+  csRef<iString> GetRenderBufferName (int index) const;
+
   /**
    * Get the string ID's for the anonymous buffers
    */
@@ -697,11 +642,48 @@ public:
 
   void ClearSubMeshes ();
   void AddSubMesh (unsigned int *triangles,
-    int tricount, iMaterialWrapper *material, bool do_mixmode, uint mixmode);
+    int tricount, iMaterialWrapper *material, uint mixmode);
+  virtual void AddSubMesh (unsigned int *triangles, int tricount, 
+    iMaterialWrapper *material)
+  {
+    if (polyMeshType != Submeshes) SetPolyMeshSubmeshes();
+    AddSubMesh (triangles, tricount, material, (uint)~0);
+  }
+  iGeneralMeshSubMesh* AddSubMesh (iRenderBuffer* indices, 
+    iMaterialWrapper *material, const char* name, uint mixmode);
+  iGeneralMeshSubMesh* FindSubMesh (const char* name) const
+  {
+    return subMeshes.FindSubMesh (name);
+  }
+  void DeleteSubMesh (iGeneralMeshSubMesh* mesh)
+  {
+    subMeshes.DeleteSubMesh (mesh);
+  }
+  size_t GetSubMeshCount () const
+  {
+    return subMeshes.GetSubMeshCount ();
+  }
+  iGeneralMeshSubMesh* GetSubMesh (size_t index) const
+  {
+    return subMeshes.GetSubMesh (index);
+  }
 
-  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  const csVector3& GetRadius ();
+  float GetRadius ();
+
+  /**\name iObjectModel implementation
+   * @{ */
+  virtual void GetObjectBoundingBox (csBox3& bbox)
+  {
+    bbox = GetObjectBoundingBox ();
+  }
+  virtual const csBox3& GetObjectBoundingBox ();
+  virtual void GetRadius (float& rad, csVector3& cent)
+  {
+    rad = GetRadius ();
+    cent = object_bbox.GetCenter ();
+  }
+  /** @} */
 
   /**
    * Calculate polygons for iPolygonMesh.
@@ -762,8 +744,6 @@ public:
   }
 
   //------------------------ iMeshObjectFactory implementation --------------
-  SCF_DECLARE_IBASE;
-
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> NewInstance ();
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
@@ -776,192 +756,15 @@ public:
   }
   virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
   { return logparent; }
-  virtual iMeshObjectType* GetMeshObjectType () const { return genmesh_type; }
-
-  //----------------------- iGeneralFactoryState implementation -------------
-  class GeneralFactoryState : public iGeneralFactoryState
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObjectFactory);
-
-    virtual void SetMixMode (uint mode)
-    {
-      scfParent->SetMixMode (mode);
-    }
-    virtual uint GetMixMode () const
-    {
-      return scfParent->GetMixMode ();
-    }
-    virtual void SetLighting (bool l)
-    {
-      scfParent->SetLighting (l);
-    }
-    virtual bool IsLighting () const
-    {
-      return scfParent->IsLighting ();
-    }
-    virtual void SetColor (const csColor& col)
-    {
-      scfParent->SetColor (col);
-    }
-    virtual const csColor& GetColor () const
-    {
-      return scfParent->GetColor ();
-    }
-    virtual void SetManualColors (bool m)
-    {
-      scfParent->SetManualColors (m);
-    }
-    virtual bool IsManualColors () const
-    {
-      return scfParent->IsManualColors ();
-    }
-    virtual void SetShadowCasting (bool m)
-    {
-      scfParent->SetShadowCasting (m);
-    }
-    virtual bool IsShadowCasting () const
-    {
-      return scfParent->IsShadowCasting ();
-    }
-    virtual void SetShadowReceiving (bool m)
-    {
-      scfParent->SetShadowReceiving (m);
-    }
-    virtual bool IsShadowReceiving () const
-    {
-      return scfParent->IsShadowReceiving ();
-    }
-
-    virtual void SetMaterialWrapper (iMaterialWrapper* material)
-    {
-      scfParent->SetMaterialWrapper (material);
-    }
-    virtual iMaterialWrapper* GetMaterialWrapper () const
-    {
-      return scfParent->GetMaterialWrapper ();
-    }
-    virtual void AddVertex (const csVector3& v,
-      const csVector2& uv, const csVector3& normal,
-      const csColor4& color)
-    {
-      scfParent->AddVertex (v, uv, normal, color);
-    }
-    virtual void SetVertexCount (int n)
-    {
-      scfParent->SetVertexCount (n);
-    }
-    virtual int GetVertexCount () const
-    {
-      return scfParent->GetVertexCount ();
-    }
-    virtual csVector3* GetVertices ()
-    {
-      return scfParent->GetVertices ();
-    }
-    virtual csVector2* GetTexels ()
-    {
-      return scfParent->GetTexels ();
-    }
-    virtual csVector3* GetNormals ()
-    {
-      return scfParent->GetNormals ();
-    }
-    virtual void AddTriangle (const csTriangle& tri)
-    {
-      scfParent->AddTriangle (tri);
-    }
-    virtual void SetTriangleCount (int n)
-    {
-      scfParent->SetTriangleCount (n);
-    }
-    virtual int GetTriangleCount () const
-    {
-      return scfParent->GetTriangleCount ();
-    }
-    virtual csTriangle* GetTriangles ()
-    {
-      return scfParent->GetTriangles ();
-    }
-    virtual csColor4* GetColors ()
-    {
-      return scfParent->GetColors ();
-    }
-    virtual void Invalidate ()
-    {
-      scfParent->Invalidate ();
-    }
-    virtual void Compress ()
-    {
-      scfParent->Compress ();
-    }
-    virtual void CalculateNormals (bool compress = true)
-    {
-      scfParent->CalculateNormals (compress);
-    }
-    virtual void GenerateBox (const csBox3& box)
-    {
-      scfParent->GenerateBox (box);
-    }
-    virtual void GenerateSphere (const csEllipsoid& ellips, int rim_vertices,
-		bool cyl_mapping = false, bool toponly = false,
-		bool reversed = false)
-    {
-      scfParent->GenerateSphere (ellips, rim_vertices,
-	  cyl_mapping, toponly, reversed);
-    }
-    virtual void SetBack2Front (bool b2f)
-    {
-      scfParent->SetBack2Front (b2f);
-    }
-    virtual bool IsAutoNormals () const
-    {
-      return scfParent->IsAutoNormals ();
-    }
-    virtual bool IsBack2Front () const
-    {
-      return scfParent->IsBack2Front ();
-    }
-    virtual void SetAnimationControlFactory (iGenMeshAnimationControlFactory*
-    	anim_ctrl_fact)
-    {
-      scfParent->SetAnimationControlFactory (anim_ctrl_fact);
-    }
-    virtual iGenMeshAnimationControlFactory* GetAnimationControlFactory () const
-    {
-      return scfParent->GetAnimationControlFactory ();
-    }
-    virtual bool AddRenderBuffer (const char *name, iRenderBuffer* buffer)
-    { return scfParent->AddRenderBuffer (name, buffer); }
-    virtual bool RemoveRenderBuffer (const char *name)
-    { return scfParent->RemoveRenderBuffer (name); }
-    virtual void ClearSubMeshes ()
-    {
-      scfParent->ClearSubMeshes ();
-    }
-    virtual void AddSubMesh (unsigned int *triangles,
-      int tricount,
-      iMaterialWrapper *material)
-    {
-      scfParent->AddSubMesh (triangles, tricount, material, false, CS_FX_COPY);
-    }
-    virtual void AddSubMesh (unsigned int *triangles,
-      int tricount, iMaterialWrapper *material, uint mixmode)
-    {
-      scfParent->AddSubMesh (triangles, tricount, material, true, mixmode);
-    }
-  } scfiGeneralFactoryState;
-  friend class GeneralFactoryState;
+  virtual iMeshObjectType* GetMeshObjectType () const; 
 
   //------------------ iPolygonMesh interface implementation ----------------//
-  struct PolyMesh : public iPolygonMesh
+  struct PolyMesh : public scfImplementation1<PolyMesh, iPolygonMesh>
   {
   private:
     csGenmeshMeshObjectFactory* factory;
     csFlags flags;
   public:
-    //SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObjectFactory);
-    SCF_DECLARE_IBASE;
-
     void SetFactory (csGenmeshMeshObjectFactory* Factory)
     { factory = Factory; }
 
@@ -977,104 +780,92 @@ public:
     virtual csFlags& GetFlags () { return flags;  }
     virtual uint32 GetChangeNumber() const { return 0; }
 
-    PolyMesh ()
+    PolyMesh () : scfImplementationType (this)
     {
-      SCF_CONSTRUCT_IBASE (0);
       flags.Set (CS_POLYMESH_TRIANGLEMESH);
     }
     virtual ~PolyMesh ()
     {
-      SCF_DESTRUCT_IBASE ();
     }
-  } scfiPolygonMesh;
+  };
+  csRef<iPolygonMesh> polygonMesh;
   friend struct PolyMesh;
+  enum { Standard, Submeshes } polyMeshType;
 
-  //------------------------- iObjectModel implementation ----------------
-  class ObjectModel : public csObjectModel
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObjectFactory);
-    virtual void GetObjectBoundingBox (csBox3& bbox)
-    {
-      bbox = scfParent->GetObjectBoundingBox ();
-    }
-    virtual void SetObjectBoundingBox (const csBox3& bbox)
-    {
-      scfParent->SetObjectBoundingBox (bbox);
-    }
-    virtual void GetRadius (csVector3& rad, csVector3& cent)
-    {
-      rad = scfParent->GetRadius ();
-      cent.Set (0.0f);
-    }
-  } scfiObjectModel;
-  friend class ObjectModel;
+  void SetPolyMeshStandard ();
+  void SetPolyMeshSubmeshes ();
+  virtual iObjectModel* GetObjectModel () { return this; }
 
-  virtual iObjectModel* GetObjectModel () { return &scfiObjectModel; }
-
-  //------------------ iShaderVariableAccessor implementation ------------
-  class eiShaderVariableAccessor : public iShaderVariableAccessor
+  /// Genmesh factory shader variable accessor
+  class ShaderVariableAccessor : 
+    public scfImplementation1<ShaderVariableAccessor, iShaderVariableAccessor>
   {
   public:
     //SCF_DECLARE_EMBEDDED_IBASE (csGenmeshMeshObjectFactory);
-    SCF_DECLARE_IBASE;
-    csGenmeshMeshObjectFactory* parent;
-    virtual ~eiShaderVariableAccessor ()
+    csWeakRef<csGenmeshMeshObjectFactory> parent;
+    virtual ~ShaderVariableAccessor ()
     {
-      SCF_DESTRUCT_IBASE ();
     }
-    eiShaderVariableAccessor (csGenmeshMeshObjectFactory* parent)
+    ShaderVariableAccessor (csGenmeshMeshObjectFactory* parent) :
+      scfImplementationType (this)
     {
-      SCF_CONSTRUCT_IBASE (0);
-      eiShaderVariableAccessor::parent = parent;
+      this->parent = parent;
     }
     virtual void PreGetValue (csShaderVariable* variable)
     {
       //scfParent->PreGetShaderVariableValue (variable);
-      parent->PreGetShaderVariableValue (variable);
+      if (parent) parent->PreGetShaderVariableValue (variable);
     }
-  } *scfiShaderVariableAccessor;
-  friend class eiShaderVariableAccessor;
+  };
+  csRef<ShaderVariableAccessor> shaderVariableAccessor;
+  friend class ShaderVariableAccessor;
 
   void PreGetShaderVariableValue (csShaderVariable* variable);
 
-
-  class eiRenderBufferAccessor : public iRenderBufferAccessor
+  /// Genmesh factory render buffer accessor
+  class RenderBufferAccessor : 
+    public scfImplementation1<RenderBufferAccessor, iRenderBufferAccessor>
   {
   public:
-    CS_LEAKGUARD_DECLARE (eiRenderBufferAccessor);
-    SCF_DECLARE_IBASE;
-    csGenmeshMeshObjectFactory* parent;
-    virtual ~eiRenderBufferAccessor ()
+    CS_LEAKGUARD_DECLARE (RenderBufferAccessor);
+    csWeakRef<csGenmeshMeshObjectFactory> parent;
+    virtual ~RenderBufferAccessor ()
     {
-      SCF_DESTRUCT_IBASE ();
     }
-    eiRenderBufferAccessor (csGenmeshMeshObjectFactory* parent)
+    RenderBufferAccessor (csGenmeshMeshObjectFactory* parent) :
+      scfImplementationType (this)
     {
-      SCF_CONSTRUCT_IBASE (0);
-      eiRenderBufferAccessor::parent = parent;
+      this->parent = parent;
     }
     virtual void PreGetBuffer (csRenderBufferHolder* holder,
     	csRenderBufferName buffer)
     {
       parent->PreGetBuffer (holder, buffer);
     }
-  } *scfiRenderBufferAccessor;
-  friend class eiRenderBufferAccessor;
+  };
+  csRef<RenderBufferAccessor> renderBufferAccessor;
+  friend class RenderBufferAccessor;
 
   void PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer);
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * Genmesh type. This is the plugin you have to use to create instances
  * of csGenmeshMeshObjectFactory.
  */
-class csGenmeshMeshObjectType : public iMeshObjectType
+class csGenmeshMeshObjectType : 
+  public scfImplementation2<csGenmeshMeshObjectType, 
+                            iMeshObjectType,
+                            iComponent>
 {
 public:
   iObjectRegistry* object_reg;
   bool do_verbose;
-
-  SCF_DECLARE_IBASE;
+  MergedSVContext::Pool mergedSVContextPool;
+  csStringHash submeshNamePool;
+  csStringID base_id;
 
   /// Constructor.
   csGenmeshMeshObjectType (iBase*);
@@ -1084,15 +875,9 @@ public:
   virtual csPtr<iMeshObjectFactory> NewFactory ();
   /// Initialize.
   bool Initialize (iObjectRegistry* object_reg);
-
-  struct eiComponent : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE(csGenmeshMeshObjectType);
-    virtual bool Initialize (iObjectRegistry* object_reg)
-    {
-      return scfParent->Initialize (object_reg);
-    }
-  } scfiComponent;
 };
 
+}
+CS_PLUGIN_NAMESPACE_END(Genmesh)
+
 #endif // __CS_GENMESH_H__
