@@ -24,6 +24,7 @@
 #include "csgfx/renderbuffer.h"
 #include "csgfx/shadervarcontext.h"
 #include "cstool/rbuflock.h"
+#include "cstool/rviewclipper.h"
 #include "csutil/dirtyaccessarray.h"
 #include "csutil/sysfunc.h"
 #include "iengine/camera.h"
@@ -44,13 +45,8 @@
 
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csHazeHull)
-  SCF_IMPLEMENTS_INTERFACE (iHazeHull)
-SCF_IMPLEMENT_IBASE_END
-
-csHazeHull::csHazeHull()
+csHazeHull::csHazeHull() : scfImplementationType(this)
 {
-  SCF_CONSTRUCT_IBASE (0);
   total_poly = 0;
   total_vert = 0;
   total_edge = 0;
@@ -80,7 +76,6 @@ csHazeHull::~csHazeHull()
   total_poly = 0;
   total_vert = 0;
   total_edge = 0;
-  SCF_DESTRUCT_IBASE ();
 }
 
 
@@ -266,18 +261,9 @@ void csHazeHull::ComputeOutline(iHazeHull *hull, const csVector3& campos,
 
 //------------ csHazeHullBox -----------------------------------
 
-SCF_IMPLEMENT_IBASE_EXT (csHazeHullBox)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeHullBox)
-SCF_IMPLEMENT_IBASE_EXT_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeHullBox::HazeHullBox)
-  SCF_IMPLEMENTS_INTERFACE (iHazeHullBox)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-csHazeHullBox::csHazeHullBox(const csVector3& a, const csVector3& b)
-  : csHazeHull()
+csHazeHullBox::csHazeHullBox(const csVector3& a, const csVector3& b) :
+  scfImplementationType(this)
 {
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeHullBox);
   min = a;
   max = b;
   /// fill with data
@@ -320,19 +306,10 @@ csHazeHullBox::csHazeHullBox(const csVector3& a, const csVector3& b)
 
 csHazeHullBox::~csHazeHullBox()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeHullBox);
 }
 
 //------------ csHazeHullCone -----------------------------------
 
-SCF_IMPLEMENT_IBASE_EXT (csHazeHullCone)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeHullCone)
-SCF_IMPLEMENT_IBASE_EXT_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeHullCone::HazeHullCone)
-  SCF_IMPLEMENTS_INTERFACE (iHazeHullCone)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 /// static helper to fill
 static void ConeFillVerts(csVector3 *verts, int nr_sides,
   const csVector3& pos, float radius)
@@ -348,10 +325,9 @@ static void ConeFillVerts(csVector3 *verts, int nr_sides,
 }
 
 csHazeHullCone::csHazeHullCone(int nr_sides, const csVector3& start,
-    const csVector3& end, float srad, float erad)
-  : csHazeHull()
+  const csVector3& end, float srad, float erad) :
+  scfImplementationType(this)
 {
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeHullCone);
   csHazeHullCone::nr_sides = nr_sides;
   csHazeHullCone::start = start;
   csHazeHullCone::end = end;
@@ -417,37 +393,20 @@ csHazeHullCone::csHazeHullCone(int nr_sides, const csVector3& start,
 
 csHazeHullCone::~csHazeHullCone()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeHullCone);
 }
 
 //------------ csHazeMeshObject -------------------------------
 
-SCF_IMPLEMENT_IBASE (csHazeMeshObject)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObject)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iObjectModel)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeState)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObject::ObjectModel)
-  SCF_IMPLEMENTS_INTERFACE (iObjectModel)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObject::HazeState)
-  SCF_IMPLEMENTS_INTERFACE (iHazeState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 csStringID csHazeMeshObject::vertex_name = csInvalidStringID;
 csStringID csHazeMeshObject::texel_name = csInvalidStringID;
 csStringID csHazeMeshObject::index_name = csInvalidStringID;
 
-csHazeMeshObject::csHazeMeshObject (csHazeMeshObjectFactory* factory)
+csHazeMeshObject::csHazeMeshObject (csHazeMeshObjectFactory* factory) :
+  scfImplementationType(this)
 {
-  SCF_CONSTRUCT_IBASE (0);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeState);
   csHazeMeshObject::factory = factory;
   logparent = 0;
-  ifactory = SCF_QUERY_INTERFACE (factory, iMeshObjectFactory);
+  ifactory = scfQueryInterface<iMeshObjectFactory> (factory);
   material = factory->GetMaterialWrapper ();
   MixMode = factory->GetMixMode ();
   initialized = false;
@@ -463,7 +422,7 @@ csHazeMeshObject::csHazeMeshObject (csHazeMeshObjectFactory* factory)
   directional = factory->GetDirectional();
   csPDelArray<csHazeLayer> *factlayers = factory->GetLayers();
   size_t i;
-  for(i = 0; i < factlayers->Length(); i++)
+  for(i = 0; i < factlayers->GetSize (); i++)
   {
     csHazeLayer *p = new csHazeLayer (factlayers->Get(i)->hull,
       factlayers->Get(i)->scale);
@@ -471,8 +430,8 @@ csHazeMeshObject::csHazeMeshObject (csHazeMeshObjectFactory* factory)
   }
 
   csRef<iStringSet> strings;
-  strings = CS_QUERY_REGISTRY_TAG_INTERFACE (factory->object_reg,
-    "crystalspace.shared.stringset", iStringSet);
+  strings = csQueryRegistryTagInterface<iStringSet>
+    (factory->object_reg, "crystalspace.shared.stringset");
 
   if ((vertex_name == csInvalidStringID) ||
     (texel_name == csInvalidStringID) ||
@@ -487,26 +446,23 @@ csHazeMeshObject::csHazeMeshObject (csHazeMeshObjectFactory* factory)
 csHazeMeshObject::~csHazeMeshObject ()
 {
   if (vis_cb) vis_cb->DecRef ();
-
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeState);
-  SCF_DESTRUCT_IBASE ();
 }
 
 void csHazeMeshObject::SetupObject ()
 {
   if (!initialized)
   {
-    bbox.StartBoundingBox( origin );
+    bbox.StartBoundingBox (origin);
     csVector3 pos;
     size_t l;
     int i;
-    for(l=0; l<layers.Length(); l++)
+    for(l=0; l<layers.GetSize (); l++)
       for(i=0; i<layers[l]->hull->GetVerticeCount(); i++)
       {
 	layers[l]->hull->GetVertex(pos, i);
-        bbox.AddBoundingVertex( pos );
+        bbox.AddBoundingVertex (pos);
       }
+    radius = csQsqrt (csSquaredDist::PointPoint (bbox.Max (), bbox.Min ()));
     initialized = true;
   }
 }
@@ -612,7 +568,7 @@ void csHazeMeshObject::GenGeometryAdapt (iRenderView *rview, iGraphics3D *g3d,
     // emit geometry
     for (int i = 0; i < 3; i++)
     {
-      GetTempIndices()->Push ((uint)GetTempVertices()->Length());
+      GetTempIndices()->Push ((uint)GetTempVertices()->GetSize ());
       GetTempVertices()->Push (campts[i]);
       GetTempTexels()->Push (uvs[i]);
     }
@@ -655,7 +611,7 @@ csRenderMesh** csHazeMeshObject::GetRenderMeshes (int &n, iRenderView* rview,
 { 
   SetupObject ();
 
-  if(layers.Length() <= 0)
+  if(layers.GetSize () <= 0)
   {
     n = 0; 
     return 0; 
@@ -754,7 +710,7 @@ csRenderMesh** csHazeMeshObject::GetRenderMeshes (int &n, iRenderView* rview,
   }
 
   bool bufferCreated;
-  const size_t vertCount = GetTempVertices()->Length();
+  const size_t vertCount = GetTempVertices()->GetSize ();
   HazeRenderBuffer& vertices = renderBuffers.GetUnusedData (bufferCreated,
     rview->GetCurrentFrameNumber());
   if (bufferCreated || (vertices.count < vertCount))
@@ -775,14 +731,14 @@ csRenderMesh** csHazeMeshObject::GetRenderMeshes (int &n, iRenderView* rview,
     vertCount);
   HazeRenderBuffer& indices = indexBuffers.GetUnusedData (bufferCreated,
     rview->GetCurrentFrameNumber());
-  if (bufferCreated || (indices.count < GetTempIndices()->Length()))
+  if (bufferCreated || (indices.count < GetTempIndices()->GetSize ()))
   {
     indices.buffer = csRenderBuffer::CreateIndexRenderBuffer (
-      GetTempIndices()->Length(), CS_BUF_STREAM, 
+      GetTempIndices()->GetSize (), CS_BUF_STREAM, 
       CS_BUFCOMP_UNSIGNED_INT, 0, vertCount-1);
   }
   indices.buffer->CopyInto (GetTempIndices()->GetArray(), 
-    GetTempIndices()->Length());
+    GetTempIndices()->GetSize ());
 
   bool rmCreated;
   csRenderMesh*& rm = rmHolder.GetUnusedMesh (rmCreated, 
@@ -797,8 +753,8 @@ csRenderMesh** csHazeMeshObject::GetRenderMeshes (int &n, iRenderView* rview,
   }
 
   int clip_portal, clip_plane, clip_z_plane;
-  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
-      clip_z_plane);
+  CS::RenderViewClipper::CalculateClipSettings (rview->GetRenderContext (),
+      frustum_mask, clip_portal, clip_plane, clip_z_plane);
 
   rm->worldspace_origin = movable->GetFullPosition ();
   rm->clip_portal = clip_portal;
@@ -808,7 +764,7 @@ csRenderMesh** csHazeMeshObject::GetRenderMeshes (int &n, iRenderView* rview,
 
   rm->object2world = camera->GetTransform ();
 
-  rm->indexend = (uint)GetTempIndices()->Length();
+  rm->indexend = (uint)GetTempIndices()->GetSize ();
 
   rm->buffers->SetRenderBuffer (CS_BUFFER_INDEX, indices.buffer);
   rm->buffers->SetRenderBuffer (CS_BUFFER_POSITION, vertices.buffer);
@@ -885,10 +841,16 @@ void csHazeMeshObject::GetObjectBoundingBox (csBox3& retbbox)
   retbbox = bbox;
 }
 
+const csBox3& csHazeMeshObject::GetObjectBoundingBox ()
+{
+  SetupObject ();
+  return bbox;
+}
+
 void csHazeMeshObject::SetObjectBoundingBox (const csBox3& inbbox)
 {
   bbox = inbbox;
-  scfiObjectModel.ShapeChanged ();
+  this->ShapeChanged ();
 }
 
 void csHazeMeshObject::HardTransform (const csReversibleTransform& t)
@@ -898,32 +860,17 @@ void csHazeMeshObject::HardTransform (const csReversibleTransform& t)
 }
 
 void csHazeMeshObject::NextFrame (csTicks /*current_time*/,
-	const csVector3& /*pos*/)
+	const csVector3& /*pos*/, uint /*currentFrame*/)
 {
   // nothing
 }
 
 //----------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csHazeMeshObjectFactory)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObjectFactory)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeFactoryState)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iHazeHullCreation)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObjectFactory::HazeFactoryState)
-  SCF_IMPLEMENTS_INTERFACE (iHazeFactoryState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObjectFactory::HazeHullCreation)
-  SCF_IMPLEMENTS_INTERFACE (iHazeHullCreation)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-csHazeMeshObjectFactory::csHazeMeshObjectFactory (csHazeMeshObjectType* pParent)
+csHazeMeshObjectFactory::csHazeMeshObjectFactory (
+  csHazeMeshObjectType* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeFactoryState);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiHazeHullCreation);
   material = 0;
   MixMode = 0;
   origin.Set(0,0,0);
@@ -935,50 +882,34 @@ csHazeMeshObjectFactory::csHazeMeshObjectFactory (csHazeMeshObjectType* pParent)
 
 csHazeMeshObjectFactory::~csHazeMeshObjectFactory ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeFactoryState);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiHazeHullCreation);
-  SCF_DESTRUCT_IBASE ();
 }
 
 csPtr<iMeshObject> csHazeMeshObjectFactory::NewInstance ()
 {
   csHazeMeshObject* cm = new csHazeMeshObject (this);
-  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (cm, iMeshObject));
+  csRef<iMeshObject> im (scfQueryInterface<iMeshObject> (cm));
   cm->DecRef ();
   return csPtr<iMeshObject> (im);
 }
 
 //----------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csHazeMeshObjectType)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObjectType)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csHazeMeshObjectType::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csHazeMeshObjectType)
 
-
-csHazeMeshObjectType::csHazeMeshObjectType (iBase* pParent)
+csHazeMeshObjectType::csHazeMeshObjectType (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csHazeMeshObjectType::~csHazeMeshObjectType ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 csPtr<iMeshObjectFactory> csHazeMeshObjectType::NewFactory ()
 {
   csHazeMeshObjectFactory* cm = new csHazeMeshObjectFactory (this);
   csRef<iMeshObjectFactory> ifact (
-  	SCF_QUERY_INTERFACE (cm, iMeshObjectFactory));
+  	scfQueryInterface<iMeshObjectFactory> (cm));
   cm->DecRef ();
   return csPtr<iMeshObjectFactory> (ifact);
 }
