@@ -29,6 +29,7 @@
 #include "csgeom/trimesh.h"
 #include "csgfx/normalmaptools.h"
 #include "csgfx/renderbuffer.h"
+#include "cstool/rviewclipper.h"
 #include "csutil/csendian.h"
 #include "csutil/csmd5.h"
 #include "csutil/memfile.h"
@@ -62,77 +63,21 @@
 
 
 
-CS_LEAKGUARD_IMPLEMENT (csInstmeshMeshObject);
-CS_LEAKGUARD_IMPLEMENT (csInstmeshMeshObjectFactory);
-
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csInstmeshMeshObject)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObject)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iShadowCaster)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iShadowReceiver)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iInstancingMeshState)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iLightingInfo)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iObjectModel)
-  {
-    static scfInterfaceID iPolygonMesh_scfID = (scfInterfaceID)-1;
-    if (iPolygonMesh_scfID == (scfInterfaceID)-1)
-      iPolygonMesh_scfID = iSCF::SCF->GetInterfaceID ("iPolygonMesh");
-    if (iInterfaceID == iPolygonMesh_scfID &&
-      scfCompatibleVersion(iVersion, scfInterfaceTraits<iPolygonMesh>::GetVersion()))
-    {
-#ifdef CS_DEBUG
-      csPrintf ("Deprecated feature use: iPolygonMesh queried from Genmesh "
-    "object; use iMeshObject->GetObjectModel()->"
-    "GetPolygonMeshColldet() instead.\n");
-#endif
-      (&scfiPolygonMesh)->IncRef ();
-      return CS_STATIC_CAST(iPolygonMesh*, &scfiPolygonMesh);
-    }
-  }
-SCF_IMPLEMENT_IBASE_END
+CS_PLUGIN_NAMESPACE_BEGIN(InstMesh)
+{
 
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObject::ShadowCaster)
-  SCF_IMPLEMENTS_INTERFACE (iShadowCaster)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObject::ShadowReceiver)
-  SCF_IMPLEMENTS_INTERFACE (iShadowReceiver)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObject::ObjectModel)
-  SCF_IMPLEMENTS_INTERFACE (iObjectModel)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObject::PolyMesh)
-  SCF_IMPLEMENTS_INTERFACE (iPolygonMesh)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObject::InstancingMeshState)
-  SCF_IMPLEMENTS_INTERFACE (iInstancingMeshState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObject::LightingInfo)
-  SCF_IMPLEMENTS_INTERFACE (iLightingInfo)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csInstmeshMeshObject::eiRenderBufferAccessor)
-  SCF_IMPLEMENTS_INTERFACE (iRenderBufferAccessor)
-SCF_IMPLEMENT_IBASE_END
+CS_LEAKGUARD_IMPLEMENT (csInstmeshMeshObject);
+CS_LEAKGUARD_IMPLEMENT (csInstmeshMeshObject::RenderBufferAccessor);
+CS_LEAKGUARD_IMPLEMENT (csInstmeshMeshObjectFactory);
 
 csInstmeshMeshObject::csInstmeshMeshObject (csInstmeshMeshObjectFactory* factory) :
+        scfImplementationType (this),
 	pseudoDynInfo (29, 32),
 	affecting_lights (29, 32)
 {
-  SCF_CONSTRUCT_IBASE (0);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiPolygonMesh);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiInstancingMeshState);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiShadowCaster);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiShadowReceiver);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiLightingInfo);
-
-  scfiRenderBufferAccessor = new eiRenderBufferAccessor (this);
+  renderBufferAccessor.AttachNew (new RenderBufferAccessor (this));
   csInstmeshMeshObject::factory = factory;
   vc = factory->vc;
   logparent = 0;
@@ -140,7 +85,6 @@ csInstmeshMeshObject::csInstmeshMeshObject (csInstmeshMeshObjectFactory* factory
   cur_movablenr = -1;
   material = 0;
   MixMode = 0;
-  vis_cb = 0;
   lit_fact_colors = 0;
   num_lit_fact_colors = 0;
   static_fact_colors = 0;
@@ -160,7 +104,7 @@ csInstmeshMeshObject::csInstmeshMeshObject (csInstmeshMeshObjectFactory* factory
 
   bufferHolder.AttachNew (new csRenderBufferHolder);
 
-  g3d = CS_QUERY_REGISTRY (factory->object_reg, iGraphics3D);
+  g3d = csQueryRegistry<iGraphics3D> (factory->object_reg);
   mesh_vertices_dirty_flag = true;
   mesh_texels_dirty_flag = true;
   mesh_normals_dirty_flag = true;
@@ -173,20 +117,10 @@ csInstmeshMeshObject::csInstmeshMeshObject (csInstmeshMeshObjectFactory* factory
 
 csInstmeshMeshObject::~csInstmeshMeshObject ()
 {
-  if (vis_cb) vis_cb->DecRef ();
   delete[] lit_fact_colors;
   delete[] static_fact_colors;
 
   ClearPseudoDynLights ();
-
-  scfiRenderBufferAccessor->DecRef ();
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiPolygonMesh);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiInstancingMeshState);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiShadowCaster);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiShadowReceiver);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiLightingInfo);
-  SCF_DESTRUCT_IBASE ();
 }
 
 size_t csInstmeshMeshObject::max_instance_id = 0;
@@ -210,37 +144,37 @@ void csInstmeshMeshObject::CalculateInstanceArrays ()
 
   object_bbox_valid = false; // @@@ Think again? Do while managing instances!
 
-  size_t fact_vt_len = factory->fact_vertices.Length ();
-  size_t vt_len = fact_vt_len * instances.Length ();
+  size_t fact_vt_len = factory->fact_vertices.GetSize ();
+  size_t vt_len = fact_vt_len * instances.GetSize ();
   csVector3* fact_vertices = factory->fact_vertices.GetArray ();
   mesh_vertices.SetMinimalCapacity (vt_len);
-  mesh_vertices.SetLength (0);
+  mesh_vertices.SetSize (0);
   csVector2* fact_texels = factory->fact_texels.GetArray ();
   mesh_texels.SetMinimalCapacity (vt_len);
-  mesh_texels.SetLength (0);
+  mesh_texels.SetSize (0);
   csVector3* fact_normals = factory->fact_normals.GetArray ();
   mesh_normals.SetMinimalCapacity (vt_len);
-  mesh_normals.SetLength (0);
+  mesh_normals.SetSize (0);
   csColor4* fact_colors = factory->fact_colors.GetArray ();
   mesh_colors.SetMinimalCapacity (vt_len);
-  mesh_colors.SetLength (0);
+  mesh_colors.SetSize (0);
 
-  size_t fact_tri_len = factory->fact_triangles.Length ();
-  size_t tri_len = fact_tri_len * instances.Length ();
+  size_t fact_tri_len = factory->fact_triangles.GetSize ();
+  size_t tri_len = fact_tri_len * instances.GetSize ();
   csTriangle* fact_triangles = factory->fact_triangles.GetArray ();
   mesh_triangles.SetMinimalCapacity (tri_len);
-  mesh_triangles.SetLength (0);
+  mesh_triangles.SetSize (0);
 
   size_t i, idx;
-  for (i = 0 ; i < instances.Length () ; i++)
+  for (i = 0 ; i < instances.GetSize () ; i++)
   {
     // @@@ Do more optimal with array copy for texels and colors?
     const csReversibleTransform& tr = instances[i].transform;
     for (idx = 0 ; idx < fact_vt_len ; idx++)
     {
-      mesh_vertices.Push (tr.Other2This (fact_vertices[idx]));
+      mesh_vertices.Push (tr.This2Other (fact_vertices[idx]));
       mesh_texels.Push (fact_texels[idx]);
-      mesh_normals.Push (tr.Other2ThisRelative (fact_normals[idx]));
+      mesh_normals.Push (tr.This2OtherRelative (fact_normals[idx]));
       mesh_colors.Push (fact_colors[idx]);
     }
     int mult = (int)(i * fact_vt_len);
@@ -255,6 +189,11 @@ void csInstmeshMeshObject::CalculateInstanceArrays ()
   }
 }
 
+iMeshObjectFactory* csInstmeshMeshObject::GetFactory () const
+{
+  return (iMeshObjectFactory*)factory;
+}
+
 size_t csInstmeshMeshObject::AddInstance (const csReversibleTransform& trans)
 {
   csInstance inst;
@@ -269,7 +208,7 @@ size_t csInstmeshMeshObject::AddInstance (const csReversibleTransform& trans)
 void csInstmeshMeshObject::RemoveInstance (size_t id)
 {
   size_t i;
-  for (i = 0 ; i < instances.Length () ; i++)
+  for (i = 0 ; i < instances.GetSize () ; i++)
     if (instances[i].id == id)
     {
       instances.DeleteIndexFast (i);
@@ -283,18 +222,35 @@ void csInstmeshMeshObject::RemoveAllInstances ()
   instances.Empty ();
   initialized = false;
 }
+void csInstmeshMeshObject::UpdateInstanceGeometry (size_t instance_idx)
+{
+  if (initialized)
+  {
+    csVector3* fact_vertices = factory->fact_vertices.GetArray ();
+    size_t fact_vt_len = factory->fact_vertices.GetSize ();
+    size_t v0_id = instance_idx * fact_vt_len;
 
+    for (size_t i = 0; i <  fact_vt_len; i++)
+    {
+      mesh_vertices[v0_id + i] = 
+        instances[instance_idx].transform.This2Other (fact_vertices[i]);
+      mesh_normals[v0_id + i] = 
+        instances[instance_idx].transform.This2OtherRelative (fact_vertices[i]);;
+    }
+  }
+  mesh_vertices_dirty_flag = true;
+  mesh_normals_dirty_flag = true;
+}
 void csInstmeshMeshObject::MoveInstance (size_t id,
     const csReversibleTransform& trans)
 {
   // @@@ Not fast? Avoid loop somehow?
   size_t i;
-  for (i = 0 ; i < instances.Length () ; i++)
+  for (i = 0 ; i < instances.GetSize () ; i++)
     if (instances[i].id == id)
     {
       instances[i].transform = trans;
-      // @@@ Do in a more optimal way! Don't set everything dirty!
-      initialized = false;
+      UpdateInstanceGeometry (i);
       return;
     }
 }
@@ -304,7 +260,7 @@ const csReversibleTransform& csInstmeshMeshObject::GetInstanceTransform (
 {
   // @@@ Not fast? Avoid loop somehow?
   size_t i;
-  for (i = 0 ; i < instances.Length () ; i++)
+  for (i = 0 ; i < instances.GetSize () ; i++)
     if (instances[i].id == id)
       return instances[i].transform;
   static csReversibleTransform dummy;
@@ -325,7 +281,7 @@ void csInstmeshMeshObject::ClearPseudoDynLights ()
 void csInstmeshMeshObject::CheckLitColors ()
 {
   if (do_manual_colors) return;
-  size_t numcol = factory->GetVertexCount () * instances.Length ();
+  size_t numcol = factory->GetVertexCount () * instances.GetSize ();
   if (numcol != num_lit_fact_colors)
   {
     ClearPseudoDynLights ();
@@ -364,32 +320,34 @@ void csInstmeshMeshObject::InitializeDefault (bool clear)
 void csInstmeshMeshObject::CalculateBBoxRadius ()
 {
   object_bbox_valid = true;
-  if (mesh_vertices.Length () == 0)
+  if (mesh_vertices.GetSize () == 0)
   {
     object_bbox.Set (0, 0, 0, 0, 0, 0);
-    radius.Set (0, 0, 0);
+    radius = 0.0f;
     return;
   }
   csVector3& v0 = mesh_vertices[0];
   object_bbox.StartBoundingBox (v0);
-  csVector3 max_sq_radius (v0.x*v0.x + v0.x*v0.x,
-        v0.y*v0.y + v0.y*v0.y, v0.z*v0.z + v0.z*v0.z);
   size_t i;
-  for (i = 1 ; i < mesh_vertices.Length () ; i++)
+  for (i = 1 ; i < mesh_vertices.GetSize () ; i++)
   {
     csVector3& v = mesh_vertices[i];
     object_bbox.AddBoundingVertexSmart (v);
-    csVector3 sq_radius (v.x*v.x + v.x*v.x,
-        v.y*v.y + v.y*v.y, v.z*v.z + v.z*v.z);
-    if (sq_radius.x > max_sq_radius.x) max_sq_radius.x = sq_radius.x;
-    if (sq_radius.y > max_sq_radius.y) max_sq_radius.y = sq_radius.y;
-    if (sq_radius.z > max_sq_radius.z) max_sq_radius.z = sq_radius.z;
   }
-  radius.Set (csQsqrt (max_sq_radius.x),
-    csQsqrt (max_sq_radius.y), csQsqrt (max_sq_radius.z));
+
+  const csVector3& center = object_bbox.GetCenter ();
+  float max_sqradius = 0.0f;
+  for (i = 0 ; i < mesh_vertices.GetSize () ; i++)
+  {
+    csVector3& v = mesh_vertices[i];
+    float sqradius = csSquaredDist::PointPoint (center, v);
+    if (sqradius > max_sqradius) max_sqradius = sqradius;
+  }
+
+  radius = csQsqrt (max_sqradius);
 }
 
-const csVector3& csInstmeshMeshObject::GetRadius ()
+float csInstmeshMeshObject::GetRadius ()
 {
   SetupObject ();
   if (!object_bbox_valid) CalculateBBoxRadius ();
@@ -480,7 +438,7 @@ bool csInstmeshMeshObject::ReadFromCache (iCacheManager* cache_mgr)
 	if (mf.Read (lid, 16) != 16) goto stop;
 	iLight *l = factory->engine->FindLightID (lid);
 	if (!l) goto stop;
-	l->AddAffectedLightingInfo (&scfiLightingInfo);
+	l->AddAffectedLightingInfo ((iLightingInfo*)this);
 
 	csShadowArray* shadowArr = new csShadowArray();
 	float* intensities = new float[num_lit_fact_colors];
@@ -589,7 +547,7 @@ void csInstmeshMeshObject::DisconnectAllLights ()
   while (it.HasNext ())
   {
     iLight* l = (iLight*)it.Next ();
-    l->RemoveAffectedLightingInfo (&scfiLightingInfo);
+    l->RemoveAffectedLightingInfo ((iLightingInfo*)this);
   }
   affecting_lights.Empty ();
   lighting_dirty = true;
@@ -670,7 +628,7 @@ bool csInstmeshMeshObject::SetMaterialWrapper (iMaterialWrapper* mat)
 void csInstmeshMeshObject::SetupShaderVariableContext ()
 {
   uint bufferMask = (uint)CS_BUFFER_ALL_MASK;
-  bufferHolder->SetAccessor (scfiRenderBufferAccessor, bufferMask);
+  bufferHolder->SetAccessor (renderBufferAccessor, bufferMask);
 }
   
 void csInstmeshMeshObject::SetupObject ()
@@ -683,12 +641,13 @@ void csInstmeshMeshObject::SetupObject ()
     lit_fact_colors = 0;
     if (!do_manual_colors)
     {
-      num_lit_fact_colors = factory->fact_vertices.Length ()
-	* instances.Length ();
+      num_lit_fact_colors = factory->fact_vertices.GetSize ()
+	* instances.GetSize ();
       lit_fact_colors = new csColor4 [num_lit_fact_colors];
       size_t i;
       for (i = 0 ; i <  num_lit_fact_colors; i++)
         lit_fact_colors[i].Set (0, 0, 0);
+      lighting_dirty = true;
       static_fact_colors = new csColor4 [num_lit_fact_colors];
       for (i = 0 ; i <  num_lit_fact_colors; i++)
         //static_fact_colors[i] = base_color;	// Initialize to base color.
@@ -726,7 +685,7 @@ void csInstmeshMeshObject::CastShadows (iMovable* movable, iFrustumView* fview)
   if (!do_lighting) return;
 
   iBase* b = (iBase *)fview->GetUserdata ();
-  csRef<iLightingProcessInfo> lpi = SCF_QUERY_INTERFACE(b,iLightingProcessInfo);
+  csRef<iLightingProcessInfo> lpi = scfQueryInterface<iLightingProcessInfo> (b);
   CS_ASSERT (lpi != 0);
 
   iLight* li = lpi->GetLight ();
@@ -736,7 +695,7 @@ void csInstmeshMeshObject::CastShadows (iMovable* movable, iFrustumView* fview)
   {
     if (!do_shadow_rec || li->GetDynamicType () == CS_LIGHT_DYNAMICTYPE_PSEUDO)
     {
-      li->AddAffectedLightingInfo (&scfiLightingInfo);
+      li->AddAffectedLightingInfo ((iLightingInfo*)this);
       if (li->GetDynamicType () != CS_LIGHT_DYNAMICTYPE_PSEUDO)
         affecting_lights.Add (li);
     }
@@ -745,7 +704,7 @@ void csInstmeshMeshObject::CastShadows (iMovable* movable, iFrustumView* fview)
   {
     if (!affecting_lights.In (li))
     {
-      li->AddAffectedLightingInfo (&scfiLightingInfo);
+      li->AddAffectedLightingInfo ((iLightingInfo*)this);
       affecting_lights.Add (li);
     }
     if (do_shadow_rec) return;
@@ -779,7 +738,7 @@ void csInstmeshMeshObject::CastShadows (iMovable* movable, iFrustumView* fview)
   csColor light_color = li->GetColor () * (256. / CS_NORMAL_LIGHT_LEVEL);
 
   csColor col;
-  int i;
+  size_t i;
   for (i = 0 ; i < factory->GetVertexCount () ; i++)
   {
     const csVector3& normal = normals[i];
@@ -869,7 +828,7 @@ void csInstmeshMeshObject::UpdateLightingOne (
 
   csColor col;
   size_t i;
-  size_t numcol = factory->GetVertexCount () * instances.Length ();
+  size_t numcol = factory->GetVertexCount () * instances.GetSize ();
   if (obj_sq_dist < SMALL_EPSILON)
   {
     for (i = 0 ; i < numcol ; i++)
@@ -920,8 +879,8 @@ Rules for color calculation:
 */
 
 void csInstmeshMeshObject::UpdateLighting (
-    const csArray<iLightSectorInfluence*>& lights,
-    iMovable* movable)
+  const csArray<iLightSectorInfluence*>& lights,
+  iMovable* movable)
 {
   size_t i;
   if (cur_movablenr != movable->GetUpdateNumber ())
@@ -932,7 +891,7 @@ void csInstmeshMeshObject::UpdateLighting (
 
   if (factory->DoFullBright ())
   {
-    size_t numcol = factory->GetVertexCount () * instances.Length ();
+    size_t numcol = factory->GetVertexCount () * instances.GetSize ();
     lighting_dirty = false;
     for (i = 0 ; i < numcol ; i++)
     {
@@ -957,7 +916,7 @@ void csInstmeshMeshObject::UpdateLighting (
     lighting_dirty = false;
     mesh_colors_dirty_flag = true;
 
-    csColor col;
+    csColor4 col;
     if (factory->engine)
     {
       factory->engine->GetAmbientLight (col);
@@ -970,7 +929,7 @@ void csInstmeshMeshObject::UpdateLighting (
     {
       col = base_color;
     }
-    size_t numcol = factory->GetVertexCount () * instances.Length ();
+    size_t numcol = factory->GetVertexCount () * instances.GetSize ();
     for (i = 0 ; i < numcol ; i++)
     {
       lit_fact_colors[i] = col + static_fact_colors[i] + colors_ptr[i];
@@ -1009,11 +968,11 @@ void csInstmeshMeshObject::UpdateLighting (
       csReversibleTransform trans = movable->GetFullTransform ();
       // the object center in world coordinates. "0" because the object
       // center in object space is obviously at (0,0,0).
-      size_t num_lights = lights.Length ();
+      size_t num_lights = lights.GetSize ();
       for (size_t l = 0 ; l < num_lights ; l++)
       {
         iLight* li = lights[l]->GetLight ();
-        li->AddAffectedLightingInfo (&scfiLightingInfo);
+        li->AddAffectedLightingInfo ((iLightingInfo*)this);
         affecting_lights.Add (li);
         UpdateLightingOne (trans, li);
       }
@@ -1030,7 +989,7 @@ void csInstmeshMeshObject::UpdateLighting (
     lighting_dirty = false;
     mesh_colors_dirty_flag = true;
 
-    size_t numcol = factory->GetVertexCount () * instances.Length ();
+    size_t numcol = factory->GetVertexCount () * instances.GetSize ();
     for (i = 0 ; i < numcol ; i++)
     {
       lit_fact_colors[i] = base_color + colors_ptr[i];
@@ -1046,70 +1005,76 @@ csRenderMesh** csInstmeshMeshObject::GetRenderMeshes (
   CheckLitColors ();
   SetupObject ();
 
+
   n = 0;
 
-  iCamera* camera = rview->GetCamera ();
-
-  int clip_portal, clip_plane, clip_z_plane;
-  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
-      clip_z_plane);
-
-  lighting_movable = movable;
-
-  if (!do_manual_colors && !do_shadow_rec && factory->light_mgr)
+  if (mesh_triangles.GetSize () > 0)
   {
-    // Remember relevant lights for later.
-    relevant_lights = factory->light_mgr->GetRelevantLights (
-    	logparent, -1, false);
-  }
+    iCamera* camera = rview->GetCamera ();
 
-  const csReversibleTransform o2wt = movable->GetFullTransform ();
-  const csVector3& wo = o2wt.GetOrigin ();
+    int clip_portal, clip_plane, clip_z_plane;
+    CS::RenderViewClipper::CalculateClipSettings (rview->GetRenderContext (),
+	frustum_mask, clip_portal, clip_plane, clip_z_plane);
 
-  // Array still needed?@@@
-  {
-    renderMeshes.SetLength (1);
+    lighting_movable = movable;
 
-    iMaterialWrapper* mater = material;
-    if (!mater) mater = factory->GetMaterialWrapper ();
-    if (!mater)
+    if (!do_manual_colors && !do_shadow_rec && factory->light_mgr)
     {
-      csPrintf ("INTERNAL ERROR: mesh used without material!\n");
-      return 0;
+      // Remember relevant lights for later.
+      relevant_lights = factory->light_mgr->GetRelevantLights (
+        logparent, -1, false);
     }
 
-    if (mater->IsVisitRequired ()) mater->Visit ();
+    const csReversibleTransform o2wt = movable->GetFullTransform ();
+    const csVector3& wo = o2wt.GetOrigin ();
 
-    bool rmCreated;
-    csRenderMesh*& meshPtr = rmHolder.GetUnusedMesh (rmCreated,
-      rview->GetCurrentFrameNumber ());
+    // Array still needed?@@@
+    {
+      renderMeshes.SetSize (1);
 
-    meshPtr->mixmode = MixMode;
-    meshPtr->clip_portal = clip_portal;
-    meshPtr->clip_plane = clip_plane;
-    meshPtr->clip_z_plane = clip_z_plane;
-    meshPtr->do_mirror = camera->IsMirrored ();
-    meshPtr->meshtype = CS_MESHTYPE_TRIANGLES;
-    meshPtr->indexstart = 0;
-    meshPtr->indexend = (unsigned int)(mesh_triangles.Length () * 3);
-    meshPtr->material = mater;
-    CS_ASSERT (mater != 0);
-    meshPtr->worldspace_origin = wo;
-    meshPtr->buffers = bufferHolder;
-    meshPtr->geometryInstance = (void*)factory;
-    meshPtr->object2world = o2wt;
+      iMaterialWrapper* mater = material;
+      if (!mater) mater = factory->GetMaterialWrapper ();
+      if (!mater)
+      {
+        csPrintf ("INTERNAL ERROR: mesh used without material!\n");
+        return 0;
+      }
 
-    renderMeshes[0] = meshPtr;
+      if (mater->IsVisitRequired ()) mater->Visit ();
+
+      bool rmCreated;
+      csRenderMesh*& meshPtr = rmHolder.GetUnusedMesh (rmCreated,
+        rview->GetCurrentFrameNumber ());
+
+      meshPtr->mixmode = CS_MIXMODE_ALPHATEST_ENABLE;
+      meshPtr->clip_portal = clip_portal;
+      meshPtr->clip_plane = clip_plane;
+      meshPtr->clip_z_plane = clip_z_plane;
+      meshPtr->do_mirror = camera->IsMirrored ();
+      meshPtr->meshtype = CS_MESHTYPE_TRIANGLES;
+      meshPtr->indexstart = 0;
+      meshPtr->indexend = (unsigned int)(mesh_triangles.GetSize () * 3);
+      meshPtr->material = mater;
+      CS_ASSERT (mater != 0);
+      meshPtr->worldspace_origin = wo;
+      meshPtr->buffers = bufferHolder;
+      meshPtr->geometryInstance = (void*)factory;
+      meshPtr->object2world = o2wt;
+
+      renderMeshes[0] = meshPtr;
+    }
+
+    n = (int)renderMeshes.GetSize ();
+    return renderMeshes.GetArray ();
   }
-
-  n = (int)renderMeshes.Length ();
-  return renderMeshes.GetArray ();
+  else 
+    return 0;
 }
 
-void csInstmeshMeshObject::GetRadius (csVector3& rad, csVector3& cent)
+void csInstmeshMeshObject::GetRadius (float& rad, csVector3& cent)
 {
   rad = GetRadius ();
-  cent.Set (0.0f);
+  cent = object_bbox.GetCenter ();
 }
 
 bool csInstmeshMeshObject::HitBeamOutline (const csVector3& start,
@@ -1177,10 +1142,10 @@ bool csInstmeshMeshObject::HitBeamObject (const csVector3& start,
   if (material)
   {
     // @@@ Submeshes not yet supported!
-    //const csPDelArray<csInstmeshSubMesh>& sm = subMeshes.Length () == 0
+    //const csPDelArray<csInstmeshSubMesh>& sm = subMeshes.GetSize () == 0
     	//? factory->GetSubMeshes ()
 	//: subMeshes;
-    //if (sm.Length () == 0)
+    //if (sm.GetSize () == 0)
     //{
       *material = csInstmeshMeshObject::material;
       if (!*material) *material = factory->GetMaterialWrapper ();
@@ -1192,7 +1157,7 @@ bool csInstmeshMeshObject::HitBeamObject (const csVector3& start,
 
 int csInstmeshMeshObject::PolyMesh::GetVertexCount ()
 {
-  return (int)scfParent->factory->GetVertexCount ();
+  return (int)parent->factory->GetVertexCount ();
 }
 
 csVector3* csInstmeshMeshObject::PolyMesh::GetVertices ()
@@ -1230,6 +1195,32 @@ csTriangle* csInstmeshMeshObject::PolyMesh::GetTriangles ()
   //return scfParent->factory->GetTriangles ();
 }
 
+size_t csInstmeshMeshObject::TriMesh::GetVertexCount ()
+{
+  return parent->factory->GetVertexCount ();
+}
+
+csVector3* csInstmeshMeshObject::TriMesh::GetVertices ()
+{
+  //@@@FIXME: data must come from mesh itself. Not factory
+  return 0;
+  //return scfParent->factory->GetVertices ();
+}
+
+size_t csInstmeshMeshObject::TriMesh::GetTriangleCount ()
+{
+  //@@@FIXME: data from mesh instead of factory
+  return 0;
+  //return scfParent->factory->GetTriangleCount ();
+}
+
+csTriangle* csInstmeshMeshObject::TriMesh::GetTriangles ()
+{
+  //@@@FIXME: data from mesh instead of factory
+  return 0;
+  //return scfParent->factory->GetTriangles ();
+}
+
 void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder, 
 					csRenderBufferName buffer)
 {
@@ -1254,23 +1245,24 @@ void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder,
           color_buffer = csRenderBuffer::CreateRenderBuffer (
             num_lit_fact_colors, 
             do_lighting ? CS_BUF_DYNAMIC : CS_BUF_STATIC,
-            CS_BUFCOMP_FLOAT, 4, false);
+            CS_BUFCOMP_FLOAT, 4);
         }
         mesh_colors_dirty_flag = false;
         const csColor4* fact_colors = 0;
         fact_colors = lit_fact_colors;
+        
         color_buffer->CopyInto (fact_colors, num_lit_fact_colors);
       }
       else
       {
-	size_t numcol = factory->fact_vertices.Length () * instances.Length ();
+	size_t numcol = factory->fact_vertices.GetSize () * instances.GetSize ();
         if (!color_buffer || 
           (color_buffer->GetSize() != (sizeof (csColor4) * numcol)))
         {
           // Recreate the render buffer only if the new data cannot fit inside
           //  the existing buffer.
           color_buffer = csRenderBuffer::CreateRenderBuffer (
-            numcol, CS_BUF_STATIC, CS_BUFCOMP_FLOAT, 4, false);
+            numcol, CS_BUF_STATIC, CS_BUFCOMP_FLOAT, 4);
         }
         mesh_colors_dirty_flag = false;
         const csColor4* fact_colors = 0;
@@ -1287,11 +1279,11 @@ void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder,
     {
       if (!vertex_buffer)
         vertex_buffer = csRenderBuffer::CreateRenderBuffer (
-          mesh_vertices.Length (), CS_BUF_STATIC,
-          CS_BUFCOMP_FLOAT, 3, false);
+          mesh_vertices.GetSize (), CS_BUF_STATIC,
+          CS_BUFCOMP_FLOAT, 3);
       mesh_vertices_dirty_flag = false;
       vertex_buffer->CopyInto ((void*)mesh_vertices.GetArray (),
-	  mesh_vertices.Length ());
+	  mesh_vertices.GetSize ());
     }
     holder->SetRenderBuffer (buffer, vertex_buffer);
     return;
@@ -1302,11 +1294,11 @@ void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder,
     {
       if (!texel_buffer)
         texel_buffer = csRenderBuffer::CreateRenderBuffer (
-          mesh_texels.Length (), CS_BUF_STATIC,
-          CS_BUFCOMP_FLOAT, 2, false);
+          mesh_texels.GetSize (), CS_BUF_STATIC,
+          CS_BUFCOMP_FLOAT, 2);
       mesh_texels_dirty_flag = false;
       texel_buffer->CopyInto ((void*)mesh_texels.GetArray (),
-	  mesh_texels.Length ());
+	  mesh_texels.GetSize ());
     }
     holder->SetRenderBuffer (buffer, texel_buffer);
     return;
@@ -1317,11 +1309,11 @@ void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder,
     {
       if (!normal_buffer)
         normal_buffer = csRenderBuffer::CreateRenderBuffer (
-          mesh_normals.Length (), CS_BUF_STATIC,
-          CS_BUFCOMP_FLOAT, 3, false);
+          mesh_normals.GetSize (), CS_BUF_STATIC,
+          CS_BUFCOMP_FLOAT, 3);
       mesh_normals_dirty_flag = false;
       normal_buffer->CopyInto ((void*)mesh_normals.GetArray (),
-	  mesh_normals.Length ());
+	  mesh_normals.GetSize ());
     }
     holder->SetRenderBuffer (buffer, normal_buffer);
     return;
@@ -1332,23 +1324,23 @@ void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder,
     {
       if (!tangent_buffer)
         tangent_buffer = csRenderBuffer::CreateRenderBuffer (
-          mesh_vertices.Length (), CS_BUF_STATIC,
+          mesh_vertices.GetSize (), CS_BUF_STATIC,
           CS_BUFCOMP_FLOAT, 3);
       if (!binormal_buffer)
         binormal_buffer = csRenderBuffer::CreateRenderBuffer (
-          mesh_vertices.Length (), CS_BUF_STATIC,
+          mesh_vertices.GetSize (), CS_BUF_STATIC,
           CS_BUFCOMP_FLOAT, 3);
       mesh_tangents_dirty_flag = false;
 
-      csVector3* tangentData = new csVector3[mesh_vertices.Length () * 2];
-      csVector3* bitangentData = tangentData + mesh_vertices.Length ();
-      csNormalMappingTools::CalculateTangents (mesh_triangles.Length (), 
-        mesh_triangles.GetArray (), mesh_vertices.Length (),
+      csVector3* tangentData = new csVector3[mesh_vertices.GetSize () * 2];
+      csVector3* bitangentData = tangentData + mesh_vertices.GetSize ();
+      csNormalMappingTools::CalculateTangents (mesh_triangles.GetSize (), 
+        mesh_triangles.GetArray (), mesh_vertices.GetSize (),
 	mesh_vertices.GetArray (), mesh_normals.GetArray (), 
         mesh_texels.GetArray (), tangentData, bitangentData);
 
-      tangent_buffer->CopyInto (tangentData, mesh_vertices.Length ());
-      binormal_buffer->CopyInto (bitangentData, mesh_vertices.Length ());
+      tangent_buffer->CopyInto (tangentData, mesh_vertices.GetSize ());
+      binormal_buffer->CopyInto (bitangentData, mesh_vertices.GetSize ());
 
       delete[] tangentData;
     }
@@ -1362,11 +1354,11 @@ void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder,
     {
       if (!index_buffer)
         index_buffer = csRenderBuffer::CreateIndexRenderBuffer (
-          mesh_triangles.Length ()*3, CS_BUF_STATIC,
-          CS_BUFCOMP_UNSIGNED_INT, 0, mesh_vertices.Length () - 1);
+          mesh_triangles.GetSize ()*3, CS_BUF_STATIC,
+          CS_BUFCOMP_UNSIGNED_INT, 0, mesh_vertices.GetSize () - 1);
       mesh_triangle_dirty_flag = false;
       index_buffer->CopyInto ((void*)mesh_triangles.GetArray (),
-	  mesh_triangles.Length ()*3);
+	  mesh_triangles.GetSize ()*3);
     }
     holder->SetRenderBuffer (buffer, index_buffer);
     return;
@@ -1375,32 +1367,21 @@ void csInstmeshMeshObject::PreGetBuffer (csRenderBufferHolder* holder,
 
 //----------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csInstmeshMeshObjectFactory)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObjectFactory)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iInstancingFactoryState)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObjectFactory::InstancingFactoryState)
-  SCF_IMPLEMENTS_INTERFACE (iInstancingFactoryState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 csInstmeshMeshObjectFactory::csInstmeshMeshObjectFactory (
-  iMeshObjectType *pParent, iObjectRegistry* object_reg)
+  iMeshObjectType *pParent, iObjectRegistry* object_reg) : 
+  scfImplementationType (this, (iBase*)pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiInstancingFactoryState);
-
   csInstmeshMeshObjectFactory::object_reg = object_reg;
 
   logparent = 0;
   instmesh_type = pParent;
 
   material = 0;
-  light_mgr = CS_QUERY_REGISTRY (object_reg, iLightManager);
+  light_mgr = csQueryRegistry<iLightManager> (object_reg);
 
-  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
-  strings = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg,
-    "crystalspace.shared.stringset", iStringSet);
+  g3d = csQueryRegistry<iGraphics3D> (object_reg);
+  strings = csQueryRegistryTagInterface<iStringSet> (object_reg,
+    "crystalspace.shared.stringset");
 
   autonormals = false;
   autonormals_compress = true;
@@ -1412,20 +1393,18 @@ csInstmeshMeshObjectFactory::csInstmeshMeshObjectFactory (
   default_shadowcasting = true;
   default_shadowreceiving = false;
 
-  csRef<iEngine> eng = CS_QUERY_REGISTRY (object_reg, iEngine);
+  csRef<iEngine> eng = csQueryRegistry<iEngine> (object_reg);
   engine = eng; // We don't want a circular reference!
 
-  vc = CS_QUERY_REGISTRY (object_reg, iVirtualClock);
+  vc = csQueryRegistry<iVirtualClock> (object_reg);
 
-  csRef<iCommandLineParser> cmdline = CS_QUERY_REGISTRY (
-  	object_reg, iCommandLineParser);
+  csRef<iCommandLineParser> cmdline = csQueryRegistry<iCommandLineParser> (
+  	object_reg);
   do_fullbright = (cmdline->GetOption ("fullbright") != 0);
 }
 
 csInstmeshMeshObjectFactory::~csInstmeshMeshObjectFactory ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiInstancingFactoryState);
-  SCF_DESTRUCT_IBASE ();
 }
 
 void csInstmeshMeshObjectFactory::CalculateBoundingVolumes ()
@@ -1434,7 +1413,7 @@ void csInstmeshMeshObjectFactory::CalculateBoundingVolumes ()
   factory_bbox.StartBoundingBox (fact_vertices[0]);
   factory_radius = csQsqrt (csSquaredDist::PointPoint (
 	fact_vertices[0], csVector3 (0)));
-  for (i = 0 ; i < fact_vertices.Length () ; i++)
+  for (i = 0 ; i < fact_vertices.GetSize () ; i++)
   {
     const csVector3& v = fact_vertices[i];
     factory_bbox.AddBoundingVertexSmart (v);
@@ -1448,7 +1427,7 @@ void csInstmeshMeshObjectFactory::AddVertex (const csVector3& v,
       const csVector2& uv, const csVector3& normal,
       const csColor4& color)
 {
-  if (fact_vertices.Length () == 0)
+  if (fact_vertices.GetSize () == 0)
   {
     factory_bbox.StartBoundingBox (v);
     factory_radius = csQsqrt (csSquaredDist::PointPoint (v,
@@ -1469,17 +1448,17 @@ void csInstmeshMeshObjectFactory::AddVertex (const csVector3& v,
 
 void csInstmeshMeshObjectFactory::Compress ()
 {
-  size_t old_num = fact_vertices.Length ();
+  //size_t old_num = fact_vertices.GetSize ();
   csCompressVertexInfo* vt = csVertexCompressor::Compress (
     	fact_vertices, fact_texels, fact_normals, fact_colors);
   if (vt)
   {
-    printf ("From %d to %d\n", int (old_num), int (fact_vertices.Length ()));
-    fflush (stdout);
+    //printf ("From %d to %d\n", int (old_num), int (fact_vertices.GetSize ()));
+    //fflush (stdout);
 
     // Now we can remap the vertices in all triangles.
     size_t i;
-    for (i = 0 ; i < fact_triangles.Length () ; i++)
+    for (i = 0 ; i < fact_triangles.GetSize () ; i++)
     {
       fact_triangles[i].a = (int)vt[fact_triangles[i].a].new_idx;
       fact_triangles[i].b = (int)vt[fact_triangles[i].b].new_idx;
@@ -1503,24 +1482,31 @@ void csInstmeshMeshObjectFactory::GenerateSphere (const csEllipsoid& sphere,
 {
   csPrimitives::GenerateSphere (sphere, num, fact_vertices, fact_texels,
       fact_normals, fact_triangles, cyl_mapping, toponly, reversed);
-  fact_colors.SetLength (fact_vertices.Length ());
-  memset (fact_colors.GetArray (), 0, sizeof (csColor4)*fact_vertices.Length ());
+  fact_colors.SetSize (fact_vertices.GetSize ());
+  memset (fact_colors.GetArray (), 0, sizeof (csColor4)*fact_vertices.GetSize ());
   CalculateBoundingVolumes ();
 }
-
+void csInstmeshMeshObjectFactory::GenerateQuad (const csVector3& v1, const csVector3& v2, 
+                  const csVector3& v3, const csVector3& v4)
+{
+  csPrimitives::GenerateQuad (v1, v2, v3, v4, fact_vertices, fact_texels,
+      fact_normals, fact_triangles);
+  fact_colors.SetSize (fact_vertices.GetSize ());
+  memset (fact_colors.GetArray (), 0, sizeof (csColor4)*fact_vertices.GetSize ());
+}
 void csInstmeshMeshObjectFactory::GenerateBox (const csBox3& box)
 {
   csPrimitives::GenerateBox (box, fact_vertices, fact_texels,
       fact_normals, fact_triangles);
-  fact_colors.SetLength (fact_vertices.Length ());
-  memset (fact_colors.GetArray (), 0, sizeof (csColor4)*fact_vertices.Length ());
+  fact_colors.SetSize (fact_vertices.GetSize ());
+  memset (fact_colors.GetArray (), 0, sizeof (csColor4)*fact_vertices.GetSize ());
 }
 
 void csInstmeshMeshObjectFactory::HardTransform (
     const csReversibleTransform& t)
 {
   size_t i;
-  for (i = 0 ; i < fact_vertices.Length () ; i++)
+  for (i = 0 ; i < fact_vertices.GetSize () ; i++)
   {
     fact_vertices[i] = t.This2Other (fact_vertices[i]);
     fact_normals[i] = t.This2OtherRelative (fact_normals[i]);
@@ -1537,46 +1523,33 @@ csPtr<iMeshObject> csInstmeshMeshObjectFactory::NewInstance ()
   cm->SetShadowCasting (default_shadowcasting);
   cm->SetShadowReceiving (default_shadowreceiving);
 
-  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (cm, iMeshObject));
+  csRef<iMeshObject> im (scfQueryInterface<iMeshObject> (cm));
   cm->DecRef ();
   return csPtr<iMeshObject> (im);
 }
 
 //----------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csInstmeshMeshObjectType)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObjectType)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstmeshMeshObjectType::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csInstmeshMeshObjectType)
 
 
-csInstmeshMeshObjectType::csInstmeshMeshObjectType (iBase* pParent)
+csInstmeshMeshObjectType::csInstmeshMeshObjectType (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
-
   do_verbose = false;
 }
 
 csInstmeshMeshObjectType::~csInstmeshMeshObjectType ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 csPtr<iMeshObjectFactory> csInstmeshMeshObjectType::NewFactory ()
 {
-  csInstmeshMeshObjectFactory* cm = new csInstmeshMeshObjectFactory (this,
-    object_reg);
+  csRef<csInstmeshMeshObjectFactory> cm;
+  cm.AttachNew (new csInstmeshMeshObjectFactory (this,
+    object_reg));
   csRef<iMeshObjectFactory> ifact (
-    SCF_QUERY_INTERFACE (cm, iMeshObjectFactory));
-  cm->DecRef ();
+    scfQueryInterface<iMeshObjectFactory> (cm));
   return csPtr<iMeshObjectFactory> (ifact);
 }
 
@@ -1585,9 +1558,12 @@ bool csInstmeshMeshObjectType::Initialize (iObjectRegistry* object_reg)
   csInstmeshMeshObjectType::object_reg = object_reg;
 
   csRef<iVerbosityManager> verbosemgr (
-    CS_QUERY_REGISTRY (object_reg, iVerbosityManager));
+    csQueryRegistry<iVerbosityManager> (object_reg));
   if (verbosemgr) 
-    do_verbose = verbosemgr->Enabled ("genmesh");
+    do_verbose = verbosemgr->Enabled ("instmesh");
 
   return true;
 }
+
+}
+CS_PLUGIN_NAMESPACE_END(InstMesh)
