@@ -1,25 +1,25 @@
 /*
-    Copyright (C) 2005 by Jorrit Tyberghein
+  Copyright (C) 2005 by Jorrit Tyberghein
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Library General Public
+  License as published by the Free Software Foundation; either
+  version 2 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Library General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  You should have received a copy of the GNU Library General Public
+  License along with this library; if not, write to the Free
+  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
 #ifndef __CS_INSTMESH_H__
 #define __CS_INSTMESH_H__
 
-#include "csgeom/objmodel.h"
+#include "cstool/objmodel.h"
 #include "csgeom/transfrm.h"
 #include "csgeom/vector3.h"
 #include "csgeom/vector4.h"
@@ -39,6 +39,7 @@
 #include "iengine/lightmgr.h"
 #include "iengine/shadcast.h"
 #include "igeom/polymesh.h"
+#include "igeom/trimesh.h"
 #include "imesh/instmesh.h"
 #include "imesh/lighting.h"
 #include "imesh/object.h"
@@ -52,9 +53,6 @@
 class csBSPTree;
 class csColor;
 class csColor4;
-class csInstmeshMeshObject;
-class csInstmeshMeshObjectFactory;
-class csInstmeshMeshObjectType;
 class csPolygonMesh;
 struct iCacheManager;
 struct iEngine;
@@ -63,6 +61,13 @@ struct iMovable;
 struct iObjectRegistry;
 struct iShadowBlockList;
 
+CS_PLUGIN_NAMESPACE_BEGIN(InstMesh)
+{
+
+class csInstmeshMeshObject;
+class csInstmeshMeshObjectFactory;
+class csInstmeshMeshObjectType;
+
 /**
  * An array giving shadow information for a pseudo-dynamic light.
  */
@@ -81,6 +86,8 @@ public:
   }
 };
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 struct csInstance
 {
   csReversibleTransform transform;
@@ -90,7 +97,14 @@ struct csInstance
 /**
  * Instmesh version of mesh object.
  */
-class csInstmeshMeshObject : public iMeshObject
+class csInstmeshMeshObject : 
+  public scfImplementationExt5<csInstmeshMeshObject, 
+                               csObjectModel,
+                               iMeshObject,
+                               iLightingInfo,
+                               iShadowCaster,
+                               iShadowReceiver,
+                               iInstancingMeshState>
 {
 private:
   csRenderMeshHolder rmHolder;
@@ -122,6 +136,7 @@ private:
   csArray<csInstance> instances;
   static size_t max_instance_id;
   void CalculateInstanceArrays ();
+  void UpdateInstanceGeometry (size_t id);
 
   iMovable* lighting_movable;
 
@@ -132,7 +147,7 @@ private:
   csRef<iMaterialWrapper> material;
   bool material_needs_visit;
   uint MixMode;
-  iMeshObjectDrawCallback* vis_cb;
+  csRef<iMeshObjectDrawCallback> vis_cb;
   bool do_lighting;
   bool do_manual_colors;
   csColor4 base_color;
@@ -148,7 +163,7 @@ private:
   size_t num_lit_fact_colors;
   csColor4* static_fact_colors;
 
-  csVector3 radius;
+  float radius;
   csBox3 object_bbox;
   bool object_bbox_valid;
 
@@ -227,9 +242,11 @@ public:
   const csColor& GetColor () const { return base_color; }
   void SetManualColors (bool m) { do_manual_colors = m; }
   bool IsManualColors () const { return do_manual_colors; }
-  void GetRadius (csVector3& rad, csVector3& cent);
+  void GetRadius (float& rad, csVector3& cent);
   void SetShadowCasting (bool m) { do_shadows = m; }
   void SetShadowReceiving (bool m) { do_shadow_rec = m; }
+  bool IsShadowCasting () const { return do_shadows; }
+  bool IsShadowReceiving () const { return do_shadow_rec; }
 
   iVirtualClock* vc;
 
@@ -258,20 +275,13 @@ public:
   void DisconnectAllLights ();
 
   //----------------------- iMeshObject implementation ----------------------
-  SCF_DECLARE_IBASE;
-
-  virtual iMeshObjectFactory* GetFactory () const
-  {
-    return (iMeshObjectFactory*)factory;
-  }
+  virtual iMeshObjectFactory* GetFactory () const;
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> Clone () { return 0; }
   virtual csRenderMesh** GetRenderMeshes (int &n, iRenderView* rview, 
     iMovable* movable, uint32 frustum_mask);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* cb)
   {
-    if (cb) cb->IncRef ();
-    if (vis_cb) vis_cb->DecRef ();
     vis_cb = cb;
   }
   virtual iMeshObjectDrawCallback* GetVisibleCallback () const
@@ -279,7 +289,7 @@ public:
     return vis_cb;
   }
   virtual void NextFrame (csTicks /*current_time*/,
-  	const csVector3& /*pos*/) { }
+    const csVector3& /*pos*/, uint /*framenr*/) { }
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
   virtual bool HitBeamOutline (const csVector3& start, const csVector3& end,
@@ -309,164 +319,34 @@ public:
    * does nothing.
    */
   virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) { }
+  virtual void BuildDecal(const csVector3* pos, float decalRadius,
+          iDecalBuilder* decalBuilder)
+  {
+  }
 
   /// Calculate bounding box and radius.
   void CalculateBBoxRadius ();
-  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  const csVector3& GetRadius ();
+  float GetRadius ();
 
-  //------------------------- iObjectModel implementation ----------------
-  class ObjectModel : public csObjectModel
+  /**\name iObjectModel implementation
+   * @{ */
+  virtual void GetObjectBoundingBox (csBox3& bbox)
   {
-    SCF_DECLARE_EMBEDDED_IBASE (csInstmeshMeshObject);
-    virtual void GetObjectBoundingBox (csBox3& bbox)
-    {
-      bbox = scfParent->GetObjectBoundingBox ();
-    }
-    virtual void SetObjectBoundingBox (const csBox3& bbox)
-    {
-      scfParent->SetObjectBoundingBox (bbox);
-    }
-    virtual void GetRadius (csVector3& rad, csVector3& cent)
-    {
-      rad = scfParent->GetRadius ();
-      cent.Set (0.0f);
-    }
-  } scfiObjectModel;
-  friend class ObjectModel;
+    bbox = GetObjectBoundingBox ();
+  }
+  virtual const csBox3& GetObjectBoundingBox ();
+  /** @} */
 
-  virtual iObjectModel* GetObjectModel () { return &scfiObjectModel; }
-
-  //------------------------- iLightingInfo interface -------------------------
-  struct LightingInfo : public iLightingInfo
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csInstmeshMeshObject);
-    virtual void InitializeDefault (bool clear)
-    {
-      scfParent->InitializeDefault (clear);
-    }
-    virtual bool ReadFromCache (iCacheManager* cache_mgr)
-    {
-      return scfParent->ReadFromCache (cache_mgr);
-    }
-    virtual bool WriteToCache (iCacheManager* cache_mgr)
-    {
-      return scfParent->WriteToCache (cache_mgr);
-    }
-    virtual void PrepareLighting ()
-    {
-      scfParent->PrepareLighting ();
-    }
-    virtual void LightChanged (iLight* light)
-    {
-      scfParent->LightChanged (light);
-    }
-    virtual void LightDisconnect (iLight* light)
-    {
-      scfParent->LightDisconnect (light);
-    }
-    virtual void DisconnectAllLights ()
-    {
-      scfParent->DisconnectAllLights ();
-    }
-  } scfiLightingInfo;
-  friend struct LightingInfo;
-
-  //-------------------- iShadowCaster interface implementation ----------
-  struct ShadowCaster : public iShadowCaster
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csInstmeshMeshObject);
-    virtual void AppendShadows (iMovable* movable, iShadowBlockList* shadows,
-    	const csVector3& origin)
-    {
-      scfParent->AppendShadows (movable, shadows, origin);
-    }
-  } scfiShadowCaster;
-  friend struct ShadowCaster;
-
-  //-------------------- iShadowReceiver interface implementation ----------
-  struct ShadowReceiver : public iShadowReceiver
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csInstmeshMeshObject);
-    virtual void CastShadows (iMovable* movable, iFrustumView* fview)
-    {
-      scfParent->CastShadows (movable, fview);
-    }
-  } scfiShadowReceiver;
-  friend struct ShadowReceiver;
-
-  //------------------------- iInstancingMeshState implementation ----------------
-  class InstancingMeshState : public iInstancingMeshState
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csInstmeshMeshObject);
-    virtual void SetMaterialWrapper (iMaterialWrapper* material)
-    { scfParent->SetMaterialWrapper (material); }
-    virtual iMaterialWrapper* GetMaterialWrapper () const
-    { return scfParent->material; }
-    virtual void SetMixMode (uint mode) { scfParent->MixMode = mode; }
-    virtual uint GetMixMode () const { return scfParent->MixMode; }
-    virtual void SetLighting (bool l) { scfParent->SetLighting (l); }
-    virtual bool IsLighting () const { return scfParent->IsLighting (); }
-    virtual void SetColor (const csColor& col) { scfParent->SetColor (col); }
-    virtual const csColor& GetColor () const { return scfParent->GetColor (); }
-    virtual void SetManualColors (bool m)
-    {
-      scfParent->SetManualColors (m);
-    }
-    virtual bool IsManualColors () const
-    {
-      return scfParent->IsManualColors ();
-    }
-    virtual void SetShadowCasting (bool m)
-    {
-      scfParent->do_shadows = m;
-    }
-    virtual bool IsShadowCasting () const
-    {
-      return scfParent->do_shadows;
-    }
-    virtual void SetShadowReceiving (bool m)
-    {
-      scfParent->do_shadow_rec = m;
-    }
-    virtual bool IsShadowReceiving () const
-    {
-      return scfParent->do_shadow_rec;
-    }
-    virtual size_t AddInstance (const csReversibleTransform& trans)
-    {
-      return scfParent->AddInstance (trans);
-    }
-    virtual void RemoveInstance (size_t id)
-    {
-      scfParent->RemoveInstance (id);
-    }
-    virtual void RemoveAllInstances ()
-    {
-      scfParent->RemoveAllInstances ();
-    }
-    virtual void MoveInstance (size_t id,
-      const csReversibleTransform& trans)
-    {
-      scfParent->MoveInstance (id, trans);
-    }
-    virtual const csReversibleTransform& GetInstanceTransform (size_t id)
-    {
-      return scfParent->GetInstanceTransform (id);
-    }
-  } scfiInstancingMeshState;
-  friend class InstancingMeshState;
+  virtual iObjectModel* GetObjectModel () { return this; }
 
   //------------------ iPolygonMesh interface implementation ----------------//
-  struct PolyMesh : public iPolygonMesh
+  struct PolyMesh : public scfImplementation1<PolyMesh, iPolygonMesh>
   {
   private:
     csFlags flags;
-
+    csInstmeshMeshObject* parent;
   public:
-    SCF_DECLARE_EMBEDDED_IBASE (csInstmeshMeshObject);
-
     virtual int GetVertexCount ();
     virtual csVector3* GetVertices ();
     virtual int GetPolygonCount ();
@@ -479,44 +359,75 @@ public:
     virtual csFlags& GetFlags () { return flags;  }
     virtual uint32 GetChangeNumber() const { return 0; }
 
-    PolyMesh ()
+    PolyMesh (csInstmeshMeshObject* parent) : scfImplementationType (this),
+      parent (parent)
     {
       flags.Set (CS_POLYMESH_TRIANGLEMESH);
     }
     virtual ~PolyMesh () { }
-  } scfiPolygonMesh;
+  };
+  csRef<PolyMesh> polygonMesh;
   friend struct PolyMesh;
 
-  class eiRenderBufferAccessor : public iRenderBufferAccessor
+  //------------------ iTriangleMesh interface implementation ----------------//
+  struct TriMesh : public scfImplementation1<TriMesh, iTriangleMesh>
+  {
+  private:
+    csFlags flags;
+    csInstmeshMeshObject* parent;
+  public:
+    virtual size_t GetVertexCount ();
+    virtual csVector3* GetVertices ();
+    virtual size_t GetTriangleCount ();
+    virtual csTriangle* GetTriangles ();
+    virtual void Lock () { }
+    virtual void Unlock () { }
+    
+    virtual csFlags& GetFlags () { return flags;  }
+    virtual uint32 GetChangeNumber() const { return 0; }
+
+    TriMesh (csInstmeshMeshObject* parent) : scfImplementationType (this),
+      parent (parent)
+    {
+    }
+    virtual ~TriMesh () { }
+  };
+  friend struct TriMesh;
+
+  class RenderBufferAccessor : 
+    public scfImplementation1<RenderBufferAccessor, iRenderBufferAccessor>
   {
   public:
-    CS_LEAKGUARD_DECLARE (eiRenderBufferAccessor);
-    SCF_DECLARE_IBASE;
-    csInstmeshMeshObject* parent;
-    virtual ~eiRenderBufferAccessor ()
+    CS_LEAKGUARD_DECLARE (RenderBufferAccessor);
+    csWeakRef<csInstmeshMeshObject> parent;
+    virtual ~RenderBufferAccessor ()
     {
-      SCF_DESTRUCT_IBASE ();
     }
-    eiRenderBufferAccessor (csInstmeshMeshObject* parent)
+    RenderBufferAccessor (csInstmeshMeshObject* parent) : 
+      scfImplementationType (this), parent (parent)
     {
-      SCF_CONSTRUCT_IBASE (0);
-      eiRenderBufferAccessor::parent = parent;
     }
     virtual void PreGetBuffer (csRenderBufferHolder* holder,
     	csRenderBufferName buffer)
     {
-      parent->PreGetBuffer (holder, buffer);
+      if (parent) parent->PreGetBuffer (holder, buffer);
     }
-  } *scfiRenderBufferAccessor;
-  friend class eiRenderBufferAccessor;
+  };
+  csRef<RenderBufferAccessor> renderBufferAccessor;
+  friend class RenderBufferAccessor;
 
   void PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer);
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * Factory for general meshes.
  */
-class csInstmeshMeshObjectFactory : public iMeshObjectFactory
+class csInstmeshMeshObjectFactory : 
+  public scfImplementation2<csInstmeshMeshObjectFactory,
+                            iMeshObjectFactory,
+                            iInstancingFactoryState>
 {
 private:
   friend class csInstmeshMeshObject;	//@@@ FIXME: remove
@@ -577,15 +488,16 @@ public:
   /// Do full bright.
   bool DoFullBright () const { return do_fullbright; }
 
-  void SetMaterialWrapper (iMaterialWrapper* material)
+  bool SetMaterialWrapper (iMaterialWrapper* material)
   {
     csInstmeshMeshObjectFactory::material = material;
+    return true;
   }
   iMaterialWrapper* GetMaterialWrapper () const { return material; }
   void AddVertex (const csVector3& v,
       const csVector2& uv, const csVector3& normal,
       const csColor4& color);
-  int GetVertexCount () const { return (int)fact_vertices.Length (); }
+  size_t GetVertexCount () const { return fact_vertices.GetSize (); }
   const csVector3* GetVertices ()
   {
     return fact_vertices.GetArray ();
@@ -607,12 +519,14 @@ public:
   {
     fact_triangles.Push (tri);
   }
-  size_t GetTriangleCount () const { return fact_triangles.Length (); }
+  size_t GetTriangleCount () const { return fact_triangles.GetSize (); }
   const csTriangle* GetTriangles () { return fact_triangles.GetArray (); }
 
   void CalculateNormals (bool compress);
   void Compress ();
   void GenerateBox (const csBox3& box);
+  void GenerateQuad (const csVector3& v1, const csVector3& v2, 
+    const csVector3& v3, const csVector3& v4);
   void GenerateSphere (const csEllipsoid& sphere, int rim_vertices,
       	bool cyl_mapping = false, bool toponly = false,
 	bool reversed = false);
@@ -674,8 +588,6 @@ public:
   }
 
   //------------------------ iMeshObjectFactory implementation --------------
-  SCF_DECLARE_IBASE;
-
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> NewInstance ();
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
@@ -690,132 +602,6 @@ public:
   { return logparent; }
   virtual iMeshObjectType* GetMeshObjectType () const { return instmesh_type; }
 
-  //----------------------- iInstancingFactoryState implementation -------------
-  class InstancingFactoryState : public iInstancingFactoryState
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csInstmeshMeshObjectFactory);
-
-    virtual void SetMixMode (uint mode)
-    {
-      scfParent->SetMixMode (mode);
-    }
-    virtual uint GetMixMode () const
-    {
-      return scfParent->GetMixMode ();
-    }
-    virtual void SetLighting (bool l)
-    {
-      scfParent->SetLighting (l);
-    }
-    virtual bool IsLighting () const
-    {
-      return scfParent->IsLighting ();
-    }
-    virtual void SetColor (const csColor& col)
-    {
-      scfParent->SetColor (col);
-    }
-    virtual const csColor& GetColor () const
-    {
-      return scfParent->GetColor ();
-    }
-    virtual void SetManualColors (bool m)
-    {
-      scfParent->SetManualColors (m);
-    }
-    virtual bool IsManualColors () const
-    {
-      return scfParent->IsManualColors ();
-    }
-    virtual void SetShadowCasting (bool m)
-    {
-      scfParent->SetShadowCasting (m);
-    }
-    virtual bool IsShadowCasting () const
-    {
-      return scfParent->IsShadowCasting ();
-    }
-    virtual void SetShadowReceiving (bool m)
-    {
-      scfParent->SetShadowReceiving (m);
-    }
-    virtual bool IsShadowReceiving () const
-    {
-      return scfParent->IsShadowReceiving ();
-    }
-
-    virtual void SetMaterialWrapper (iMaterialWrapper* material)
-    {
-      scfParent->SetMaterialWrapper (material);
-    }
-    virtual iMaterialWrapper* GetMaterialWrapper () const
-    {
-      return scfParent->GetMaterialWrapper ();
-    }
-    virtual void AddVertex (const csVector3& v,
-      const csVector2& uv, const csVector3& normal,
-      const csColor4& color)
-    {
-      scfParent->AddVertex (v, uv, normal, color);
-    }
-    virtual size_t GetVertexCount () const
-    {
-      return scfParent->GetVertexCount ();
-    }
-    virtual const csVector3* GetVertices ()
-    {
-      return scfParent->GetVertices ();
-    }
-    virtual const csVector2* GetTexels ()
-    {
-      return scfParent->GetTexels ();
-    }
-    virtual const csVector3* GetNormals ()
-    {
-      return scfParent->GetNormals ();
-    }
-    virtual void AddTriangle (const csTriangle& tri)
-    {
-      scfParent->AddTriangle (tri);
-    }
-    virtual size_t GetTriangleCount () const
-    {
-      return scfParent->GetTriangleCount ();
-    }
-    virtual const csTriangle* GetTriangles ()
-    {
-      return scfParent->GetTriangles ();
-    }
-    virtual const csColor4* GetColors ()
-    {
-      return scfParent->GetColors ();
-    }
-    virtual void Compress ()
-    {
-      scfParent->Compress ();
-    }
-    virtual void CalculateNormals (bool compress = true)
-    {
-      scfParent->CalculateNormals (compress);
-    }
-    virtual void GenerateBox (const csBox3& box)
-    {
-      scfParent->GenerateBox (box);
-    }
-    virtual void GenerateSphere (const csEllipsoid& sphere, int rim_vertices,
-		bool cyl_mapping = false, bool toponly = false,
-		bool reversed = false)
-    {
-      scfParent->GenerateSphere (sphere, rim_vertices,
-	  cyl_mapping, toponly, reversed);
-    }
-    virtual bool IsAutoNormals () const
-    {
-      return scfParent->IsAutoNormals ();
-    }
-  } scfiInstancingFactoryState;
-  friend class InstancingFactoryState;
-
   virtual iObjectModel* GetObjectModel () { return 0; }
 };
 
@@ -823,14 +609,15 @@ public:
  * Instmesh type. This is the plugin you have to use to create instances
  * of csInstmeshMeshObjectFactory.
  */
-class csInstmeshMeshObjectType : public iMeshObjectType
+class csInstmeshMeshObjectType : 
+  public scfImplementation2<csInstmeshMeshObjectType,
+                            iMeshObjectType,
+                            iComponent>
 {
 public:
   iObjectRegistry* object_reg;
   bool do_verbose;
 
-  SCF_DECLARE_IBASE;
-
   /// Constructor.
   csInstmeshMeshObjectType (iBase*);
   /// Destructor.
@@ -839,15 +626,9 @@ public:
   virtual csPtr<iMeshObjectFactory> NewFactory ();
   /// Initialize.
   bool Initialize (iObjectRegistry* object_reg);
-
-  struct eiComponent : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE(csInstmeshMeshObjectType);
-    virtual bool Initialize (iObjectRegistry* object_reg)
-    {
-      return scfParent->Initialize (object_reg);
-    }
-  } scfiComponent;
 };
 
+}
+CS_PLUGIN_NAMESPACE_END(InstMesh)
+
 #endif // __CS_INSTMESH_H__
