@@ -52,6 +52,7 @@ CS_IMPLEMENT_PLUGIN
 enum
 {
   XMLTOKEN_BOX = 1,
+  XMLTOKEN_QUAD,
   XMLTOKEN_SPHERE,
   XMLTOKEN_LIGHTING,
   XMLTOKEN_DEFAULTCOLOR,
@@ -71,67 +72,28 @@ enum
   XMLTOKEN_INSTANCE
 };
 
-SCF_IMPLEMENT_IBASE (csInstFactoryLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstFactoryLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csInstFactorySaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstFactorySaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csInstMeshLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstMeshLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csInstMeshSaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csInstMeshSaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csInstFactoryLoader)
 SCF_IMPLEMENT_FACTORY (csInstFactorySaver)
 SCF_IMPLEMENT_FACTORY (csInstMeshLoader)
 SCF_IMPLEMENT_FACTORY (csInstMeshSaver)
 
-
-csInstFactoryLoader::csInstFactoryLoader (iBase* pParent)
+csInstFactoryLoader::csInstFactoryLoader (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csInstFactoryLoader::~csInstFactoryLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csInstFactoryLoader::Initialize (iObjectRegistry* object_reg)
 {
   csInstFactoryLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
   xmltokens.Register ("box", XMLTOKEN_BOX);
+  xmltokens.Register ("quad", XMLTOKEN_QUAD);
   xmltokens.Register ("sphere", XMLTOKEN_SPHERE);
   xmltokens.Register ("material", XMLTOKEN_MATERIAL);
   xmltokens.Register ("factory", XMLTOKEN_FACTORY);
@@ -161,15 +123,8 @@ static float GetDef (iDocumentNode* node, const char* attrname, float def)
 csPtr<iBase> csInstFactoryLoader::Parse (iDocumentNode* node,
 	iStreamSource*, iLoaderContext* ldr_context, iBase* /* context */)
 {
-  csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (object_reg,
-  	iPluginManager));
-  csRef<iMeshObjectType> type (CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-  	"crystalspace.mesh.object.instmesh", iMeshObjectType));
-  if (!type)
-  {
-    type = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.mesh.object.instmesh",
-    	iMeshObjectType);
-  }
+  csRef<iMeshObjectType> type = csLoadPluginCheck<iMeshObjectType> (
+  	object_reg, "crystalspace.mesh.object.instmesh", false);
   if (!type)
   {
     synldr->ReportError (
@@ -181,7 +136,7 @@ csPtr<iBase> csInstFactoryLoader::Parse (iDocumentNode* node,
   csRef<iInstancingFactoryState> state;
 
   fact = type->NewFactory ();
-  state = SCF_QUERY_INTERFACE (fact, iInstancingFactoryState);
+  state = scfQueryInterface<iInstancingFactoryState> (fact);
 
   bool auto_normals = false;
   bool auto_normals_nocompress = false;
@@ -235,7 +190,7 @@ csPtr<iBase> csInstFactoryLoader::Parse (iDocumentNode* node,
 	  uint mm;
 	  if (!synldr->ParseMixmode (child, mm))
 	    return 0;
-          state->SetMixMode (mm);
+          fact->SetMixMode (mm);
 	}
 	break;
       case XMLTOKEN_MATERIAL:
@@ -249,56 +204,82 @@ csPtr<iBase> csInstFactoryLoader::Parse (iDocumentNode* node,
 		child, "Couldn't find material '%s'!", matname);
             return 0;
 	  }
-	  state->SetMaterialWrapper (mat);
+	  fact->SetMaterialWrapper (mat);
 	}
 	break;
       case XMLTOKEN_BOX:
         {
-	  csBox3 box;
-	  if (!synldr->ParseBox (child, box))
-	    return 0;
-	  state->GenerateBox (box);
-	}
+          csBox3 box;
+          if (!synldr->ParseBox (child, box))
+            return 0;
+          state->GenerateBox (box);
+        }
+        break;
+      case XMLTOKEN_QUAD:
+        {
+          csVector3 v1, v2, v3, v4;
+          csRef<iDocumentNode> c = child->GetNode ("v1");
+          if (c)
+            if (!synldr->ParseVector (c, v1))
+              return 0;
+
+          c = child->GetNode ("v2");
+          if (c)
+            if (!synldr->ParseVector (c, v2))
+              return 0;
+
+          c = child->GetNode ("v3");
+          if (c)
+            if (!synldr->ParseVector (c, v3))
+              return 0;
+
+          c = child->GetNode ("v4");
+          if (c)
+            if (!synldr->ParseVector (c, v4))
+              return 0;
+
+          state->GenerateQuad (v1, v2, v3, v4);
+        }
         break;
       case XMLTOKEN_SPHERE:
         {
-	  csVector3 center (0, 0, 0);
-	  int rim_vertices = 8;
-	  csEllipsoid ellips;
-	  csRef<iDocumentAttribute> attr;
-	  csRef<iDocumentNode> c = child->GetNode ("center");
-	  if (c)
-	    if (!synldr->ParseVector (c, ellips.GetCenter ()))
-	      return 0;
-	  c = child->GetNode ("radius");
-	  if (c)
-	  {
-	    if (!synldr->ParseVector (c, ellips.GetRadius ()))
-	      return 0;
-	  }
-	  else
-	  {
-	    attr = child->GetAttribute ("radius");
-	    float radius;
-	    if (attr) radius = attr->GetValueAsFloat ();
-	    else radius = 1.0f;
-	    ellips.SetRadius (csVector3 (radius, radius, radius));
-	  }
-	  attr = child->GetAttribute ("rimvertices");
-	  if (attr) rim_vertices = attr->GetValueAsInt ();
-	  bool cylmapping, toponly, reversed;
-	  if (!synldr->ParseBoolAttribute (child, "cylindrical", cylmapping,
-	  	false, false))
-  	    return 0;
-	  if (!synldr->ParseBoolAttribute (child, "toponly", toponly,
-	  	false, false))
-  	    return 0;
-	  if (!synldr->ParseBoolAttribute (child, "reversed", reversed,
-	  	false, false))
-  	    return 0;
-	  state->GenerateSphere (ellips, rim_vertices,
-	      cylmapping, toponly, reversed);
-	}
+          csVector3 center (0, 0, 0);
+          int rim_vertices = 8;
+          csEllipsoid ellips;
+          csRef<iDocumentAttribute> attr;
+          csRef<iDocumentNode> c = child->GetNode ("center");
+          if (c)
+            if (!synldr->ParseVector (c, ellips.GetCenter ()))
+              return 0;
+          c = child->GetNode ("radius");
+          if (c)
+          {
+            if (!synldr->ParseVector (c, ellips.GetRadius ()))
+              return 0;
+          }
+          else
+          {
+            attr = child->GetAttribute ("radius");
+            float radius;
+            if (attr) radius = attr->GetValueAsFloat ();
+            else radius = 1.0f;
+            ellips.SetRadius (csVector3 (radius, radius, radius));
+          }
+          attr = child->GetAttribute ("rimvertices");
+          if (attr) rim_vertices = attr->GetValueAsInt ();
+          bool cylmapping, toponly, reversed;
+          if (!synldr->ParseBoolAttribute (child, "cylindrical", cylmapping,
+            false, false))
+            return 0;
+          if (!synldr->ParseBoolAttribute (child, "toponly", toponly,
+            false, false))
+            return 0;
+          if (!synldr->ParseBoolAttribute (child, "reversed", reversed,
+            false, false))
+            return 0;
+          state->GenerateSphere (ellips, rim_vertices,
+            cylmapping, toponly, reversed);
+        }
         break;
       case XMLTOKEN_AUTONORMALS:
         if (!synldr->ParseBool (child, auto_normals, true))
@@ -367,23 +348,20 @@ csPtr<iBase> csInstFactoryLoader::Parse (iDocumentNode* node,
 
 //---------------------------------------------------------------------------
 
-csInstFactorySaver::csInstFactorySaver (iBase* pParent)
+csInstFactorySaver::csInstFactorySaver (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csInstFactorySaver::~csInstFactorySaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csInstFactorySaver::Initialize (iObjectRegistry* object_reg)
 {
   csInstFactorySaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
 
@@ -399,9 +377,9 @@ bool csInstFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
   if (obj)
   {
     csRef<iInstancingFactoryState> gfact = 
-      SCF_QUERY_INTERFACE (obj, iInstancingFactoryState);
+      scfQueryInterface<iInstancingFactoryState> (obj);
     csRef<iMeshObjectFactory> meshfact = 
-      SCF_QUERY_INTERFACE (obj, iMeshObjectFactory);
+      scfQueryInterface<iMeshObjectFactory> (obj);
     if (!gfact) return false;
     if (!meshfact) return false;
 
@@ -430,7 +408,7 @@ bool csInstFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
       csVector2 texel = gfact->GetTexels()[i];
       csVector3 normal = gfact->GetNormals()[i];
       csColor4 color = gfact->GetColors()[i];
-      synldr->WriteVector(triaNode, &vertex);
+      synldr->WriteVector(triaNode, vertex);
       triaNode->SetAttributeAsFloat("u", texel.x);
       triaNode->SetAttributeAsFloat("v", texel.y);
       triaNode->SetAttributeAsFloat("nx", normal.x);
@@ -461,11 +439,11 @@ bool csInstFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
       csRef<iDocumentNode> colorNode = 
         paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
       colorNode->SetValue("defaultcolor");
-      synldr->WriteColor(colorNode, &col);
+      synldr->WriteColor(colorNode, col);
     }
 
     //Writedown Material tag
-    iMaterialWrapper* mat = gfact->GetMaterialWrapper();
+    iMaterialWrapper* mat = meshfact->GetMaterialWrapper();
     if (mat)
     {
       const char* matname = mat->QueryObject()->GetName();
@@ -481,7 +459,7 @@ bool csInstFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
     }    
 
     //Writedown Mixmode tag
-    int mixmode = gfact->GetMixMode();
+    int mixmode = meshfact->GetMixMode();
     csRef<iDocumentNode> mixmodeNode = 
       paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
     mixmodeNode->SetValue("mixmode");
@@ -519,23 +497,20 @@ bool csInstFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
 
 //---------------------------------------------------------------------------
 
-csInstMeshLoader::csInstMeshLoader (iBase* pParent)
+csInstMeshLoader::csInstMeshLoader (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csInstMeshLoader::~csInstMeshLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csInstMeshLoader::Initialize (iObjectRegistry* object_reg)
 {
   csInstMeshLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
   xmltokens.Register ("material", XMLTOKEN_MATERIAL);
   xmltokens.Register ("factory", XMLTOKEN_FACTORY);
@@ -549,6 +524,14 @@ bool csInstMeshLoader::Initialize (iObjectRegistry* object_reg)
   return true;
 }
 
+#define CHECK_MESH(m) \
+  if (!m) { \
+    synldr->ReportError ( \
+	"crystalspace.instmeshloader.parse.unknownfactory", \
+	child, "Specify the factory first!"); \
+    return 0; \
+  }
+
 csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 	iStreamSource*, iLoaderContext* ldr_context, iBase*)
 {
@@ -584,6 +567,7 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 	      return false;
             trans.SetO2TTranslation (v);
 	  }
+	  CHECK_MESH (meshstate);
 	  meshstate->AddInstance (trans);
 	}
 	break;
@@ -592,16 +576,19 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 	  bool r;
 	  if (!synldr->ParseBool (child, r, true))
 	    return 0;
+	  CHECK_MESH (meshstate);
 	  meshstate->SetManualColors (r);
 	}
 	break;
       case XMLTOKEN_NOSHADOWS:
 	{
+	  CHECK_MESH (meshstate);
 	  meshstate->SetShadowCasting (false);
 	}
 	break;
       case XMLTOKEN_LOCALSHADOWS:
 	{
+	  CHECK_MESH (meshstate);
 	  meshstate->SetShadowReceiving (true);
 	}
 	break;
@@ -610,6 +597,7 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 	  bool r;
 	  if (!synldr->ParseBool (child, r, true))
 	    return 0;
+	  CHECK_MESH (meshstate);
 	  meshstate->SetLighting (r);
 	}
 	break;
@@ -618,7 +606,8 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 	  csColor col;
 	  if (!synldr->ParseColor (child, col))
 	    return 0;
-	  meshstate->SetColor (col);
+	  CHECK_MESH (mesh);
+	  mesh->SetColor (col);
 	}
 	break;
       case XMLTOKEN_FACTORY:
@@ -632,8 +621,8 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 		child, "Couldn't find factory '%s'!", factname);
 	    return 0;
 	  }
-	  factstate = SCF_QUERY_INTERFACE (fact->GetMeshObjectFactory(), 
-	    iInstancingFactoryState);
+	  factstate =  
+	    scfQueryInterface<iInstancingFactoryState> (fact->GetMeshObjectFactory());
 	  if (!factstate)
 	  {
       	    synldr->ReportError (
@@ -644,7 +633,7 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 	  }
 	  mesh = fact->GetMeshObjectFactory ()->NewInstance ();
 	  CS_ASSERT (mesh != 0);
-          meshstate = SCF_QUERY_INTERFACE (mesh, iInstancingMeshState);
+          meshstate = scfQueryInterface<iInstancingMeshState> (mesh);
 	  if (!meshstate)
 	  {
       	    synldr->ReportError (
@@ -666,7 +655,8 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 		child, "Couldn't find material '%s'!", matname);
             return 0;
 	  }
-	  meshstate->SetMaterialWrapper (mat);
+	  CHECK_MESH (mesh);
+	  mesh->SetMaterialWrapper (mat);
 	}
 	break;
       case XMLTOKEN_MIXMODE:
@@ -674,7 +664,8 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 	  uint mm;
 	  if (!synldr->ParseMixmode (child, mm))
 	    return 0;
-          meshstate->SetMixMode (mm);
+	  CHECK_MESH (mesh);
+          mesh->SetMixMode (mm);
 	}
 	break;
       default:
@@ -688,23 +679,20 @@ csPtr<iBase> csInstMeshLoader::Parse (iDocumentNode* node,
 
 //---------------------------------------------------------------------------
 
-csInstMeshSaver::csInstMeshSaver (iBase* pParent)
+csInstMeshSaver::csInstMeshSaver (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csInstMeshSaver::~csInstMeshSaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csInstMeshSaver::Initialize (iObjectRegistry* object_reg)
 {
   csInstMeshSaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
 
@@ -720,8 +708,8 @@ bool csInstMeshSaver::WriteDown (iBase* obj, iDocumentNode* parent,
   if (obj)
   {
     csRef<iInstancingMeshState> gmesh = 
-      SCF_QUERY_INTERFACE (obj, iInstancingMeshState);
-    csRef<iMeshObject> mesh = SCF_QUERY_INTERFACE (obj, iMeshObject);
+      scfQueryInterface<iInstancingMeshState> (obj);
+    csRef<iMeshObject> mesh = scfQueryInterface<iMeshObject> (obj);
     if (!gmesh) return false;
     if (!mesh) return false;
 
@@ -752,18 +740,19 @@ bool csInstMeshSaver::WriteDown (iBase* obj, iDocumentNode* parent,
         ->SetValue("localshadows");
 
     //Writedown Color tag
-    csColor col = gmesh->GetColor();
+    csColor col;
+    mesh->GetColor(col);
     csRef<iDocumentNode> colorNode = 
       paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
     colorNode->SetValue("color");
-    synldr->WriteColor(colorNode, &col);
+    synldr->WriteColor(colorNode, col);
 
     //Writedown ManualColor tag
     synldr->WriteBool(paramsNode, "manualcolors",
                       gmesh->IsManualColors(), true);
 
     //Writedown Material tag
-    iMaterialWrapper* mat = gmesh->GetMaterialWrapper();
+    iMaterialWrapper* mat = mesh->GetMaterialWrapper();
     if (mat)
     {
       const char* matname = mat->QueryObject()->GetName();
@@ -779,7 +768,7 @@ bool csInstMeshSaver::WriteDown (iBase* obj, iDocumentNode* parent,
     }    
 
     //Writedown Mixmode tag
-    int mixmode = gmesh->GetMixMode();
+    int mixmode = mesh->GetMixMode();
     csRef<iDocumentNode> mixmodeNode = 
       paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
     mixmodeNode->SetValue("mixmode");
