@@ -26,6 +26,8 @@
 #include "csgeom/transfrm.h"
 #include "csgfx/renderbuffer.h"
 #include "cstool/rbuflock.h"
+#include "cstool/rviewclipper.h"
+#include "csutil/scfarray.h"
 
 #include "iengine/movable.h"
 #include "iengine/rview.h"
@@ -42,67 +44,33 @@
 
 #include "spr2d.h"
 
-
-CS_LEAKGUARD_IMPLEMENT (csSprite2DMeshObject);
-CS_LEAKGUARD_IMPLEMENT (csSprite2DMeshObjectFactory);
-
 CS_IMPLEMENT_PLUGIN
 
-SCF_IMPLEMENT_IBASE (csSprite2DMeshObject)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObject)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iObjectModel)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iSprite2DState)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iParticle)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObject::ObjectModel)
-  SCF_IMPLEMENTS_INTERFACE (iObjectModel)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObject::Sprite2DState)
-  SCF_IMPLEMENTS_INTERFACE (iSprite2DState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObject::Particle)
-  SCF_IMPLEMENTS_INTERFACE (iParticle)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSprite2DMeshObject::eiRenderBufferAccessor)
-  SCF_IMPLEMENTS_INTERFACE (iRenderBufferAccessor)
-SCF_IMPLEMENT_IBASE_END
-
-csSprite2DMeshObject::csSprite2DMeshObject (csSprite2DMeshObjectFactory* factory)
+CS_PLUGIN_NAMESPACE_BEGIN(Spr2D)
 {
-  SCF_CONSTRUCT_IBASE (0);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiSprite2DState);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiParticle);
-  csSprite2DMeshObject::factory = factory;
-  logparent = 0;
-  ifactory = SCF_QUERY_INTERFACE (factory, iMeshObjectFactory);
+
+CS_LEAKGUARD_IMPLEMENT (csSprite2DMeshObject);
+CS_LEAKGUARD_IMPLEMENT (csSprite2DMeshObject::RenderBufferAccessor);
+CS_LEAKGUARD_IMPLEMENT (csSprite2DMeshObjectFactory);
+
+csSprite2DMeshObject::csSprite2DMeshObject (csSprite2DMeshObjectFactory* factory) :
+  scfImplementationType (this), uvani (0), vertices_dirty (true), 
+  texels_dirty (true), colors_dirty (true), indicesSize ((size_t)-1), 
+  logparent (0), factory (factory), initialized (false), current_lod (1), 
+  current_features (0)
+{
+  ifactory = scfQueryInterface<iMeshObjectFactory> (factory);
   material = factory->GetMaterialWrapper ();
   lighting = factory->HasLighting ();
   MixMode = factory->GetMixMode ();
-  initialized = false;
-  vis_cb = 0;
-  current_lod = 1;
-  current_features = 0;
-  uvani = 0;
 
-  vertices_dirty = true;
-  texels_dirty = true;
-  colors_dirty = true;
-  indicesSize = (size_t)-1;
+  scfVertices.AttachNew (new scfArrayWrap<iColoredVertices, 
+    csColoredVertices> (vertices));
 }
 
 csSprite2DMeshObject::~csSprite2DMeshObject ()
 {
-  if (vis_cb) vis_cb->DecRef ();
   delete uvani;
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiObjectModel);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiSprite2DState);
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiParticle);
-  SCF_DESTRUCT_IBASE ();
 }
 
 void csSprite2DMeshObject::SetupObject ()
@@ -113,7 +81,7 @@ void csSprite2DMeshObject::SetupObject ()
     float max_sq_dist = 0;
     size_t i;
     bbox_2d.StartBoundingBox(vertices[0].pos);
-    for (i = 0 ; i < vertices.Length () ; i++)
+    for (i = 0 ; i < vertices.GetSize () ; i++)
     {
       csSprite2DVertex& v = vertices[i];
       bbox_2d.AddBoundingVertexSmart(v.pos);
@@ -127,12 +95,11 @@ void csSprite2DMeshObject::SetupObject ()
       float sqdist = v.pos.x*v.pos.x + v.pos.y*v.pos.y;
       if (sqdist > max_sq_dist) max_sq_dist = sqdist;
     }
-    float max_dist = csQsqrt (max_sq_dist);
-    radius.Set (max_dist, max_dist, max_dist);
+    radius = csQsqrt (max_sq_dist);
 
     bufferHolder.AttachNew (new csRenderBufferHolder);
     csRef<iRenderBufferAccessor> newAccessor;
-    newAccessor.AttachNew (new eiRenderBufferAccessor (this));
+    newAccessor.AttachNew (new RenderBufferAccessor (this));
     bufferHolder->SetAccessor (newAccessor, (uint32)CS_BUFFER_ALL_MASK);
     svcontext.AttachNew (new csShaderVariableContext);
   }
@@ -157,7 +124,7 @@ void csSprite2DMeshObject::UpdateLighting (
   //}
 
   int i;
-  int num_lights = (int)lights.Length ();
+  int num_lights = (int)lights.GetSize ();
   for (i = 0; i < num_lights; i++)
   {
     iLight* li = lights[i]->GetLight ();
@@ -175,7 +142,7 @@ void csSprite2DMeshObject::UpdateLighting (
     light_color *= cosinus * li->GetBrightnessAtDistance (wor_dist);
     color += light_color;
   }
-  for (size_t j = 0 ; j < vertices.Length () ; j++)
+  for (size_t j = 0 ; j < vertices.GetSize () ; j++)
   {
     vertices[j].color = vertices[j].color_init + color;
     vertices[j].color.Clamp (2, 2, 2);
@@ -224,8 +191,8 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
     temp /= movable->GetFullTransform ();
 
   int clip_portal, clip_plane, clip_z_plane;
-  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane, 
-    clip_z_plane);
+  CS::RenderViewClipper::CalculateClipSettings (rview->GetRenderContext (),
+      frustum_mask, clip_portal, clip_plane, clip_z_plane);
 
   csReversibleTransform tr_o2c;
   tr_o2c.SetO2TTranslation (-temp.Other2This (offset));
@@ -258,7 +225,7 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
   rm->worldspace_origin = movable->GetFullPosition ();
 
   rm->object2world = tr_o2c.GetInverse () * camera->GetTransform ();
-  rm->indexend = (uint)vertices.Length();
+  rm->indexend = (uint)vertices.GetSize ();
 
 
   n = 1; 
@@ -271,20 +238,20 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
 
   if (buffer == CS_BUFFER_INDEX)
   {
-    size_t indexSize = vertices.Length();
+    size_t indexSize = vertices.GetSize ();
     if (!index_buffer.IsValid() || 
       (indicesSize != indexSize))
     {
       index_buffer = csRenderBuffer::CreateIndexRenderBuffer (
 	indexSize, CS_BUF_DYNAMIC, 
-	CS_BUFCOMP_UNSIGNED_INT, 0, vertices.Length() - 1);
+	CS_BUFCOMP_UNSIGNED_INT, 0, vertices.GetSize () - 1);
       
       holder->SetRenderBuffer (CS_BUFFER_INDEX, index_buffer);
 
       csRenderBufferLock<uint> indexLock (index_buffer);
       uint* ptr = indexLock;
 
-      for (size_t i = 0; i < vertices.Length(); i++)
+      for (size_t i = 0; i < vertices.GetSize (); i++)
       {
 	*ptr++ = (uint)i;
       }
@@ -298,7 +265,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
       int texels_count;
       const csVector2 *uvani_uv = 0;
       if (!uvani)
-	texels_count = (int)vertices.Length ();
+	texels_count = (int)vertices.GetSize ();
       else
 	uvani_uv = uvani->GetVertices (texels_count);
 	  
@@ -334,7 +301,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
   {
     if (colors_dirty)
     {
-      size_t color_size = vertices.Length();
+      size_t color_size = vertices.GetSize ();
       if (!color_buffer.IsValid() || (color_buffer->GetSize() 
 	!= color_size * sizeof(float) * 2))
       {
@@ -346,7 +313,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
 
       csRenderBufferLock<csColor> colorLock (color_buffer);
 
-      for (size_t i = 0; i < vertices.Length(); i++)
+      for (size_t i = 0; i < vertices.GetSize (); i++)
       {
 	colorLock[i] = vertices[i].color;
       }
@@ -357,7 +324,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
   {
     if (vertices_dirty)
     {
-      size_t vertices_size = vertices.Length();
+      size_t vertices_size = vertices.GetSize ();
       if (!vertex_buffer.IsValid() || (vertex_buffer->GetSize() 
 	!= vertices_size * sizeof(float) * 3))
       {
@@ -369,7 +336,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
 
       csRenderBufferLock<csVector3> vertexLock (vertex_buffer);
 
-      for (size_t i = 0; i < vertices.Length(); i++)
+      for (size_t i = 0; i < vertices.GetSize (); i++)
       {
 	vertexLock[i].Set (vertices[i].pos.x, vertices[i].pos.y, 0.0f);
       }
@@ -384,6 +351,13 @@ void csSprite2DMeshObject::GetObjectBoundingBox (csBox3& bbox)
   bbox.Set (-radius, radius);
 }
 
+const csBox3& csSprite2DMeshObject::GetObjectBoundingBox ()
+{
+  SetupObject ();
+  obj_bbox.Set (-radius, radius);
+  return obj_bbox;
+}
+
 void csSprite2DMeshObject::SetObjectBoundingBox (const csBox3&)
 {
   // @@@ TODO
@@ -399,9 +373,9 @@ void csSprite2DMeshObject::CreateRegularVertices (int n, bool setuv)
 {
   double angle_inc = TWO_PI / n;
   double angle = 0.0;
-  vertices.SetLength (n);
+  vertices.SetSize (n);
   size_t i;
-  for (i = 0; i < vertices.Length (); i++, angle += angle_inc)
+  for (i = 0; i < vertices.GetSize (); i++, angle += angle_inc)
   {
     vertices [i].pos.y = cos (angle);
     vertices [i].pos.x = sin (angle);
@@ -419,11 +393,11 @@ void csSprite2DMeshObject::CreateRegularVertices (int n, bool setuv)
   texels_dirty = true;
   colors_dirty = true;
 
-  scfiObjectModel.ShapeChanged ();
+  ShapeChanged ();
 }
 
 void csSprite2DMeshObject::NextFrame (csTicks current_time,
-	const csVector3& /*pos*/)
+	const csVector3& /*pos*/, uint /*currentFrame*/)
 {
   if (uvani && !uvani->halted)
   {
@@ -433,167 +407,156 @@ void csSprite2DMeshObject::NextFrame (csTicks current_time,
   }
 }
 
-void csSprite2DMeshObject::Particle::UpdateLighting (
+void csSprite2DMeshObject::UpdateLighting (
 	const csArray<iLightSectorInfluence*>& lights,
 	const csReversibleTransform& transform)
 {
   csVector3 new_pos = transform.This2Other (part_pos);
-  scfParent->UpdateLighting (lights, new_pos);
+  UpdateLighting (lights, new_pos);
 }
 
    
-csRenderMesh** csSprite2DMeshObject::Particle::GetRenderMeshes (int& n, 
-	iRenderView* rview, iMovable* movable, uint32 frustum_mask)
+void csSprite2DMeshObject::AddColor (const csColor& col)
 {
-  return scfParent->GetRenderMeshes (n, rview, movable, frustum_mask, part_pos);
-}
-
-void csSprite2DMeshObject::Particle::SetColor (const csColor& col)
-{
-  csColoredVertices& vertices = scfParent->GetVertices ();
+  iColoredVertices* vertices = GetVertices ();
   size_t i;
-  for (i = 0 ; i < vertices.Length () ; i++)
-    vertices[i].color_init = col;
-  if (!scfParent->lighting)
-    for (i = 0 ; i < vertices.Length () ; i++)
-      vertices[i].color = col;
-
-  scfParent->colors_dirty = true;
+  for (i = 0 ; i < vertices->GetSize(); i++)
+    vertices->Get (i).color_init += col;
+  if (!lighting)
+    for (i = 0 ; i < vertices->GetSize(); i++)
+      vertices->Get (i).color = vertices->Get (i).color_init;
 
+  colors_dirty = true;
 }
 
-void csSprite2DMeshObject::Particle::AddColor (const csColor& col)
+bool csSprite2DMeshObject::SetColor (const csColor& col)
 {
-  csColoredVertices& vertices = scfParent->GetVertices ();
+  iColoredVertices* vertices = GetVertices ();
   size_t i;
-  for (i = 0 ; i < vertices.Length () ; i++)
-    vertices[i].color_init += col;
-  if (!scfParent->lighting)
-    for (i = 0 ; i < vertices.Length () ; i++)
-      vertices[i].color = vertices[i].color_init;
-
-  scfParent->colors_dirty = true;
+  for (i = 0 ; i < vertices->GetSize(); i++)
+    vertices->Get (i).color_init = col;
+  if (!lighting)
+    for (i = 0 ; i < vertices->GetSize(); i++)
+      vertices->Get (i).color = col;
 
+  colors_dirty = true;
+  return true;
 }
 
-void csSprite2DMeshObject::Particle::ScaleBy (float factor)
+void csSprite2DMeshObject::ScaleBy (float factor)
 {
-  csColoredVertices& vertices = scfParent->GetVertices ();
+  iColoredVertices* vertices = GetVertices ();
   size_t i;
-  for (i = 0; i < vertices.Length (); i++)
-    vertices[i].pos *= factor;
+  for (i = 0 ; i < vertices->GetSize(); i++)
+    vertices->Get (i).pos *= factor;
 
-  scfParent->vertices_dirty = true;
-
-  scfParent->scfiObjectModel.ShapeChanged ();
+  vertices_dirty = true;
+  ShapeChanged ();
 }
 
-void csSprite2DMeshObject::Particle::Rotate (float angle)
+void csSprite2DMeshObject::Rotate (float angle)
 {
-  csColoredVertices& vertices = scfParent->GetVertices ();
+  iColoredVertices* vertices = GetVertices ();
   size_t i;
-  for (i = 0; i < vertices.Length (); i++)
-    vertices[i].pos.Rotate (angle);
+  for (i = 0 ; i < vertices->GetSize(); i++)
+    vertices->Get (i).pos.Rotate (angle);
 
-  scfParent->vertices_dirty = true;
-
-  scfParent->scfiObjectModel.ShapeChanged ();
+  vertices_dirty = true;
+  ShapeChanged ();
 }
 
-void csSprite2DMeshObject::Sprite2DState::SetUVAnimation (const char *name,
+void csSprite2DMeshObject::SetUVAnimation (const char *name,
 	int style, bool loop)
 {
   if (name)
   {
-    iSprite2DUVAnimation *ani = scfParent->factory->GetUVAnimation (name);
+    iSprite2DUVAnimation *ani = factory->GetUVAnimation (name);
     if (ani && ani->GetFrameCount ())
     {
-      scfParent->uvani = new uvAnimationControl ();
-      scfParent->uvani->ani = ani;
-      scfParent->uvani->last_time = 0;
-      scfParent->uvani->frameindex = 0;
-      scfParent->uvani->framecount = ani->GetFrameCount ();
-      scfParent->uvani->frame = ani->GetFrame (0);
-      scfParent->uvani->style = style;
-      scfParent->uvani->counter = 0;
-      scfParent->uvani->loop = loop;
-      scfParent->uvani->halted = false;
+      uvani = new uvAnimationControl ();
+      uvani->ani = ani;
+      uvani->last_time = 0;
+      uvani->frameindex = 0;
+      uvani->framecount = ani->GetFrameCount ();
+      uvani->frame = ani->GetFrame (0);
+      uvani->style = style;
+      uvani->counter = 0;
+      uvani->loop = loop;
+      uvani->halted = false;
     }
   }
   else
   {
     // stop animation and show the normal texture
-    delete scfParent->uvani;
-    scfParent->uvani = 0;
+    delete uvani;
+    uvani = 0;
   }
 }
 
-void csSprite2DMeshObject::Sprite2DState::StopUVAnimation (int idx)
+void csSprite2DMeshObject::StopUVAnimation (int idx)
 {
-  if (scfParent->uvani)
+  if (uvani)
   {
     if (idx != -1)
     {
-      scfParent->uvani->frameindex = MIN(MAX(idx, 0),
-      	scfParent->uvani->framecount-1);
-      scfParent->uvani->frame = scfParent->uvani->ani->GetFrame (
-      	scfParent->uvani->frameindex);
+      uvani->frameindex = MIN(MAX(idx, 0), uvani->framecount-1);
+      uvani->frame = uvani->ani->GetFrame (uvani->frameindex);
     }
-    scfParent->uvani->halted = true;
+    uvani->halted = true;
   }
 }
 
-void csSprite2DMeshObject::Sprite2DState::PlayUVAnimation (int idx, int style, bool loop)
+void csSprite2DMeshObject::PlayUVAnimation (int idx, int style, bool loop)
 {
-  if (scfParent->uvani)
+  if (uvani)
   {
     if (idx != -1)
     {
-      scfParent->uvani->frameindex = MIN(MAX(idx, 0), scfParent->uvani->framecount-1);
-      scfParent->uvani->frame = scfParent->uvani->ani->GetFrame (scfParent->uvani->frameindex);
+      uvani->frameindex = MIN(MAX(idx, 0), uvani->framecount-1);
+      uvani->frame = uvani->ani->GetFrame (uvani->frameindex);
     }
-    scfParent->uvani->halted = false;
-    scfParent->uvani->counter = 0;
-    scfParent->uvani->last_time = 0;
-    scfParent->uvani->loop = loop;
-    scfParent->uvani->style = style;
+    uvani->halted = false;
+    uvani->counter = 0;
+    uvani->last_time = 0;
+    uvani->loop = loop;
+    uvani->style = style;
   }
 }
 
-int csSprite2DMeshObject::Sprite2DState::GetUVAnimationCount () const
+int csSprite2DMeshObject::GetUVAnimationCount () const
 {
-  return scfParent->factory->GetUVAnimationCount ();
+  return factory->GetUVAnimationCount ();
 }
 
-iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::CreateUVAnimation ()
+iSprite2DUVAnimation *csSprite2DMeshObject::CreateUVAnimation ()
 {
-  return scfParent->factory->CreateUVAnimation ();
+  return factory->CreateUVAnimation ();
 }
 
-void csSprite2DMeshObject::Sprite2DState::RemoveUVAnimation (
+void csSprite2DMeshObject::RemoveUVAnimation (
 	iSprite2DUVAnimation *anim)
 {
-  scfParent->factory->RemoveUVAnimation (anim);
+  factory->RemoveUVAnimation (anim);
 }
 
-iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
+iSprite2DUVAnimation *csSprite2DMeshObject::GetUVAnimation (
 	const char *name) const
 {
-  return scfParent->factory->GetUVAnimation (name);
+  return factory->GetUVAnimation (name);
 }
 
-iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
+iSprite2DUVAnimation *csSprite2DMeshObject::GetUVAnimation (
 	int idx) const
 {
-  return scfParent->factory->GetUVAnimation (idx);
+  return factory->GetUVAnimation (idx);
 }
 
-iSprite2DUVAnimation *csSprite2DMeshObject::Sprite2DState::GetUVAnimation (
+iSprite2DUVAnimation *csSprite2DMeshObject::GetUVAnimation (
 	int idx, int &style, bool &loop) const
 {
-  style = scfParent->uvani->style;
-  loop = scfParent->uvani->loop;
-  return scfParent->factory->GetUVAnimation (idx);
+  style = uvani->style;
+  loop = uvani->loop;
+  return factory->GetUVAnimation (idx);
 }
 
 
@@ -714,7 +677,7 @@ bool csSprite2DMeshObject::HitBeamOutline(const csVector3& start,
   csMatrix3 o2t;
   CheckBeam (start, pl, sqr, o2t);
   csVector3 r = o2t * isect;
-  int trail, len = (int)vertices.Length();
+  int trail, len = (int)vertices.GetSize ();
   trail = len - 1;
   csVector2 isec(r.x, r.y);
   int i;
@@ -726,79 +689,49 @@ bool csSprite2DMeshObject::HitBeamOutline(const csVector3& start,
 
 //----------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csSprite2DMeshObjectFactory)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObjectFactory)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iSprite2DFactoryState)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObjectFactory::Sprite2DFactoryState)
-  SCF_IMPLEMENTS_INTERFACE (iSprite2DFactoryState)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iMeshObjectType* pParent,
-	iObjectRegistry* object_reg)
+  iObjectRegistry* object_reg) : scfImplementationType (this, pParent),
+  material (0), logparent (0), spr2d_type (pParent), MixMode (0), 
+  lighting (true), object_reg (object_reg)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiSprite2DFactoryState);
-  material = 0;
-  MixMode = 0;
-  lighting = true;
-  logparent = 0;
-  spr2d_type = pParent;
-  csSprite2DMeshObjectFactory::object_reg = object_reg;
-  light_mgr = CS_QUERY_REGISTRY (object_reg, iLightManager);
-  g3d = CS_QUERY_REGISTRY (object_reg, iGraphics3D);
-  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg,
-    "crystalspace.shared.stringset", iStringSet);
+  light_mgr = csQueryRegistry<iLightManager> (object_reg);
+  g3d = csQueryRegistry<iGraphics3D> (object_reg);
+  csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+    object_reg, "crystalspace.shared.stringset");
 }
 
 csSprite2DMeshObjectFactory::~csSprite2DMeshObjectFactory ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiSprite2DFactoryState);
-  SCF_DESTRUCT_IBASE ();
 }
 
 csPtr<iMeshObject> csSprite2DMeshObjectFactory::NewInstance ()
 {
-  csSprite2DMeshObject* cm = new csSprite2DMeshObject (this);
-  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (cm, iMeshObject));
-  cm->DecRef ();
+  csRef<csSprite2DMeshObject> cm;
+  cm.AttachNew (new csSprite2DMeshObject (this));
+  csRef<iMeshObject> im (scfQueryInterface<iMeshObject> (cm));
   return csPtr<iMeshObject> (im);
 }
 
 //----------------------------------------------------------------------
 
-SCF_IMPLEMENT_IBASE (csSprite2DMeshObjectType)
-  SCF_IMPLEMENTS_INTERFACE (iMeshObjectType)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DMeshObjectType::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csSprite2DMeshObjectType)
 
-
-csSprite2DMeshObjectType::csSprite2DMeshObjectType (iBase* pParent)
+csSprite2DMeshObjectType::csSprite2DMeshObjectType (iBase* pParent) :
+  scfImplementationType (this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite2DMeshObjectType::~csSprite2DMeshObjectType ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 csPtr<iMeshObjectFactory> csSprite2DMeshObjectType::NewFactory ()
 {
-  csSprite2DMeshObjectFactory* cm = new csSprite2DMeshObjectFactory (this,
-  	object_reg);
+  csRef<csSprite2DMeshObjectFactory> cm;
+  cm.AttachNew (new csSprite2DMeshObjectFactory (this,
+  	object_reg));
   csRef<iMeshObjectFactory> ifact =
-  	SCF_QUERY_INTERFACE (cm, iMeshObjectFactory);
-  cm->DecRef ();
+  	scfQueryInterface<iMeshObjectFactory> (cm);
   return csPtr<iMeshObjectFactory> (ifact);
 }
 
@@ -807,3 +740,6 @@ bool csSprite2DMeshObjectType::Initialize (iObjectRegistry* object_reg)
   csSprite2DMeshObjectType::object_reg = object_reg;
   return true;
 }
+
+}
+CS_PLUGIN_NAMESPACE_END(Spr2D)
