@@ -59,65 +59,25 @@ enum
   XMLTOKEN_ANIMATE
 };
 
-SCF_IMPLEMENT_IBASE (csSprite2DFactoryLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DFactoryLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSprite2DFactorySaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DFactorySaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSprite2DLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
-SCF_IMPLEMENT_IBASE (csSprite2DSaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSprite2DSaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csSprite2DFactoryLoader)
 SCF_IMPLEMENT_FACTORY (csSprite2DFactorySaver)
 SCF_IMPLEMENT_FACTORY (csSprite2DLoader)
 SCF_IMPLEMENT_FACTORY (csSprite2DSaver)
 
-
-csSprite2DFactoryLoader::csSprite2DFactoryLoader (iBase* pParent)
+csSprite2DFactoryLoader::csSprite2DFactoryLoader (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite2DFactoryLoader::~csSprite2DFactoryLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite2DFactoryLoader::Initialize (iObjectRegistry* object_reg)
 {
   csSprite2DFactoryLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
   xmltokens.Register ("lighting", XMLTOKEN_LIGHTING);
   xmltokens.Register ("material", XMLTOKEN_MATERIAL);
@@ -201,15 +161,8 @@ bool csSprite2DFactoryLoader::ParseAnim (iDocumentNode* node,
 csPtr<iBase> csSprite2DFactoryLoader::Parse (iDocumentNode* node,
 	iStreamSource*, iLoaderContext* ldr_context, iBase* /* context */)
 {
-  csRef<iPluginManager> plugin_mgr (CS_QUERY_REGISTRY (object_reg,
-  	iPluginManager));
-  csRef<iMeshObjectType> type (CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-  	"crystalspace.mesh.object.sprite.2d", iMeshObjectType));
-  if (!type)
-  {
-    type = CS_LOAD_PLUGIN (plugin_mgr, "crystalspace.mesh.object.sprite.2d",
-    	iMeshObjectType);
-  }
+  csRef<iMeshObjectType> type = csLoadPluginCheck<iMeshObjectType> (
+  	object_reg, "crystalspace.mesh.object.sprite.2d", false);
   if (!type)
   {
     synldr->ReportError (
@@ -219,7 +172,7 @@ csPtr<iBase> csSprite2DFactoryLoader::Parse (iDocumentNode* node,
   }
   csRef<iMeshObjectFactory> fact (type->NewFactory ());
   csRef<iSprite2DFactoryState> spr2dLook (
-  	SCF_QUERY_INTERFACE (fact, iSprite2DFactoryState));
+  	scfQueryInterface<iSprite2DFactoryState> (fact));
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -241,7 +194,7 @@ csPtr<iBase> csSprite2DFactoryLoader::Parse (iDocumentNode* node,
 		child, "Couldn't find material named '%s'", matname);
             return 0;
 	  }
-	  spr2dLook->SetMaterialWrapper (mat);
+	  fact->SetMaterialWrapper (mat);
 	}
 	break;
       case XMLTOKEN_LIGHTING:
@@ -257,7 +210,7 @@ csPtr<iBase> csSprite2DFactoryLoader::Parse (iDocumentNode* node,
 	  uint mm;
 	  if (!synldr->ParseMixmode (child, mm))
 	    return 0;
-          spr2dLook->SetMixMode (mm);
+          fact->SetMixMode (mm);
 	}
 	break;
       case XMLTOKEN_UVANIMATION:
@@ -275,23 +228,20 @@ csPtr<iBase> csSprite2DFactoryLoader::Parse (iDocumentNode* node,
 }
 
 //---------------------------------------------------------------------------
-csSprite2DFactorySaver::csSprite2DFactorySaver (iBase* pParent)
+csSprite2DFactorySaver::csSprite2DFactorySaver (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite2DFactorySaver::~csSprite2DFactorySaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite2DFactorySaver::Initialize (iObjectRegistry* object_reg)
 {
   csSprite2DFactorySaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
 
@@ -304,13 +254,13 @@ bool csSprite2DFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
   csRef<iDocumentNode> paramsNode = parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
   paramsNode->SetValue("params");
 
-  csRef<iSprite2DFactoryState> spritefact = SCF_QUERY_INTERFACE (obj, iSprite2DFactoryState);
-  csRef<iMeshObjectFactory> meshfact = SCF_QUERY_INTERFACE (obj, iMeshObjectFactory);
+  csRef<iSprite2DFactoryState> spritefact = scfQueryInterface<iSprite2DFactoryState> (obj);
+  csRef<iMeshObjectFactory> meshfact = scfQueryInterface<iMeshObjectFactory> (obj);
   if (!spritefact) return false;
   if (!meshfact) return false;
 
   //Writedown Material tag
-  iMaterialWrapper* mat = spritefact->GetMaterialWrapper();
+  iMaterialWrapper* mat = meshfact->GetMaterialWrapper();
   if (mat)
   {
     const char* matname = mat->QueryObject()->GetName();
@@ -354,7 +304,7 @@ bool csSprite2DFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
   }
 
   //Writedown Mixmode tag
-  int mixmode = spritefact->GetMixMode();
+  int mixmode = meshfact->GetMixMode();
   csRef<iDocumentNode> mixmodeNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
   mixmodeNode->SetValue("mixmode");
   synldr->WriteMixmode(mixmodeNode, mixmode, true);
@@ -363,23 +313,20 @@ bool csSprite2DFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
 }
 //---------------------------------------------------------------------------
 
-csSprite2DLoader::csSprite2DLoader (iBase* pParent)
+csSprite2DLoader::csSprite2DLoader (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite2DLoader::~csSprite2DLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite2DLoader::Initialize (iObjectRegistry* object_reg)
 {
   csSprite2DLoader::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
 
   xmltokens.Register ("color", XMLTOKEN_COLOR);
   xmltokens.Register ("factory", XMLTOKEN_FACTORY);
@@ -392,13 +339,22 @@ bool csSprite2DLoader::Initialize (iObjectRegistry* object_reg)
   return true;
 }
 
+#define CHECK_MESH(m) \
+  if (!m) { \
+    synldr->ReportError ( \
+	"crystalspace.sprite2dloader.parse.unknownfactory", \
+	child, "Specify the factory first!"); \
+    return 0; \
+  }
+
+
 csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 				iStreamSource*, iLoaderContext* ldr_context,
 				iBase*)
 {
   csRef<iMeshObject> mesh;
   csRef<iSprite2DState> spr2dLook;
-  csColoredVertices* verts = 0;
+  iColoredVertices* verts = 0;
   int vnum = 0;
   int uvnum = 0;
   int colnum = 0;
@@ -424,7 +380,7 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 	    return 0;
 	  }
 	  mesh = fact->GetMeshObjectFactory ()->NewInstance ();
-          spr2dLook = SCF_QUERY_INTERFACE (mesh, iSprite2DState);
+          spr2dLook = scfQueryInterface<iSprite2DState> (mesh);
 	  if (!spr2dLook)
 	  {
       	    synldr->ReportError (
@@ -433,7 +389,7 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 		factname);
 	    return 0;
 	  }
-	  verts = &(spr2dLook->GetVertices ());
+	  verts = spr2dLook->GetVertices ();
 	}
 	break;
       case XMLTOKEN_MATERIAL:
@@ -447,7 +403,8 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 		child, "Couldn't find material '%s'!", matname);
             return 0;
 	  }
-	  spr2dLook->SetMaterialWrapper (mat);
+	  CHECK_MESH (mesh);
+	  mesh->SetMaterialWrapper (mat);
 	}
 	break;
       case XMLTOKEN_MIXMODE:
@@ -455,7 +412,8 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 	  uint mm;
 	  if (!synldr->ParseMixmode (child, mm))
 	    return 0;
-          spr2dLook->SetMixMode (mm);
+	  CHECK_MESH (mesh);
+          mesh->SetMixMode (mm);
 	}
 	break;
       case XMLTOKEN_V:
@@ -464,11 +422,12 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 	  float y = child->GetAttributeValueAsFloat ("y");
 	  vnum++;
 	  if (vnum > uvnum && vnum > colnum)
-	    verts->SetLength (vnum);
-	  (*verts)[vnum-1].pos.x = x;
-	  (*verts)[vnum-1].pos.y = y;
-	  (*verts)[vnum-1].color_init.Set (0, 0, 0);
-	  (*verts)[vnum-1].color.Set (0, 0, 0);
+	    verts->SetSize (vnum);
+          csSprite2DVertex& v = verts->Get (vnum-1);
+	  v.pos.x = x;
+	  v.pos.y = y;
+	  v.color_init.Set (0, 0, 0);
+	  v.color.Set (0, 0, 0);
         }
         break;
       case XMLTOKEN_UV:
@@ -477,9 +436,10 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 	  float v = child->GetAttributeValueAsFloat ("v");
 	  uvnum++;
 	  if (uvnum > vnum && uvnum > colnum)
-	    verts->SetLength (uvnum);
-	  (*verts)[uvnum-1].u = u;
-	  (*verts)[uvnum-1].v = v;
+	    verts->SetSize (uvnum);
+          csSprite2DVertex& V = verts->Get (uvnum-1);
+	  V.u = u;
+	  V.v = v;
         }
         break;
       case XMLTOKEN_COLOR:
@@ -489,10 +449,11 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 	  float b = child->GetAttributeValueAsFloat ("blue");
 	  colnum++;
 	  if (colnum > vnum && colnum > uvnum)
-	    verts->SetLength (colnum);
-	  (*verts)[colnum-1].color_init.red = r;
-	  (*verts)[colnum-1].color_init.green = g;
-	  (*verts)[colnum-1].color_init.blue = b;
+	    verts->SetSize (colnum);
+          csSprite2DVertex& v = verts->Get (colnum-1);
+	  v.color_init.red = r;
+	  v.color_init.green = g;
+	  v.color_init.blue = b;
         }
         break;
       case XMLTOKEN_LIGHTING:
@@ -500,11 +461,13 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
           bool do_lighting;
 	  if (!synldr->ParseBool (child, do_lighting, true))
 	    return 0;
+	  CHECK_MESH (spr2dLook);
           spr2dLook->SetLighting (do_lighting);
         }
         break;
       case XMLTOKEN_ANIMATE:
         {
+	  CHECK_MESH (spr2dLook);
           bool loop = false;
 	  int timing = 0;
 	  const char* animname = child->GetAttributeValue ("name");
@@ -535,23 +498,20 @@ csPtr<iBase> csSprite2DLoader::Parse (iDocumentNode* node,
 
 //---------------------------------------------------------------------------
 
-csSprite2DSaver::csSprite2DSaver (iBase* pParent)
+csSprite2DSaver::csSprite2DSaver (iBase* pParent) :
+  scfImplementationType(this, pParent)
 {
-  SCF_CONSTRUCT_IBASE (pParent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
 }
 
 csSprite2DSaver::~csSprite2DSaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE(scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSprite2DSaver::Initialize (iObjectRegistry* object_reg)
 {
   csSprite2DSaver::object_reg = object_reg;
-  reporter = CS_QUERY_REGISTRY (object_reg, iReporter);
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
+  reporter = csQueryRegistry<iReporter> (object_reg);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
 
@@ -564,8 +524,8 @@ bool csSprite2DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
   csRef<iDocumentNode> paramsNode = parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
   paramsNode->SetValue("params");
 
-  csRef<iSprite2DState> sprite = SCF_QUERY_INTERFACE (obj, iSprite2DState);
-  csRef<iMeshObject> mesh = SCF_QUERY_INTERFACE (obj, iMeshObject);
+  csRef<iSprite2DState> sprite = scfQueryInterface<iSprite2DState> (obj);
+  csRef<iMeshObject> mesh = scfQueryInterface<iMeshObject> (obj);
 
   if (!sprite) return false;
   if (!mesh) return false;
@@ -584,23 +544,21 @@ bool csSprite2DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
   }
 
   //Writedown vertex tag
-  csColoredVertices vertex = sprite->GetVertices();
-  csColoredVertices::Iterator iter = vertex.GetIterator();
-  while (iter.HasNext())
+  iColoredVertices* vertices = sprite->GetVertices();
+  for (size_t i = 0; i < vertices->GetSize(); i++)
   {
-    csSprite2DVertex vertex = iter.Next();
+    const csSprite2DVertex& vertex = vertices->Get (i);
 
     csRef<iDocumentNode> vNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
     vNode->SetValue("v");
     vNode->SetAttributeAsFloat("x", vertex.pos.x);
-    vNode->SetAttributeAsFloat("x", vertex.pos.y);
+    vNode->SetAttributeAsFloat("y", vertex.pos.y);
   }
 
   //Writedown uv tag
-  iter.Reset();
-  while (iter.HasNext())
+  for (size_t i = 0; i < vertices->GetSize(); i++)
   {
-    csSprite2DVertex vertex = iter.Next();
+    const csSprite2DVertex& vertex = vertices->Get (i);
 
     csRef<iDocumentNode> uvNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
     uvNode->SetValue("uv");
@@ -610,7 +568,7 @@ bool csSprite2DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
   }
 
   //Writedown Material tag
-  iMaterialWrapper* mat = sprite->GetMaterialWrapper();
+  iMaterialWrapper* mat = mesh->GetMaterialWrapper();
   if (mat)
   {
     const char* matname = mat->QueryObject()->GetName();
@@ -624,10 +582,9 @@ bool csSprite2DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
   }    
 
   //Writedown color tag
-  iter.Reset();
-  while (iter.HasNext())
+  for (size_t i = 0; i < vertices->GetSize(); i++)
   {
-    csSprite2DVertex vertex = iter.Next();
+    const csSprite2DVertex& vertex = vertices->Get (i);
 
     csRef<iDocumentNode> colorNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
     colorNode->SetValue("color");
@@ -651,12 +608,14 @@ bool csSprite2DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
     const char* animname = anim->GetName();
     uvaniNode->SetAttribute("name", animname);
     synldr->WriteBool(uvaniNode,"loop",loop,false);
-    csRef<iDocumentNode> styleNode = styleNode->GetNode ("style");
-    styleNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValueAsInt(style);
+    
+    csRef<iDocumentNode> styleNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    styleNode->SetValue ("style");
+    styleNode->CreateNodeBefore (CS_NODE_TEXT, 0)->SetValueAsInt(style);
   }
 
   //Writedown Mixmode tag
-  int mixmode = sprite->GetMixMode();
+  int mixmode = mesh->GetMixMode();
   csRef<iDocumentNode> mixmodeNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
   mixmodeNode->SetValue("mixmode");
   synldr->WriteMixmode(mixmodeNode, mixmode, true);
