@@ -26,6 +26,7 @@
 #include "csgeom/polyclip.h"
 #include "csgeom/sphere.h"
 #include "cstool/rbuflock.h"
+#include "cstool/rviewclipper.h"
 #include "csutil/csendian.h"
 #include "csutil/csmd5.h"
 #include "csutil/dirtyaccessarray.h"
@@ -52,7 +53,7 @@
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginSpr3d
+CS_PLUGIN_NAMESPACE_BEGIN(Spr3d)
 {
 
 CS_IMPLEMENT_STATIC_CLASSVAR (csSprite3DMeshObject, mesh, GetLODMesh, csTriangleMesh, ())
@@ -139,7 +140,7 @@ void csSprite3DMeshObjectFactory::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (object_reg, iReporter));
+  csRef<iReporter> rep (csQueryRegistry<iReporter> (object_reg));
   if (rep)
     rep->ReportV (severity, "crystalspace.mesh.sprite.3d", msg, arg);
   else
@@ -161,6 +162,11 @@ csSprite3DMeshObjectFactory::csSprite3DMeshObjectFactory (
   SetPolygonMeshColldet (pm);
   SetPolygonMeshViscull (0);
   SetPolygonMeshShadows (0);
+  csSprite3DMeshObjectType* type = static_cast<csSprite3DMeshObjectType*> (
+      pParent);
+  csRef<TriMesh> trimesh;
+  trimesh.AttachNew (new TriMesh (this));
+  SetTriangleData (type->base_id, trimesh);
 
   logparent = 0;
   spr3d_type = pParent;
@@ -197,7 +203,7 @@ void csSprite3DMeshObjectFactory::AddVertices (int num)
   size_t frame;
 
   int oldvt = GetVertexCount ();
-  for (frame = 0; frame < frames.Length(); frame++)
+  for (frame = 0; frame < frames.GetSize (); frame++)
   {
     normals.Get (frame)->SetVertexCount (oldvt + num);
     memset (normals.Get (frame)->GetVertices () + oldvt, 0,
@@ -213,7 +219,7 @@ void csSprite3DMeshObjectFactory::AddTriangle (int a, int b, int c)
   texel_mesh->AddTriangle (a, b, c);
 }
 
-csPtr<iMeshObject> csSprite3DMeshObjectFactory::NewInstance ()
+void csSprite3DMeshObjectFactory::SetupFactory ()
 {
   if (!initialized)
   {
@@ -221,7 +227,11 @@ csPtr<iMeshObject> csSprite3DMeshObjectFactory::NewInstance ()
     GenerateLOD ();
     ComputeBoundingBox ();
   }
+}
 
+csPtr<iMeshObject> csSprite3DMeshObjectFactory::NewInstance ()
+{
+  SetupFactory ();
   csSprite3DMeshObject* spr = new csSprite3DMeshObject ();
   spr->SetFactory (this);
   spr->EnableTweening (do_tweening);
@@ -230,7 +240,7 @@ csPtr<iMeshObject> csSprite3DMeshObjectFactory::NewInstance ()
   spr->SetLightingQualityConfig (GetLightingQualityConfig());
   spr->SetAction ("default");
   spr->InitSprite ();
-  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (spr, iMeshObject));
+  csRef<iMeshObject> im (scfQueryInterface<iMeshObject> (spr));
   spr->DecRef ();
   return csPtr<iMeshObject> (im);
 }
@@ -263,7 +273,7 @@ void csSprite3DMeshObjectFactory::GenerateLOD ()
   csVector2* new_texels = new csVector2 [GetVertexCount ()];
   csVector3* new_vertices = new csVector3 [GetVertexCount ()];
   csVector3* new_normals = new csVector3 [GetVertexCount ()];
-  for (i = 0 ; i < (int)texels.Length () ; i++)
+  for (i = 0 ; i < (int)texels.GetSize () ; i++)
   {
     int j;
     csPoly2D* tx = texels.Get (i);
@@ -307,36 +317,32 @@ void csSprite3DMeshObjectFactory::ComputeBoundingBox ()
 
     const csVector3& v0 = GetVertex (frame, 0);
     box.StartBoundingBox (v0);
-    csVector3 max_sq_radius (v0.x*v0.x + v0.x*v0.x,
-    	v0.y*v0.y + v0.y*v0.y, v0.z*v0.z + v0.z*v0.z);
     for ( vertex = 1 ; vertex < GetVertexCount() ; vertex++ )
     {
       const csVector3& v = GetVertex (frame, vertex);
       box.AddBoundingVertexSmart (v);
-      csVector3 sq_radius (v.x*v.x + v.x*v.x, v.y*v.y + v.y*v.y,
-      	v.z*v.z + v.z*v.z);
-      if (sq_radius.x > max_sq_radius.x) max_sq_radius.x = sq_radius.x;
-      if (sq_radius.y > max_sq_radius.y) max_sq_radius.y = sq_radius.y;
-      if (sq_radius.z > max_sq_radius.z) max_sq_radius.z = sq_radius.z;
     }
+    float radius = csQsqrt (
+    	csSquaredDist::PointPoint (box.Max (), box.Min ()));
 
     ((csSpriteFrame*)GetFrame (frame))->SetBoundingBox (box);
-    ((csSpriteFrame*)GetFrame (frame))->SetRadius (csVector3 (
-    	csQsqrt (max_sq_radius.x),
-	csQsqrt (max_sq_radius.y),
-	csQsqrt (max_sq_radius.z)));
+    ((csSpriteFrame*)GetFrame (frame))->SetRadius (radius);
+    if (frame == 0)
+      global_bbox = box;
+    else
+      global_bbox += box;
   }
 }
 
 iSpriteFrame* csSprite3DMeshObjectFactory::AddFrame ()
 {
-  csSpriteFrame* fr = new csSpriteFrame ((int)frames.Length(), 
-    (int)texels.Length());
+  csSpriteFrame* fr = new csSpriteFrame ((int)frames.GetSize (), 
+    (int)texels.GetSize ());
   csPoly3D* nr = new csPoly3D ();
   csPoly2D* tx = new csPoly2D ();
   csPoly3D* vr = new csPoly3D ();
 
-  if (frames.Length() > 0)
+  if (frames.GetSize () > 0)
   {
     nr->SetVertexCount (GetVertexCount ());
     tx->SetVertexCount (GetVertexCount ());
@@ -394,9 +400,12 @@ iSpriteSocket* csSprite3DMeshObjectFactory::FindSocket (iMeshWrapper *mesh) cons
 
   return 0;
 }
-void csSprite3DMeshObjectFactory::SetMaterialWrapper (iMaterialWrapper *material)
+
+bool csSprite3DMeshObjectFactory::SetMaterialWrapper (
+	iMaterialWrapper *material)
 {
   cstxt = material;
+  return true;
 }
 
 void csSprite3DMeshObjectFactory::ComputeNormals (csSpriteFrame* frame)
@@ -441,13 +450,13 @@ void csSprite3DMeshObjectFactory::ComputeNormals (csSpriteFrame* frame)
   for (i = 0; i < GetVertexCount(); i++)
   {
     csTriangleVertexCost &vt = tri_verts->GetVertex (i);
-    if (vt.con_triangles.Length ())
+    if (vt.con_triangles.GetSize ())
     {
-      csVector3 &n = CS_CONST_CAST (csVector3&, GetNormal (frame_number, i));
+      csVector3 &n = const_cast<csVector3&> (GetNormal (frame_number, i));
       if (n.IsZero())
       {
         n.Set (0,0,0);
-        for (j = 0; j < vt.con_triangles.Length (); j++)
+        for (j = 0; j < vt.con_triangles.GetSize (); j++)
           n += tri_normals [vt.con_triangles[j]];
         float norm = n.Norm ();
         if (norm)
@@ -458,7 +467,7 @@ void csSprite3DMeshObjectFactory::ComputeNormals (csSpriteFrame* frame)
     {
       // If there are no connecting triangles then we just
       // initialize the normal to a default value.
-      CS_CONST_CAST (csVector3&, 
+      const_cast<csVector3&> (
 	GetNormal (frame_number, i)).Set (1, 0, 0);
     }
   }
@@ -560,11 +569,11 @@ void csSprite3DMeshObjectFactory::MergeNormals (int base, int frame)
   for (i = 0; i < GetVertexCount(); i++)
   {
     csTriangleVertexCost &vt = tv->GetVertex (i);
-    if (vt.con_triangles.Length ())
+    if (vt.con_triangles.GetSize ())
     {
       csVector3 &n = fr_normals[i];
       n.Set (tri_normals[vt.con_triangles[0]]);
-      for (j = 1; j < vt.con_triangles.Length (); j++)
+      for (j = 1; j < vt.con_triangles.GetSize (); j++)
         n += tri_normals [vt.con_triangles[j]];
       float norm = n.Norm ();
       if (norm)
@@ -640,49 +649,47 @@ void csSprite3DMeshObjectFactory::HardTransform (const csReversibleTransform& t)
     verts[0] = t.This2Other (verts[0]);
     csVector3& v0 = verts[0];
     box.StartBoundingBox (v0);
-    csVector3 max_sq_radius (v0.x*v0.x + v0.x*v0.x,
-    	v0.y*v0.y + v0.y*v0.y, v0.z*v0.z + v0.z*v0.z);
     for (j = 1 ; j < num ; j++)
     {
       csVector3& v = verts[j];
       v = t.This2Other (v);
       box.AddBoundingVertexSmart (v);
-      csVector3 sq_radius (v.x*v.x + v.x*v.x, v.y*v.y + v.y*v.y,
-		v.z*v.z + v.z*v.z);
-      if (sq_radius.x > max_sq_radius.x) max_sq_radius.x = sq_radius.x;
-      if (sq_radius.y > max_sq_radius.y) max_sq_radius.y = sq_radius.y;
-      if (sq_radius.z > max_sq_radius.z) max_sq_radius.z = sq_radius.z;
     }
+    float radius = csQsqrt (
+    	csSquaredDist::PointPoint (box.Max (), box.Min ()));
     ((csSpriteFrame*)GetFrame (i))->SetBoundingBox (box);
-    ((csSpriteFrame*)GetFrame (i))->SetRadius (csVector3 (
-    	csQsqrt (max_sq_radius.x),
-	csQsqrt (max_sq_radius.y),
-	csQsqrt (max_sq_radius.z)));
-
+    ((csSpriteFrame*)GetFrame (i))->SetRadius (radius);
   }
   ShapeChanged ();
 }
 
 void csSprite3DMeshObjectFactory::GetObjectBoundingBox (csBox3& b)
 {
-  csSpriteFrame* cframe = ((csSpriteAction2*)GetAction (0))->GetCsFrame (0);
-  cframe->GetBoundingBox (b);
+  SetupFactory ();
+  b = global_bbox;
 }
 
-void csSprite3DMeshObjectFactory::SetObjectBoundingBox (const csBox3&)
+const csBox3& csSprite3DMeshObjectFactory::GetObjectBoundingBox ()
 {
-  // @@@ TODO
+  SetupFactory ();
+  return global_bbox;
 }
 
-void csSprite3DMeshObjectFactory::GetRadius (csVector3& rad, csVector3& cent)
+void csSprite3DMeshObjectFactory::SetObjectBoundingBox (const csBox3& b)
 {
-  csVector3 r; csBox3 bbox;
-  GetObjectBoundingBox (bbox);
+  SetupFactory ();
+  global_bbox = b;
+}
+
+void csSprite3DMeshObjectFactory::GetRadius (float& rad, csVector3& cent)
+{
+  float r;
+  const csBox3& bbox = GetObjectBoundingBox ();
   cent = bbox.GetCenter();
 
   csSpriteFrame* cframe = ((csSpriteAction2*)GetAction (0))->GetCsFrame (0);
   cframe->GetRadius (r);
-  rad =  r;
+  rad = r;
 }
 
 csMeshedPolygon* csSprite3DMeshObjectFactory::PolyMesh::GetPolygons ()
@@ -705,16 +712,16 @@ csMeshedPolygon* csSprite3DMeshObjectFactory::PolyMesh::GetPolygons ()
 //=============================================================================
 
 /// Static vertex array.
-typedef csDirtyAccessArray<csVector3> spr3d_tr_verts;
+typedef csDirtyAccessArrayRefCounted<csVector3> spr3d_tr_verts;
 CS_IMPLEMENT_STATIC_VAR (Get_tr_verts, spr3d_tr_verts, ())
 /// Static uv array.
-typedef csDirtyAccessArray<csVector2> spr3d_uv_verts;
+typedef csDirtyAccessArrayRefCounted<csVector2> spr3d_uv_verts;
 CS_IMPLEMENT_STATIC_VAR (Get_uv_verts, spr3d_uv_verts, ())
 /// The list of object vertices.
-typedef csDirtyAccessArray<csVector3> spr3d_obj_verts;
+typedef csDirtyAccessArrayRefCounted<csVector3> spr3d_obj_verts;
 CS_IMPLEMENT_STATIC_VAR (Get_obj_verts, spr3d_obj_verts, ())
 /// The list of tween vertices.
-typedef csDirtyAccessArray<csVector3> spr3d_tween_verts;
+typedef csDirtyAccessArrayRefCounted<csVector3> spr3d_tween_verts;
 CS_IMPLEMENT_STATIC_VAR (Get_tween_verts, spr3d_tween_verts, ())
 
 spr3d_tr_verts *tr_verts = 0;
@@ -930,12 +937,12 @@ void csSprite3DMeshObject::GenerateSpriteLOD (int num_vts)
 
 void csSprite3DMeshObject::UpdateWorkTables (int max_size)
 {
-  if ((size_t)max_size > tr_verts->Length ())
+  if ((size_t)max_size > tr_verts->GetSize ())
   {
-    tr_verts->SetLength (max_size);
-    uv_verts->SetLength (max_size);
-    obj_verts->SetLength (max_size);
-    tween_verts->SetLength (max_size);
+    tr_verts->SetSize (max_size);
+    uv_verts->SetSize (max_size);
+    obj_verts->SetSize (max_size);
+    tween_verts->SetSize (max_size);
   }
 }
 
@@ -953,8 +960,7 @@ void csSprite3DMeshObject::GetTransformedBoundingBox (
 
   CS_ASSERT (cur_action != 0);
   csSpriteFrame* cframe = cur_action->GetCsFrame (cur_frame);
-  csBox3 box;
-  cframe->GetBoundingBox (box);
+  const csBox3& box = cframe->GetBoundingBox ();
   camera_bbox.StartBoundingBox (trans * box.GetCorner (0));
   camera_bbox.AddBoundingVertexSmart (trans * box.GetCorner (1));
   camera_bbox.AddBoundingVertexSmart (trans * box.GetCorner (2));
@@ -1020,15 +1026,22 @@ void csSprite3DMeshObject::GetObjectBoundingBox (csBox3& b)
   cframe->GetBoundingBox (b);
 }
 
+const csBox3& csSprite3DMeshObject::GetObjectBoundingBox ()
+{
+  CS_ASSERT (cur_action != 0);
+  csSpriteFrame* cframe = cur_action->GetCsFrame (cur_frame);
+  return cframe->GetBoundingBox ();
+}
+
 void csSprite3DMeshObject::SetObjectBoundingBox (const csBox3&)
 {
   // @@@ TODO
 }
 
-void csSprite3DMeshObject::GetRadius (csVector3& rad, csVector3& cent)
+void csSprite3DMeshObject::GetRadius (float& rad, csVector3& cent)
 {
-  csVector3 r; csBox3 bbox;
-  GetObjectBoundingBox(bbox);
+  float r;
+  const csBox3& bbox = GetObjectBoundingBox();
   cent = bbox.GetCenter();
 
   CS_ASSERT (cur_action != 0);
@@ -1103,8 +1116,8 @@ csRenderMesh** csSprite3DMeshObject::GetRenderMeshes (int& n,
   if (!movable->IsFullTransformIdentity ())
     tr_o2c /= movable->GetFullTransform ();
   int clip_portal, clip_plane, clip_z_plane;
-  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
-  	clip_z_plane);
+  CS::RenderViewClipper::CalculateClipSettings (rview->GetRenderContext (),
+      frustum_mask, clip_portal, clip_plane, clip_z_plane);
 
   bool rmCreated;
   csRenderMesh*& rmesh = rmHolder.GetUnusedMesh (rmCreated,
@@ -1149,8 +1162,7 @@ csRenderMesh** csSprite3DMeshObject::GetRenderMeshes (int& n,
   if (IsLodEnabled ())
   {
     // reduce LOD based on distance from camera to center of sprite
-    csBox3 obox;
-    GetObjectBoundingBox (obox);
+    const csBox3& obox = GetObjectBoundingBox ();
     csVector3 obj_center = (obox.Min () + obox.Max ()) / 2;
     csVector3 wor_center;
     if (movable->IsFullTransformIdentity ())
@@ -1620,8 +1632,7 @@ void csSprite3DMeshObject::UpdateLightingFast (
   // convert frame number in current action to absolute frame number
   int tf_idx = cur_action->GetCsFrame (cur_frame)->GetAnmIndex();
 
-  csBox3 obox;
-  GetObjectBoundingBox (obox);
+  const csBox3& obox = GetObjectBoundingBox ();
   csVector3 obj_center = (obox.Min () + obox.Max ()) / 2;
   csVector3 wor_center;
   csReversibleTransform movtrans;
@@ -1654,7 +1665,7 @@ void csSprite3DMeshObject::UpdateLightingFast (
   float amb_b = b / 128.0;
 #endif
 
-  int num_lights = (int)lights.Length ();
+  int num_lights = (int)lights.GetSize ();
   for (light_num = 0 ; light_num < num_lights ; light_num++)
   {
     iLight* li = lights[light_num]->GetLight ();
@@ -1746,8 +1757,7 @@ void csSprite3DMeshObject::UpdateLightingLQ (
   int tf_idx = cur_action->GetCsFrame     (cur_frame)->GetAnmIndex ();
   int nf_idx = cur_action->GetCsNextFrame (cur_frame)->GetAnmIndex ();
 
-  csBox3 obox;
-  GetObjectBoundingBox (obox);
+  const csBox3& obox = GetObjectBoundingBox ();
   csVector3 obj_center = (obox.Min () + obox.Max ()) / 2;
   csVector3 wor_center;
   csReversibleTransform movtrans;
@@ -1761,7 +1771,7 @@ void csSprite3DMeshObject::UpdateLightingLQ (
   }
   csColor color;
 
-  int num_lights = (int)lights.Length ();
+  int num_lights = (int)lights.GetSize ();
   for (i = 0 ; i < num_lights ; i++)
   {
     iLight* li = lights[i]->GetLight ();
@@ -1847,7 +1857,7 @@ void csSprite3DMeshObject::UpdateLightingHQ (
   csColor color;
 
   csReversibleTransform movtrans = movable->GetFullTransform ();
-  int num_lights = (int)lights.Length ();
+  int num_lights = (int)lights.GetSize ();
   for (i = 0 ; i < num_lights ; i++)
   {
     iLight* li = lights[i]->GetLight ();
@@ -1965,7 +1975,7 @@ void csSprite3DMeshObject::PositionChild (iMeshObject* child,
 {
   iSpriteSocket* socket = 0;
   size_t i;
-  for(i=0;i<sockets.Length();i++)
+  for(i=0;i<sockets.GetSize ();i++)
   {
     if(sockets[i]->GetMeshWrapper())
     {
@@ -2180,6 +2190,9 @@ csSprite3DMeshObjectType::~csSprite3DMeshObjectType ()
 
 bool csSprite3DMeshObjectType::Initialize (iObjectRegistry* object_reg)
 {
+  csRef<iStringSet> strset = csQueryRegistryTagInterface<iStringSet> (
+      object_reg, "crystalspace.shared.stringset");
+  base_id = strset->Request ("base");
   csSprite3DMeshObjectType::object_reg = object_reg;
   vc = csQueryRegistry<iVirtualClock> (object_reg);
   csRef<iEngine> eng = csQueryRegistry<iEngine> (object_reg);
@@ -2247,4 +2260,5 @@ bool csSprite3DMeshObjectType::GetOptionDescription (int idx,
   return true;
 }
 
-} // namespace cspluginSpr3d
+}
+CS_PLUGIN_NAMESPACE_END(Spr3d)
