@@ -22,10 +22,11 @@
 #include "csgeom/box.h"
 #include "csgeom/math2d.h"
 #include "csgeom/math3d.h"
-#include "csgeom/objmodel.h"
+#include "cstool/objmodel.h"
 #include "csgeom/poly2d.h"
 #include "csgeom/poly3d.h"
 #include "csgeom/trimeshlod.h"
+#include "igeom/trimesh.h"
 #include "csgfx/shadervarcontext.h"
 #include "csqint.h"
 #include "cstool/rendermeshholder.h"
@@ -58,7 +59,7 @@
 struct iObjectRegistry;
 struct iEngine;
 
-namespace cspluginSpr3d
+CS_PLUGIN_NAMESPACE_BEGIN(Spr3d)
 {
 
 /**
@@ -103,7 +104,7 @@ private:
   /// Bounding box in object space for this frame.
   csBox3 box;
   /// Radius in object space for this frame.
-  csVector3 radius;
+  float radius;
 
 public:
   ///
@@ -140,15 +141,20 @@ public:
    */
   void GetBoundingBox (csBox3& b) const { b = box; }
 
+  /**
+   * Get the bounding box in object space.
+   */
+  const csBox3& GetBoundingBox () const { return box; }
+
   /**
    * Set the radius of this frame in object space.
    */
-  void SetRadius (const csVector3& r) { radius = r; }
+  void SetRadius (float r) { radius = r; }
 
   /**
    * Get the radius of this frame in object space.
    */
-  void GetRadius (csVector3& r) const { r = radius; }
+  void GetRadius (float& r) const { r = radius; }
 };
 
 /**
@@ -174,11 +180,11 @@ public:
   virtual char const* GetName () const
   { return name; }
   /// Get total number of frames in this action
-  virtual int GetFrameCount () { return (int)frames.Length (); }
+  virtual int GetFrameCount () { return (int)frames.GetSize (); }
   /// Query the frame number f.
   csSpriteFrame* GetCsFrame (int f)
   {
-    return ((size_t)f < frames.Length ())
+    return ((size_t)f < frames.GetSize ())
     	? frames [f]
 	: (csSpriteFrame*)0;
 	}
@@ -188,20 +194,20 @@ public:
     if (!reverse_action)
     {
       f++;
-      return (size_t)f<frames.Length() ? frames[f] : frames[0];
+      return (size_t)f<frames.GetSize () ? frames[f] : frames[0];
     }
     else
     {
       f--;
       return f>=0
         ? frames[f]
-	: frames[frames.Length()-1];
+	: frames[frames.GetSize ()-1];
     }
   }
   /// Query the frame number f.
   virtual iSpriteFrame* GetFrame (int f)
   {
-    return (iSpriteFrame*)(((size_t)f < frames.Length ())
+    return (iSpriteFrame*)(((size_t)f < frames.GetSize ())
     	? frames [f]
 	: (csSpriteFrame*)0);
   }
@@ -211,7 +217,7 @@ public:
     if (!reverse_action)
     {
       f++;
-      return (iSpriteFrame*)((size_t)f<frames.Length()
+      return (iSpriteFrame*)((size_t)f<frames.GetSize ()
       	? frames[f]
 	: frames[0]);
     }
@@ -220,7 +226,7 @@ public:
       f--;
       return (iSpriteFrame*)(f>=0
       	? frames[f]
-	: frames[frames.Length()-1]);
+	: frames[frames.GetSize ()-1]);
     }
   }
   /// Get delay for frame number f
@@ -280,6 +286,8 @@ public:
 
 class csSprite3DMeshObject;
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
 /**
  * A 3D sprite based on a triangle mesh with a single texture.
  * Animation is done with frames.
@@ -317,6 +325,9 @@ private:
   /// The sockets.
   csPDelArray<csSpriteSocket> sockets;
 
+  /// Global bounding box over all frames.
+  csBox3 global_bbox;
+
   /// Enable tweening.
   bool do_tweening;
 
@@ -480,7 +491,7 @@ public:
   const csVector2& GetTexel (int frame, int vertex) const
   { return (*texels.Get(frame)) [vertex]; }
   void SetTexel (int frame, int vertex, const csVector2 &val)
-  { CS_CONST_CAST (csVector2&, GetTexel (frame, vertex)) = val; }
+  { const_cast<csVector2&> (GetTexel (frame, vertex)) = val; }
   /// Get array of texels.
   csVector2* GetTexels (int frame) const
   { return (*texels.Get(frame)).GetVertices (); }
@@ -492,7 +503,7 @@ public:
   const csVector3& GetVertex (int frame, int vertex) const
   { return (*vertices.Get(frame)) [vertex]; }
   void SetVertex (int frame, int vertex, const csVector3 &val)
-  { CS_CONST_CAST (csVector3&, GetVertex (frame, vertex)) = val; }
+  { const_cast<csVector3&> (GetVertex (frame, vertex)) = val; }
   /// Get vertex array.
   csVector3* GetVertices (int frame) const
   { return (*vertices.Get(frame)).GetVertices (); }
@@ -504,7 +515,7 @@ public:
   const csVector3& GetNormal (int frame, int vertex) const
   { return (*normals.Get(frame)) [vertex]; }
   void SetNormal (int frame, int vertex, const csVector3 &val)
-  { CS_CONST_CAST (csVector3&, GetNormal (frame, vertex)) = val; }
+  { const_cast<csVector3&> (GetNormal (frame, vertex)) = val; }
   /// Get normal array.
   csVector3* GetNormals (int frame) const
   { return (*normals.Get(frame)).GetVertices (); }
@@ -534,11 +545,11 @@ public:
   /// find a named frame into the sprite.
   iSpriteFrame* FindFrame (const char * name) const;
   /// Query the number of frames
-  int GetFrameCount () const { return (int)frames.Length (); }
+  int GetFrameCount () const { return (int)frames.GetSize (); }
   /// Query the frame number f
   iSpriteFrame* GetFrame (int f) const
   {
-    return ((size_t)f < frames.Length ())
+    return ((size_t)f < frames.GetSize ())
   	? (csSpriteFrame *)frames [f]
 	: (csSpriteFrame*)0;
   }
@@ -552,7 +563,7 @@ public:
   { return (csSpriteAction2 *)actions [0]; }
   /// Get number of actions in sprite
   int GetActionCount () const
-  { return (int)actions.Length (); }
+  { return (int)actions.GetSize (); }
   /// Get action number No
   iSpriteAction* GetAction (int No) const
   { return (csSpriteAction2 *)actions [No]; }
@@ -564,11 +575,11 @@ public:
   /// find a socked based on the sprite attached to it
   iSpriteSocket* FindSocket (iMeshWrapper *mesh) const;
   /// Query the number of sockets
-  int GetSocketCount () const { return (int)sockets.Length (); }
+  int GetSocketCount () const { return (int)sockets.GetSize (); }
   /// Query the socket number f
   iSpriteSocket* GetSocket (int f) const
   {
-    return ((size_t)f < sockets.Length ())
+    return ((size_t)f < sockets.GetSize ())
   	? (csSpriteSocket *)sockets [f]
 	: (csSpriteSocket*)0;
   }
@@ -578,7 +589,7 @@ public:
   iMaterialWrapper* GetMaterialWrapper () const
   { return cstxt; }
   /// Set the material used for this sprite
-  void SetMaterialWrapper (iMaterialWrapper *material);
+  bool SetMaterialWrapper (iMaterialWrapper *material);
 
   /**
    * Compute all normals in a frame.
@@ -620,9 +631,11 @@ public:
   void ClearLODListeners ();
   void SetupLODListeners (iSharedVariable* varm, iSharedVariable* vara);
 
+  void SetupFactory ();
   void GetObjectBoundingBox (csBox3& bbox);
+  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  void GetRadius (csVector3& rad, csVector3 &cent);
+  void GetRadius (float& rad, csVector3 &cent);
 
   /**\name iMeshObjectFactory implementation
    * @{ */
@@ -693,6 +706,51 @@ public:
   friend struct PolyMesh;
   /** @} */
 
+  /**\name iTriangleMesh implementation
+   * @{ */
+  struct TriMesh : public scfImplementation1<TriMesh, iTriangleMesh>
+  {
+  private:
+    csSprite3DMeshObjectFactory* factory;
+    csFlags flags;
+
+  public:
+    virtual size_t GetVertexCount ()
+    {
+      return factory->GetVertexCount ();
+    }
+    virtual csVector3* GetVertices ()
+    {
+      return factory->GetVertices (0);
+    }
+    virtual size_t GetTriangleCount ()
+    {
+      return factory->GetTriangleCount ();
+    }
+    virtual csTriangle* GetTriangles ()
+    {
+      return factory->GetTriangles ();
+    }
+
+    virtual void Lock () { } //PM@@@
+    virtual void Unlock () { }
+ 
+    virtual csFlags& GetFlags () { return flags;  }
+    virtual uint32 GetChangeNumber() const { return 0; }
+
+    TriMesh (csSprite3DMeshObjectFactory* Factory) : 
+      scfImplementationType (this), factory(Factory)
+    {
+    }
+    virtual ~TriMesh ()
+    {
+      Cleanup ();
+    }
+    void Cleanup () { }
+  };
+  friend struct TriMesh;
+  /** @} */
+
   virtual iObjectModel* GetObjectModel () { return this; }
 
   /**\name iLODControl implementation
@@ -719,9 +777,15 @@ public:
     varm = lod_varm;
     vara = lod_vara;
   }
+  
+  iTerraFormer* GetTerraFormerColldet () { return 0; }
+  virtual iTerrainSystem* GetTerrainColldet () { return 0; }
+
   /** @} */
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * A 3D sprite based on a triangle mesh with a single texture.
  * Animation is done with frames.
@@ -1013,7 +1077,7 @@ private:
   ///
   bool force_otherskin;
 
-  iMeshObjectDrawCallback* vis_cb;
+  csRef<iMeshObjectDrawCallback> vis_cb;
 
   /**
    * Camera space bounding box is cached here.
@@ -1370,8 +1434,9 @@ public:
   void SetupLODListeners (iSharedVariable* varm, iSharedVariable* vara);
 
   void GetObjectBoundingBox (csBox3& bbox);
+  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  void GetRadius (csVector3& rad, csVector3 &cent);
+  void GetRadius (float& rad, csVector3 &cent);
 
   /// Create and add a new socket to the sprite.
   iSpriteSocket* AddSocket ();
@@ -1380,11 +1445,11 @@ public:
   /// find a socked based on the sprite attached to it
   iSpriteSocket* FindSocket (iMeshWrapper *mesh) const;
   /// Query the number of sockets
-  int GetSocketCount () const { return (int)sockets.Length (); }
+  int GetSocketCount () const { return (int)sockets.GetSize (); }
   /// Query the socket number f
   iSpriteSocket* GetSocket (int f) const
   {
-    return ((size_t)f < sockets.Length ())
+    return ((size_t)f < sockets.GetSize ())
   	? (csSpriteSocket *)sockets [f]
 	: (csSpriteSocket*)0;
   }
@@ -1393,8 +1458,8 @@ public:
    * @{ */
   virtual iMeshObjectFactory* GetFactory () const
   {
-    csRef<iMeshObjectFactory> ifact (SCF_QUERY_INTERFACE (factory,
-    	iMeshObjectFactory));
+    csRef<iMeshObjectFactory> ifact (
+    	scfQueryInterface<iMeshObjectFactory> (factory));
     return ifact;	// DecRef is ok here.
   }
   virtual csFlags& GetFlags () { return flags; }
@@ -1403,15 +1468,14 @@ public:
     iMovable* movable, uint32 frustum_mask);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* cb)
   {
-    if (cb) cb->IncRef ();
-    if (vis_cb) vis_cb->DecRef ();
     vis_cb = cb;
   }
   virtual iMeshObjectDrawCallback* GetVisibleCallback () const
   {
     return vis_cb;
   }
-  virtual void NextFrame (csTicks current_time,const csVector3& new_pos)
+  virtual void NextFrame (csTicks current_time,const csVector3& new_pos, 
+    uint /*currentFrame*/)
   {   
     OldNextFrame (current_time, new_pos, single_step, !loopaction);
   }
@@ -1442,6 +1506,10 @@ public:
    */
   virtual void PositionChild (iMeshObject* child,csTicks current_time);
   /** @} */
+  virtual void BuildDecal(const csVector3* pos, float decalRadius,
+          iDecalBuilder* decalBuilder)
+  {
+  }
 
   virtual iObjectModel* GetObjectModel () { return factory->GetObjectModel (); }
 
@@ -1519,6 +1587,9 @@ private:
   csRef<iVirtualClock> vc;
   csWeakRef<iEngine> engine;
 
+public:
+  csStringID base_id;
+
 public:
   /// Constructor.
   csSprite3DMeshObjectType (iBase*);
@@ -1575,6 +1646,7 @@ public:
   /** @} */
 };
 
-} // namespace cspluginSpr3d
+}
+CS_PLUGIN_NAMESPACE_END(Spr3d)
 
 #endif // __CS_SPR3D_H__
