@@ -52,8 +52,7 @@
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginSpr3dLoader
-{
+using namespace CS::Plugins::Spr3dLoader;
 
 enum
 {
@@ -170,7 +169,7 @@ csPtr<iBase> csSprite3DFactoryLoader::Parse (iDocumentNode* node,
 		  child, "Couldn't find material named '%s'", matname);
             return 0;
 	  }
-	  spr3dLook->SetMaterialWrapper (mat);
+	  fact->SetMaterialWrapper (mat);
         }
         break;
 
@@ -382,7 +381,7 @@ bool csSprite3DFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
     if (!meshfact) return false;
 
     //Writedown Material tag
-    iMaterialWrapper* mat = spritefact->GetMaterialWrapper();
+    iMaterialWrapper* mat = meshfact->GetMaterialWrapper();
     if (mat)
     {
       const char* matname = mat->QueryObject()->GetName();
@@ -415,7 +414,7 @@ bool csSprite3DFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
         csVector3 vertex = spritefact->GetVertex(i,j);
         csVector2 texel = spritefact->GetTexel(i,j);
         //vertexNode->SetAttribute("name", sprite_frame->GetName());
-        synldr->WriteVector(vertexNode, &vertex);
+        synldr->WriteVector(vertexNode, vertex);
         vertexNode->SetAttributeAsFloat("u", texel.x);
         vertexNode->SetAttributeAsFloat("v", texel.y);
       }
@@ -631,7 +630,7 @@ csPtr<iBase> csSprite3DLoader::Parse (iDocumentNode* node,
 	  uint mm;
 	  if (!synldr->ParseMixmode (child, mm))
 	    return 0;
-          spr3dLook->SetMixMode (mm);
+          mesh->SetMixMode (mm);
 	}
 	break;
       case XMLTOKEN_TWEEN:
@@ -720,7 +719,7 @@ bool csSprite3DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
     csRef<iDocumentNode> colorNode = 
       paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
     colorNode->SetValue("basecolor");
-    synldr->WriteColor(colorNode, &col);
+    synldr->WriteColor(colorNode, col);
 
     //Writedown Action tag
     const char* actionname = sprite->GetCurAction()->GetName();
@@ -758,4 +757,3 @@ bool csSprite3DSaver::WriteDown (iBase* obj, iDocumentNode* parent,
   return true;
 }
 
-} // namespace cspluginSpr3dLoader
