@@ -23,6 +23,7 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "csgeom/polyclip.h"
 #include "csgeom/quaternion.h"
 #include "csgeom/sphere.h"
+#include "cstool/rviewclipper.h"
 #include "csgfx/renderbuffer.h"
 #include "csgfx/shadervarcontext.h"
 #include "csutil/bitarray.h"
@@ -32,6 +33,7 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "csutil/dirtyaccessarray.h"
 #include "csutil/memfile.h"
 #include "csutil/randomgen.h"
+#include "csutil/hash.h"
 #include "csutil/sysfunc.h"
 #include "cstool/rbuflock.h"
 
@@ -53,14 +55,17 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 
 #include "sprcal3d.h"
+#include <cal3d/coretrack.h>
+#include <cal3d/corekeyframe.h>
 
 // STL include required by cal3d
 #include <string>
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginSprCal3d
+CS_PLUGIN_NAMESPACE_BEGIN(SprCal3d)
 {
+
 CS_LEAKGUARD_IMPLEMENT (csCal3DMesh);
 CS_LEAKGUARD_IMPLEMENT (csSpriteCal3DMeshObject);
 CS_LEAKGUARD_IMPLEMENT (csSpriteCal3DMeshObjectFactory);
@@ -124,13 +129,13 @@ void csSpriteCal3DSocket::SetMeshWrapper (iMeshWrapper* mesh)
 size_t csSpriteCal3DSocket::AttachSecondary (iMeshWrapper * mesh, csReversibleTransform trans)
 {
   secondary_meshes.Push(csSpriteCal3DSocketMesh(mesh, trans));
-  return secondary_meshes.Length()-1;
+  return secondary_meshes.GetSize ()-1;
 }
 
 void csSpriteCal3DSocket::DetachSecondary (const char* mesh_name)
 {
   size_t a=FindSecondary(mesh_name);
-  if (a < secondary_meshes.Length())
+  if (a < secondary_meshes.GetSize ())
     secondary_meshes.DeleteIndex(a);
 }
 
@@ -141,13 +146,13 @@ void csSpriteCal3DSocket::DetachSecondary (size_t index)
 
 size_t csSpriteCal3DSocket::FindSecondary (const char* mesh_name)
 {
-  for (size_t a=0; a<secondary_meshes.Length(); ++a)
+  for (size_t a=0; a<secondary_meshes.GetSize (); ++a)
   {
     if (strcmp (secondary_meshes[a].mesh->QueryObject()->GetName(), 
       mesh_name) == 0)
       return a;
   }
-  return secondary_meshes.Length();
+  return secondary_meshes.GetSize ();
 }
 
 //--------------------------------------------------------------------------
@@ -172,7 +177,9 @@ csSpriteCal3DMeshObjectFactory::csSpriteCal3DMeshObjectFactory (
 
   csSpriteCal3DMeshObjectFactory::object_reg = object_reg;
 
-  light_mgr = CS_QUERY_REGISTRY (object_reg, iLightManager);
+  light_mgr = csQueryRegistry<iLightManager> (object_reg);
+
+  skel_factory.AttachNew (new csCal3dSkeletonFactory ());
 }
 
 csSpriteCal3DMeshObjectFactory::~csSpriteCal3DMeshObjectFactory ()
@@ -228,6 +235,7 @@ bool csSpriteCal3DMeshObjectFactory::LoadCoreSkeleton (iVFS *vfs,
     if (skel)
     {
       calCoreModel.setCoreSkeleton (skel.get());
+      skel_factory->SetSkeleton (&calCoreModel);
       return true;
     }
     else
@@ -319,7 +327,7 @@ int csSpriteCal3DMeshObjectFactory::LoadCoreMorphTarget (
     const char *filename,
     const char *name)
 {
-  if (mesh_index < 0 || meshes.Length() <= (size_t)mesh_index)
+  if (mesh_index < 0 || meshes.GetSize () <= (size_t)mesh_index)
   {
     return -1;
   }
@@ -370,12 +378,12 @@ bool csSpriteCal3DMeshObjectFactory::AddMorphTarget( int morphanimation_index,
   }
   csArray<csString>& morph_target = meshes[mesh_index]->morph_target_name;
   size_t i;
-  for (i=0; i<morph_target.Length(); i++)
+  for (i=0; i<morph_target.GetSize (); i++)
   {
     if (morph_target[i] == morphtarget_name)
       break;
   }
-  if(i==morph_target.Length())
+  if(i==morph_target.GetSize ())
   {
     return false;
   }
@@ -386,16 +394,16 @@ bool csSpriteCal3DMeshObjectFactory::AddMorphTarget( int morphanimation_index,
 
 int csSpriteCal3DMeshObjectFactory::GetMorphTargetCount(int mesh_id)
 {
-  if (mesh_id < 0|| meshes.Length() <= (size_t)mesh_id)
+  if (mesh_id < 0|| meshes.GetSize () <= (size_t)mesh_id)
   {
     return -1;
   }
-  return (int)meshes[mesh_id]->morph_target_name.Length();
+  return (int)meshes[mesh_id]->morph_target_name.GetSize ();
 }
 
 const char *csSpriteCal3DMeshObjectFactory::GetMeshName(int idx)
 {
-  if ((size_t)idx >= meshes.Length())
+  if ((size_t)idx >= meshes.GetSize ())
     return 0;
 
   return meshes[idx]->name;
@@ -403,7 +411,7 @@ const char *csSpriteCal3DMeshObjectFactory::GetMeshName(int idx)
 
 bool csSpriteCal3DMeshObjectFactory::IsMeshDefault(int idx)
 {
-  if ((size_t)idx >= meshes.Length())
+  if ((size_t)idx >= meshes.GetSize ())
     return false;
 
   return meshes[idx]->attach_by_default;
@@ -511,7 +519,7 @@ iSpriteCal3DSocket* csSpriteCal3DMeshObjectFactory::FindSocket (
 
 int csSpriteCal3DMeshObjectFactory::FindMeshName (const char *meshName)
 {
-  for (size_t i=0; i<meshes.Length(); i++)
+  for (size_t i=0; i<meshes.GetSize (); i++)
   {
     if (meshes[i]->name == meshName)
       return (int)i;
@@ -536,7 +544,7 @@ const char* csSpriteCal3DMeshObjectFactory::GetDefaultMaterial (
 
 const char *csSpriteCal3DMeshObjectFactory::GetMorphAnimationName(int idx)
 {
-  if ((size_t)idx >= morph_animation_names.Length())
+  if ((size_t)idx >= morph_animation_names.GetSize ())
     return 0;
 
   return morph_animation_names[idx];
@@ -545,7 +553,7 @@ const char *csSpriteCal3DMeshObjectFactory::GetMorphAnimationName(int idx)
 int csSpriteCal3DMeshObjectFactory::FindMorphAnimationName (
     const char *meshName)
 {
-  for (size_t i=0; i<morph_animation_names.Length(); i++)
+  for (size_t i=0; i<morph_animation_names.GetSize (); i++)
   {
     if (morph_animation_names[i] == meshName)
       return (int)i;
@@ -553,7 +561,6 @@ int csSpriteCal3DMeshObjectFactory::FindMorphAnimationName (
   return -1;
 }
 
-
 bool csSpriteCal3DMeshObjectFactory::AddCoreMaterial(iMaterialWrapper *mat)
 {
   CalCoreMaterial *newmat = new CalCoreMaterial;
@@ -600,7 +607,7 @@ csPtr<iMeshObject> csSpriteCal3DMeshObjectFactory::NewInstance ()
   spr->updateanim_sqdistance3 = sprcal3d_type->updateanim_sqdistance3;
   spr->updateanim_skip3 = sprcal3d_type->updateanim_skip3;
 
-  csRef<iMeshObject> im (SCF_QUERY_INTERFACE (spr, iMeshObject));
+  csRef<iMeshObject> im (scfQueryInterface<iMeshObject> (spr));
   spr->DecRef ();
   return csPtr<iMeshObject> (im);
 }
@@ -608,7 +615,7 @@ csPtr<iMeshObject> csSpriteCal3DMeshObjectFactory::NewInstance ()
 bool csSpriteCal3DMeshObjectFactory::RegisterAnimCallback(
     const char *anim, CalAnimationCallback *callback,float min_interval)
 {
-  for (size_t i=0; i<anims.Length(); i++)
+  for (size_t i=0; i<anims.GetSize (); i++)
   {
     if (anims[i]->name == anim)
     {
@@ -623,7 +630,7 @@ bool csSpriteCal3DMeshObjectFactory::RegisterAnimCallback(
 bool csSpriteCal3DMeshObjectFactory::RemoveAnimCallback(
     const char *anim, CalAnimationCallback *callback)
 {
-  for (size_t i=0; i<anims.Length(); i++)
+  for (size_t i=0; i<anims.GetSize (); i++)
   {
     if (anims[i]->name == anim)
     {
@@ -638,8 +645,9 @@ bool csSpriteCal3DMeshObjectFactory::RemoveAnimCallback(
 void csSpriteCal3DMeshObjectFactory::HardTransform (
     const csReversibleTransform& t)
 {
-  csQuaternion quat (t.GetO2T ());
-  CalQuaternion quatrot(quat.x,quat.y,quat.z,quat.r);
+  csQuaternion quat;
+  quat.SetMatrix (t.GetO2T ());
+  CalQuaternion quatrot(quat.v.x,quat.v.y,quat.v.z,quat.w);
   csVector3 trans (t.GetOrigin () );
   CalVector translation (trans.x,trans.y,trans.z);
 
@@ -654,10 +662,12 @@ void csSpriteCal3DMeshObjectFactory::HardTransform (
     ->getVectorCoreBone();
 
   // loop through all root core bones
-  std::list<int>::iterator iteratorRootCoreBoneId;
-  for (iteratorRootCoreBoneId = pCoreSkeleton->getListRootCoreBoneId().begin()
-    ; iteratorRootCoreBoneId != pCoreSkeleton->getListRootCoreBoneId().end()
-    ; ++iteratorRootCoreBoneId)
+  std::vector<int>& rootCoreBones = 
+    pCoreSkeleton->getVectorRootCoreBoneId();
+  std::vector<int>::iterator iteratorRootCoreBoneId;
+  for (iteratorRootCoreBoneId = rootCoreBones.begin();
+       iteratorRootCoreBoneId != rootCoreBones.end();
+       ++iteratorRootCoreBoneId)
   {
     CalCoreBone *bone = vectorCoreBone[*iteratorRootCoreBoneId];
     CalQuaternion bonerot = bone->getRotation();
@@ -674,36 +684,172 @@ void csSpriteCal3DMeshObjectFactory::HardTransform (
   {
     CalCoreAnimation *anim = calCoreModel.getCoreAnimation(i);
     if (!anim) continue;
-
-    const uint trackCount = anim->getTrackCount();
-    std::vector<CalTransform> poses = anim->getPoses();
-    const uint frameCount = (uint)poses.size() / trackCount;
-
     // loop through all root core bones
-    std::list<int>::iterator iteratorRootCoreBoneId;
-    for (iteratorRootCoreBoneId = pCoreSkeleton->getListRootCoreBoneId().begin()
-        ; iteratorRootCoreBoneId != pCoreSkeleton->getListRootCoreBoneId().end()
-    ; ++iteratorRootCoreBoneId)
+    for (iteratorRootCoreBoneId = rootCoreBones.begin();
+         iteratorRootCoreBoneId != rootCoreBones.end();
+         ++iteratorRootCoreBoneId)
     {
-      int boneTrack = anim->getTrackAssignment (*iteratorRootCoreBoneId);
-      if (boneTrack < 0) continue;
-      for (uint j = 0; j < frameCount; j++)
+      CalCoreTrack *track = anim->getCoreTrack(*iteratorRootCoreBoneId);
+      if (!track) continue;
+      for (int j=0; j<track->getCoreKeyframeCount(); j++)
       {
-        CalTransform& tf = poses[j * trackCount + boneTrack];
-        CalQuaternion bonerot = tf.getRotation();
-        CalVector bonevec = tf.getTranslation();
-        bonerot *= quatrot;
-        bonevec *= quatrot;
-        bonevec += translation;
-        tf.setRotation (bonerot);
-        tf.setTranslation (bonevec);
+	CalCoreKeyframe *frame = track->getCoreKeyframe(j);
+	CalQuaternion bonerot = frame->getRotation();
+	CalVector bonevec = frame->getTranslation();
+	bonerot *= quatrot;
+	bonevec *= quatrot;
+	bonevec += translation;
+	frame->setRotation(bonerot);
+	frame->setTranslation(bonevec);
       }
     }
-    anim->setPoses (poses, trackCount);
   }
 //  calCoreModel.getCoreSkeleton()->calculateBoundingBoxes(&calCoreModel);
 }
 
+//---------------------------csCal3dSkeletonFactory---------------------------
+
+csCal3dSkeletonFactory::csCal3dSkeletonFactory () :
+scfImplementationType(this), core_skeleton(0) 
+{
+}
+
+void csCal3dSkeletonFactory::SetSkeleton (CalCoreModel *model)
+{
+  core_model = model;
+  core_skeleton = model->getCoreSkeleton ();
+  std::vector<CalCoreBone*> bvect = core_skeleton->getVectorCoreBone ();
+  for (size_t i = 0; i < bvect.size (); i++)
+  {
+    csRef<csCal3dSkeletonBoneFactory> newFact;
+    newFact.AttachNew (new csCal3dSkeletonBoneFactory (bvect[i], this));
+    bones_factories.Push (newFact);
+  }
+
+  //now we can setup parents and childres
+  for (size_t i = 0; i < bvect.size (); i++)
+  {
+    bones_factories[i]->Initialize ();
+    bones_names.Put (csHashComputer<const char*>::ComputeHash (bones_factories[i]->GetName ()), i);
+  }
+}
+int csCal3dSkeletonFactory::GetCoreBoneId (iSkeletonBoneFactory *core_bone)
+{
+  return core_skeleton->getCoreBoneId (core_bone->GetName ());
+}
+size_t csCal3dSkeletonFactory::FindBoneIndex (const char *name)
+{
+  size_t b_idx = bones_names.Get (
+    csHashComputer<const char*>::ComputeHash (name), csArrayItemNotFound);
+  return b_idx;
+}
+iSkeletonBoneFactory *csCal3dSkeletonFactory::FindBone (const char *name)
+{
+  size_t b_idx = bones_names.Get (
+    csHashComputer<const char*>::ComputeHash (name), csArrayItemNotFound);
+  if (b_idx != csArrayItemNotFound)
+    return bones_factories[b_idx];
+  return 0;
+}
+iSkeletonAnimation *csCal3dSkeletonFactory::CreateAnimation (const char *name)
+{
+  csCal3dSkeletonAnimation *anim = new csCal3dSkeletonAnimation (this);
+  anim->SetName (name);
+  animations_names.Put (csHashComputer<const char*>::ComputeHash (name),
+    animations.Push (anim));
+  core_model->addCoreAnimation (anim->GetCoreAnimation ());
+  return anim;
+}
+iSkeletonAnimation *csCal3dSkeletonFactory::FindAnimation (const char *name) 
+{
+  size_t idx = animations_names.Get (
+    csHashComputer<const char*>::ComputeHash (name), csArrayItemNotFound);
+  if (idx != csArrayItemNotFound)
+    return animations[idx];
+  return 0;
+}
+
+iSkeletonBoneFactory* csCal3dSkeletonFactory::GetBone (size_t i)
+{ 
+  return bones_factories[i];
+}
+
+iSkeletonAnimation* csCal3dSkeletonFactory::GetAnimation (size_t idx)
+{ 
+  return animations[idx];
+}
+
+//---------------------------csCal3dSkeletonBoneFactory---------------------------
+
+csCal3dSkeletonBoneFactory::csCal3dSkeletonBoneFactory (CalCoreBone *core_bone,
+                                                        csCal3dSkeletonFactory* skelfact) :
+scfImplementationType(this), core_bone(core_bone), skeleton_factory(skelfact) 
+{
+}
+bool csCal3dSkeletonBoneFactory::Initialize ()
+{
+  std::list<int> children_ids = core_bone->getListChildId ();
+  for (std::list<int>::iterator it = children_ids.begin (); it != children_ids.end (); it++)
+  {
+    csRef<iSkeletonBoneFactory> skel_bone = skeleton_factory->GetBone ((*it));
+    children.Push (skel_bone);
+  }
+  int bid = core_bone->getParentId ();
+  if (bid != -1)
+    parent = skeleton_factory->GetBone (bid);
+  
+  return false;
+}
+//---------------------------csCal3dSkeletonAnimation---------------------------
+csCal3dSkeletonAnimation::csCal3dSkeletonAnimation (csCal3dSkeletonFactory *skel_fact) : 
+scfImplementationType(this), skel_fact(skel_fact) 
+{
+  animation = new CalCoreAnimation ();
+}
+csCal3dSkeletonAnimation::~csCal3dSkeletonAnimation ()
+{
+}
+iSkeletonAnimationKeyFrame *csCal3dSkeletonAnimation::CreateFrame (const char* name)
+{
+  csCal3dAnimationKeyFrame *frame = new csCal3dAnimationKeyFrame (this);
+  frames.Push (frame);
+  return frame;
+}
+//---------------------------csCal3dAnimationKeyFrame---------------------------
+void csCal3dAnimationKeyFrame::AddTransform (iSkeletonBoneFactory *bone,
+    csReversibleTransform &transform, bool relative)
+{
+  int b_id = animation->GetSkeletonFactory ()->GetCoreBoneId (bone);
+  if (b_id != -1)
+  {
+    CalCoreTrack *track = animation->GetCoreAnimation ()->getCoreTrack (b_id);
+    if (!track)
+    {
+      track = new CalCoreTrack ();
+      animation->GetCoreAnimation ()->addCoreTrack (track);
+    }
+    CalCoreKeyframe *keyframe = new CalCoreKeyframe ();
+    csVector3 pos = transform.GetOrigin ();
+    keyframe->setTranslation (CalVector (pos.x, pos.y, pos.z));
+
+    csQuaternion csquat;
+    csquat.SetMatrix (transform.GetO2T ());
+    csquat.Norm ();
+    keyframe->setRotation (CalQuaternion (csquat.v.x, csquat.v.y, csquat.v.z, csquat.w));
+    if (track->addCoreKeyframe (keyframe))
+    {
+      bones.Push (bone);
+      bones_key_frames.Push (keyframe);
+    }
+  }
+}
+void csCal3dAnimationKeyFrame::SetDuration (csTicks time)
+{
+  for (size_t i = 0; i < bones_key_frames.GetSize (); i++)
+  {
+    bones_key_frames[i]->setTime (CS_TIME_2_CAL_TIME (time));
+  }
+}
 //=============================================================================
 
 void csSpriteCal3DMeshObject::DefaultAnimTimeUpdateHandler::UpdatePosition(
@@ -742,10 +888,12 @@ csSpriteCal3DMeshObject::csSpriteCal3DMeshObject (iBase *pParent,
   // set the material set of the whole model
   vis_cb = 0;
   is_idling = false;
+  idle_override_interval = 0;
 
   meshVersion = 0;
   bboxVersion = (uint)-1;
   default_idle_anim = -1;
+  idle_action = -1;
   last_locked_anim = -1;
 
   do_update = -1;
@@ -757,6 +905,7 @@ csSpriteCal3DMeshObject::csSpriteCal3DMeshObject (iBase *pParent,
   updateanim_skip3 = 1000;  // Animate very rarely.
 
   anim_time_handler.AttachNew (new DefaultAnimTimeUpdateHandler());
+  calModel.getPhysique()->setAxisFactorX(-1.0f);
 }
 
 csSpriteCal3DMeshObject::~csSpriteCal3DMeshObject ()
@@ -769,17 +918,17 @@ void csSpriteCal3DMeshObject::SetFactory (csSpriteCal3DMeshObjectFactory* tmpl)
 {
   factory = tmpl;
 
-  CalSkeleton *skeleton;
+  CalSkeleton *cal_skeleton;
   CalBone *bone;
-  skeleton = calModel.getSkeleton();
-  std::vector < CalBone *> &bones = skeleton->getVectorBone();
-  int i;
-  for (i=0; i < (int)bones.size(); i++)
+  cal_skeleton = calModel.getSkeleton();
+  std::vector < CalBone *> &bones = cal_skeleton->getVectorBone();
+
+  for (size_t i = 0; i < bones.size(); i++)
   {
     bone = bones[i];
     bone->calculateState ();
   }
-  skeleton->calculateState ();
+  cal_skeleton->calculateState ();
 
   // attach all default meshes to the model
   int meshId;
@@ -805,7 +954,7 @@ void csSpriteCal3DMeshObject::SetFactory (csSpriteCal3DMeshObjectFactory* tmpl)
 
   // Copy the sockets list down to the mesh
   iSpriteCal3DSocket *factory_socket,*new_socket;
-  for (i=0; i<tmpl->GetSocketCount(); i++)
+  for (int i = 0; i < tmpl->GetSocketCount(); i++)
   {
     factory_socket = tmpl->GetSocket(i);
     new_socket = AddSocket();  // mesh now
@@ -815,21 +964,19 @@ void csSpriteCal3DMeshObject::SetFactory (csSpriteCal3DMeshObjectFactory* tmpl)
     new_socket->SetMeshIndex (factory_socket->GetMeshIndex() );
     new_socket->SetMeshWrapper (0);
   }
+
+  skeleton = new csCal3dSkeleton (cal_skeleton, factory->skel_factory, this);
 }
 
 
-void csSpriteCal3DMeshObject::GetRadius (csVector3& rad, csVector3& cent)
+void csSpriteCal3DMeshObject::GetRadius (float& rad, csVector3& cent)
 {
   cent.Set (object_bbox.GetCenter());
-  csVector3 maxbox, minbox;
 
   RecalcBoundingBox (object_bbox);
-  maxbox = object_bbox.Max();
-  minbox = object_bbox.Min();
-  float r1 = (maxbox.x-minbox.x)/2;
-  float r2 = (maxbox.y-minbox.y)/2;
-  float r3 = (maxbox.z-minbox.z)/2;
-  rad.Set(r1,r2,r3);
+  const csVector3& maxbox = object_bbox.Max();
+  const csVector3& minbox = object_bbox.Min();
+  rad = csQsqrt (csSquaredDist::PointPoint (minbox, maxbox));
 }
 
 #define CAL3D_EXACT_BOXES true 
@@ -860,16 +1007,22 @@ void csSpriteCal3DMeshObject::GetObjectBoundingBox (csBox3& bbox)
   bbox = object_bbox;
 }
 
+const csBox3& csSpriteCal3DMeshObject::GetObjectBoundingBox ()
+{
+  RecalcBoundingBox (object_bbox);
+  return object_bbox;
+}
+
 void csSpriteCal3DMeshObject::SetObjectBoundingBox (const csBox3& bbox)
 {
   object_bbox = bbox;
   ShapeChanged ();
 }
 
-void csSpriteCal3DMeshObjectFactory::GetRadius (csVector3& rad, csVector3& cent)
+void csSpriteCal3DMeshObjectFactory::GetRadius (float& rad, csVector3& cent)
 {
   cent.Set(0,0,0);
-  rad.Set(1,1,1);
+  rad = 1.0f;
 }
 
 void csSpriteCal3DMeshObjectFactory::GetObjectBoundingBox (csBox3& bbox)
@@ -888,6 +1041,12 @@ void csSpriteCal3DMeshObjectFactory::GetObjectBoundingBox (csBox3& bbox)
   }
 }
 
+const csBox3& csSpriteCal3DMeshObjectFactory::GetObjectBoundingBox ()
+{
+  obj_bbox = GetObjectBoundingBox ();
+  return obj_bbox;
+}
+
 void csSpriteCal3DMeshObjectFactory::SetObjectBoundingBox (const csBox3&)
 {
   // @@@ TODO
@@ -939,7 +1098,7 @@ void csSpriteCal3DMeshObject::UpdateLightingSubmesh (
   // center in object space is obviously at (0,0,0).
   csColor color;
 
-  size_t num_lights = lights.Length ();
+  size_t num_lights = lights.GetSize ();
 
   // Make sure colors array exists and set all to ambient
   InitSubmeshLighting (mesh, submesh, pCalRenderer, movable, colors);
@@ -1028,7 +1187,7 @@ bool csSpriteCal3DMeshObject::HitBeamOutline (const csVector3& start,
   bool hit = false;
   std::vector<CalBone *> vectorBone = calModel.getSkeleton()->getVectorBone();
   csArray<bool> bboxhits;
-  bboxhits.SetLength(vectorBone.size());
+  bboxhits.SetSize (vectorBone.size());
   int b = 0;
   std::vector<CalBone *>::iterator iteratorBone = vectorBone.begin();
   while (iteratorBone != vectorBone.end())
@@ -1067,7 +1226,7 @@ bool csSpriteCal3DMeshObject::HitBeamOutline (const csVector3& start,
     csVector3 tsect;
 
     size_t m;
-    for (m = 0; m < meshes.Length(); m++)
+    for (m = 0; m < meshes.GetSize (); m++)
     {
       if (!meshes[m].vertex_buffer)
         GetVertexBufferIndex (m, 0);
@@ -1156,7 +1315,7 @@ bool csSpriteCal3DMeshObject::HitBeamObject (const csVector3& start,
   bool hit = false;
   std::vector<CalBone *> vectorBone = calModel.getSkeleton()->getVectorBone();
   csArray<bool> bboxhits;
-  bboxhits.SetLength(vectorBone.size());
+  bboxhits.SetSize (vectorBone.size());
   int b = 0;
   std::vector<CalBone *>::iterator iteratorBone = vectorBone.begin();
   while (iteratorBone != vectorBone.end())
@@ -1197,7 +1356,7 @@ bool csSpriteCal3DMeshObject::HitBeamObject (const csVector3& start,
     csVector3 tsect;
 
     size_t m;
-    for (m = 0; m < meshes.Length(); m++)
+    for (m = 0; m < meshes.GetSize (); m++)
     {
       if (!meshes[m].vertex_buffer)
         GetVertexBufferIndex (m, 0);
@@ -1292,7 +1451,7 @@ void csSpriteCal3DMeshObject::PositionChild (iMeshObject* child,
 {
   iSpriteCal3DSocket* socket = 0;
   size_t i;
-  for ( i = 0; i < sockets.Length(); i++)
+  for ( i = 0; i < sockets.GetSize (); i++)
   {
     if(sockets[i]->GetMeshWrapper())
     {
@@ -1397,8 +1556,8 @@ csRenderMesh** csSpriteCal3DMeshObject::GetRenderMeshes (int &n,
     tr_o2c /= movable->GetFullTransform ();
 
   int clip_portal, clip_plane, clip_z_plane;
-  rview->CalculateClipSettings (frustum_mask, clip_portal, clip_plane,
-    clip_z_plane);
+  CS::RenderViewClipper::CalculateClipSettings (rview->GetRenderContext (),
+      frustum_mask, clip_portal, clip_plane, clip_z_plane);
   csVector3 camera_origin = tr_o2c.GetT2OTranslation ();
 
   // Distance between camera and object. Use this for LOD.
@@ -1432,8 +1591,8 @@ csRenderMesh** csSpriteCal3DMeshObject::GetRenderMeshes (int &n,
   const csReversibleTransform o2wt = movable->GetFullTransform ();
   const csVector3& wo = o2wt.GetOrigin ();
 
-  rendermeshes.SetSize (meshes.Length());
-  for (size_t m = 0; m < rendermeshes.Length(); m++)
+  rendermeshes.SetSize (meshes.GetSize ());
+  for (size_t m = 0; m < rendermeshes.GetSize (); m++)
   {
     csRenderMesh* rm = factory->sprcal3d_type->rmHolder.GetUnusedMesh (
       created, currentFrame);
@@ -1451,10 +1610,15 @@ csRenderMesh** csSpriteCal3DMeshObject::GetRenderMeshes (int &n,
     ((MeshAccessor*)rm->buffers->GetAccessor())->movable = movable;
   }
 
-  n = (int)rendermeshes.Length();
+  n = (int)rendermeshes.GetSize ();
   return rendermeshes.GetArray();
 }
-
+void csSpriteCal3DMeshObject::NextFrame (csTicks current_time, const csVector3& /*new_pos*/,
+                                         uint /*currentFrame*/)
+{
+  if (!skeleton->UpdatedByGraveyard ())
+    Advance (current_time);
+}
 bool csSpriteCal3DMeshObject::Advance (csTicks current_time)
 {
   if (do_update != -1)
@@ -1474,16 +1638,19 @@ bool csSpriteCal3DMeshObject::Advance (csTicks current_time)
   if (anim_time_handler.IsValid())
     anim_time_handler->UpdatePosition (delta, &calModel);
 
+  skeleton->UpdateNotify (current_time);
+
   if (current_time)
     last_update_time = current_time;
 
   if (is_idling) // check for override and play if time
   {
     idle_override_interval -= delta;
-    if (idle_override_interval <= 0)
+    if ((idle_override_interval <= 0) && (default_idle_anim != -1))
     {
+      csRandomGen rng;
+      SetIdleOverrides(&rng,default_idle_anim);
       SetAnimAction(factory->anims[idle_action]->name,.25,.25);
-      idle_override_interval = 20;
     }
   }
   meshVersion++;
@@ -1529,8 +1696,8 @@ int csSpriteCal3DMeshObject::FindAnim(const char *name)
 
 void csSpriteCal3DMeshObject::ClearAllAnims()
 {
-  while (active_anims.Length())
-    ClearAnimCyclePos ((int)(active_anims.Length() - 1), 0);
+  while (active_anims.GetSize ())
+    ClearAnimCyclePos ((int)(active_anims.GetSize () - 1), 0);
 
   if (last_locked_anim != -1)
   {
@@ -1576,7 +1743,7 @@ bool csSpriteCal3DMeshObject::AddAnimCycle(int idx, float weight, float delay)
 
 int csSpriteCal3DMeshObject::FindAnimCyclePos(int idx) const
 {
-  for (size_t i = active_anims.Length(); i-- > 0; )
+  for (size_t i = active_anims.GetSize (); i-- > 0; )
     if (active_anims[i].anim->index == idx)
       return (int)i;
   return -1;
@@ -1584,7 +1751,7 @@ int csSpriteCal3DMeshObject::FindAnimCyclePos(int idx) const
 
 int csSpriteCal3DMeshObject::FindAnimCycleNamePos(char const* name) const
 {
-  for (size_t i = active_anims.Length(); i-- > 0; )
+  for (size_t i = active_anims.GetSize (); i-- > 0; )
     if (active_anims[i].anim->name == name)
       return (int)i;
   return -1;
@@ -1617,7 +1784,7 @@ bool csSpriteCal3DMeshObject::ClearAnimCycle (const char *name, float delay)
 
 size_t csSpriteCal3DMeshObject::GetActiveAnimCount()
 {
-  return active_anims.Length();
+  return active_anims.GetSize ();
 }
 
 bool csSpriteCal3DMeshObject::GetActiveAnims (csSpriteCal3DActiveAnim* buffer, 
@@ -1626,7 +1793,7 @@ bool csSpriteCal3DMeshObject::GetActiveAnims (csSpriteCal3DActiveAnim* buffer,
   if ((buffer == 0) || (max_length == 0))
     return false;
 
-  size_t i, n = csMin (active_anims.Length(), max_length);
+  size_t i, n = csMin (active_anims.GetSize (), max_length);
 
   for (i=0; i<n; i++)
   {
@@ -1634,7 +1801,7 @@ bool csSpriteCal3DMeshObject::GetActiveAnims (csSpriteCal3DActiveAnim* buffer,
     buffer[i].index = a.anim->index;
     buffer[i].weight = a.weight;
   }
-  return i == active_anims.Length();
+  return i == active_anims.GetSize ();
 }
 
 void csSpriteCal3DMeshObject::SetActiveAnims(const csSpriteCal3DActiveAnim* buffer, 
@@ -1651,7 +1818,7 @@ void csSpriteCal3DMeshObject::SetActiveAnims(const csSpriteCal3DActiveAnim* buff
 bool csSpriteCal3DMeshObject::SetAnimAction(int idx, float delayIn,
                                             float delayOut)
 {
-  if (idx < 0 || (size_t)idx >=factory->anims.Length() )
+  if (idx < 0 || (size_t)idx >=factory->anims.GetSize () )
     return false;
 
   calModel.getMixer()->executeAction(idx,delayIn,delayOut,
@@ -1681,8 +1848,6 @@ void csSpriteCal3DMeshObject::SetIdleOverrides(csRandomGen *rng,int which)
 {
   csCal3DAnimation *anim = factory->anims[which];
 
-  is_idling = true;
-
   // Determine interval till next override.
   idle_override_interval = rng->Get(anim->max_interval - anim->min_interval)
     + anim->min_interval;
@@ -1707,6 +1872,12 @@ void csSpriteCal3DMeshObject::SetIdleOverrides(csRandomGen *rng,int which)
 void csSpriteCal3DMeshObject::SetDefaultIdleAnim(const char *name)
 {
     default_idle_anim = FindAnim(name);
+    if( default_idle_anim != -1 )
+    {
+      float max_interval(factory->anims[default_idle_anim]->max_interval);
+      if(idle_override_interval > max_interval)
+        idle_override_interval = max_interval;
+    }
 }
 
 bool csSpriteCal3DMeshObject::SetVelocity(float vel,csRandomGen *rng)
@@ -1716,6 +1887,8 @@ bool csSpriteCal3DMeshObject::SetVelocity(float vel,csRandomGen *rng)
   ClearAllAnims();
   if (!vel)
   {
+    is_idling = true;
+    SetTimeFactor(1);
     if (default_idle_anim != -1)
     {
       AddAnimCycle(default_idle_anim,1,0);
@@ -1727,9 +1900,10 @@ bool csSpriteCal3DMeshObject::SetVelocity(float vel,csRandomGen *rng)
     {
       if (factory->anims[i]->type == iSpriteCal3DState::C3D_ANIM_TYPE_IDLE)
       {
+        default_idle_anim = i;
         AddAnimCycle(i,1,0);
-    if (rng)
-      SetIdleOverrides(rng,i);
+        if (rng)
+          SetIdleOverrides(rng,i);
         return true;
       }
     }
@@ -1747,6 +1921,21 @@ bool csSpriteCal3DMeshObject::SetVelocity(float vel,csRandomGen *rng)
   }
 
   is_idling = false;
+
+  // Remove idle-animation that should not be played while moving too fast:
+  if( idle_action != -1)
+  {
+    /* Default value for max_vel is 0 if it is not set in the cal3d file.
+       Ideally, we would just test for (max_vel < vel). However, this would
+       break expected behaviour for old .cal3d files, and hence require
+       additional explanation. */
+    if((factory->anims[idle_action]->max_velocity) && (factory->anims[idle_action]->max_velocity < vel))
+    {
+      calModel.getMixer()->removeAction(idle_action);
+      idle_action = -1;
+    }
+  }
+
   // first look for animations with a base velocity that exactly matches
   bool found_match = false;
   for (i=0; i<count; i++)
@@ -1910,7 +2099,7 @@ int csSpriteCal3DMeshObject::ComputeVertexCount (int meshIdx)
 
 size_t csSpriteCal3DMeshObject::FindMesh( int mesh_id )
 {
-  for (size_t z = 0; z < meshes.Length(); z++)
+  for (size_t z = 0; z < meshes.GetSize (); z++)
   {
     if (meshes[z].calCoreMeshID == mesh_id)
       return z;
@@ -1989,7 +2178,7 @@ bool csSpriteCal3DMeshObject::BlendMorphTarget(int morph_animation_id,
     float weight, float delay)
 {
   if(morph_animation_id < 0||
-    factory->morph_animation_names.Length() <= (size_t)morph_animation_id)
+    factory->morph_animation_names.GetSize () <= (size_t)morph_animation_id)
   {
     return false;
   }
@@ -2000,7 +2189,7 @@ bool csSpriteCal3DMeshObject::ClearMorphTarget(int morph_animation_id,
     float delay)
 {
   if(morph_animation_id < 0||
-    factory->morph_animation_names.Length() <= (size_t)morph_animation_id)
+    factory->morph_animation_names.GetSize () <= (size_t)morph_animation_id)
   {
     return false;
   }
@@ -2061,6 +2250,8 @@ bool csSpriteCal3DMeshObject::SetMaterial(const char *mesh_name,
     return false;
 
   meshes[meshIdx].render_mesh.material = mat;
+  meshes[meshIdx].matRef = mat;
+
   return true;
 }
 
@@ -2196,6 +2387,115 @@ void csSpriteCal3DMeshObject::MeshAccessor::PreGetBuffer
     meshobj->factory->DefaultGetBuffer (mesh, holder, buffer);
 }
 
+//---------------------------csCal3dSkeleton---------------------------
+csCal3dSkeleton::csCal3dSkeleton (CalSkeleton* skeleton,
+    csCal3dSkeletonFactory* skel_factory,
+    csSpriteCal3DMeshObject *mesh_object) :
+      scfImplementationType(this), skeleton(skeleton),
+      skeleton_factory (skel_factory) 
+{
+  csCal3dSkeleton::mesh_object = mesh_object;
+  graveyard = csQueryRegistry<iSkeletonGraveyard> (mesh_object->object_reg);
+  if (graveyard)
+    // @@@ This is a memory leak! There is a circular reference between
+    // this Cal3DSkeleton and the list of skeletons in the graveyard.
+    graveyard->AddSkeleton (this);
+
+  std::vector<CalBone*> cal_bones = skeleton->getVectorBone ();
+  for (size_t i = 0; i < cal_bones.size (); i++)
+  {
+    bones.Push (new csCal3dSkeletonBone (cal_bones[i],
+	  skel_factory->GetBone (i), this));
+  }
+  for (size_t i = 0; i < cal_bones.size (); i++)
+  {
+    bones[i]->Initialize ();
+    bones_names.Put (csHashComputer<const char*>::ComputeHash (
+	  bones[i]->GetName ()), i);
+  }
+}
+
+void csCal3dSkeleton::UpdateNotify (const csTicks &current_ticks)
+{
+  for (size_t i = 0; i < update_callbacks.GetSize (); i++)
+  {
+    update_callbacks[i]->Execute (this, current_ticks);
+  }
+}
+size_t csCal3dSkeleton::FindBoneIndex (const char *name)
+{
+  size_t b_idx = bones_names.Get (
+    csHashComputer<const char*>::ComputeHash (name), csArrayItemNotFound);
+  return b_idx;
+}
+iSkeletonBone *csCal3dSkeleton::FindBone (const char *name)
+{
+  size_t b_idx = bones_names.Get (
+    csHashComputer<const char*>::ComputeHash (name), csArrayItemNotFound);
+  if (b_idx != csArrayItemNotFound)
+    return bones[b_idx];
+  return 0;
+}
+//---------------------------csCal3dSkeletonBone---------------------------
+
+csCal3dSkeletonBone::csCal3dSkeletonBone (CalBone *bone, iSkeletonBoneFactory *factory,
+                                          csCal3dSkeleton *skeleton) :
+scfImplementationType(this), bone(bone), factory(factory), skeleton(skeleton) 
+{
+}
+bool csCal3dSkeletonBone::Initialize ()
+{
+  std::list<int> children_ids = bone->getCoreBone ()->getListChildId ();
+  for (std::list<int>::iterator it = children_ids.begin (); it != children_ids.end (); it++)
+  {
+    csRef<iSkeletonBone> skel_bone = skeleton->GetBone ((*it));
+    children.Push (skel_bone);
+  }
+  int bid = bone->getCoreBone ()->getParentId ();
+  if (bid != -1)
+    parent = skeleton->GetBone (bid);
+
+  name = bone->getCoreBone ()->getName ().c_str ();
+
+  return true;
+}
+csReversibleTransform &csCal3dSkeletonBone::GetFullTransform ()
+{
+  CalQuaternion quat = bone->getRotationAbsolute ();
+  csQuaternion csquat (quat.x, quat.y, quat.z, quat.w);
+  csMatrix3 mat (csquat);
+
+  CalVector calv = bone->getTranslationAbsolute (); 
+  csVector3 csv (calv[0], calv[1], calv[2]); 
+
+  global_transform = csReversibleTransform (mat, csv); 
+  return global_transform;
+}
+csReversibleTransform &csCal3dSkeletonBone::GetTransform ()
+{
+  CalQuaternion quat = bone->getRotation();
+  csQuaternion csquat (quat.x, quat.y, quat.z, quat.w);
+  csMatrix3 mat (csquat);
+
+  CalVector calv = bone->getTranslation(); 
+	csVector3 csv (calv[0], calv[1], calv[2]); 
+  
+  local_transform = csReversibleTransform (mat, csv);
+  return local_transform;
+}
+
+void csCal3dSkeletonBone::SetTransform (const csReversibleTransform &transform)
+{
+  csQuaternion csquat;
+  csquat.SetMatrix (transform.GetO2T ());
+  csquat = csquat.Unit ();
+  CalQuaternion quat (csquat.v.x, csquat.v.y, csquat.v.z, csquat.w);
+  bone->setRotation (quat);
+  csVector3 vect = transform.GetOrigin();
+  bone->setTranslation (CalVector (vect[0], vect[1], vect[2]));
+  local_transform = transform;
+}
+
 //----------------------------------------------------------------------
 
 SCF_IMPLEMENT_FACTORY (csSpriteCal3DMeshObjectType)
@@ -2247,4 +2547,6 @@ csPtr<iMeshObjectFactory> csSpriteCal3DMeshObjectType::NewFactory ()
   return csPtr<iMeshObjectFactory> (cm);
 }
 
-} // namespace cspluginSprCal3d
+
+}
+CS_PLUGIN_NAMESPACE_END(SprCal3d)
