@@ -22,7 +22,7 @@
 #include "csgeom/box.h"
 #include "csgeom/math2d.h"
 #include "csgeom/math3d.h"
-#include "csgeom/objmodel.h"
+#include "cstool/objmodel.h"
 #include "csgeom/poly2d.h"
 #include "csgeom/poly3d.h"
 #include "cstool/rendermeshholder.h"
@@ -30,6 +30,7 @@
 #include "csutil/dirtyaccessarray.h"
 #include "csutil/flags.h"
 #include "csutil/hash.h"
+#include "csutil/weakrefarr.h"
 #include "csutil/hashr.h"
 #include "csutil/leakguard.h"
 #include "csutil/parray.h"
@@ -44,6 +45,7 @@
 #include "imesh/lighting.h"
 #include "imesh/object.h"
 #include "imesh/spritecal3d.h"
+#include "imesh/skeleton.h"
 #include "iutil/comp.h"
 #include "iutil/eventh.h"
 #include "iutil/virtclk.h"
@@ -52,13 +54,9 @@
 #include "ivideo/rendermesh.h"
 #include "ivideo/rndbuf.h"
 
-// Hack: work around problems caused by #defining 'new'
-#if defined(CS_EXTENSIVE_MEMDEBUG) || defined(CS_MEMORY_TRACKER)
-# undef new
-#endif
-#include <new>
-
+#include "csutil/custom_new_disable.h"
 #include <cal3d/cal3d.h>
+#include "csutil/custom_new_enable.h"
 
 struct iObjectRegistry;
 struct iEngine;
@@ -66,7 +64,7 @@ struct iMaterialWrapper;
 
 #define ALL_LOD_FEATURES (CS_LOD_TRIANGLE_REDUCTION|CS_LOD_DISTANCE_REDUCTION)
 
-namespace cspluginSprCal3d
+CS_PLUGIN_NAMESPACE_BEGIN(SprCal3d)
 {
 
 class csSpriteCal3DMeshObjectType;
@@ -84,7 +82,7 @@ struct csCal3DAnimation
   bool     lock;
 };
 
-/** This is a core mesh stored inside the factory. 
+/** This is a core mesh stored inside the factory.
   * These form the main list of available core meshes that can be attached
   * to a model.
   */
@@ -102,8 +100,8 @@ struct csCal3DMesh
  * A socket for specifying where sprites can plug into
  * other sprites.
  */
-class csSpriteCal3DSocket : 
-  public scfImplementation1<csSpriteCal3DSocket, 
+class csSpriteCal3DSocket :
+  public scfImplementation1<csSpriteCal3DSocket,
 			    iSpriteCal3DSocket>
 {
 private:
@@ -175,7 +173,7 @@ public:
    * primary mesh)
    */
   virtual size_t GetSecondaryCount () const
-  { return (int)secondary_meshes.Length(); }
+  { return (int)secondary_meshes.GetSize (); }
   /// Get the attached secondary mesh at the given index
   virtual iMeshWrapper * GetSecondaryMesh (size_t index)
   { return secondary_meshes[index].mesh; }
@@ -197,14 +195,18 @@ public:
 
 class csSpriteCal3DMeshObject;
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
+class csCal3dSkeletonFactory;
+
 /**
  * A Cal3D sprite based on a triangle mesh with a single texture.  Animation is
  * done with frames.  This class represents a template from which a
  * csSpriteCal3D class can be made.
  */
-class csSpriteCal3DMeshObjectFactory : 
+class csSpriteCal3DMeshObjectFactory :
   public scfImplementationExt3<csSpriteCal3DMeshObjectFactory,
-			       csObjectModel,	
+			       csObjectModel,
 			       iMeshObjectFactory,
 			       iSpriteCal3DFactoryState,
 			       iLODControl>
@@ -214,7 +216,7 @@ private:
 
   /// Material handle as returned by iTextureManager.
   iMeshFactoryWrapper* logparent;
-  
+
   csSpriteCal3DMeshObjectType* sprcal3d_type;
 
   /// If true then this factory has been initialized.
@@ -229,6 +231,9 @@ private:
   csString basePath;
   csPDelArray<csSpriteCal3DSocket> sockets;
   csFlags flags;
+  csBox3 obj_bbox;
+
+  csRef<csCal3dSkeletonFactory> skel_factory;
 
   struct MeshBuffers
   {
@@ -250,9 +255,9 @@ public:
    * used for the isometric engine).
    */
   csWeakRef<iEngine> engine;
-  
+
   /// Create the sprite template.
-  csSpriteCal3DMeshObjectFactory (csSpriteCal3DMeshObjectType* pParent, 
+  csSpriteCal3DMeshObjectFactory (csSpriteCal3DMeshObjectType* pParent,
     iObjectRegistry* object_reg);
   /// Destroy the template.
   virtual ~csSpriteCal3DMeshObjectFactory ();
@@ -270,25 +275,25 @@ public:
   void CalculateAllBoneBoundingBoxes();
 
   bool LoadCoreSkeleton(iVFS *vfs,const char *filename);
-  
+
   int  LoadCoreAnimation(iVFS *vfs,const char *filename,const char *name,
     int type,float base_vel, float min_vel,float max_vel,int min_interval,
     int max_interval,int idle_pct, bool lock);
-  
+
   /** Load a core mesh for the factory.  Reads in the mesh details and stores
     * them in a list for use by models.
     * \param vfs  File system to load mesh from.
     * \param filename The VFS file path to load the mesh from.
     * \param name The name the mesh should be given inside the list.
     * \param attach True if this mesh should be attached to all new models.
-    *               false if it is hidden when a model is created. 
+    *               false if it is hidden when a model is created.
     * \param defmat The default material for this mesh.
-    * 
+    *
     * \return The id of the mesh that cal3d as assigned to it.
-    */ 
+    */
   int LoadCoreMesh(iVFS *vfs,const char *filename,const char *name,
     bool attach,iMaterialWrapper *defmat);
-    
+
   int LoadCoreMorphTarget(iVFS *vfs,int mesh_index,const char *filename,
     const char *name);
   int AddMorphAnimation(const char *name);
@@ -301,12 +306,12 @@ public:
     float min_interval);
   bool RemoveAnimCallback(const char *anim, CalAnimationCallback *callback);
 
-  int GetMeshCount() { return (int)meshes.Length(); }
-  int GetMorphAnimationCount() { return (int)morph_animation_names.Length(); }
+  int GetMeshCount() { return (int)meshes.GetSize (); }
+  int GetMorphAnimationCount() { return (int)morph_animation_names.GetSize (); }
   int GetMorphTargetCount(int mesh_id);
   const char *GetMeshName(int idx);
-  int  FindMeshName(const char *meshName);  
-  const char* GetDefaultMaterial( const char* meshName );  
+  int  FindMeshName(const char *meshName);
+  const char* GetDefaultMaterial( const char* meshName );
   const char *GetMorphAnimationName(int idx);
   int  FindMorphAnimationName(const char *meshName);
   bool IsMeshDefault(int idx);
@@ -320,11 +325,11 @@ public:
   /// find a socked based on the sprite attached to it
   iSpriteCal3DSocket* FindSocket (iMeshWrapper *mesh) const;
   /// Query the number of sockets
-  int GetSocketCount () const { return (int)sockets.Length (); }
+  int GetSocketCount () const { return (int)sockets.GetSize (); }
   /// Query the socket number f
   iSpriteCal3DSocket* GetSocket (int f) const
   {
-    return ((size_t)f < sockets.Length()) ?
+    return ((size_t)f < sockets.GetSize ()) ?
       (csSpriteCal3DSocket*)sockets [f] : (csSpriteCal3DSocket*)0;
   }
   /** @} */
@@ -340,16 +345,23 @@ public:
   { logparent = lp; }
   virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
   { return logparent; }
-  virtual iMeshObjectType* GetMeshObjectType () const 
+  virtual iMeshObjectType* GetMeshObjectType () const
   { return (iMeshObjectType*)sprcal3d_type; }
+  virtual bool SetMaterialWrapper (iMaterialWrapper*) { return false; }
+  virtual iMaterialWrapper* GetMaterialWrapper () const { return 0; }
+  virtual void SetMixMode (uint) { }
+  virtual uint GetMixMode () const { return 0; }
   /** @} */
 
   /**\name iObjectModel implementation
    * @{ */
   void GetObjectBoundingBox (csBox3& bbox, csVector3 *verts,int vertCount);
   void GetObjectBoundingBox (csBox3& bbox);
+  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  void GetRadius (csVector3& rad, csVector3& cent);
+  void GetRadius (float& rad, csVector3& cent);
+  virtual iTerraFormer* GetTerraFormerColldet () { return 0; }
+  virtual iTerrainSystem* GetTerrainColldet () { return 0; }
   /** @} */
 
   virtual iObjectModel* GetObjectModel () { return this; }
@@ -376,12 +388,14 @@ public:
   /** @} */
 };
 
+class csCal3dSkeleton;
+
 /**
  * A Cal3D sprite based on a triangle mesh with a single texture.
  * Animation is done with frames (a frame may be controlled by
  * a skeleton).
  */
-class csSpriteCal3DMeshObject : 
+class csSpriteCal3DMeshObject :
   public scfImplementationExt4<csSpriteCal3DMeshObject,
 			       csObjectModel,
 			       iMeshObject,
@@ -396,8 +410,7 @@ private:
     float weight;
   };
 
-  iObjectRegistry* object_reg;
-  iMeshObjectDrawCallback* vis_cb;
+  csRef<iMeshObjectDrawCallback> vis_cb;
   uint32 current_features;  // LOD Control thing
   iMeshWrapper* logparent;
   CalModel calModel;
@@ -408,11 +421,13 @@ private:
   float idle_override_interval;
   int   idle_action;
 
+  csRef<csCal3dSkeleton> skeleton;
+
   // Optimization: only update animation when true.
   int do_update;	// If 0 we update, else we decrease.
 
-  
-  // User defined position update things  
+
+  // User defined position update things
   csRef<iAnimTimeUpdateHandler> anim_time_handler;
 
   csFlags flags;
@@ -424,8 +439,8 @@ private:
   csPDelArray<csSpriteCal3DSocket> sockets;
 
   size_t FindMesh( int mesh_id );
-    
-  class MeshAccessor : 
+
+  class MeshAccessor :
     public scfImplementation1<MeshAccessor, iRenderBufferAccessor>
   {
   protected:
@@ -442,7 +457,7 @@ private:
   public:
     iMovable* movable;
 
-    MeshAccessor (csSpriteCal3DMeshObject* meshobj, int mesh) : 
+    MeshAccessor (csSpriteCal3DMeshObject* meshobj, int mesh) :
       scfImplementationType (this)
     {
       MeshAccessor::meshobj = meshobj;
@@ -456,12 +471,12 @@ private:
     {
       return meshobj->FindMesh (mesh);
     }
-    
-    virtual ~MeshAccessor() 
+
+    virtual ~MeshAccessor()
     {
     }
     virtual void PreGetBuffer (csRenderBufferHolder*, csRenderBufferName);
-    
+
   };
   friend class MeshAccessor;
 
@@ -469,8 +484,8 @@ private:
   /**
    * Default animation time update handler (simply invokes CalModel::update()).
    */
-  struct DefaultAnimTimeUpdateHandler : 
-    public scfImplementation1<DefaultAnimTimeUpdateHandler, 
+  struct DefaultAnimTimeUpdateHandler :
+    public scfImplementation1<DefaultAnimTimeUpdateHandler,
 			      iAnimTimeUpdateHandler>
   {
     DefaultAnimTimeUpdateHandler() : scfImplementationType (this) {}
@@ -486,10 +501,10 @@ private:
    * - A cal3d submesh has one material(* Although in some places CS allows
    *   only a material per mesh)
    * - Of the meshes, several can be 'attached' at a time - that is,
-   *   actually used and rendered. Every mesh cam at most be only attached 
+   *   actually used and rendered. Every mesh cam at most be only attached
    *   once.
    * For CS rendering, since we limit the material to one per mesh, we also
-   * use one rendermesh per cal mesh. 
+   * use one rendermesh per cal mesh.
    * The total number of RMs may vary with instances and their attached meshes.
    */
   uint meshVersion;
@@ -498,8 +513,8 @@ private:
   bool lighting_dirty;
 
   /** A mesh attached to the model.  These form the list of meshes that
-    * the current model has attached to it. 
-    */    
+    * the current model has attached to it.
+    */
   struct Mesh
   {
     /// Cal3d ID of the mesh
@@ -510,20 +525,21 @@ private:
     csRef<iRenderBuffer> vertex_buffer;
     uint vertexVersion;
     csRenderMesh render_mesh;
+    csRef<iMaterialWrapper> matRef;
 
     Mesh() : vertexVersion((uint)~0) {}
   };
-  
+
   /// List of attached meshes. It's important that the order matches that of cal3d.
   csArray<Mesh> meshes;
-  
+
   static int CompareMeshIndexKey (const Mesh& m, int const& id);
   static int CompareMeshMesh (const Mesh& m1, const Mesh& m2);
-  
+
   // Vertices are handled differently since they're also needed by HitBeam*().
   csRef<iRenderBuffer> GetVertexBufferIndex (size_t index, CalRenderer *pCalRenderer);
   csRef<iRenderBuffer> GetVertexBufferCal (int mesh_id, CalRenderer *pCalRenderer);
-  void GetVertexBufferCal (int mesh_id, CalRenderer *pCalRenderer, 
+  void GetVertexBufferCal (int mesh_id, CalRenderer *pCalRenderer,
     csRef<iRenderBuffer>* vertex_buffer);
   int ComputeVertexCount (int mesh);
 
@@ -537,7 +553,7 @@ private:
   void InitSubmeshLighting (int mesh, int submesh, CalRenderer *pCalRenderer,
     iMovable* movable, csColor* colors);
   void UpdateLightingSubmesh (const csArray<iLightSectorInfluence*>& lights,
-      iMovable*, CalRenderer*, int mesh, int submesh, float* have_normals, 
+      iMovable*, CalRenderer*, int mesh, int submesh, float* have_normals,
       csColor* colors);
 
 public:
@@ -551,6 +567,8 @@ public:
 public:
   CS_LEAKGUARD_DECLARE (csSpriteCal3DMeshObject);
 
+  iObjectRegistry* object_reg;
+
   /// The parent.
   csSpriteCal3DMeshObjectFactory* factory;
 
@@ -615,18 +633,20 @@ public:
 
   virtual iMeshObjectFactory* GetFactory () const
   {
-    csRef<iMeshObjectFactory> ifact (SCF_QUERY_INTERFACE (factory,
-    	iMeshObjectFactory));
+    csRef<iMeshObjectFactory> ifact (
+    	scfQueryInterface<iMeshObjectFactory> (factory));
     return ifact;	// DecRef is ok here.
   }
+  virtual void BuildDecal(const csVector3* pos, float decalRadius,
+          iDecalBuilder* decalBuilder)
+  {
+  }
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> Clone () { return 0; }
   virtual csRenderMesh **GetRenderMeshes (int &n, iRenderView* rview,
     iMovable* movable, uint32 frustum_mask);
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* cb)
   {
-    if (cb) cb->IncRef ();
-    if (vis_cb) vis_cb->DecRef ();
     vis_cb = cb;
   }
   virtual iMeshObjectDrawCallback* GetVisibleCallback () const
@@ -635,10 +655,8 @@ public:
   }
   virtual bool Advance (csTicks current_time);
 
-  virtual void NextFrame (csTicks current_time, const csVector3& /*new_pos*/)
-  {   
-    Advance (current_time);
-  }
+  virtual void NextFrame (csTicks current_time, const csVector3& /*new_pos*/,
+    uint /*currentFrame*/);
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
   virtual void SetMeshWrapper (iMeshWrapper* lp) { logparent = lp; }
@@ -673,22 +691,22 @@ public:
   float GetTimeFactor();
 
   bool AttachCoreMesh(const char *meshname);
-  
+
   /** Attach a 'core' mesh to this model.
     * \param calCoreMeshID The Cal3D ID of the mesh to attach.
     * \param iMatWrapID The material to use for the attached mesh.
     *
-    * \return True if the attachment was successful. 
+    * \return True if the attachment was successful.
     */
   bool AttachCoreMesh(int calCoreMeshID,iMaterialWrapper* iMatWrapID);
-  
+
   bool DetachCoreMesh(const char *meshname);
-  
+
   /** Detach a 'core' mesh to this model.
     * \param calCoreMeshID The Cal3D ID of the mesh to attach.
     *
-    * \return True if the removal was successful. 
-    */  
+    * \return True if the removal was successful.
+    */
   bool DetachCoreMesh(int calCoreMeshID);
 
   bool BlendMorphTarget(int morph_animation_id, float weight, float delay);
@@ -701,11 +719,11 @@ public:
   /// find a socked based on the sprite attached to it
   iSpriteCal3DSocket* FindSocket (iMeshWrapper *mesh) const;
   /// Query the number of sockets
-  int GetSocketCount () const { return (int)sockets.Length (); }
+  int GetSocketCount () const { return (int)sockets.GetSize (); }
   /// Query the socket number f
   iSpriteCal3DSocket* GetSocket (int f) const
   {
-    return ((size_t)f < sockets.Length()) ?
+    return ((size_t)f < sockets.GetSize ()) ?
       (csSpriteCal3DSocket*)sockets[f] : (csSpriteCal3DSocket*)0;
   }
 
@@ -723,7 +741,10 @@ public:
   {
     calModel.getMixer()->setAnimationTime(animationTime);
   }
-  CalModel *GetCal3DModel() { return &calModel; }
+  CalModel *GetCal3DModel () { return &calModel; }
+
+  csPtr<iSkeleton> GetSkeleton () {return scfQueryInterface<iSkeleton> (
+      skeleton);}
 
   void SetAnimTimeUpdateHandler (iAnimTimeUpdateHandler*);
   /** @} */
@@ -754,16 +775,276 @@ public:
    * @{ */
   void GetObjectBoundingBox (csBox3& bbox, csVector3 *verts, int vertCount);
   void GetObjectBoundingBox (csBox3& bbox);
+  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  void GetRadius (csVector3& rad, csVector3& cent);
+  void GetRadius (float& rad, csVector3& cent);
+  virtual iTerraFormer* GetTerraFormerColldet () { return 0; }
   /** @} */
 };
 
+class csCal3dSkeletonBoneFactory;
+class csCal3dSkeletonAnimation;
+
+class csCal3dSkeletonFactory : public 
+  scfImplementation1<csCal3dSkeletonFactory, iSkeletonFactory> 
+{
+  CalCoreSkeleton *core_skeleton;
+  CalCoreModel *core_model;
+  csString name;
+  csRefArray<csCal3dSkeletonBoneFactory> bones_factories;
+  csHash<size_t, uint> bones_names;
+  csRefArray<csCal3dSkeletonAnimation> animations;
+  csHash<size_t, uint> animations_names;
+
+public:
+
+  csCal3dSkeletonFactory ();
+
+  void SetSkeleton (CalCoreModel *model);
+  CalCoreSkeleton *GetCal3dSkeleton () {return core_skeleton;}
+  int GetCoreBoneId (iSkeletonBoneFactory *core_bone);
+
+  /**\name iSkeletonFactory implementation
+   * @{ */
+  const char* GetName () const {return name.GetData ();}
+  void SetName (const char* name) {csCal3dSkeletonFactory::name = name;}
+  iSkeletonBoneFactory *CreateBone (const char *name) {return 0;}
+  iSkeletonAnimation *CreateAnimation (const char *name);
+  iSkeletonAnimation *CreateScript(const char *name) {return CreateAnimation (name);}
+  iSkeletonAnimation *FindAnimation (const char *name);
+  iSkeletonAnimation *FindScript (const char *name) {return FindAnimation (name);}
+  iSkeletonBoneFactory *FindBone (const char *name);
+  size_t FindBoneIndex (const char *name);
+  size_t GetBonesCount () const {return bones_factories.GetSize ();}
+  iSkeletonBoneFactory *GetBone (size_t i);
+  iSkeletonGraveyard *GetGraveyard  () {return 0;}
+  iSkeletonSocketFactory *CreateSocket(const char *name, iSkeletonBoneFactory *bone) {return 0;}
+  iSkeletonSocketFactory *FindSocket(const char *name) {return 0;}
+  iSkeletonSocketFactory *GetSocket (int i) {return 0;}
+  void RemoveSocket (int i) {;}
+  size_t GetSocketsCount () {return 0;}   
+  size_t GetAnimationsCount () {return animations.GetSize ();}
+  iSkeletonAnimation *GetAnimation (size_t idx);
+  /** @} */
+};
+
+class csCal3dSkeletonBoneFactory : public 
+  scfImplementation1<csCal3dSkeletonBoneFactory, iSkeletonBoneFactory>
+{
+
+  CalCoreBone *core_bone;
+  csWeakRef<csCal3dSkeletonFactory> skeleton_factory;
+  csWeakRefArray<iSkeletonBoneFactory> children;
+  csWeakRef<iSkeletonBoneFactory> parent;
+  csBox3 skin_box;
+
+  csReversibleTransform local_transform;
+  csReversibleTransform global_transform;
+
+public:
+
+  csCal3dSkeletonBoneFactory (CalCoreBone *core_bone, csCal3dSkeletonFactory* skelfact);
+  bool Initialize ();
+  CalCoreBone *GetCoreBone () {return core_bone;}
+
+  /**\name iSkeletonBoneFactory implementation
+   * @{ */
+  const char* GetName () const {return core_bone->getName ().c_str ();}
+  void SetName (const char* name) {}
+  csReversibleTransform &GetTransform () {return local_transform;}
+  void SetTransform (const csReversibleTransform &transform) {;}
+  csReversibleTransform &GetFullTransform () {return global_transform;}
+  void SetParent (iSkeletonBoneFactory *parent) {;}
+  iSkeletonBoneFactory *GetParent () {return 0;}
+  size_t GetChildrenCount () {return children.GetSize ();}
+  iSkeletonBoneFactory *GetChild (size_t i) {return 0;}
+  iSkeletonBoneFactory *FindChild (const char *name) {return 0;}
+  size_t FindChildIndex (iSkeletonBoneFactory *child) {return 0;}
+  void SetSkinBox (csBox3 & box) {;}
+  csBox3 &GetSkinBox () {return skin_box;}
+  iSkeletonBoneRagdollInfo *GetRagdollInfo () {return 0;}
+  /** @} */
+};
+
+#define CAL_TIME_2_CS_TIME(time) time * 1000
+#define CS_TIME_2_CAL_TIME(time) ((float)time) / 1000
+class csCal3dSkeletonAnimation;
+
+class csCal3dAnimationKeyFrame : public 
+  scfImplementation1<csCal3dAnimationKeyFrame, iSkeletonAnimationKeyFrame>
+{
+  csCal3dSkeletonAnimation *animation;
+  csArray<iSkeletonBoneFactory *> bones;
+  csArray<CalCoreKeyframe *> bones_key_frames;
+  csReversibleTransform trans;
+  
+public:
+
+  csCal3dAnimationKeyFrame (csCal3dSkeletonAnimation *animation) : 
+      scfImplementationType(this), animation(animation) {;}
+ 
+  /**\name iSkeletonAnimationKeyFrame implementation
+   * @{ */
+  const char* GetName () const {return "";}
+  void SetName (const char* name) {;}
+  csTicks GetDuration () {return 0;}
+  void SetDuration (csTicks time);
+  size_t GetTransformsCount() {return 0;}
+  void AddTransform (iSkeletonBoneFactory *bone,csReversibleTransform &transform,
+    bool relative = false);
+  csReversibleTransform &GetTransform (iSkeletonBoneFactory *bone) {return trans;}
+  bool GetTransform (iSkeletonBoneFactory *bone, csReversibleTransform &dst_trans) {return false;}
+  void SetTransform(iSkeletonBoneFactory *bone, csReversibleTransform &transform) {;}
+  bool GetKeyFrameData (iSkeletonBoneFactory *bone_fact,  csQuaternion & rot,
+    csVector3 & pos, csQuaternion &tangent, bool &relative) {return false;}
+  /** @} */
+};
+
+class csCal3dSkeletonAnimation : public scfImplementation1<csCal3dSkeletonAnimation, iSkeletonAnimation>
+{
+  CalCoreAnimation *animation;
+  csWeakRef<csCal3dSkeletonFactory> skel_fact;
+  csRefArray<csCal3dAnimationKeyFrame> frames;
+
+public:
+
+  csCal3dSkeletonAnimation (csCal3dSkeletonFactory *skel_fact);
+  virtual ~csCal3dSkeletonAnimation ();    
+
+  CalCoreAnimation *GetCoreAnimation () {return animation;}
+  csCal3dSkeletonFactory *GetSkeletonFactory () {return skel_fact;}
+
+  /**\name iSkeletonAnimation implementation
+   * @{ */
+  const char* GetName () const {return animation->getName ().c_str ();}
+  void SetName (const char* name) {animation->setName (std::string (name));}
+  csTicks GetTime () {return CAL_TIME_2_CS_TIME(animation->getDuration ());}
+  void SetTime (csTicks time) {animation->setDuration (CS_TIME_2_CAL_TIME(time));}
+  float GetSpeed () {return 0;}
+  void SetSpeed (float speed) {;}
+  void SetFactor (float factor) {;}
+  float GetFactor () {return 1;}
+  void SetLoop (bool loop) {;}
+  bool GetLoop () {return false;}
+  iSkeletonAnimationKeyFrame *CreateFrame (const char* name);
+  size_t GetFramesCount () {return frames.GetSize ();}
+  iSkeletonAnimationKeyFrame *GetFrame (size_t i) {return frames[i];}
+  size_t FindFrameIndex (const char *name) {return csArrayItemNotFound;}
+  void RemoveFrame (size_t i) {;}
+  void RemoveAllFrames () {;}
+  void RecalcSpline (){;}
+  /** @} */
+};
+
+class csCal3dSkeletonBone : public scfImplementation1<csCal3dSkeletonBone, iSkeletonBone>
+{
+  CalBone *bone;
+  csWeakRef<iSkeletonBoneFactory> factory;
+  csWeakRef<csCal3dSkeleton> skeleton;
+  csWeakRef<iSkeletonBone> parent;
+  csWeakRefArray<iSkeletonBone> children;
+  csBox3 skin_box;
+
+  csString name;
+  csReversibleTransform local_transform;
+  csReversibleTransform global_transform;
+
+public:
+  
+  csCal3dSkeletonBone (CalBone *bone, iSkeletonBoneFactory *factory,
+    csCal3dSkeleton *skeleton);
+
+  bool Initialize ();
+
+  /**\name iSkeletonBone implementation
+   * @{ */
+  const char* GetName () const {return name.GetData ();}
+  void SetName (const char* name) {csCal3dSkeletonBone::name = name;}
+  csReversibleTransform &GetTransform ();
+  void SetTransform (const csReversibleTransform &transform);
+  csReversibleTransform &GetFullTransform ();
+  void SetParent (iSkeletonBone *parent) {;} 
+  iSkeletonBone *GetParent () {return parent;}
+  size_t GetChildrenCount () {return children.GetSize ();}
+  iSkeletonBone *GetChild (size_t i) {return 0;}
+  iSkeletonBone *FindChild (const char *name) {return 0;}
+  size_t FindChildIndex (iSkeletonBone *child) {return 0;}
+  void SetSkinBox (csBox3 &box) {;}
+  csBox3 &GetSkinBox () {return skin_box;}
+  void SetUpdateCallback (iSkeletonBoneUpdateCallback *callback) {;}
+  iSkeletonBoneUpdateCallback *GetUpdateCallback () {return 0;}
+  iSkeletonBoneFactory *GetFactory () {return factory;}
+  void SetTransformMode (csBoneTransformType mode) {;}
+  csBoneTransformType GetTransformMode () {return CS_BTT_SCRIPT;}
+  /** @} */
+};
+
+class csCal3dSkeleton : public scfImplementation1<csCal3dSkeleton, iSkeleton>
+{
+  CalSkeleton* skeleton;
+  csString name;
+  csRefArray<csCal3dSkeletonBone> bones;
+  csWeakRef<csCal3dSkeletonFactory> skeleton_factory;
+  csRefArray<iSkeletonUpdateCallback> update_callbacks;
+  csHash<size_t, uint> bones_names;
+  csWeakRef<iSkeletonGraveyard> graveyard;
+  csSpriteCal3DMeshObject *mesh_object;
+
+public:
+
+  csCal3dSkeleton (CalSkeleton* skeleton, csCal3dSkeletonFactory* skel_factory, 
+    csSpriteCal3DMeshObject *mesh_object);
+
+  /**\name iSkeleton implementation
+   * @{ */
+  const char* GetName () const {return name.GetData ();}
+  void SetName (const char* name) {csCal3dSkeleton::name = name;}
+  size_t GetBonesCount () {return bones.GetSize ();}
+  iSkeletonBone *GetBone (size_t i) {return bones[i];}
+  iSkeletonBone *FindBone (const char *name);
+  size_t FindBoneIndex (const char *name);
+  iSkeletonAnimation* Execute (const char *scriptname) {return 0;}
+  iSkeletonAnimation* Append (const char *scriptname) {return 0;}
+  void ClearPendingAnimations () {;}
+  void ClearPendingScripts () {ClearPendingAnimations ();}
+  size_t GetAnimationsCount () {return 0;}
+  size_t GetScriptsCount () {return GetAnimationsCount ();}
+  iSkeletonAnimation* GetAnimation (size_t i) {return 0;}
+  iSkeletonAnimation* GetScript (size_t i) {return GetAnimation (i);}
+  iSkeletonAnimation* FindAnimation (const char *scriptname) {return 0;}
+  iSkeletonAnimation* FindScript (const char *scriptname) {return FindAnimation (scriptname);}
+  iSkeletonSocket* FindSocket (const char *socketname) {return 0;}
+  void StopAll () {;}
+  void Stop (const char* scriptname) {;}
+  iSkeletonFactory *GetFactory () {return skeleton_factory;}
+  void SetAnimationCallback (iSkeletonAnimationCallback *cb) {;}
+  void SetScriptCallback (iSkeletonAnimationCallback *cb) {SetAnimationCallback (cb);}
+  size_t AddUpdateCallback(iSkeletonUpdateCallback *update_callback) 
+  {return update_callbacks.Push (update_callback);}
+  size_t GetUpdateCallbacksCount () 
+  {return update_callbacks.GetSize ();}
+  iSkeletonUpdateCallback *GetUpdateCallback (size_t callback_idx) 
+  {return update_callbacks[callback_idx];}
+  void RemoveUpdateCallback(size_t callback_idx) 
+  {update_callbacks.DeleteIndex (callback_idx);}
+  iSkeletonAnimationInstance *Play (const char *animation_name) {return 0;}
+  void Stop (iSkeletonAnimationInstance *anim_instance) {;}
+  bool UpdateAnimation (csTicks current_time) {return mesh_object->Advance (current_time);} 
+  void UpdateBones () {skeleton->calculateState ();}
+  /** @} */
+
+  bool UpdatedByGraveyard () {return graveyard != 0;}
+  void UpdateNotify (const csTicks &current_ticks);
+
+};
+
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * Sprite Cal3D type. This is the plugin you have to use to create instances
  * of csSpriteCal3DMeshObjectFactory.
  */
-class csSpriteCal3DMeshObjectType : 
+class csSpriteCal3DMeshObjectType :
   public scfImplementation2<csSpriteCal3DMeshObjectType,
 			    iMeshObjectType,
 			    iComponent>
@@ -823,6 +1104,7 @@ public:
   csRef<iPolygonMesh> nullPolyMesh;
 };
 
-} // namespace cspluginSprCal3d
+}
+CS_PLUGIN_NAMESPACE_END(SprCal3d)
 
 #endif // __CS_SPRCAL3D_H__
