@@ -19,9 +19,10 @@
 #ifndef __CS_TERRFUNC_H__
 #define __CS_TERRFUNC_H__
 
-#include "csgeom/objmodel.h"
+#include "cstool/objmodel.h"
 #include "csgeom/transfrm.h"
 #include "csgeom/vector3.h"
+#include "igeom/trimesh.h"
 #include "cstool/rendermeshholder.h"
 #include "csutil/bitarray.h"
 #include "csutil/cscolor.h"
@@ -30,12 +31,13 @@
 #include "csutil/list.h"
 #include "csutil/refarr.h"
 #include "csutil/sysfunc.h"
+#include "csutil/scf_implementation.h"
 #include "csutil/thread.h"
 #include "csutil/weakref.h"
 #include "iengine/lightmgr.h"
 #include "iengine/mesh.h"
 #include "iengine/shadcast.h"
-#include "igeom/objmodel.h"
+#include "imesh/objmodel.h"
 #include "imesh/lighting.h"
 #include "imesh/object.h"
 #include "imesh/terrain.h"
@@ -47,14 +49,18 @@
 struct iEngine;
 struct iMaterialWrapper;
 struct iObjectRegistry;
+class csSegment3;
+
+CS_PLUGIN_NAMESPACE_BEGIN(BruteBlock)
+{
+
 class csTerrainQuad;
 class csTerrainObject;
 class csTerrainFactory;
-class csSegment3;
 
 /**
-* This is one block in the terrain.
-*/
+ * This is one block in the terrain.
+ */
 class csTerrBlock : public csRefCount
 {
 public:
@@ -108,6 +114,9 @@ public:
 public:
   csTerrBlock (csTerrainObject *terr);
   ~csTerrBlock ();
+  
+  /// Load data from Former
+  void LoadData ();
 
   /// Generate mesh
   void SetupMesh ();
@@ -115,11 +124,14 @@ public:
   /// Detach the node from the tree
   void Detach ();
 
+  /// Set all child neighbours that equal a to b
+  void ReplaceChildNeighbours(csTerrBlock *a, csTerrBlock *b);
+
   /// Split block in 4 children
-  void Split ();
+  bool Split ();
 
   /// Merge block
-  void Merge ();
+  bool Merge ();
 
   /// Checks if something needs to be merged or split
   void CalcLOD ();
@@ -138,90 +150,6 @@ public:
 };
 
 
-#if 0
-// The block builder is not currently used.
-class csBlockBuilder : public csRunnable
-{
-private:
-  csTerrainObject* terr;
-  csList< csRef<csTerrBlock> > pendingbuilds;
-  int buildcount;
-  int refcount;
-  bool run;
-  csRef<csCondition> processlock;
-  csRef<csMutex> listlock;
-
-  void Process ()
-  {
-    while (true)
-    {
-      csRef<csTerrBlock> block;
-      if (listlock->LockWait ())
-      {
-        if (pendingbuilds.IsEmpty ())
-          break;
-
-        block = pendingbuilds.Last ();
-        pendingbuilds.PopBack ();
-        listlock->Release ();
-      }
-
-      if (block->GetRefCount ()>1)
-        block->SetupMesh ();
-    }
-  }
-
-public:
-
-  csBlockBuilder::csBlockBuilder (csTerrainObject *t)
-  {
-    processlock = csCondition::Create ();
-    listlock = csMutex::Create ();
-    run = true;
-    terr = t;
-    refcount = 1;
-    buildcount = 0;
-  }
-
-  virtual csBlockBuilder::~csBlockBuilder ()
-  {
-  }
-
-  virtual void Run ()
-  {
-    listlock->LockWait ();
-    while (run)
-    {
-      processlock->Wait (listlock);
-      listlock->Release ();
-      if (!run)
-        break;
-      Process ();
-    }
-  }
-
-  void BuildBlock (csTerrBlock *block)
-  {
-    if (listlock->LockWait ())
-    {
-      pendingbuilds.PushFront (block);
-      listlock->Release ();
-      processlock->Signal ();
-    }
-  }
-
-  void Stop ()
-  {
-    run = false;
-    processlock->Signal ();
-  }
-
-  virtual void IncRef() {refcount++;}
-  virtual void DecRef() {refcount--;}
-  virtual int GetRefCount() {return refcount;}
-};
-#endif
-
 /**
  * An array giving shadow information for a pseudo-dynamic light.
  */
@@ -239,7 +167,15 @@ public:
   }
 };
 
-class csTerrainObject : public iMeshObject
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
+class csTerrainObject : 
+  public scfImplementationExt4<csTerrainObject,
+                               csObjectModel,
+                               iMeshObject,
+                               iShadowReceiver,
+                               iLightingInfo,
+                               iTerrainObjectState>
 {
 private:
   friend class csTerrBlock;
@@ -261,6 +197,7 @@ private:
   csRefArray<iImage> alphas;
   csRef<iShaderVariableContext> baseContext;
   csRefArray<iShaderVariableContext> paletteContexts;
+  bool materialAlphaMaps;
 
   float error_tolerance;
   float lod_distance;
@@ -278,9 +215,6 @@ private:
   float block_minsize;
   int block_res;
 
-  //csBlockBuilder *builder;
-  csRef<csThread> buildthread;
-
   csRef<iTerraFormer> terraformer;
 
   //bool use_singlemap;
@@ -288,6 +222,9 @@ private:
   csArray<csBitArray> globalMaterialsUsed;
   int materialMapW, materialMapH;
   float wm, hm;	// Scales to map between material map and object space.
+  
+  csString materialMapFile;
+  bool materialMapRaw;
 
   csDirtyAccessArray<csRenderMesh*>* returnMeshes;
   csRenderMeshHolder rmHolder;
@@ -323,6 +260,7 @@ private:
   * mesh, the LOD meshes, normals, ...
   */
   void SetupObject ();
+  csTerrainObject *neighbor[4];
 
   //=============
   // Lighting.
@@ -344,6 +282,10 @@ private:
   void UpdateColors (iMovable* movable);
   //=============
 
+  bool LODCalc;
+
+  csStringID stringVertices;
+
 public:
   CS_LEAKGUARD_DECLARE (csTerrainObject);
 
@@ -351,6 +293,15 @@ public:
   csTerrainObject (iObjectRegistry* object_reg, csTerrainFactory* factory);
   virtual ~csTerrainObject ();
 
+  /// Set the neighbor above (1 on Z axis) (For use in combining multiple brute meshes)
+  void SetTopNeighbor(iTerrainObjectState *top);
+  /// Set the neighbor to the right (1 on X axis) (For use in combining multiple brute meshes)
+  void SetRightNeighbor(iTerrainObjectState *right);
+  /// Set the neighbor to the left (-1 on X axis) (For use in combining multiple brute meshes)
+  void SetLeftNeighbor(iTerrainObjectState *left);
+  /// Set the neighbor below (-1 on Z axis) (For use in combining multiple brute meshes)
+  void SetBottomNeighbor(iTerrainObjectState *bottom);
+
   const csDirtyAccessArray<csColor>& GetStaticColors () const
   {
     return staticColors;
@@ -409,10 +360,14 @@ public:
   int CollisionDetect (iMovable *m, csTransform *p);
 
   void GetObjectBoundingBox (csBox3& bbox);
+  const csBox3& GetObjectBoundingBox ();
   void SetObjectBoundingBox (const csBox3& bbox);
-  void GetRadius (csVector3& rad, csVector3& cent);
+  void GetRadius (float& rad, csVector3& cent);
+
+  bool SetCurrentMaterialAlphaMaps (const csArray<csArray<char> >& data, 
+                                    int w, int h);
+  bool SetCurrentMaterialMap (const csArray<char>& data, int x, int y);
 
-  SCF_DECLARE_IBASE;
   ///--------------------- iMeshObject implementation ---------------------
 
   virtual csFlags& GetFlags () { return flags; }
@@ -438,7 +393,8 @@ public:
     return vis_cb;
   }
 
-  virtual void NextFrame (csTicks, const csVector3& /*pos*/) { }
+  virtual void NextFrame (csTicks, const csVector3& /*pos*/,
+    uint /*currentFrame*/) { }
 
   virtual void HardTransform (const csReversibleTransform&) { }
   virtual bool SupportsHardTransform () const { return false; }
@@ -457,6 +413,9 @@ public:
     csVector3& isect, float* pr, int* polygon_idx = 0,
     iMaterialWrapper** material = 0);
 
+  virtual void BuildDecal(const csVector3* pos, float decalRadius,
+          iDecalBuilder* decalBuilder);
+
   virtual void InvalidateMaterialHandles () { }
   virtual void PositionChild (iMeshObject* /*child*/,
   	csTicks /*current_time*/) { }
@@ -478,14 +437,12 @@ public:
   void SetStaticLighting (bool enable);
 
   //------------------ iPolygonMesh interface implementation ----------------//
-  struct PolyMesh : public iPolygonMesh
+  struct PolyMesh : public scfImplementation1<PolyMesh, iPolygonMesh>
   {
   private:
     csTerrainObject* terrain;
     csFlags flags;
   public:
-    SCF_DECLARE_IBASE;
-
     void SetTerrain (csTerrainObject* t)
     {
       terrain = t;
@@ -505,178 +462,82 @@ public:
     virtual csFlags& GetFlags () { return flags;  }
     virtual uint32 GetChangeNumber() const { return 0; }
 
-    PolyMesh ()
-    { SCF_CONSTRUCT_IBASE (0); }
+    PolyMesh () : scfImplementationType (this)
+    { }
     virtual ~PolyMesh ()
-    { SCF_DESTRUCT_IBASE (); }
-  } scfiPolygonMesh;
+    { }
+  };
+  csRef<PolyMesh> polygonMesh;
   friend struct PolyMesh;
 
-  //------------------------- iObjectModel implementation ----------------
-  class eiObjectModel : public csObjectModel
+  //------------------ iTriangleMesh interface implementation ----------------//
+  struct TriMesh : public scfImplementation1<TriMesh, iTriangleMesh>
   {
-    SCF_DECLARE_EMBEDDED_IBASE (csTerrainObject);
-    virtual void GetObjectBoundingBox (csBox3& bbox)
+  private:
+    csTerrainObject* terrain;
+    csFlags flags;
+  public:
+    void SetTerrain (csTerrainObject* t)
     {
-      scfParent->GetObjectBoundingBox (bbox);
+      terrain = t;
     }
-    virtual void SetObjectBoundingBox (const csBox3& bbox)
-    {
-      scfParent->SetObjectBoundingBox (bbox);
-    }
-    virtual void GetRadius (csVector3& rad, csVector3& cent)
-    {
-      scfParent->GetRadius (rad, cent);
-    }
-  } scfiObjectModel;
-  friend class ObjectModel;
+    void Cleanup () { }
 
-  virtual iObjectModel* GetObjectModel () { return &scfiObjectModel; }
+    virtual size_t GetVertexCount ();
+    virtual csVector3* GetVertices ();
+    virtual size_t GetTriangleCount ();
+    virtual csTriangle* GetTriangles ();
+    virtual void Lock () { }
+    virtual void Unlock () { }
 
-  //-------------------- iShadowReceiver interface implementation ----------
-  struct ShadowReceiver : public iShadowReceiver
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csTerrainObject);
-    virtual void CastShadows (iMovable* movable, iFrustumView* fview)
-    {
-      scfParent->CastShadows (movable, fview);
-    }
-  } scfiShadowReceiver;
-  friend struct ShadowReceiver;
+    virtual csFlags& GetFlags () { return flags;  }
+    virtual uint32 GetChangeNumber() const { return 0; }
 
-  //------------------------- iLightingInfo interface -------------------------
-  struct LightingInfo : public iLightingInfo
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csTerrainObject);
-    virtual void InitializeDefault (bool clear)
-    {
-      scfParent->InitializeDefault (clear);
-    }
-    virtual bool ReadFromCache (iCacheManager* cache_mgr)
-    {
-      return scfParent->ReadFromCache (cache_mgr);
-    }
-    virtual bool WriteToCache (iCacheManager* cache_mgr)
-    {
-      return scfParent->WriteToCache (cache_mgr);
-    }
-    virtual void PrepareLighting ()
-    {
-      scfParent->PrepareLighting ();
-    }
-    virtual void LightChanged (iLight* light)
-    {
-      scfParent->LightChanged (light);
-    }
-    virtual void LightDisconnect (iLight* light)
-    {
-      scfParent->LightDisconnect (light);
-    }
-    virtual void DisconnectAllLights ()
-    {
-      scfParent->DisconnectAllLights ();
-    }
-  } scfiLightingInfo;
-  friend struct LightingInfo;
+    TriMesh () : scfImplementationType (this)
+    { }
+    virtual ~TriMesh ()
+    { }
+  };
+  friend struct TriMesh;
 
-  //------------------ iTerrainObjectState implementation ----------------
+  /**\name iObjectModel implementation
+   * @{ */
+  virtual iTerraFormer* GetTerraFormerColldet ()
+  { return terraformer; }
+  virtual iTerrainSystem* GetTerrainColldet () { return 0; }
+  /** @} */
 
+  virtual iObjectModel* GetObjectModel () { return this; }
+
+  /**\name iTerrainObjectState implementation
+   * @{ */
   bool SetMaterialPalette (const csArray<iMaterialWrapper*>& pal);
   csArray<iMaterialWrapper*> GetMaterialPalette ();
+  const csArray<iMaterialWrapper*> &GetMaterialPalette () const { return palette; }
   bool SetMaterialAlphaMaps (const csArray<csArray<char> >& data, int w, int h);
   bool SetMaterialAlphaMaps (const csArray<iImage*>& maps);
   bool SetMaterialMap (const csArray<char>& data, int x, int y);
   bool SetMaterialMap (iImage* map);
   bool SetLODValue (const char* parameter, float value);
-  float GetLODValue (const char* parameter);
-
-  class eiTerrainObjectState : public iTerrainObjectState
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csTerrainObject);
-    virtual bool SetMaterialPalette (const csArray<iMaterialWrapper*>& pal)
-    {
-      return scfParent->SetMaterialPalette (pal);
-    }
-
-    virtual const csArray<iMaterialWrapper*> &GetMaterialPalette () const
-    {
-      return scfParent->palette;
-    }
-
-    virtual bool SetMaterialMap (const csArray<char> &data, int x, int y)
-    {
-      return scfParent->SetMaterialMap (data, x, y);
-    }
-
-    virtual bool SetMaterialMap (iImage* map)
-    {
-      return scfParent->SetMaterialMap (map);
-    }
-
-    virtual bool SetMaterialAlphaMaps (const csArray<csArray<char> >& data,
-    	int w, int h)
-    {
-      return scfParent->SetMaterialAlphaMaps (data, w, h);
-    }
-
-    virtual bool SetMaterialAlphaMaps (const csArray<iImage*>& maps)
-    {
-      return scfParent->SetMaterialAlphaMaps (maps);
-    }
-
-    virtual bool SetLODValue (const char* parameter, float value)
-    {
-      return scfParent->SetLODValue (parameter, value);
-    }
-
-    virtual float GetLODValue (const char* parameter) const
-    {
-      return scfParent->GetLODValue (parameter);
-    }
-
-    virtual bool SaveState (const char* /*filename*/)
-    {
-      return true;
-    }
-
-    virtual bool RestoreState (const char* /*filename*/)
-    {
-      return true;
-    }
-
-    virtual int CollisionDetect (iMovable *m, csTransform *p)
-    {
-      return scfParent->CollisionDetect (m, p);
-    }
-
-    virtual void SetStaticLighting (bool enable)
-    {
-      scfParent->SetStaticLighting (enable);
-    }
-
-    virtual bool GetStaticLighting ()
-    {
-      return scfParent->staticlighting;
-    }
-
-    virtual void SetCastShadows (bool enable)
-    {
-      scfParent->castshadows = enable;
-    }
-
-    virtual bool GetCastShadows ()
-    {
-      return scfParent->castshadows;
-    }
-
-  } scfiTerrainObjectState;
-  friend class eiTerrainObjectState;
+  float GetLODValue (const char* parameter) const;
+  void SetMaterialMapFile (const char* file, int width, int height, bool raw);
+  const char* GetMaterialMapFile (int& width, int& height, bool& raw);
+  bool SaveState (const char* /*filename*/) { return true; }
+  bool RestoreState (const char* /*filename*/) { return true; }
+  bool GetStaticLighting () { return staticlighting; }
+  void SetCastShadows (bool enable) { castshadows = enable; }
+  bool GetCastShadows () { return castshadows; }
+  /** @} */
 };
 
 /**
 * Factory for terrain.
 */
-class csTerrainFactory : public iMeshObjectFactory
+class csTerrainFactory : 
+  public scfImplementationExt2<csTerrainFactory,
+                               csObjectModel,
+                               iMeshObjectFactory,
+                               iTerrainFactoryState>
 {
 private:
   iMeshFactoryWrapper* logparent;
@@ -684,6 +545,7 @@ private:
   iMeshObjectType* brute_type;
 
   csFlags flags;
+  csBox3 obj_bbox;
 
 public:
   CS_LEAKGUARD_DECLARE (csTerrainFactory);
@@ -706,8 +568,6 @@ public:
   /// Destructor.
   virtual ~csTerrainFactory ();
 
-  SCF_DECLARE_IBASE;
-
   virtual csPtr<iMeshObject> NewInstance ();
   virtual csPtr<iMeshObjectFactory> Clone () { return 0; }
   virtual void HardTransform (const csReversibleTransform&) { }
@@ -718,82 +578,48 @@ public:
   { return logparent; }
   virtual iMeshObjectType* GetMeshObjectType () const { return brute_type; }
 
+  /**\name iTerrainFactoryState implementation
+   * @{ */
   void SetTerraFormer (iTerraFormer* form);
   iTerraFormer* GetTerraFormer ();
   void SetSamplerRegion (const csBox2& region);
   const csBox2& GetSamplerRegion ();
+  bool SaveState (const char* /*filename*/) { return true; }
+  bool RestoreState (const char* /*filename*/) { return true; }
+  /** @} */
 
   virtual csFlags& GetFlags () { return flags; }
 
-  class eiTerrainFactoryState : public iTerrainFactoryState
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csTerrainFactory);
-    virtual void SetTerraFormer (iTerraFormer* form)
-    {
-      scfParent->SetTerraFormer (form);
-    }
+  /**\name iObjectModel implementation
+   * @{ */
+  iTerraFormer* GetTerraFormerColldet () { return terraformer; }
+  void GetObjectBoundingBox (csBox3& /*bbox*/) { }
+  const csBox3& GetObjectBoundingBox () { return obj_bbox; }
+  void SetObjectBoundingBox (const csBox3& /*bbox*/) { }
+  void GetRadius (float& /*rad*/, csVector3& /*cent*/) { }
+  /** @} */
 
-    virtual iTerraFormer* GetTerraFormer ()
-    {
-      return scfParent->GetTerraFormer ();
-    }
-
-    virtual void SetSamplerRegion (const csBox2& region)
-    {
-      scfParent->SetSamplerRegion (region);
-    }
-
-    virtual const csBox2& GetSamplerRegion ()
-    {
-      return scfParent->GetSamplerRegion ();
-    }
-
-    virtual bool SaveState (const char* /*filename*/)
-    {
-      return true;
-    }
-    virtual bool RestoreState (const char* /*filename*/)
-    {
-      return true;
-    }
-  } scfiTerrainFactoryState;
-
-  friend class eiTerrainFactoryState;
-
-  //------------------------- iObjectModel implementation ----------------
-  class eiObjectModel : public csObjectModel
-  {
-    SCF_DECLARE_EMBEDDED_IBASE (csTerrainFactory);
-    virtual void GetObjectBoundingBox (csBox3& /*bbox*/)
-    {
-      //scfParent->GetObjectBoundingBox (bbox);
-    }
-    virtual void SetObjectBoundingBox (const csBox3& /*bbox*/)
-    {
-      //scfParent->SetObjectBoundingBox (bbox);
-    }
-    virtual void GetRadius (csVector3& /*rad*/, csVector3& /*cent*/)
-    {
-      //scfParent->GetRadius (rad, cent);
-    }
-  } scfiObjectModel;
-  friend class eiObjectModel;
-
-  virtual iObjectModel* GetObjectModel () { return &scfiObjectModel; }
+  virtual iObjectModel* GetObjectModel () { return this; }
+  virtual bool SetMaterialWrapper (iMaterialWrapper*) { return false; }
+  virtual iMaterialWrapper* GetMaterialWrapper () const { return 0; }
+  virtual void SetMixMode (uint) { }
+  virtual uint GetMixMode () const { return 0; }
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
 * TerrFunc type. This is the plugin you have to use to create instances
 * of csTerrainFactory.
 */
-class csTerrainObjectType : public iMeshObjectType
+class csTerrainObjectType :
+  public scfImplementation2<csTerrainObjectType,
+    iMeshObjectType, iComponent>
 {
 private:
   iObjectRegistry *object_reg;
 
 public:
-  SCF_DECLARE_IBASE;
-
   /// Constructor.
   csTerrainObjectType (iBase*);
   /// Destructor.
@@ -802,13 +628,11 @@ public:
   /// create a new factory.
   virtual csPtr<iMeshObjectFactory> NewFactory ();
 
-  struct eiComponent : public iComponent
-  {
-    SCF_DECLARE_EMBEDDED_IBASE(csTerrainObjectType);
-    virtual bool Initialize (iObjectRegistry* p)
-    { scfParent->object_reg = p; return true; }
-  } scfiComponent;
-  friend struct eiComponent;
+  virtual bool Initialize (iObjectRegistry* p)
+  { this->object_reg = p; return true; }
 };
 
+}
+CS_PLUGIN_NAMESPACE_END(BruteBlock)
+
 #endif // __CS_TERRFUNC_H__
