@@ -20,6 +20,7 @@
 
 #include "csutil/cscolor.h"
 #include "csutil/sysfunc.h"
+#include "csutil/stringarray.h"
 #include "csutil/refarr.h"
 
 #include "iengine/material.h"
@@ -31,6 +32,7 @@
 #include "imesh/terrain.h"
 #include "iutil/comp.h"
 #include "iutil/document.h"
+#include "iutil/object.h"
 #include "iutil/objreg.h"
 #include "iutil/plugin.h"
 #include "iutil/vfs.h"
@@ -56,34 +58,22 @@ enum
   XMLTOKEN_CASTSHADOWS
 };
 
-SCF_IMPLEMENT_IBASE (csTerrainFactoryLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csTerrainFactoryLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csTerrainFactoryLoader)
 
-csTerrainFactoryLoader::csTerrainFactoryLoader (iBase* parent)
+csTerrainFactoryLoader::csTerrainFactoryLoader (iBase* parent) :
+  scfImplementationType(this, parent)
 {
-  SCF_CONSTRUCT_IBASE (parent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
 }
 
 csTerrainFactoryLoader::~csTerrainFactoryLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csTerrainFactoryLoader::Initialize (iObjectRegistry* objreg)
 {
   object_reg = objreg;
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
-  vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
+  vfs = csQueryRegistry<iVFS> (object_reg);
 
   xmltokens.Register ("plugin", XMLTOKEN_PLUGIN);
   xmltokens.Register ("terraformer", XMLTOKEN_TERRAFORMER);
@@ -94,8 +84,8 @@ bool csTerrainFactoryLoader::Initialize (iObjectRegistry* objreg)
 csPtr<iBase> csTerrainFactoryLoader::Parse (iDocumentNode* node,
   iStreamSource*, iLoaderContext* /*ldr_context*/, iBase* /*context*/)
 {
-  csRef<iPluginManager> plugin_mgr = CS_QUERY_REGISTRY (object_reg,
-    iPluginManager);
+  csRef<iPluginManager> plugin_mgr = 
+    csQueryRegistry<iPluginManager> (object_reg);
 
   csRef<iMeshObjectFactory> fact;
   csRef<iTerrainFactoryState> state;
@@ -112,13 +102,8 @@ csPtr<iBase> csTerrainFactoryLoader::Parse (iDocumentNode* node,
       case XMLTOKEN_PLUGIN:
       {
         const char* pluginname = child->GetContentsValue ();
-        csRef<iMeshObjectType> type = CS_QUERY_PLUGIN_CLASS (plugin_mgr,
-          pluginname, iMeshObjectType);
-        if (!type)
-        {
-          type = CS_LOAD_PLUGIN (plugin_mgr, 
-            pluginname, iMeshObjectType);
-        }
+        csRef<iMeshObjectType> type = csLoadPluginCheck<iMeshObjectType> (
+          plugin_mgr, pluginname);
         if (!type)
         {
           synldr->ReportError ("crystalspace.terrain.loader.factory",
@@ -131,7 +116,7 @@ csPtr<iBase> csTerrainFactoryLoader::Parse (iDocumentNode* node,
           synldr->ReportError ("crystalspace.terrain.loader.factory",
             node, "Could not create a factory from %s", pluginname);
         }
-        state = SCF_QUERY_INTERFACE (fact, iTerrainFactoryState);
+        state = scfQueryInterface<iTerrainFactoryState> (fact);
         if (!state)
         {
           synldr->ReportError ("crystalspace.terrain.loader.factory",
@@ -142,8 +127,8 @@ csPtr<iBase> csTerrainFactoryLoader::Parse (iDocumentNode* node,
       case XMLTOKEN_TERRAFORMER:
       {
         const char* name = child->GetContentsValue ();
-        csRef<iTerraFormer> form = CS_QUERY_REGISTRY_TAG_INTERFACE (object_reg,
-	  name, iTerraFormer);
+        csRef<iTerraFormer> form = csQueryRegistryTagInterface<iTerraFormer>
+	  (object_reg, name);
 	if (form == 0) 
 	{
           synldr->ReportError ("crystalspace.terrain.factory.loader",
@@ -175,77 +160,90 @@ csPtr<iBase> csTerrainFactoryLoader::Parse (iDocumentNode* node,
   return csPtr<iBase> (fact);
 }
 
-SCF_IMPLEMENT_IBASE (csTerrainFactorySaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csTerrainFactorySaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csTerrainFactorySaver)
 
-csTerrainFactorySaver::csTerrainFactorySaver (iBase* parent)
+csTerrainFactorySaver::csTerrainFactorySaver (iBase* parent) :
+  scfImplementationType(this, parent)
 { 
-  SCF_CONSTRUCT_IBASE (parent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
 }
 
 csTerrainFactorySaver::~csTerrainFactorySaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csTerrainFactorySaver::Initialize (iObjectRegistry* objreg)
 {
   object_reg = objreg;
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
-//TBD
-bool csTerrainFactorySaver::WriteDown (iBase* /*obj*/, iDocumentNode* parent,
+
+bool csTerrainFactorySaver::WriteDown (iBase* obj, iDocumentNode* parent,
 	iStreamSource*)
 {
   if (!parent) return false; //you never know...
   
   csRef<iDocumentNode> paramsNode = parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
   paramsNode->SetValue("params");
-  paramsNode->CreateNodeBefore(CS_NODE_COMMENT, 0)->SetValue
-    ("iSaverPlugin not yet supported for terrain mesh");
-  paramsNode=0;
+  
+  if (obj)
+  {
+    csRef<iTerrainFactoryState> tfact = 
+      scfQueryInterface<iTerrainFactoryState> (obj);
+    csRef<iMeshObjectFactory> meshfact = 
+      scfQueryInterface<iMeshObjectFactory> (obj);
+    if (!tfact) return false;
+    if (!meshfact) return false;
+    
+    // Write plugin
+    csRef<iDocumentNode> pluginNode = 
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    pluginNode->SetValue("plugin");
+    
+    csRef<iFactory> factory = 
+      scfQueryInterface<iFactory> (meshfact->GetMeshObjectType());
+    const char* pluginname = factory->QueryClassID();
+    if (!(pluginname && *pluginname)) return false;
+    
+    pluginNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValue (pluginname);
+    
+    // Write terraformer
+    csRef<iDocumentNode> terraFormerNode = 
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    terraFormerNode->SetValue("terraformer");
+    
+    const char* terraformer = tfact->GetTerraFormer ()->QueryObject ()->GetName ();
+    terraFormerNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValue (terraformer);
+    
+    // Write sampleregion
+    csRef<iDocumentNode> sampleRegionNode = 
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    sampleRegionNode->SetValue("sampleregion");
+    
+    csBox2 box = tfact->GetSamplerRegion ();
+    csBox3 box3 (box.MinX(), box.MinY(), 0, box.MaxX(), box.MaxY(), 0);
+    synldr->WriteBox (sampleRegionNode, box3);
+  }
   
   return true;
 }
 
-SCF_IMPLEMENT_IBASE (csTerrainObjectLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csTerrainObjectLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csTerrainObjectLoader)
 
-csTerrainObjectLoader::csTerrainObjectLoader (iBase* parent)
+csTerrainObjectLoader::csTerrainObjectLoader (iBase* parent) :
+  scfImplementationType(this, parent)
 {
-  SCF_CONSTRUCT_IBASE (parent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
 }
 
 csTerrainObjectLoader::~csTerrainObjectLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csTerrainObjectLoader::Initialize (iObjectRegistry* objreg)
 {
   object_reg = objreg;
-  synldr = CS_QUERY_REGISTRY (object_reg, iSyntaxService);
-  vfs = CS_QUERY_REGISTRY (object_reg, iVFS);
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
+  vfs = csQueryRegistry<iVFS> (object_reg);
 
   xmltokens.Register ("factory", XMLTOKEN_FACTORY);
   xmltokens.Register ("color", XMLTOKEN_COLOR);
@@ -259,6 +257,14 @@ bool csTerrainObjectLoader::Initialize (iObjectRegistry* objreg)
   return true;
 }
 
+#define CHECK_MESH(m) \
+  if (!m) { \
+    synldr->ReportError ( \
+	"crystalspace.sprite3dloader.parse.unknownfactory", \
+	child, "Specify the factory first!"); \
+    return 0; \
+  }
+
 csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node, 
   iStreamSource*, iLoaderContext* ldr_context, iBase* /*context*/)
 {
@@ -266,10 +272,6 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
   csRef<iTerrainObjectState> state;
   bool palette_set = false;
   bool material_map_set = false;
-  bool one_material_map_used = false;
-
-  csRefArray<iImage> alphamaps_ref;
-  csArray<iImage*> alphamaps;
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -292,7 +294,7 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
           return 0;
         }
         mesh = fact->GetMeshObjectFactory ()->NewInstance ();
-        state = SCF_QUERY_INTERFACE (mesh, iTerrainObjectState);
+        state = scfQueryInterface<iTerrainObjectState> (mesh);
 	if (!state)
 	{
       	  synldr->ReportError (
@@ -312,6 +314,7 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
             child, "Error reading color value!");
           return 0;
         }
+	CHECK_MESH (mesh);
         mesh->SetColor (c);
         break;
       }
@@ -325,6 +328,7 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
             child, "Couldn't find material '%s'!", matname);
           return 0;
         }
+	CHECK_MESH (mesh);
         mesh->SetMaterialWrapper (mat);
         break;
       }
@@ -337,109 +341,28 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
             child, "Error parsing material palette!");
           return 0;
         }
+	CHECK_MESH (state);
         state->SetMaterialPalette (pal);
 	palette_set = true;
         break;
       }
       case XMLTOKEN_MATERIALMAP:
       {
-        if (!palette_set)
-	{
-          synldr->ReportError ("crystalspace.terrain.factory.loader",
-              child, "First set a material palette before <materialmap>!");
-          return 0;
-	}
-	if (alphamaps.Length () > 0)
-	{
-          synldr->ReportError ("crystalspace.terrain.factory.loader",
-              child, "You can't combine material alhpamaps with materialmap!");
-          return 0;
-	}
-	material_map_set = true;
-	one_material_map_used = true;
-        const char* imagefile = child->GetAttributeValue ("image");
-        const char *arrayfile = child->GetAttributeValue ("raw");
-        int width = child->GetAttributeValueAsInt ("width");
-        int height = child->GetAttributeValueAsInt ("height");
-        if (imagefile != 0)
-        {
-	  csRef<iLoader> loader = CS_QUERY_REGISTRY (object_reg, iLoader);
-          csRef<iImage> map = loader->LoadImage(imagefile,CS_IMGFMT_PALETTED8);
-          if (map == 0) 
-          {
-            synldr->ReportError ("crystalspace.terrain.factory.loader",
-              child, "Error reading in image file for heightmap '%s'", 
-              imagefile);
-            return 0;
-          }
-          state->SetMaterialMap (map);
-        }
-        else if (arrayfile != 0 && width != 0 && height != 0) 
-        {
-          csRef<iFile> file = vfs->Open (arrayfile, VFS_FILE_READ);
-          if (file == 0) 
-          {
-            synldr->ReportError ("crystalspace.terrain.factory.loader",
-              child, "Error reading in raw file for heightmap '%s'", 
-              arrayfile);
-            return 0;
-          }
-          csArray<char> array;
-          int index = 0;
-          while (!file->AtEOF())
-          {
-            file->Read (&array.GetExtend (index++), sizeof (char));
-          }
-          state->SetMaterialMap (array, width, height);
-        }
-        else
-        {
-          synldr->ReportError ("crystalpace.terrain.object.loader",
-            child, "No image or raw file specified for material map");
-          return 0;
-        }
+        synldr->ReportError ("crystalspace.terrain.factory.loader",
+            child, "Materialmaps are now handled by the Formers!");
+        return 0;
         break;
       }
       case XMLTOKEN_MATERIALALPHAMAP:
       {
-        if (!palette_set)
-	{
-          synldr->ReportError ("crystalspace.terrain.factory.loader",
-              child, "First set a material palette before <materialmap>!");
-          return 0;
-	}
-	if (one_material_map_used)
-	{
-          synldr->ReportError ("crystalspace.terrain.factory.loader",
-              child, "You are already using materialmap. Can't add alpha maps now!");
-          return 0;
-	}
-	material_map_set = true;
-        const char* imagefile = child->GetAttributeValue ("image");
-        if (imagefile != 0)
-        {
-	  csRef<iLoader> loader = CS_QUERY_REGISTRY (object_reg, iLoader);
-          csRef<iImage> map = loader->LoadImage(imagefile,CS_IMGFMT_PALETTED8);
-          if (map == 0) 
-          {
-            synldr->ReportError ("crystalspace.terrain.factory.loader",
-              child, "Error reading in image file for heightmap '%s'", 
-              imagefile);
-            return 0;
-          }
-	  alphamaps_ref.Push (map);
-	  alphamaps.Push (map);
-        }
-        else
-        {
-          synldr->ReportError ("crystalpace.terrain.object.loader",
-            child, "No image file specified for materialraw  p");
-          return 0;
-        }
+        synldr->ReportError ("crystalspace.terrain.factory.loader",
+            child, "Alphamaps are now handled by the Formers!");
+        return 0;
         break;
       }
       case XMLTOKEN_LODVALUE:
       {
+//@@@
         if (material_map_set)
 	{
           synldr->ReportError ("crystalspace.terrain.factory.loader",
@@ -454,6 +377,7 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
           return 0;
 	}
         float val = child->GetContentsValueAsFloat ();
+	CHECK_MESH (state);
 	state->SetLODValue (name, val);
 	break;
       }
@@ -462,6 +386,7 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
 	  bool staticLighting;
 	  if (!synldr->ParseBool (child, staticLighting, true))
 	    return 0;
+	  CHECK_MESH (state);
 	  state->SetStaticLighting (staticLighting);
 	}
 	break;
@@ -470,6 +395,7 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
 	  bool castShadows;
 	  if (!synldr->ParseBool (child, castShadows, true))
 	    return 0;
+	  CHECK_MESH (state);
 	  state->SetCastShadows (castShadows);
 	}
 	break;
@@ -479,11 +405,6 @@ csPtr<iBase> csTerrainObjectLoader::Parse (iDocumentNode* node,
     }
   }
 
-  if (alphamaps.Length () > 0)
-  {
-    state->SetMaterialAlphaMaps (alphamaps);
-  }
-
   return csPtr<iBase>(mesh);
 }
 
@@ -522,45 +443,155 @@ bool csTerrainObjectLoader::ParseMaterialPalette (iDocumentNode *node,
   return true;
 }
 
-SCF_IMPLEMENT_IBASE (csTerrainObjectSaver)
-  SCF_IMPLEMENTS_INTERFACE (iSaverPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csTerrainObjectSaver::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
-
 SCF_IMPLEMENT_FACTORY (csTerrainObjectSaver)
 
-csTerrainObjectSaver::csTerrainObjectSaver (iBase* parent)
+csTerrainObjectSaver::csTerrainObjectSaver (iBase* parent) :
+  scfImplementationType(this, parent)
 {
-  SCF_CONSTRUCT_IBASE (parent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
 }
 
 csTerrainObjectSaver::~csTerrainObjectSaver ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csTerrainObjectSaver::Initialize (iObjectRegistry *objreg)
 {
   object_reg = objreg;
+  synldr = csQueryRegistry<iSyntaxService> (object_reg);
   return true;
 }
-//TBD
-bool csTerrainObjectSaver::WriteDown (iBase* /*obj*/, iDocumentNode* parent,
+
+bool csTerrainObjectSaver::WriteDown (iBase* obj, iDocumentNode* parent,
 	iStreamSource*)
 {
   if (!parent) return false; //you never know...
   
   csRef<iDocumentNode> paramsNode = parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
   paramsNode->SetValue("params");
-  paramsNode->CreateNodeBefore(CS_NODE_COMMENT, 0)->SetValue
-    ("iSaverPlugin not yet supported for terrain mesh");
-  paramsNode=0;
+  
+  if (obj)
+  {
+    csRef<iTerrainObjectState> tmesh = 
+      scfQueryInterface<iTerrainObjectState> (obj);
+    csRef<iMeshObject> mesh = scfQueryInterface<iMeshObject> (obj);
+    if (!tmesh) return false;
+    if (!mesh) return false;
+
+    //Writedown Factory tag
+    iMeshFactoryWrapper* fact = mesh->GetFactory()->GetMeshFactoryWrapper ();
+    if (fact)
+    {
+      const char* factname = fact->QueryObject()->GetName();
+      if (factname && *factname)
+      {
+        csRef<iDocumentNode> factNode = 
+          paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        factNode->SetValue("factory");
+        factNode->CreateNodeBefore(CS_NODE_TEXT, 0)->SetValue(factname);
+      }    
+    }
+
+    //Writedown castshadow tag
+    if (!tmesh->GetCastShadows())
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0)->SetValue("castshadows");
+
+    //Writedown staticlighting tag
+    if (!tmesh->GetStaticLighting())
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0)
+        ->SetValue("staticlighting");
+
+    //Writedown Color tag
+    csColor col;
+    if (mesh->GetColor(col))
+    {
+      csRef<iDocumentNode> colorNode = 
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      colorNode->SetValue("color");
+      synldr->WriteColor(colorNode, col);
+    }
+
+    //Writedown Material tag
+    iMaterialWrapper* mat = mesh->GetMaterialWrapper();
+    if (mat)
+    {
+      const char* matname = mat->QueryObject()->GetName();
+      if (matname && *matname)
+      {
+        csRef<iDocumentNode> matNode = 
+          paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        matNode->SetValue("material");
+        csRef<iDocumentNode> matnameNode = 
+          matNode->CreateNodeBefore(CS_NODE_TEXT, 0);
+        matnameNode->SetValue(matname);
+      }    
+    }
+    
+    // Write materialpalette
+    csArray<iMaterialWrapper*> matpalette = tmesh->GetMaterialPalette ();
+    csRef<iDocumentNode> matpaletteNode = 
+      paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+    matpaletteNode->SetValue ("materialpalette");
+    for (size_t i = 0; i < matpalette.GetSize(); i++)
+    {
+      if (matpalette[i])
+      {
+        const char* matname = matpalette[i]->QueryObject ()->GetName ();;
+        
+        csRef<iDocumentNode> matNode =
+          matpaletteNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+        matNode->SetValue("material");
+        csRef<iDocumentNode> matnameNode = 
+          matNode->CreateNodeBefore(CS_NODE_TEXT, 0);
+        matnameNode->SetValue(matname);
+      }
+    }
+    
+    // Write lodvalues
+    csStringArray lodparams;
+    lodparams.Push ("splatting distance");
+    lodparams.Push ("block resolution");
+    lodparams.Push ("block split distance");
+    lodparams.Push ("minimum block size");
+    lodparams.Push ("cd resolution");
+    lodparams.Push ("cd lod cost");
+    lodparams.Push ("lightmap resolution");
+    
+    for (size_t i = 0; i < lodparams.GetSize(); i++)
+    {
+      float value = tmesh->GetLODValue (lodparams[i]);
+      
+      csRef<iDocumentNode> lodvalueNode =
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      lodvalueNode->SetValue("lodvalue");
+      
+      lodvalueNode->SetAttribute ("name", lodparams[i]);
+      
+      csRef<iDocumentNode> lodvalueValueNode = 
+        lodvalueNode->CreateNodeBefore(CS_NODE_TEXT, 0);
+      lodvalueValueNode->SetValueAsFloat(value);
+    }
+    
+    // Write materialmap
+    int matmapW, matmapH;
+    bool raw;
+    const char* matmapFile = tmesh->GetMaterialMapFile (matmapW, matmapH, raw);
+    if (matmapFile)
+    {
+      csRef<iDocumentNode> matmapNode =
+        paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
+      matmapNode->SetValue("materialmap");
+      if (raw)
+      {
+        matmapNode->SetAttribute ("raw", matmapFile);
+        matmapNode->SetAttributeAsInt ("width", matmapW);
+        matmapNode->SetAttributeAsInt ("height", matmapH);
+      }
+      else
+      {
+        matmapNode->SetAttribute ("image", matmapFile);
+      }
+    }
+  }
   
   return true;
 }
