@@ -24,7 +24,7 @@
 #include "csutil/array.h"
 #include "polygon.h"
 
-namespace cspluginThing
+CS_PLUGIN_NAMESPACE_BEGIN(Thing)
 {
 
 class csThingObjectType;
@@ -97,6 +97,7 @@ public:
   }
 };
 
-} // namespace cspluginThing
+}
+CS_PLUGIN_NAMESPACE_END(Thing)
 
 #endif // __CS_PARRAYS_H__
