@@ -20,11 +20,12 @@
 #define __CS_THING_H__
 
 #include "csgeom/csrect.h"
-#include "csgeom/objmodel.h"
+#include "cstool/objmodel.h"
 #include "csgeom/pmtools.h"
 #include "csgeom/subrec.h"
 #include "csgeom/transfrm.h"
-#include "csgfx/memimage.h"
+#include "igeom/trimesh.h"
+#include "csgfx/imagememory.h"
 #include "csgfx/shadervar.h"
 #include "csutil/array.h"
 #include "csutil/blockallocator.h"
@@ -49,6 +50,7 @@
 #include "iutil/dbghelp.h"
 #include "iutil/eventh.h"
 #include "iutil/pluginconfig.h"
+#include "ivideo/rendermesh.h"
 #include "ivideo/shader/shader.h"
 #include "ivideo/txtmgr.h"
 
@@ -66,7 +68,7 @@ struct iFrustumView;
 struct iMaterialWrapper;
 struct iPolygonBuffer;
 
-namespace cspluginThing
+CS_PLUGIN_NAMESPACE_BEGIN(Thing)
 {
 
 class csThing;
@@ -170,6 +172,73 @@ private:
   }
 };
 
+#include "csutil/win32/msvc_deprecated_warn_off.h"
+
+/**
+ * A helper class for iTriangleMesh implementations used by csThing.
+ */
+class TriMeshHelper : 
+  public scfImplementation1<TriMeshHelper, 
+			    iTriangleMesh>
+{
+public:
+  /**
+   * Make a triangle mesh helper.
+   */
+  TriMeshHelper (uint32 flag) : scfImplementationType (this), 
+    vertices (0), triangles (0), poly_flag (flag),
+    locked (0)
+  {
+  }
+  virtual ~TriMeshHelper ()
+  {
+    Cleanup ();
+  }
+
+  void Setup ();
+  void SetThing (csThingStatic* thing);
+
+  virtual size_t GetVertexCount ()
+  {
+    Setup ();
+    return num_verts;
+  }
+  virtual csVector3* GetVertices ()
+  {
+    Setup ();
+    return vertices;
+  }
+  virtual size_t GetTriangleCount ()
+  {
+    Setup ();
+    return num_tri;
+  }
+  virtual csTriangle* GetTriangles ()
+  {
+    Setup ();
+    return triangles;
+  }
+  virtual void Lock () { locked++; }
+  virtual void Unlock ();
+
+  virtual csFlags& GetFlags () { return flags;  }
+  virtual uint32 GetChangeNumber() const { return 0; }
+
+  void Cleanup ();
+  void ForceCleanup ();
+
+private:
+  csThingStatic* thing;
+  uint32 static_data_nr;	// To see if the static thing has changed.
+  csVector3* vertices;		// Array of vertices (points to obj_verts).
+  size_t num_verts;		// Total number of vertices.
+  csFlags flags;
+  csTriangle* triangles;
+  size_t num_tri;
+  uint32 poly_flag;
+  int locked;
+};
+
 /**
  * The static data for a thing.
  */
@@ -231,9 +300,7 @@ public:
   csBox3 obj_bbox;
 
   /// Radius of object in object space.
-  csVector3 obj_radius;
-  /// Full radius of object in object space.
-  float max_obj_radius;
+  float obj_radius;
 
   /// Static polys which share the same material
   struct csStaticPolyGroup
@@ -261,7 +328,7 @@ public:
   struct StaticSuperLM
   {
     int width, height;
-    csSubRectangles* rects;
+    CS::SubRectangles* rects;
     int freeLumels;
 
     StaticSuperLM (int w, int h) : width(w), height(h)
@@ -274,11 +341,11 @@ public:
       delete rects;
     }
 
-    csSubRectangles* GetRects ()
+    CS::SubRectangles* GetRects ()
     {
       if (rects == 0)
       {
-	rects = new csSubRectangles (csRect (0, 0, width, height));
+	rects = new CS::SubRectangles (csRect (0, 0, width, height));
       }
       return rects;
     }
@@ -331,6 +398,7 @@ public:
     LightmapTexAccessor (csThing* instance, size_t polyIndex);
     void PreGetValue (csShaderVariable *variable);
   };
+
 public:
   csThingStatic (iBase* parent, csThingObjectType* thing_type);
   virtual ~csThingStatic ();
@@ -382,7 +450,7 @@ public:
    * Get the bounding box in object space for this polygon set.
    * This is calculated based on the oriented bounding box.
    */
-  void GetBoundingBox (csBox3& box);
+  const csBox3& GetBoundingBox ();
 
   /**
    * Set the bounding box in object space for this polygon set.
@@ -392,7 +460,7 @@ public:
   /**
    * Get the radius in object space for this polygon set.
    */
-  void GetRadius (csVector3& rad, csVector3& cent);
+  void GetRadius (float& rad, csVector3& cent);
 
   //----------------------------------------------------------------------
   // Vertex handling functions
@@ -463,7 +531,7 @@ public:
     csVector3 &isect,
     float *pr);
 
-  virtual int GetPolygonCount () { return (int)static_polygons.Length (); }
+  virtual int GetPolygonCount () { return (int)static_polygons.GetSize (); }
   virtual void RemovePolygon (int idx);
   virtual void RemovePolygons ();
 
@@ -531,15 +599,8 @@ public:
 
   virtual bool AddPolygonRenderBuffer (int polygon_idx, const char* name,
     iRenderBuffer* buffer);
-
-  virtual void SetMixMode (uint mode)
-  {
-    mixmode = mode;
-  }
-  virtual uint GetMixMode () const
-  {
-    return mixmode;
-  }
+  virtual bool GetLightmapLayout (int polygon_idx, size_t& slm, 
+    csRect& slmSubRect, float* slmCoord);
 
   //-------------------- iMeshObjectFactory interface implementation ----------
 
@@ -553,18 +614,28 @@ public:
   virtual iMeshFactoryWrapper* GetMeshFactoryWrapper () const
   { return logparent; }
   virtual iMeshObjectType* GetMeshObjectType () const { return thingmesh_type; }
+  virtual bool SetMaterialWrapper (iMaterialWrapper*) { return false; }
+  virtual iMaterialWrapper* GetMaterialWrapper () const { return 0; }
+  virtual void SetMixMode (uint mode)
+  { mixmode = mode; }
+  virtual uint GetMixMode () const
+  { return mixmode; }
 
   //-------------------- iPolygonMesh interface implementation ----------------
-  PolyMeshHelper scfiPolygonMesh;
+  csRef<PolyMeshHelper> polygonMesh;
   //-------------------- CD iPolygonMesh implementation -----------------------
-  PolyMeshHelper scfiPolygonMeshCD;
+  csRef<PolyMeshHelper> polygonMeshCD;
   //-------------------- Lower detail iPolygonMesh implementation -------------
-  PolyMeshHelper scfiPolygonMeshLOD;
+  csRef<PolyMeshHelper> polygonMeshLOD;
 
   //-------------------- iObjectModel implementation --------------------------
   virtual void GetObjectBoundingBox (csBox3& bbox)
   {
-    GetBoundingBox (bbox);
+    bbox = GetBoundingBox ();
+  }
+  virtual const csBox3& GetObjectBoundingBox ()
+  {
+    return GetBoundingBox ();
   }
   virtual void SetObjectBoundingBox (const csBox3& bbox)
   {
@@ -576,8 +647,12 @@ public:
     const csArray<RepMaterial>& repMaterials, uint mixmode);
 
   virtual iObjectModel* GetObjectModel () { return (iObjectModel*)this; }
+  virtual iTerraFormer* GetTerraFormerColldet () { return 0; }
+  virtual iTerrainSystem* GetTerrainColldet () { return 0; }
 };
 
+#include "csutil/win32/msvc_deprecated_warn_on.h"
+
 /**
  * A Thing is a set of polygons. A thing can be used for the
  * outside of a sector or else to augment the sector with
@@ -755,7 +830,7 @@ private:
   	csZBufMode zMode);
 
   /// Generate a cachename based on geometry.
-  char* GenerateCacheName ();
+  csString GenerateCacheName ();
 
 public:
   CS_LEAKGUARD_DECLARE (csThing);
@@ -807,7 +882,7 @@ public:
 
   /// Get the number of polygons in this thing.
   int GetPolygonCount ()
-  { return (int)polygons.Length (); }
+  { return (int)polygons.GetSize (); }
 
   /// Get the specified polygon from this set.
   csPolygon3DStatic *GetPolygon3DStatic (int idx)
@@ -867,11 +942,6 @@ public:
    */
   void GetBoundingBox (iMovable* movable, csBox3& box);
 
-  /**
-   * Get a write object for a vis culling system.
-   */
-  iPolygonMesh* GetWriteObject ();
-
   //----------------------------------------------------------------------
   // Lighting
   //----------------------------------------------------------------------
@@ -906,7 +976,7 @@ public:
    */
   iTextureHandle* GetPolygonTexture (size_t index)
   {
-    return index < litPolys.Length() ? litPolys[index]->SLM->GetTexture() : 0;
+    return index < litPolys.GetSize () ? litPolys[index]->SLM->GetTexture() : 0;
   }
   /// Ensure lightmap textures are up-to-date
   void UpdateDirtyLMs ();
@@ -992,6 +1062,10 @@ public:
   /// Prepare.
   virtual void Prepare ()
   { PrepareForUse (); }
+  virtual csPtr<iImage> GetPolygonLightmap (int polygon_idx);
+  virtual bool GetPolygonPDLight (int polygon_idx, size_t pdlight_index, 
+    csRef<iImage>& map, iLight*& light);
+  iMaterialWrapper* GetReplacedMaterial (iMaterialWrapper* oldMat);
   /** @} */
 
   //-------------------- iMeshObject interface implementation ----------
@@ -1004,7 +1078,8 @@ public:
   virtual void SetVisibleCallback (iMeshObjectDrawCallback* /*cb*/) { }
   virtual iMeshObjectDrawCallback* GetVisibleCallback () const
   { return 0; }
-  virtual void NextFrame (csTicks /*current_time*/,const csVector3& /*pos*/)
+  virtual void NextFrame (csTicks /*current_time*/,const csVector3& /*pos*/,
+    uint /*currentFrame*/)
   { }
   virtual void SetMeshWrapper (iMeshWrapper* lp) { logparent = lp; }
   virtual iMeshWrapper* GetMeshWrapper () const { return logparent; }
@@ -1017,13 +1092,14 @@ public:
   virtual bool SetMaterialWrapper (iMaterialWrapper*) { return false; }
   virtual iMaterialWrapper* GetMaterialWrapper () const { return 0; }
   virtual void PositionChild (iMeshObject* /*child*/, csTicks /*current_time*/) { }
-
+  virtual void BuildDecal(const csVector3* pos, float decalRadius,
+	iDecalBuilder* decalBuilder);
   //-------------------- iPolygonMesh interface implementation ----------------
-  PolyMeshHelper scfiPolygonMesh;
+  //csRef<PolyMeshHelper> polygonMesh;
   //-------------------- CD iPolygonMesh implementation -----------------------
-  PolyMeshHelper scfiPolygonMeshCD;
+  //csRef<PolyMeshHelper> polygonMeshCD;
   //-------------------- Lower detail iPolygonMesh implementation -------------
-  PolyMeshHelper scfiPolygonMeshLOD;
+  //csRef<PolyMeshHelper> polygonMeshLOD;
 };
 
 struct intar2 { int ar[2]; };
@@ -1078,6 +1154,11 @@ public:
 
   int maxLightmapW, maxLightmapH;
   float maxSLMSpaceWaste;
+
+  csStringID base_id;
+  csStringID colldet_id;
+  csStringID viscull_id;
+
 public:
   /// Constructor.
   csThingObjectType (iBase*);
@@ -1138,6 +1219,7 @@ public:
   /** @} */
 };
 
-} // namespace cspluginThing
+}
+CS_PLUGIN_NAMESPACE_END(Thing)
 
 #endif // __CS_THING_H__
