@@ -59,7 +59,7 @@
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginThingLdr
+CS_PLUGIN_NAMESPACE_BEGIN(ThingLoader)
 {
 
 enum
@@ -325,26 +325,25 @@ bool csThingLoader::ParseTextureMapping (
   return true;
 }
 
-class MissingSectorCallback : public iPortalCallback
+class MissingSectorCallback : 
+  public scfImplementation1<MissingSectorCallback,
+                            iPortalCallback>
 {
 public:
   csRef<iLoaderContext> ldr_context;
-  char* sectorname;
+  csString sectorname;
   bool autoresolve;
 
-  SCF_DECLARE_IBASE;
   MissingSectorCallback (iLoaderContext* ldr_context, const char* sector,
-  	bool autoresolve)
+    bool autoresolve) : scfImplementationType (this), 
+    ldr_context (ldr_context), sectorname (sector), autoresolve (autoresolve)
   {
-    SCF_CONSTRUCT_IBASE (0);
     MissingSectorCallback::ldr_context = ldr_context;
-    sectorname = csStrNew (sector);
+    sectorname = sector;
     MissingSectorCallback::autoresolve = autoresolve;
   }
   virtual ~MissingSectorCallback ()
   {
-    delete[] sectorname;
-    SCF_DESTRUCT_IBASE ();
   }
   
   virtual bool Traverse (iPortal* portal, iBase* /*context*/)
@@ -355,25 +354,20 @@ public:
     // For efficiency reasons we deallocate the name here.
     if (!autoresolve)
     {
-      delete[] sectorname;
-      sectorname = 0;
+      sectorname.Free();
       portal->RemoveMissingSectorCallback (this);
     }
     return true;
   }
 };
 
-SCF_IMPLEMENT_IBASE (MissingSectorCallback)
-  SCF_IMPLEMENTS_INTERFACE (iPortalCallback)
-SCF_IMPLEMENT_IBASE_END
-
 bool csThingLoader::ParsePortal (
 	iDocumentNode* node, iLoaderContext* ldr_context,
 	uint32 &flags, bool &mirror, bool &warp, int& msv,
 	csMatrix3 &m, csVector3 &before, csVector3 &after,
 	iString* destSector, bool& autoresolve)
 {
-  destSector->Clear ();
+  destSector->Empty ();
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
   {
@@ -539,7 +533,7 @@ bool csThingLoader::ParsePoly3d (
         {
 	  int vt_idx = child->GetContentsValueAsInt ();
 	  bool ignore = false;
-	  int cnt = (int)vertices_to_add.Length ();
+	  int cnt = (int)vertices_to_add.GetSize ();
 	  for (int i = 0 ; i < cnt ; i++)
 	  {
 	    if (vertices_to_add[i] == vt_idx+vt_offset)
@@ -595,7 +589,7 @@ bool csThingLoader::ParsePoly3d (
     }
   }
 
-  if (vertices_to_add.Length () < 3)
+  if (vertices_to_add.GetSize () < 3)
   {
     synldr->ReportError ("crystalspace.thingldr.polygon", node,
       "Polygon '%s' contains just %d vertices!",
@@ -605,7 +599,7 @@ bool csThingLoader::ParsePoly3d (
   }
 
   thing_fact_state->SetPolygonVertexIndices (CS_POLYRANGE_LAST,
-  	(int)vertices_to_add.Length (), vertices_to_add.GetArray ());
+  	(int)vertices_to_add.GetSize (), vertices_to_add.GetArray ());
 
   mat = thing_fact_state->GetPolygonMaterial (CS_POLYINDEX_LAST);
   csRef<iMaterialEngine> mateng = scfQueryInterface<iMaterialEngine> (
@@ -798,12 +792,13 @@ bool csThingLoader::ParsePoly3d (
     bool do_warp = false;
     bool do_mirror = false;
     int msv = -1;
-    scfString destSectorName;
+    csRef<scfString> destSectorName;
+    destSectorName.AttachNew (new scfString);
 
     bool autoresolve = false;
     if (ParsePortal (portal_node, ldr_context,
 	      flags, do_mirror, do_warp, msv,
-	      m_w, v_w_before, v_w_after, &destSectorName, autoresolve))
+	      m_w, v_w_before, v_w_after, destSectorName, autoresolve))
     {
       iSector* destSector;
       // If autoresolve is true we clear the sector since we want the callback
@@ -811,7 +806,7 @@ bool csThingLoader::ParsePoly3d (
       if (autoresolve)
         destSector = 0;
       else
-        destSector = ldr_context->FindSector (destSectorName.GetData ());
+        destSector = ldr_context->FindSector (destSectorName->GetData ());
       int cnt = thing_fact_state->GetPolygonVertexCount (CS_POLYINDEX_LAST);
       csVector3* portal_verts = new csVector3[cnt];
       int i;
@@ -824,7 +819,7 @@ bool csThingLoader::ParsePoly3d (
         portal_pri = mesh->GetRenderPriority ();
       csString pc_name;
       pc_name.Format ("__portals_%d_%s__", portal_pri,
-      	destSectorName.GetData ());
+      	destSectorName->GetData ());
 
       iPortal* portal;
       csRef<iMeshWrapper> portal_mesh = engine->CreatePortal (
@@ -841,10 +836,10 @@ bool csThingLoader::ParsePoly3d (
 
       if (!destSector)
       {
-	MissingSectorCallback* mscb = new MissingSectorCallback (
-	    	ldr_context, destSectorName.GetData (), autoresolve);
+	csRef<MissingSectorCallback> mscb;
+        mscb.AttachNew (new MissingSectorCallback (
+	    	ldr_context, destSectorName->GetData (), autoresolve));
 	portal->SetMissingSectorCallback (mscb);
-	mscb->DecRef ();
       }
 
       if (is_texture_transparent)
@@ -895,15 +890,15 @@ bool csThingLoader::ParsePoly3d (
   else if (set_colldet == -1)
     thing_fact_state->ResetPolygonFlags (CS_POLYRANGE_LAST, CS_POLY_COLLDET);
 
-  for (size_t i  = 0; i < renderbuffers.Length(); i++)
+  for (size_t i  = 0; i < renderbuffers.GetSize (); i++)
   {
     const ParsedRB& rb = renderbuffers[i];
-    if (rb.buf->GetElementCount() != vertices_to_add.Length())
+    if (rb.buf->GetElementCount() != vertices_to_add.GetSize ())
     {
       synldr->ReportError ("crystalspace.thingldr.polygon", rb.node,
 	"Render buffer element count does not match polygon vertex count: "
 	"%zu != %zu", rb.buf->GetElementCount(), 
-	vertices_to_add.Length());
+	vertices_to_add.GetSize ());
       return false;
     }
     if (!thing_fact_state->AddPolygonRenderBuffer (CS_POLYINDEX_LAST, 
@@ -956,10 +951,9 @@ return false; \
 #define CREATE_FACTORY_IF_NEEDED \
 if (!info.thing_fact_state) \
 { \
-  csRef<iMeshObjectFactory> fact; \
-  fact = info.type->NewFactory (); \
-  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (fact); \
-  info.obj = fact->NewInstance (); \
+  info.fact = info.type->NewFactory (); \
+  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (info.fact); \
+  info.obj = info.fact->NewInstance (); \
   info.thing_state = scfQueryInterface<iThingState> (info.obj); \
 }
 
@@ -1024,6 +1018,7 @@ if (!info.thing_fact_state) \
               child, "Couldn't find thing factory '%s'!", factname);
             return false;
           }
+	  info.fact = fact->GetMeshObjectFactory ();
 	  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (
 	  	fact->GetMeshObjectFactory ());
 	  if (!info.thing_fact_state)
@@ -1075,6 +1070,7 @@ if (!info.thing_fact_state) \
               child, "Object '%s' is not a thing!", meshname);
             return false;
 	  }
+	  info.fact = wrap->GetMeshObject ()->GetFactory ();
 	  info.thing_fact_state = scfQueryInterface<iThingFactoryState>
 	    (wrap->GetMeshObject ()->GetFactory());
 	  info.obj = wrap->GetFactory ()->GetMeshObjectFactory ()
@@ -1175,7 +1171,7 @@ Nag to Jorrit about this feature if you want it.");
 	  if (synldr->ParseMixmode (child, mixmode))
 	  {
 	    if (info.load_factory)
-              info.thing_fact_state->SetMixMode (mixmode);
+              info.fact->SetMixMode (mixmode);
 	    else
               info.thing_state->SetMixMode (mixmode);
 	  }
@@ -1198,7 +1194,7 @@ Nag to Jorrit about this feature if you want it.");
   if (mixmode == CS_FX_COPY)
   {
     // Mixmode is copy, delete all polygons that became portals.
-    size_t i = polygons_to_delete.Length ();
+    size_t i = polygons_to_delete.GetSize ();
     while (i-- > 0)
       info.thing_fact_state->RemovePolygon (polygons_to_delete[i]);
   }
@@ -1253,7 +1249,7 @@ csPtr<iBase> csThingLoader::Parse (iDocumentNode* node,
       return 0;
     }
     size_t i;
-    for (i = 0 ; i < info.replace_materials.Length () ; i++)
+    for (i = 0 ; i < info.replace_materials.GetSize () ; i++)
     {
       RepMaterial& rm = info.replace_materials[i];
       iMaterialWrapper* old_mat = ldr_context->FindMaterial (rm.oldmat);
@@ -1314,15 +1310,14 @@ csPtr<iBase> csThingFactoryLoader::Parse (iDocumentNode* node,
     scfQueryInterface<iThingEnvironment> (info.type);
   csRef<iEngine> engine = csQueryRegistry<iEngine> (object_reg);
 
-  csRef<iMeshObjectFactory> fact;
-  fact = info.type->NewFactory ();
-  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (fact);
+  info.fact = info.type->NewFactory ();
+  info.thing_fact_state = scfQueryInterface<iThingFactoryState> (info.fact);
 
   bool baduv = false;
   if (!LoadThingPart (te, node, ldr_context, object_reg, reporter, synldr, info,
   	engine, 0, true, 0, baduv))
   {
-    fact = 0;
+    info.fact = 0;
   }
 
   if (baduv)
@@ -1332,7 +1327,7 @@ csPtr<iBase> csThingFactoryLoader::Parse (iDocumentNode* node,
 		node, "Bad UV coordinates for polygons in this thing!");
   }
 
-  return csPtr<iBase> (fact);
+  return csPtr<iBase> (info.fact);
 }
 
 //---------------------------------------------------------------------------
@@ -1413,7 +1408,7 @@ bool csThingSaver::WriteFactory (iBase* obj, iDocumentNode* parent)
       csRef<iDocumentNode> vertNode = paramsNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
       vertNode->SetValue("v");
       csVector3 vertex = thingfact->GetVertex(vertidx);
-      synldr->WriteVector(vertNode, &vertex);
+      synldr->WriteVector(vertNode, vertex);
     }  
     iMaterialWrapper* material = 0;
     for (int polyidx = 0; polyidx < thingfact->GetPolygonCount(); polyidx++)
@@ -1451,10 +1446,10 @@ bool csThingSaver::WriteFactory (iBase* obj, iDocumentNode* parent)
         thingfact->GetPolygonTextureMapping(polyidx, m, v);
         csRef<iDocumentNode> matrixNode = texmapNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
         matrixNode->SetValue("matrix");
-        synldr->WriteMatrix(matrixNode, &m);
+        synldr->WriteMatrix(matrixNode, m);
         csRef<iDocumentNode> vectorNode = texmapNode->CreateNodeBefore(CS_NODE_ELEMENT, 0);
         vectorNode->SetValue("v");
-        synldr->WriteVector(vectorNode, &v);
+        synldr->WriteVector(vectorNode, v);
       }
       //Writedown Colldet tag
       bool colldet = thingfact->GetPolygonFlags(polyidx).Check(CS_POLY_COLLDET);
@@ -1480,5 +1475,5 @@ bool csThingSaver::WriteFactory (iBase* obj, iDocumentNode* parent)
   return true;
 }
 
-} // namespace cspluginThingLdr
-
+}
+CS_PLUGIN_NAMESPACE_END(ThingLoader)
