@@ -25,12 +25,12 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "iengine/movable.h"
 #include "imesh/genmesh.h"
 
+#include "csutil/custom_new_disable.h"
+
 //bullet includes
 #include <CcdPhysics/CcdPhysicsController.h>
 #include <Dynamics/RigidBody.h>
 #include <CcdPhysics/CcdPhysicsEnvironment.h>
-#include <ConstraintSolver/SimpleConstraintSolver.h>
-#include <CollisionDispatch/ToiContactDispatcher.h>
 #include <BroadphaseCollision/SimpleBroadphase.h>
 #include <CollisionShapes/BoxShape.h>
 #include <CollisionShapes/EmptyShape.h>
@@ -40,12 +40,20 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <SimdTransform.h>
 #include <SimdMatrix3x3.h>
 #include <Dynamics/MassProps.h>
+#include <CollisionDispatch/CollisionDispatcher.h>
+
+#include "csutil/custom_new_enable.h"
+
 
 #include "bullet.h"
 
+class TriangleCallback;
 
 CS_IMPLEMENT_PLUGIN
 
+CS_PLUGIN_NAMESPACE_BEGIN(Bullet)
+{
+
 SCF_IMPLEMENT_FACTORY (csBulletDynamics)
 
 #define ODE2CSMATRIX(odemat, csmat) \
@@ -73,7 +81,7 @@ bool csBulletDynamics::Initialize (iObjectRegistry* object_reg)
 csPtr<iDynamicSystem> csBulletDynamics::CreateSystem ()
 {
   csBulletDynamicsSystem* system = new csBulletDynamicsSystem ();
-  csRef<iDynamicSystem> isystem (SCF_QUERY_INTERFACE (system, iDynamicSystem));
+  csRef<iDynamicSystem> isystem (scfQueryInterface<iDynamicSystem> (system));
   systems.Push (system);
   isystem->DecRef ();
 
@@ -101,20 +109,27 @@ void csBulletDynamics::Step (float stepsize)
   {
     systems[i]->Step (stepsize);
   }
+  //step callbacks
+  step_callbacks.Compact ();
+  for (size_t i = 0; i < step_callbacks.GetSize (); i++)
+  {
+    step_callbacks[i]->Step (stepsize);
+  }
 }
 //-------------------------------csBulletDynamicsSystem----------------------------------------------//
 
 csBulletDynamicsSystem::csBulletDynamicsSystem ()
 :  scfImplementationType (this)
 {
-  ConstraintSolver* solver = new SimpleConstraintSolver;
-  ToiContactDispatcher* dispatcher = new	ToiContactDispatcher(solver);
-  BroadphaseInterface* broadphase = new SimpleBroadphase();
+  CollisionDispatcher* dispatcher = new CollisionDispatcher ();
+  SimpleBroadphase* broadphase = new SimpleBroadphase();
 
   bullet_sys = new CcdPhysicsEnvironment(dispatcher,broadphase);
 }
 csBulletDynamicsSystem::~csBulletDynamicsSystem ()
 {
+  bodies.DeleteAll ();
+  delete bullet_sys;
 }
 void csBulletDynamicsSystem::SetGravity (const csVector3& v)
 {
@@ -125,29 +140,29 @@ const csVector3 csBulletDynamicsSystem::GetGravity () const
 {
   return gravity;
 }
-void csBulletDynamicsSystem::SetLinearDampener (float d)
+void csBulletDynamicsSystem::SetLinearDampener (float)
 {
 }
 float csBulletDynamicsSystem::GetLinearDampener () const
 {
   return 0;
 }
-void csBulletDynamicsSystem::SetRollingDampener (float d)
+void csBulletDynamicsSystem::SetRollingDampener (float)
 {
 }
 float csBulletDynamicsSystem::GetRollingDampener () const
 {
   return 0;
 }
-void csBulletDynamicsSystem::EnableAutoDisable (bool enable)
+void csBulletDynamicsSystem::EnableAutoDisable (bool)
 {
 }
 bool csBulletDynamicsSystem::AutoDisableEnabled ()
 {
   return false;
 }
-void csBulletDynamicsSystem::SetAutoDisableParams (float linear, float angular, int steps,
-                                                   float time)
+void csBulletDynamicsSystem::SetAutoDisableParams (float /*linear*/,
+  float /*angular*/, int /*steps*/, float /*time*/)
 {
 }
 void csBulletDynamicsSystem::Step (float stepsize)
@@ -163,20 +178,20 @@ csPtr<iRigidBody> csBulletDynamicsSystem::CreateBody ()
   csBulletRigidBody *b = new csBulletRigidBody (this);
   bullet_sys->addCcdPhysicsController (b->GetBulletBody ());
 
-  csRef<iRigidBody> ib = SCF_QUERY_INTERFACE (b, iRigidBody);
+  csRef<iRigidBody> ib = scfQueryInterface<iRigidBody> (b);
   bodies.Push (ib);
   ib->DecRef ();
 
   return (csPtr<iRigidBody>) ib;
 }
-void csBulletDynamicsSystem::RemoveBody (iRigidBody* body)
+void csBulletDynamicsSystem::RemoveBody (iRigidBody*)
 {
 }
-iRigidBody *csBulletDynamicsSystem::FindBody (const char *name)
+iRigidBody *csBulletDynamicsSystem::FindBody (const char * /*name*/)
 {
   return 0;
 }
-iRigidBody *csBulletDynamicsSystem::GetBody (unsigned int index)
+iRigidBody *csBulletDynamicsSystem::GetBody (unsigned int /*index*/)
 {
   return 0;
 }
@@ -188,37 +203,37 @@ csPtr<iBodyGroup> csBulletDynamicsSystem::CreateGroup ()
 {
   return 0;
 }
-void csBulletDynamicsSystem::RemoveGroup (iBodyGroup* group)
+void csBulletDynamicsSystem::RemoveGroup (iBodyGroup*)
 {
 }
 csPtr<iJoint> csBulletDynamicsSystem::CreateJoint ()
 {
   return 0;
 }
-void csBulletDynamicsSystem::RemoveJoint (iJoint* joint)
+void csBulletDynamicsSystem::RemoveJoint (iJoint*)
 {
 }
 iDynamicsMoveCallback* csBulletDynamicsSystem::GetDefaultMoveCallback ()
 {
   return 0;
 }
-bool csBulletDynamicsSystem::AttachColliderMesh (iMeshWrapper* mesh,
-                                                 const csOrthoTransform& trans, float friction,
-                                                 float elasticity, float softness)
+bool csBulletDynamicsSystem::AttachColliderMesh (iMeshWrapper*,
+  const csOrthoTransform&, float /*friction*/,
+  float /*elasticity*/, float /*softness*/)
 {
   return false;
 }
 
-bool csBulletDynamicsSystem::AttachColliderCylinder (float length, float radius,
-                                                     const csOrthoTransform& trans, float friction,
-                                                     float elasticity, float softness)
+bool csBulletDynamicsSystem::AttachColliderCylinder (float /*length*/,
+  float /*radius*/, const csOrthoTransform&, float /*frictio*/,
+  float /*elasticity*/, float /*softness*/)
 {
   return false;
 }
 
 bool csBulletDynamicsSystem::AttachColliderBox (const csVector3 &size,
-                                                const csOrthoTransform& trans, float friction,
-                                                float elasticity, float softness)
+  const csOrthoTransform& trans, float friction,
+  float elasticity, float softness)
 {
   csBulletCollider *bulletc = new csBulletCollider (this);
   bullet_sys->addCcdPhysicsController (bulletc->GetBulletController ());
@@ -233,19 +248,20 @@ bool csBulletDynamicsSystem::AttachColliderBox (const csVector3 &size,
   return true;
 }
 
-bool csBulletDynamicsSystem::AttachColliderSphere (float radius, const csVector3 &offset,
-                                                   float friction, float elasticity, float softness)
+bool csBulletDynamicsSystem::AttachColliderSphere (float /*radius*/,
+  const csVector3 &/*offset*/, float /*friction*/,
+  float /*elasticity*/, float /*softness*/)
 {
   return false;
 }
 
-bool csBulletDynamicsSystem::AttachColliderPlane (const csPlane3 &plane, float friction,
-                                                  float elasticity, float softness)
+bool csBulletDynamicsSystem::AttachColliderPlane (const csPlane3 &,
+  float /*friction*/, float /*elasticity*/, float /*softness*/)
 {
   return false;
 }
 
-void csBulletDynamicsSystem::AttachCollider (iDynamicsSystemCollider* collider)
+void csBulletDynamicsSystem::AttachCollider (iDynamicsSystemCollider*)
 {
 }
 
@@ -255,6 +271,15 @@ csRef<iDynamicsSystemCollider> csBulletDynamicsSystem::CreateCollider ()
 }
 
 //-------------------------------csBulletRigidBody----------------------------------------------//
+
+class EmptyShape_Fixed : public EmptyShape
+{
+public:
+  void ProcessAllTriangles(TriangleCallback* callback,
+    const SimdVector3& aabbMin, const SimdVector3& aabbMax) const
+  { }
+};
+
 csBulletRigidBody::csBulletRigidBody (csBulletDynamicsSystem* dynsys)
 :  scfImplementationType (this)
 {
@@ -265,11 +290,11 @@ csBulletRigidBody::csBulletRigidBody (csBulletDynamicsSystem* dynsys)
 
   CcdConstructionInfo ccdObjectCi;
 
-  ccdObjectCi.m_collisionShape = new EmptyShape ();
+  ccdObjectCi.m_collisionShape = new EmptyShape_Fixed ();
   ccdObjectCi.m_collisionShape->SetMargin (1);
-  ccdObjectCi.m_broadphaseHandle = 0;
   ccdObjectCi.m_gravity = SimdVector3(0,0,0);
   ccdObjectCi.m_localInertiaTensor = SimdVector3(0,0,0);
+  ccdObjectCi.m_physicsEnv = dynsys->GetBulletSys ();
   mass = ccdObjectCi.m_mass = 1;
 
   ccdObjectCi.m_MotionState = ms;
@@ -319,8 +344,7 @@ void csBulletRigidBody::ResetShape ()
 {
   if (pc->GetCollisionShape ()->GetShapeType () != EMPTY_SHAPE_PROXYTYPE)
   {
-    BroadphaseProxy* bpproxy = (BroadphaseProxy*)pc->m_broadphaseHandle;
-    bpproxy->SetClientObjectType(pc->GetRigidBody()->GetCollisionShape()->GetShapeType());
+    BroadphaseProxy* bpproxy = (BroadphaseProxy*)pc->GetRigidBody ()->m_broadphaseHandle;
     ds->GetBulletSys ()->GetBroadphase()->CleanProxyFromPairs(bpproxy);
 
     SimdVector3 newinertia;
@@ -330,16 +354,17 @@ void csBulletRigidBody::ResetShape ()
   }
 }
 
-bool csBulletRigidBody::AttachColliderMesh (iMeshWrapper* mesh,
-                                            const csOrthoTransform& trans, float friction, float density,
-                                            float elasticity, float softness)
+bool csBulletRigidBody::AttachColliderMesh (iMeshWrapper*,
+  const csOrthoTransform&, float /*friction*/, float /*density*/,
+  float /*elasticity*/, float /*softness*/)
 {
   return false;
 }
 
-bool csBulletRigidBody::AttachColliderCylinder (float length, float radius,
-                                                const csOrthoTransform& trans, float friction, float density,
-                                                float elasticity, float softness)
+bool csBulletRigidBody::AttachColliderCylinder (float /*length*/, float /*radius*/,
+                                                const csOrthoTransform& /*trans*/, float /*friction*/,
+                                                float /*density*/, float /*elasticity*/, 
+                                                float /*softness*/)
 {
   //if (pc->GetRigidBody ()->GetCollisionShape ())
   //  delete pc->GetCollisionShape ();
@@ -352,11 +377,12 @@ bool csBulletRigidBody::AttachColliderCylinder (float length, float radius,
 }
 
 bool csBulletRigidBody::AttachColliderBox (const csVector3 &size,
-                                           const csOrthoTransform& trans, float friction, float density,
-                                           float elasticity, float softness)
+                                           const csOrthoTransform& /*trans*/, float friction,
+                                           float /*density*/, float /*elasticity*/, 
+                                           float /*softness*/)
 {
-  if (pc->GetRigidBody ()->GetCollisionShape ())
-    delete pc->GetCollisionShape ();
+  //if (pc->GetRigidBody ()->GetCollisionShape ())
+  //  delete pc->GetCollisionShape ();
 
   SimdVector3 s (size.x / 2, size.y / 2, size.z / 2); 
   pc->GetRigidBody ()->SetCollisionShape (new BoxShape (s));
@@ -367,9 +393,9 @@ bool csBulletRigidBody::AttachColliderBox (const csVector3 &size,
   return true;
 }
 
-bool csBulletRigidBody::AttachColliderSphere (float radius, const csVector3 &offset,
-                                              float friction, float density, float elasticity,
-                                              float softness)
+bool csBulletRigidBody::AttachColliderSphere (float radius, const csVector3 &/*offset*/,
+                                              float friction, float /*density*/, float /*elasticity*/,
+                                              float /*softness*/)
 {
   if (pc->GetRigidBody ()->GetCollisionShape ())
     delete pc->GetCollisionShape ();
@@ -382,20 +408,21 @@ bool csBulletRigidBody::AttachColliderSphere (float radius, const csVector3 &off
   return true;
 }
 
-bool csBulletRigidBody::AttachColliderPlane (const csPlane3 &plane, float friction,
-                                             float density, float elasticity, float softness)
+bool csBulletRigidBody::AttachColliderPlane (const csPlane3 &,
+  float /*friction*/, float /*density*/,
+  float /*elasticity*/, float /*softness*/)
 {
   return false;
 }
 
-void csBulletRigidBody::AttachCollider (iDynamicsSystemCollider* collider)
+void csBulletRigidBody::AttachCollider (iDynamicsSystemCollider*)
 {
 }
 
 void csBulletRigidBody::DestroyColliders ()
 {
 }
-void csBulletRigidBody::DestroyCollider (iDynamicsSystemCollider* collider)
+void csBulletRigidBody::DestroyCollider (iDynamicsSystemCollider*)
 {
 }
 
@@ -413,14 +440,14 @@ const csVector3 csBulletRigidBody::GetPosition () const
 
 void csBulletRigidBody::SetOrientation (const csMatrix3& rot)
 {
-  csQuaternion q (rot);
-  pc->setOrientation (q.x, q.y, q.z, q.r);
+  csQuaternion q; q.SetMatrix (rot);
+  pc->setOrientation (q.v.x, q.v.y, q.v.z, q.w);
 }
 
 const csMatrix3 csBulletRigidBody::GetOrientation () const
 {
   csQuaternion q;
-  ms->getWorldOrientation(q.x,q.y,q.z,q.r);
+  ms->getWorldOrientation(q.v.x,q.v.y,q.v.z,q.w);
   return csMatrix3 (q).GetTranspose ();
 }
 
@@ -455,8 +482,8 @@ const csVector3 csBulletRigidBody::GetAngularVelocity () const
   return csVector3 (0);
 }
 
-void csBulletRigidBody::SetProperties (float mass, const csVector3& center,
-                                       const csMatrix3& inertia)
+void csBulletRigidBody::SetProperties (float mass, const csVector3& /*center*/,
+                                       const csMatrix3& /*inertia*/)
 {
   csBulletRigidBody::mass = mass;
   
@@ -466,8 +493,8 @@ void csBulletRigidBody::SetProperties (float mass, const csVector3& center,
   pc->GetRigidBody ()->setGravity (SimdVector3 (g.x, g.y, g.z));
 }
 
-void csBulletRigidBody::GetProperties (float* mass, csVector3* center,
-                                       csMatrix3* inertia)
+void csBulletRigidBody::GetProperties (float* /*mass*/, csVector3* /*center*/,
+                                       csMatrix3* /*inertia*/)
 {
 }
 
@@ -484,7 +511,7 @@ csMatrix3 csBulletRigidBody::GetInertia ()
 {
   return csMatrix3 ();
 }
-void csBulletRigidBody::AdjustTotalMass (float targetmass)
+void csBulletRigidBody::AdjustTotalMass (float /*targetmass*/)
 {
 }
 void csBulletRigidBody::AddForce (const csVector3& force)
@@ -492,28 +519,28 @@ void csBulletRigidBody::AddForce (const csVector3& force)
   SimdVector3 f (force.x, force.y, force.z);
   pc->GetRigidBody ()->applyCentralForce(f);
 }
-void csBulletRigidBody::AddTorque (const csVector3& force)
+void csBulletRigidBody::AddTorque (const csVector3& /*force*/)
 {
 }
-void csBulletRigidBody::AddRelForce (const csVector3& force)
+void csBulletRigidBody::AddRelForce (const csVector3& /*force*/)
 {
 }
-void csBulletRigidBody::AddRelTorque (const csVector3& force) 
+void csBulletRigidBody::AddRelTorque (const csVector3& /*force*/) 
 {
 }
-void csBulletRigidBody::AddForceAtPos (const csVector3& force, const csVector3& pos)
+void csBulletRigidBody::AddForceAtPos (const csVector3& /*force*/, const csVector3& /*pos*/)
 {
 }
-void csBulletRigidBody::AddForceAtRelPos (const csVector3& force,
-                                          const csVector3& pos)
+void csBulletRigidBody::AddForceAtRelPos (const csVector3& /*force*/,
+                                          const csVector3& /*pos*/)
 {
 }
-void csBulletRigidBody::AddRelForceAtPos (const csVector3& force,
-                                          const csVector3& pos)
+void csBulletRigidBody::AddRelForceAtPos (const csVector3& /*force*/,
+                                          const csVector3& /*pos*/)
 {
 }
-void csBulletRigidBody::AddRelForceAtRelPos (const csVector3& force,
-                                             const csVector3& pos)
+void csBulletRigidBody::AddRelForceAtRelPos (const csVector3& /*force*/,
+                                             const csVector3& /*pos*/)
 {
 }
 const csVector3 csBulletRigidBody::GetForce () const
@@ -541,13 +568,14 @@ void csBulletRigidBody::SetMoveCallback (iDynamicsMoveCallback* cb)
   if (move_cb) move_cb->DecRef ();
   move_cb = cb;
 }
-void csBulletRigidBody::SetCollisionCallback (iDynamicsCollisionCallback* cb)
+void csBulletRigidBody::SetCollisionCallback (iDynamicsCollisionCallback*)
 {
 }
-void csBulletRigidBody::Collision (iRigidBody *other)
+void csBulletRigidBody::Collision (iRigidBody *, const csVector3&,
+      const csVector3&, float)
 {
 }
-csRef<iDynamicsSystemCollider> csBulletRigidBody::GetCollider (unsigned int index)
+csRef<iDynamicsSystemCollider> csBulletRigidBody::GetCollider (unsigned int)
 {
   return 0;
 }
@@ -644,7 +672,7 @@ void csBulletDefaultMoveCallback::Execute (iMeshWrapper* mesh,
   mesh->GetMovable ()->UpdateMove ();
 }
 
-void csBulletDefaultMoveCallback::Execute (csOrthoTransform& t)
+void csBulletDefaultMoveCallback::Execute (csOrthoTransform&)
 {
   /* do nothing by default */
 }
@@ -660,9 +688,8 @@ csBulletCollider::csBulletCollider (csBulletDynamicsSystem* dynsys)
   ms->setWorldPosition (0, 0, 0);
   ms->setWorldOrientation (0,0,0,1);
 
-  ccdObjectCi.m_collisionShape = new EmptyShape ();
+  ccdObjectCi.m_collisionShape = new EmptyShape_Fixed ();
   ccdObjectCi.m_collisionShape->SetMargin (1.f);
-  ccdObjectCi.m_broadphaseHandle = 0;
   ccdObjectCi.m_gravity = SimdVector3(0,0,0);
   ccdObjectCi.m_localInertiaTensor = SimdVector3(0,0,0);
   ccdObjectCi.m_mass = 0; //collider need 0 mass
@@ -678,8 +705,7 @@ csBulletCollider::~csBulletCollider ()
 }
 void csBulletCollider::ResetShape ()
 {
-  BroadphaseProxy* bpproxy = (BroadphaseProxy*)pc->m_broadphaseHandle;
-  bpproxy->SetClientObjectType(pc->GetRigidBody()->GetCollisionShape()->GetShapeType());
+  BroadphaseProxy* bpproxy = (BroadphaseProxy*)pc->GetRigidBody ()->m_broadphaseHandle;
   ds->GetBulletSys ()->GetBroadphase()->CleanProxyFromPairs(bpproxy);
 
   SimdVector3 newinertia;
@@ -688,15 +714,15 @@ void csBulletCollider::ResetShape ()
   pc->GetRigidBody()->updateInertiaTensor();
 }
 
-bool csBulletCollider::CreateSphereGeometry (const csSphere& sphere)
+bool csBulletCollider::CreateSphereGeometry (const csSphere&)
 {
   return false;
 }
-bool csBulletCollider::CreatePlaneGeometry (const csPlane3& plane)
+bool csBulletCollider::CreatePlaneGeometry (const csPlane3&)
 {
   return false;
 }
-bool csBulletCollider::CreateMeshGeometry (iMeshWrapper *mesh)
+bool csBulletCollider::CreateMeshGeometry (iMeshWrapper *)
 {
   return false;
 }
@@ -712,25 +738,26 @@ bool csBulletCollider::CreateBoxGeometry (const csVector3& size)
 
   return true;
 }
-bool csBulletCollider::CreateCCylinderGeometry (float length, float radius)
+bool csBulletCollider::CreateCapsuleGeometry (float /*length*/,
+  float /*radius*/)
 {
   return false;
 }
 void csBulletCollider::SetCollisionCallback (
-  iDynamicsColliderCollisionCallback* cb)
+  iDynamicsColliderCollisionCallback*)
 {
 }
 void csBulletCollider::SetFriction (float friction)
 {
   pc->GetRigidBody ()->setFriction (friction);
 }
-void csBulletCollider::SetSoftness (float softness)
+void csBulletCollider::SetSoftness (float)
 {
 }
-void csBulletCollider::SetDensity (float density)
+void csBulletCollider::SetDensity (float)
 {
 }
-void csBulletCollider::SetElasticity (float elasticity)
+void csBulletCollider::SetElasticity (float)
 {
 }
 float csBulletCollider::GetFriction ()
@@ -778,7 +805,7 @@ void csBulletCollider::FillWithColliderGeometry (csRef<iGeneralFactoryState> gen
 csOrthoTransform csBulletCollider::GetTransform ()
 {
   csQuaternion q;
-  ms->getWorldOrientation(q.x,q.y,q.z,q.r);
+  ms->getWorldOrientation(q.v.x,q.v.y,q.v.z,q.w);
   csMatrix3 rot (q);
 
   csVector3 v;
@@ -792,26 +819,27 @@ csOrthoTransform csBulletCollider::GetLocalTransform ()
 }
 void csBulletCollider::SetTransform (const csOrthoTransform& trans)
 {
-  csQuaternion q (trans.GetO2T ());
-  pc->setOrientation (q.x, q.y, q.z, q.r);
-  ms->setWorldOrientation (q.x, q.y, q.z, q.r);
+  csQuaternion q; q.SetMatrix (trans.GetO2T ());
+  pc->setOrientation (q.v.x, q.v.y, q.v.z, q.w);
+  ms->setWorldOrientation (q.v.x, q.v.y, q.v.z, q.w);
   csVector3 pos = trans.GetOrigin ();
   pc->setPosition (pos.x, pos.y, pos.z);
   ms->setWorldPosition (pos.x, pos.y, pos.z);
 }
-bool csBulletCollider::GetBoxGeometry (csVector3& size)
+bool csBulletCollider::GetBoxGeometry (csVector3& /*size*/)
 {
   return false;
 }
-bool csBulletCollider::GetSphereGeometry (csSphere& sphere)
+bool csBulletCollider::GetSphereGeometry (csSphere&)
 {
   return false;
 }
-bool csBulletCollider::GetPlaneGeometry (csPlane3& plane)
+bool csBulletCollider::GetPlaneGeometry (csPlane3&)
 {
   return false;
 }
-bool csBulletCollider::GetCylinderGeometry (float& length, float& radius)
+bool csBulletCollider::GetCylinderGeometry (float& /*length*/,
+  float& /*radius*/)
 {
   return false;
 }
@@ -824,4 +852,7 @@ void csBulletCollider::MakeDynamic ()
 bool csBulletCollider::IsStatic ()
 {
   return false;
-}
\ No newline at end of file
+}
+
+}
+CS_PLUGIN_NAMESPACE_END(Bullet)
