@@ -19,7 +19,9 @@
 #include "cssysdef.h"
 
 #include "csutil/csendian.h"
+#include "csutil/csstring.h"
 #include "csutil/dirtyaccessarray.h"
+#include "csutil/scf.h"
 
 #include "iengine/engine.h"
 #include "iengine/mesh.h"
@@ -42,37 +44,24 @@
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginSimpleFormerLoader
-{
-
-SCF_IMPLEMENT_IBASE (csSimpleFormerLoader)
-  SCF_IMPLEMENTS_INTERFACE (iLoaderPlugin)
-  SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
-SCF_IMPLEMENT_IBASE_END
-
-SCF_IMPLEMENT_EMBEDDED_IBASE (csSimpleFormerLoader::eiComponent)
-  SCF_IMPLEMENTS_INTERFACE (iComponent)
-SCF_IMPLEMENT_EMBEDDED_IBASE_END
+using namespace CS::Plugins::SimpleFormerLoader;
 
 SCF_IMPLEMENT_FACTORY (csSimpleFormerLoader)
 
-csSimpleFormerLoader::csSimpleFormerLoader (iBase* parent)
+csSimpleFormerLoader::csSimpleFormerLoader (iBase* parent) :
+  scfImplementationType(this, parent)
 {
-  SCF_CONSTRUCT_IBASE (parent);
-  SCF_CONSTRUCT_EMBEDDED_IBASE (scfiComponent);
 }
 
 csSimpleFormerLoader::~csSimpleFormerLoader ()
 {
-  SCF_DESTRUCT_EMBEDDED_IBASE (scfiComponent);
-  SCF_DESTRUCT_IBASE ();
 }
 
 bool csSimpleFormerLoader::Initialize (iObjectRegistry* object_reg)
 {
   objreg = object_reg;
-  synldr = CS_QUERY_REGISTRY (objreg, iSyntaxService);
-  pluginmgr = CS_QUERY_REGISTRY (objreg, iPluginManager);
+  synldr = csQueryRegistry<iSyntaxService> (objreg);
+  pluginmgr = csQueryRegistry<iPluginManager> (objreg);
 
   InitTokenTable (xmltokens);
   return true;
@@ -90,8 +79,12 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
       node, "Could not loader crystalspace.terraformer.simple plugin");
     return 0;
   }
-  csRef<iSimpleFormerState> state = SCF_QUERY_INTERFACE (former, 
-        iSimpleFormerState);
+  csRef<iSimpleFormerState> state =  
+        scfQueryInterface<iSimpleFormerState> (former);
+
+  uint alphamapcount = 0;
+  bool material_map_set = false;
+  bool one_material_map_used = false;
 
   csRef<iDocumentNodeIterator> it = node->GetNodes ();
   while (it->HasNext ())
@@ -106,6 +99,7 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
       {
         const char *name = child->GetContentsValue ();
         objreg->Register (former, name);
+        former->QueryObject ()->SetName (name);
         break;
       }
       case XMLTOKEN_HEIGHTMAP: 
@@ -114,7 +108,7 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
 	if ((format == 0) || (strcmp (format, "image") == 0))
 	{
 	  const char *image = child->GetContentsValue ();
-	  csRef<iLoader> loader = CS_QUERY_REGISTRY (objreg, iLoader);
+	  csRef<iLoader> loader = csQueryRegistry<iLoader> (objreg);
 	  csRef<iImage> map = loader->LoadImage (image, CS_IMGFMT_ANY);
 	  if (map == 0) 
 	  {
@@ -176,7 +170,7 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
       case XMLTOKEN_INTMAP: 
       {
         const char *image = child->GetContentsValue ();
-	csRef<iLoader> loader = CS_QUERY_REGISTRY (objreg, iLoader);
+	csRef<iLoader> loader = csQueryRegistry<iLoader> (objreg);
         csRef<iImage> map = loader->LoadImage (image);
         if (map == 0) 
         {
@@ -187,8 +181,8 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
 	int scale = child->GetAttributeValueAsInt ("scale");
 	int offset = child->GetAttributeValueAsInt ("offset");
 	const char* typestring = child->GetAttributeValue ("type");
-        csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (
-	  objreg, "crystalspace.shared.stringset", iStringSet);
+        csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+	  objreg, "crystalspace.shared.stringset");
         csStringID type = strings->Request (typestring);
         state->SetIntegerMap (type, map, scale, offset);
         break;
@@ -196,7 +190,7 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
       case XMLTOKEN_FLOATMAP: 
       {
         const char *image = child->GetContentsValue ();
-	csRef<iLoader> loader = CS_QUERY_REGISTRY (objreg, iLoader);
+	csRef<iLoader> loader = csQueryRegistry<iLoader> (objreg);
         csRef<iImage> map = loader->LoadImage (image);
         if (map == 0) 
         {
@@ -207,8 +201,8 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
 	float scale = child->GetAttributeValueAsFloat ("scale");
 	float offset = child->GetAttributeValueAsFloat ("offset");
 	const char* typestring = child->GetAttributeValue ("type");
-        csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (
-	  objreg, "crystalspace.shared.stringset", iStringSet);
+        csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+	  objreg, "crystalspace.shared.stringset");
         csStringID type = strings->Request (typestring);
         state->SetFloatMap (type, map, scale, offset);
         break;
@@ -237,11 +231,112 @@ csPtr<iBase> csSimpleFormerLoader::Parse (iDocumentNode* node,
         state->SetOffset (v);
         break;
       }
+      case XMLTOKEN_MATERIALMAP:
+      {
+//@@@
+/*
+        if (!palette_set)
+        {
+          synldr->ReportError ("crystalspace.terrain.factory.loader",
+              child, "First set a material palette before <materialmap>!");
+          return 0;
+        }
+*/
+        material_map_set = true;
+        one_material_map_used = true;
+        const char* imagefile = child->GetAttributeValue ("image");
+        const char *arrayfile = child->GetAttributeValue ("raw");
+        int width = child->GetAttributeValueAsInt ("width");
+        int height = child->GetAttributeValueAsInt ("height");
+        if (imagefile != 0)
+        {
+          csRef<iLoader> loader = csQueryRegistry<iLoader> (objreg);
+          csRef<iImage> map = loader->LoadImage(imagefile,CS_IMGFMT_PALETTED8);
+          if (map == 0)
+          {
+            synldr->ReportError ("crystalspace.terrain.factory.loader",
+              child, "Error reading in image file for heightmap '%s'",
+              imagefile);
+            return 0;
+          }
+
+          csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+            objreg, "crystalspace.shared.stringset");
+
+          state->SetIntegerMap(strings->Request("materialmap"), map);
+//          state->SetMaterialMapFile (imagefile, map->GetWidth (),
+//          map->GetHeight ());
+        }
+        else if (arrayfile != 0 && width != 0 && height != 0)
+        {
+//@@@ fixme or remove
+        synldr->ReportError ("crystalspace.terrain.factory.loader",
+              child, "Using raw files is broken! Complain to Fossi.");
+        return 0;
+          csRef<iVFS> vfs = csQueryRegistry<iVFS> (objreg);
+          csRef<iFile> file = vfs->Open (arrayfile, VFS_FILE_READ);
+          if (file == 0)
+          {
+
+            synldr->ReportError ("crystalspace.terrain.factory.loader",
+              child, "Error reading in raw file for heightmap '%s'",
+              arrayfile);
+            return 0;
+          }
+          csArray<char> array;
+          int index = 0;
+          while (!file->AtEOF())
+          {
+            file->Read (&array.GetExtend (index++), sizeof (char));
+          }
+//          state->SetIntegerMap (array, width, height);
+//          state->SetMaterialMapFile (arrayfile, width,
+//          height, true);
+        }
+        else
+        {
+          synldr->ReportError ("crystalpace.terrain.object.loader",
+            child, "No image or raw file specified for material map");
+          return 0;
+        }
+        break;
+      }
+      case XMLTOKEN_MATERIALALPHAMAP:
+      {
+        const char* imagefile = child->GetAttributeValue ("image");
+        if (imagefile != 0)
+        {
+          csRef<iLoader> loader = csQueryRegistry<iLoader> (objreg);
+          csRef<iImage> map = loader->LoadImage(imagefile,CS_IMGFMT_PALETTED8);
+          if (map == 0)
+          {
+            synldr->ReportError ("crystalspace.terraformer.simple.loader",
+              child, "Error reading in image file for heightmap '%s'",
+              imagefile);
+            return 0;
+          }
+          csString id = csString("alphamap ");
+          id += alphamapcount++;
+
+          csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+            objreg, "crystalspace.shared.stringset");
+
+          state->SetIntegerMap(strings->Request(id), map);
+        }
+        else
+        {
+          synldr->ReportError ("crystalspace.terraformer.simple.loader",
+            child, "No image file specified for materialraw  p");
+          return 0;
+        }
+        break;
+      }
       default:
         synldr->ReportError ("crystalspace.terraformer.simple.loader",
           child, "Unknown token!");
     }
   }
+
   return csPtr<iBase> (former);
 }
 
@@ -283,6 +378,12 @@ struct GetterUint32
   { return sizeof(uint32); }
 };
 
+namespace CS
+{
+namespace Plugins
+{
+namespace SimpleFormerLoader
+{
 template<typename Tgetter>
 class RawHeightmapReader
 {
@@ -296,7 +397,7 @@ public:
   {
     const size_t num = w * h;
     csDirtyAccessArray<float> fdata;
-    fdata.SetLength (num);
+    fdata.SetSize (num);
     for (size_t i = 0; i < num; i++)
     {
       Tgetter::Get (buf, fdata[i]);
@@ -344,11 +445,14 @@ public:
     return true;
   }
 };
+}
+}
+}
 
 csRef<iDataBuffer> csSimpleFormerLoader::GetDataBuffer (iDocumentNode* child)
 {
   const char *filename = child->GetContentsValue ();
-  csRef<iVFS> vfs = CS_QUERY_REGISTRY (objreg, iVFS);
+  csRef<iVFS> vfs = csQueryRegistry<iVFS> (objreg);
   csRef<iDataBuffer> buf = vfs->ReadFile (filename, false);
   if (buf == 0) 
   {
@@ -439,4 +543,3 @@ bool csSimpleFormerLoader::LoadHeightmapRawFloatBE (iDocumentNode* child,
   return reader.ReadRawMap (child);
 }
 
-} // namespace cspluginSimpleFormerLoader
