@@ -31,6 +31,7 @@
 #include "iutil/cmdline.h"
 
 #include "csplugincommon/directx/directdetection.h"
+#include "csplugincommon/directx/error.h"
 
 #include "csutil/win32/wintools.h"
 #include "csplugincommon/canvas/softfontcache.h"
@@ -90,7 +91,7 @@ bool csGraphics2DDDraw3::Initialize (iObjectRegistry *object_reg)
   if (!csGraphics2D::Initialize (object_reg))
     return false;
 
-  m_piWin32Assistant = CS_QUERY_REGISTRY (object_reg, iWin32Assistant);
+  m_piWin32Assistant = csQueryRegistry<iWin32Assistant> (object_reg);
   if (!m_piWin32Assistant)
   {
     MessageBox (0, 
@@ -102,8 +103,8 @@ bool csGraphics2DDDraw3::Initialize (iObjectRegistry *object_reg)
 
   DDetection.object_reg = object_reg;
 
-  csRef<iCommandLineParser> cmdline = CS_QUERY_REGISTRY (object_reg,
-						   iCommandLineParser);
+  csRef<iCommandLineParser> cmdline = 
+						   csQueryRegistry<iCommandLineParser> (object_reg);
   m_bHardwareCursor = config->GetBool ("Video.SystemMouseCursor", true);
   if (cmdline->GetOption ("sysmouse")) m_bHardwareCursor = true;
   if (cmdline->GetOption ("nosysmouse")) m_bHardwareCursor = false;
@@ -157,7 +158,7 @@ bool csGraphics2DDDraw3::Open ()
 
   if (DirectDevice == 0)
   {
-    InitFail (DD_FALSE, "Error creating DirectDevice\n");
+    InitFail ((HRESULT)~0, "Error creating DirectDevice");
     return false;
   }
 
@@ -172,7 +173,7 @@ bool csGraphics2DDDraw3::Open ()
   HRESULT ddrval;
   if ((ddrval = DirectDrawCreate ((LPGUID)pGuid, &m_lpDD, 0)) != DD_OK)
   {
-    InitFail (ddrval, "DirectDrawCreate FAILED (Code: %08lx)\n");
+    InitFail (ddrval, "DirectDrawCreate FAILED");
     return false;
   }
 
@@ -184,7 +185,7 @@ bool csGraphics2DDDraw3::Open ()
   HDC DC = GetDC (0);
   int desktopDepth = GetDeviceCaps (DC, BITSPIXEL);
   ReleaseDC (0, DC);
-  if (InitSurfaces () != DD_OK)
+  if (FAILED (InitSurfaces ()))
     return false;
 
   // Determine if switching from FS to windowed is allowed
@@ -404,7 +405,7 @@ bool csGraphics2DDDraw3::BeginDraw ()
   while (ret == DDERR_WASSTILLDRAWING)
     ret = m_lpddsBack->Lock (0, &ddsd, DDLOCK_SURFACEMEMORYPTR, 0);
 
-  if (ret != DD_OK)
+  if (FAILED (ret))
   {
     FinishDraw ();
     return false;
@@ -419,8 +420,7 @@ bool csGraphics2DDDraw3::BeginDraw ()
 void csGraphics2DDDraw3::FinishDraw ()
 {
   csGraphics2D::FinishDraw ();
-  if (FrameBufferLocked)
-    return;
+  if (FrameBufferLocked) return;
 
   if (m_bLocked)
   {
@@ -457,7 +457,7 @@ void csGraphics2DDDraw3::SetColorPalette ()
 
       if (!CreateIdentityPalette (Palette))
       {
-        InitFail (DD_FALSE, "Error creating Identity Palette.\n");
+        InitFail ((HRESULT)~0, "Error creating Identity Palette");
         return;
       }
 
@@ -619,7 +619,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
   hRet = m_lpDD->SetCooperativeLevel (m_hWnd, FullScreen ?
     (DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN) : DDSCL_NORMAL);
   if (hRet != DD_OK)
-    return InitFail (hRet, "SetCooperativeLevel FAILED (Code: %08lx)\n");
+    return InitFail (hRet, "SetCooperativeLevel FAILED");
 
   // Set window style bits
   SetWindowLong (m_hWnd, GWL_STYLE, FullScreen ? FULLSCREEN_STYLE : WINDOW_STYLE);
@@ -635,7 +635,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     ddsd.dwFlags = DDSD_CAPS;
     ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
     if ((hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsPrimary, 0)) != DD_OK)
-      return InitFail (hRet, "Cannot create primary surface for DirectDraw (Code: %08lx)\n");
+      return InitFail (hRet, "Cannot create primary surface for DirectDraw");
 
     // Create the backbuffer. In fullscreen mode by default we don't
     // use the backbuffer, but we use it in single-buffered modes.
@@ -645,7 +645,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
     if ((hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsBack, 0)) != DD_OK)
     {
-      InitFail (hRet, "CreateSurface for backbuffer FAILED (Code: %08lx)\n");
+      InitFail (hRet, "CreateSurface for backbuffer FAILED");
       return false;
     }
 
@@ -654,7 +654,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
       // Create a clipper object since this is for a Windowed render
       LPDIRECTDRAWCLIPPER pClipper;
       if ((hRet = m_lpDD->CreateClipper (0, &pClipper, 0)) != DD_OK)
-        return InitFail (hRet, "CreateClipper FAILED (Code: %08lx)\n");
+        return InitFail (hRet, "CreateClipper FAILED");
 
       // Associate the clipper with the window
       pClipper->SetHWnd (0, m_hWnd);
@@ -672,14 +672,14 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
     ddsd.dwBackBufferCount = 1;
 
     hRet = m_lpDD->CreateSurface (&ddsd, &m_lpddsPrimary, 0);
-    if (hRet != DD_OK)
-      return InitFail (hRet, "Cannot create primary surface for DirectDraw FAILED (Code: %08lx)\n");
+    if (FAILED (hRet))
+      return InitFail (hRet, "Cannot create primary surface for DirectDraw");
 
     ZeroMemory (&ddscaps, sizeof (ddscaps));
     ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
     hRet = m_lpddsPrimary->GetAttachedSurface (&ddscaps, &m_lpddsBack);
-    if (hRet != DD_OK)
-      return InitFail (hRet, "GetAttachedSurface FAILED (Code: %08lx)\n");
+    if (FAILED (hRet))
+      return InitFail (hRet, "GetAttachedSurface FAILED");
   }
 
   // get the pixel format
@@ -687,7 +687,7 @@ HRESULT csGraphics2DDDraw3::InitSurfaces ()
   ddpf.dwSize = sizeof (ddpf);
   hRet = m_lpddsPrimary->GetPixelFormat (&ddpf);
   if (hRet != DD_OK)
-    return InitFail (hRet, "Can't get pixel format (Code: %08lx)\n");
+    return InitFail (hRet, "Can't get pixel format");
 
   pfmt.RedMask = ddpf.dwRBitMask;
   pfmt.GreenMask = ddpf.dwGBitMask;
@@ -755,9 +755,17 @@ HRESULT csGraphics2DDDraw3::ChangeCoopLevel ()
     FinishDraw ();
   }
 
+  // Unlock the surface before switching the coop level
+  bool wasLocked = m_bLocked;
+  if (m_bLocked)
+  {
+    m_lpddsBack->Unlock (0);
+    m_bLocked = false;
+  }
+
   // Release all objects that need to be re-created for the new device
   if (FAILED (hRet = ReleaseAllObjects ()))
-    return InitFail (hRet, "ReleaseAllObjects FAILED (Code: %08lx)\n");
+    return InitFail (hRet, "ReleaseAllObjects FAILED");
 
   // In case we're coming from a fullscreen mode, restore the window size
   if (!FullScreen)
@@ -771,8 +779,24 @@ HRESULT csGraphics2DDDraw3::ChangeCoopLevel ()
   // Re-create the surfaces
   hRet = InitSurfaces ();
 
+  // Restore locked state
+  if (wasLocked)
+  {
+    DDSURFACEDESC ddsd;
+    ddsd.dwSize = sizeof (ddsd);
+    ddsd.lpSurface = 0;
+    HRESULT ret = DDERR_WASSTILLDRAWING;
+    while (ret == DDERR_WASSTILLDRAWING)
+      ret = m_lpddsBack->Lock (0, &ddsd, DDLOCK_SURFACEMEMORYPTR, 0);
+
+    if (FAILED (ret)) return ret;
+
+    Memory = (unsigned char *)ddsd.lpSurface;
+    m_bLocked = true;
+  }
+
   // Now restore the contents of backbuffer
-  if (oldBuffer)
+  if (SUCCEEDED (hRet) && oldBuffer)
   {
     int times;
     for (times = (m_bDoubleBuffer && FullScreen) ? 2 : 1; times; times--)
@@ -784,8 +808,8 @@ HRESULT csGraphics2DDDraw3::ChangeCoopLevel ()
 	FinishDraw ();
 	Print (0);
       }
-    delete [] oldBuffer;
   }
+  delete [] oldBuffer;
 
   return hRet;
 }
@@ -799,7 +823,13 @@ HRESULT csGraphics2DDDraw3::InitFail (HRESULT hRet, LPCTSTR szError)
   ReleaseAllObjects ();
   if (m_lpDD)
     m_lpDD->RestoreDisplayMode ();
-  Report (CS_REPORTER_SEVERITY_ERROR, szError, hRet);
+  if (hRet == (HRESULT)~0)
+    Report (CS_REPORTER_SEVERITY_ERROR, "%s", szError);
+  else
+    Report (CS_REPORTER_SEVERITY_ERROR, "%s\nError: %s [%s]", szError,
+    csDirectXError::GetErrorDescription (hRet),
+    csDirectXError::GetErrorSymbol (hRet));
+
   DestroyWindow (m_hWnd);
   return hRet;
 }
@@ -810,7 +840,7 @@ void csGraphics2DDDraw3::ClearSystemPalette ()
   HPALETTE BlackPal, OldPal;
   HDC hdc;
 
-  Palette = (LOGPALETTE*)malloc(sizeof(LOGPALETTE)+sizeof(PALETTEENTRY)*256);
+  Palette = (LOGPALETTE*)cs_malloc(sizeof(LOGPALETTE)+sizeof(PALETTEENTRY)*256);
 
   Palette->palNumEntries = 256;
   Palette->palVersion = 0x300;
@@ -836,7 +866,7 @@ void csGraphics2DDDraw3::ClearSystemPalette ()
   hdc = GetDC (0);
 
   BlackPal = CreatePalette (Palette);
-  free((void*)Palette);
+  cs_free((void*)Palette);
 
   OldPal = SelectPalette (hdc, BlackPal, FALSE);
   RealizePalette (hdc);
@@ -854,7 +884,7 @@ bool csGraphics2DDDraw3::CreateIdentityPalette (csRGBpixel *p)
   if (m_hWndPalette)
     DeleteObject (m_hWndPalette);
 
-  Palette = (LOGPALETTE*)malloc(sizeof(LOGPALETTE)+sizeof(PALETTEENTRY)*256);
+  Palette = (LOGPALETTE*)cs_malloc(sizeof(LOGPALETTE)+sizeof(PALETTEENTRY)*256);
 
   Palette->palNumEntries = 256;
   Palette->palVersion = 0x300;
@@ -878,7 +908,7 @@ bool csGraphics2DDDraw3::CreateIdentityPalette (csRGBpixel *p)
 
   m_hWndPalette = CreatePalette (Palette);
 
-  free((void*)Palette);
+  cs_free((void*)Palette);
 
   if (!m_hWndPalette)
     return false;
