@@ -24,6 +24,7 @@
 #include "sdl2d.h"
 #include "csgeom/csrect.h"
 #include "csutil/csinput.h"
+#include "csutil/eventnames.h"
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
 #include "ivaria/reporter.h"
@@ -184,7 +185,7 @@ void csGraphics2DSDL::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep = CS_QUERY_REGISTRY (object_reg, iReporter);
+  csRef<iReporter> rep = csQueryRegistry<iReporter> (object_reg);
   if (rep)
     rep->ReportV (severity, "crystalspace.canvas.sdl", msg, arg);
   else
@@ -337,7 +338,7 @@ bool csGraphics2DSDL::Open()
   th_lock = SDL_CreateMutex();
   SDL_CreateThread(drawing_thread, (void *)this);
 #else
-  *((void **)&Memory) = screen->pixels;
+  Memory = (uint8*)screen->pixels;
 #endif
 
   switch (Depth)
@@ -382,11 +383,13 @@ bool csGraphics2DSDL::Open()
   csEventID PreProcess = csevPreProcess (object_reg);
   csEventID PostProcess = csevPostProcess (object_reg);
 
-  csRef<iEventQueue> q = CS_QUERY_REGISTRY(object_reg, iEventQueue);
+  csRef<iEventQueue> q = csQueryRegistry<iEventQueue> (object_reg);
   if (q != 0)
   {
     csEventID events[] = { PreProcess, PostProcess, CS_EVENTLIST_END };
-    q->RegisterListener (scfiEventHandler, events);
+    // csGraphics2D::Initialize already setup weakEventHandler using
+    // RegisterWeakListener so we use plain RegisterListener here.
+    q->RegisterListener (weakEventHandler, events);
     if (!EventOutlet.IsValid())
       EventOutlet = q->CreateEventOutlet (this);
   }
