@@ -24,6 +24,7 @@
 #include "csgeom/csrect.h"
 #include "csutil/cfgacc.h"
 #include "csutil/event.h"
+#include "csutil/eventnames.h"
 #include "iutil/plugin.h"
 #include "iutil/cfgmgr.h"
 #include "iutil/cmdline.h"
@@ -61,7 +62,7 @@ void csGraphics2DXLib::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (object_reg, iReporter));
+  csRef<iReporter> rep (csQueryRegistry<iReporter> (object_reg));
   if (rep)
     rep->ReportV (severity, "crystalspace.canvas.softx", msg, arg);
   else
@@ -80,7 +81,7 @@ bool csGraphics2DXLib::Initialize (iObjectRegistry *object_reg)
     return false;
 
   csRef<iPluginManager> plugin_mgr (
-  	CS_QUERY_REGISTRY (object_reg, iPluginManager));
+  	csQueryRegistry<iPluginManager> (object_reg));
 
   xwin = CS_LOAD_PLUGIN (plugin_mgr, CS_XWIN_SCF_ID, iXWindow);
   if (!xwin)
@@ -95,8 +96,8 @@ bool csGraphics2DXLib::Initialize (iObjectRegistry *object_reg)
   bool do_shm;
   // Query system settings
   csConfigAccess Config(object_reg, "/config/video.cfg");
-  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
-						   iCommandLineParser));
+  csRef<iCommandLineParser> cmdline (
+						   csQueryRegistry<iCommandLineParser> (object_reg));
   sim_depth = Config->GetInt ("Video.SimulateDepth", 0);
 
   do_shm = Config->GetBool ("Video.XSHM", true);
@@ -120,11 +121,13 @@ bool csGraphics2DXLib::Initialize (iObjectRegistry *object_reg)
     }
   }
 
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
   if (q != 0)
   {
     // Tell event queue to call us on broadcast messages
-    q->RegisterListener (scfiEventHandler, csevCommandLineHelp(object_reg));
+    // csGraphics2D::Initialize already setup weakEventHandler using
+    // RegisterWeakListener so we use plain RegisterListener here.
+    q->RegisterListener (weakEventHandler, csevCommandLineHelp(object_reg));
     // Create the event outlet
     EventOutlet = q->CreateEventOutlet (this);
   }
