@@ -29,25 +29,29 @@
 #include "wx/wx.h"
 #endif
 
-#include "csutil/csuctransform.h"
-#include "csutil/sysfunc.h"
-#include "csutil/scf.h"
-#include "csutil/csinput.h"
-#include "csgeom/csrect.h"
 #include "csutil/cfgacc.h"
-#include "iutil/plugin.h"
-#include "iutil/eventh.h"
-#include "iutil/comp.h"
-#include "ivideo/texture.h"
+#include "csutil/csinput.h"
+#include "csutil/csuctransform.h"
+#include "csutil/eventnames.h"
+#include "csutil/scf.h"
+#include "csutil/sysfunc.h"
+#include "csgeom/csrect.h"
+
 #include "iengine/texture.h"
 #include "iutil/cfgfile.h"
+#include "iutil/comp.h"
+#include "iutil/eventh.h"
 #include "iutil/eventq.h"
 #include "iutil/objreg.h"
+#include "iutil/plugin.h"
 #include "ivaria/reporter.h"
+#include "ivideo/texture.h"
 
 #ifdef WIN32
 #include "csutil/win32/win32.h"
 #include "csutil/win32/wintools.h"
+#include <GL/gl.h>
+#include "csplugincommon/opengl/glcommon2d.h"
 #elif !defined(CS_PLATFORM_MACOSX)
 #define USE_GLX
 #endif
@@ -73,7 +77,7 @@ void csGraphics2DWX::Report (int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (object_reg, iReporter));
+  csRef<iReporter> rep (csQueryRegistry<iReporter> (object_reg));
   if (rep)
     rep->ReportV (severity, "crystalspace.canvas.glwx2d", msg, arg);
   else
@@ -95,7 +99,7 @@ bool csGraphics2DWX::Initialize (iObjectRegistry *object_reg)
   pfmt.PalEntries = 0;
 
   // Create the event outlet
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
   if (q != 0)
     EventOutlet = q->CreateEventOutlet (this);
 
@@ -117,11 +121,7 @@ void* csGraphics2DWX::GetProcAddress (const char *funcname)
 #ifdef WIN32
   return (void*)wglGetProcAddress (funcname);
 #elif defined(USE_GLX)
-  #if defined(GLX_VERSION_1_4)
-    return (void*) glXGetProcAddressARB ((const GLubyte*) funcname);
-  #else
-    return (void*) glXGetProcAddress ((const GLubyte*) funcname);
-  #endif
+    return (void*)glXGetProcAddressARB ((const GLubyte*) funcname);
 #else
   return 0;
 #endif
@@ -129,7 +129,7 @@ void* csGraphics2DWX::GetProcAddress (const char *funcname)
 
 csGraphics2DWX::~csGraphics2DWX ()
 {
-  Close ();  
+  Close ();
   // theCanvas is destroyed by wxWindows
 }
 
@@ -161,7 +161,7 @@ bool csGraphics2DWX::Open()
   if (is_open) return true;
 
 #ifdef WIN32
-  csRef<iWin32Assistant> w32(CS_QUERY_REGISTRY(object_reg, iWin32Assistant));
+  csRef<iWin32Assistant> w32(csQueryRegistry<iWin32Assistant> (object_reg));
   if(w32 != 0) w32->UseOwnMessageLoop(false);
 #endif
 
@@ -170,7 +170,7 @@ bool csGraphics2DWX::Open()
   if(myParent == 0)
   {
     Report (CS_REPORTER_SEVERITY_ERROR, "Parent frame in wxGLCanvas not initialized!!");
-	return false;
+  return false;
   }
 
   /*
@@ -184,12 +184,63 @@ bool csGraphics2DWX::Open()
     ((accumBits % 32) == 0) ? accumBits / 4 : accumBits / 3;
   */
 
+#ifdef WIN32
+
+  csGLPixelFormatPicker picker (this);
+
+  int pixelFormat = -1;
+
+  PIXELFORMATDESCRIPTOR pfd;
+  pixelFormat = FindPixelFormat (picker, pfd);
+
+  currentFormat[glpfvColorBits] = pfd.cColorBits;
+  currentFormat[glpfvAlphaBits] = pfd.cAlphaBits;
+  currentFormat[glpfvDepthBits] = pfd.cDepthBits;
+  currentFormat[glpfvStencilBits] = pfd.cStencilBits;
+  currentFormat[glpfvAccumColorBits] = pfd.cAccumBits;
+  currentFormat[glpfvAccumAlphaBits] = pfd.cAccumAlphaBits;
+
+  Depth = pfd.cColorBits; 
+
+  pfmt.PixelBytes = (pfd.cColorBits == 32) ? 4 : (pfd.cColorBits + 7) >> 3;
+  pfmt.RedBits = pfd.cRedBits;
+  pfmt.RedShift = pfd.cRedShift;
+  pfmt.RedMask = ((1 << pfd.cRedBits) - 1) << pfd.cRedShift;
+  pfmt.GreenBits = pfd.cGreenBits;
+  pfmt.GreenShift = pfd.cGreenShift;
+  pfmt.GreenMask = ((1 << pfd.cGreenBits) - 1) << pfd.cGreenShift;
+  pfmt.BlueBits = pfd.cBlueBits;
+  pfmt.BlueShift = pfd.cBlueShift;
+  pfmt.BlueMask = ((1 << pfd.cBlueBits) - 1) << pfd.cBlueShift;
+  pfmt.AlphaBits = pfd.cAlphaBits;
+  pfmt.AlphaShift = pfd.cAlphaShift;
+  pfmt.AlphaMask = ((1 << pfd.cAlphaBits) - 1) << pfd.cAlphaShift;
+  pfmt.PalEntries = 0;
+ 
   int desired_attributes[] =
+  {
+      WX_GL_RGBA,
+      WX_GL_DOUBLEBUFFER,
+      WX_GL_DEPTH_SIZE, currentFormat[glpfvDepthBits],
+      WX_GL_MIN_RED, pfd.cRedBits,
+      WX_GL_MIN_BLUE, pfd.cBlueBits,
+      WX_GL_MIN_GREEN, pfd.cGreenBits,
+      WX_GL_MIN_ALPHA, currentFormat[glpfvAlphaBits],
+      WX_GL_STENCIL_SIZE, currentFormat[glpfvStencilBits],
+      WX_GL_MIN_ACCUM_RED, pfd.cAccumRedBits,
+      WX_GL_MIN_ACCUM_BLUE, pfd.cAccumBlueBits,
+      WX_GL_MIN_ACCUM_GREEN, pfd.cAccumGreenBits,
+      WX_GL_MIN_ACCUM_ALPHA, currentFormat[glpfvAccumAlphaBits],
+      0
+  };
+#else
+    int desired_attributes[] =
     {
       WX_GL_RGBA,
       WX_GL_DOUBLEBUFFER,
       0
     };
+#endif
 
   /*WX_GL_DEPTH_SIZE, format[glpfvDepthBits],
     WX_GL_MIN_RED, colorComponentSize,
@@ -215,48 +266,24 @@ bool csGraphics2DWX::Open()
                              wxPoint(0, 0), wxSize(w, h), 0, wxT(""), desired_attributes);
 
   if(theCanvas == 0) Report(CS_REPORTER_SEVERITY_ERROR, "Failed creating GL Canvas!");
-  theCanvas->Show(true);
+  theCanvas->Show (true);
+  theCanvas->SetCurrent ();
 
 #ifdef WIN32
-  {
-    HDC hDC = (HDC)theCanvas->GetHDC();
-    PIXELFORMATDESCRIPTOR pfd;
-    if (DescribePixelFormat (hDC, ::GetPixelFormat (hDC), 
-      sizeof(PIXELFORMATDESCRIPTOR), &pfd) == 0)
-    {
-      DWORD error = GetLastError();
-      char* msg = cswinGetErrorMessage (error);
-      Report (CS_REPORTER_SEVERITY_ERROR,
-	"DescribePixelFormat failed: %s [%" PRId32 "]",
-	msg, error);
-      delete[] msg;
-    }
-    else
-    {
-      currentFormat[glpfvColorBits] = pfd.cColorBits;
-      currentFormat[glpfvAlphaBits] = pfd.cAlphaBits;
-      currentFormat[glpfvDepthBits] = pfd.cDepthBits;
-      currentFormat[glpfvStencilBits] = pfd.cStencilBits;
-      currentFormat[glpfvAccumColorBits] = pfd.cAccumBits;
-      currentFormat[glpfvAccumAlphaBits] = pfd.cAccumAlphaBits;
-
-      Depth = pfd.cColorBits; 
-    }
-  }
 #elif defined(USE_GLX)
   {
     Display* dpy = (Display*) wxGetDisplay ();
     GLXContext active_GLContext = glXGetCurrentContext();
     XVisualInfo *xvis = (XVisualInfo*)theCanvas->m_vi;
-    
+
     Report (CS_REPORTER_SEVERITY_NOTIFY, "Video driver GL/X version %s",
-      glXIsDirect (dpy, active_GLContext) ? "(direct renderer)" : 
+      glXIsDirect (dpy, active_GLContext) ? "(direct renderer)" :
       "(indirect renderer)");
     if (!glXIsDirect (dpy, active_GLContext))
     {
       Report (CS_REPORTER_SEVERITY_WARNING,
-	"Indirect rendering may indicate a flawed OpenGL setup if you run on "
-	"a local X server.");
+  "Indirect rendering may indicate a flawed OpenGL setup if you run on "
+  "a local X server.");
     }
 
     Depth = xvis->depth;
@@ -291,7 +318,7 @@ bool csGraphics2DWX::Open()
       color_bits += pfmt.BlueBits;
       glXGetConfig(dpy, xvis, GLX_ALPHA_SIZE, &alpha_bits);
       pfmt.AlphaBits = alpha_bits;
-        
+
       int bit;
       // Fun hack, xvis doesn't provide alpha mask
       bit=0; while (bit < alpha_bits) pfmt.AlphaMask |= (1<<(bit++));
@@ -302,7 +329,7 @@ bool csGraphics2DWX::Open()
       bit=0; while (!(pfmt.BlueMask & (1<<bit))) bit++; pfmt.BlueShift = bit;
       if (pfmt.AlphaMask)
       {
-	bit=0; while (!(pfmt.AlphaMask & (1<<bit))) bit++; pfmt.AlphaShift = bit;
+  bit=0; while (!(pfmt.AlphaMask & (1<<bit))) bit++; pfmt.AlphaShift = bit;
       }
     }
 
@@ -332,13 +359,13 @@ bool csGraphics2DWX::Open()
     {
       if (pfmt.RedMask > pfmt.BlueMask)
       {
-	Report (CS_REPORTER_SEVERITY_NOTIFY, "R%d:G%d:B%d:A%d, ",
-	  pfmt.RedBits, pfmt.GreenBits, pfmt.BlueBits, alpha_bits);
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "R%d:G%d:B%d:A%d, ",
+    pfmt.RedBits, pfmt.GreenBits, pfmt.BlueBits, alpha_bits);
       }
       else
       {
-	Report (CS_REPORTER_SEVERITY_NOTIFY, "B%d:G%d:R%d:A%d, ",
-	  pfmt.BlueBits, pfmt.GreenBits, pfmt.RedBits, alpha_bits);
+  Report (CS_REPORTER_SEVERITY_NOTIFY, "B%d:G%d:R%d:A%d, ",
+    pfmt.BlueBits, pfmt.GreenBits, pfmt.RedBits, alpha_bits);
       }
     }
 
@@ -355,6 +382,124 @@ bool csGraphics2DWX::Open()
 
   return true;
 }
+#ifdef WIN32
+
+struct DummyWndInfo
+{
+  int pixelFormat;
+  csGraphics2DWX* this_;
+  csGraphics2DGLCommon::GLPixelFormat* chosenFormat;
+};
+int csGraphics2DWX::FindPixelFormatGDI (HDC hDC, 
+                                        csGLPixelFormatPicker& picker)
+{
+  int newPixelFormat = 0;
+  GLPixelFormat& format = currentFormat;
+  while (picker.GetNextFormat (format))
+  {
+    PIXELFORMATDESCRIPTOR pfd = {
+      sizeof(PIXELFORMATDESCRIPTOR),  /* size */
+      1,                              /* version */
+      PFD_SUPPORT_OPENGL |
+      PFD_DOUBLEBUFFER |              /* support double-buffering */
+      PFD_DRAW_TO_WINDOW,
+      PFD_TYPE_RGBA,                  /* color type */
+      format[glpfvColorBits],         /* prefered color depth */
+      0, 0, 0, 0, 0, 0,               /* color bits (ignored) */
+      format[glpfvAlphaBits],         /* no alpha buffer */
+      0,                              /* alpha bits (ignored) */
+      format[glpfvAccumColorBits],	/* accumulation buffer */
+      0, 0, 0,			/* accum bits (ignored) */
+      format[glpfvAccumAlphaBits],	/* accum alpha bits */
+      format[glpfvDepthBits],         /* depth buffer */
+      format[glpfvStencilBits],	/* stencil buffer */
+      0,                              /* no auxiliary buffers */
+      PFD_MAIN_PLANE,                 /* main layer */
+      0,                              /* reserved */
+      0, 0, 0                         /* no layer, visible, damage masks */
+    };
+
+    newPixelFormat = ChoosePixelFormat (hDC, &pfd);
+
+    if (newPixelFormat == 0)
+    {
+      DWORD error = GetLastError();
+      char* msg = cswinGetErrorMessage (error);
+      Report (CS_REPORTER_SEVERITY_ERROR,
+        "ChoosePixelFormat failed: %s [%" PRId32 "]",
+        msg, error);
+      delete[] msg;
+    }
+
+    if (DescribePixelFormat (hDC, newPixelFormat, 
+      sizeof(PIXELFORMATDESCRIPTOR), &pfd) == 0)
+    {
+      DWORD error = GetLastError();
+      char* msg = cswinGetErrorMessage (error);
+      Report (CS_REPORTER_SEVERITY_ERROR,
+        "DescribePixelFormat failed: %s [%" PRId32 "]",
+        msg, error);
+      delete[] msg;
+    }
+
+    if (!(pfd.dwFlags & PFD_GENERIC_FORMAT) ||
+      (pfd.dwFlags & PFD_GENERIC_ACCELERATED))
+    {
+      return newPixelFormat;
+    }
+  }
+
+  return newPixelFormat;
+}
+
+
+int csGraphics2DWX::FindPixelFormat (csGLPixelFormatPicker& picker, PIXELFORMATDESCRIPTOR& pfd)
+{
+  static const char* dummyClassName = "CSGL_DummyWindow";
+
+  HINSTANCE ModuleHandle = GetModuleHandle(0);
+
+  WNDCLASSA wc;
+  wc.hCursor        = 0;
+  wc.hIcon	    = 0;
+  wc.lpszMenuName   = 0;
+  wc.lpszClassName  = dummyClassName;
+  wc.hbrBackground  = (HBRUSH)(COLOR_BTNFACE + 1);
+  wc.hInstance      = ModuleHandle;
+  wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
+  wc.lpfnWndProc    = DefWindowProc;
+  wc.cbClsExtra     = 0;
+  wc.cbWndExtra     = 0;
+
+  if (!RegisterClassA (&wc)) return false;
+
+  HWND wnd = CreateWindowA (dummyClassName, 0, 0, CW_USEDEFAULT, 
+    CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 0, 0,
+    ModuleHandle, 0);
+
+  int pixel_format = FindPixelFormatGDI (GetDC (wnd), picker);
+
+  if (DescribePixelFormat (GetDC (wnd), pixel_format,
+    sizeof(PIXELFORMATDESCRIPTOR), &pfd) == 0)
+  {
+    DWORD error = GetLastError();
+    char* msg = cswinGetErrorMessage (error);
+    Report (CS_REPORTER_SEVERITY_ERROR,
+      "DescribePixelFormat failed: %s [%" PRId32 "]",
+      msg, error);
+    delete[] msg;
+  }
+
+  DestroyWindow (wnd);
+
+  UnregisterClassA (dummyClassName, ModuleHandle);
+
+  ext.Reset();
+
+  return 0;
+}
+
+#endif
 
 void csGraphics2DWX::Close(void)
 {
@@ -419,7 +564,7 @@ void csGraphics2DWX::SetFullScreen (bool yesno)
 
 void csGraphics2DWX::AllowResize (bool iAllow)
 {
-	AllowResizing = iAllow;
+  AllowResizing = iAllow;
 }
 
 BEGIN_EVENT_TABLE(csGLCanvas, wxGLCanvas)
@@ -429,6 +574,7 @@ BEGIN_EVENT_TABLE(csGLCanvas, wxGLCanvas)
   EVT_KEY_DOWN( csGLCanvas::OnKeyDown )
   EVT_KEY_UP( csGLCanvas::OnKeyUp )
   EVT_ENTER_WINDOW( csGLCanvas::OnEnterWindow )
+  EVT_LEAVE_WINDOW( csGLCanvas::OnLeaveWindow )
   EVT_MOUSE_EVENTS( csGLCanvas::OnMouseEvent )
 END_EVENT_TABLE()
 
@@ -437,7 +583,7 @@ csGLCanvas::csGLCanvas(csGraphics2DWX* g, wxWindow *parent,
                        const wxPoint& pos,
                        const wxSize& size, long style,
                        const wxString& name, int* attr)
-  : wxGLCanvas(parent, id, pos, size, style | wxWANTS_CHARS, name, attr), 
+  : wxGLCanvas(parent, id, pos, size, style | wxWANTS_CHARS, name, attr),
     g2d(g)
 {
   int w, h;
@@ -467,7 +613,14 @@ csGLCanvas::~csGLCanvas()
 
 void csGLCanvas::OnEnterWindow( wxMouseEvent& WXUNUSED(event) )
 {
-  SetFocus();
+  csRef<iEventNameRegistry> enr = csQueryRegistry<iEventNameRegistry> (g2d->object_reg);
+  g2d->EventOutlet->Broadcast(csevFocusGained(enr));
+}
+
+void csGLCanvas::OnLeaveWindow( wxMouseEvent& WXUNUSED(event) )
+{
+  csRef<iEventNameRegistry> enr = csQueryRegistry<iEventNameRegistry> (g2d->object_reg);
+  g2d->EventOutlet->Broadcast(csevFocusLost(enr));
 }
 
 void csGLCanvas::OnPaint( wxPaintEvent& WXUNUSED(event) )
@@ -528,6 +681,17 @@ void csGLCanvas::OnMouseEvent( wxMouseEvent& event )
   {
     g2d->EventOutlet->Mouse(csmbRight, false, event.GetX(), event.GetY());
   }
+  else if(event.GetEventType() == wxEVT_MOUSEWHEEL)
+  {
+   if (event.GetWheelRotation() > 0)
+   {
+    g2d->EventOutlet->Mouse(csmbWheelUp, true, event.GetX(), event.GetY());
+   }
+   else
+   {
+     g2d->EventOutlet->Mouse(csmbWheelDown, true, event.GetX(), event.GetY());
+   }
+  }
 }
 
 static bool wxCodeToCSCode(int wxkey, utf32_char& raw, utf32_char& cooked)
@@ -536,7 +700,7 @@ static bool wxCodeToCSCode(int wxkey, utf32_char& raw, utf32_char& cooked)
     case WXK_ ## wx: { raw = CSKEY_ ## r; cooked = CSKEY_ ## c; return true; }
 #define MAPC(wx, r, c) \
     case WXK_ ## wx: { raw = CSKEY_ ## r; cooked = c; return true; }
-  switch(wxkey) 
+  switch(wxkey)
   {
     MAP (BACK,            BACKSPACE,    BACKSPACE)
     MAP (TAB,             TAB,          TAB)
@@ -569,7 +733,9 @@ static bool wxCodeToCSCode(int wxkey, utf32_char& raw, utf32_char& cooked)
     MAP (NUMPAD_DOWN,     PAD2,         DOWN)
     MAPC (NUMPAD3,        PAD3,         '3')
     MAP (NUMPAD_NEXT,     PAD3,         PGDN)
+#if wxVERSION_NUMBER < 2700
     MAP (NUMPAD_PAGEDOWN, PAD3,         PGDN)
+#endif
     MAPC (NUMPAD4,        PAD4,         '4')
     MAP (NUMPAD_LEFT,     PAD4,         LEFT)
     MAPC (NUMPAD5,        PAD5,         '5')
@@ -581,7 +747,9 @@ static bool wxCodeToCSCode(int wxkey, utf32_char& raw, utf32_char& cooked)
     MAP (NUMPAD_UP,       PAD8,         UP)
     MAPC (NUMPAD9,        PAD9,         '9')
     MAP (NUMPAD_PRIOR,    PAD9,         PGUP)
+#if wxVERSION_NUMBER < 2700
     MAP (NUMPAD_PAGEUP,   PAD9,         PGUP)
+#endif
     MAPC (MULTIPLY,       PADMULT,      '*')
     MAPC (NUMPAD_MULTIPLY,PADMULT,      '*')
     MAPC (ADD,            PADPLUS,      '+')
@@ -607,8 +775,10 @@ static bool wxCodeToCSCode(int wxkey, utf32_char& raw, utf32_char& cooked)
     MAP (F12,             F12,          F12)
     MAP (NUMLOCK,         PADNUM,       PADNUM)
     MAP (SCROLL,          SCROLLLOCK,   SCROLLLOCK)
+#if wxVERSION_NUMBER < 2700
     MAP (PAGEUP,          PGUP,         PGUP)
     MAP (PAGEDOWN,        PGDN,         PGDN)
+#endif
     MAP (NUMPAD_ENTER,    PADENTER,     ENTER)
     default: return false;
   }
@@ -630,7 +800,7 @@ void csGLCanvas::EmitKeyEvent(wxKeyEvent& event, bool down)
     cskey_cooked_new = event.GetKeyCode();
 #endif
   if (cskey_raw == 0)
-    csUnicodeTransform::MapToLower (cskey_cooked_new, &cskey_raw, 1, 
+    csUnicodeTransform::MapToLower (cskey_cooked_new, &cskey_raw, 1,
       csUcMapSimple);
   if (cskey_cooked == 0) cskey_cooked = cskey_cooked_new;
   if (cskey_raw != 0) g2d->EventOutlet->Key (cskey_raw, cskey_cooked, down);
