@@ -26,6 +26,7 @@
 #include "csgeom/csrect.h"
 #include "csutil/callstack.h"
 #include "csutil/cfgacc.h"
+#include "csutil/eventnames.h"
 #include "csutil/scf.h"
 #include "iutil/plugin.h"
 #include "iutil/cfgmgr.h"
@@ -129,8 +130,8 @@ bool csXWindow::Initialize (iObjectRegistry *object_reg)
   this->object_reg = object_reg;
   this->name_reg = csEventNameRegistry::GetRegistry(object_reg);
   csConfigAccess Config(object_reg, "/config/video.cfg");
-  csRef<iCommandLineParser> cmdline (CS_QUERY_REGISTRY (object_reg,
-						   iCommandLineParser));
+  csRef<iCommandLineParser> cmdline (
+						   csQueryRegistry<iCommandLineParser> (object_reg));
   do_hwmouse = Config->GetBool ("Video.SystemMouseCursor", true);
   if (cmdline->GetOption ("sysmouse")) do_hwmouse = true;
   if (cmdline->GetOption ("nosysmouse")) do_hwmouse = false;
@@ -159,7 +160,7 @@ bool csXWindow::Initialize (iObjectRegistry *object_reg)
   memset (MouseCursor, 0, sizeof (MouseCursor));
 
   // Create the event outlet
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
   if (q != 0)
     EventOutlet = q->CreateEventOutlet (this);
 
@@ -168,7 +169,7 @@ bool csXWindow::Initialize (iObjectRegistry *object_reg)
 		       &opcode, &first_event, &first_error))
   {
     csRef<iPluginManager> plugin_mgr (
-    	CS_QUERY_REGISTRY(object_reg, iPluginManager));
+    	csQueryRegistry<iPluginManager> (object_reg));
     xf86vm = CS_LOAD_PLUGIN (plugin_mgr, CS_XEXT_XF86VM_SCF_ID, iXExtF86VM);
   }
   return true;
@@ -263,8 +264,8 @@ bool csXWindow::Open ()
   XSetWMProtocols (dpy, wm_win, &wm_delete_window, 1);
 
   XClassHint *class_hint = XAllocClassHint();
-  class_hint->res_name = CS_CONST_CAST (char*, win_title.GetData());
-  class_hint->res_class = CS_CONST_CAST (char*, win_title.GetData());
+  class_hint->res_name = const_cast<char*> (win_title.GetData());
+  class_hint->res_class = const_cast<char*> (win_title.GetData());
   XmbSetWMProperties (dpy, wm_win,
 		      0, 0, 0, 0,
 		      0, 0, class_hint);
@@ -356,7 +357,7 @@ bool csXWindow::Open ()
   // Tell event queue to call us on every frame
   if (!scfiEventHandler)
     scfiEventHandler.AttachNew (new EventHandler (this));
-  csRef<iEventQueue> q (CS_QUERY_REGISTRY(object_reg, iEventQueue));
+  csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
   if (q != 0) {
     csEventID events[3] = { csevPreProcess(name_reg), 
 			    csevCommandLineHelp(name_reg), 
@@ -510,7 +511,7 @@ bool csXWindow::HandleEvent (iEvent &Event)
   utf32_char csKeyRaw = 0, csKeyCooked = 0;
   int charcount;
   char charcode [8];
-  bool down;
+  bool prevdown, down;
   bool resize = false;
 
   if (Event.Name == csevCommandLineHelp(name_reg))
@@ -541,7 +542,7 @@ bool csXWindow::HandleEvent (iEvent &Event)
         XRefreshKeyboardMapping (&event.xmapping);
 	break;
       case ClientMessage:
-	if (CS_STATIC_CAST(Atom, event.xclient.data.l[0]) == wm_delete_window)
+	if (static_cast<Atom> (event.xclient.data.l[0]) == wm_delete_window)
 	{
 	  EventOutlet->Broadcast (csevCanvasClose (name_reg, Canvas), (intptr_t)this);
 	  EventOutlet->Broadcast (csevQuit (name_reg));
@@ -563,6 +564,7 @@ bool csXWindow::HandleEvent (iEvent &Event)
         // Neat trick: look in event queue if we have KeyPress events ahead
 	// with same keycode. If this is the case, discard the KeyUp event
 	// in favour of KeyDown since this is most (sure?) an autorepeat
+        prevdown = (event.type == KeyPress);
         XCheckIfEvent (event.xkey.display, &event, CheckKeyPress,
 	  (XPointer)&event);
         down = (event.type == KeyPress);
@@ -717,7 +719,7 @@ bool csXWindow::HandleEvent (iEvent &Event)
 #undef MAP_KEY
         }
         if (csKeyRaw || csKeyCooked)
-          EventOutlet->Key(csKeyRaw, csKeyCooked, down);
+          EventOutlet->Key(csKeyRaw, csKeyCooked, down, prevdown != down);
         break;
       case FocusIn:
 	{
