@@ -32,7 +32,7 @@
 
 CS_IMPLEMENT_PLUGIN
 
-namespace cspluginJPGimg
+CS_PLUGIN_NAMESPACE_BEGIN(JPGImageIO)
 {
 
 CS_LEAKGUARD_IMPLEMENT (ImageJpgFile);
@@ -52,7 +52,7 @@ void Report (iObjectRegistry *object_reg, int severity, const char* msg, ...)
 {
   va_list arg;
   va_start (arg, msg);
-  csRef<iReporter> rep (CS_QUERY_REGISTRY (object_reg, iReporter));
+  csRef<iReporter> rep (csQueryRegistry<iReporter> (object_reg));
   if (rep)
     rep->ReportV (severity, "crystalspace.graphic.image.io.jpeg", 
       msg, arg);
@@ -99,7 +99,7 @@ struct jpg_datastore{
   size_t len;
 
   jpg_datastore () { data = 0; len = 0; }
-  ~jpg_datastore () { free (data); }
+  ~jpg_datastore () { cs_free (data); }
 };
 
 
@@ -127,7 +127,7 @@ static jpeg_boolean empty_output_buffer (j_compress_ptr cinfo)
 {
   my_dst_mgr *dest = (my_dst_mgr*)cinfo->dest;
 
-  dest->ds->data = (unsigned char*)realloc (dest->ds->data,
+  dest->ds->data = (unsigned char*)cs_realloc (dest->ds->data,
     dest->ds->len + sizeof(JOCTET) * my_dst_mgr::buf_len);
   if (!dest->ds->data)
     ERREXITS(cinfo,JERR_OUT_OF_MEMORY, "Could not reallocate enough memory");
@@ -147,7 +147,7 @@ static void term_destination (j_compress_ptr cinfo)
 
   if (len > 0)
   {
-    dest->ds->data = (unsigned char*)realloc (dest->ds->data,
+    dest->ds->data = (unsigned char*)cs_realloc (dest->ds->data,
       dest->ds->len + sizeof(JOCTET) * len);
     if (!dest->ds->data)
       ERREXITS(cinfo,JERR_OUT_OF_MEMORY, "Could not reallocate enough memory");
@@ -187,8 +187,8 @@ void csJPGImageIO::SetDithering (bool enable)
   ImageJpgFile::dither = enable;
 }
 
-csPtr<iDataBuffer> csJPGImageIO::Save(iImage *Image, iImageIO::FileFormatDescription*,
-  const char* extraoptions)
+csPtr<iDataBuffer> csJPGImageIO::Save(iImage *Image,
+    iImageIO::FileFormatDescription*, const char* extraoptions)
 {
   int format = Image->GetFormat ();
   switch (format & CS_IMGFMT_MASK)
@@ -613,4 +613,5 @@ csRef<iImageFileLoader> ImageJpgFile::InitLoader (csRef<iDataBuffer> source)
   return loader;
 }
 
-} // namespace cspluginJPGimg
+}
+CS_PLUGIN_NAMESPACE_END(JPGImageIO)
