@@ -19,7 +19,6 @@
 #ifndef __CS_PNGIMAGE_H__
 #define __CS_PNGIMAGE_H__
 
-#include "csgfx/memimage.h"
 #include "igraphic/imageio.h"
 #include "csutil/leakguard.h"
 #include "iutil/eventh.h"
@@ -27,7 +26,7 @@
 #include "iutil/databuff.h"
 #include "csplugincommon/imageloader/commonimagefile.h"
 
-namespace cspluginPNGimg
+CS_PLUGIN_NAMESPACE_BEGIN(PNGImageIO)
 {
 
 /**
@@ -105,6 +104,7 @@ public:
   CS_LEAKGUARD_DECLARE (ImagePngFile);
 };
 
-} // namespace cspluginPNGimg
+}
+CS_PLUGIN_NAMESPACE_END(PNGImageIO)
 
 #endif // __CS_PNGIMAGE_H__
