@@ -22,22 +22,32 @@
 
 #include "gl_r2t_framebuf.h"
 
+CS_PLUGIN_NAMESPACE_BEGIN(gl3d)
+{
+
 class csGLRender2TextureEXTfbo : public csGLRender2TextureFramebuf
 {
   bool enableFBO;
   GLuint framebuffer, depthRB, stencilRB;
   int fb_w, fb_h;
+  iTextureHandle* txthandle;
+  csString fboMsg;
 
   void FreeBuffers();
+  const char* FBStatusStr (GLenum status);
 public:
   csGLRender2TextureEXTfbo (csGLGraphics3D* G3D) 
     : csGLRender2TextureFramebuf (G3D), enableFBO(true), framebuffer (0),
-      depthRB(0), stencilRB(0) { }
+      depthRB(0), stencilRB(0), fb_w(-1), fb_h(-1), txthandle(0) { }
   virtual ~csGLRender2TextureEXTfbo();
 
-  virtual void SetRenderTarget (iTextureHandle* handle, bool persistent);
+  virtual void SetRenderTarget (iTextureHandle* handle, bool persistent,
+  	int subtexture);
   virtual void FinishDraw ();
 };
 
+}
+CS_PLUGIN_NAMESPACE_END(gl3d)
+
 #endif // __CS_GL_R2T_EXT_FB_O_H__
 
