@@ -26,55 +26,51 @@
 #include <GL/gl.h>
 #endif
 
-#include "csgeom/csrect.h"
-#include "csgeom/poly3d.h"
-#include "csgeom/transfrm.h"
-#include "csgeom/vector2.h"
-#include "csgeom/vector3.h"
-#include "csgfx/shadervarcontext.h"
-#include "csutil/cfgacc.h"
-#include "csutil/cscolor.h"
-#include "csutil/csstring.h"
-#include "csutil/csuctransform.h"
-#include "csutil/flags.h"
-#include "csutil/formatter.h"
-#include "csutil/parray.h"
-#include "csutil/scf.h"
-#include "csutil/scfstrset.h"
-#include "csutil/weakref.h"
-#include "csutil/weakrefarr.h"
-
 #include "iutil/comp.h"
 #include "iutil/dbghelp.h"
 #include "iutil/event.h"
 #include "iutil/eventh.h"
-#include "iutil/strset.h"
-#include "ivaria/bugplug.h"
-#include "ivideo/graph2d.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/halo.h"
-#include "ivideo/rndbuf.h"
-#include "ivideo/shader/shader.h"
 
+#include "csgeom/csrect.h"
+#include "csgeom/poly3d.h"
+#include "csgfx/shadervarcontext.h"
+#include "csutil/cfgacc.h"
+#include "csutil/csstring.h"
+#include "csutil/formatter.h"
+#include "csutil/parray.h"
+#include "csutil/scf_implementation.h"
+#include "csutil/weakref.h"
+#include "csutil/weakrefarr.h"
+
+#include "csplugincommon/opengl/glextmanager.h"
 #include "csplugincommon/opengl/glstates.h"
 
 #include "gl_txtmgr.h"
 #include "gl_renderbuffer.h"
 #include "gl_r2t_backend.h"
 
-class csGLTextureHandle;
-class csGLTextureManager;
 struct csGLExtensionManager;
-class csGLVBOBufferManager;
-class csGLRenderBuffer;
+
+struct iBugPlug;
 struct iClipper2D;
-struct iObjectRegistry;
-struct iTextureManager;
-struct iRenderBufferManager;
 struct iLightingManager;
+struct iObjectRegistry;
+struct iRenderBufferManager;
+struct iShaderManager;
+struct iTextureManager;
 
 struct iEvent;
 
+CS_PLUGIN_NAMESPACE_BEGIN(gl3d)
+{
+
+class csGLTextureHandle;
+class csGLTextureManager;
+
+/* The purpose of this class is to generate a formatted string, but only
+   if it's actually used. */
 class MakeAString
 {
 public:
@@ -144,7 +140,7 @@ public:
 #endif
 
 class csGLGraphics3D;
-class csOpenGLHalo : public iHalo
+class csOpenGLHalo : public scfImplementation1<csOpenGLHalo, iHalo>
 {
   /// The halo color
   float R, G, B;
@@ -160,8 +156,6 @@ class csOpenGLHalo : public iHalo
   csGLGraphics3D* G3D;
 
 public:
-  SCF_DECLARE_IBASE;
-
   csOpenGLHalo (float iR, float iG, float iB, unsigned char *iAlpha,
     int iWidth, int iHeight, csGLGraphics3D* iG3D);
 
@@ -189,11 +183,11 @@ class csGLGraphics3D : public scfImplementation3<csGLGraphics3D,
 {
 private:
   //friend declarations
+  friend class csGLBasicTextureHandle;
   friend class csGLSuperLightmap;
   friend class csGLRendererLightmap;
   friend class csGLTextureHandle;
   friend class csGLTextureManager;
-  friend class eiShaderRenderInterface;
 
   iObjectRegistry* object_reg;
   csRef<iGraphics2D> G2D;
@@ -214,6 +208,7 @@ private:
   csZBufMode current_zmode;
   bool zmesh;
   bool forceWireframe;
+  bool fixedFunctionForcefulEnable;
 
   int asp_center_x, asp_center_y;
   float aspect, inv_aspect;
@@ -228,6 +223,20 @@ private:
   // GetObjectToCamera() because it returns a `const&'.
   csReversibleTransform other2cam;
 
+  enum csPortalStatus {
+    // Floating portal status flags
+    // 'status' field of csClipPortal structure has the 
+    // CS_PORTALSTATUS_ZCLEARED flag when depth buffer 
+    // under the portal is cleared. Any portal from stack
+    // may have the flag
+    CS_PORTALSTATUS_ZCLEARED = 0x00000001, 
+    // 'status' field of csClipPortal structure has the 
+    // CS_PORTALSTATUS_SFILLED flag when stencil buffer under the portal is
+    // initialized with 'stencil_clip_value'. Just one portal from stack
+    // may have the flag
+    CS_PORTALSTATUS_SFILLED = 0x00000002
+  };
+    
   // Structure used for maintaining a stack of clipper portals.
   struct csClipPortal
   {
@@ -235,13 +244,26 @@ private:
     int num_poly;
     csPlane3 normal;
     csFlags flags;
+    csFlags status;
     csClipPortal () : poly (0) { }
     ~csClipPortal () { delete[] poly; }
   };
   csPDelArray<csClipPortal> clipportal_stack;
   bool clipportal_dirty;
   int clipportal_floating;
-
+  
+  //Calculate if the portal clipportal_stack[index] is
+  //mirrored by preceding portals. Each reflection reverses
+  //vertex order of portal polygon.
+  bool IsPortalMirrored(size_t index)
+  {
+    bool mirror = false;
+    for (size_t i = 0; i < index; i++) 
+      if (clipportal_stack[i]->flags.Check (CS_OPENPORTAL_MIRROR)) 
+        mirror = !mirror;
+    return mirror;
+  }
+  
   //csReversibleTransform object2camera;
   csReversibleTransform world2camera;
 
@@ -312,7 +334,7 @@ private:
   csRef<iTextureHandle> render_target;
   csGLRender2TextureBackend* r2tbackend;
 
-  /// Should we use special buffertype (VBO) or just systemmeory
+  /// Should we use special buffertype (VBO) or just system memory
   bool use_hw_render_buffers;
   csGLDRAWRANGEELEMENTS glDrawRangeElements;
   static GLvoid csAPIENTRY myDrawRangeElements (GLenum mode, GLuint start, 
@@ -335,6 +357,8 @@ private:
 	
   void Report (int severity, const char* msg, ...);
 
+  void ParseByteSize (const char* sizeStr, size_t& size);
+
   int GetMaxTextureSize () const { return rendercaps.maxTexWidth; }
 
   // Enables offsetting of Z values
@@ -400,7 +424,7 @@ private:
  */ //iTextureHandle* texunit[16]; // @@@ Hardcoded max number of units
   bool texunitenabled[16]; // @@@ Hardcoded max number of units
   GLuint texunittarget[16]; // @@@ Hardcoded max number of units
-  csRef<csGLTextureHandle> needNPOTSfixup[16]; // @@@ Hardcoded max number of units
+  csRef<csGLBasicTextureHandle> needNPOTSfixup[16]; // @@@ Hardcoded max number of units
   /// Array of buffers used for NPOTS texture coord fixup
   csArray<csRef<iRenderBuffer> > npotsFixupScrap;
   /// Whether an NPOTS scrap is attached to a TC bufer
@@ -428,11 +452,22 @@ private:
   csRef<iRenderBuffer> DoNPOTSFixup (iRenderBuffer* buffer, int unit);
   csRef<iRenderBuffer> DoColorFixup (iRenderBuffer* buffer);
 
+  // Minimal float depth(z) difference to store
+  // different values in depth buffer. Of course, for standard depth
+  // range [0.0, 1.0]. See glDepthRange() function reference
+  // It is equal to: 1.0/(2^GL_DEPTH_BITS-1) if GL_DEPTH_BITS<=24 and
+  // 1.0/(2^24-1) when GL_DEPTH_BITS>24 
+  float depth_epsilon;
+  
   // Draw a 2D polygon (screen space coordinates) with correct Z information
   // given the plane. This function will not set up any texture mapping,
   // shading, or color. Expects identity MODELVIEW and PROJECTION matrices!
   void Draw2DPolygon (csVector2* poly, int num_poly, const csPlane3& normal);
 
+  // The same as Draw2DPolygon but z-coordinate for all vertices = 1 - depth_epsilon
+  // At the moment the function is not used in depth writing operations
+  void DrawScreenPolygon (csVector2* poly, int num_poly);
+  
   bool enableDelaySwap;
   bool wantToSwap;
   int delayClearFlags;
@@ -553,13 +588,15 @@ public:
   { return current_zmode; }
   
   virtual void SetWorldToCamera (const csReversibleTransform& w2c);
+  virtual const csReversibleTransform& GetWorldToCamera () { return world2camera; }
 
   /// Set the current render target (0 for screen).
   virtual void SetRenderTarget (iTextureHandle* handle,
-	  bool persistent = false)
+	  bool persistent = false,
+	  int subtexture = 0)
   {
     render_target = handle;
-    r2tbackend->SetRenderTarget (handle, persistent);
+    r2tbackend->SetRenderTarget (handle, persistent, subtexture);
 
     int hasRenderTarget = (handle != 0) ? 1 : 0;
     G2D->PerformExtension ("userendertarget", hasRenderTarget);
@@ -576,9 +613,10 @@ public:
 
   /// Begin drawing in the renderer
   bool BeginDraw (int drawflags);
-
   /// Indicate that drawing is finished
   void FinishDraw ();
+  /// Return current draw flags
+  int GetCurrentDrawFlags() const;
 
   /// Do backbuffer printing
   void Print (csRect const* area);
@@ -586,7 +624,7 @@ public:
   /// Drawroutine. Only way to draw stuff
   void DrawMesh (const csCoreRenderMesh* mymesh,
     const csRenderMeshModes& modes,
-    const csArray<csShaderVariable*> &stacks);
+    const iShaderVarStack* stacks);
 
   /// Draw a 2D sprite
   virtual void DrawPixmap (iTextureHandle *hTex, int sx, int sy,
@@ -670,6 +708,9 @@ public:
   virtual float GetZBuffValue (int, int);
 
   virtual void RemoveFromCache (iRendererLightmap*) { }  
+
+  virtual bool PerformExtension (char const* command, ...);
+  virtual bool PerformExtensionV (char const* command, va_list args);
   //=========================================================================
 
 
@@ -726,5 +767,8 @@ public:
   { }
 };
 
+}
+CS_PLUGIN_NAMESPACE_END(gl3d)
+
 #endif // __CS_GL_RENDER3D_H__
 
