@@ -37,9 +37,6 @@
 
 CS_LEAKGUARD_IMPLEMENT (csShaderGLAFP);
 
-SCF_IMPLEMENT_IBASE_EXT(csShaderGLAFP)
-SCF_IMPLEMENT_IBASE_EXT_END
-
 void csShaderGLAFP::Report (int severity, const char* msg, ...)
 {
   va_list args;
@@ -63,14 +60,14 @@ void csShaderGLAFP::Deactivate()
 
 void csShaderGLAFP::SetupState (const csRenderMesh* /*mesh*/, 
                                 csRenderMeshModes& /*modes*/,
-	                        const csShaderVarStack &stacks)
+	                        const iShaderVarStack* stacks)
 {
   size_t i;
   const csGLExtensionManager* ext = shaderPlug->ext;
   csRef<csShaderVariable> var;
 
   // set variables
-  for(i = 0; i < variablemap.Length(); ++i)
+  for(i = 0; i < variablemap.GetSize (); ++i)
   {
     VariableMapEntry& mapping = variablemap[i];
 
@@ -87,7 +84,6 @@ void csShaderGLAFP::SetupState (const csRenderMesh* /*mesh*/,
     case csShaderVariable::INT:
     case csShaderVariable::FLOAT:
     case csShaderVariable::VECTOR2:
-    case csShaderVariable::COLOR:
     case csShaderVariable::VECTOR3:
     case csShaderVariable::VECTOR4:
       {
@@ -144,7 +140,6 @@ void csShaderGLAFP::SetupState (const csRenderMesh* /*mesh*/,
         case csShaderVariable::INT:
         case csShaderVariable::FLOAT:
         case csShaderVariable::VECTOR2:
-        case csShaderVariable::COLOR:
         case csShaderVariable::VECTOR3:
         case csShaderVariable::VECTOR4:
           {
@@ -316,7 +311,7 @@ bool csShaderGLAFP::Compile()
 {
   shaderPlug->Open ();
 
-  for (size_t i = 0; i < variablemap.Length ();)
+  for (size_t i = 0; i < variablemap.GetSize ();)
   {
     int dest;
     if (sscanf (variablemap[i].destination, "register %d", &dest) != 1)
