@@ -35,9 +35,6 @@
 
 CS_LEAKGUARD_IMPLEMENT (csShaderGLAVP);
 
-SCF_IMPLEMENT_IBASE_EXT(csShaderGLAVP)
-SCF_IMPLEMENT_IBASE_EXT_END
-
 void csShaderGLAVP::Report (int severity, const char* msg, ...)
 {
   va_list args;
@@ -61,14 +58,14 @@ void csShaderGLAVP::Deactivate()
 
 void csShaderGLAVP::SetupState (const csRenderMesh* /*mesh*/, 
                                 csRenderMeshModes& /*modes*/,
-	                        const csShaderVarStack &stacks)
+	                        const iShaderVarStack* stacks)
 {
   size_t i;
   const csGLExtensionManager* ext = shaderPlug->ext;
   csRef<csShaderVariable> var;
 
   // set variables
-  for(i = 0; i < variablemap.Length(); ++i)
+  for(i = 0; i < variablemap.GetSize (); ++i)
   {
     VariableMapEntry& mapping = variablemap[i];
 
@@ -85,7 +82,6 @@ void csShaderGLAVP::SetupState (const csRenderMesh* /*mesh*/,
     case csShaderVariable::INT:
     case csShaderVariable::FLOAT:
     case csShaderVariable::VECTOR2:
-    case csShaderVariable::COLOR:
     case csShaderVariable::VECTOR3:
     case csShaderVariable::VECTOR4:
       {
@@ -142,7 +138,6 @@ void csShaderGLAVP::SetupState (const csRenderMesh* /*mesh*/,
         case csShaderVariable::INT:
         case csShaderVariable::FLOAT:
         case csShaderVariable::VECTOR2:
-        case csShaderVariable::COLOR:
         case csShaderVariable::VECTOR3:
         case csShaderVariable::VECTOR4:
           {
@@ -283,11 +278,14 @@ bool csShaderGLAVP::LoadProgramStringToGL ()
     if (end)
       *(end-1) = 0;
 
-    Report (CS_REPORTER_SEVERITY_WARNING, 
-      "Couldn't load vertex program \"%s\"", description.GetDataSafe ());
-    Report (CS_REPORTER_SEVERITY_WARNING, "Program error at: \"%s\"", start);
-    Report (CS_REPORTER_SEVERITY_WARNING, "Error string: '%s'", 
-      programErrorString);
+    if (doVerbose)
+    {
+      Report (CS_REPORTER_SEVERITY_WARNING, 
+        "Couldn't load vertex program \"%s\"", description.GetDataSafe ());
+      Report (CS_REPORTER_SEVERITY_WARNING, "Program error at: \"%s\"", start);
+      Report (CS_REPORTER_SEVERITY_WARNING, "Error string: '%s'", 
+        programErrorString);
+    }
     return false;
   }
   else
@@ -309,8 +307,8 @@ bool csShaderGLAVP::Load (iShaderDestinationResolver*, iDocumentNode* program)
   if(!program)
     return false;
 
-  csRef<iStringSet> strings = CS_QUERY_REGISTRY_TAG_INTERFACE (
-    shaderPlug->object_reg, "crystalspace.shared.stringset", iStringSet);
+  csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
+    shaderPlug->object_reg, "crystalspace.shared.stringset");
 
   csRef<iDocumentNode> variablesnode = program->GetNode("arbvp");
   if (variablesnode)
@@ -334,7 +332,7 @@ bool csShaderGLAVP::Load (iShaderDestinationResolver*, const char* program,
   programBuffer.AttachNew (new csDataBuffer (csStrNew (program),
     strlen (program)));
 
-  for (size_t i = 0; i < mappings.Length(); i++)
+  for (size_t i = 0; i < mappings.GetSize (); i++)
   {
     variablemap.Push (VariableMapEntry (mappings[i]));
   }
@@ -346,7 +344,7 @@ bool csShaderGLAVP::Compile()
 {
   shaderPlug->Open ();
 
-  for (size_t i = 0; i < variablemap.Length ();)
+  for (size_t i = 0; i < variablemap.GetSize ();)
   {
     int dest;
     if (sscanf (variablemap[i].destination, "register %d", &dest) != 1)
