@@ -55,15 +55,27 @@ csGLShaderFVP::~csGLShaderFVP ()
 
 void csGLShaderFVP::Activate ()
 {
+  if (!do_lighting && shaderPlug->fixedFunctionForcefulEnable)
+  {
+    const GLenum state = GL_LIGHTING;
+    GLboolean s = glIsEnabled (state);
+    if (s) glDisable (state); else glEnable (state);
+    glBegin (GL_TRIANGLES);  glEnd ();
+    if (s) glEnable (state); else glDisable (state);
+  }
 }
 
 void csGLShaderFVP::Deactivate()
 {
 }
 
+const uint tuFlags = csGLStateCache::activateMatrix
+  | csGLStateCache::activateTexGen
+  | csGLStateCache::activateTexEnable;
+
 void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/, 
                                 csRenderMeshModes& /*modes*/,
-				const csShaderVarStack &stacks)
+				const iShaderVarStack* stacks)
 {
   size_t i;
 
@@ -72,9 +84,10 @@ void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/,
   if (do_lighting)
   {
     csVector4 v;
-    const csVector4 zero (0);
+    const csVector4 zero (0, 0, 0, 1);
+    const csVector4 one (1);
 
-    for(i = 0; i < lights.Length(); ++i)
+    for(i = 0; i < lights.GetSize (); ++i)
     {
       GLenum glLight = (GLenum)GL_LIGHT0+i;
       glEnable (glLight);
@@ -107,11 +120,12 @@ void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/,
       glLightfv (glLight, GL_SPOT_DIRECTION, (float*)&v);
     }
 
-    const csVector4 one (1);
     v = GetParamVectorVal (stacks, matAmbient, one);
     glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, (float*)&v);
     v = GetParamVectorVal (stacks, matDiffuse, one);
     glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, (float*)&v);
+    v = GetParamVectorVal (stacks, matEmission, zero);
+    glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (float*)&v);
     v = GetParamVectorVal (stacks, matSpecular, zero);
     glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, (float*)&v);
     float f = GetParamFloatVal (stacks, matSpecularExp, 0);
@@ -139,13 +153,13 @@ void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/,
     statecache->Enable_GL_LIGHTING ();
   }
 
-  for (i=0; i<layers.Length (); i++)
+  for (i=0; i<layers.GetSize (); i++)
   {
-    statecache->SetActiveTU ((int)i);
-    statecache->ActivateTU ();
+    statecache->SetCurrentTU ((int)i);
+    statecache->ActivateTU (tuFlags);
     if (layers[i].texgen == TEXGEN_PROJECTION)
     {
-        if (!lights.Length())
+        if (!lights.GetSize ())
         {
 	  continue;
         }
@@ -164,7 +178,7 @@ void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/,
         statecache->Enable_GL_TEXTURE_GEN_S ();
         statecache->Enable_GL_TEXTURE_GEN_T ();
         statecache->Enable_GL_TEXTURE_GEN_R ();
-        glEnable(GL_TEXTURE_GEN_Q);
+        statecache->Enable_GL_TEXTURE_GEN_Q ();
         glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
         glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
         glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
@@ -216,7 +230,7 @@ void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/,
     }
     else if (layers[i].texgen == TEXGEN_TEXTURE3D)
     {
-        if (!lights.Length())
+        if (!lights.GetSize ())
         {
 	  continue;
         }
@@ -432,11 +446,11 @@ void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/,
       }
     }
 
-    if (layers[i].texMatrixOps.Length() > 0)
+    if (layers[i].texMatrixOps.GetSize () > 0)
     {
       statecache->SetMatrixMode (GL_TEXTURE);
 
-      for (size_t j = 0; j < layers[i].texMatrixOps.Length(); j++)
+      for (size_t j = 0; j < layers[i].texMatrixOps.GetSize (); j++)
       {
 	TexMatrixOp& op = layers[i].texMatrixOps[j];
 
@@ -513,8 +527,11 @@ void csGLShaderFVP::SetupState (const csRenderMesh* /*mesh*/,
     }
   }
 
-  statecache->SetActiveTU (0);
-  statecache->ActivateTU ();
+  if (shaderPlug->ext->CS_GL_ARB_multitexture)
+  {
+    statecache->SetCurrentTU (0);
+    statecache->ActivateTU (tuFlags);
+  }
 
   var = csGetShaderVariableFromStack (stacks, primcolvar);
   if (var)
@@ -531,7 +548,7 @@ void csGLShaderFVP::ResetState ()
 
   if (do_lighting)
   {
-    for (i = 0; i < lights.Length(); ++i)
+    for (i = 0; i < lights.GetSize (); ++i)
       glDisable ((GLenum)GL_LIGHT0+i);
 
     if (colorMaterial != 0)
@@ -542,24 +559,28 @@ void csGLShaderFVP::ResetState ()
     statecache->Disable_GL_LIGHTING ();
   }
 
-  for (i=0; i<layers.Length (); i++)
+  i = layers.GetSize ();
+  while (i-- > 0)
   {
-    statecache->SetActiveTU ((int)i);
-    statecache->ActivateTU ();
+    statecache->SetCurrentTU ((int)i);
+    statecache->ActivateTU (tuFlags);
     if ((layers[i].texgen != TEXGEN_NONE) ||
-      (layers[i].texMatrixOps.Length() > 0))
+      (layers[i].texMatrixOps.GetSize () > 0))
     {
       statecache->Disable_GL_TEXTURE_GEN_S ();
       statecache->Disable_GL_TEXTURE_GEN_T ();
       statecache->Disable_GL_TEXTURE_GEN_R ();
-	  glDisable(GL_TEXTURE_GEN_Q);
+      statecache->Disable_GL_TEXTURE_GEN_Q ();
 
       statecache->SetMatrixMode (GL_TEXTURE);
       glLoadIdentity ();
     }
   }
-  statecache->SetActiveTU (0);
-  statecache->ActivateTU ();
+  if (shaderPlug->ext->CS_GL_ARB_multitexture)
+  {
+    statecache->SetCurrentTU (0);
+    statecache->ActivateTU (tuFlags);
+  }
 }
 
 bool csGLShaderFVP::ParseTexMatrixOp (iDocumentNode* node, 
@@ -725,8 +746,8 @@ bool csGLShaderFVP::Load(iShaderDestinationResolver* resolve,
   string_world2camera = strings->Request ("world2camera transform");
   string_object2world = strings->Request ("object2world transform");
 
-  csRef<iShaderManager> shadermgr = CS_QUERY_REGISTRY(
-  	objectReg, iShaderManager);
+  csRef<iShaderManager> shadermgr = 
+  	csQueryRegistry<iShaderManager> (objectReg);
 
   csRef<iDocumentNode> variablesnode = program->GetNode("fixedvp");
   if(variablesnode)
@@ -743,7 +764,7 @@ bool csGLShaderFVP::Load(iShaderDestinationResolver* resolve,
         case XMLTOKEN_LIGHT:
           {
             do_lighting = true;
-	    LightingEntry& entry = lights.GetExtend (lights.Length());
+	    LightingEntry& entry = lights.GetExtend (lights.GetSize ());
 
             entry.lightnum = child->GetAttributeValueAsInt("num");
 
@@ -778,8 +799,8 @@ bool csGLShaderFVP::Load(iShaderDestinationResolver* resolve,
 		variablesnode, "'layer' attribute invalid");
 	      return false;
 	    }
-            if (layers.Length ()<= (size_t)layer)
-              layers.SetLength (layer+1);
+            if (layers.GetSize ()<= (size_t)layer)
+              layers.SetSize (layer+1);
 	    if (!ParseProgramParam (child, layers[layer].constcolor, ParamFloat | 
 	      ParamVector3 | ParamVector4 | ParamShaderExp))
 	      return false;
@@ -805,8 +826,8 @@ bool csGLShaderFVP::Load(iShaderDestinationResolver* resolve,
 		variablesnode, "'layer' attribute invalid");
 	      return false;
 	    }
-            if (layers.Length () <= (size_t)layer)
-              layers.SetLength (layer+1);
+            if (layers.GetSize () <= (size_t)layer)
+              layers.SetSize (layer+1);
             const char* str;
             if ((str = child->GetAttributeValue ("type")))
             {
@@ -870,8 +891,8 @@ bool csGLShaderFVP::Load(iShaderDestinationResolver* resolve,
 		variablesnode, "'layer' attribute invalid");
 	      return false;
 	    }
-            if (layers.Length () <= (size_t)layer)
-              layers.SetLength (layer+1);
+            if (layers.GetSize () <= (size_t)layer)
+              layers.SetSize (layer+1);
 	    if (!ParseTexMatrix (child, layers[layer].texMatrixOps))
 	      return false;
 	  }
@@ -906,13 +927,18 @@ bool csGLShaderFVP::Load(iShaderDestinationResolver* resolve,
 	    }
 	  }
 	  break;
+	case XMLTOKEN_MATAMBIENT:
+	  if (!ParseProgramParam (child, matAmbient,
+	    ParamVector | ParamShaderExp))
+	    return false;
+	  break;
 	case XMLTOKEN_MATDIFFUSE:
 	  if (!ParseProgramParam (child, matDiffuse,
 	    ParamVector | ParamShaderExp))
 	    return false;
 	  break;
-	case XMLTOKEN_MATAMBIENT:
-	  if (!ParseProgramParam (child, matAmbient,
+	case XMLTOKEN_MATEMISSION:
+	  if (!ParseProgramParam (child, matEmission,
 	    ParamVector | ParamShaderExp))
 	    return false;
 	  break;
@@ -967,7 +993,10 @@ bool csGLShaderFVP::Compile()
 
   size_t i;
 
-  for (i=0; i<layers.Length (); i++)
+  if (layers.GetSize () > (size_t)shaderPlug->texUnits)
+    return false;
+
+  for (i=0; i<layers.GetSize (); i++)
     if ((layers[i].texgen == TEXGEN_REFLECT_CUBE) &&
       !shaderPlug->ext->CS_GL_ARB_texture_cube_map)
       return false;
@@ -976,10 +1005,10 @@ bool csGLShaderFVP::Compile()
     && !shaderPlug->ext->CS_GL_EXT_separate_specular_color)
     return false;
 
-  g3d = CS_QUERY_REGISTRY (objectReg, iGraphics3D);
+  g3d = csQueryRegistry<iGraphics3D> (objectReg);
 
   //get a statecache
-  csRef<iGraphics2D> g2d = CS_QUERY_REGISTRY (objectReg, iGraphics2D);
+  csRef<iGraphics2D> g2d = csQueryRegistry<iGraphics2D> (objectReg);
   g2d->PerformExtension ("getstatecache", &statecache);
 
   return true;
