@@ -32,25 +32,27 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "glshader_ps1.h"
 #include "ps1_parser.h"
 
+CS_PLUGIN_NAMESPACE_BEGIN(GLShaderPS1)
+{
+
+const csPixelShaderParser::PS_InstructionData 
+  csPixelShaderParser::PS_Instructions[CS_PS_INS_END_OF_LIST] = {
+  {0, 0, false}, /* CS_PS_INS_INVALID */
+#define PS_INSTR(instr, args, psflags)				            \
+  {(psflags) & CS_PS_MASKVERSIONS, args, !((psflags) & CS_PS_UNSUPPORTED)},
+#define PS_VER_INSTR(x,y)						    \
+  {CS_PS_ALLVERSIONS, 0, true},
+#include "ps1_instr.inc"
+};
+
 void csPixelShaderParser::RegisterInstructions ()
 {
-  PS_Instructions[CS_PS_INS_INVALID].arguments = 0;
-  PS_Instructions[CS_PS_INS_INVALID].supported = false;
-  PS_Instructions[CS_PS_INS_INVALID].versions = 0;
-#define PS_INSTR(instr, args, psversion)				    \
-  PS_Instructions[CS_PS_INS_ ## instr].arguments = args;		    \
-  PS_Instructions[CS_PS_INS_ ## instr].versions = psversion;		    \
-  PS_Instructions[CS_PS_INS_ ## instr].supported = true;		    \
+#define PS_INSTR(instr, args, psflags)				            \
   instrStrings.Register (#instr, CS_PS_INS_ ## instr);
 #define PS_VER_INSTR(x,y)						    \
-  PS_Instructions[CS_PS_INS_PS_ ## x ## _ ## y].arguments = 0;		    \
-  PS_Instructions[CS_PS_INS_PS_ ## x ## _ ## y].versions = CS_PS_ALLVERSIONS;\
-  PS_Instructions[CS_PS_INS_PS_ ## x ## _ ## y].supported = true;	    \
   instrStrings.Register ("PS_" #x "_" #y, CS_PS_INS_PS_ ## x ## _ ## y);    \
   instrStrings.Register ("PS." #x "." #y, CS_PS_INS_PS_ ## x ## _ ## y);   
 #include "ps1_instr.inc"
-
-  PS_Instructions[CS_PS_INS_BEM].supported = false;
 }
 
 void csPixelShaderParser::Report (int severity, const char* msg, ...)
@@ -183,7 +185,7 @@ bool csPixelShaderParser::GetInstruction (const char *str,
   if (comment != (size_t)-1) line.Truncate (comment);
   line.Trim ();
   
-  if(line.Length() < 1) return true; // Ignore blank lines
+  if(line.Length () < 1) return true; // Ignore blank lines
 
   // Ignore pairing modifier
   if (line.GetAt (0) == '+')
@@ -459,17 +461,15 @@ bool csPixelShaderParser::ParseProgram (iDataBuffer* program)
   return true;
 }
 
-#ifdef CS_DEBUG
-
 static char GetRegType (csPSRegisterType type)
 {
   switch (type)
   {
-    case CS_PS_REG_TEX:		return 't'; break;
-    case CS_PS_REG_CONSTANT:	return 'c'; break;
-    case CS_PS_REG_TEMP:	return 'r'; break;
-    case CS_PS_REG_COLOR:	return 'v'; break;
-    default:			return '?'; break;
+    case CS_PS_REG_TEX:		return 't';
+    case CS_PS_REG_CONSTANT:	return 'c';
+    case CS_PS_REG_TEMP:	return 'r';
+    case CS_PS_REG_COLOR:	return 'v';
+    default:			return '?';
   }
 }
 
@@ -501,10 +501,9 @@ static void GetSrcRegname (csPSRegisterType type, int num, uint mods,
   if (mods & CS_PS_RMOD_XYZ) str << ".xyz";
   if (mods & CS_PS_RMOD_XYW) str << ".xyw";
 }
-#endif
 
 void csPixelShaderParser::GetInstructionString (
-  const csPSProgramInstruction& instr, csString& str)
+  const csPSProgramInstruction& instr, csString& str) const
 {
   str << instrStrings.Request (instr.instruction);
   if (instr.inst_mods & CS_PS_IMOD_X2) str << "_x2";
@@ -516,39 +515,47 @@ void csPixelShaderParser::GetInstructionString (
   if (instr.inst_mods & CS_PS_IMOD_SAT) str << "_sat";
 }
 
-#ifdef CS_DEBUG
+void csPixelShaderParser::GetInstructionLine (
+  const csPSProgramInstruction& instr, csString& str) const
+{
+  csString instrStr;
+  GetInstructionString (instr, instrStr);
+  str << instrStr;
+
+  str << ' ';
+  str << GetRegType (instr.dest_reg);
+  str << instr.dest_reg_num;
+
+  if (instr.dest_reg_mods != 0) str << '.';
+  if (instr.dest_reg_mods & CS_PS_WMASK_RED) str << 'r';
+  if (instr.dest_reg_mods & CS_PS_WMASK_GREEN) str << 'g';
+  if (instr.dest_reg_mods & CS_PS_WMASK_BLUE) str << 'b';
+  if (instr.dest_reg_mods & CS_PS_WMASK_ALPHA) str << 'a';
+
+  for (int j = 0; j < 3; j++)
+  {
+    if (instr.src_reg[j] == CS_PS_REG_NONE) break;
+    str << ", ";
+    
+    GetSrcRegname (instr.src_reg[j], instr.src_reg_num[j], 
+	instr.src_reg_mods[j], str);
+  }
+}
+
 void csPixelShaderParser::WriteProgram (
 	const csArray<csPSProgramInstruction>& instrs, 
 	csString& str)
 {
-  for(size_t i = 0; i < instrs.Length (); i++)
+  for(size_t i = 0; i < instrs.GetSize (); i++)
   {
     const csPSProgramInstruction& instr = instrs.Get (i);
 
-    csString instrStr;
-    GetInstructionString (instr, instrStr);
-    str << instrStr;
-
-    str << ' ';
-    str << GetRegType (instr.dest_reg);
-    str << instr.dest_reg_num;
-
-    if (instr.dest_reg_mods != 0) str << '.';
-    if (instr.dest_reg_mods & CS_PS_WMASK_RED) str << 'r';
-    if (instr.dest_reg_mods & CS_PS_WMASK_GREEN) str << 'g';
-    if (instr.dest_reg_mods & CS_PS_WMASK_BLUE) str << 'b';
-    if (instr.dest_reg_mods & CS_PS_WMASK_ALPHA) str << 'a';
-
-    for (int j = 0; j < 3; j++)
-    {
-      if (instr.src_reg[j] == CS_PS_REG_NONE) break;
-      str << ", ";
-      
-      GetSrcRegname (instr.src_reg[j], instr.src_reg_num[j], 
-	instr.src_reg_mods[j], str);
-    }
-
+    csString line;
+    GetInstructionLine (instr, line);
+    str << line;
     str << '\n';
   }
 }
-#endif
+
+}
+CS_PLUGIN_NAMESPACE_END(GLShaderPS1)
