@@ -29,10 +29,10 @@
 
 #include "scanline_base.h"
 
-namespace cspluginSoftshader
+CS_PLUGIN_NAMESPACE_BEGIN(SoftShader)
 {
 
-using namespace CrystalSpace::SoftShader;
+using namespace CS::PluginCommon::SoftShader;
 
 class csSoftShader : 
   public scfImplementation3<csSoftShader, 
@@ -75,7 +75,8 @@ public:
   /** @} */
 };
 
-} // namespace cspluginSoftshader
+}
+CS_PLUGIN_NAMESPACE_END(SoftShader)
 
 #endif //__SOFTSHADER_H__
 
