@@ -33,9 +33,10 @@
 #include "cstool/rbuflock.h"
 #include "csutil/cscolor.h"
 #include "csutil/event.h"
+#include "csutil/scfarray.h"
 #include "csutil/scfstrset.h"
 #include "csutil/sysfunc.h"
-#include "iengine/portal.h"
+#include "csutil/eventnames.h"
 #include "iutil/cfgfile.h"
 #include "iutil/cmdline.h"
 #include "iutil/event.h"
@@ -51,11 +52,12 @@
 #include "sft3dcom.h"
 #include "soft_txt.h"
 #include "clip_znear.h"
+#include "vertextransform.h"
 
-namespace cspluginSoft3d
+CS_PLUGIN_NAMESPACE_BEGIN(Soft3D)
 {
 
-  using namespace CrystalSpace::SoftShader;
+using namespace CS::PluginCommon::SoftShader;
 
 ///---------------------------------------------------------------------------
 
@@ -81,13 +83,10 @@ csSoftwareGraphics3DCommon::csSoftwareGraphics3DCommon (iBase* parent) :
   z_buffer = 0;
   line_table = 0;
 
-  Caps.CanClip = true;
   Caps.minTexHeight = 2;
   Caps.minTexWidth = 2;
   Caps.maxTexHeight = 1024;
   Caps.maxTexWidth = 1024;
-  Caps.MaxAspectRatio = 32768;
-  Caps.NeedsPO2Maps = true;
   Caps.SupportsPointSprites = false;
   Caps.DestinationAlpha = false;
   Caps.StencilShadows = false;
@@ -109,13 +108,15 @@ csSoftwareGraphics3DCommon::csSoftwareGraphics3DCommon (iBase* parent) :
   memset (processedColorsFlag, 0, sizeof (processedColorsFlag));
   memset (triDraw, 0, sizeof (triDraw));
   specifica = 0;
+
+  current_drawflags = 0;
 }
 
 csSoftwareGraphics3DCommon::~csSoftwareGraphics3DCommon ()
 {
   if (scfiEventHandler)
   {
-    csRef<iEventQueue> q (CS_QUERY_REGISTRY (object_reg, iEventQueue));
+    csRef<iEventQueue> q (csQueryRegistry<iEventQueue> (object_reg));
     if (q != 0)
       q->RemoveListener (scfiEventHandler);
     scfiEventHandler->DecRef ();
@@ -208,11 +209,14 @@ bool csSoftwareGraphics3DCommon::Open ()
   csRef<iPluginManager> plugin_mgr (
     csQueryRegistry<iPluginManager> (object_reg));
 
-  CS_QUERY_REGISTRY_PLUGIN(shadermgr, object_reg,
-    "crystalspace.graphics3d.shadermanager", iShaderManager);
+  shadermgr = csQueryRegistryOrLoad<iShaderManager> (object_reg,
+    "crystalspace.graphics3d.shadermanager");
+  if (!shadermgr) return false;
 
   SetupSpecifica();
 
+  vertexTransform = new VertexTransform;
+
   csRef<iDefaultShader> shaderPlugin = csLoadPlugin<iDefaultShader> (
     plugin_mgr, "crystalspace.graphics3d.shader.software");
   if (!shaderPlugin.IsValid())
@@ -270,6 +274,7 @@ void csSoftwareGraphics3DCommon::Close ()
   memset (triDraw, 0, sizeof (triDraw));
   delete specifica; specifica = 0;
   delete[] ilaceSaveBuf; ilaceSaveBuf = 0;
+  delete vertexTransform; vertexTransform = 0;
 
   G2D->Close ();
   width = height = -1;
@@ -487,7 +492,7 @@ bool csSoftwareGraphics3DCommon::BeginDraw (int DrawFlags)
 {
   clipportal_dirty = true;
   clipportal_floating = 0;
-  CS_ASSERT (clipportal_stack.Length () == 0);
+  CS_ASSERT (clipportal_stack.GetSize () == 0);
 
   if ((G2D->GetWidth() != display_width) ||
       (G2D->GetHeight() != display_height))
@@ -499,6 +504,7 @@ bool csSoftwareGraphics3DCommon::BeginDraw (int DrawFlags)
     if (!G2D->BeginDraw())
       return false;
   }
+  current_drawflags = DrawFlags;
   bool wasSmallerActive = smallerActive;
   smallerActive = do_smaller_rendering 
     && ((DrawFlags & CSDRAW_MASK2D3D) == CSDRAW_3DGRAPHICS)
@@ -623,7 +629,7 @@ void csSoftwareGraphics3DCommon::FinishDraw ()
         }
       }
     
-      SetRenderTarget (0, false);
+      SetRenderTarget (0, false, 0);
       if (oldIlaceMode != -1) do_interlaced = oldIlaceMode;
     }
   }
@@ -631,6 +637,7 @@ void csSoftwareGraphics3DCommon::FinishDraw ()
   if (DrawMode & (CSDRAW_2DGRAPHICS | CSDRAW_3DGRAPHICS))
     G2D->FinishDraw ();
   DrawMode = 0;
+  current_drawflags = 0;
 }
 
 #define SMALL_D 0.01
@@ -664,16 +671,17 @@ void csSoftwareGraphics3DCommon::OpenPortal (size_t numVertices,
   // number. Otherwise we start at one.
   if (clipportal_floating)
     clipportal_floating++;
-  else if (flags.Check(CS_PORTAL_FLOAT))
+  else if (flags.Check(CS_OPENPORTAL_FLOAT))
     clipportal_floating = 1;
 }
 
 void csSoftwareGraphics3DCommon::ClosePortal ()
 {
-  if (clipportal_stack.Length () <= 0) return;
+  if (clipportal_stack.GetSize () <= 0) return;
   csClipPortal* cp = clipportal_stack.Pop ();
 
-  if (cp->flags.Check(CS_PORTAL_ZFILL))
+  if (cp->flags.Check(CS_OPENPORTAL_ZFILL)
+    && (DrawMode != 0)) // polyrast_ZFill is only valid after a BefinDraw()
   {
     CS_ALLOC_STACK_ARRAY(csVector3, vertices, cp->num_poly);
     for (int v = 0; v < cp->num_poly; v++)
@@ -751,7 +759,8 @@ float csSoftwareGraphics3DCommon::GetZBuffValue (int x, int y)
 }
 
 void csSoftwareGraphics3DCommon::SetRenderTarget (iTextureHandle* handle,
-	bool persistent)
+	bool persistent,
+	int subtexture)
 {
   render_target = handle;
   rt_onscreen = !persistent;
@@ -900,7 +909,8 @@ void csSoftwareGraphics3DCommon::DrawSimpleMesh (const csSimpleRenderMesh &mesh,
 
   rmesh.object2world = mesh.object2world;
 
-  csShaderVarStack stacks;
+  csRef<iShaderVarStack> stacks;
+  stacks.AttachNew (new scfArray<iShaderVarStack>);
   shadermgr->PushVariables (stacks);
   scrapContext.PushVariables (stacks);
 
@@ -994,8 +1004,10 @@ static iRenderBuffer* ColorFixup (iRenderBuffer* srcBuffer,
 
 void csSoftwareGraphics3DCommon::DrawMesh (const csCoreRenderMesh* mesh,
     const csRenderMeshModes& modes,
-    const csArray<csShaderVariable*> &stacks)
+    const iShaderVarStack* stacks)
 {
+  if (DrawMode == 0) return;
+
   ScanlineRendererHelper aNameThatDoesNotReallyMatter (this);
   if (!scanlineRenderer) return;
 
@@ -1057,45 +1069,10 @@ void csSoftwareGraphics3DCommon::DrawMesh (const csCoreRenderMesh* mesh,
 
   csRenderBufferLock<uint8, iRenderBuffer*> indices (indexbuf, CS_BUF_LOCK_READ);
 
-  const csMatrix3& w2c_m = w2c.GetO2T();
-  const csMatrix3& o2w_m = mesh->object2world.GetO2T();
-  const csVector3& w2c_t = w2c.GetO2TTranslation();
-  const csVector3& o2w_t = mesh->object2world.GetO2TTranslation();
-  csReversibleTransform object2camera (
-    o2w_m * w2c_m,
-    w2c_t + w2c_m.GetTranspose()*o2w_t);
-
-  const size_t rangeStart = indexbuf->GetRangeStart();
-  const size_t rangeEnd = indexbuf->GetRangeEnd();
-
-  if (!object2camera.IsIdentity())
-  {
-    if (!translatedVerts.IsValid()
-      || (translatedVerts->GetElementCount() <= rangeEnd))
-    {
-      translatedVerts = csRenderBuffer::CreateRenderBuffer (
-	rangeEnd + 1, CS_BUF_STREAM, CS_BUFCOMP_FLOAT, 3);
-    }
-
-    iRenderBuffer* vbuf = activebuffers[VATTR_SPEC(POSITION)];
-    csRenderBufferLock<csVector3, iRenderBuffer*> f1 (vbuf, CS_BUF_LOCK_READ);
-    if (!f1) return; 
-    csVector3* tr_verts = 
-      (csVector3*)translatedVerts->Lock (CS_BUF_LOCK_NORMAL);
-
-    size_t i;
-    const size_t maxVert = csMin (rangeEnd, vbuf->GetElementCount());
-    // Make sure we don't process too many vertices;
-    for (i = rangeStart; i <= maxVert; i++)
-    {
-      tr_verts[i] = object2camera.This2Other (f1[i]);
-    }
-
-    translatedVerts->Release();
-
-    activebuffers[VATTR_SPEC(POSITION)] = translatedVerts;
-  }
+  size_t rangeStart = indexbuf->GetRangeStart();
+  size_t rangeEnd = indexbuf->GetRangeEnd();
 
+  // @@@ Hm... color processing, probably *after* TransformVertices()...
   const bool alphaScale = ((modes.mixmode & CS_FX_MASK_ALPHA) != 0);
   if (pixelBGR || alphaScale)
   {
@@ -1121,22 +1098,44 @@ void csSoftwareGraphics3DCommon::DrawMesh (const csCoreRenderMesh* mesh,
     }
   }
 
+  VerticesLTN inBuffers;
+  VerticesLTN outBuffers;
+  size_t useCompNum[activeBufferCount];
+  size_t* compPtr = useCompNum;
+  const size_t* compNum = renderInfoMesh.bufferComps;
+  for (size_t b = 0; b < activeBufferCount; b++)
+  {
+    if (renderInfoMesh.desiredBuffers & (1 << b))
+    {
+      *compPtr++ = *compNum++;
+    }
+    else if (b == CS_SOFT3D_VA_BUFINDEX(POSITION))
+    {
+      *compPtr++ = 3;
+    }
+  }
+  uint useBuffers = 
+    renderInfoMesh.desiredBuffers | CS_SOFT3D_BUFFERFLAG(POSITION);
+  inBuffers.Linearize (activebuffers, useCompNum, useBuffers);
+  outBuffers.SetupEmpty (useCompNum, useBuffers);
+
+  csTriangle* triangles;
+  size_t trianglesCount;
+  vertexTransform->TransformVertices (mesh->object2world, w2c,
+    indexbuf, mesh->meshtype, inBuffers, 
+    mesh->indexstart, mesh->indexend, triangles, trianglesCount,
+    outBuffers, rangeStart, rangeEnd);
+
   const csRenderMeshType meshtype = mesh->meshtype;
   if ((meshtype >= CS_MESHTYPE_TRIANGLES) 
     && (meshtype <= CS_MESHTYPE_TRIANGLEFAN))
   {
-    const csRenderBufferComponentType compType =
-      indexbuf->GetComponentType();
-    size_t indexSize = csRenderBufferComponentSizes[compType];
-    uint8* tri = indices + mesh->indexstart*indexSize;
-    const uint8* triEnd = indices + mesh->indexend*indexSize;
-
     const uint triDrawIndex = 
       CS_MIXMODE_BLENDOP_SRC(usedModes.mixmode)*CS_MIXMODE_FACT_COUNT
       + CS_MIXMODE_BLENDOP_DST(usedModes.mixmode);
 
-    triDraw[triDrawIndex]->DrawMesh (activebuffers, rangeStart, rangeEnd, mesh, 
-      renderInfoMesh, meshtype, tri, triEnd, compType);
+    triDraw[triDrawIndex]->DrawMesh (outBuffers, rangeStart, rangeEnd, mesh, 
+      renderInfoMesh, triangles, trianglesCount);
   }
 }
 
@@ -1147,4 +1146,11 @@ void csSoftwareGraphics3DCommon::DrawPixmap (iTextureHandle *hTex,
   specifica->DrawPixmap (this, hTex, sx, sy, sw, sh, tx, ty, tw, th, Alpha);
 }
 
-} // namespace cspluginSoft3d
+int csSoftwareGraphics3DCommon::GetCurrentDrawFlags () const
+{
+  return current_drawflags;
+}
+
+
+}
+CS_PLUGIN_NAMESPACE_END(Soft3D)
