@@ -51,32 +51,41 @@ struct csTriangle;
 
 #include "polyrast.h"
 
-namespace cspluginSoft3d
+CS_PLUGIN_NAMESPACE_BEGIN(Soft3D)
 {
+
 // Forward declaration
 class csSoftwareGraphics3DCommon;
+class VerticesLTN;
 
-#define VATTR_SPEC(x)           (CS_VATTRIB_ ## x - CS_VATTRIB_SPECIFIC_FIRST)
-#define VATTR_GEN(x)							      \
-  ((CS_VATTRIB_ ## x - CS_VATTRIB_GENERIC_FIRST) + CS_VATTRIB_SPECIFIC_LAST + 1)
+struct iVertexTransform
+{
+  virtual ~iVertexTransform () {}
 
-static const size_t activeBufferCount = CS_VATTRIB_SPECIFIC_LAST - 
-  CS_VATTRIB_SPECIFIC_FIRST + 1;
-static const size_t activeTextureCount = 4;
+  virtual void SetNearPlane (const csPlane3& near_plane) = 0;
+  virtual void DisableNearPlane () = 0;
+
+  virtual void TransformVertices (const csReversibleTransform& object2world,
+    const csReversibleTransform& world2cam, iRenderBuffer* inIndices,
+    csRenderMeshType meshType, const VerticesLTN& inBuffers,
+    size_t inIndexStart, size_t inIndexEnd,
+    csTriangle*& outTriangles, size_t& outTrianglesCount,
+    VerticesLTN& outBuffers, size_t& rangeStart, size_t& rangeEnd) = 0;
+};
 
 struct iTriangleDrawer
 {
   virtual ~iTriangleDrawer() {}
-  virtual void DrawMesh (iRenderBuffer* activebuffers[], 
+  virtual void DrawMesh (const VerticesLTN& buffers, 
     size_t rangeStart, size_t rangeEnd, 
     const csCoreRenderMesh* mesh, 
     const iScanlineRenderer::RenderInfoMesh& scanRenderInfoMesh,
-    const csRenderMeshType meshtype, uint8* tri, const uint8* triEnd,
-    csRenderBufferComponentType compType) = 0;
+    csTriangle* triangles, size_t triangleCount) = 0;
 };
 
 struct iPixTypeSpecifica
 {
+  virtual ~iPixTypeSpecifica() {}
   virtual void DrawPixmap (csSoftwareGraphics3DCommon* G3D, 
     iTextureHandle *hTex, int sx, int sy, int sw, int sh, 
     int tx, int ty, int tw, int th, 
@@ -127,6 +136,8 @@ protected:
   long z_buf_size;
   csZBufMode zBufMode;
 
+  int current_drawflags;
+
   /**
    * Addresses of all lines for this frame. This table is used to avoid
    * calls to GetPixelAt in the main renderer loop. It is filled every frame
@@ -210,7 +221,7 @@ protected:
 
   iRenderBuffer* activebuffers[activeBufferCount];
   csSoftwareTextureHandle* activeSoftTex[activeTextureCount]; 
-  csRef<iRenderBuffer> translatedVerts;
+  iVertexTransform* vertexTransform;
   csRef<csRenderBuffer> processedColors[2];
   bool processedColorsFlag[2];
   /// The perspective corrected vertices.
@@ -235,6 +246,7 @@ protected:
 	theG3D->scanlineRenderer = theG3D->defaultRenderer;
 	theG3D->defaultRendererState->SetFlatColor (csVector4 (1, 1, 1, 1));
 	theG3D->defaultRendererState->SetShift (0, 0);
+	theG3D->defaultRendererState->SetColorSum (false);
 	unsetRenderer = true;
       }
       else
@@ -340,6 +352,9 @@ public:
   /// End the frame and do a page swap.
   virtual void FinishDraw ();
 
+  /// Return draw flags from current BeginDraw
+  virtual int GetCurrentDrawFlags() const;
+
   /// Draw a line in camera space.
   virtual void DrawLine (const csVector3& v1, const csVector3& v2,
     float fov, int color);
@@ -392,6 +407,8 @@ public:
     this->w2c = w2c;
     shadermgr->GetVariableAdd (string_world2camera)->SetValue (w2c);
   }
+  virtual const csReversibleTransform& GetWorldToCamera ()
+  { return w2c; }
   /// Set optional clipper.
   virtual void SetClipper (iClipper2D* clipper, int cliptype);
   /// Get clipper.
@@ -407,10 +424,15 @@ public:
   {
     do_near_plane = true;
     near_plane = pl;
+    vertexTransform->SetNearPlane (pl);
   }
 
   /// Reset near clip plane (i.e. disable it).
-  virtual void ResetNearPlane () { do_near_plane = false; }
+  virtual void ResetNearPlane () 
+  { 
+    do_near_plane = false; 
+    vertexTransform->DisableNearPlane ();
+  }
 
   /// Get near clip plane.
   virtual const csPlane3& GetNearPlane () const { return near_plane; }
@@ -429,7 +451,8 @@ public:
 
   /// Get the vertex buffer manager.
 
-  virtual void SetRenderTarget (iTextureHandle* handle, bool persistent);
+  virtual void SetRenderTarget (iTextureHandle* handle, bool persistent,
+  	int subtexture);
   virtual iTextureHandle* GetRenderTarget () const { return render_target; }
 
   /// Get Z-buffer value at given X,Y position
@@ -598,45 +621,18 @@ public:
   /// Drawroutine. Only way to draw stuff
   virtual void DrawMesh (const csCoreRenderMesh* mymesh,
     const csRenderMeshModes& modes,
-    const csArray<csShaderVariable*> &stacks);
+    const iShaderVarStack* stacks);
   void DrawSimpleMesh (const csSimpleRenderMesh &mesh, uint flags = 0);
 
+  bool PerformExtension (char const* /*command*/, ...) { return false; }
+  bool PerformExtensionV (char const* /*command*/, va_list /*args*/)
+  { return false; }
+
   /// Controls shadow drawing
   virtual void SetShadowState (int /*state*/)
   {
   }
 
-  /// Get maximum number of simultaneous vertex lights supported
-  virtual int GetMaxLights () const
-  {
-    return 0;
-  }
-
-  /// Sets a parameter for light i
-  virtual void SetLightParameter (int /*i*/, int /*param*/, csVector3 /*value*/)
-  {
-  }
-
-  /// Enables light i
-  virtual void EnableLight (int /*i*/)
-  {
-  }
-
-  /// Disables light i
-  virtual void DisableLight (int /*i*/)
-  {
-  }
-
-  /// Enable vertex lighting
-  virtual void EnablePVL ()
-  {
-  }
-
-  /// Disable vertex lighting
-  virtual void DisablePVL ()
-  {
-  }
-
   /// Set a renderstate boolean.
   virtual bool SetRenderState (G3D_RENDERSTATEOPTION /*op*/, long /*val*/)
   {
@@ -694,6 +690,7 @@ public:
   /** @} */
 };
 
-} // namespace cspluginSoft3d
+}
+CS_PLUGIN_NAMESPACE_END(Soft3D)
 
 #endif // __CS_SFTR3DCOM_H__
