@@ -20,18 +20,20 @@
 #include "soft_txt.h"
 #include "csgfx/bakekeycolor.h"
 #include "csgfx/imagemanipulate.h"
-#include "csgfx/memimage.h"
+#include "csgfx/imagememory.h"
 #include "csgfx/packrgb.h"
+#include "csgfx/textureformatstrings.h"
 #include "csgfx/xorpat.h"
 #include "iutil/cfgfile.h"
 #include "iutil/event.h"
 #include "iutil/eventq.h"
+#include "iutil/string.h"
 #include "igraphic/image.h"
 #include "ivaria/reporter.h"
 #include "csqint.h"
 #include "soft_g3d.h"
 
-namespace cspluginSoft3d
+CS_PLUGIN_NAMESPACE_BEGIN(Soft3D)
 {
 
 void csSoftwareTexture::compute_masks ()
@@ -51,7 +53,7 @@ void csSoftwareTexture::ImageToBitmap (iImage *Image)
     img = Image;
 
   size_t pixNum = w * h;
-  bitmap = new uint32[pixNum];
+  bitmap = (uint32*)cs_malloc (pixNum * sizeof (uint32));
 #ifdef CS_LITTLE_ENDIAN
   if (csPackRGBA::IsRGBpixelSane() 
     && !parent->texman->G3D->pixelBGR)
@@ -115,6 +117,27 @@ csSoftwareTextureHandle::csSoftwareTextureHandle (
     alphaType = csAlphaMode::alphaNone;
 }
 
+csSoftwareTextureHandle::csSoftwareTextureHandle (
+  csSoftwareTextureManager *texman, int w, int h,
+  bool alpha, int flags) : csTextureHandle (texman, 
+    flags | CS_TEXTURE_NOMIPMAPS), texman (texman)
+{
+  if (flags & CS_TEXTURE_3D)
+  {
+    int newwidth = 0, newheight = 0, newdepth = 0;
+    AdjustSizePo2 (w, h, 1, newwidth, newheight, newdepth);
+    w = newwidth; h = newheight;
+  }
+  memset (tex, 0, sizeof (tex));
+  tex[0] = new csSoftwareTexture (this, w, h);
+  prepared = true;
+
+  if (alpha)
+    alphaType = csAlphaMode::alphaSmooth;
+  else
+    alphaType = csAlphaMode::alphaNone;
+}
+
 csSoftwareTextureHandle::~csSoftwareTextureHandle ()
 {
   if (texman) texman->UnregisterTexture (this);
@@ -171,6 +194,13 @@ void csSoftwareTextureHandle::CreateMipmaps ()
       tex [i] = new csSoftwareTexture (this, newImage);
     }
   }
+  else
+  {
+    for (int i = 1; i < 4; i++)
+    {
+      tex [i] = 0;
+    }
+  }
 }
 
 void csSoftwareTextureHandle::PrepareInt ()
@@ -256,133 +286,6 @@ bool csSoftwareTextureHandle::GetRendererDimensions (int &mw, int &mh)
   return true;
 }
 
-//----------------------------------------------------------------------------//
-
-void csSoftRendererLightmap::DecRef ()
-{
-  if (scfRefCount == 1)							
-  {									
-    CS_ASSERT (slm != 0);
-    slm->FreeRLM (this);
-    return;								
-  }									
-  scfRefCount--;							
-}
-
-csSoftRendererLightmap::csSoftRendererLightmap () : 
-  scfImplementationType (this)
-{
-  dirty = false;
-  data = 0;
-
-  lightCellSize = 0;
-  memset (cacheData, 0, sizeof (cacheData));
-}
-
-csSoftRendererLightmap::~csSoftRendererLightmap ()
-{
-  delete[] data;
-}
-
-void csSoftRendererLightmap::SetSize (size_t lmPixels)
-{
-  delete[] data;
-  lmSize = lmPixels;
-  data = new csRGBpixel[lmSize];
-  lmSize *= sizeof (csRGBpixel);
-}
-
-void csSoftRendererLightmap::GetSLMCoords (int& left, int& top, 
-                                           int& width, int& height)
-{
-  left = rect.xmin; top  = rect.xmin;
-  width = rect.Width (); height = rect.Height ();
-}
-
-void csSoftRendererLightmap::SetData (csRGBpixel* data)
-{
-  memcpy (csSoftRendererLightmap::data, data,
-    lmSize);
-  dirty = true;
-}
-
-void csSoftRendererLightmap::SetLightCellSize (int size)
-{
-  lightCellSize = size;
-  lightCellShift = csLog2 (lightCellSize);
-}
-
-int csSoftRendererLightmap::GetLightCellSize ()
-{
-  return lightCellSize;
-}
-
-//---------------------------------------------------------------------------
-
-csSoftSuperLightmap::csSoftSuperLightmap (csSoftwareTextureManager* texman, 
-					  int width, int height) : 
-  scfImplementationType (this), RLMs(32)
-{
-  w = width;
-  h = height;
-  tex.AttachNew (new csSoftwareTextureHandle (texman, 0, 0));
-}
-
-csSoftSuperLightmap::~csSoftSuperLightmap ()
-{
-}
-
-void csSoftSuperLightmap::FreeRLM (csSoftRendererLightmap* rlm)
-{
-  // IncRef() ourselves manually.
-  // Otherwise freeing the RLM could trigger our own destruction -
-  // causing an assertion in block allocator (due to how BA frees items and
-  // the safety assertions on BA destruction.)
-  scfRefCount++;
-  int id = idmap.GetKey (rlm, ~0);
-  if (id != ~0)
-  {
-    idmap.Delete (id, rlm);
-  }
-  RLMs.Free (rlm);
-  DecRef ();
-}
-
-csSoftRendererLightmap* csSoftSuperLightmap::GetRlmForID (int id)
-{
-  return idmap.Get (id, 0);
-}
-
-csPtr<iRendererLightmap> csSoftSuperLightmap::RegisterLightmap (int left, int top, 
-                                                                int width, int height)
-{
-  csSoftRendererLightmap* rlm = RLMs.Alloc ();
-  rlm->SetSize (width * height);
-  rlm->slm = this;
-  rlm->rect.Set (left, top, left + width, top + height);
-
-  rlm->u1 = left;
-  rlm->v1 = top;
-  rlm->u2 = left + width;
-  rlm->v2 = top  + height;
-
-  const int id = ComputeRlmID (left, top);
-  idmap.Put (id, rlm);
-
-  return csPtr<iRendererLightmap> (rlm);
-}
-
-csPtr<iImage> csSoftSuperLightmap::Dump ()
-{
-  return 0;
-}
-
-iTextureHandle* csSoftSuperLightmap::GetTexture ()
-{
-  return tex;
-}
-
-
 //----------------------------------------------- csSoftwareTextureManager ---//
 
 csSoftwareTextureManager::csSoftwareTextureManager (
@@ -432,16 +335,13 @@ uint32 csSoftwareTextureManager::encode_rgb (int r, int g, int b)
 }
 
 csPtr<iTextureHandle> csSoftwareTextureManager::RegisterTexture (iImage* image,
-  int flags)
+  int flags, iString* fail_reason)
 {
   if (!image)
   {
-    G3D->Report(CS_REPORTER_SEVERITY_BUG,
-      "BAAAD!!! csSoftwareTextureManager::RegisterTexture with 0 image!");
-
-    csRef<iImage> im (csCreateXORPatternImage(32, 32, 5));
-    image = im;
-    im->IncRef ();	// Avoid smart pointer cleanup. @@@ UGLY
+    if (fail_reason) fail_reason->Replace (
+      "No image given to RegisterTexture!");
+    return 0;
   }
 
   csSoftwareTextureHandle *txt = new csSoftwareTextureHandle (
@@ -450,6 +350,38 @@ csPtr<iTextureHandle> csSoftwareTextureManager::RegisterTexture (iImage* image,
   return csPtr<iTextureHandle> (txt);
 }
 
+csPtr<iTextureHandle> csSoftwareTextureManager::CreateTexture (int w, int h,
+      csImageType imagetype, const char* format, int flags,
+      iString* fail_reason)
+{
+  CS::StructuredTextureFormat texFormat (
+    CS::TextureFormatStrings::ConvertStructured (format));
+  if (!texFormat.IsValid()) 
+  {
+    if (fail_reason) fail_reason->Replace ("invalid texture format");
+    return 0;
+  }
+
+  uint compMask = texFormat.GetComponentMask();
+  if ((compMask != CS::StructuredTextureFormat::compRGB)
+    && (compMask != CS::StructuredTextureFormat::compRGBA))
+  {
+    if (fail_reason) fail_reason->Replace ("texture format must be RGB or RGBA");
+    return 0;
+  }
+
+  if (imagetype != csimg2D)
+  {
+    if (fail_reason) fail_reason->Replace ("only 2D textures are supported");
+    return 0;
+  }
+
+  csSoftwareTextureHandle *txt = new csSoftwareTextureHandle (
+    this, w, h, compMask == CS::StructuredTextureFormat::compRGBA, flags);
+  textures.Push (txt);
+  return csPtr<iTextureHandle> (txt);
+}
+
 void csSoftwareTextureManager::UnregisterTexture (
 		csSoftwareTextureHandle* handle)
 {
@@ -458,9 +390,9 @@ void csSoftwareTextureManager::UnregisterTexture (
 }
 
 csPtr<iSuperLightmap> csSoftwareTextureManager::CreateSuperLightmap (
-  int width, int height)
+  int /*width*/, int /*height*/)
 {
-  return csPtr<iSuperLightmap> (new csSoftSuperLightmap (this, width, height));
+  return 0;
 }
 
 void csSoftwareTextureManager::GetMaxTextureSize (int& w, int& h, int& aspect)
@@ -469,16 +401,5 @@ void csSoftwareTextureManager::GetMaxTextureSize (int& w, int& h, int& aspect)
   aspect = 32768;
 }
 
-void csSoftwareTextureManager::GetLightmapRendererCoords (
-  int /*slmWidth*/, int /*slmHeight*/, int lm_x1, int lm_y1, int /*lm_x2*/,
-  int /*lm_y2*/, float& lm_u1, float& lm_v1, float &lm_u2, float& lm_v2)
-{
-  lm_u1 = lm_x1;
-  lm_v1 = lm_y1;
-  /*lm_u2 = lm_x2 + 1;
-  lm_v2 = lm_y2 + 1;
-  lm_u1 = lm_x1;
-  lm_v1 =*/ lm_u2 = lm_v2 = 0.0f;
 }
-
-} // namespace cspluginSoft3d
+CS_PLUGIN_NAMESPACE_END(Soft3D)
